using System;

namespace Godot
{
    /// <summary>
    /// The ARVR Anchor point is a spatial node that maps a real world location identified by the AR platform to a position within the game world. For example, as long as plane detection in ARKit is on, ARKit will identify and update the position of planes (tables, floors, etc) and create anchors for them.
    /// This node is mapped to one of the anchors through its unique id. When you receive a signal that a new anchor is available you should add this node to your scene for that anchor. You can predefine nodes and set the id and the nodes will simply remain on 0,0,0 until a plane is recognised.
    /// Keep in mind that as long as plane detection is enable the size, placing and orientation of an anchor will be updates as the detection logic learns more about the real world out there especially if only part of the surface is in view.
    /// </summary>
    public class ARVRAnchor : Spatial
    {
        /// <summary>
        /// The anchor's id. You can set this before the anchor itself exists. The first anchor gets an id of [code]1[/code], the second an id of [code]2[/code], etc. When anchors get removed, the engine can then assign the corresponding id to new anchors. The most common situation where anchors 'disappear' is when the AR server identifies that two anchors represent different parts of the same plane and merges them.
        /// </summary>
        public int AnchorId
        {
            get
            {
                return GetAnchorId();
            }
            set
            {
                SetAnchorId(value);
            }
        }


        private const string nativeName = "ARVRAnchor";

        public ARVRAnchor() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ARVRAnchor_Ctor(this);
        }

        internal ARVRAnchor(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_anchor_id");

        [GodotMethod("set_anchor_id")]
        public void SetAnchorId(int anchorId)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, Object.GetPtr(this), anchorId);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_anchor_id");

        [GodotMethod("get_anchor_id")]
        public int GetAnchorId()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_anchor_name");

        /// <summary>
        /// Returns the name given to this anchor.
        /// </summary>
        [GodotMethod("get_anchor_name")]
        public string GetAnchorName()
        {
            return NativeCalls.godot_icall_0_2(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_is_active");

        /// <summary>
        /// Returns true if the anchor is being tracked and false if no anchor with this id is currently known.
        /// </summary>
        [GodotMethod("get_is_active")]
        public bool GetIsActive()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_size");

        /// <summary>
        /// Returns the estimated size of the plane that was detected. Say when the anchor relates to a table in the real world, this is the estimated size of the surface of that table.
        /// </summary>
        [GodotMethod("get_size")]
        public Vector3 GetSize()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_4, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_plane");

        /// <summary>
        /// Returns a plane aligned with our anchor, handy for intersection testing
        /// </summary>
        [GodotMethod("get_plane")]
        public Plane GetPlane()
        {
            object ret = NativeCalls.godot_icall_0_5(method_bind_5, Object.GetPtr(this));
            return (Plane)ret;
        }
    }
}
