using System;

namespace Godot
{
    /// <summary>
    /// This is a helper spatial node that is linked to the tracking of controllers. It also offers several handy pass throughs to the state of buttons and such on the controllers.
    /// Controllers are linked by their id. You can create controller nodes before the controllers are available. Say your game always uses two controllers (one for each hand) you can predefine the controllers with id 1 and 2 and they will become active as soon as the controllers are identified. If you expect additional controllers to be used you should react to the signals and add ARVRController nodes to your scene.
    /// The position of the controller node is automatically updated by the ARVR Server. This makes this node ideal to add child  nodes to visualise the controller.
    /// </summary>
    public class ARVRController : Spatial
    {
        /// <summary>
        /// The controller's id.
        /// A controller id of 0 is unbound and will always result in an inactive node. Controller id 1 is reserved for the first controller that identifies itself as the left hand controller and id 2 is reserved for the first controller that identifies itself as the right hand controller.
        /// For any other controller that the [ARVRServer] detects we continue with controller id 3.
        /// When a controller is turned off, its slot is freed. This ensures controllers will keep the same id even when controllers with lower ids are turned off.
        /// </summary>
        public int ControllerId
        {
            get
            {
                return GetControllerId();
            }
            set
            {
                SetControllerId(value);
            }
        }

        /// <summary>
        /// The degree to which the tracker rumbles. Ranges from [code]0.0[/code] to [code]1.0[/code] with precision [code].01[/code]. If changed, updates [member ARVRPositionalTracker.rumble] accordingly.
        /// </summary>
        public float Rumble
        {
            get
            {
                return GetRumble();
            }
            set
            {
                SetRumble(value);
            }
        }


        private const string nativeName = "ARVRController";

        public ARVRController() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ARVRController_Ctor(this);
        }

        internal ARVRController(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_controller_id");

        [GodotMethod("set_controller_id")]
        public void SetControllerId(int controllerId)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, Object.GetPtr(this), controllerId);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_controller_id");

        [GodotMethod("get_controller_id")]
        public int GetControllerId()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_controller_name");

        /// <summary>
        /// If active, returns the name of the associated controller if provided by the AR/VR SDK used.
        /// </summary>
        [GodotMethod("get_controller_name")]
        public string GetControllerName()
        {
            return NativeCalls.godot_icall_0_2(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_joystick_id");

        /// <summary>
        /// Returns the ID of the joystick object bound to this. Every controller tracked by the ARVR Server that has buttons and axis will also be registered as a joystick within Godot. This means that all the normal joystick tracking and input mapping will work for buttons and axis found on the AR/VR controllers. This ID is purely offered as information so you can link up the controller with its joystick entry.
        /// </summary>
        [GodotMethod("get_joystick_id")]
        public int GetJoystickId()
        {
            return NativeCalls.godot_icall_0_1(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_button_pressed");

        /// <summary>
        /// Returns [code]true[/code] if the button at index [code]button[/code] is pressed.
        /// </summary>
        [GodotMethod("is_button_pressed")]
        public int IsButtonPressed(int button)
        {
            return NativeCalls.godot_icall_1_6(method_bind_4, Object.GetPtr(this), button);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_joystick_axis");

        /// <summary>
        /// Returns the value of the given axis for things like triggers, touchpads, etc. that are embedded into the controller.
        /// </summary>
        [GodotMethod("get_joystick_axis")]
        public float GetJoystickAxis(int axis)
        {
            return NativeCalls.godot_icall_1_7(method_bind_5, Object.GetPtr(this), axis);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_is_active");

        /// <summary>
        /// Returns [code]true[/code] if the bound controller is active. ARVR systems attempt to track active controllers.
        /// </summary>
        [GodotMethod("get_is_active")]
        public bool GetIsActive()
        {
            return NativeCalls.godot_icall_0_3(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_hand");

        /// <summary>
        /// Returns the hand holding this controller, if known. See TRACKER_* constants in [ARVRPositionalTracker].
        /// </summary>
        [GodotMethod("get_hand")]
        public ARVRPositionalTracker.TrackerHand GetHand()
        {
            return NativeCalls.godot_icall_0_8(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rumble");

        [GodotMethod("get_rumble")]
        public float GetRumble()
        {
            return NativeCalls.godot_icall_0_9(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rumble");

        [GodotMethod("set_rumble")]
        public void SetRumble(float rumble)
        {
            NativeCalls.godot_icall_1_10(method_bind_9, Object.GetPtr(this), rumble);
        }
    }
}
