using System;

namespace Godot
{
    /// <summary>
    /// This class needs to be implemented to make an AR or VR platform available to Godot and these should be implemented as C++ modules or GDNative modules (note that for GDNative the subclass ARVRScriptInterface should be used). Part of the interface is exposed to GDScript so you can detect, enable and configure an AR or VR platform.
    /// Interfaces should be written in such a way that simply enabling them will give us a working setup. You can query the available interfaces through ARVRServer.
    /// </summary>
    public class ARVRInterface : Reference
    {
        public enum Tracking_status
        {
            NormalTracking = 0,
            ExcessiveMotion = 1,
            InsufficientFeatures = 2,
            UnknownTracking = 3,
            NotTracking = 4
        }

        public enum Eyes
        {
            Mono = 0,
            Left = 1,
            Right = 2
        }

        public enum Capabilities
        {
            None = 0,
            Mono = 1,
            Stereo = 2,
            Ar = 4,
            External = 8
        }


        /// <summary>
        /// Is this our primary interface?
        /// </summary>
        public bool InterfaceIsPrimary
        {
            get
            {
                return IsPrimary();
            }
            set
            {
                SetIsPrimary(value);
            }
        }

        /// <summary>
        /// Has this interface been initialized?
        /// </summary>
        public bool InterfaceIsInitialized
        {
            get
            {
                return IsInitialized();
            }
            set
            {
                SetIsInitialized(value);
            }
        }

        /// <summary>
        /// On an AR interface, is our anchor detection enabled?
        /// </summary>
        public bool ArIsAnchorDetectionEnabled
        {
            get
            {
                return GetAnchorDetectionIsEnabled();
            }
            set
            {
                SetAnchorDetectionIsEnabled(value);
            }
        }


        private const string nativeName = "ARVRInterface";

        internal ARVRInterface() {}

        internal ARVRInterface(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_name");

        /// <summary>
        /// Returns the name of this interface (OpenVR, OpenHMD, ARKit, etc).
        /// </summary>
        [GodotMethod("get_name")]
        public string GetName()
        {
            return NativeCalls.godot_icall_0_2(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_capabilities");

        /// <summary>
        /// Returns a combination of flags providing information about the capabilities of this interface.
        /// </summary>
        [GodotMethod("get_capabilities")]
        public int GetCapabilities()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_primary");

        [GodotMethod("is_primary")]
        public bool IsPrimary()
        {
            return NativeCalls.godot_icall_0_3(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_is_primary");

        [GodotMethod("set_is_primary")]
        public void SetIsPrimary(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_3, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_initialized");

        [GodotMethod("is_initialized")]
        public bool IsInitialized()
        {
            return NativeCalls.godot_icall_0_3(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_is_initialized");

        [GodotMethod("set_is_initialized")]
        public void SetIsInitialized(bool initialized)
        {
            NativeCalls.godot_icall_1_11(method_bind_5, Object.GetPtr(this), initialized);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "initialize");

        /// <summary>
        /// Call this to initialize this interface. The first interface that is initialized is identified as the primary interface and it will be used for rendering output.
        /// After initializing the interface you want to use you then need to enable the AR/VR mode of a viewport and rendering should commence.
        /// Note that you must enable the AR/VR mode on the main viewport for any device that uses the main output of Godot such as for mobile VR.
        /// If you do this for a platform that handles its own output (such as OpenVR) Godot will show just one eye without distortion on screen. Alternatively you can add a separate viewport node to your scene and enable AR/VR on that viewport and it will be used to output to the HMD leaving you free to do anything you like in the main window such as using a separate camera as a spectator camera or render out something completely different.
        /// While currently not used you can activate additional interfaces, you may wish to do this if you want to track controllers from other platforms. However at this point in time only one interface can render to an HMD.
        /// </summary>
        [GodotMethod("initialize")]
        public bool Initialize()
        {
            return NativeCalls.godot_icall_0_3(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "uninitialize");

        /// <summary>
        /// Turns the interface off.
        /// </summary>
        [GodotMethod("uninitialize")]
        public void Uninitialize()
        {
            NativeCalls.godot_icall_0_12(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tracking_status");

        /// <summary>
        /// If supported, returns the status of our tracking. This will allow you to provide feedback to the user whether there are issues with positional tracking.
        /// </summary>
        [GodotMethod("get_tracking_status")]
        public ARVRInterface.Tracking_status GetTrackingStatus()
        {
            return NativeCalls.godot_icall_0_13(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_render_targetsize");

        /// <summary>
        /// Returns the resolution at which we should render our intermediate results before things like lens distortion are applied by the VR platform.
        /// </summary>
        [GodotMethod("get_render_targetsize")]
        public Vector2 GetRenderTargetsize()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_9, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_stereo");

        /// <summary>
        /// Returns true if the current output of this interface is in stereo.
        /// </summary>
        [GodotMethod("is_stereo")]
        public bool IsStereo()
        {
            return NativeCalls.godot_icall_0_3(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_anchor_detection_is_enabled");

        [GodotMethod("get_anchor_detection_is_enabled")]
        public bool GetAnchorDetectionIsEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_anchor_detection_is_enabled");

        [GodotMethod("set_anchor_detection_is_enabled")]
        public void SetAnchorDetectionIsEnabled(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_12, Object.GetPtr(this), enable);
        }
    }
}
