using System;

namespace Godot
{
    /// <summary>
    /// This is a special node within the AR/VR system that maps the physical location of the center of our tracking space to the virtual location within our game world.
    /// There should be only one of these nodes in your scene and you must have one. All the ARVRCamera, ARVRController and ARVRAnchor nodes should be direct children of this node for spatial tracking to work correctly.
    /// It is the position of this node that you update when you're character needs to move through your game world while we're not moving in the real world. Movement in the real world is always in relation to this origin point.
    /// So say that your character is driving a car, the ARVROrigin node should be a child node of this car. If you implement a teleport system to move your character, you change the position of this node. Etc.
    /// </summary>
    public class ARVROrigin : Spatial
    {
        /// <summary>
        /// Allows you to adjust the scale to your game's units. Most AR/VR platforms assume a scale of 1 game world unit = 1 meter in the real world.
        /// Note that this method is a passthrough to the [ARVRServer] itself.
        /// </summary>
        public float WorldScale
        {
            get
            {
                return GetWorldScale();
            }
            set
            {
                SetWorldScale(value);
            }
        }


        private const string nativeName = "ARVROrigin";

        public ARVROrigin() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ARVROrigin_Ctor(this);
        }

        internal ARVROrigin(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_world_scale");

        [GodotMethod("set_world_scale")]
        public void SetWorldScale(float worldScale)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), worldScale);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_world_scale");

        [GodotMethod("get_world_scale")]
        public float GetWorldScale()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }
    }
}
