using System;

namespace Godot
{
    /// <summary>
    /// An instance of this object represents a device that is tracked such as a controller or anchor point. HMDs aren't represented here as they are fully handled internally.
    /// As controllers are turned on and the AR/VR interface detects them instances of this object are automatically added to this list of active tracking objects accessible through the ARVRServer
    /// The ARVRController and ARVRAnchor both consume objects of this type and should be the objects you use in game. The positional trackers are just the under the hood objects that make this all work and are mostly exposed so GDNative based interfaces can interact with them.
    /// </summary>
    public class ARVRPositionalTracker : Object
    {
        public enum TrackerHand
        {
            HandUnknown = 0,
            LeftHand = 1,
            RightHand = 2
        }


        /// <summary>
        /// The degree to which the tracker rumbles. Ranges from [code]0.0[/code] to [code]1.0[/code] with precision [code].01[/code].
        /// </summary>
        public float Rumble
        {
            get
            {
                return GetRumble();
            }
            set
            {
                SetRumble(value);
            }
        }


        private const string nativeName = "ARVRPositionalTracker";

        public ARVRPositionalTracker() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ARVRPositionalTracker_Ctor(this);
        }

        internal ARVRPositionalTracker(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_type");

        /// <summary>
        /// Returns the tracker's type.
        /// </summary>
        [GodotMethod("get_type")]
        public ARVRServer.TrackerType GetType()
        {
            return NativeCalls.godot_icall_0_15(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_name");

        /// <summary>
        /// Returns the controller or anchor point's name if available.
        /// </summary>
        [GodotMethod("get_name")]
        public string GetName()
        {
            return NativeCalls.godot_icall_0_2(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_joy_id");

        /// <summary>
        /// If this is a controller that is being tracked the controller will also be represented by a joystick entry with this id.
        /// </summary>
        [GodotMethod("get_joy_id")]
        public int GetJoyId()
        {
            return NativeCalls.godot_icall_0_1(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tracks_orientation");

        /// <summary>
        /// Returns [code]true[/code] if this device tracks orientation.
        /// </summary>
        [GodotMethod("get_tracks_orientation")]
        public bool GetTracksOrientation()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_orientation");

        /// <summary>
        /// Returns the controller's orientation matrix.
        /// </summary>
        [GodotMethod("get_orientation")]
        public Basis GetOrientation()
        {
            object ret = NativeCalls.godot_icall_0_16(method_bind_4, Object.GetPtr(this));
            return (Basis)ret;
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tracks_position");

        /// <summary>
        /// Returns [code]true[/code] if this device tracks position.
        /// </summary>
        [GodotMethod("get_tracks_position")]
        public bool GetTracksPosition()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_position");

        /// <summary>
        /// Returns the world-space controller position.
        /// </summary>
        [GodotMethod("get_position")]
        public Vector3 GetPosition()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_6, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_hand");

        /// <summary>
        /// Returns the hand holding this tracker, if known. See TRACKER_* constants.
        /// </summary>
        [GodotMethod("get_hand")]
        public ARVRPositionalTracker.TrackerHand GetHand()
        {
            return NativeCalls.godot_icall_0_8(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_transform");

        /// <summary>
        /// Returns the transform combining this device's orientation and position.
        /// </summary>
        [GodotMethod("get_transform")]
        public Transform GetTransform(bool adjustByReferenceFrame)
        {
            object ret = NativeCalls.godot_icall_1_17(method_bind_8, Object.GetPtr(this), adjustByReferenceFrame);
            return (Transform)ret;
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rumble");

        [GodotMethod("get_rumble")]
        public float GetRumble()
        {
            return NativeCalls.godot_icall_0_9(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rumble");

        [GodotMethod("set_rumble")]
        public void SetRumble(float rumble)
        {
            NativeCalls.godot_icall_1_10(method_bind_10, Object.GetPtr(this), rumble);
        }
    }
}
