using System;

namespace Godot
{
    /// <summary>
    /// The AR/VR Server is the heart of our AR/VR solution and handles all the processing.
    /// </summary>
    public static class ARVRServer
    {
        public enum RotationMode
        {
            ResetFullRotation = 0,
            ResetButKeepTilt = 1,
            DontResetRotation = 2
        }

        public enum TrackerType
        {
            Controller = 1,
            Basestation = 2,
            Anchor = 4,
            AnyKnown = 127,
            Unknown = 128,
            Any = 255
        }


        /// <summary>
        /// Allows you to adjust the scale to your game's units. Most AR/VR platforms assume a scale of 1 game world unit = 1 meter in the real world.
        /// </summary>
        public static float WorldScale
        {
            get
            {
                return GetWorldScale();
            }
            set
            {
                SetWorldScale(value);
            }
        }


        private const string nativeName = "ARVRServer";
        internal static IntPtr ptr = NativeCalls.godot_icall_ARVRServer_get_singleton();

        private static IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_world_scale");

        [GodotMethod("get_world_scale")]
        public static float GetWorldScale()
        {
            return NativeCalls.godot_icall_0_9(method_bind_0, ptr);
        }

        private static IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_world_scale");

        [GodotMethod("set_world_scale")]
        public static void SetWorldScale(float arg0)
        {
            NativeCalls.godot_icall_1_10(method_bind_1, ptr, arg0);
        }

        private static IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_reference_frame");

        /// <summary>
        /// Gets our reference frame transform, mostly used internally and exposed for GDNative build interfaces.
        /// </summary>
        [GodotMethod("get_reference_frame")]
        public static Transform GetReferenceFrame()
        {
            object ret = NativeCalls.godot_icall_0_18(method_bind_2, ptr);
            return (Transform)ret;
        }

        private static IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "center_on_hmd");

        /// <summary>
        /// This is a really important function to understand correctly. AR and VR platforms all handle positioning slightly differently.
        /// For platforms that do not offer spatial tracking our origin point (0,0,0) is the location of our HMD but you have little control over the direction the player is facing in the real world.
        /// For platforms that do offer spatial tracking our origin point depends very much on the system. For OpenVR our origin point is usually the center of the tracking space, on the ground. For other platforms its often the location of the tracking camera.
        /// This method allows you to center our tracker on the location of the HMD, it will take the current location of the HMD and use that to adjust all our tracking data in essence realigning the real world to your players current position in your game world.
        /// For this method to produce usable results tracking information should be available and this often takes a few frames after starting your game.
        /// You should call this method after a few seconds have passed, when the user requests a realignment of the display holding a designated button on a controller for a short period of time, and when implementing a teleport mechanism.
        /// </summary>
        [GodotMethod("center_on_hmd")]
        public static void CenterOnHmd(ARVRServer.RotationMode rotationMode, bool keepHeight)
        {
            NativeCalls.godot_icall_2_19(method_bind_3, ptr, rotationMode, keepHeight);
        }

        private static IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_interface_count");

        /// <summary>
        /// Get the number of interfaces currently registered with the AR/VR server. If you're game supports multiple AR/VR platforms you can look throught the available interface and either present the user with a selection or simply try an initialize each interface and use the first one that returns true.
        /// </summary>
        [GodotMethod("get_interface_count")]
        public static int GetInterfaceCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_4, ptr);
        }

        private static IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_interface");

        /// <summary>
        /// Get the interface registered at a given index in our list of interfaces.
        /// </summary>
        [GodotMethod("get_interface")]
        public static ARVRInterface GetInterface(int idx)
        {
            return NativeCalls.godot_icall_1_20(method_bind_5, ptr, idx);
        }

        private static IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_interfaces");

        /// <summary>
        /// Returns a list of available interfaces with both id and name of the interface.
        /// </summary>
        [GodotMethod("get_interfaces")]
        public static object[] GetInterfaces()
        {
            return NativeCalls.godot_icall_0_21(method_bind_6, ptr);
        }

        private static IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "find_interface");

        /// <summary>
        /// Find an interface by its name. Say that you're making a game that uses specific capabilities of an AR/VR platform you can find the interface for that platform by name and initialize it.
        /// </summary>
        [GodotMethod("find_interface")]
        public static ARVRInterface FindInterface(string name)
        {
            return NativeCalls.godot_icall_1_22(method_bind_7, ptr, name);
        }

        private static IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tracker_count");

        /// <summary>
        /// Get the number of trackers currently registered.
        /// </summary>
        [GodotMethod("get_tracker_count")]
        public static int GetTrackerCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_8, ptr);
        }

        private static IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tracker");

        /// <summary>
        /// Get the positional tracker at the given ID.
        /// </summary>
        [GodotMethod("get_tracker")]
        public static ARVRPositionalTracker GetTracker(int idx)
        {
            return NativeCalls.godot_icall_1_23(method_bind_9, ptr, idx);
        }

        private static IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_primary_interface");

        /// <summary>
        /// Changes the primary interface to the specified interface. Again mostly exposed for GDNative interfaces.
        /// </summary>
        [GodotMethod("set_primary_interface")]
        public static void SetPrimaryInterface(ARVRInterface @interface)
        {
            NativeCalls.godot_icall_1_24(method_bind_10, ptr, Object.GetPtr(@interface));
        }
    }
}
