using System;

namespace Godot
{
    /// <summary>
    /// A* (A star) is a computer algorithm that is widely used in pathfinding and graph traversal, the process of plotting an efficiently directed path between multiple points. It enjoys widespread use due to its performance and accuracy. Godot's A* implementation make use of vectors as points.
    /// You must add points manually with [method AStar.add_point] and create segments manually with [method AStar.connect_points]. So you can test if there is a path between two points with the [method AStar.are_points_connected] function, get the list of existing ids in the found path with [method AStar.get_id_path], or the points list with [method AStar.get_point_path].
    /// </summary>
    public class AStar : Reference
    {
        private const string nativeName = "AStar";

        public AStar() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AStar_Ctor(this);
        }

        internal AStar(bool memoryOwn) : base(memoryOwn) {}

        /// <summary>
        /// Called when computing the cost between two connected points.
        /// </summary>
        [GodotMethod("_compute_cost")]
        public virtual float _ComputeCost(int fromId, int toId)
        {
            return default(float);
        }

        /// <summary>
        /// Called when estimating the cost between a point and the path's ending point.
        /// </summary>
        [GodotMethod("_estimate_cost")]
        public virtual float _EstimateCost(int fromId, int toId)
        {
            return default(float);
        }

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_available_point_id");

        /// <summary>
        /// Returns the next available point id with no point associated to it.
        /// </summary>
        [GodotMethod("get_available_point_id")]
        public int GetAvailablePointId()
        {
            return NativeCalls.godot_icall_0_1(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_point");

        /// <summary>
        /// Adds a new point at the given position with the given identifier. The algorithm prefers points with lower [code]weight_scale[/code] to form a path. The [code]id[/code] must be 0 or larger, and the [code]weight_scale[/code] must be 1 or larger.
        /// [codeblock]
        /// var as = AStar.new()
        /// as.add_point(1, Vector3(1,0,0), 4) # Adds the point (1,0,0) with weight_scale=4 and id=1
        /// [/codeblock]
        /// If there already exists a point for the given id, its position and weight scale are updated to the given values.
        /// </summary>
        [GodotMethod("add_point")]
        public void AddPoint(int id, Vector3 position, float weightScale = 1f)
        {
            NativeCalls.godot_icall_3_25(method_bind_1, Object.GetPtr(this), id, ref position, weightScale);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_point_position");

        /// <summary>
        /// Returns the position of the point associated with the given id.
        /// </summary>
        [GodotMethod("get_point_position")]
        public Vector3 GetPointPosition(int id)
        {
            object ret = NativeCalls.godot_icall_1_26(method_bind_2, Object.GetPtr(this), id);
            return (Vector3)ret;
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_point_position");

        /// <summary>
        /// Sets the position for the point with the given id.
        /// </summary>
        [GodotMethod("set_point_position")]
        public void SetPointPosition(int id, Vector3 position)
        {
            NativeCalls.godot_icall_2_27(method_bind_3, Object.GetPtr(this), id, ref position);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_point_weight_scale");

        /// <summary>
        /// Returns the weight scale of the point associated with the given id.
        /// </summary>
        [GodotMethod("get_point_weight_scale")]
        public float GetPointWeightScale(int id)
        {
            return NativeCalls.godot_icall_1_7(method_bind_4, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_point_weight_scale");

        /// <summary>
        /// Sets the [code]weight_scale[/code] for the point with the given id.
        /// </summary>
        [GodotMethod("set_point_weight_scale")]
        public void SetPointWeightScale(int id, float weightScale)
        {
            NativeCalls.godot_icall_2_28(method_bind_5, Object.GetPtr(this), id, weightScale);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_point");

        /// <summary>
        /// Removes the point associated with the given id from the points pool.
        /// </summary>
        [GodotMethod("remove_point")]
        public void RemovePoint(int id)
        {
            NativeCalls.godot_icall_1_0(method_bind_6, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_point");

        /// <summary>
        /// Returns whether a point associated with the given id exists.
        /// </summary>
        [GodotMethod("has_point")]
        public bool HasPoint(int id)
        {
            return NativeCalls.godot_icall_1_29(method_bind_7, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_points");

        /// <summary>
        /// Returns an array of all points.
        /// </summary>
        [GodotMethod("get_points")]
        public object[] GetPoints()
        {
            return NativeCalls.godot_icall_0_21(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_point_connections");

        /// <summary>
        /// Returns an array with the ids of the points that form the connect with the given point.
        /// [codeblock]
        /// var as = AStar.new()
        /// as.add_point(1, Vector3(0,0,0))
        /// as.add_point(2, Vector3(0,1,0))
        /// as.add_point(3, Vector3(1,1,0))
        /// as.add_point(4, Vector3(2,0,0))
        /// as.connect_points(1, 2, true)
        /// as.connect_points(1, 3, true)
        /// var neighbors = as.get_point_connections(1) # returns [2, 3]
        /// [/codeblock]
        /// </summary>
        [GodotMethod("get_point_connections")]
        public int[] GetPointConnections(int id)
        {
            return NativeCalls.godot_icall_1_30(method_bind_9, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "connect_points");

        /// <summary>
        /// Creates a segment between the given points.
        /// [codeblock]
        /// var as = AStar.new()
        /// as.add_point(1, Vector3(1,1,0))
        /// as.add_point(2, Vector3(0,5,0))
        /// as.connect_points(1, 2, false) # If bidirectional=false it's only possible to go from point 1 to point 2
        /// # and not from point 2 to point 1.
        /// [/codeblock]
        /// </summary>
        [GodotMethod("connect_points")]
        public void ConnectPoints(int id, int toId, bool bidirectional = true)
        {
            NativeCalls.godot_icall_3_31(method_bind_10, Object.GetPtr(this), id, toId, bidirectional);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "disconnect_points");

        /// <summary>
        /// Deletes the segment between the given points.
        /// </summary>
        [GodotMethod("disconnect_points")]
        public void DisconnectPoints(int id, int toId)
        {
            NativeCalls.godot_icall_2_32(method_bind_11, Object.GetPtr(this), id, toId);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "are_points_connected");

        /// <summary>
        /// Returns whether there is a connection/segment between the given points.
        /// </summary>
        [GodotMethod("are_points_connected")]
        public bool ArePointsConnected(int id, int toId)
        {
            return NativeCalls.godot_icall_2_33(method_bind_12, Object.GetPtr(this), id, toId);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear");

        /// <summary>
        /// Clears all the points and segments.
        /// </summary>
        [GodotMethod("clear")]
        public void Clear()
        {
            NativeCalls.godot_icall_0_12(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_closest_point");

        /// <summary>
        /// Returns the id of the closest point to [code]to_position[/code]. Returns -1 if there are no points in the points pool.
        /// </summary>
        [GodotMethod("get_closest_point")]
        public int GetClosestPoint(Vector3 toPosition)
        {
            return NativeCalls.godot_icall_1_34(method_bind_14, Object.GetPtr(this), ref toPosition);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_closest_position_in_segment");

        /// <summary>
        /// Returns the closest position to [code]to_position[/code] that resides inside a segment between two connected points.
        /// [codeblock]
        /// var as = AStar.new()
        /// as.add_point(1, Vector3(0,0,0))
        /// as.add_point(2, Vector3(0,5,0))
        /// as.connect_points(1, 2)
        /// var res = as.get_closest_position_in_segment(Vector3(3,3,0)) # returns (0, 3, 0)
        /// [/codeblock]
        /// The result is in the segment that goes from [code]y=0[/code] to [code]y=5[/code]. It's the closest position in the segment to the given point.
        /// </summary>
        [GodotMethod("get_closest_position_in_segment")]
        public Vector3 GetClosestPositionInSegment(Vector3 toPosition)
        {
            object ret = NativeCalls.godot_icall_1_35(method_bind_15, Object.GetPtr(this), ref toPosition);
            return (Vector3)ret;
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_point_path");

        /// <summary>
        /// Returns an array with the points that are in the path found by AStar between the given points. The array is ordered from the starting point to the ending point of the path.
        /// </summary>
        [GodotMethod("get_point_path")]
        public Vector3[] GetPointPath(int fromId, int toId)
        {
            return NativeCalls.godot_icall_2_36(method_bind_16, Object.GetPtr(this), fromId, toId);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_id_path");

        /// <summary>
        /// Returns an array with the ids of the points that form the path found by AStar between the given points. The array is ordered from the starting point to the ending point of the path.
        /// [codeblock]
        /// var as = AStar.new()
        /// as.add_point(1, Vector3(0,0,0))
        /// as.add_point(2, Vector3(0,1,0), 1) # default weight is 1
        /// as.add_point(3, Vector3(1,1,0))
        /// as.add_point(4, Vector3(2,0,0))
        /// as.connect_points(1, 2, false)
        /// as.connect_points(2, 3, false)
        /// as.connect_points(4, 3, false)
        /// as.connect_points(1, 4, false)
        /// as.connect_points(5, 4, false)
        /// var res = as.get_id_path(1, 3) # returns [1, 2, 3]
        /// [/codeblock]
        /// If you change the 2nd point's weight to 3, then the result will be [code][1, 4, 3][/code] instead, because now even though the distance is longer, it's "easier" to get through point 4 than through point 2.
        /// </summary>
        [GodotMethod("get_id_path")]
        public int[] GetIdPath(int fromId, int toId)
        {
            return NativeCalls.godot_icall_2_37(method_bind_17, Object.GetPtr(this), fromId, toId);
        }
    }
}
