using System;

namespace Godot
{
    /// <summary>
    /// Animations are created using a [SpriteFrames] resource, which can be configured in the editor via the SpriteFrames panel.
    /// </summary>
    public class AnimatedSprite3D : SpriteBase3D
    {
        /// <summary>
        /// The [SpriteFrames] resource containing the animation(s).
        /// </summary>
        public SpriteFrames Frames
        {
            get
            {
                return GetSpriteFrames();
            }
            set
            {
                SetSpriteFrames(value);
            }
        }

        /// <summary>
        /// The current animation from the [code]frames[/code] resource. If this value changes, the [code]frame[/code] counter is reset.
        /// </summary>
        public string Animation
        {
            get
            {
                return GetAnimation();
            }
            set
            {
                SetAnimation(value);
            }
        }

        /// <summary>
        /// The displayed animation frame's index.
        /// </summary>
        public int Frame
        {
            get
            {
                return GetFrame();
            }
            set
            {
                SetFrame(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the [member animation] is currently playing.
        /// </summary>
        public bool Playing
        {
            get
            {
                return _IsPlaying();
            }
            set
            {
                _SetPlaying(value);
            }
        }


        private const string nativeName = "AnimatedSprite3D";

        public AnimatedSprite3D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AnimatedSprite3D_Ctor(this);
        }

        internal AnimatedSprite3D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_sprite_frames");

        [GodotMethod("set_sprite_frames")]
        public void SetSpriteFrames(SpriteFrames spriteFrames)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(spriteFrames));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_sprite_frames");

        [GodotMethod("get_sprite_frames")]
        public SpriteFrames GetSpriteFrames()
        {
            return NativeCalls.godot_icall_0_44(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_animation");

        [GodotMethod("set_animation")]
        public void SetAnimation(string animation)
        {
            NativeCalls.godot_icall_1_43(method_bind_2, Object.GetPtr(this), animation);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_animation");

        [GodotMethod("get_animation")]
        public string GetAnimation()
        {
            return NativeCalls.godot_icall_0_2(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_playing");

        internal void _SetPlaying(bool playing)
        {
            NativeCalls.godot_icall_1_11(method_bind_4, Object.GetPtr(this), playing);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_is_playing");

        internal bool _IsPlaying()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "play");

        /// <summary>
        /// Play the animation set in parameter. If no parameter is provided, the current animation is played.
        /// </summary>
        [GodotMethod("play")]
        public void Play(string anim = "")
        {
            NativeCalls.godot_icall_1_43(method_bind_6, Object.GetPtr(this), anim);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "stop");

        /// <summary>
        /// Stop the current animation (does not reset the frame counter).
        /// </summary>
        [GodotMethod("stop")]
        public void Stop()
        {
            NativeCalls.godot_icall_0_12(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_playing");

        /// <summary>
        /// Return true if an animation if currently being played.
        /// </summary>
        [GodotMethod("is_playing")]
        public bool IsPlaying()
        {
            return NativeCalls.godot_icall_0_3(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_frame");

        [GodotMethod("set_frame")]
        public void SetFrame(int frame)
        {
            NativeCalls.godot_icall_1_0(method_bind_9, Object.GetPtr(this), frame);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_frame");

        [GodotMethod("get_frame")]
        public int GetFrame()
        {
            return NativeCalls.godot_icall_0_1(method_bind_10, Object.GetPtr(this));
        }
    }
}
