using System;

namespace Godot
{
    /// <summary>
    /// An Animation resource contains data used to animate everything in the engine. Animations are divided into tracks, and each track must be linked to a node. The state of that node can be changed through time, by adding timed keys (events) to the track.
    /// Animations are just data containers, and must be added to odes such as an [AnimationPlayer] or [AnimationTreePlayer] to be played back.
    /// </summary>
    public class Animation : Resource
    {
        public enum TrackType
        {
            Value = 0,
            Transform = 1,
            Method = 2
        }

        public enum UpdateMode
        {
            Continuous = 0,
            Discrete = 1,
            Trigger = 2
        }

        public enum InterpolationType
        {
            Nearest = 0,
            Linear = 1,
            Cubic = 2
        }


        /// <summary>
        /// The total length of the animation (in seconds). Note that length is not delimited by the last key, as this one may be before or after the end to ensure correct interpolation and looping.
        /// </summary>
        public float Length
        {
            get
            {
                return GetLength();
            }
            set
            {
                SetLength(value);
            }
        }

        /// <summary>
        /// A flag indicating that the animation must loop. This is uses for correct interpolation of animation cycles, and for hinting the player that it must restart the animation.
        /// </summary>
        public bool Loop
        {
            get
            {
                return HasLoop();
            }
            set
            {
                SetLoop(value);
            }
        }

        /// <summary>
        /// The animation step value.
        /// </summary>
        public float Step
        {
            get
            {
                return GetStep();
            }
            set
            {
                SetStep(value);
            }
        }


        private const string nativeName = "Animation";

        public Animation() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Animation_Ctor(this);
        }

        internal Animation(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_track");

        /// <summary>
        /// Add a track to the Animation. The track type must be specified as any of the values in the TYPE_* enumeration.
        /// </summary>
        [GodotMethod("add_track")]
        public int AddTrack(Animation.TrackType type, int atPosition = -1)
        {
            return NativeCalls.godot_icall_2_46(method_bind_0, Object.GetPtr(this), type, atPosition);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_track");

        /// <summary>
        /// Remove a track by specifying the track index.
        /// </summary>
        [GodotMethod("remove_track")]
        public void RemoveTrack(int idx)
        {
            NativeCalls.godot_icall_1_0(method_bind_1, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_track_count");

        /// <summary>
        /// Return the amount of tracks in the animation.
        /// </summary>
        [GodotMethod("get_track_count")]
        public int GetTrackCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_get_type");

        /// <summary>
        /// Get the type of a track.
        /// </summary>
        [GodotMethod("track_get_type")]
        public Animation.TrackType TrackGetType(int idx)
        {
            return NativeCalls.godot_icall_1_47(method_bind_3, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_get_path");

        /// <summary>
        /// Get the path of a track. for more information on the path format, see [method track_set_path]
        /// </summary>
        [GodotMethod("track_get_path")]
        public NodePath TrackGetPath(int idx)
        {
            IntPtr ret = NativeCalls.godot_icall_1_48(method_bind_4, Object.GetPtr(this), idx);
            return new NodePath(ret);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_set_path");

        /// <summary>
        /// Set the path of a track. Paths must be valid scene-tree paths to a node, and must be specified starting from the parent node of the node that will reproduce the animation. Tracks that control properties or bones must append their name after the path, separated by ":". Example: "character/skeleton:ankle" or "character/mesh:transform/local"
        /// </summary>
        [GodotMethod("track_set_path")]
        public void TrackSetPath(int idx, NodePath path)
        {
            NativeCalls.godot_icall_2_49(method_bind_5, Object.GetPtr(this), idx, NodePath.GetPtr(path));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "find_track");

        /// <summary>
        /// Return the index of the specified track. If the track is not found, return -1.
        /// </summary>
        [GodotMethod("find_track")]
        public int FindTrack(NodePath path)
        {
            return NativeCalls.godot_icall_1_50(method_bind_6, Object.GetPtr(this), NodePath.GetPtr(path));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_move_up");

        /// <summary>
        /// Move a track up.
        /// </summary>
        [GodotMethod("track_move_up")]
        public void TrackMoveUp(int idx)
        {
            NativeCalls.godot_icall_1_0(method_bind_7, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_move_down");

        /// <summary>
        /// Move a track down.
        /// </summary>
        [GodotMethod("track_move_down")]
        public void TrackMoveDown(int idx)
        {
            NativeCalls.godot_icall_1_0(method_bind_8, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_set_imported");

        /// <summary>
        /// Set the given track as imported or not.
        /// </summary>
        [GodotMethod("track_set_imported")]
        public void TrackSetImported(int idx, bool imported)
        {
            NativeCalls.godot_icall_2_51(method_bind_9, Object.GetPtr(this), idx, imported);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_is_imported");

        /// <summary>
        /// Return true if the given track is imported. Else, return false.
        /// </summary>
        [GodotMethod("track_is_imported")]
        public bool TrackIsImported(int idx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_10, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_set_enabled");

        /// <summary>
        /// Enables/disables the given track. Tracks are enabled by default.
        /// </summary>
        [GodotMethod("track_set_enabled")]
        public void TrackSetEnabled(int idx, bool enabled)
        {
            NativeCalls.godot_icall_2_51(method_bind_11, Object.GetPtr(this), idx, enabled);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_is_enabled");

        /// <summary>
        /// Returns [code]true[/code] if the track at index [code]idx[/code] is enabled.
        /// </summary>
        [GodotMethod("track_is_enabled")]
        public bool TrackIsEnabled(int idx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_12, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "transform_track_insert_key");

        /// <summary>
        /// Insert a transform key for a transform track.
        /// </summary>
        [GodotMethod("transform_track_insert_key")]
        public int TransformTrackInsertKey(int idx, float time, Vector3 location, Quat rotation, Vector3 scale)
        {
            return NativeCalls.godot_icall_5_52(method_bind_13, Object.GetPtr(this), idx, time, ref location, ref rotation, ref scale);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_insert_key");

        /// <summary>
        /// Insert a generic key in a given track.
        /// </summary>
        [GodotMethod("track_insert_key")]
        public void TrackInsertKey(int idx, float time, object key, float transition = (float)1)
        {
            NativeCalls.godot_icall_4_53(method_bind_14, Object.GetPtr(this), idx, time, key, transition);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_remove_key");

        /// <summary>
        /// Remove a key by index in a given track.
        /// </summary>
        [GodotMethod("track_remove_key")]
        public void TrackRemoveKey(int idx, int keyIdx)
        {
            NativeCalls.godot_icall_2_32(method_bind_15, Object.GetPtr(this), idx, keyIdx);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_remove_key_at_position");

        /// <summary>
        /// Remove a key by position (seconds) in a given track.
        /// </summary>
        [GodotMethod("track_remove_key_at_position")]
        public void TrackRemoveKeyAtPosition(int idx, float position)
        {
            NativeCalls.godot_icall_2_28(method_bind_16, Object.GetPtr(this), idx, position);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_set_key_value");

        /// <summary>
        /// Set the value of an existing key.
        /// </summary>
        [GodotMethod("track_set_key_value")]
        public void TrackSetKeyValue(int idx, int key, object value)
        {
            NativeCalls.godot_icall_3_54(method_bind_17, Object.GetPtr(this), idx, key, value);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_set_key_transition");

        /// <summary>
        /// Set the transition curve (easing) for a specific key (see built-in math function "ease").
        /// </summary>
        [GodotMethod("track_set_key_transition")]
        public void TrackSetKeyTransition(int idx, int keyIdx, float transition)
        {
            NativeCalls.godot_icall_3_55(method_bind_18, Object.GetPtr(this), idx, keyIdx, transition);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_get_key_transition");

        /// <summary>
        /// Return the transition curve (easing) for a specific key (see built-in math function "ease").
        /// </summary>
        [GodotMethod("track_get_key_transition")]
        public float TrackGetKeyTransition(int idx, int keyIdx)
        {
            return NativeCalls.godot_icall_2_56(method_bind_19, Object.GetPtr(this), idx, keyIdx);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_get_key_count");

        /// <summary>
        /// Return the amount of keys in a given track.
        /// </summary>
        [GodotMethod("track_get_key_count")]
        public int TrackGetKeyCount(int idx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_20, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_get_key_value");

        /// <summary>
        /// Return the value of a given key in a given track.
        /// </summary>
        [GodotMethod("track_get_key_value")]
        public object TrackGetKeyValue(int idx, int keyIdx)
        {
            return NativeCalls.godot_icall_2_57(method_bind_21, Object.GetPtr(this), idx, keyIdx);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_get_key_time");

        /// <summary>
        /// Return the time at which the key is located.
        /// </summary>
        [GodotMethod("track_get_key_time")]
        public float TrackGetKeyTime(int idx, int keyIdx)
        {
            return NativeCalls.godot_icall_2_56(method_bind_22, Object.GetPtr(this), idx, keyIdx);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_find_key");

        /// <summary>
        /// Find the key index by time in a given track. Optionally, only find it if the exact time is given.
        /// </summary>
        [GodotMethod("track_find_key")]
        public int TrackFindKey(int idx, float time, bool exact = false)
        {
            return NativeCalls.godot_icall_3_58(method_bind_23, Object.GetPtr(this), idx, time, exact);
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_set_interpolation_type");

        /// <summary>
        /// Set the interpolation type of a given track, from the INTERPOLATION_* enum.
        /// </summary>
        [GodotMethod("track_set_interpolation_type")]
        public void TrackSetInterpolationType(int idx, Animation.InterpolationType interpolation)
        {
            NativeCalls.godot_icall_2_59(method_bind_24, Object.GetPtr(this), idx, interpolation);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_get_interpolation_type");

        /// <summary>
        /// Return the interpolation type of a given track, from the INTERPOLATION_* enum.
        /// </summary>
        [GodotMethod("track_get_interpolation_type")]
        public Animation.InterpolationType TrackGetInterpolationType(int idx)
        {
            return NativeCalls.godot_icall_1_60(method_bind_25, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_set_interpolation_loop_wrap");

        /// <summary>
        /// If [code]true[/code] the track at [code]idx[/code] wraps the interpolation loop.
        /// </summary>
        [GodotMethod("track_set_interpolation_loop_wrap")]
        public void TrackSetInterpolationLoopWrap(int idx, bool interpolation)
        {
            NativeCalls.godot_icall_2_51(method_bind_26, Object.GetPtr(this), idx, interpolation);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "track_get_interpolation_loop_wrap");

        /// <summary>
        /// Returns [code]true[/code] if the track at [code]idx[/code] wraps the interpolation loop. Default value: [code]true[/code].
        /// </summary>
        [GodotMethod("track_get_interpolation_loop_wrap")]
        public bool TrackGetInterpolationLoopWrap(int idx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_27, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "transform_track_interpolate");

        /// <summary>
        /// Return the interpolated value of a transform track at a given time (in seconds). An array consisting of 3 elements: position ([Vector3]), rotation ([Quat]) and scale ([Vector3]).
        /// </summary>
        [GodotMethod("transform_track_interpolate")]
        public object[] TransformTrackInterpolate(int idx, float timeSec)
        {
            return NativeCalls.godot_icall_2_61(method_bind_28, Object.GetPtr(this), idx, timeSec);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "value_track_set_update_mode");

        /// <summary>
        /// Set the update mode (UPDATE_*) of a value track.
        /// </summary>
        [GodotMethod("value_track_set_update_mode")]
        public void ValueTrackSetUpdateMode(int idx, Animation.UpdateMode mode)
        {
            NativeCalls.godot_icall_2_62(method_bind_29, Object.GetPtr(this), idx, mode);
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "value_track_get_update_mode");

        /// <summary>
        /// Return the update mode of a value track.
        /// </summary>
        [GodotMethod("value_track_get_update_mode")]
        public Animation.UpdateMode ValueTrackGetUpdateMode(int idx)
        {
            return NativeCalls.godot_icall_1_63(method_bind_30, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "value_track_get_key_indices");

        /// <summary>
        /// Return all the key indices of a value track, given a position and delta time.
        /// </summary>
        [GodotMethod("value_track_get_key_indices")]
        public int[] ValueTrackGetKeyIndices(int idx, float timeSec, float delta)
        {
            return NativeCalls.godot_icall_3_64(method_bind_31, Object.GetPtr(this), idx, timeSec, delta);
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "method_track_get_key_indices");

        /// <summary>
        /// Return all the key indices of a method track, given a position and delta time.
        /// </summary>
        [GodotMethod("method_track_get_key_indices")]
        public int[] MethodTrackGetKeyIndices(int idx, float timeSec, float delta)
        {
            return NativeCalls.godot_icall_3_64(method_bind_32, Object.GetPtr(this), idx, timeSec, delta);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "method_track_get_name");

        /// <summary>
        /// Return the method name of a method track.
        /// </summary>
        [GodotMethod("method_track_get_name")]
        public string MethodTrackGetName(int idx, int keyIdx)
        {
            return NativeCalls.godot_icall_2_65(method_bind_33, Object.GetPtr(this), idx, keyIdx);
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "method_track_get_params");

        /// <summary>
        /// Return the arguments values to be called on a method track for a given key in a given track.
        /// </summary>
        [GodotMethod("method_track_get_params")]
        public object[] MethodTrackGetParams(int idx, int keyIdx)
        {
            return NativeCalls.godot_icall_2_66(method_bind_34, Object.GetPtr(this), idx, keyIdx);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_length");

        [GodotMethod("set_length")]
        public void SetLength(float timeSec)
        {
            NativeCalls.godot_icall_1_10(method_bind_35, Object.GetPtr(this), timeSec);
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_length");

        [GodotMethod("get_length")]
        public float GetLength()
        {
            return NativeCalls.godot_icall_0_9(method_bind_36, Object.GetPtr(this));
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_loop");

        [GodotMethod("set_loop")]
        public void SetLoop(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_37, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_loop");

        [GodotMethod("has_loop")]
        public bool HasLoop()
        {
            return NativeCalls.godot_icall_0_3(method_bind_38, Object.GetPtr(this));
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_step");

        [GodotMethod("set_step")]
        public void SetStep(float sizeSec)
        {
            NativeCalls.godot_icall_1_10(method_bind_39, Object.GetPtr(this), sizeSec);
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_step");

        [GodotMethod("get_step")]
        public float GetStep()
        {
            return NativeCalls.godot_icall_0_9(method_bind_40, Object.GetPtr(this));
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear");

        /// <summary>
        /// Clear the animation (clear all tracks and reset all).
        /// </summary>
        [GodotMethod("clear")]
        public void Clear()
        {
            NativeCalls.godot_icall_0_12(method_bind_41, Object.GetPtr(this));
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "copy_track");

        /// <summary>
        /// Adds a new track that is a copy of the given track from [code]to_animation[/code].
        /// </summary>
        [GodotMethod("copy_track")]
        public void CopyTrack(int track, Animation toAnimation)
        {
            NativeCalls.godot_icall_2_67(method_bind_42, Object.GetPtr(this), track, Object.GetPtr(toAnimation));
        }
    }
}
