using System;

namespace Godot
{
    /// <summary>
    /// An animation player is used for general purpose playback of [Animation] resources. It contains a dictionary of animations (referenced by name) and custom blend times between their transitions. Additionally, animations can be played and blended in different channels.
    /// </summary>
    public class AnimationPlayer : Node
    {
        public enum AnimationProcessMode
        {
            Physics = 0,
            Idle = 1
        }


        /// <summary>
        /// The node from which node path references will travel. Default value: [code]".."[/code].
        /// </summary>
        public NodePath RootNode
        {
            get
            {
                return GetRoot();
            }
            set
            {
                SetRoot(value);
            }
        }

        /// <summary>
        /// The name of the current animation, "" if not playing anything. When being set, does not restart the animation. See also [method play]. Default value: [code]""[/code].
        /// </summary>
        public string CurrentAnimation
        {
            get
            {
                return GetCurrentAnimation();
            }
            set
            {
                SetCurrentAnimation(value);
            }
        }

        /// <summary>
        /// The name of the animation to play when the scene loads. Default value: [code]""[/code].
        /// </summary>
        public string Autoplay
        {
            get
            {
                return GetAutoplay();
            }
            set
            {
                SetAutoplay(value);
            }
        }

        /// <summary>
        /// The length (in seconds) of the currently being played animation.
        /// </summary>
        public float CurrentAnimationLength
        {
            get
            {
                return GetCurrentAnimationLength();
            }
        }

        /// <summary>
        /// The position (in seconds) of the currently playing animation.
        /// </summary>
        public float CurrentAnimationPosition
        {
            get
            {
                return GetCurrentAnimationPosition();
            }
        }

        /// <summary>
        /// The process notification in which to update animations. Default value: [enum ANIMATION_PROCESS_IDLE].
        /// </summary>
        public AnimationPlayer.AnimationProcessMode PlaybackProcessMode
        {
            get
            {
                return GetAnimationProcessMode();
            }
            set
            {
                SetAnimationProcessMode(value);
            }
        }

        /// <summary>
        /// The default time in which to blend animations. Ranges from 0 to 4096 with 0.01 precision. Default value: [code]0[/code].
        /// </summary>
        public float PlaybackDefaultBlendTime
        {
            get
            {
                return GetDefaultBlendTime();
            }
            set
            {
                SetDefaultBlendTime(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], updates animations in response to process-related notifications. Default value: [code]true[/code].
        /// </summary>
        public bool PlaybackActive
        {
            get
            {
                return IsActive();
            }
            set
            {
                SetActive(value);
            }
        }

        /// <summary>
        /// The speed scaling ratio. For instance, if this value is 1 then the animation plays at normal speed. If it's 0.5 then it plays at half speed. If it's 2 then it plays at double speed. Default value: [code]1[/code].
        /// </summary>
        public float PlaybackSpeed
        {
            get
            {
                return GetSpeedScale();
            }
            set
            {
                SetSpeedScale(value);
            }
        }


        private const string nativeName = "AnimationPlayer";

        public AnimationPlayer() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AnimationPlayer_Ctor(this);
        }

        internal AnimationPlayer(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_animation");

        /// <summary>
        /// Adds [code]animation[/code] to the player accessible with the key [code]name[/code].
        /// </summary>
        [GodotMethod("add_animation")]
        public Error AddAnimation(string name, Animation animation)
        {
            return NativeCalls.godot_icall_2_68(method_bind_0, Object.GetPtr(this), name, Object.GetPtr(animation));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_animation");

        /// <summary>
        /// Remove the animation with key [code]name[/code].
        /// </summary>
        [GodotMethod("remove_animation")]
        public void RemoveAnimation(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_1, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rename_animation");

        /// <summary>
        /// Rename an existing animation with key [code]name[/code] to [code]newname[/code].
        /// </summary>
        [GodotMethod("rename_animation")]
        public void RenameAnimation(string name, string newname)
        {
            NativeCalls.godot_icall_2_69(method_bind_2, Object.GetPtr(this), name, newname);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_animation");

        /// <summary>
        /// Returns [code]true[/code] if the [code]AnimationPlayer[/code] stores an [Animation] with key [code]name[/code].
        /// </summary>
        [GodotMethod("has_animation")]
        public bool HasAnimation(string name)
        {
            return NativeCalls.godot_icall_1_70(method_bind_3, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_animation");

        /// <summary>
        /// Returns the [Animation] with key [code]name[/code] or [code]null[/code] if not found.
        /// </summary>
        [GodotMethod("get_animation")]
        public Animation GetAnimation(string name)
        {
            return NativeCalls.godot_icall_1_71(method_bind_4, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_animation_list");

        /// <summary>
        /// Returns the list of stored animation names.
        /// </summary>
        [GodotMethod("get_animation_list")]
        public string[] GetAnimationList()
        {
            return NativeCalls.godot_icall_0_72(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "animation_set_next");

        /// <summary>
        /// Triggers the [code]anim_to[/code] animation when the [code]anim_from[/code] animation completes.
        /// </summary>
        [GodotMethod("animation_set_next")]
        public void AnimationSetNext(string animFrom, string animTo)
        {
            NativeCalls.godot_icall_2_69(method_bind_6, Object.GetPtr(this), animFrom, animTo);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "animation_get_next");

        /// <summary>
        /// Returns the name of the next animation in the queue.
        /// </summary>
        [GodotMethod("animation_get_next")]
        public string AnimationGetNext(string animFrom)
        {
            return NativeCalls.godot_icall_1_73(method_bind_7, Object.GetPtr(this), animFrom);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_blend_time");

        /// <summary>
        /// Specify a blend time (in seconds) between two animations, referenced by their names.
        /// </summary>
        [GodotMethod("set_blend_time")]
        public void SetBlendTime(string animFrom, string animTo, float sec)
        {
            NativeCalls.godot_icall_3_74(method_bind_8, Object.GetPtr(this), animFrom, animTo, sec);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_blend_time");

        /// <summary>
        /// Get the blend time (in seconds) between two animations, referenced by their names.
        /// </summary>
        [GodotMethod("get_blend_time")]
        public float GetBlendTime(string animFrom, string animTo)
        {
            return NativeCalls.godot_icall_2_75(method_bind_9, Object.GetPtr(this), animFrom, animTo);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_default_blend_time");

        [GodotMethod("set_default_blend_time")]
        public void SetDefaultBlendTime(float sec)
        {
            NativeCalls.godot_icall_1_10(method_bind_10, Object.GetPtr(this), sec);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_default_blend_time");

        [GodotMethod("get_default_blend_time")]
        public float GetDefaultBlendTime()
        {
            return NativeCalls.godot_icall_0_9(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "play");

        /// <summary>
        /// Play the animation with key [code]name[/code]. Custom speed and blend times can be set. If custom speed is negative (-1), 'from_end' being true can play the
        /// animation backwards.
        /// </summary>
        [GodotMethod("play")]
        public void Play(string name = "", float customBlend = (float)-1, float customSpeed = 1f, bool fromEnd = false)
        {
            NativeCalls.godot_icall_4_76(method_bind_12, Object.GetPtr(this), name, customBlend, customSpeed, fromEnd);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "play_backwards");

        /// <summary>
        /// Play the animation with key [code]name[/code] in reverse.
        /// </summary>
        [GodotMethod("play_backwards")]
        public void PlayBackwards(string name = "", float customBlend = (float)-1)
        {
            NativeCalls.godot_icall_2_77(method_bind_13, Object.GetPtr(this), name, customBlend);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "stop");

        /// <summary>
        /// Stop the currently playing animation. If [code]reset[/code] is [code]true[/code], the anim position is reset to [code]0[/code].
        /// </summary>
        [GodotMethod("stop")]
        public void Stop(bool reset = true)
        {
            NativeCalls.godot_icall_1_11(method_bind_14, Object.GetPtr(this), reset);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_playing");

        /// <summary>
        /// Returns [code]true[/code] if playing an animation.
        /// </summary>
        [GodotMethod("is_playing")]
        public bool IsPlaying()
        {
            return NativeCalls.godot_icall_0_3(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_current_animation");

        [GodotMethod("set_current_animation")]
        public void SetCurrentAnimation(string anim)
        {
            NativeCalls.godot_icall_1_43(method_bind_16, Object.GetPtr(this), anim);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_current_animation");

        [GodotMethod("get_current_animation")]
        public string GetCurrentAnimation()
        {
            return NativeCalls.godot_icall_0_2(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "queue");

        /// <summary>
        /// Queue an animation for playback once the current one is done.
        /// </summary>
        [GodotMethod("queue")]
        public void Queue(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_18, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_queue");

        /// <summary>
        /// Clears all queued, unplayed animations.
        /// </summary>
        [GodotMethod("clear_queue")]
        public void ClearQueue()
        {
            NativeCalls.godot_icall_0_12(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_active");

        [GodotMethod("set_active")]
        public void SetActive(bool active)
        {
            NativeCalls.godot_icall_1_11(method_bind_20, Object.GetPtr(this), active);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_active");

        [GodotMethod("is_active")]
        public bool IsActive()
        {
            return NativeCalls.godot_icall_0_3(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_speed_scale");

        [GodotMethod("set_speed_scale")]
        public void SetSpeedScale(float speed)
        {
            NativeCalls.godot_icall_1_10(method_bind_22, Object.GetPtr(this), speed);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_speed_scale");

        [GodotMethod("get_speed_scale")]
        public float GetSpeedScale()
        {
            return NativeCalls.godot_icall_0_9(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_autoplay");

        [GodotMethod("set_autoplay")]
        public void SetAutoplay(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_24, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_autoplay");

        [GodotMethod("get_autoplay")]
        public string GetAutoplay()
        {
            return NativeCalls.godot_icall_0_2(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_root");

        [GodotMethod("set_root")]
        public void SetRoot(NodePath path)
        {
            NativeCalls.godot_icall_1_78(method_bind_26, Object.GetPtr(this), NodePath.GetPtr(path));
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_root");

        [GodotMethod("get_root")]
        public NodePath GetRoot()
        {
            IntPtr ret = NativeCalls.godot_icall_0_79(method_bind_27, Object.GetPtr(this));
            return new NodePath(ret);
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "find_animation");

        /// <summary>
        /// Returns the name of [code]animation[/code] or empty string if not found.
        /// </summary>
        [GodotMethod("find_animation")]
        public string FindAnimation(Animation animation)
        {
            return NativeCalls.godot_icall_1_80(method_bind_28, Object.GetPtr(this), Object.GetPtr(animation));
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_caches");

        /// <summary>
        /// [code]AnimationPlayer[/code] caches animated nodes. It may not notice if a node disappears, so clear_caches forces it to update the cache again.
        /// </summary>
        [GodotMethod("clear_caches")]
        public void ClearCaches()
        {
            NativeCalls.godot_icall_0_12(method_bind_29, Object.GetPtr(this));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_animation_process_mode");

        [GodotMethod("set_animation_process_mode")]
        public void SetAnimationProcessMode(AnimationPlayer.AnimationProcessMode mode)
        {
            NativeCalls.godot_icall_1_81(method_bind_30, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_animation_process_mode");

        [GodotMethod("get_animation_process_mode")]
        public AnimationPlayer.AnimationProcessMode GetAnimationProcessMode()
        {
            return NativeCalls.godot_icall_0_82(method_bind_31, Object.GetPtr(this));
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_current_animation_position");

        [GodotMethod("get_current_animation_position")]
        public float GetCurrentAnimationPosition()
        {
            return NativeCalls.godot_icall_0_9(method_bind_32, Object.GetPtr(this));
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_current_animation_length");

        [GodotMethod("get_current_animation_length")]
        public float GetCurrentAnimationLength()
        {
            return NativeCalls.godot_icall_0_9(method_bind_33, Object.GetPtr(this));
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "seek");

        /// <summary>
        /// Seek the animation to the [code]seconds[/code] point in time (in seconds). If [code]update[/code] is [code]true[/code], the animation updates too, otherwise it updates at process time.
        /// </summary>
        [GodotMethod("seek")]
        public void Seek(float seconds, bool update = false)
        {
            NativeCalls.godot_icall_2_83(method_bind_34, Object.GetPtr(this), seconds, update);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "advance");

        /// <summary>
        /// Shifts position in the animation timeline. Delta is the time in seconds to shift.
        /// </summary>
        [GodotMethod("advance")]
        public void Advance(float delta)
        {
            NativeCalls.godot_icall_1_10(method_bind_35, Object.GetPtr(this), delta);
        }
    }
}
