using System;

namespace Godot
{
    /// <summary>
    /// A node graph tool for blending multiple animations bound to an [AnimationPlayer]. Especially useful for animating characters or other skeleton-based rigs. It can combine several animations to form a desired pose.
    /// It takes [Animation]s from an [AnimationPlayer] node and mixes them depending on the graph.
    /// </summary>
    public class AnimationTreePlayer : Node
    {
        public enum AnimationProcessMode
        {
            Physics = 0,
            Idle = 1
        }

        public enum NodeType
        {
            Output = 0,
            Animation = 1,
            Oneshot = 2,
            Mix = 3,
            Blend2 = 4,
            Blend3 = 5,
            Blend4 = 6,
            Timescale = 7,
            Timeseek = 8,
            Transition = 9
        }


        /// <summary>
        /// The thread in which to update animations. Default value: [enum ANIMATION_PROCESS_IDLE].
        /// </summary>
        public AnimationTreePlayer.AnimationProcessMode PlaybackProcessMode
        {
            get
            {
                return GetAnimationProcessMode();
            }
            set
            {
                SetAnimationProcessMode(value);
            }
        }

        /// <summary>
        /// The path to the [AnimationPlayer] from which this [code]AnimationTreePlayer[/code] binds animations to animation nodes.
        /// Once set, Animation nodes can be added to the AnimationTreePlayer.
        /// </summary>
        public NodePath MasterPlayer
        {
            get
            {
                return GetMasterPlayer();
            }
            set
            {
                SetMasterPlayer(value);
            }
        }

        /// <summary>
        /// The node from which to relatively access other nodes. Default value: [code]".."[/code].
        /// It accesses the Bones, so it should point to the same Node the AnimationPlayer would point its Root Node at.
        /// </summary>
        public NodePath BasePath
        {
            get
            {
                return GetBasePath();
            }
            set
            {
                SetBasePath(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the [code]AnimationTreePlayer[/code] is able to play animations. Default value: [code]false[/code].
        /// </summary>
        public bool Active
        {
            get
            {
                return IsActive();
            }
            set
            {
                SetActive(value);
            }
        }


        private const string nativeName = "AnimationTreePlayer";

        public AnimationTreePlayer() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AnimationTreePlayer_Ctor(this);
        }

        internal AnimationTreePlayer(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_node");

        /// <summary>
        /// Adds a [code]type[/code] node to the graph with name [code]id[/code].
        /// </summary>
        [GodotMethod("add_node")]
        public void AddNode(AnimationTreePlayer.NodeType type, string id)
        {
            NativeCalls.godot_icall_2_84(method_bind_0, Object.GetPtr(this), type, id);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "node_exists");

        /// <summary>
        /// Check if a node exists (by name).
        /// </summary>
        [GodotMethod("node_exists")]
        public bool NodeExists(string node)
        {
            return NativeCalls.godot_icall_1_70(method_bind_1, Object.GetPtr(this), node);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "node_rename");

        /// <summary>
        /// Rename a node in the graph.
        /// </summary>
        [GodotMethod("node_rename")]
        public Error NodeRename(string node, string newName)
        {
            return NativeCalls.godot_icall_2_85(method_bind_2, Object.GetPtr(this), node, newName);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "node_get_type");

        /// <summary>
        /// Get the node type, will return from NODE_* enum.
        /// </summary>
        [GodotMethod("node_get_type")]
        public AnimationTreePlayer.NodeType NodeGetType(string id)
        {
            return NativeCalls.godot_icall_1_86(method_bind_3, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "node_get_input_count");

        /// <summary>
        /// Return the input count for a given node. Different types of nodes have different amount of inputs.
        /// </summary>
        [GodotMethod("node_get_input_count")]
        public int NodeGetInputCount(string id)
        {
            return NativeCalls.godot_icall_1_87(method_bind_4, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "node_get_input_source");

        /// <summary>
        /// Return the input source for a given node input.
        /// </summary>
        [GodotMethod("node_get_input_source")]
        public string NodeGetInputSource(string id, int idx)
        {
            return NativeCalls.godot_icall_2_88(method_bind_5, Object.GetPtr(this), id, idx);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "animation_node_set_animation");

        /// <summary>
        /// Binds a new [Animation] from the [member master_player] to the [code]AnimationTreePlayer[/code]'s animation node with name [code]id[/code].
        /// </summary>
        [GodotMethod("animation_node_set_animation")]
        public void AnimationNodeSetAnimation(string id, Animation animation)
        {
            NativeCalls.godot_icall_2_89(method_bind_6, Object.GetPtr(this), id, Object.GetPtr(animation));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "animation_node_get_animation");

        /// <summary>
        /// Returns the [AnimationPlayer]'s [Animation] bound to the [code]AnimationTreePlayer[/code]'s animation node with name [code]id[/code].
        /// </summary>
        [GodotMethod("animation_node_get_animation")]
        public Animation AnimationNodeGetAnimation(string id)
        {
            return NativeCalls.godot_icall_1_71(method_bind_7, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "animation_node_set_master_animation");

        /// <summary>
        /// Binds the [Animation] named [code]source[/code] from [member master_player] to the animation node [code]id[/code]. Recalculates caches.
        /// </summary>
        [GodotMethod("animation_node_set_master_animation")]
        public void AnimationNodeSetMasterAnimation(string id, string source)
        {
            NativeCalls.godot_icall_2_69(method_bind_8, Object.GetPtr(this), id, source);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "animation_node_get_master_animation");

        /// <summary>
        /// Returns the name of the [member master_player]'s [Animation] bound to this animation node.
        /// </summary>
        [GodotMethod("animation_node_get_master_animation")]
        public string AnimationNodeGetMasterAnimation(string id)
        {
            return NativeCalls.godot_icall_1_73(method_bind_9, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "animation_node_set_filter_path");

        /// <summary>
        /// If [code]enable[/code] is [code]true[/code], the animation node with ID [code]id[/code] turns off the track modifying the property at [code]path[/code]. The modified node's children continue to animate.
        /// </summary>
        [GodotMethod("animation_node_set_filter_path")]
        public void AnimationNodeSetFilterPath(string id, NodePath path, bool enable)
        {
            NativeCalls.godot_icall_3_90(method_bind_10, Object.GetPtr(this), id, NodePath.GetPtr(path), enable);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "oneshot_node_set_fadein_time");

        /// <summary>
        /// Sets fade in time of a OneShot node given its name and value in seconds.
        /// </summary>
        [GodotMethod("oneshot_node_set_fadein_time")]
        public void OneshotNodeSetFadeinTime(string id, float timeSec)
        {
            NativeCalls.godot_icall_2_77(method_bind_11, Object.GetPtr(this), id, timeSec);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "oneshot_node_get_fadein_time");

        /// <summary>
        /// Returns fade in time of a OneShot node given its name.
        /// </summary>
        [GodotMethod("oneshot_node_get_fadein_time")]
        public float OneshotNodeGetFadeinTime(string id)
        {
            return NativeCalls.godot_icall_1_91(method_bind_12, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "oneshot_node_set_fadeout_time");

        /// <summary>
        /// Sets fade out time of a OneShot node given its name and value in seconds.
        /// </summary>
        [GodotMethod("oneshot_node_set_fadeout_time")]
        public void OneshotNodeSetFadeoutTime(string id, float timeSec)
        {
            NativeCalls.godot_icall_2_77(method_bind_13, Object.GetPtr(this), id, timeSec);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "oneshot_node_get_fadeout_time");

        /// <summary>
        /// Returns fade out time of a OneShot node given its name.
        /// </summary>
        [GodotMethod("oneshot_node_get_fadeout_time")]
        public float OneshotNodeGetFadeoutTime(string id)
        {
            return NativeCalls.godot_icall_1_91(method_bind_14, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "oneshot_node_set_autorestart");

        /// <summary>
        /// Sets autorestart property of a OneShot node given its name and value.
        /// </summary>
        [GodotMethod("oneshot_node_set_autorestart")]
        public void OneshotNodeSetAutorestart(string id, bool enable)
        {
            NativeCalls.godot_icall_2_92(method_bind_15, Object.GetPtr(this), id, enable);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "oneshot_node_set_autorestart_delay");

        /// <summary>
        /// Sets autorestart delay of a OneShot node given its name and value in seconds.
        /// </summary>
        [GodotMethod("oneshot_node_set_autorestart_delay")]
        public void OneshotNodeSetAutorestartDelay(string id, float delaySec)
        {
            NativeCalls.godot_icall_2_77(method_bind_16, Object.GetPtr(this), id, delaySec);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "oneshot_node_set_autorestart_random_delay");

        /// <summary>
        /// Sets autorestart random delay of a OneShot node given its name and value in seconds.
        /// </summary>
        [GodotMethod("oneshot_node_set_autorestart_random_delay")]
        public void OneshotNodeSetAutorestartRandomDelay(string id, float randSec)
        {
            NativeCalls.godot_icall_2_77(method_bind_17, Object.GetPtr(this), id, randSec);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "oneshot_node_has_autorestart");

        /// <summary>
        /// Returns whether a OneShot node will auto restart given its name.
        /// </summary>
        [GodotMethod("oneshot_node_has_autorestart")]
        public bool OneshotNodeHasAutorestart(string id)
        {
            return NativeCalls.godot_icall_1_70(method_bind_18, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "oneshot_node_get_autorestart_delay");

        /// <summary>
        /// Returns autostart delay of a OneShot node given its name.
        /// </summary>
        [GodotMethod("oneshot_node_get_autorestart_delay")]
        public float OneshotNodeGetAutorestartDelay(string id)
        {
            return NativeCalls.godot_icall_1_91(method_bind_19, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "oneshot_node_get_autorestart_random_delay");

        /// <summary>
        /// Returns autostart random delay of a OneShot node given its name.
        /// </summary>
        [GodotMethod("oneshot_node_get_autorestart_random_delay")]
        public float OneshotNodeGetAutorestartRandomDelay(string id)
        {
            return NativeCalls.godot_icall_1_91(method_bind_20, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "oneshot_node_start");

        /// <summary>
        /// Starts a OneShot node given its name.
        /// </summary>
        [GodotMethod("oneshot_node_start")]
        public void OneshotNodeStart(string id)
        {
            NativeCalls.godot_icall_1_43(method_bind_21, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "oneshot_node_stop");

        /// <summary>
        /// Stops the OneShot node with name [code]id[/code].
        /// </summary>
        [GodotMethod("oneshot_node_stop")]
        public void OneshotNodeStop(string id)
        {
            NativeCalls.godot_icall_1_43(method_bind_22, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "oneshot_node_is_active");

        /// <summary>
        /// Returns whether a OneShot node is active given its name.
        /// </summary>
        [GodotMethod("oneshot_node_is_active")]
        public bool OneshotNodeIsActive(string id)
        {
            return NativeCalls.godot_icall_1_70(method_bind_23, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "oneshot_node_set_filter_path");

        /// <summary>
        /// If [code]enable[/code] is [code]true[/code], the oneshot node with ID [code]id[/code] turns off the track modifying the property at [code]path[/code]. The modified node's children continue to animate.
        /// </summary>
        [GodotMethod("oneshot_node_set_filter_path")]
        public void OneshotNodeSetFilterPath(string id, NodePath path, bool enable)
        {
            NativeCalls.godot_icall_3_90(method_bind_24, Object.GetPtr(this), id, NodePath.GetPtr(path), enable);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mix_node_set_amount");

        /// <summary>
        /// Sets mix amount of a Mix node given its name and value.
        /// A Mix node adds input b to input a by a the amount given by ratio.
        /// </summary>
        [GodotMethod("mix_node_set_amount")]
        public void MixNodeSetAmount(string id, float ratio)
        {
            NativeCalls.godot_icall_2_77(method_bind_25, Object.GetPtr(this), id, ratio);
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "mix_node_get_amount");

        /// <summary>
        /// Returns mix amount of a Mix node given its name.
        /// </summary>
        [GodotMethod("mix_node_get_amount")]
        public float MixNodeGetAmount(string id)
        {
            return NativeCalls.godot_icall_1_91(method_bind_26, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "blend2_node_set_amount");

        /// <summary>
        /// Sets the blend amount of a Blend2 node given its name and value.
        /// A Blend2 Node blends two animations with the amount between 0 and 1.
        /// At 0, Output is input a.
        /// Towards 1, the influence of a gets lessened, the influence of b gets raised.
        /// At 1, Output is input b.
        /// </summary>
        [GodotMethod("blend2_node_set_amount")]
        public void Blend2NodeSetAmount(string id, float blend)
        {
            NativeCalls.godot_icall_2_77(method_bind_27, Object.GetPtr(this), id, blend);
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "blend2_node_get_amount");

        /// <summary>
        /// Returns the blend amount of a Blend2 node given its name.
        /// </summary>
        [GodotMethod("blend2_node_get_amount")]
        public float Blend2NodeGetAmount(string id)
        {
            return NativeCalls.godot_icall_1_91(method_bind_28, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "blend2_node_set_filter_path");

        /// <summary>
        /// If [code]enable[/code] is [code]true[/code], the blend2 node with ID [code]id[/code] turns off the track modifying the property at [code]path[/code]. The modified node's children continue to animate.
        /// </summary>
        [GodotMethod("blend2_node_set_filter_path")]
        public void Blend2NodeSetFilterPath(string id, NodePath path, bool enable)
        {
            NativeCalls.godot_icall_3_90(method_bind_29, Object.GetPtr(this), id, NodePath.GetPtr(path), enable);
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "blend3_node_set_amount");

        /// <summary>
        /// Sets the blend amount of a Blend3 node given its name and value.
        /// A Blend3 Node blends three animations with the amount between -1 and 1.
        /// At -1, Output is input b-.
        /// From -1 to 0, the influence of b- gets lessened, the influence of a gets raised and the influence of b+ is 0.
        /// At 0, Output is input a.
        /// From 0 to 1, the influence of a gets lessened, the influence of b+ gets raised and the influence of b+ is 0.
        /// At 1, Output is input b+.
        /// </summary>
        [GodotMethod("blend3_node_set_amount")]
        public void Blend3NodeSetAmount(string id, float blend)
        {
            NativeCalls.godot_icall_2_77(method_bind_30, Object.GetPtr(this), id, blend);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "blend3_node_get_amount");

        /// <summary>
        /// Returns the blend amount of a Blend3 node given its name.
        /// </summary>
        [GodotMethod("blend3_node_get_amount")]
        public float Blend3NodeGetAmount(string id)
        {
            return NativeCalls.godot_icall_1_91(method_bind_31, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "blend4_node_set_amount");

        /// <summary>
        /// Sets the blend amount of a Blend4 node given its name and value.
        /// A Blend4 Node blends two pairs of animations.
        /// The two pairs are blended like blend2 and then added together.
        /// </summary>
        [GodotMethod("blend4_node_set_amount")]
        public void Blend4NodeSetAmount(string id, Vector2 blend)
        {
            NativeCalls.godot_icall_2_93(method_bind_32, Object.GetPtr(this), id, ref blend);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "blend4_node_get_amount");

        /// <summary>
        /// Returns the blend amount of a Blend4 node given its name.
        /// </summary>
        [GodotMethod("blend4_node_get_amount")]
        public Vector2 Blend4NodeGetAmount(string id)
        {
            object ret = NativeCalls.godot_icall_1_94(method_bind_33, Object.GetPtr(this), id);
            return (Vector2)ret;
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "timescale_node_set_scale");

        /// <summary>
        /// Sets the time scale of the TimeScale node with name [code]id[/code] to [code]scale[/code].
        /// The timescale node is used to speed [Animation]s up if the scale is above 1 or slow them down if it is below 1.
        /// If applied after a blend or mix, affects all input animations to that blend or mix.
        /// </summary>
        [GodotMethod("timescale_node_set_scale")]
        public void TimescaleNodeSetScale(string id, float scale)
        {
            NativeCalls.godot_icall_2_77(method_bind_34, Object.GetPtr(this), id, scale);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "timescale_node_get_scale");

        /// <summary>
        /// Returns time scale value of the TimeScale node with name [code]id[/code].
        /// </summary>
        [GodotMethod("timescale_node_get_scale")]
        public float TimescaleNodeGetScale(string id)
        {
            return NativeCalls.godot_icall_1_91(method_bind_35, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "timeseek_node_seek");

        /// <summary>
        /// Sets the time seek value of the TimeSeek node with name [code]id[/code] to [code]seconds[/code]
        /// This functions as a seek in the [Animation] or the blend or mix of [Animation]s input in it.
        /// </summary>
        [GodotMethod("timeseek_node_seek")]
        public void TimeseekNodeSeek(string id, float seconds)
        {
            NativeCalls.godot_icall_2_77(method_bind_36, Object.GetPtr(this), id, seconds);
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "transition_node_set_input_count");

        /// <summary>
        /// Resizes the number of inputs available for the transition node with name [code]id[/code].
        /// </summary>
        [GodotMethod("transition_node_set_input_count")]
        public void TransitionNodeSetInputCount(string id, int count)
        {
            NativeCalls.godot_icall_2_95(method_bind_37, Object.GetPtr(this), id, count);
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "transition_node_get_input_count");

        /// <summary>
        /// Returns the number of inputs for the transition node with name [code]id[/code]. You can add inputs by rightclicking on the transition node.
        /// </summary>
        [GodotMethod("transition_node_get_input_count")]
        public int TransitionNodeGetInputCount(string id)
        {
            return NativeCalls.godot_icall_1_87(method_bind_38, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "transition_node_delete_input");

        /// <summary>
        /// Deletes the input at [code]input_idx[/code] for the transition node with name [code]id[/code].
        /// </summary>
        [GodotMethod("transition_node_delete_input")]
        public void TransitionNodeDeleteInput(string id, int inputIdx)
        {
            NativeCalls.godot_icall_2_95(method_bind_39, Object.GetPtr(this), id, inputIdx);
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "transition_node_set_input_auto_advance");

        /// <summary>
        /// The transition node with name [code]id[/code] advances to its next input automatically when the input at [code]input_idx[/code] completes.
        /// </summary>
        [GodotMethod("transition_node_set_input_auto_advance")]
        public void TransitionNodeSetInputAutoAdvance(string id, int inputIdx, bool enable)
        {
            NativeCalls.godot_icall_3_96(method_bind_40, Object.GetPtr(this), id, inputIdx, enable);
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "transition_node_has_input_auto_advance");

        /// <summary>
        /// Returns [code]true[/code] if the input at [code]input_idx[/code] on transition node with name [code]id[/code] is set to automatically advance to the next input upon completion.
        /// </summary>
        [GodotMethod("transition_node_has_input_auto_advance")]
        public bool TransitionNodeHasInputAutoAdvance(string id, int inputIdx)
        {
            return NativeCalls.godot_icall_2_97(method_bind_41, Object.GetPtr(this), id, inputIdx);
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "transition_node_set_xfade_time");

        /// <summary>
        /// The transition node with name [code]id[/code] sets its cross fade time to [code]time_sec[/code].
        /// </summary>
        [GodotMethod("transition_node_set_xfade_time")]
        public void TransitionNodeSetXfadeTime(string id, float timeSec)
        {
            NativeCalls.godot_icall_2_77(method_bind_42, Object.GetPtr(this), id, timeSec);
        }

        private IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "transition_node_get_xfade_time");

        /// <summary>
        /// Returns the cross fade time for the transition node with name [code]id[/code].
        /// </summary>
        [GodotMethod("transition_node_get_xfade_time")]
        public float TransitionNodeGetXfadeTime(string id)
        {
            return NativeCalls.godot_icall_1_91(method_bind_43, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_44 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "transition_node_set_current");

        /// <summary>
        /// The transition node with name [code]id[/code] sets its current input at [code]input_idx[/code].
        /// </summary>
        [GodotMethod("transition_node_set_current")]
        public void TransitionNodeSetCurrent(string id, int inputIdx)
        {
            NativeCalls.godot_icall_2_95(method_bind_44, Object.GetPtr(this), id, inputIdx);
        }

        private IntPtr method_bind_45 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "transition_node_get_current");

        /// <summary>
        /// Returns the index of the currently evaluated input for the transition node with name [code]id[/code].
        /// </summary>
        [GodotMethod("transition_node_get_current")]
        public int TransitionNodeGetCurrent(string id)
        {
            return NativeCalls.godot_icall_1_87(method_bind_45, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_46 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "node_set_position");

        /// <summary>
        /// Sets position of a node in the graph given its name and position.
        /// </summary>
        [GodotMethod("node_set_position")]
        public void NodeSetPosition(string id, Vector2 screenPosition)
        {
            NativeCalls.godot_icall_2_93(method_bind_46, Object.GetPtr(this), id, ref screenPosition);
        }

        private IntPtr method_bind_47 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "node_get_position");

        /// <summary>
        /// Returns position of a node in the graph given its name.
        /// </summary>
        [GodotMethod("node_get_position")]
        public Vector2 NodeGetPosition(string id)
        {
            object ret = NativeCalls.godot_icall_1_94(method_bind_47, Object.GetPtr(this), id);
            return (Vector2)ret;
        }

        private IntPtr method_bind_48 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_node");

        /// <summary>
        /// Removes the animation node with name [code]id[/code].
        /// </summary>
        [GodotMethod("remove_node")]
        public void RemoveNode(string id)
        {
            NativeCalls.godot_icall_1_43(method_bind_48, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_49 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "connect_nodes");

        /// <summary>
        /// Connects node [code]id[/code] to [code]dst_id[/code] at the specified input slot.
        /// </summary>
        [GodotMethod("connect_nodes")]
        public Error ConnectNodes(string id, string dstId, int dstInputIdx)
        {
            return NativeCalls.godot_icall_3_98(method_bind_49, Object.GetPtr(this), id, dstId, dstInputIdx);
        }

        private IntPtr method_bind_50 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "are_nodes_connected");

        /// <summary>
        /// Returns whether node [code]id[/code] and [code]dst_id[/code] are connected at the specified slot.
        /// </summary>
        [GodotMethod("are_nodes_connected")]
        public bool AreNodesConnected(string id, string dstId, int dstInputIdx)
        {
            return NativeCalls.godot_icall_3_99(method_bind_50, Object.GetPtr(this), id, dstId, dstInputIdx);
        }

        private IntPtr method_bind_51 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "disconnect_nodes");

        /// <summary>
        /// Disconnects nodes connected to [code]id[/code] at the specified input slot.
        /// </summary>
        [GodotMethod("disconnect_nodes")]
        public void DisconnectNodes(string id, int dstInputIdx)
        {
            NativeCalls.godot_icall_2_95(method_bind_51, Object.GetPtr(this), id, dstInputIdx);
        }

        private IntPtr method_bind_52 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_active");

        [GodotMethod("set_active")]
        public void SetActive(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_52, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_53 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_active");

        [GodotMethod("is_active")]
        public bool IsActive()
        {
            return NativeCalls.godot_icall_0_3(method_bind_53, Object.GetPtr(this));
        }

        private IntPtr method_bind_54 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_base_path");

        [GodotMethod("set_base_path")]
        public void SetBasePath(NodePath path)
        {
            NativeCalls.godot_icall_1_78(method_bind_54, Object.GetPtr(this), NodePath.GetPtr(path));
        }

        private IntPtr method_bind_55 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_base_path");

        [GodotMethod("get_base_path")]
        public NodePath GetBasePath()
        {
            IntPtr ret = NativeCalls.godot_icall_0_79(method_bind_55, Object.GetPtr(this));
            return new NodePath(ret);
        }

        private IntPtr method_bind_56 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_master_player");

        [GodotMethod("set_master_player")]
        public void SetMasterPlayer(NodePath nodepath)
        {
            NativeCalls.godot_icall_1_78(method_bind_56, Object.GetPtr(this), NodePath.GetPtr(nodepath));
        }

        private IntPtr method_bind_57 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_master_player");

        [GodotMethod("get_master_player")]
        public NodePath GetMasterPlayer()
        {
            IntPtr ret = NativeCalls.godot_icall_0_79(method_bind_57, Object.GetPtr(this));
            return new NodePath(ret);
        }

        private IntPtr method_bind_58 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_list");

        /// <summary>
        /// Returns a [PoolStringArray] containing the name of all nodes.
        /// </summary>
        [GodotMethod("get_node_list")]
        public string[] GetNodeList()
        {
            return NativeCalls.godot_icall_0_72(method_bind_58, Object.GetPtr(this));
        }

        private IntPtr method_bind_59 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_animation_process_mode");

        [GodotMethod("set_animation_process_mode")]
        public void SetAnimationProcessMode(AnimationTreePlayer.AnimationProcessMode mode)
        {
            NativeCalls.godot_icall_1_100(method_bind_59, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_60 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_animation_process_mode");

        [GodotMethod("get_animation_process_mode")]
        public AnimationTreePlayer.AnimationProcessMode GetAnimationProcessMode()
        {
            return NativeCalls.godot_icall_0_101(method_bind_60, Object.GetPtr(this));
        }

        private IntPtr method_bind_61 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "advance");

        /// <summary>
        /// Shifts position in the animation timeline. Delta is the time in seconds to shift.
        /// </summary>
        [GodotMethod("advance")]
        public void Advance(float delta)
        {
            NativeCalls.godot_icall_1_10(method_bind_61, Object.GetPtr(this), delta);
        }

        private IntPtr method_bind_62 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "reset");

        /// <summary>
        /// Resets this [code]AnimationTreePlayer[/code].
        /// </summary>
        [GodotMethod("reset")]
        public void Reset()
        {
            NativeCalls.godot_icall_0_12(method_bind_62, Object.GetPtr(this));
        }

        private IntPtr method_bind_63 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "recompute_caches");

        /// <summary>
        /// Manually recalculates the cache of track information generated from animation nodes. Needed when external sources modify the animation nodes' state.
        /// </summary>
        [GodotMethod("recompute_caches")]
        public void RecomputeCaches()
        {
            NativeCalls.godot_icall_0_12(method_bind_63, Object.GetPtr(this));
        }
    }
}
