using System;

namespace Godot
{
    /// <summary>
    /// 3D area that detects [CollisionObject] nodes overlapping, entering, or exiting. Can also alter or override local physics parameters (gravity, damping).
    /// </summary>
    public class Area : CollisionObject
    {
        public enum SpaceOverrideEnum
        {
            Disabled = 0,
            Combine = 1,
            CombineReplace = 2,
            Replace = 3,
            ReplaceCombine = 4
        }


        /// <summary>
        /// Override mode for gravity and damping calculations within this area. See the SPACE_OVERRIDE_* constants for values.
        /// </summary>
        public Area.SpaceOverrideEnum SpaceOverride
        {
            get
            {
                return GetSpaceOverrideMode();
            }
            set
            {
                SetSpaceOverrideMode(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] gravity is calculated from a point (set via [code]gravity_vec[/code]). Also see [code]space_override[/code]. Default value: [code]false[/code].
        /// </summary>
        public bool GravityPoint
        {
            get
            {
                return IsGravityAPoint();
            }
            set
            {
                SetGravityIsPoint(value);
            }
        }

        /// <summary>
        /// The falloff factor for point gravity. The greater the value, the faster gravity decreases with distance.
        /// </summary>
        public float GravityDistanceScale
        {
            get
            {
                return GetGravityDistanceScale();
            }
            set
            {
                SetGravityDistanceScale(value);
            }
        }

        /// <summary>
        /// The area's gravity vector (not normalized). If gravity is a point (see [method is_gravity_a_point]), this will be the point of attraction.
        /// </summary>
        public Vector3 GravityVec
        {
            get
            {
                return GetGravityVector();
            }
            set
            {
                SetGravityVector(value);
            }
        }

        /// <summary>
        /// The area's gravity intensity (ranges from -1024 to 1024). This value multiplies the gravity vector. This is useful to alter the force of gravity without altering its direction.
        /// </summary>
        public float Gravity
        {
            get
            {
                return GetGravity();
            }
            set
            {
                SetGravity(value);
            }
        }

        /// <summary>
        /// The rate at which objects stop moving in this area. Represents the linear velocity lost per second. Values range from [code]0[/code] (no damping) to [code]1[/code] (full damping).
        /// </summary>
        public float LinearDamp
        {
            get
            {
                return GetLinearDamp();
            }
            set
            {
                SetLinearDamp(value);
            }
        }

        /// <summary>
        /// The rate at which objects stop spinning in this area. Represents the angular velocity lost per second. Values range from [code]0[/code] (no damping) to [code]1[/code] (full damping).
        /// </summary>
        public float AngularDamp
        {
            get
            {
                return GetAngularDamp();
            }
            set
            {
                SetAngularDamp(value);
            }
        }

        /// <summary>
        /// The area's priority. Higher priority areas are processed first. Default value: 0.
        /// </summary>
        public float Priority
        {
            get
            {
                return GetPriority();
            }
            set
            {
                SetPriority(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the area detects bodies or areas entering and exiting it. Default value: [code]true[/code].
        /// </summary>
        public bool Monitoring
        {
            get
            {
                return IsMonitoring();
            }
            set
            {
                SetMonitoring(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] other monitoring areas can detect this area. Default value: [code]true[/code].
        /// </summary>
        public bool Monitorable
        {
            get
            {
                return IsMonitorable();
            }
            set
            {
                SetMonitorable(value);
            }
        }

        /// <summary>
        /// The area's physics layer(s). Collidable objects can exist in any of 32 different layers. A contact is detected if object A is in any of the layers that object B scans, or object B is in any layers that object A scans. See also [code]collision_mask[/code].
        /// </summary>
        public int CollisionLayer
        {
            get
            {
                return GetCollisionLayer();
            }
            set
            {
                SetCollisionLayer(value);
            }
        }

        /// <summary>
        /// The physics layers this area scans to determine collision detection.
        /// </summary>
        public int CollisionMask
        {
            get
            {
                return GetCollisionMask();
            }
            set
            {
                SetCollisionMask(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the area's audio bus overrides the default audio bus. Default value: [code]false[/code].
        /// </summary>
        public bool AudioBusOverride
        {
            get
            {
                return IsOverridingAudioBus();
            }
            set
            {
                SetAudioBusOverride(value);
            }
        }

        /// <summary>
        /// The name of the area's audio bus.
        /// </summary>
        public string AudioBusName
        {
            get
            {
                return GetAudioBus();
            }
            set
            {
                SetAudioBus(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the area applies reverb to its associated audio.
        /// </summary>
        public bool ReverbBusEnable
        {
            get
            {
                return IsUsingReverbBus();
            }
            set
            {
                SetUseReverbBus(value);
            }
        }

        /// <summary>
        /// The reverb bus name to use for this area's associated audio.
        /// </summary>
        public string ReverbBusName
        {
            get
            {
                return GetReverbBus();
            }
            set
            {
                SetReverbBus(value);
            }
        }

        /// <summary>
        /// The degree to which this area applies reverb to its associated audio. Ranges from [code]0[/code] to [code]1[/code] with [code]0.1[/code] precision.
        /// </summary>
        public float ReverbBusAmount
        {
            get
            {
                return GetReverbAmount();
            }
            set
            {
                SetReverbAmount(value);
            }
        }

        /// <summary>
        /// The degree to which this area's reverb is a uniform effect. Ranges from [code]0[/code] to [code]1[/code] with [code]0.1[/code] precision.
        /// </summary>
        public float ReverbBusUniformity
        {
            get
            {
                return GetReverbUniformity();
            }
            set
            {
                SetReverbUniformity(value);
            }
        }


        private const string nativeName = "Area";

        public Area() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Area_Ctor(this);
        }

        internal Area(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_space_override_mode");

        [GodotMethod("set_space_override_mode")]
        public void SetSpaceOverrideMode(Area.SpaceOverrideEnum enable)
        {
            NativeCalls.godot_icall_1_102(method_bind_0, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_space_override_mode");

        [GodotMethod("get_space_override_mode")]
        public Area.SpaceOverrideEnum GetSpaceOverrideMode()
        {
            return NativeCalls.godot_icall_0_103(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_gravity_is_point");

        [GodotMethod("set_gravity_is_point")]
        public void SetGravityIsPoint(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_2, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_gravity_a_point");

        [GodotMethod("is_gravity_a_point")]
        public bool IsGravityAPoint()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_gravity_distance_scale");

        [GodotMethod("set_gravity_distance_scale")]
        public void SetGravityDistanceScale(float distanceScale)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), distanceScale);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_gravity_distance_scale");

        [GodotMethod("get_gravity_distance_scale")]
        public float GetGravityDistanceScale()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_gravity_vector");

        [GodotMethod("set_gravity_vector")]
        public void SetGravityVector(Vector3 vector)
        {
            NativeCalls.godot_icall_1_104(method_bind_6, Object.GetPtr(this), ref vector);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_gravity_vector");

        [GodotMethod("get_gravity_vector")]
        public Vector3 GetGravityVector()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_7, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_gravity");

        [GodotMethod("set_gravity")]
        public void SetGravity(float gravity)
        {
            NativeCalls.godot_icall_1_10(method_bind_8, Object.GetPtr(this), gravity);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_gravity");

        [GodotMethod("get_gravity")]
        public float GetGravity()
        {
            return NativeCalls.godot_icall_0_9(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_angular_damp");

        [GodotMethod("set_angular_damp")]
        public void SetAngularDamp(float angularDamp)
        {
            NativeCalls.godot_icall_1_10(method_bind_10, Object.GetPtr(this), angularDamp);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_angular_damp");

        [GodotMethod("get_angular_damp")]
        public float GetAngularDamp()
        {
            return NativeCalls.godot_icall_0_9(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_linear_damp");

        [GodotMethod("set_linear_damp")]
        public void SetLinearDamp(float linearDamp)
        {
            NativeCalls.godot_icall_1_10(method_bind_12, Object.GetPtr(this), linearDamp);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_linear_damp");

        [GodotMethod("get_linear_damp")]
        public float GetLinearDamp()
        {
            return NativeCalls.godot_icall_0_9(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_priority");

        [GodotMethod("set_priority")]
        public void SetPriority(float priority)
        {
            NativeCalls.godot_icall_1_10(method_bind_14, Object.GetPtr(this), priority);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_priority");

        [GodotMethod("get_priority")]
        public float GetPriority()
        {
            return NativeCalls.godot_icall_0_9(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_mask");

        [GodotMethod("set_collision_mask")]
        public void SetCollisionMask(int collisionMask)
        {
            NativeCalls.godot_icall_1_0(method_bind_16, Object.GetPtr(this), collisionMask);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_mask");

        [GodotMethod("get_collision_mask")]
        public int GetCollisionMask()
        {
            return NativeCalls.godot_icall_0_1(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_layer");

        [GodotMethod("set_collision_layer")]
        public void SetCollisionLayer(int collisionLayer)
        {
            NativeCalls.godot_icall_1_0(method_bind_18, Object.GetPtr(this), collisionLayer);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_layer");

        [GodotMethod("get_collision_layer")]
        public int GetCollisionLayer()
        {
            return NativeCalls.godot_icall_0_1(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_mask_bit");

        /// <summary>
        /// Set/clear individual bits on the collision mask. This simplifies editing which [code]Area[/code] layers this [code]Area[/code] scans.
        /// </summary>
        [GodotMethod("set_collision_mask_bit")]
        public void SetCollisionMaskBit(int bit, bool value)
        {
            NativeCalls.godot_icall_2_51(method_bind_20, Object.GetPtr(this), bit, value);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_mask_bit");

        /// <summary>
        /// Returns an individual bit on the collision mask.
        /// </summary>
        [GodotMethod("get_collision_mask_bit")]
        public bool GetCollisionMaskBit(int bit)
        {
            return NativeCalls.godot_icall_1_29(method_bind_21, Object.GetPtr(this), bit);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_layer_bit");

        /// <summary>
        /// Set/clear individual bits on the layer mask. This simplifies editing this [code]Area[code]'s layers.
        /// </summary>
        [GodotMethod("set_collision_layer_bit")]
        public void SetCollisionLayerBit(int bit, bool value)
        {
            NativeCalls.godot_icall_2_51(method_bind_22, Object.GetPtr(this), bit, value);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_layer_bit");

        /// <summary>
        /// Returns an individual bit on the layer mask.
        /// </summary>
        [GodotMethod("get_collision_layer_bit")]
        public bool GetCollisionLayerBit(int bit)
        {
            return NativeCalls.godot_icall_1_29(method_bind_23, Object.GetPtr(this), bit);
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_monitorable");

        [GodotMethod("set_monitorable")]
        public void SetMonitorable(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_24, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_monitorable");

        [GodotMethod("is_monitorable")]
        public bool IsMonitorable()
        {
            return NativeCalls.godot_icall_0_3(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_monitoring");

        [GodotMethod("set_monitoring")]
        public void SetMonitoring(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_26, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_monitoring");

        [GodotMethod("is_monitoring")]
        public bool IsMonitoring()
        {
            return NativeCalls.godot_icall_0_3(method_bind_27, Object.GetPtr(this));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_overlapping_bodies");

        /// <summary>
        /// Returns a list of intersecting [PhysicsBody]s. For performance reasons (collisions are all processed at the same time) this list is modified once during the physics step, not immediately after objects are moved. Consider using signals instead.
        /// </summary>
        [GodotMethod("get_overlapping_bodies")]
        public object[] GetOverlappingBodies()
        {
            return NativeCalls.godot_icall_0_21(method_bind_28, Object.GetPtr(this));
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_overlapping_areas");

        /// <summary>
        /// Returns a list of intersecting [code]Area[/code]s. For performance reasons (collisions are all processed at the same time) this list is modified once during the physics step, not immediately after objects are moved. Consider using signals instead.
        /// </summary>
        [GodotMethod("get_overlapping_areas")]
        public object[] GetOverlappingAreas()
        {
            return NativeCalls.godot_icall_0_21(method_bind_29, Object.GetPtr(this));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "overlaps_body");

        /// <summary>
        /// If [code]true[/code] the given body overlaps the Area. Note that the result of this test is not immediate after moving objects. For performance, list of overlaps is updated once per frame and before the physics step. Consider using signals instead.
        /// </summary>
        [GodotMethod("overlaps_body")]
        public bool OverlapsBody(Node body)
        {
            return NativeCalls.godot_icall_1_105(method_bind_30, Object.GetPtr(this), Object.GetPtr(body));
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "overlaps_area");

        /// <summary>
        /// If [code]true[/code] the given area overlaps the Area. Note that the result of this test is not immediate after moving objects. For performance, list of overlaps is updated once per frame and before the physics step. Consider using signals instead.
        /// </summary>
        [GodotMethod("overlaps_area")]
        public bool OverlapsArea(Node area)
        {
            return NativeCalls.godot_icall_1_105(method_bind_31, Object.GetPtr(this), Object.GetPtr(area));
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_audio_bus_override");

        [GodotMethod("set_audio_bus_override")]
        public void SetAudioBusOverride(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_32, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_overriding_audio_bus");

        [GodotMethod("is_overriding_audio_bus")]
        public bool IsOverridingAudioBus()
        {
            return NativeCalls.godot_icall_0_3(method_bind_33, Object.GetPtr(this));
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_audio_bus");

        [GodotMethod("set_audio_bus")]
        public void SetAudioBus(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_34, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_audio_bus");

        [GodotMethod("get_audio_bus")]
        public string GetAudioBus()
        {
            return NativeCalls.godot_icall_0_2(method_bind_35, Object.GetPtr(this));
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_use_reverb_bus");

        [GodotMethod("set_use_reverb_bus")]
        public void SetUseReverbBus(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_36, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_using_reverb_bus");

        [GodotMethod("is_using_reverb_bus")]
        public bool IsUsingReverbBus()
        {
            return NativeCalls.godot_icall_0_3(method_bind_37, Object.GetPtr(this));
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_reverb_bus");

        [GodotMethod("set_reverb_bus")]
        public void SetReverbBus(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_38, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_reverb_bus");

        [GodotMethod("get_reverb_bus")]
        public string GetReverbBus()
        {
            return NativeCalls.godot_icall_0_2(method_bind_39, Object.GetPtr(this));
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_reverb_amount");

        [GodotMethod("set_reverb_amount")]
        public void SetReverbAmount(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_40, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_reverb_amount");

        [GodotMethod("get_reverb_amount")]
        public float GetReverbAmount()
        {
            return NativeCalls.godot_icall_0_9(method_bind_41, Object.GetPtr(this));
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_reverb_uniformity");

        [GodotMethod("set_reverb_uniformity")]
        public void SetReverbUniformity(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_42, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_reverb_uniformity");

        [GodotMethod("get_reverb_uniformity")]
        public float GetReverbUniformity()
        {
            return NativeCalls.godot_icall_0_9(method_bind_43, Object.GetPtr(this));
        }
    }
}
