using System;

namespace Godot
{
    /// <summary>
    /// [Texture] resource aimed at managing big textures files that pack multiple smaller textures. Consists of a [Texture], a margin that defines the border width,
    /// and a region that defines the actual area of the AtlasTexture.
    /// </summary>
    public class AtlasTexture : Texture
    {
        /// <summary>
        /// The texture that contains the atlas. Can be any [Texture] subtype.
        /// </summary>
        public Texture Atlas
        {
            get
            {
                return GetAtlas();
            }
            set
            {
                SetAtlas(value);
            }
        }

        /// <summary>
        /// The AtlasTexture's used region.
        /// </summary>
        public Rect2 Region
        {
            get
            {
                return GetRegion();
            }
            set
            {
                SetRegion(value);
            }
        }

        /// <summary>
        /// The margin around the region. The [Rect2]'s 'size' parameter ('w' and 'h' in the editor) resizes the texture so it fits within the margin.
        /// </summary>
        public Rect2 Margin
        {
            get
            {
                return GetMargin();
            }
            set
            {
                SetMargin(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] clips the area outside of the region to avoid bleeding of the surrounding texture pixels.
        /// </summary>
        public bool FilterClip
        {
            get
            {
                return HasFilterClip();
            }
            set
            {
                SetFilterClip(value);
            }
        }


        private const string nativeName = "AtlasTexture";

        public AtlasTexture() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AtlasTexture_Ctor(this);
        }

        internal AtlasTexture(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_atlas");

        [GodotMethod("set_atlas")]
        public void SetAtlas(Texture atlas)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(atlas));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_atlas");

        [GodotMethod("get_atlas")]
        public Texture GetAtlas()
        {
            return NativeCalls.godot_icall_0_119(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_region");

        [GodotMethod("set_region")]
        public void SetRegion(Rect2 region)
        {
            NativeCalls.godot_icall_1_120(method_bind_2, Object.GetPtr(this), ref region);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_region");

        [GodotMethod("get_region")]
        public Rect2 GetRegion()
        {
            object ret = NativeCalls.godot_icall_0_121(method_bind_3, Object.GetPtr(this));
            return (Rect2)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_margin");

        [GodotMethod("set_margin")]
        public void SetMargin(Rect2 margin)
        {
            NativeCalls.godot_icall_1_120(method_bind_4, Object.GetPtr(this), ref margin);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_margin");

        [GodotMethod("get_margin")]
        public Rect2 GetMargin()
        {
            object ret = NativeCalls.godot_icall_0_121(method_bind_5, Object.GetPtr(this));
            return (Rect2)ret;
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_filter_clip");

        [GodotMethod("set_filter_clip")]
        public void SetFilterClip(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_6, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_filter_clip");

        [GodotMethod("has_filter_clip")]
        public bool HasFilterClip()
        {
            return NativeCalls.godot_icall_0_3(method_bind_7, Object.GetPtr(this));
        }
    }
}
