using System;

namespace Godot
{
    /// <summary>
    /// Dynamic range compressor reduces the level of the sound when the amplitude goes over a certain threshold in Decibels. One of the main uses of a compressor is to increase the dynamic range by clipping as little as possible (when sound goes over 0dB).
    /// Compressor has many uses in the mix:
    /// - In the Master bus to compress the whole output (Although a [AudioEffectLimiter] is probably better)
    /// - In voice channels to ensure they sound as balanced as possible.
    /// - Sidechained. Sidechained, which can reduce the sound level sidechained with another audio bus for threshold detection.. This technique is very common in video game mixing to download the level of Music/SFX while voices are being heard.
    /// - Accentuates transients by using a wider attack, making effects sound more punchy.
    /// </summary>
    public class AudioEffectCompressor : AudioEffect
    {
        /// <summary>
        /// The level above which compression is applied to the audio. Value can range from -60 to 0. Default value: [code]0[/code].
        /// </summary>
        public float Threshold
        {
            get
            {
                return GetThreshold();
            }
            set
            {
                SetThreshold(value);
            }
        }

        /// <summary>
        /// Amount of compression applied to the audio once it passes the threshold level. The higher the ratio the more the loud parts of the audio will be compressed. Value can range from 1 to 48. Default value: [code]4[/code].
        /// </summary>
        public float Ratio
        {
            get
            {
                return GetRatio();
            }
            set
            {
                SetRatio(value);
            }
        }

        /// <summary>
        /// Gain applied to the output signal.
        /// </summary>
        public float Gain
        {
            get
            {
                return GetGain();
            }
            set
            {
                SetGain(value);
            }
        }

        /// <summary>
        /// Compressor's reaction time when the signal exceeds the threshold. Value can range from 20 to 2000. Default value: [code]20ms[/code].
        /// </summary>
        public float AttackUs
        {
            get
            {
                return GetAttackUs();
            }
            set
            {
                SetAttackUs(value);
            }
        }

        /// <summary>
        /// Compressor's delay time to stop reducing the signal after the signal level falls below the threshold. Value can range from 20 to 2000. Default value: [code]250ms[/code].
        /// </summary>
        public float ReleaseMs
        {
            get
            {
                return GetReleaseMs();
            }
            set
            {
                SetReleaseMs(value);
            }
        }

        /// <summary>
        /// Balance between original signal and effect signal. Value can range from 0 (totally dry) to 1 (totally wet). Default value: [code]1[/code].
        /// </summary>
        public float Mix
        {
            get
            {
                return GetMix();
            }
            set
            {
                SetMix(value);
            }
        }

        /// <summary>
        /// Reduce the sound level using another audio bus for threshold detection.
        /// </summary>
        public string Sidechain
        {
            get
            {
                return GetSidechain();
            }
            set
            {
                SetSidechain(value);
            }
        }


        private const string nativeName = "AudioEffectCompressor";

        public AudioEffectCompressor() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AudioEffectCompressor_Ctor(this);
        }

        internal AudioEffectCompressor(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_threshold");

        [GodotMethod("set_threshold")]
        public void SetThreshold(float threshold)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), threshold);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_threshold");

        [GodotMethod("get_threshold")]
        public float GetThreshold()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ratio");

        [GodotMethod("set_ratio")]
        public void SetRatio(float ratio)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), ratio);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ratio");

        [GodotMethod("get_ratio")]
        public float GetRatio()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_gain");

        [GodotMethod("set_gain")]
        public void SetGain(float gain)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), gain);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_gain");

        [GodotMethod("get_gain")]
        public float GetGain()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_attack_us");

        [GodotMethod("set_attack_us")]
        public void SetAttackUs(float attackUs)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), attackUs);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_attack_us");

        [GodotMethod("get_attack_us")]
        public float GetAttackUs()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_release_ms");

        [GodotMethod("set_release_ms")]
        public void SetReleaseMs(float releaseMs)
        {
            NativeCalls.godot_icall_1_10(method_bind_8, Object.GetPtr(this), releaseMs);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_release_ms");

        [GodotMethod("get_release_ms")]
        public float GetReleaseMs()
        {
            return NativeCalls.godot_icall_0_9(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_mix");

        [GodotMethod("set_mix")]
        public void SetMix(float mix)
        {
            NativeCalls.godot_icall_1_10(method_bind_10, Object.GetPtr(this), mix);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mix");

        [GodotMethod("get_mix")]
        public float GetMix()
        {
            return NativeCalls.godot_icall_0_9(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_sidechain");

        [GodotMethod("set_sidechain")]
        public void SetSidechain(string sidechain)
        {
            NativeCalls.godot_icall_1_43(method_bind_12, Object.GetPtr(this), sidechain);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_sidechain");

        [GodotMethod("get_sidechain")]
        public string GetSidechain()
        {
            return NativeCalls.godot_icall_0_2(method_bind_13, Object.GetPtr(this));
        }
    }
}
