using System;

namespace Godot
{
    /// <summary>
    /// Modify the sound and make it dirty. Different types are available : clip, tan, lofi (bit crushing), overdrive, or waveshape.
    /// By distorting the waveform the frequency content change, which will often make the sound "crunchy" or "abrasive". For games, it can simulate sound coming from some saturated device or speaker very efficiently.
    /// </summary>
    public class AudioEffectDistortion : AudioEffect
    {
        public enum ModeEnum
        {
            Clip = 0,
            Atan = 1,
            Lofi = 2,
            Overdrive = 3,
            Waveshape = 4
        }


        /// <summary>
        /// Distortion type. Default value: [code]MODE_CLIP[/code].
        /// </summary>
        public AudioEffectDistortion.ModeEnum Mode
        {
            get
            {
                return GetMode();
            }
            set
            {
                SetMode(value);
            }
        }

        /// <summary>
        /// Increases or decreases the volume before the effect. Value can range from -60 to 60. Default value: [code]0[/code].
        /// </summary>
        public float PreGain
        {
            get
            {
                return GetPreGain();
            }
            set
            {
                SetPreGain(value);
            }
        }

        /// <summary>
        /// High-pass filter. Frequencies higher than this value will not be affected by the distortion. Value can range from 1 to 20000. Default value: [code]16000[/code].
        /// </summary>
        public float KeepHfHz
        {
            get
            {
                return GetKeepHfHz();
            }
            set
            {
                SetKeepHfHz(value);
            }
        }

        /// <summary>
        /// Distortion power. Value can range from 0 to 1. Default value: [code]0[/code].
        /// </summary>
        public float Drive
        {
            get
            {
                return GetDrive();
            }
            set
            {
                SetDrive(value);
            }
        }

        /// <summary>
        /// Increases or decreases the volume after the effect. Value can range from -80 to 24. Default value: [code]0[/code].
        /// </summary>
        public float PostGain
        {
            get
            {
                return GetPostGain();
            }
            set
            {
                SetPostGain(value);
            }
        }


        private const string nativeName = "AudioEffectDistortion";

        public AudioEffectDistortion() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AudioEffectDistortion_Ctor(this);
        }

        internal AudioEffectDistortion(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_mode");

        [GodotMethod("set_mode")]
        public void SetMode(AudioEffectDistortion.ModeEnum mode)
        {
            NativeCalls.godot_icall_1_122(method_bind_0, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mode");

        [GodotMethod("get_mode")]
        public AudioEffectDistortion.ModeEnum GetMode()
        {
            return NativeCalls.godot_icall_0_123(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_pre_gain");

        [GodotMethod("set_pre_gain")]
        public void SetPreGain(float preGain)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), preGain);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_pre_gain");

        [GodotMethod("get_pre_gain")]
        public float GetPreGain()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_keep_hf_hz");

        [GodotMethod("set_keep_hf_hz")]
        public void SetKeepHfHz(float keepHfHz)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), keepHfHz);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_keep_hf_hz");

        [GodotMethod("get_keep_hf_hz")]
        public float GetKeepHfHz()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_drive");

        [GodotMethod("set_drive")]
        public void SetDrive(float drive)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), drive);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_drive");

        [GodotMethod("get_drive")]
        public float GetDrive()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_post_gain");

        [GodotMethod("set_post_gain")]
        public void SetPostGain(float postGain)
        {
            NativeCalls.godot_icall_1_10(method_bind_8, Object.GetPtr(this), postGain);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_post_gain");

        [GodotMethod("get_post_gain")]
        public float GetPostGain()
        {
            return NativeCalls.godot_icall_0_9(method_bind_9, Object.GetPtr(this));
        }
    }
}
