using System;

namespace Godot
{
    /// <summary>
    /// A limiter is similar to a compressor, but it’s less flexible and designed to disallow sound going over a given dB threshold. Adding one in the Master Bus is always recommended to reduce the effects of clipping.
    /// Soft clipping starts to reduce the peaks a little below the threshold level and progressively increases its effect as the input level increases such that the threshold is never exceeded.
    /// </summary>
    public class AudioEffectLimiter : AudioEffect
    {
        /// <summary>
        /// The waveform's maximum allowed value. Value can range from -20 to -0.1. Default value: [code]-0.1dB[/code].
        /// </summary>
        public float CeilingDb
        {
            get
            {
                return GetCeilingDb();
            }
            set
            {
                SetCeilingDb(value);
            }
        }

        /// <summary>
        /// Threshold from which the limiter begins to be active. Value can range from -30 to 0. Default value: [code]0dB[/code].
        /// </summary>
        public float ThresholdDb
        {
            get
            {
                return GetThresholdDb();
            }
            set
            {
                SetThresholdDb(value);
            }
        }

        /// <summary>
        /// Applies a gain to the limited waves. Value can range from 0 to 6. Default value: [code]2dB[/code].
        /// </summary>
        public float SoftClipDb
        {
            get
            {
                return GetSoftClipDb();
            }
            set
            {
                SetSoftClipDb(value);
            }
        }

        /// <summary>
        /// </summary>
        public float SoftClipRatio
        {
            get
            {
                return GetSoftClipRatio();
            }
            set
            {
                SetSoftClipRatio(value);
            }
        }


        private const string nativeName = "AudioEffectLimiter";

        public AudioEffectLimiter() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AudioEffectLimiter_Ctor(this);
        }

        internal AudioEffectLimiter(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ceiling_db");

        [GodotMethod("set_ceiling_db")]
        public void SetCeilingDb(float ceiling)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), ceiling);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ceiling_db");

        [GodotMethod("get_ceiling_db")]
        public float GetCeilingDb()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_threshold_db");

        [GodotMethod("set_threshold_db")]
        public void SetThresholdDb(float threshold)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), threshold);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_threshold_db");

        [GodotMethod("get_threshold_db")]
        public float GetThresholdDb()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_soft_clip_db");

        [GodotMethod("set_soft_clip_db")]
        public void SetSoftClipDb(float softClip)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), softClip);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_soft_clip_db");

        [GodotMethod("get_soft_clip_db")]
        public float GetSoftClipDb()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_soft_clip_ratio");

        [GodotMethod("set_soft_clip_ratio")]
        public void SetSoftClipRatio(float softClip)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), softClip);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_soft_clip_ratio");

        [GodotMethod("get_soft_clip_ratio")]
        public float GetSoftClipRatio()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }
    }
}
