using System;

namespace Godot
{
    /// <summary>
    /// Combines phase-shifted signals with the original signal. The movement of the phase-shifted signals is controlled using a Low Frequency Oscillator.
    /// </summary>
    public class AudioEffectPhaser : AudioEffect
    {
        /// <summary>
        /// Determines the minimum frequency affected by the LFO modulations. Value can range from 10 to 10000. Default value: [code]440hz[/code].
        /// </summary>
        public float RangeMinHz
        {
            get
            {
                return GetRangeMinHz();
            }
            set
            {
                SetRangeMinHz(value);
            }
        }

        /// <summary>
        /// Determines the maximum frequency affected by the LFO modulations. Value can range from 10 to 10000. Default value: [code]1600hz[/code].
        /// </summary>
        public float RangeMaxHz
        {
            get
            {
                return GetRangeMaxHz();
            }
            set
            {
                SetRangeMaxHz(value);
            }
        }

        /// <summary>
        /// Adjusts the rate at which the effect sweeps up and down across the frequency range.
        /// </summary>
        public float RateHz
        {
            get
            {
                return GetRateHz();
            }
            set
            {
                SetRateHz(value);
            }
        }

        /// <summary>
        /// Output percent of modified sound. Value can range from 0.1 to 0.9. Default value: [code]0.7[/code].
        /// </summary>
        public float Feedback
        {
            get
            {
                return GetFeedback();
            }
            set
            {
                SetFeedback(value);
            }
        }

        /// <summary>
        /// Governs how high the filter frequencies sweep. Low value will primarily affect bass frequencies. High value can sweep high into the treble. Value can range from 0.1 to 4. Default value: [code]1[/code].
        /// </summary>
        public float Depth
        {
            get
            {
                return GetDepth();
            }
            set
            {
                SetDepth(value);
            }
        }


        private const string nativeName = "AudioEffectPhaser";

        public AudioEffectPhaser() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AudioEffectPhaser_Ctor(this);
        }

        internal AudioEffectPhaser(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_range_min_hz");

        [GodotMethod("set_range_min_hz")]
        public void SetRangeMinHz(float hz)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), hz);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_range_min_hz");

        [GodotMethod("get_range_min_hz")]
        public float GetRangeMinHz()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_range_max_hz");

        [GodotMethod("set_range_max_hz")]
        public void SetRangeMaxHz(float hz)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), hz);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_range_max_hz");

        [GodotMethod("get_range_max_hz")]
        public float GetRangeMaxHz()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rate_hz");

        [GodotMethod("set_rate_hz")]
        public void SetRateHz(float hz)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), hz);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rate_hz");

        [GodotMethod("get_rate_hz")]
        public float GetRateHz()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_feedback");

        [GodotMethod("set_feedback")]
        public void SetFeedback(float fbk)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), fbk);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_feedback");

        [GodotMethod("get_feedback")]
        public float GetFeedback()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_depth");

        [GodotMethod("set_depth")]
        public void SetDepth(float depth)
        {
            NativeCalls.godot_icall_1_10(method_bind_8, Object.GetPtr(this), depth);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_depth");

        [GodotMethod("get_depth")]
        public float GetDepth()
        {
            return NativeCalls.godot_icall_0_9(method_bind_9, Object.GetPtr(this));
        }
    }
}
