using System;

namespace Godot
{
    /// <summary>
    /// Plays audio that dampens with distance from screen center.
    /// </summary>
    public class AudioStreamPlayer2D : Node2D
    {
        /// <summary>
        /// The [AudioStream] object to be played.
        /// </summary>
        public AudioStream Stream
        {
            get
            {
                return GetStream();
            }
            set
            {
                SetStream(value);
            }
        }

        /// <summary>
        /// Base volume without dampening.
        /// </summary>
        public float VolumeDb
        {
            get
            {
                return GetVolumeDb();
            }
            set
            {
                SetVolumeDb(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] audio is playing.
        /// </summary>
        public bool Playing
        {
            get
            {
                return IsPlaying();
            }
            set
            {
                _SetPlaying(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] audio plays when added to scene tree. Default value: [code]false[/code].
        /// </summary>
        public bool Autoplay
        {
            get
            {
                return IsAutoplayEnabled();
            }
            set
            {
                SetAutoplay(value);
            }
        }

        /// <summary>
        /// Maximum distance from which audio is still hearable.
        /// </summary>
        public float MaxDistance
        {
            get
            {
                return GetMaxDistance();
            }
            set
            {
                SetMaxDistance(value);
            }
        }

        /// <summary>
        /// Dampens audio over distance with this as an exponent.
        /// </summary>
        public float Attenuation
        {
            get
            {
                return GetAttenuation();
            }
            set
            {
                SetAttenuation(value);
            }
        }

        /// <summary>
        /// Bus on which this audio is playing.
        /// </summary>
        public string Bus
        {
            get
            {
                return GetBus();
            }
            set
            {
                SetBus(value);
            }
        }

        /// <summary>
        /// Areas in which this sound plays.
        /// </summary>
        public int AreaMask
        {
            get
            {
                return GetAreaMask();
            }
            set
            {
                SetAreaMask(value);
            }
        }


        private const string nativeName = "AudioStreamPlayer2D";

        public AudioStreamPlayer2D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AudioStreamPlayer2D_Ctor(this);
        }

        internal AudioStreamPlayer2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_stream");

        [GodotMethod("set_stream")]
        public void SetStream(AudioStream stream)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(stream));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_stream");

        [GodotMethod("get_stream")]
        public AudioStream GetStream()
        {
            return NativeCalls.godot_icall_0_133(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_volume_db");

        [GodotMethod("set_volume_db")]
        public void SetVolumeDb(float volumeDb)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), volumeDb);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_volume_db");

        [GodotMethod("get_volume_db")]
        public float GetVolumeDb()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "play");

        /// <summary>
        /// Plays the audio from the given position 'from_position', in seconds.
        /// </summary>
        [GodotMethod("play")]
        public void Play(float fromPosition = 0f)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), fromPosition);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "seek");

        /// <summary>
        /// Sets the position from which audio will be played, in seconds.
        /// </summary>
        [GodotMethod("seek")]
        public void Seek(float toPosition)
        {
            NativeCalls.godot_icall_1_10(method_bind_5, Object.GetPtr(this), toPosition);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "stop");

        /// <summary>
        /// Stops the audio.
        /// </summary>
        [GodotMethod("stop")]
        public void Stop()
        {
            NativeCalls.godot_icall_0_12(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_playing");

        [GodotMethod("is_playing")]
        public bool IsPlaying()
        {
            return NativeCalls.godot_icall_0_3(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_playback_position");

        /// <summary>
        /// Returns the position in the [AudioStream].
        /// </summary>
        [GodotMethod("get_playback_position")]
        public float GetPlaybackPosition()
        {
            return NativeCalls.godot_icall_0_9(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bus");

        [GodotMethod("set_bus")]
        public void SetBus(string bus)
        {
            NativeCalls.godot_icall_1_43(method_bind_9, Object.GetPtr(this), bus);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bus");

        [GodotMethod("get_bus")]
        public string GetBus()
        {
            return NativeCalls.godot_icall_0_2(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_autoplay");

        [GodotMethod("set_autoplay")]
        public void SetAutoplay(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_11, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_autoplay_enabled");

        [GodotMethod("is_autoplay_enabled")]
        public bool IsAutoplayEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_12, Object.GetPtr(this));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_playing");

        internal void _SetPlaying(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_13, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_max_distance");

        [GodotMethod("set_max_distance")]
        public void SetMaxDistance(float pixels)
        {
            NativeCalls.godot_icall_1_10(method_bind_14, Object.GetPtr(this), pixels);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_max_distance");

        [GodotMethod("get_max_distance")]
        public float GetMaxDistance()
        {
            return NativeCalls.godot_icall_0_9(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_attenuation");

        [GodotMethod("set_attenuation")]
        public void SetAttenuation(float curve)
        {
            NativeCalls.godot_icall_1_10(method_bind_16, Object.GetPtr(this), curve);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_attenuation");

        [GodotMethod("get_attenuation")]
        public float GetAttenuation()
        {
            return NativeCalls.godot_icall_0_9(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_area_mask");

        [GodotMethod("set_area_mask")]
        public void SetAreaMask(int mask)
        {
            NativeCalls.godot_icall_1_0(method_bind_18, Object.GetPtr(this), mask);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_area_mask");

        [GodotMethod("get_area_mask")]
        public int GetAreaMask()
        {
            return NativeCalls.godot_icall_0_1(method_bind_19, Object.GetPtr(this));
        }
    }
}
