using System;

namespace Godot
{
    /// <summary>
    /// Plays a sound effect with directed sound effects, dampens with distance if needed, generates effect of hearable position in space.
    /// </summary>
    public class AudioStreamPlayer3D : Spatial
    {
        public enum AttenuationModelEnum
        {
            InverseDistance = 0,
            InverseSquareDistance = 1,
            Logarithmic = 2
        }

        public enum OutOfRangeModeEnum
        {
            Mix = 0,
            Pause = 1
        }

        public enum DopplerTrackingEnum
        {
            Disabled = 0,
            IdleStep = 1,
            PhysicsStep = 2
        }


        /// <summary>
        /// The [AudioStream] object to be played.
        /// </summary>
        public AudioStream Stream
        {
            get
            {
                return GetStream();
            }
            set
            {
                SetStream(value);
            }
        }

        /// <summary>
        /// Decides if audio should get quieter with distance linearly, quadratically or logarithmically.
        /// </summary>
        public AudioStreamPlayer3D.AttenuationModelEnum AttenuationModel
        {
            get
            {
                return GetAttenuationModel();
            }
            set
            {
                SetAttenuationModel(value);
            }
        }

        /// <summary>
        /// Base sound level unaffected by dampening, in dB.
        /// </summary>
        public float UnitDb
        {
            get
            {
                return GetUnitDb();
            }
            set
            {
                SetUnitDb(value);
            }
        }

        /// <summary>
        /// Factor for the attenuation effect.
        /// </summary>
        public float UnitSize
        {
            get
            {
                return GetUnitSize();
            }
            set
            {
                SetUnitSize(value);
            }
        }

        /// <summary>
        /// Sets the absolute maximum of the soundlevel, in dB.
        /// </summary>
        public float MaxDb
        {
            get
            {
                return GetMaxDb();
            }
            set
            {
                SetMaxDb(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], audio is playing.
        /// </summary>
        public bool Playing
        {
            get
            {
                return IsPlaying();
            }
            set
            {
                _SetPlaying(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] audio plays when added to scene tree. Default value: [code]false[/code].
        /// </summary>
        public bool Autoplay
        {
            get
            {
                return IsAutoplayEnabled();
            }
            set
            {
                SetAutoplay(value);
            }
        }

        /// <summary>
        /// Sets the distance from which the 'out_of_range_mode' takes effect. Has no effect if set to 0.
        /// </summary>
        public float MaxDistance
        {
            get
            {
                return GetMaxDistance();
            }
            set
            {
                SetMaxDistance(value);
            }
        }

        /// <summary>
        /// Decides if audio should pause when source is outside of 'max_distance' range.
        /// </summary>
        public AudioStreamPlayer3D.OutOfRangeModeEnum OutOfRangeMode
        {
            get
            {
                return GetOutOfRangeMode();
            }
            set
            {
                SetOutOfRangeMode(value);
            }
        }

        /// <summary>
        /// Bus on which this audio is playing.
        /// </summary>
        public string Bus
        {
            get
            {
                return GetBus();
            }
            set
            {
                SetBus(value);
            }
        }

        /// <summary>
        /// Areas in which this sound plays.
        /// </summary>
        public int AreaMask
        {
            get
            {
                return GetAreaMask();
            }
            set
            {
                SetAreaMask(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the audio should be dampened according to the direction of the sound.
        /// </summary>
        public bool EmissionAngleEnabled
        {
            get
            {
                return IsEmissionAngleEnabled();
            }
            set
            {
                SetEmissionAngleEnabled(value);
            }
        }

        /// <summary>
        /// The angle in which the audio reaches cameras undampened.
        /// </summary>
        public float EmissionAngleDegrees
        {
            get
            {
                return GetEmissionAngle();
            }
            set
            {
                SetEmissionAngle(value);
            }
        }

        /// <summary>
        /// dampens audio if camera is outside of 'emission_angle_degrees' and 'emission_angle_enabled' is set by this factor, in dB.
        /// </summary>
        public float EmissionAngleFilterAttenuationDb
        {
            get
            {
                return GetEmissionAngleFilterAttenuationDb();
            }
            set
            {
                SetEmissionAngleFilterAttenuationDb(value);
            }
        }

        /// <summary>
        /// Dampens audio above this frequency, in Hz.
        /// </summary>
        public float AttenuationFilterCutoffHz
        {
            get
            {
                return GetAttenuationFilterCutoffHz();
            }
            set
            {
                SetAttenuationFilterCutoffHz(value);
            }
        }

        /// <summary>
        /// Amount how much the filter affects the loudness, in dB.
        /// </summary>
        public float AttenuationFilterDb
        {
            get
            {
                return GetAttenuationFilterDb();
            }
            set
            {
                SetAttenuationFilterDb(value);
            }
        }

        /// <summary>
        /// Decides in which step the Doppler effect should be calculated.
        /// </summary>
        public AudioStreamPlayer3D.DopplerTrackingEnum DopplerTracking
        {
            get
            {
                return GetDopplerTracking();
            }
            set
            {
                SetDopplerTracking(value);
            }
        }


        private const string nativeName = "AudioStreamPlayer3D";

        public AudioStreamPlayer3D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_AudioStreamPlayer3D_Ctor(this);
        }

        internal AudioStreamPlayer3D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_stream");

        [GodotMethod("set_stream")]
        public void SetStream(AudioStream stream)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(stream));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_stream");

        [GodotMethod("get_stream")]
        public AudioStream GetStream()
        {
            return NativeCalls.godot_icall_0_133(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_unit_db");

        [GodotMethod("set_unit_db")]
        public void SetUnitDb(float unitDb)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), unitDb);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_unit_db");

        [GodotMethod("get_unit_db")]
        public float GetUnitDb()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_unit_size");

        [GodotMethod("set_unit_size")]
        public void SetUnitSize(float unitSize)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), unitSize);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_unit_size");

        [GodotMethod("get_unit_size")]
        public float GetUnitSize()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_max_db");

        [GodotMethod("set_max_db")]
        public void SetMaxDb(float maxDb)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), maxDb);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_max_db");

        [GodotMethod("get_max_db")]
        public float GetMaxDb()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "play");

        /// <summary>
        /// Plays the audio from the given position 'from_position', in seconds.
        /// </summary>
        [GodotMethod("play")]
        public void Play(float fromPosition = 0f)
        {
            NativeCalls.godot_icall_1_10(method_bind_8, Object.GetPtr(this), fromPosition);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "seek");

        /// <summary>
        /// Sets the position from which audio will be played, in seconds.
        /// </summary>
        [GodotMethod("seek")]
        public void Seek(float toPosition)
        {
            NativeCalls.godot_icall_1_10(method_bind_9, Object.GetPtr(this), toPosition);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "stop");

        /// <summary>
        /// Stops the audio.
        /// </summary>
        [GodotMethod("stop")]
        public void Stop()
        {
            NativeCalls.godot_icall_0_12(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_playing");

        [GodotMethod("is_playing")]
        public bool IsPlaying()
        {
            return NativeCalls.godot_icall_0_3(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_playback_position");

        /// <summary>
        /// Returns the position in the [AudioStream].
        /// </summary>
        [GodotMethod("get_playback_position")]
        public float GetPlaybackPosition()
        {
            return NativeCalls.godot_icall_0_9(method_bind_12, Object.GetPtr(this));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bus");

        [GodotMethod("set_bus")]
        public void SetBus(string bus)
        {
            NativeCalls.godot_icall_1_43(method_bind_13, Object.GetPtr(this), bus);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bus");

        [GodotMethod("get_bus")]
        public string GetBus()
        {
            return NativeCalls.godot_icall_0_2(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_autoplay");

        [GodotMethod("set_autoplay")]
        public void SetAutoplay(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_15, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_autoplay_enabled");

        [GodotMethod("is_autoplay_enabled")]
        public bool IsAutoplayEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_playing");

        internal void _SetPlaying(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_17, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_max_distance");

        [GodotMethod("set_max_distance")]
        public void SetMaxDistance(float metres)
        {
            NativeCalls.godot_icall_1_10(method_bind_18, Object.GetPtr(this), metres);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_max_distance");

        [GodotMethod("get_max_distance")]
        public float GetMaxDistance()
        {
            return NativeCalls.godot_icall_0_9(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_area_mask");

        [GodotMethod("set_area_mask")]
        public void SetAreaMask(int mask)
        {
            NativeCalls.godot_icall_1_0(method_bind_20, Object.GetPtr(this), mask);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_area_mask");

        [GodotMethod("get_area_mask")]
        public int GetAreaMask()
        {
            return NativeCalls.godot_icall_0_1(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_emission_angle");

        [GodotMethod("set_emission_angle")]
        public void SetEmissionAngle(float degrees)
        {
            NativeCalls.godot_icall_1_10(method_bind_22, Object.GetPtr(this), degrees);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_emission_angle");

        [GodotMethod("get_emission_angle")]
        public float GetEmissionAngle()
        {
            return NativeCalls.godot_icall_0_9(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_emission_angle_enabled");

        [GodotMethod("set_emission_angle_enabled")]
        public void SetEmissionAngleEnabled(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_24, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_emission_angle_enabled");

        [GodotMethod("is_emission_angle_enabled")]
        public bool IsEmissionAngleEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_emission_angle_filter_attenuation_db");

        [GodotMethod("set_emission_angle_filter_attenuation_db")]
        public void SetEmissionAngleFilterAttenuationDb(float db)
        {
            NativeCalls.godot_icall_1_10(method_bind_26, Object.GetPtr(this), db);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_emission_angle_filter_attenuation_db");

        [GodotMethod("get_emission_angle_filter_attenuation_db")]
        public float GetEmissionAngleFilterAttenuationDb()
        {
            return NativeCalls.godot_icall_0_9(method_bind_27, Object.GetPtr(this));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_attenuation_filter_cutoff_hz");

        [GodotMethod("set_attenuation_filter_cutoff_hz")]
        public void SetAttenuationFilterCutoffHz(float degrees)
        {
            NativeCalls.godot_icall_1_10(method_bind_28, Object.GetPtr(this), degrees);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_attenuation_filter_cutoff_hz");

        [GodotMethod("get_attenuation_filter_cutoff_hz")]
        public float GetAttenuationFilterCutoffHz()
        {
            return NativeCalls.godot_icall_0_9(method_bind_29, Object.GetPtr(this));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_attenuation_filter_db");

        [GodotMethod("set_attenuation_filter_db")]
        public void SetAttenuationFilterDb(float db)
        {
            NativeCalls.godot_icall_1_10(method_bind_30, Object.GetPtr(this), db);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_attenuation_filter_db");

        [GodotMethod("get_attenuation_filter_db")]
        public float GetAttenuationFilterDb()
        {
            return NativeCalls.godot_icall_0_9(method_bind_31, Object.GetPtr(this));
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_attenuation_model");

        [GodotMethod("set_attenuation_model")]
        public void SetAttenuationModel(AudioStreamPlayer3D.AttenuationModelEnum model)
        {
            NativeCalls.godot_icall_1_136(method_bind_32, Object.GetPtr(this), model);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_attenuation_model");

        [GodotMethod("get_attenuation_model")]
        public AudioStreamPlayer3D.AttenuationModelEnum GetAttenuationModel()
        {
            return NativeCalls.godot_icall_0_137(method_bind_33, Object.GetPtr(this));
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_out_of_range_mode");

        [GodotMethod("set_out_of_range_mode")]
        public void SetOutOfRangeMode(AudioStreamPlayer3D.OutOfRangeModeEnum mode)
        {
            NativeCalls.godot_icall_1_138(method_bind_34, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_out_of_range_mode");

        [GodotMethod("get_out_of_range_mode")]
        public AudioStreamPlayer3D.OutOfRangeModeEnum GetOutOfRangeMode()
        {
            return NativeCalls.godot_icall_0_139(method_bind_35, Object.GetPtr(this));
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_doppler_tracking");

        [GodotMethod("set_doppler_tracking")]
        public void SetDopplerTracking(AudioStreamPlayer3D.DopplerTrackingEnum mode)
        {
            NativeCalls.godot_icall_1_140(method_bind_36, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_doppler_tracking");

        [GodotMethod("get_doppler_tracking")]
        public AudioStreamPlayer3D.DopplerTrackingEnum GetDopplerTracking()
        {
            return NativeCalls.godot_icall_0_141(method_bind_37, Object.GetPtr(this));
        }
    }
}
