using System;

namespace Godot
{
    /// <summary>
    /// BaseButton is the abstract base class for buttons, so it shouldn't be used directly (it doesn't display anything). Other types of buttons inherit from it.
    /// </summary>
    public class BaseButton : Control
    {
        public enum ActionModeEnum
        {
            Press = 0,
            Release = 1
        }

        public enum DrawMode
        {
            Normal = 0,
            Pressed = 1,
            Hover = 2,
            Disabled = 3
        }


        /// <summary>
        /// If [code]true[/code] the button is in disabled state and can't be clicked or toggled.
        /// </summary>
        public bool Disabled
        {
            get
            {
                return IsDisabled();
            }
            set
            {
                SetDisabled(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the button is in toggle mode. Makes the button flip state between pressed and unpressed each time its area is clicked.
        /// </summary>
        public bool ToggleMode
        {
            get
            {
                return IsToggleMode();
            }
            set
            {
                SetToggleMode(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the button's state is pressed. Means the button is pressed down or toggled (if toggle_mode is active).
        /// </summary>
        public bool Pressed
        {
            get
            {
                return IsPressed();
            }
            set
            {
                SetPressed(value);
            }
        }

        /// <summary>
        /// Determines when the button is considered clicked, one of the ACTION_MODE_* constants.
        /// </summary>
        public BaseButton.ActionModeEnum ActionMode
        {
            get
            {
                return GetActionMode();
            }
            set
            {
                SetActionMode(value);
            }
        }

        /// <summary>
        /// Focus access mode to use when switching between enabled/disabled (see [method Control.set_focus_mode] and [member disabled]).
        /// </summary>
        public Control.FocusModeEnum EnabledFocusMode
        {
            get
            {
                return GetEnabledFocusMode();
            }
            set
            {
                SetEnabledFocusMode(value);
            }
        }

        /// <summary>
        /// [Shortcut] associated to the button.
        /// </summary>
        public ShortCut Shortcut
        {
            get
            {
                return GetShortcut();
            }
            set
            {
                SetShortcut(value);
            }
        }

        /// <summary>
        /// [ButtonGroup] associated to the button.
        /// </summary>
        public ButtonGroup Group
        {
            get
            {
                return GetButtonGroup();
            }
            set
            {
                SetButtonGroup(value);
            }
        }


        private const string nativeName = "BaseButton";

        internal BaseButton() {}

        internal BaseButton(bool memoryOwn) : base(memoryOwn) {}

        /// <summary>
        /// Called when button is pressed.
        /// </summary>
        [GodotMethod("_pressed")]
        public virtual void _Pressed()
        {
            return;
        }

        /// <summary>
        /// Called when button is toggled (only if toggle_mode is active).
        /// </summary>
        [GodotMethod("_toggled")]
        public virtual void _Toggled(bool buttonPressed)
        {
            return;
        }

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_pressed");

        [GodotMethod("set_pressed")]
        public void SetPressed(bool pressed)
        {
            NativeCalls.godot_icall_1_11(method_bind_0, Object.GetPtr(this), pressed);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_pressed");

        [GodotMethod("is_pressed")]
        public bool IsPressed()
        {
            return NativeCalls.godot_icall_0_3(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_hovered");

        /// <summary>
        /// Return true if mouse entered the button before it exit.
        /// </summary>
        [GodotMethod("is_hovered")]
        public bool IsHovered()
        {
            return NativeCalls.godot_icall_0_3(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_toggle_mode");

        [GodotMethod("set_toggle_mode")]
        public void SetToggleMode(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_3, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_toggle_mode");

        [GodotMethod("is_toggle_mode")]
        public bool IsToggleMode()
        {
            return NativeCalls.godot_icall_0_3(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_disabled");

        [GodotMethod("set_disabled")]
        public void SetDisabled(bool disabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_5, Object.GetPtr(this), disabled);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_disabled");

        [GodotMethod("is_disabled")]
        public bool IsDisabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_action_mode");

        [GodotMethod("set_action_mode")]
        public void SetActionMode(BaseButton.ActionModeEnum mode)
        {
            NativeCalls.godot_icall_1_158(method_bind_7, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_action_mode");

        [GodotMethod("get_action_mode")]
        public BaseButton.ActionModeEnum GetActionMode()
        {
            return NativeCalls.godot_icall_0_159(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_draw_mode");

        /// <summary>
        /// Return the visual state used to draw the button. This is useful mainly when implementing your own draw code by either overriding _draw() or connecting to "draw" signal. The visual state of the button is defined by the DRAW_* enum.
        /// </summary>
        [GodotMethod("get_draw_mode")]
        public BaseButton.DrawMode GetDrawMode()
        {
            return NativeCalls.godot_icall_0_160(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_enabled_focus_mode");

        [GodotMethod("set_enabled_focus_mode")]
        public void SetEnabledFocusMode(Control.FocusModeEnum mode)
        {
            NativeCalls.godot_icall_1_161(method_bind_10, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_enabled_focus_mode");

        [GodotMethod("get_enabled_focus_mode")]
        public Control.FocusModeEnum GetEnabledFocusMode()
        {
            return NativeCalls.godot_icall_0_162(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shortcut");

        [GodotMethod("set_shortcut")]
        public void SetShortcut(ShortCut shortcut)
        {
            NativeCalls.godot_icall_1_24(method_bind_12, Object.GetPtr(this), Object.GetPtr(shortcut));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shortcut");

        [GodotMethod("get_shortcut")]
        public ShortCut GetShortcut()
        {
            return NativeCalls.godot_icall_0_163(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_button_group");

        [GodotMethod("set_button_group")]
        public void SetButtonGroup(ButtonGroup buttonGroup)
        {
            NativeCalls.godot_icall_1_24(method_bind_14, Object.GetPtr(this), Object.GetPtr(buttonGroup));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_button_group");

        [GodotMethod("get_button_group")]
        public ButtonGroup GetButtonGroup()
        {
            return NativeCalls.godot_icall_0_164(method_bind_15, Object.GetPtr(this));
        }
    }
}
