using System;

namespace Godot
{
    /// <summary>
    /// Renders text using [code]*.fnt[/code] fonts containing texture atlases. Supports distance fields. For using vector font files like TTF directly, see [DynamicFont].
    /// </summary>
    public class BitmapFont : Font
    {
        public object[] Textures
        {
            get
            {
                return _GetTextures();
            }
            set
            {
                _SetTextures(value);
            }
        }

        public int[] Chars
        {
            get
            {
                return _GetChars();
            }
            set
            {
                _SetChars(value);
            }
        }

        public int[] Kernings
        {
            get
            {
                return _GetKernings();
            }
            set
            {
                _SetKernings(value);
            }
        }

        /// <summary>
        /// Total font height (ascent plus descent) in pixels.
        /// </summary>
        public float Height
        {
            get
            {
                return GetHeight();
            }
            set
            {
                SetHeight(value);
            }
        }

        /// <summary>
        /// Ascent (number of pixels above the baseline).
        /// </summary>
        public float Ascent
        {
            get
            {
                return GetAscent();
            }
            set
            {
                SetAscent(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] distance field hint is enabled.
        /// </summary>
        public bool DistanceField
        {
            get
            {
                return IsDistanceFieldHint();
            }
            set
            {
                SetDistanceFieldHint(value);
            }
        }

        /// <summary>
        /// The fallback font.
        /// </summary>
        public BitmapFont Fallback
        {
            get
            {
                return GetFallback();
            }
            set
            {
                SetFallback(value);
            }
        }


        private const string nativeName = "BitmapFont";

        public BitmapFont() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_BitmapFont_Ctor(this);
        }

        internal BitmapFont(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create_from_fnt");

        /// <summary>
        /// Creates a BitmapFont from the [code]*.fnt[/code] file at [code]path[/code].
        /// </summary>
        [GodotMethod("create_from_fnt")]
        public Error CreateFromFnt(string path)
        {
            return NativeCalls.godot_icall_1_170(method_bind_0, Object.GetPtr(this), path);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_height");

        [GodotMethod("set_height")]
        public void SetHeight(float px)
        {
            NativeCalls.godot_icall_1_10(method_bind_1, Object.GetPtr(this), px);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ascent");

        [GodotMethod("set_ascent")]
        public void SetAscent(float px)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), px);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_kerning_pair");

        /// <summary>
        /// Adds a kerning pair to the [code]BitmapFont[/code] as a difference. Kerning pairs are special cases where a typeface advance is determined by the next character.
        /// </summary>
        [GodotMethod("add_kerning_pair")]
        public void AddKerningPair(int charA, int charB, int kerning)
        {
            NativeCalls.godot_icall_3_128(method_bind_3, Object.GetPtr(this), charA, charB, kerning);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_kerning_pair");

        /// <summary>
        /// Returns a kerning pair as a difference.
        /// </summary>
        [GodotMethod("get_kerning_pair")]
        public int GetKerningPair(int charA, int charB)
        {
            return NativeCalls.godot_icall_2_171(method_bind_4, Object.GetPtr(this), charA, charB);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_texture");

        /// <summary>
        /// Adds a texture to the [code]BitmapFont[/code].
        /// </summary>
        [GodotMethod("add_texture")]
        public void AddTexture(Texture texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_5, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_char");

        /// <summary>
        /// Adds a character to the font, where [code]character[/code] is the unicode value, [code]texture[/code] is the texture index, [code]rect[/code] is the region in the texture (in pixels!), [code]align[/code] is the (optional) alignment for the character and [code]advance[/code] is the (optional) advance.
        /// &lt;param name="align"&gt;If the param is null, then the default value is new Vector2(0, 0)&lt;/param&gt;
        /// </summary>
        [GodotMethod("add_char")]
        public void AddChar(int character, int texture, Rect2 rect, Nullable<Vector2> align = null, float advance = (float)-1)
        {
            Vector2 align_in = align.HasValue ? align.Value : new Vector2(0, 0);
            NativeCalls.godot_icall_5_172(method_bind_6, Object.GetPtr(this), character, texture, ref rect, ref align_in, advance);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture_count");

        /// <summary>
        /// Returns the number of textures in the BitmapFont atlas.
        /// </summary>
        [GodotMethod("get_texture_count")]
        public int GetTextureCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture");

        /// <summary>
        /// Returns the font atlas texture at index [code]idx[/code].
        /// </summary>
        [GodotMethod("get_texture")]
        public Texture GetTexture(int idx)
        {
            return NativeCalls.godot_icall_1_157(method_bind_8, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_char_size");

        /// <summary>
        /// Returns the size of a character, optionally taking kerning into account if the next character is provided.
        /// </summary>
        [GodotMethod("get_char_size")]
        public Vector2 GetCharSize(int @char, int next = 0)
        {
            object ret = NativeCalls.godot_icall_2_173(method_bind_9, Object.GetPtr(this), @char, next);
            return (Vector2)ret;
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_distance_field_hint");

        [GodotMethod("set_distance_field_hint")]
        public void SetDistanceFieldHint(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_10, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear");

        /// <summary>
        /// Clears all the font data and settings.
        /// </summary>
        [GodotMethod("clear")]
        public void Clear()
        {
            NativeCalls.godot_icall_0_12(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_chars");

        internal void _SetChars(int[] arg0)
        {
            NativeCalls.godot_icall_1_174(method_bind_12, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_chars");

        internal int[] _GetChars()
        {
            return NativeCalls.godot_icall_0_175(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_kernings");

        internal void _SetKernings(int[] arg0)
        {
            NativeCalls.godot_icall_1_174(method_bind_14, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_kernings");

        internal int[] _GetKernings()
        {
            return NativeCalls.godot_icall_0_175(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_textures");

        internal void _SetTextures(object[] arg0)
        {
            NativeCalls.godot_icall_1_154(method_bind_16, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_textures");

        internal object[] _GetTextures()
        {
            return NativeCalls.godot_icall_0_21(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fallback");

        [GodotMethod("set_fallback")]
        public void SetFallback(BitmapFont fallback)
        {
            NativeCalls.godot_icall_1_24(method_bind_18, Object.GetPtr(this), Object.GetPtr(fallback));
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fallback");

        [GodotMethod("get_fallback")]
        public BitmapFont GetFallback()
        {
            return NativeCalls.godot_icall_0_176(method_bind_19, Object.GetPtr(this));
        }
    }
}
