using System;

namespace Godot
{
    /// <summary>
    /// Button is the standard themed button. It can contain text and an icon, and will display them according to the current [Theme].
    /// </summary>
    public class Button : BaseButton
    {
        public enum TextAlign
        {
            Left = 0,
            Center = 1,
            Right = 2
        }


        /// <summary>
        /// The button's text that will be displayed inside the button's area.
        /// </summary>
        public string Text
        {
            get
            {
                return GetText();
            }
            set
            {
                SetText(value);
            }
        }

        /// <summary>
        /// Button's icon, if text is present the icon will be placed before the text.
        /// </summary>
        public Texture Icon
        {
            get
            {
                return GetButtonIcon();
            }
            set
            {
                SetButtonIcon(value);
            }
        }

        /// <summary>
        /// Flat buttons don't display decoration.
        /// </summary>
        public bool Flat
        {
            get
            {
                return IsFlat();
            }
            set
            {
                SetFlat(value);
            }
        }

        /// <summary>
        /// When this property is enabled, text that is too large to fit the button is clipped, when disabled the Button will always be wide enough to hold the text. This property is disabled by default.
        /// </summary>
        public bool ClipText
        {
            get
            {
                return GetClipText();
            }
            set
            {
                SetClipText(value);
            }
        }

        /// <summary>
        /// Text alignment policy for the button's text, use one of the ALIGN_* constants.
        /// </summary>
        public Button.TextAlign Align
        {
            get
            {
                return GetTextAlign();
            }
            set
            {
                SetTextAlign(value);
            }
        }


        private const string nativeName = "Button";

        public Button() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Button_Ctor(this);
        }

        internal Button(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_text");

        [GodotMethod("set_text")]
        public void SetText(string text)
        {
            NativeCalls.godot_icall_1_43(method_bind_0, Object.GetPtr(this), text);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_text");

        [GodotMethod("get_text")]
        public string GetText()
        {
            return NativeCalls.godot_icall_0_2(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_button_icon");

        [GodotMethod("set_button_icon")]
        public void SetButtonIcon(Texture texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_2, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_button_icon");

        [GodotMethod("get_button_icon")]
        public Texture GetButtonIcon()
        {
            return NativeCalls.godot_icall_0_119(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_flat");

        [GodotMethod("set_flat")]
        public void SetFlat(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_4, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_clip_text");

        [GodotMethod("set_clip_text")]
        public void SetClipText(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_5, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_clip_text");

        [GodotMethod("get_clip_text")]
        public bool GetClipText()
        {
            return NativeCalls.godot_icall_0_3(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_text_align");

        [GodotMethod("set_text_align")]
        public void SetTextAlign(Button.TextAlign align)
        {
            NativeCalls.godot_icall_1_179(method_bind_7, Object.GetPtr(this), align);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_text_align");

        [GodotMethod("get_text_align")]
        public Button.TextAlign GetTextAlign()
        {
            return NativeCalls.godot_icall_0_180(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_flat");

        [GodotMethod("is_flat")]
        public bool IsFlat()
        {
            return NativeCalls.godot_icall_0_3(method_bind_9, Object.GetPtr(this));
        }
    }
}
