using System;

namespace Godot
{
    /// <summary>
    /// Camera is a special node that displays what is visible from its current location. Cameras register themselves in the nearest [Viewport] node (when ascending the tree). Only one camera can be active per viewport. If no viewport is available ascending the tree, the Camera will register in the global viewport. In other words, a Camera just provides [i]3D[/i] display capabilities to a [Viewport], and, without one, a scene registered in that [Viewport] (or higher viewports) can't be displayed.
    /// </summary>
    public class Camera : Spatial
    {
        public enum KeepAspectEnum
        {
            Width = 0,
            Height = 1
        }

        public enum ProjectionEnum
        {
            Perspective = 0,
            Orthogonal = 1
        }

        public enum DopplerTrackingEnum
        {
            Disabled = 0,
            IdleStep = 1,
            PhysicsStep = 2
        }


        /// <summary>
        /// The axis to lock during [member fov]/[member size] adjustments.
        /// </summary>
        public Camera.KeepAspectEnum KeepAspect
        {
            get
            {
                return GetKeepAspectMode();
            }
            set
            {
                SetKeepAspectMode(value);
            }
        }

        /// <summary>
        /// The culling mask that describes which 3D render layers are rendered by this camera.
        /// </summary>
        public int CullMask
        {
            get
            {
                return GetCullMask();
            }
            set
            {
                SetCullMask(value);
            }
        }

        /// <summary>
        /// Set the [Environment] to use for this Camera.
        /// </summary>
        public Environment Environment
        {
            get
            {
                return GetEnvironment();
            }
            set
            {
                SetEnvironment(value);
            }
        }

        /// <summary>
        /// The horizontal (X) offset of the Camear viewport.
        /// </summary>
        public float HOffset
        {
            get
            {
                return GetHOffset();
            }
            set
            {
                SetHOffset(value);
            }
        }

        /// <summary>
        /// The horizontal (Y) offset of the Camear viewport.
        /// </summary>
        public float VOffset
        {
            get
            {
                return GetVOffset();
            }
            set
            {
                SetVOffset(value);
            }
        }

        /// <summary>
        /// If not [code]DOPPLER_TRACKING_DISABLED[/code] this Camera will simulate the Doppler effect for objects changed in particular [code]_process[/code] methods. Default value: [code]DOPPLER_TRACKING_DISABLED[/code].
        /// </summary>
        public Camera.DopplerTrackingEnum DopplerTracking
        {
            get
            {
                return GetDopplerTracking();
            }
            set
            {
                SetDopplerTracking(value);
            }
        }

        /// <summary>
        /// The camera's projection mode. In [code]PROJECTION_PERSPECTIVE[/code] mode, objects' z-distance from the camera's local space scales their perceived size.
        /// </summary>
        public Camera.ProjectionEnum Projection
        {
            get
            {
                return GetProjection();
            }
            set
            {
                SetProjection(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the ancestor [Viewport] is currently using this Camera. Default value: [code]false[/code].
        /// </summary>
        public bool Current
        {
            get
            {
                return IsCurrent();
            }
            set
            {
                SetCurrent(value);
            }
        }

        /// <summary>
        /// The camera's field of view angle (in degrees). Only applicable in perspective mode. Since [member keep_aspect] locks one axis, [code]fov[/code] sets the other axis' field of view angle.
        /// </summary>
        public float Fov
        {
            get
            {
                return GetFov();
            }
            set
            {
                SetFov(value);
            }
        }

        /// <summary>
        /// The camera's size measured as 1/2 the width or height. Only applicable in orthogonal mode. Since [member keep_aspect] locks on axis, [code]size[/code] sets the other axis' size length.
        /// </summary>
        public float Size
        {
            get
            {
                return GetSize();
            }
            set
            {
                SetSize(value);
            }
        }

        /// <summary>
        /// The distance to the near culling boundary for this Camera relative to its local z-axis.
        /// </summary>
        public float Near
        {
            get
            {
                return GetZnear();
            }
            set
            {
                SetZnear(value);
            }
        }

        /// <summary>
        /// The distance to the far culling boundary for this Camera relative to its local z-axis.
        /// </summary>
        public float Far
        {
            get
            {
                return GetZfar();
            }
            set
            {
                SetZfar(value);
            }
        }


        private const string nativeName = "Camera";

        public Camera() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Camera_Ctor(this);
        }

        internal Camera(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "project_ray_normal");

        /// <summary>
        /// Returns a normal vector in worldspace, that is the result of projecting a point on the [Viewport] rectangle by the camera projection. This is useful for casting rays in the form of (origin, normal) for object intersection or picking.
        /// </summary>
        [GodotMethod("project_ray_normal")]
        public Vector3 ProjectRayNormal(Vector2 screenPoint)
        {
            object ret = NativeCalls.godot_icall_1_183(method_bind_0, Object.GetPtr(this), ref screenPoint);
            return (Vector3)ret;
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "project_local_ray_normal");

        /// <summary>
        /// Returns a normal vector from the screen point location directed along the camera. Orthogonal cameras are normalized. Perspective cameras account for perspective, screen width/height, etc.
        /// </summary>
        [GodotMethod("project_local_ray_normal")]
        public Vector3 ProjectLocalRayNormal(Vector2 screenPoint)
        {
            object ret = NativeCalls.godot_icall_1_183(method_bind_1, Object.GetPtr(this), ref screenPoint);
            return (Vector3)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "project_ray_origin");

        /// <summary>
        /// Returns a 3D position in worldspace, that is the result of projecting a point on the [Viewport] rectangle by the camera projection. This is useful for casting rays in the form of (origin, normal) for object intersection or picking.
        /// </summary>
        [GodotMethod("project_ray_origin")]
        public Vector3 ProjectRayOrigin(Vector2 screenPoint)
        {
            object ret = NativeCalls.godot_icall_1_183(method_bind_2, Object.GetPtr(this), ref screenPoint);
            return (Vector3)ret;
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "unproject_position");

        /// <summary>
        /// Returns how a 3D point in worldspace maps to a 2D coordinate in the [Viewport] rectangle.
        /// </summary>
        [GodotMethod("unproject_position")]
        public Vector2 UnprojectPosition(Vector3 worldPoint)
        {
            object ret = NativeCalls.godot_icall_1_184(method_bind_3, Object.GetPtr(this), ref worldPoint);
            return (Vector2)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_position_behind");

        /// <summary>
        /// Returns [code]true[/code] if the given position is behind the Camera.
        /// </summary>
        [GodotMethod("is_position_behind")]
        public bool IsPositionBehind(Vector3 worldPoint)
        {
            return NativeCalls.godot_icall_1_185(method_bind_4, Object.GetPtr(this), ref worldPoint);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "project_position");

        /// <summary>
        /// Returns how a 2D coordinate in the Viewport rectangle maps to a 3D point in worldspace.
        /// </summary>
        [GodotMethod("project_position")]
        public Vector3 ProjectPosition(Vector2 screenPoint)
        {
            object ret = NativeCalls.godot_icall_1_183(method_bind_5, Object.GetPtr(this), ref screenPoint);
            return (Vector3)ret;
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_perspective");

        /// <summary>
        /// Set the camera projection to perspective mode, by specifying a [i]FOV[/i] Y angle in degrees (FOV means Field of View), and the [i]near[/i] and [i]far[/i] clip planes in worldspace units.
        /// </summary>
        [GodotMethod("set_perspective")]
        public void SetPerspective(float fov, float zNear, float zFar)
        {
            NativeCalls.godot_icall_3_186(method_bind_6, Object.GetPtr(this), fov, zNear, zFar);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_orthogonal");

        /// <summary>
        /// Set the camera projection to orthogonal mode, by specifying a width and the [i]near[/i] and [i]far[/i] clip planes in worldspace units. (As a hint, 2D games often use this projection, with values specified in pixels)
        /// </summary>
        [GodotMethod("set_orthogonal")]
        public void SetOrthogonal(float size, float zNear, float zFar)
        {
            NativeCalls.godot_icall_3_186(method_bind_7, Object.GetPtr(this), size, zNear, zFar);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "make_current");

        /// <summary>
        /// Make this camera the current Camera for the [Viewport] (see class description). If the Camera Node is outside the scene tree, it will attempt to become current once it's added.
        /// </summary>
        [GodotMethod("make_current")]
        public void MakeCurrent()
        {
            NativeCalls.godot_icall_0_12(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_current");

        /// <summary>
        /// If this is the current Camera, remove it from being current. If it is inside the node tree, request to make the next Camera current, if any.
        /// </summary>
        [GodotMethod("clear_current")]
        public void ClearCurrent()
        {
            NativeCalls.godot_icall_0_12(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_current");

        [GodotMethod("set_current")]
        public void SetCurrent(bool arg0)
        {
            NativeCalls.godot_icall_1_11(method_bind_10, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_current");

        [GodotMethod("is_current")]
        public bool IsCurrent()
        {
            return NativeCalls.godot_icall_0_3(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_camera_transform");

        /// <summary>
        /// Get the camera transform. Subclassed cameras (such as CharacterCamera) may provide different transforms than the [Node] transform.
        /// </summary>
        [GodotMethod("get_camera_transform")]
        public Transform GetCameraTransform()
        {
            object ret = NativeCalls.godot_icall_0_18(method_bind_12, Object.GetPtr(this));
            return (Transform)ret;
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fov");

        [GodotMethod("get_fov")]
        public float GetFov()
        {
            return NativeCalls.godot_icall_0_9(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_size");

        [GodotMethod("get_size")]
        public float GetSize()
        {
            return NativeCalls.godot_icall_0_9(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_zfar");

        [GodotMethod("get_zfar")]
        public float GetZfar()
        {
            return NativeCalls.godot_icall_0_9(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_znear");

        [GodotMethod("get_znear")]
        public float GetZnear()
        {
            return NativeCalls.godot_icall_0_9(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fov");

        [GodotMethod("set_fov")]
        public void SetFov(float arg0)
        {
            NativeCalls.godot_icall_1_10(method_bind_17, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_size");

        [GodotMethod("set_size")]
        public void SetSize(float arg0)
        {
            NativeCalls.godot_icall_1_10(method_bind_18, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_zfar");

        [GodotMethod("set_zfar")]
        public void SetZfar(float arg0)
        {
            NativeCalls.godot_icall_1_10(method_bind_19, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_znear");

        [GodotMethod("set_znear")]
        public void SetZnear(float arg0)
        {
            NativeCalls.godot_icall_1_10(method_bind_20, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_projection");

        [GodotMethod("get_projection")]
        public Camera.ProjectionEnum GetProjection()
        {
            return NativeCalls.godot_icall_0_187(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_projection");

        [GodotMethod("set_projection")]
        public void SetProjection(Camera.ProjectionEnum arg0)
        {
            NativeCalls.godot_icall_1_188(method_bind_22, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_h_offset");

        [GodotMethod("set_h_offset")]
        public void SetHOffset(float ofs)
        {
            NativeCalls.godot_icall_1_10(method_bind_23, Object.GetPtr(this), ofs);
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_h_offset");

        [GodotMethod("get_h_offset")]
        public float GetHOffset()
        {
            return NativeCalls.godot_icall_0_9(method_bind_24, Object.GetPtr(this));
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_v_offset");

        [GodotMethod("set_v_offset")]
        public void SetVOffset(float ofs)
        {
            NativeCalls.godot_icall_1_10(method_bind_25, Object.GetPtr(this), ofs);
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_v_offset");

        [GodotMethod("get_v_offset")]
        public float GetVOffset()
        {
            return NativeCalls.godot_icall_0_9(method_bind_26, Object.GetPtr(this));
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_cull_mask");

        [GodotMethod("set_cull_mask")]
        public void SetCullMask(int mask)
        {
            NativeCalls.godot_icall_1_0(method_bind_27, Object.GetPtr(this), mask);
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cull_mask");

        [GodotMethod("get_cull_mask")]
        public int GetCullMask()
        {
            return NativeCalls.godot_icall_0_1(method_bind_28, Object.GetPtr(this));
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_environment");

        [GodotMethod("set_environment")]
        public void SetEnvironment(Environment env)
        {
            NativeCalls.godot_icall_1_24(method_bind_29, Object.GetPtr(this), Object.GetPtr(env));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_environment");

        [GodotMethod("get_environment")]
        public Environment GetEnvironment()
        {
            return NativeCalls.godot_icall_0_189(method_bind_30, Object.GetPtr(this));
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_keep_aspect_mode");

        [GodotMethod("set_keep_aspect_mode")]
        public void SetKeepAspectMode(Camera.KeepAspectEnum mode)
        {
            NativeCalls.godot_icall_1_190(method_bind_31, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_keep_aspect_mode");

        [GodotMethod("get_keep_aspect_mode")]
        public Camera.KeepAspectEnum GetKeepAspectMode()
        {
            return NativeCalls.godot_icall_0_191(method_bind_32, Object.GetPtr(this));
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_doppler_tracking");

        [GodotMethod("set_doppler_tracking")]
        public void SetDopplerTracking(Camera.DopplerTrackingEnum mode)
        {
            NativeCalls.godot_icall_1_192(method_bind_33, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_doppler_tracking");

        [GodotMethod("get_doppler_tracking")]
        public Camera.DopplerTrackingEnum GetDopplerTracking()
        {
            return NativeCalls.godot_icall_0_193(method_bind_34, Object.GetPtr(this));
        }
    }
}
