using System;

namespace Godot
{
    /// <summary>
    /// Camera node for 2D scenes. It forces the screen (current layer) to scroll following this node. This makes it easier (and faster) to program scrollable scenes than manually changing the position of [CanvasItem] based nodes.
    /// This node is intended to be a simple helper to get things going quickly and it may happen often that more functionality is desired to change how the camera works. To make your own custom camera node, simply inherit from [Node2D] and change the transform of the canvas by calling get_viewport().set_canvas_transform(m) in [Viewport].
    /// </summary>
    public class Camera2D : Node2D
    {
        public enum AnchorModeEnum
        {
            FixedTopLeft = 0,
            DragCenter = 1
        }


        /// <summary>
        /// The camera's offset, useful for looking around or camera shake animations.
        /// </summary>
        public Vector2 Offset
        {
            get
            {
                return GetOffset();
            }
            set
            {
                SetOffset(value);
            }
        }

        /// <summary>
        /// The Camera2D's anchor point. See [code]ANCHOR_MODE_*[/code] constants.
        /// </summary>
        public Camera2D.AnchorModeEnum AnchorMode
        {
            get
            {
                return GetAnchorMode();
            }
            set
            {
                SetAnchorMode(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the camera rotates with the target. Default value: [code]false[/code]
        /// </summary>
        public bool Rotating
        {
            get
            {
                return IsRotating();
            }
            set
            {
                SetRotating(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the camera is the active camera for the current scene. Only one camera can be current, so setting a different camera [code]current[/code] will disable this one.
        /// </summary>
        public bool Current
        {
            get
            {
                return IsCurrent();
            }
            set
            {
                _SetCurrent(value);
            }
        }

        /// <summary>
        /// The camera's zoom relative to the viewport. Values larger than [code]Vector2(1, 1)[/code] zoom out and smaller values zoom in. For an example, use [code]Vector2(0.5, 0.5)[/code] for a 2x zoom in, and [code]Vector2(4, 4)[/code] for a 4x zoom out.
        /// </summary>
        public Vector2 Zoom
        {
            get
            {
                return GetZoom();
            }
            set
            {
                SetZoom(value);
            }
        }

        /// <summary>
        /// The custom [Viewport] node attached to the [code]Camera2D[/code]. If null or not a [Viewport], uses the default viewport instead.
        /// </summary>
        public Node CustomViewport
        {
            get
            {
                return GetCustomViewport();
            }
            set
            {
                SetCustomViewport(value);
            }
        }

        /// <summary>
        /// Left scroll limit in pixels. The camera stops moving when reaching this value.
        /// </summary>
        public int LimitLeft
        {
            get
            {
                return GetLimit((Margin)0);
            }
            set
            {
                SetLimit((Margin)0, value);
            }
        }

        /// <summary>
        /// Top scroll limit in pixels. The camera stops moving when reaching this value.
        /// </summary>
        public int LimitTop
        {
            get
            {
                return GetLimit((Margin)1);
            }
            set
            {
                SetLimit((Margin)1, value);
            }
        }

        /// <summary>
        /// Right scroll limit in pixels. The camera stops moving when reaching this value.
        /// </summary>
        public int LimitRight
        {
            get
            {
                return GetLimit((Margin)2);
            }
            set
            {
                SetLimit((Margin)2, value);
            }
        }

        /// <summary>
        /// Bottom scroll limit in pixels. The camera stops moving when reaching this value.
        /// </summary>
        public int LimitBottom
        {
            get
            {
                return GetLimit((Margin)3);
            }
            set
            {
                SetLimit((Margin)3, value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the camera smoothly stops when reaches its limits. Default value: [code]false[/code]
        /// </summary>
        public bool LimitSmoothed
        {
            get
            {
                return IsLimitSmoothingEnabled();
            }
            set
            {
                SetLimitSmoothingEnabled(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the camera only moves when reaching the horizontal drag margins. If [code]false[/code] the camera moves horizontally regardless of margins. Default value: [code]true[/code].
        /// </summary>
        public bool DragMarginHEnabled
        {
            get
            {
                return IsHDragEnabled();
            }
            set
            {
                SetHDragEnabled(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the camera only moves when reaching the vertical drag margins. If [code]false[/code] the camera moves vertically regardless of margins. Default value: [code]true[/code].
        /// </summary>
        public bool DragMarginVEnabled
        {
            get
            {
                return IsVDragEnabled();
            }
            set
            {
                SetVDragEnabled(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the camera smoothly moves towards the target at [member smoothing_speed]. Default value: [code]false[/code]
        /// </summary>
        public bool SmoothingEnabled
        {
            get
            {
                return IsFollowSmoothingEnabled();
            }
            set
            {
                SetEnableFollowSmoothing(value);
            }
        }

        /// <summary>
        /// Speed in pixels per second of the camera's smoothing effect when [member smoothing_enabled] is [code]true[/code]
        /// </summary>
        public float SmoothingSpeed
        {
            get
            {
                return GetFollowSmoothing();
            }
            set
            {
                SetFollowSmoothing(value);
            }
        }

        /// <summary>
        /// The vertical offset of the camera, relative to the drag margins. Default value: [code]0[/code]
        /// </summary>
        public float OffsetV
        {
            get
            {
                return GetVOffset();
            }
            set
            {
                SetVOffset(value);
            }
        }

        /// <summary>
        /// The horizontal offset of the camera, relative to the drag margins. Default value: [code]0[/code]
        /// </summary>
        public float OffsetH
        {
            get
            {
                return GetHOffset();
            }
            set
            {
                SetHOffset(value);
            }
        }

        /// <summary>
        /// Left margin needed to drag the camera. A value of [code]1[/code] makes the camera move only when reaching the edge of the screen.
        /// </summary>
        public float DragMarginLeft
        {
            get
            {
                return GetDragMargin((Margin)0);
            }
            set
            {
                SetDragMargin((Margin)0, value);
            }
        }

        /// <summary>
        /// Top margin needed to drag the camera. A value of [code]1[/code] makes the camera move only when reaching the edge of the screen.
        /// </summary>
        public float DragMarginTop
        {
            get
            {
                return GetDragMargin((Margin)1);
            }
            set
            {
                SetDragMargin((Margin)1, value);
            }
        }

        /// <summary>
        /// Right margin needed to drag the camera. A value of [code]1[/code] makes the camera move only when reaching the edge of the screen.
        /// </summary>
        public float DragMarginRight
        {
            get
            {
                return GetDragMargin((Margin)2);
            }
            set
            {
                SetDragMargin((Margin)2, value);
            }
        }

        /// <summary>
        /// Bottom margin needed to drag the camera. A value of [code]1[/code] makes the camera move only when reaching the edge of the screen.
        /// </summary>
        public float DragMarginBottom
        {
            get
            {
                return GetDragMargin((Margin)3);
            }
            set
            {
                SetDragMargin((Margin)3, value);
            }
        }

        /// <summary>
        /// If [code]true[/code] draws the camera's screen rectangle in the editor. Default value: [code]false[/code]
        /// </summary>
        public bool EditorDrawScreen
        {
            get
            {
                return IsScreenDrawingEnabled();
            }
            set
            {
                SetScreenDrawingEnabled(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] draws the camera's limits rectangle in the editor. Default value: [code]true[/code]
        /// </summary>
        public bool EditorDrawLimits
        {
            get
            {
                return IsLimitDrawingEnabled();
            }
            set
            {
                SetLimitDrawingEnabled(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] draws the camera's drag margin rectangle in the editor. Default value: [code]false[/code]
        /// </summary>
        public bool EditorDrawDragMargin
        {
            get
            {
                return IsMarginDrawingEnabled();
            }
            set
            {
                SetMarginDrawingEnabled(value);
            }
        }


        private const string nativeName = "Camera2D";

        public Camera2D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Camera2D_Ctor(this);
        }

        internal Camera2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_offset");

        [GodotMethod("set_offset")]
        public void SetOffset(Vector2 offset)
        {
            NativeCalls.godot_icall_1_45(method_bind_0, Object.GetPtr(this), ref offset);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_offset");

        [GodotMethod("get_offset")]
        public Vector2 GetOffset()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_1, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_anchor_mode");

        [GodotMethod("set_anchor_mode")]
        public void SetAnchorMode(Camera2D.AnchorModeEnum anchorMode)
        {
            NativeCalls.godot_icall_1_194(method_bind_2, Object.GetPtr(this), anchorMode);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_anchor_mode");

        [GodotMethod("get_anchor_mode")]
        public Camera2D.AnchorModeEnum GetAnchorMode()
        {
            return NativeCalls.godot_icall_0_195(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rotating");

        [GodotMethod("set_rotating")]
        public void SetRotating(bool rotating)
        {
            NativeCalls.godot_icall_1_11(method_bind_4, Object.GetPtr(this), rotating);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_rotating");

        [GodotMethod("is_rotating")]
        public bool IsRotating()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "make_current");

        /// <summary>
        /// Make this the current 2D camera for the scene (viewport and layer), in case there's many cameras in the scene.
        /// </summary>
        [GodotMethod("make_current")]
        public void MakeCurrent()
        {
            NativeCalls.godot_icall_0_12(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_current");

        /// <summary>
        /// Removes any [code]Camera2D[/code] from the ancestor [Viewport]'s internal currently-assigned camera.
        /// </summary>
        [GodotMethod("clear_current")]
        public void ClearCurrent()
        {
            NativeCalls.godot_icall_0_12(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_current");

        internal void _SetCurrent(bool current)
        {
            NativeCalls.godot_icall_1_11(method_bind_8, Object.GetPtr(this), current);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_current");

        [GodotMethod("is_current")]
        public bool IsCurrent()
        {
            return NativeCalls.godot_icall_0_3(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_limit");

        [GodotMethod("set_limit")]
        public void SetLimit(Margin margin, int limit)
        {
            NativeCalls.godot_icall_2_196(method_bind_10, Object.GetPtr(this), margin, limit);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_limit");

        [GodotMethod("get_limit")]
        public int GetLimit(Margin margin)
        {
            return NativeCalls.godot_icall_1_197(method_bind_11, Object.GetPtr(this), margin);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_limit_smoothing_enabled");

        [GodotMethod("set_limit_smoothing_enabled")]
        public void SetLimitSmoothingEnabled(bool limitSmoothingEnabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_12, Object.GetPtr(this), limitSmoothingEnabled);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_limit_smoothing_enabled");

        [GodotMethod("is_limit_smoothing_enabled")]
        public bool IsLimitSmoothingEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_v_drag_enabled");

        [GodotMethod("set_v_drag_enabled")]
        public void SetVDragEnabled(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_14, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_v_drag_enabled");

        [GodotMethod("is_v_drag_enabled")]
        public bool IsVDragEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_h_drag_enabled");

        [GodotMethod("set_h_drag_enabled")]
        public void SetHDragEnabled(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_16, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_h_drag_enabled");

        [GodotMethod("is_h_drag_enabled")]
        public bool IsHDragEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_v_offset");

        [GodotMethod("set_v_offset")]
        public void SetVOffset(float ofs)
        {
            NativeCalls.godot_icall_1_10(method_bind_18, Object.GetPtr(this), ofs);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_v_offset");

        [GodotMethod("get_v_offset")]
        public float GetVOffset()
        {
            return NativeCalls.godot_icall_0_9(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_h_offset");

        [GodotMethod("set_h_offset")]
        public void SetHOffset(float ofs)
        {
            NativeCalls.godot_icall_1_10(method_bind_20, Object.GetPtr(this), ofs);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_h_offset");

        [GodotMethod("get_h_offset")]
        public float GetHOffset()
        {
            return NativeCalls.godot_icall_0_9(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_drag_margin");

        [GodotMethod("set_drag_margin")]
        public void SetDragMargin(Margin margin, float dragMargin)
        {
            NativeCalls.godot_icall_2_198(method_bind_22, Object.GetPtr(this), margin, dragMargin);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_drag_margin");

        [GodotMethod("get_drag_margin")]
        public float GetDragMargin(Margin margin)
        {
            return NativeCalls.godot_icall_1_199(method_bind_23, Object.GetPtr(this), margin);
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_camera_position");

        /// <summary>
        /// Return the camera position.
        /// </summary>
        [GodotMethod("get_camera_position")]
        public Vector2 GetCameraPosition()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_24, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_camera_screen_center");

        /// <summary>
        /// Returns the location of the [code]Camera2D[/code]'s screen-center, relative to the origin.
        /// </summary>
        [GodotMethod("get_camera_screen_center")]
        public Vector2 GetCameraScreenCenter()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_25, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_zoom");

        [GodotMethod("set_zoom")]
        public void SetZoom(Vector2 zoom)
        {
            NativeCalls.godot_icall_1_45(method_bind_26, Object.GetPtr(this), ref zoom);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_zoom");

        [GodotMethod("get_zoom")]
        public Vector2 GetZoom()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_27, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_custom_viewport");

        [GodotMethod("set_custom_viewport")]
        public void SetCustomViewport(Node viewport)
        {
            NativeCalls.godot_icall_1_42(method_bind_28, Object.GetPtr(this), Object.GetPtr(viewport));
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_custom_viewport");

        [GodotMethod("get_custom_viewport")]
        public Node GetCustomViewport()
        {
            return NativeCalls.godot_icall_0_200(method_bind_29, Object.GetPtr(this));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_follow_smoothing");

        [GodotMethod("set_follow_smoothing")]
        public void SetFollowSmoothing(float followSmoothing)
        {
            NativeCalls.godot_icall_1_10(method_bind_30, Object.GetPtr(this), followSmoothing);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_follow_smoothing");

        [GodotMethod("get_follow_smoothing")]
        public float GetFollowSmoothing()
        {
            return NativeCalls.godot_icall_0_9(method_bind_31, Object.GetPtr(this));
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_enable_follow_smoothing");

        [GodotMethod("set_enable_follow_smoothing")]
        public void SetEnableFollowSmoothing(bool followSmoothing)
        {
            NativeCalls.godot_icall_1_11(method_bind_32, Object.GetPtr(this), followSmoothing);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_follow_smoothing_enabled");

        [GodotMethod("is_follow_smoothing_enabled")]
        public bool IsFollowSmoothingEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_33, Object.GetPtr(this));
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "force_update_scroll");

        /// <summary>
        /// Force the camera to update scroll immediately.
        /// </summary>
        [GodotMethod("force_update_scroll")]
        public void ForceUpdateScroll()
        {
            NativeCalls.godot_icall_0_12(method_bind_34, Object.GetPtr(this));
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "reset_smoothing");

        /// <summary>
        /// Set the camera's position immediately to its current smoothing destination.
        /// This has no effect if smoothing is disabled.
        /// </summary>
        [GodotMethod("reset_smoothing")]
        public void ResetSmoothing()
        {
            NativeCalls.godot_icall_0_12(method_bind_35, Object.GetPtr(this));
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "align");

        /// <summary>
        /// Align the camera to the tracked node
        /// </summary>
        [GodotMethod("align")]
        public void Align()
        {
            NativeCalls.godot_icall_0_12(method_bind_36, Object.GetPtr(this));
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_screen_drawing_enabled");

        [GodotMethod("set_screen_drawing_enabled")]
        public void SetScreenDrawingEnabled(bool screenDrawingEnabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_37, Object.GetPtr(this), screenDrawingEnabled);
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_screen_drawing_enabled");

        [GodotMethod("is_screen_drawing_enabled")]
        public bool IsScreenDrawingEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_38, Object.GetPtr(this));
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_limit_drawing_enabled");

        [GodotMethod("set_limit_drawing_enabled")]
        public void SetLimitDrawingEnabled(bool limitDrawingEnabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_39, Object.GetPtr(this), limitDrawingEnabled);
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_limit_drawing_enabled");

        [GodotMethod("is_limit_drawing_enabled")]
        public bool IsLimitDrawingEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_40, Object.GetPtr(this));
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_margin_drawing_enabled");

        [GodotMethod("set_margin_drawing_enabled")]
        public void SetMarginDrawingEnabled(bool marginDrawingEnabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_41, Object.GetPtr(this), marginDrawingEnabled);
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_margin_drawing_enabled");

        [GodotMethod("is_margin_drawing_enabled")]
        public bool IsMarginDrawingEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_42, Object.GetPtr(this));
        }
    }
}
