using System;
using System.Collections.Generic;

namespace Godot
{
    /// <summary>
    /// Base class of anything 2D. Canvas items are laid out in a tree and children inherit and extend the transform of their parent. CanvasItem is extended by [Control], for anything GUI related, and by [Node2D] for anything 2D engine related.
    /// Any CanvasItem can draw. For this, the "update" function must be called, then NOTIFICATION_DRAW will be received on idle time to request redraw. Because of this, canvas items don't need to be redraw on every frame, improving the performance significantly. Several functions for drawing on the CanvasItem are provided (see draw_* functions). They can only be used inside the notification, signal or _draw() overrides function, though.
    /// Canvas items are draw in tree order. By default, children are on top of their parents so a root CanvasItem will be drawn behind everything (this can be changed per item though).
    /// Canvas items can also be hidden (hiding also their subtree). They provide many means for changing standard parameters such as opacity (for it and the subtree) and self opacity, blend mode.
    /// Ultimately, a transform notification can be requested, which will notify the node that its global position changed in case the parent tree changed.
    /// </summary>
    public class CanvasItem : Node
    {
        public const int NotificationTransformChanged = 29;
        public const int NotificationDraw = 30;
        public const int NotificationVisibilityChanged = 31;
        public const int NotificationEnterCanvas = 32;
        public const int NotificationExitCanvas = 33;

        public enum BlendMode
        {
            Mix = 0,
            Add = 1,
            Sub = 2,
            Mul = 3,
            PremultAlpha = 4
        }


        /// <summary>
        /// If [code]true[/code] this [code]CanvasItem[/code] is drawn. Default value: [code]true[/code].
        /// </summary>
        public bool Visible
        {
            get
            {
                return IsVisible();
            }
            set
            {
                SetVisible(value);
            }
        }

        /// <summary>
        /// The color applied to textures on this [code]CanvasItem[/code]. Default value: [code]Color(1, 1, 1, 1)[/code] (opaque "white").
        /// </summary>
        public Color Modulate
        {
            get
            {
                return GetModulate();
            }
            set
            {
                SetModulate(value);
            }
        }

        /// <summary>
        /// The color applied to textures on this [code]CanvasItem[/code]. This is not inherited by children [code]CanvasItem[/code]s. Default value: [code]Color(1, 1, 1, 1)[/code] (opaque "white")..
        /// </summary>
        public Color SelfModulate
        {
            get
            {
                return GetSelfModulate();
            }
            set
            {
                SetSelfModulate(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the object draws behind its parent. Default value: [code]false[/code].
        /// </summary>
        public bool ShowBehindParent
        {
            get
            {
                return IsDrawBehindParentEnabled();
            }
            set
            {
                SetDrawBehindParent(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the object draws on top of its parent. Default value: [code]true[/code].
        /// </summary>
        public bool ShowOnTop
        {
            get
            {
                return _IsOnTop();
            }
            set
            {
                _SetOnTop(value);
            }
        }

        /// <summary>
        /// The rendering layers in which this [code]CanvasItem[/code] responds to [Light2D] nodes. Default value: [code]1[/code].
        /// </summary>
        public int LightMask
        {
            get
            {
                return GetLightMask();
            }
            set
            {
                SetLightMask(value);
            }
        }

        /// <summary>
        /// The material applied to textures on this [code]CanvasItem[/code]. Default value: [code]null[/code].
        /// </summary>
        public Material Material
        {
            get
            {
                return GetMaterial();
            }
            set
            {
                SetMaterial(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the parent [code]CanvasItem[/code]'s [member material] property is used as this one's material. Default value: [code]false[/code].
        /// </summary>
        public bool UseParentMaterial
        {
            get
            {
                return GetUseParentMaterial();
            }
            set
            {
                SetUseParentMaterial(value);
            }
        }


        private const string nativeName = "CanvasItem";

        internal CanvasItem() {}

        internal CanvasItem(bool memoryOwn) : base(memoryOwn) {}

        /// <summary>
        /// Called (if exists) to draw the canvas item.
        /// </summary>
        [GodotMethod("_draw")]
        public virtual void _Draw()
        {
            return;
        }

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_canvas_item");

        /// <summary>
        /// Return the canvas item RID used by [VisualServer] for this item.
        /// </summary>
        [GodotMethod("get_canvas_item")]
        public RID GetCanvasItem()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_0, Object.GetPtr(this));
            return new RID(ret);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_visible");

        [GodotMethod("set_visible")]
        public void SetVisible(bool visible)
        {
            NativeCalls.godot_icall_1_11(method_bind_1, Object.GetPtr(this), visible);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_visible");

        [GodotMethod("is_visible")]
        public bool IsVisible()
        {
            return NativeCalls.godot_icall_0_3(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_visible_in_tree");

        /// <summary>
        /// Returns [code]true[/code] if the node is in the [SceneTree] and is visible on-screen.
        /// </summary>
        [GodotMethod("is_visible_in_tree")]
        public bool IsVisibleInTree()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "show");

        /// <summary>
        /// Show the CanvasItem currently hidden.
        /// </summary>
        [GodotMethod("show")]
        public void Show()
        {
            NativeCalls.godot_icall_0_12(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "hide");

        /// <summary>
        /// Hide the CanvasItem currently visible.
        /// </summary>
        [GodotMethod("hide")]
        public void Hide()
        {
            NativeCalls.godot_icall_0_12(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "update");

        /// <summary>
        /// Queue the CanvasItem for update. [code]NOTIFICATION_DRAW[/code] will be called on idle time to request redraw.
        /// </summary>
        [GodotMethod("update")]
        public void Update()
        {
            NativeCalls.godot_icall_0_12(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_as_toplevel");

        /// <summary>
        /// Sets as top level. This means that it will not inherit transform from parent canvas items.
        /// </summary>
        [GodotMethod("set_as_toplevel")]
        public void SetAsToplevel(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_7, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_set_as_toplevel");

        /// <summary>
        /// Return if set as toplevel. See [method set_as_toplevel].
        /// </summary>
        [GodotMethod("is_set_as_toplevel")]
        public bool IsSetAsToplevel()
        {
            return NativeCalls.godot_icall_0_3(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_light_mask");

        [GodotMethod("set_light_mask")]
        public void SetLightMask(int lightMask)
        {
            NativeCalls.godot_icall_1_0(method_bind_9, Object.GetPtr(this), lightMask);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_light_mask");

        [GodotMethod("get_light_mask")]
        public int GetLightMask()
        {
            return NativeCalls.godot_icall_0_1(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_modulate");

        [GodotMethod("set_modulate")]
        public void SetModulate(Color modulate)
        {
            NativeCalls.godot_icall_1_202(method_bind_11, Object.GetPtr(this), ref modulate);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_modulate");

        [GodotMethod("get_modulate")]
        public Color GetModulate()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_12, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_self_modulate");

        [GodotMethod("set_self_modulate")]
        public void SetSelfModulate(Color selfModulate)
        {
            NativeCalls.godot_icall_1_202(method_bind_13, Object.GetPtr(this), ref selfModulate);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_self_modulate");

        [GodotMethod("get_self_modulate")]
        public Color GetSelfModulate()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_14, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_draw_behind_parent");

        [GodotMethod("set_draw_behind_parent")]
        public void SetDrawBehindParent(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_15, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_draw_behind_parent_enabled");

        [GodotMethod("is_draw_behind_parent_enabled")]
        public bool IsDrawBehindParentEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_on_top");

        internal void _SetOnTop(bool onTop)
        {
            NativeCalls.godot_icall_1_11(method_bind_17, Object.GetPtr(this), onTop);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_is_on_top");

        internal bool _IsOnTop()
        {
            return NativeCalls.godot_icall_0_3(method_bind_18, Object.GetPtr(this));
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_line");

        /// <summary>
        /// Draws a line from a 2D point to another, with a given color and width. It can be optionally antialiased.
        /// </summary>
        [GodotMethod("draw_line")]
        public void DrawLine(Vector2 from, Vector2 to, Color color, float width = 1f, bool antialiased = false)
        {
            NativeCalls.godot_icall_5_204(method_bind_19, Object.GetPtr(this), ref from, ref to, ref color, width, antialiased);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_polyline");

        /// <summary>
        /// Draws interconnected line segments with a uniform [code]color[/code] and [code]width[/code] and optional antialiasing.
        /// </summary>
        [GodotMethod("draw_polyline")]
        public void DrawPolyline(Vector2[] points, Color color, float width = 1f, bool antialiased = false)
        {
            NativeCalls.godot_icall_4_205(method_bind_20, Object.GetPtr(this), points, ref color, width, antialiased);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_polyline_colors");

        /// <summary>
        /// Draws interconnected line segments with a uniform [code]width[/code], segment-by-segment coloring, and optional antialiasing. Colors assigned to line segments match by index between [code]points[/code] and [code]colors[/code].
        /// </summary>
        [GodotMethod("draw_polyline_colors")]
        public void DrawPolylineColors(Vector2[] points, Color[] colors, float width = 1f, bool antialiased = false)
        {
            NativeCalls.godot_icall_4_206(method_bind_21, Object.GetPtr(this), points, colors, width, antialiased);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_multiline");

        /// <summary>
        /// Draws multiple, parallel lines with a uniform [code]color[/code] and [code]width[/code] and optional antialiasing.
        /// </summary>
        [GodotMethod("draw_multiline")]
        public void DrawMultiline(Vector2[] points, Color color, float width = 1f, bool antialiased = false)
        {
            NativeCalls.godot_icall_4_205(method_bind_22, Object.GetPtr(this), points, ref color, width, antialiased);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_multiline_colors");

        /// <summary>
        /// Draws multiple, parallel lines with a uniform [code]width[/code], segment-by-segment coloring, and optional antialiasing. Colors assigned to line segments match by index between [code]points[/code] and [code]colors[/code].
        /// </summary>
        [GodotMethod("draw_multiline_colors")]
        public void DrawMultilineColors(Vector2[] points, Color[] colors, float width = 1f, bool antialiased = false)
        {
            NativeCalls.godot_icall_4_206(method_bind_23, Object.GetPtr(this), points, colors, width, antialiased);
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_rect");

        /// <summary>
        /// Draws a colored rectangle.
        /// </summary>
        [GodotMethod("draw_rect")]
        public void DrawRect(Rect2 rect, Color color, bool filled = true)
        {
            NativeCalls.godot_icall_3_207(method_bind_24, Object.GetPtr(this), ref rect, ref color, filled);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_circle");

        /// <summary>
        /// Draws a colored circle.
        /// </summary>
        [GodotMethod("draw_circle")]
        public void DrawCircle(Vector2 position, float radius, Color color)
        {
            NativeCalls.godot_icall_3_208(method_bind_25, Object.GetPtr(this), ref position, radius, ref color);
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_texture");

        /// <summary>
        /// Draws a texture at a given position.
        /// &lt;param name="modulate"&gt;If the param is null, then the default value is new Color(1, 1, 1, 1)&lt;/param&gt;
        /// </summary>
        [GodotMethod("draw_texture")]
        public void DrawTexture(Texture texture, Vector2 position, Nullable<Color> modulate = null, Texture normalMap = null)
        {
            Color modulate_in = modulate.HasValue ? modulate.Value : new Color(1, 1, 1, 1);
            NativeCalls.godot_icall_4_209(method_bind_26, Object.GetPtr(this), Object.GetPtr(texture), ref position, ref modulate_in, Object.GetPtr(normalMap));
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_texture_rect");

        /// <summary>
        /// Draws a textured rectangle at a given position, optionally modulated by a color. Transpose swaps the x and y coordinates when reading the texture.
        /// &lt;param name="modulate"&gt;If the param is null, then the default value is new Color(1, 1, 1, 1)&lt;/param&gt;
        /// </summary>
        [GodotMethod("draw_texture_rect")]
        public void DrawTextureRect(Texture texture, Rect2 rect, bool tile, Nullable<Color> modulate = null, bool transpose = false, Texture normalMap = null)
        {
            Color modulate_in = modulate.HasValue ? modulate.Value : new Color(1, 1, 1, 1);
            NativeCalls.godot_icall_6_210(method_bind_27, Object.GetPtr(this), Object.GetPtr(texture), ref rect, tile, ref modulate_in, transpose, Object.GetPtr(normalMap));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_texture_rect_region");

        /// <summary>
        /// Draws a textured rectangle region at a given position, optionally modulated by a color. Transpose swaps the x and y coordinates when reading the texture.
        /// &lt;param name="modulate"&gt;If the param is null, then the default value is new Color(1, 1, 1, 1)&lt;/param&gt;
        /// </summary>
        [GodotMethod("draw_texture_rect_region")]
        public void DrawTextureRectRegion(Texture texture, Rect2 rect, Rect2 srcRect, Nullable<Color> modulate = null, bool transpose = false, Texture normalMap = null, bool clipUv = true)
        {
            Color modulate_in = modulate.HasValue ? modulate.Value : new Color(1, 1, 1, 1);
            NativeCalls.godot_icall_7_211(method_bind_28, Object.GetPtr(this), Object.GetPtr(texture), ref rect, ref srcRect, ref modulate_in, transpose, Object.GetPtr(normalMap), clipUv);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_style_box");

        /// <summary>
        /// Draws a styled rectangle.
        /// </summary>
        [GodotMethod("draw_style_box")]
        public void DrawStyleBox(StyleBox styleBox, Rect2 rect)
        {
            NativeCalls.godot_icall_2_212(method_bind_29, Object.GetPtr(this), Object.GetPtr(styleBox), ref rect);
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_primitive");

        /// <summary>
        /// Draws a custom primitive, 1 point for a point, 2 points for a line, 3 points for a triangle and 4 points for a quad.
        /// </summary>
        [GodotMethod("draw_primitive")]
        public void DrawPrimitive(Vector2[] points, Color[] colors, Vector2[] uvs, Texture texture = null, float width = 1f, Texture normalMap = null)
        {
            NativeCalls.godot_icall_6_213(method_bind_30, Object.GetPtr(this), points, colors, uvs, Object.GetPtr(texture), width, Object.GetPtr(normalMap));
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_polygon");

        /// <summary>
        /// Draws a polygon of any amount of points, convex or concave.
        /// &lt;param name="uvs"&gt;If the param is null, then the default value is new Vector2[] {}&lt;/param&gt;
        /// </summary>
        [GodotMethod("draw_polygon")]
        public void DrawPolygon(Vector2[] points, Color[] colors, Vector2[] uvs = null, Texture texture = null, Texture normalMap = null, bool antialiased = false)
        {
            Vector2[] uvs_in = uvs != null ? uvs : new Vector2[] {};
            NativeCalls.godot_icall_6_214(method_bind_31, Object.GetPtr(this), points, colors, uvs_in, Object.GetPtr(texture), Object.GetPtr(normalMap), antialiased);
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_colored_polygon");

        /// <summary>
        /// Draws a colored polygon of any amount of points, convex or concave.
        /// &lt;param name="uvs"&gt;If the param is null, then the default value is new Vector2[] {}&lt;/param&gt;
        /// </summary>
        [GodotMethod("draw_colored_polygon")]
        public void DrawColoredPolygon(Vector2[] points, Color color, Vector2[] uvs = null, Texture texture = null, Texture normalMap = null, bool antialiased = false)
        {
            Vector2[] uvs_in = uvs != null ? uvs : new Vector2[] {};
            NativeCalls.godot_icall_6_215(method_bind_32, Object.GetPtr(this), points, ref color, uvs_in, Object.GetPtr(texture), Object.GetPtr(normalMap), antialiased);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_string");

        /// <summary>
        /// Draws a string using a custom font.
        /// &lt;param name="modulate"&gt;If the param is null, then the default value is new Color(1, 1, 1, 1)&lt;/param&gt;
        /// </summary>
        [GodotMethod("draw_string")]
        public void DrawString(Font font, Vector2 position, string text, Nullable<Color> modulate = null, int clipW = -1)
        {
            Color modulate_in = modulate.HasValue ? modulate.Value : new Color(1, 1, 1, 1);
            NativeCalls.godot_icall_5_216(method_bind_33, Object.GetPtr(this), Object.GetPtr(font), ref position, text, ref modulate_in, clipW);
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_char");

        /// <summary>
        /// Draws a string character using a custom font. Returns the advance, depending on the char width and kerning with an optional next char.
        /// &lt;param name="modulate"&gt;If the param is null, then the default value is new Color(1, 1, 1, 1)&lt;/param&gt;
        /// </summary>
        [GodotMethod("draw_char")]
        public float DrawChar(Font font, Vector2 position, string @char, string next, Nullable<Color> modulate = null)
        {
            Color modulate_in = modulate.HasValue ? modulate.Value : new Color(1, 1, 1, 1);
            return NativeCalls.godot_icall_5_217(method_bind_34, Object.GetPtr(this), Object.GetPtr(font), ref position, @char, next, ref modulate_in);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_set_transform");

        /// <summary>
        /// Sets a custom transform for drawing via components. Anything drawn afterwards will be transformed by this.
        /// </summary>
        [GodotMethod("draw_set_transform")]
        public void DrawSetTransform(Vector2 position, float rotation, Vector2 scale)
        {
            NativeCalls.godot_icall_3_218(method_bind_35, Object.GetPtr(this), ref position, rotation, ref scale);
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "draw_set_transform_matrix");

        /// <summary>
        /// Sets a custom transform for drawing via matrix. Anything drawn afterwards will be transformed by this.
        /// </summary>
        [GodotMethod("draw_set_transform_matrix")]
        public void DrawSetTransformMatrix(Transform2D xform)
        {
            NativeCalls.godot_icall_1_219(method_bind_36, Object.GetPtr(this), ref xform);
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_transform");

        /// <summary>
        /// Get the transform matrix of this item.
        /// </summary>
        [GodotMethod("get_transform")]
        public Transform2D GetTransform()
        {
            object ret = NativeCalls.godot_icall_0_220(method_bind_37, Object.GetPtr(this));
            return (Transform2D)ret;
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_global_transform");

        /// <summary>
        /// Get the global transform matrix of this item.
        /// </summary>
        [GodotMethod("get_global_transform")]
        public Transform2D GetGlobalTransform()
        {
            object ret = NativeCalls.godot_icall_0_220(method_bind_38, Object.GetPtr(this));
            return (Transform2D)ret;
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_global_transform_with_canvas");

        /// <summary>
        /// Get the global transform matrix of this item in relation to the canvas.
        /// </summary>
        [GodotMethod("get_global_transform_with_canvas")]
        public Transform2D GetGlobalTransformWithCanvas()
        {
            object ret = NativeCalls.godot_icall_0_220(method_bind_39, Object.GetPtr(this));
            return (Transform2D)ret;
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_viewport_transform");

        /// <summary>
        /// Get this item's transform in relation to the viewport.
        /// </summary>
        [GodotMethod("get_viewport_transform")]
        public Transform2D GetViewportTransform()
        {
            object ret = NativeCalls.godot_icall_0_220(method_bind_40, Object.GetPtr(this));
            return (Transform2D)ret;
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_viewport_rect");

        /// <summary>
        /// Get the viewport's boundaries as a [Rect2].
        /// </summary>
        [GodotMethod("get_viewport_rect")]
        public Rect2 GetViewportRect()
        {
            object ret = NativeCalls.godot_icall_0_121(method_bind_41, Object.GetPtr(this));
            return (Rect2)ret;
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_canvas_transform");

        /// <summary>
        /// Get the transform matrix of this item's canvas.
        /// </summary>
        [GodotMethod("get_canvas_transform")]
        public Transform2D GetCanvasTransform()
        {
            object ret = NativeCalls.godot_icall_0_220(method_bind_42, Object.GetPtr(this));
            return (Transform2D)ret;
        }

        private IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_local_mouse_position");

        /// <summary>
        /// Get the mouse position relative to this item's position.
        /// </summary>
        [GodotMethod("get_local_mouse_position")]
        public Vector2 GetLocalMousePosition()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_43, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_44 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_global_mouse_position");

        /// <summary>
        /// Get the global position of the mouse.
        /// </summary>
        [GodotMethod("get_global_mouse_position")]
        public Vector2 GetGlobalMousePosition()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_44, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_45 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_canvas");

        /// <summary>
        /// Return the [RID] of the [World2D] canvas where this item is in.
        /// </summary>
        [GodotMethod("get_canvas")]
        public RID GetCanvas()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_45, Object.GetPtr(this));
            return new RID(ret);
        }

        private IntPtr method_bind_46 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_world_2d");

        /// <summary>
        /// Get the [World2D] where this item is in.
        /// </summary>
        [GodotMethod("get_world_2d")]
        public World2D GetWorld2d()
        {
            return NativeCalls.godot_icall_0_221(method_bind_46, Object.GetPtr(this));
        }

        private IntPtr method_bind_47 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_material");

        [GodotMethod("set_material")]
        public void SetMaterial(Material material)
        {
            NativeCalls.godot_icall_1_24(method_bind_47, Object.GetPtr(this), Object.GetPtr(material));
        }

        private IntPtr method_bind_48 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_material");

        [GodotMethod("get_material")]
        public Material GetMaterial()
        {
            return NativeCalls.godot_icall_0_222(method_bind_48, Object.GetPtr(this));
        }

        private IntPtr method_bind_49 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_use_parent_material");

        [GodotMethod("set_use_parent_material")]
        public void SetUseParentMaterial(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_49, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_50 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_use_parent_material");

        [GodotMethod("get_use_parent_material")]
        public bool GetUseParentMaterial()
        {
            return NativeCalls.godot_icall_0_3(method_bind_50, Object.GetPtr(this));
        }

        private IntPtr method_bind_51 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_notify_local_transform");

        /// <summary>
        /// If [code]enable[/code] is [code]true[/code], children will be updated with local transform data.
        /// </summary>
        [GodotMethod("set_notify_local_transform")]
        public void SetNotifyLocalTransform(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_51, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_52 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_local_transform_notification_enabled");

        /// <summary>
        /// Returns [code]true[/code] if local transform notifications are communicated to children.
        /// </summary>
        [GodotMethod("is_local_transform_notification_enabled")]
        public bool IsLocalTransformNotificationEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_52, Object.GetPtr(this));
        }

        private IntPtr method_bind_53 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_notify_transform");

        /// <summary>
        /// If [code]enable[/code] is [code]true[/code], children will be updated with global transform data.
        /// </summary>
        [GodotMethod("set_notify_transform")]
        public void SetNotifyTransform(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_53, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_54 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_transform_notification_enabled");

        /// <summary>
        /// Returns [code]true[/code] if global transform notifications are communicated to children.
        /// </summary>
        [GodotMethod("is_transform_notification_enabled")]
        public bool IsTransformNotificationEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_54, Object.GetPtr(this));
        }

        private IntPtr method_bind_55 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "make_canvas_position_local");

        /// <summary>
        /// Assigns [code]screen_point[/code] as this node's new local transform.
        /// </summary>
        [GodotMethod("make_canvas_position_local")]
        public Vector2 MakeCanvasPositionLocal(Vector2 screenPoint)
        {
            object ret = NativeCalls.godot_icall_1_223(method_bind_55, Object.GetPtr(this), ref screenPoint);
            return (Vector2)ret;
        }

        private IntPtr method_bind_56 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "make_input_local");

        /// <summary>
        /// Transformations issued by [code]event[/code]'s inputs are applied in local space instead of global space.
        /// </summary>
        [GodotMethod("make_input_local")]
        public InputEvent MakeInputLocal(InputEvent @event)
        {
            return NativeCalls.godot_icall_1_224(method_bind_56, Object.GetPtr(this), Object.GetPtr(@event));
        }
    }
}
