using System;

namespace Godot
{
    /// <summary>
    /// Canvas drawing layer. [CanvasItem] nodes that are direct or indirect children of a [code]CanvasLayer[/code] will be drawn in that layer. The layer is a numeric index that defines the draw order. The default 2D scene renders with index 0, so a [code]CanvasLayer[/code] with index -1 will be drawn below, and one with index 1 will be drawn above. This is very useful for HUDs (in layer 1+ or above), or backgrounds (in layer -1 or below).
    /// </summary>
    public class CanvasLayer : Node
    {
        /// <summary>
        /// Layer index for draw order. Lower values are drawn first. Default value: [code]1[/code].
        /// </summary>
        public int Layer
        {
            get
            {
                return GetLayer();
            }
            set
            {
                SetLayer(value);
            }
        }

        /// <summary>
        /// The layer's base offset.
        /// </summary>
        public Vector2 Offset
        {
            get
            {
                return GetOffset();
            }
            set
            {
                SetOffset(value);
            }
        }

        /// <summary>
        /// The layer's rotation in degrees.
        /// </summary>
        public float RotationDegrees
        {
            get
            {
                return GetRotationDegrees();
            }
            set
            {
                SetRotationDegrees(value);
            }
        }

        /// <summary>
        /// The layer's rotation in radians.
        /// </summary>
        public float Rotation
        {
            get
            {
                return GetRotation();
            }
            set
            {
                SetRotation(value);
            }
        }

        /// <summary>
        /// The layer's scale.
        /// </summary>
        public Vector2 Scale
        {
            get
            {
                return GetScale();
            }
            set
            {
                SetScale(value);
            }
        }

        /// <summary>
        /// The layer's transform.
        /// </summary>
        public Transform2D Transform
        {
            get
            {
                return GetTransform();
            }
            set
            {
                SetTransform(value);
            }
        }

        /// <summary>
        /// The custom [Viewport] node assigned to the [code]CanvasLayer[/code]. If null, uses the default viewport instead.
        /// </summary>
        public Node CustomViewport
        {
            get
            {
                return GetCustomViewport();
            }
            set
            {
                SetCustomViewport(value);
            }
        }


        private const string nativeName = "CanvasLayer";

        public CanvasLayer() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_CanvasLayer_Ctor(this);
        }

        internal CanvasLayer(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_layer");

        [GodotMethod("set_layer")]
        public void SetLayer(int layer)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, Object.GetPtr(this), layer);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_layer");

        [GodotMethod("get_layer")]
        public int GetLayer()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_transform");

        [GodotMethod("set_transform")]
        public void SetTransform(Transform2D transform)
        {
            NativeCalls.godot_icall_1_219(method_bind_2, Object.GetPtr(this), ref transform);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_transform");

        [GodotMethod("get_transform")]
        public Transform2D GetTransform()
        {
            object ret = NativeCalls.godot_icall_0_220(method_bind_3, Object.GetPtr(this));
            return (Transform2D)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_offset");

        [GodotMethod("set_offset")]
        public void SetOffset(Vector2 offset)
        {
            NativeCalls.godot_icall_1_45(method_bind_4, Object.GetPtr(this), ref offset);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_offset");

        [GodotMethod("get_offset")]
        public Vector2 GetOffset()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_5, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rotation");

        [GodotMethod("set_rotation")]
        public void SetRotation(float radians)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), radians);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rotation");

        [GodotMethod("get_rotation")]
        public float GetRotation()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rotation_degrees");

        [GodotMethod("set_rotation_degrees")]
        public void SetRotationDegrees(float degrees)
        {
            NativeCalls.godot_icall_1_10(method_bind_8, Object.GetPtr(this), degrees);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rotation_degrees");

        [GodotMethod("get_rotation_degrees")]
        public float GetRotationDegrees()
        {
            return NativeCalls.godot_icall_0_9(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_scale");

        [GodotMethod("set_scale")]
        public void SetScale(Vector2 scale)
        {
            NativeCalls.godot_icall_1_45(method_bind_10, Object.GetPtr(this), ref scale);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_scale");

        [GodotMethod("get_scale")]
        public Vector2 GetScale()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_11, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_custom_viewport");

        [GodotMethod("set_custom_viewport")]
        public void SetCustomViewport(Node viewport)
        {
            NativeCalls.godot_icall_1_42(method_bind_12, Object.GetPtr(this), Object.GetPtr(viewport));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_custom_viewport");

        [GodotMethod("get_custom_viewport")]
        public Node GetCustomViewport()
        {
            return NativeCalls.godot_icall_0_200(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_world_2d");

        /// <summary>
        /// Return the [World2D] used by this layer.
        /// </summary>
        [GodotMethod("get_world_2d")]
        public World2D GetWorld2d()
        {
            return NativeCalls.godot_icall_0_221(method_bind_14, Object.GetPtr(this));
        }
    }
}
