using System;

namespace Godot
{
    /// <summary>
    /// CollisionObject2D is the base class for 2D physics objects. It can hold any number of 2D collision [Shape2D]s. Each shape must be assigned to a [i]shape owner[/i]. The CollisionObject2D can have any number of shape owners. Shape owners are not nodes and do not appear in the editor, but are accessible through code using the [code]shape_owner_*[/code] methods.
    /// </summary>
    public class CollisionObject2D : Node2D
    {
        /// <summary>
        /// If [code]true[/code] this object is pickable. A pickable object can detect the mouse pointer entering/leaving, and if the mouse is inside it, report input events.
        /// </summary>
        public bool InputPickable
        {
            get
            {
                return IsPickable();
            }
            set
            {
                SetPickable(value);
            }
        }


        private const string nativeName = "CollisionObject2D";

        internal CollisionObject2D() {}

        internal CollisionObject2D(bool memoryOwn) : base(memoryOwn) {}

        /// <summary>
        /// Accepts unhandled [InputEvent]s. [code]shape_idx[/code] is the child index of the clicked [Shape2D]. Connect to the [code]input_event[/code] signal to easily pick up these events.
        /// </summary>
        [GodotMethod("_input_event")]
        public virtual void _InputEvent(Object viewport, InputEvent @event, int shapeIdx)
        {
            return;
        }

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rid");

        /// <summary>
        /// Returns the object's [RID].
        /// </summary>
        [GodotMethod("get_rid")]
        public RID GetRid()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_0, Object.GetPtr(this));
            return new RID(ret);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_pickable");

        [GodotMethod("set_pickable")]
        public void SetPickable(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_1, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_pickable");

        [GodotMethod("is_pickable")]
        public bool IsPickable()
        {
            return NativeCalls.godot_icall_0_3(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create_shape_owner");

        /// <summary>
        /// Creates a new shape owner for the given object. Returns [code]owner_id[/code] of the new owner for future reference.
        /// </summary>
        [GodotMethod("create_shape_owner")]
        public int CreateShapeOwner(Object owner)
        {
            return NativeCalls.godot_icall_1_229(method_bind_3, Object.GetPtr(this), Object.GetPtr(owner));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_shape_owner");

        /// <summary>
        /// Removes the given shape owner.
        /// </summary>
        [GodotMethod("remove_shape_owner")]
        public void RemoveShapeOwner(int ownerId)
        {
            NativeCalls.godot_icall_1_0(method_bind_4, Object.GetPtr(this), ownerId);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shape_owners");

        /// <summary>
        /// Returns an [Array] of [code]owner_id[/code] identifiers. You can use these ids in other methods that take [code]owner_id[/code] as an argument.
        /// </summary>
        [GodotMethod("get_shape_owners")]
        public object[] GetShapeOwners()
        {
            return NativeCalls.godot_icall_0_21(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shape_owner_set_transform");

        /// <summary>
        /// Sets the [Transform2D] of the given shape owner.
        /// </summary>
        [GodotMethod("shape_owner_set_transform")]
        public void ShapeOwnerSetTransform(int ownerId, Transform2D transform)
        {
            NativeCalls.godot_icall_2_234(method_bind_6, Object.GetPtr(this), ownerId, ref transform);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shape_owner_get_transform");

        /// <summary>
        /// Returns the shape owner's [Transform2D].
        /// </summary>
        [GodotMethod("shape_owner_get_transform")]
        public Transform2D ShapeOwnerGetTransform(int ownerId)
        {
            object ret = NativeCalls.godot_icall_1_235(method_bind_7, Object.GetPtr(this), ownerId);
            return (Transform2D)ret;
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shape_owner_get_owner");

        /// <summary>
        /// Returns the parent object of the given shape owner.
        /// </summary>
        [GodotMethod("shape_owner_get_owner")]
        public Object ShapeOwnerGetOwner(int ownerId)
        {
            return NativeCalls.godot_icall_1_232(method_bind_8, Object.GetPtr(this), ownerId);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shape_owner_set_disabled");

        /// <summary>
        /// If [code]true[/code] disables the given shape owner.
        /// </summary>
        [GodotMethod("shape_owner_set_disabled")]
        public void ShapeOwnerSetDisabled(int ownerId, bool disabled)
        {
            NativeCalls.godot_icall_2_51(method_bind_9, Object.GetPtr(this), ownerId, disabled);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_shape_owner_disabled");

        /// <summary>
        /// If [code]true[/code] the shape owner and its shapes are disabled.
        /// </summary>
        [GodotMethod("is_shape_owner_disabled")]
        public bool IsShapeOwnerDisabled(int ownerId)
        {
            return NativeCalls.godot_icall_1_29(method_bind_10, Object.GetPtr(this), ownerId);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shape_owner_set_one_way_collision");

        /// <summary>
        /// If [code]enable[/code] is [code]true[/code], collisions for the shape owner originating from this [code]CollisionObject2D[/code] will not be reported to collided with [code]CollisionObject2D[/code]s.
        /// </summary>
        [GodotMethod("shape_owner_set_one_way_collision")]
        public void ShapeOwnerSetOneWayCollision(int ownerId, bool enable)
        {
            NativeCalls.godot_icall_2_51(method_bind_11, Object.GetPtr(this), ownerId, enable);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_shape_owner_one_way_collision_enabled");

        /// <summary>
        /// Returns [code]true[/code] if collisions for the shape owner originating from this [code]CollisionObject2D[/code] will not be reported to collided with [code]CollisionObject2D[/code]s.
        /// </summary>
        [GodotMethod("is_shape_owner_one_way_collision_enabled")]
        public bool IsShapeOwnerOneWayCollisionEnabled(int ownerId)
        {
            return NativeCalls.godot_icall_1_29(method_bind_12, Object.GetPtr(this), ownerId);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shape_owner_add_shape");

        /// <summary>
        /// Adds a [Shape2D] to the shape owner.
        /// </summary>
        [GodotMethod("shape_owner_add_shape")]
        public void ShapeOwnerAddShape(int ownerId, Shape2D shape)
        {
            NativeCalls.godot_icall_2_67(method_bind_13, Object.GetPtr(this), ownerId, Object.GetPtr(shape));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shape_owner_get_shape_count");

        /// <summary>
        /// Returns the number of shapes the given shape owner contains.
        /// </summary>
        [GodotMethod("shape_owner_get_shape_count")]
        public int ShapeOwnerGetShapeCount(int ownerId)
        {
            return NativeCalls.godot_icall_1_6(method_bind_14, Object.GetPtr(this), ownerId);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shape_owner_get_shape");

        /// <summary>
        /// Returns the [Shape2D] with the given id from the given shape owner.
        /// </summary>
        [GodotMethod("shape_owner_get_shape")]
        public Shape2D ShapeOwnerGetShape(int ownerId, int shapeId)
        {
            return NativeCalls.godot_icall_2_236(method_bind_15, Object.GetPtr(this), ownerId, shapeId);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shape_owner_get_shape_index");

        /// <summary>
        /// Returns the child index of the [Shape2D] with the given id from the given shape owner.
        /// </summary>
        [GodotMethod("shape_owner_get_shape_index")]
        public int ShapeOwnerGetShapeIndex(int ownerId, int shapeId)
        {
            return NativeCalls.godot_icall_2_171(method_bind_16, Object.GetPtr(this), ownerId, shapeId);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shape_owner_remove_shape");

        /// <summary>
        /// Removes a shape from the given shape owner.
        /// </summary>
        [GodotMethod("shape_owner_remove_shape")]
        public void ShapeOwnerRemoveShape(int ownerId, int shapeId)
        {
            NativeCalls.godot_icall_2_32(method_bind_17, Object.GetPtr(this), ownerId, shapeId);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shape_owner_clear_shapes");

        /// <summary>
        /// Removes all shapes from the shape owner.
        /// </summary>
        [GodotMethod("shape_owner_clear_shapes")]
        public void ShapeOwnerClearShapes(int ownerId)
        {
            NativeCalls.godot_icall_1_0(method_bind_18, Object.GetPtr(this), ownerId);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shape_find_owner");

        /// <summary>
        /// Returns the [code]owner_id[/code] of the given shape.
        /// </summary>
        [GodotMethod("shape_find_owner")]
        public int ShapeFindOwner(int shapeIndex)
        {
            return NativeCalls.godot_icall_1_6(method_bind_19, Object.GetPtr(this), shapeIndex);
        }
    }
}
