using System;

namespace Godot
{
    /// <summary>
    /// Allows editing a collision polygon's vertices on a selected plane. Can also set a depth perpendicular to that plane. This class is only available in the editor. It will not appear in the scene tree at runtime. Creates a [Shape] for gameplay. Properties modified during gameplay will have no effect.
    /// </summary>
    public class CollisionPolygon : Spatial
    {
        /// <summary>
        /// Length that the resulting collision extends in either direction perpendicular to its polygon.
        /// </summary>
        public float Depth
        {
            get
            {
                return GetDepth();
            }
            set
            {
                SetDepth(value);
            }
        }

        /// <summary>
        /// If true, no collision will be produced.
        /// </summary>
        public bool Disabled
        {
            get
            {
                return IsDisabled();
            }
            set
            {
                SetDisabled(value);
            }
        }

        /// <summary>
        /// Array of vertices which define the polygon.
        /// </summary>
        public Vector2[] Polygon
        {
            get
            {
                return GetPolygon();
            }
            set
            {
                SetPolygon(value);
            }
        }


        private const string nativeName = "CollisionPolygon";

        public CollisionPolygon() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_CollisionPolygon_Ctor(this);
        }

        internal CollisionPolygon(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_depth");

        [GodotMethod("set_depth")]
        public void SetDepth(float depth)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), depth);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_depth");

        [GodotMethod("get_depth")]
        public float GetDepth()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_polygon");

        [GodotMethod("set_polygon")]
        public void SetPolygon(Vector2[] polygon)
        {
            NativeCalls.godot_icall_1_237(method_bind_2, Object.GetPtr(this), polygon);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_polygon");

        [GodotMethod("get_polygon")]
        public Vector2[] GetPolygon()
        {
            return NativeCalls.godot_icall_0_238(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_disabled");

        [GodotMethod("set_disabled")]
        public void SetDisabled(bool disabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_4, Object.GetPtr(this), disabled);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_disabled");

        [GodotMethod("is_disabled")]
        public bool IsDisabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }
    }
}
