using System;

namespace Godot
{
    /// <summary>
    /// Editor facility for creating and editing collision shapes in 3D space. You can use this node to represent all sorts of collision shapes, for example, add this to an [Area] to give it a detection shape, or add it to a [PhysicsBody] to give create solid object. [b]IMPORTANT[/b]: this is an Editor-only helper to create shapes, use [method get_shape] to get the actual shape.
    /// </summary>
    public class CollisionShape : Spatial
    {
        /// <summary>
        /// The actual shape owned by this collision shape.
        /// </summary>
        public Shape Shape
        {
            get
            {
                return GetShape();
            }
            set
            {
                SetShape(value);
            }
        }

        /// <summary>
        /// A disabled collision shape has no effect in the world.
        /// </summary>
        public bool Disabled
        {
            get
            {
                return IsDisabled();
            }
            set
            {
                SetDisabled(value);
            }
        }


        private const string nativeName = "CollisionShape";

        public CollisionShape() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_CollisionShape_Ctor(this);
        }

        internal CollisionShape(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "resource_changed");

        /// <summary>
        /// If this method exists within a script it will be called whenever the shape resource has been modified.
        /// </summary>
        [GodotMethod("resource_changed")]
        public void ResourceChanged(Resource resource)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(resource));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shape");

        [GodotMethod("set_shape")]
        public void SetShape(Shape shape)
        {
            NativeCalls.godot_icall_1_24(method_bind_1, Object.GetPtr(this), Object.GetPtr(shape));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shape");

        [GodotMethod("get_shape")]
        public Shape GetShape()
        {
            return NativeCalls.godot_icall_0_241(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_disabled");

        [GodotMethod("set_disabled")]
        public void SetDisabled(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_3, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_disabled");

        [GodotMethod("is_disabled")]
        public bool IsDisabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "make_convex_from_brothers");

        /// <summary>
        /// Sets the collision shape's shape to the addition of all its convexed [MeshInstance] siblings geometry.
        /// </summary>
        [GodotMethod("make_convex_from_brothers")]
        public void MakeConvexFromBrothers()
        {
            NativeCalls.godot_icall_0_12(method_bind_5, Object.GetPtr(this));
        }
    }
}
