using System;

namespace Godot
{
    /// <summary>
    /// Editor facility for creating and editing collision shapes in 2D space. You can use this node to represent all sorts of collision shapes, for example, add this to an [Area2D] to give it a detection shape, or add it to a [PhysicsBody2D] to give create solid object. [b]IMPORTANT[/b]: this is an Editor-only helper to create shapes, use [method get_shape] to get the actual shape.
    /// </summary>
    public class CollisionShape2D : Node2D
    {
        /// <summary>
        /// The actual shape owned by this collision shape.
        /// </summary>
        public Shape2D Shape
        {
            get
            {
                return GetShape();
            }
            set
            {
                SetShape(value);
            }
        }

        /// <summary>
        /// A disabled collision shape has no effect in the world.
        /// </summary>
        public bool Disabled
        {
            get
            {
                return IsDisabled();
            }
            set
            {
                SetDisabled(value);
            }
        }

        /// <summary>
        /// Sets whether this collision shape should only detect collision on one side (top or bottom).
        /// </summary>
        public bool OneWayCollision
        {
            get
            {
                return IsOneWayCollisionEnabled();
            }
            set
            {
                SetOneWayCollision(value);
            }
        }


        private const string nativeName = "CollisionShape2D";

        public CollisionShape2D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_CollisionShape2D_Ctor(this);
        }

        internal CollisionShape2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shape");

        [GodotMethod("set_shape")]
        public void SetShape(Shape2D shape)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(shape));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shape");

        [GodotMethod("get_shape")]
        public Shape2D GetShape()
        {
            return NativeCalls.godot_icall_0_242(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_disabled");

        [GodotMethod("set_disabled")]
        public void SetDisabled(bool disabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_2, Object.GetPtr(this), disabled);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_disabled");

        [GodotMethod("is_disabled")]
        public bool IsDisabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_one_way_collision");

        [GodotMethod("set_one_way_collision")]
        public void SetOneWayCollision(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_4, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_one_way_collision_enabled");

        [GodotMethod("is_one_way_collision_enabled")]
        public bool IsOneWayCollisionEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }
    }
}
