using System;

namespace Godot
{
    /// <summary>
    /// The joint can rotate the bodies across an axis defined by the local x-axes of the [Joint].
    /// The twist axis is initiated as the x-axis of the [Joint].
    /// Once the Bodies swing, the twist axis is calculated as the middle of the x-axes of the Joint in the local space of the two Bodies.
    /// </summary>
    public class ConeTwistJoint : Joint
    {
        public enum Param
        {
            SwingSpan = 0,
            TwistSpan = 1,
            Bias = 2,
            Softness = 3,
            Relaxation = 4,
            Max = 5
        }


        /// <summary>
        /// Swing is rotation from side to side, around the axis perpendicular to the twist axis.
        /// The swing span defines, how much rotation will not get corrected allong the swing axis.
        /// Could be defined as looseness in the [code]ConeTwistJoint[/code].
        /// If below 0.05, this behaviour is locked. Default value: [code]PI/4[/code].
        /// </summary>
        public float SwingSpan
        {
            get
            {
                return _GetSwingSpan();
            }
            set
            {
                _SetSwingSpan(value);
            }
        }

        /// <summary>
        /// Twist is the rotation around the twist axis, this value defined how far the joint can twist.
        /// Twist is locked if below 0.05.
        /// </summary>
        public float TwistSpan
        {
            get
            {
                return _GetTwistSpan();
            }
            set
            {
                _SetTwistSpan(value);
            }
        }

        /// <summary>
        /// The speed with which the swing or twist will take place.
        /// The higher, the faster.
        /// </summary>
        public float Bias
        {
            get
            {
                return GetParam((ConeTwistJoint.Param)2);
            }
            set
            {
                SetParam((ConeTwistJoint.Param)2, value);
            }
        }

        /// <summary>
        /// The ease with which the joint starts to twist. If it's too low, it takes more force to start twisting the joint.
        /// </summary>
        public float Softness
        {
            get
            {
                return GetParam((ConeTwistJoint.Param)3);
            }
            set
            {
                SetParam((ConeTwistJoint.Param)3, value);
            }
        }

        /// <summary>
        /// Defines, how fast the swing- and twist-speed-difference on both sides gets synced.
        /// </summary>
        public float Relaxation
        {
            get
            {
                return GetParam((ConeTwistJoint.Param)4);
            }
            set
            {
                SetParam((ConeTwistJoint.Param)4, value);
            }
        }


        private const string nativeName = "ConeTwistJoint";

        public ConeTwistJoint() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ConeTwistJoint_Ctor(this);
        }

        internal ConeTwistJoint(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_param");

        [GodotMethod("set_param")]
        public void SetParam(ConeTwistJoint.Param param, float value)
        {
            NativeCalls.godot_icall_2_247(method_bind_0, Object.GetPtr(this), param, value);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_param");

        [GodotMethod("get_param")]
        public float GetParam(ConeTwistJoint.Param param)
        {
            return NativeCalls.godot_icall_1_248(method_bind_1, Object.GetPtr(this), param);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_swing_span");

        internal void _SetSwingSpan(float swingSpan)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), swingSpan);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_swing_span");

        internal float _GetSwingSpan()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_twist_span");

        internal void _SetTwistSpan(float twistSpan)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), twistSpan);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_twist_span");

        internal float _GetTwistSpan()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }
    }
}
