using System;

namespace Godot
{
    /// <summary>
    /// This helper class can be used to store [Variant] values on the filesystem using INI-style formatting. The stored values are indentified by a section and a key:
    /// [codeblock]
    /// [section]
    /// some_key=42
    /// string_example="Hello World!"
    /// a_vector=Vector3( 1, 0, 2 )
    /// [/codeblock]
    /// The stored data can be saved to or parsed from a file, though ConfigFile objects can also be used directly without accessing the filesystem.
    /// The following example shows how to parse an INI-style file from the system, read its contents and store new values in it:
    /// [codeblock]
    /// var config = ConfigFile.new()
    /// var err = config.load("user://settings.cfg")
    /// if err == OK: # if not, something went wrong with the file loading
    /// # Look for the display/width pair, and default to 1024 if missing
    /// var screen_width = get_value("display", "width", 1024)
    /// # Store a variable if and only if it hasn't been defined yet
    /// if not config.has_section_key("audio", "mute"):
    /// config.set_value("audio", "mute", false)
    /// # Save the changes by overwriting the previous file
    /// config.save("user://settings.cfg")
    /// [/codeblock]
    /// </summary>
    public class ConfigFile : Reference
    {
        private const string nativeName = "ConfigFile";

        public ConfigFile() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ConfigFile_Ctor(this);
        }

        internal ConfigFile(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_value");

        /// <summary>
        /// Assigns a value to the specified key of the the specified section. If the section and/or the key do not exist, they are created. Passing a [code]null[/code] value deletes the specified key if it exists, and deletes the section if it ends up empty once the key has been removed.
        /// </summary>
        [GodotMethod("set_value")]
        public void SetValue(string section, string key, object value)
        {
            NativeCalls.godot_icall_3_249(method_bind_0, Object.GetPtr(this), section, key, value);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_value");

        /// <summary>
        /// Returns the current value for the specified section and key. If the section and/or the key do not exist, the method returns the value of the optional [code]default[/code] argument, or [code]null[/code] if it is omitted.
        /// </summary>
        [GodotMethod("get_value")]
        public object GetValue(string section, string key, object @default = null)
        {
            return NativeCalls.godot_icall_3_250(method_bind_1, Object.GetPtr(this), section, key, @default);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_section");

        /// <summary>
        /// Returns [code]true[/code] if the specified section exists.
        /// </summary>
        [GodotMethod("has_section")]
        public bool HasSection(string section)
        {
            return NativeCalls.godot_icall_1_70(method_bind_2, Object.GetPtr(this), section);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_section_key");

        /// <summary>
        /// Returns [code]true[/code] if the specified section-key pair exists.
        /// </summary>
        [GodotMethod("has_section_key")]
        public bool HasSectionKey(string section, string key)
        {
            return NativeCalls.godot_icall_2_251(method_bind_3, Object.GetPtr(this), section, key);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_sections");

        /// <summary>
        /// Returns an array of all defined section identifiers.
        /// </summary>
        [GodotMethod("get_sections")]
        public string[] GetSections()
        {
            return NativeCalls.godot_icall_0_72(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_section_keys");

        /// <summary>
        /// Returns an array of all defined key identifiers in the specified section.
        /// </summary>
        [GodotMethod("get_section_keys")]
        public string[] GetSectionKeys(string section)
        {
            return NativeCalls.godot_icall_1_252(method_bind_5, Object.GetPtr(this), section);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "erase_section");

        /// <summary>
        /// Deletes the specified section along with all the key-value pairs inside.
        /// </summary>
        [GodotMethod("erase_section")]
        public void EraseSection(string section)
        {
            NativeCalls.godot_icall_1_43(method_bind_6, Object.GetPtr(this), section);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "load");

        /// <summary>
        /// Loads the config file specified as a parameter. The file's contents are parsed and loaded in the ConfigFile object which the method was called on. Returns one of the [code]OK[/code], [code]FAILED[/code] or [code]ERR_*[/code] constants listed in [@GlobalScope]. If the load was successful, the return value is [code]OK[/code].
        /// </summary>
        [GodotMethod("load")]
        public Error Load(string path)
        {
            return NativeCalls.godot_icall_1_170(method_bind_7, Object.GetPtr(this), path);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "save");

        /// <summary>
        /// Saves the contents of the ConfigFile object to the file specified as a parameter. The output file uses an INI-style structure. Returns one of the [code]OK[/code], [code]FAILED[/code] or [code]ERR_*[/code] constants listed in [@GlobalScope]. If the load was successful, the return value is [code]OK[/code].
        /// </summary>
        [GodotMethod("save")]
        public Error Save(string path)
        {
            return NativeCalls.godot_icall_1_170(method_bind_8, Object.GetPtr(this), path);
        }
    }
}
