using System;

namespace Godot
{
    /// <summary>
    /// Base class for all User Interface or [i]UI[/i] related nodes. [code]Control[/code] features a bounding rectangle that defines its extents, an anchor position relative to its parent and margins that represent an offset to the anchor. The margins update automatically when the node, any of its parents, or the screen size change.
    /// For more information on Godot's UI system, anchors, margins, and containers, see the related tutorials in the manual. To build flexible UIs, you'll need a mix of UI elements that inherit from [code]Control[/code] and [Container] nodes.
    /// [b]User Interface nodes and input[/b]
    /// Godot sends input events to the scene's root node first, by calling [method Node._input]. [method Node._input] forwards the event down the node tree to the nodes under the mouse cursor, or on keyboard focus. To do so, it calls [method MainLoop._input_event]. Call [method accept_event] so no other node receives the event. Once you accepted an input, it becomes handled so [method Node._unhandled_input] will not process it.
    /// Only one [code]Control[/code] node can be in keyboard focus. Only the node in focus will receive keyboard events. To get the foucs, call [method set_focus_mode]. [code]Control[/code] nodes lose focus when another node grabs it, or if you hide the node in focus.
    /// Call [method set_ignore_mouse] to tell a [code]Control[/code] node to ignore mouse or touch events. You'll need it if you place an icon on top of a button.
    /// [Theme] resources change the Control's appearance. If you change the [Theme] on a [code]Control[/code] node, it affects all of its children. To override some of the theme's parameters, call one of the [code]add_*_override[/code] methods, like [method add_font_override]. You can override the theme with the inspector.
    /// </summary>
    public class Control : CanvasItem
    {
        public const int NotificationResized = 40;
        public const int NotificationMouseEnter = 41;
        public const int NotificationMouseExit = 42;
        public const int NotificationFocusEnter = 43;
        public const int NotificationFocusExit = 44;
        public const int NotificationThemeChanged = 45;
        public const int NotificationModalClose = 46;

        public enum Anchor
        {
            Begin = 0,
            End = 1
        }

        public enum FocusModeEnum
        {
            None = 0,
            Click = 1,
            All = 2
        }

        public enum LayoutPresetMode
        {
            Minsize = 0,
            KeepWidth = 1,
            KeepHeight = 2,
            KeepSize = 3
        }

        public enum MouseFilterEnum
        {
            Stop = 0,
            Pass = 1,
            Ignore = 2
        }

        public enum CursorShape
        {
            Arrow = 0,
            Ibeam = 1,
            PointingHand = 2,
            Cross = 3,
            Wait = 4,
            Busy = 5,
            Drag = 6,
            CanDrop = 7,
            Forbidden = 8,
            Vsize = 9,
            Hsize = 10,
            Bdiagsize = 11,
            Fdiagsize = 12,
            Move = 13,
            Vsplit = 14,
            Hsplit = 15,
            Help = 16
        }

        public enum GrowDirection
        {
            Begin = 0,
            End = 1
        }

        public enum SizeFlags
        {
            Fill = 1,
            Expand = 2,
            ExpandFill = 3,
            ShrinkCenter = 4,
            ShrinkEnd = 8
        }

        public enum LayoutPreset
        {
            TopLeft = 0,
            TopRight = 1,
            BottomLeft = 2,
            BottomRight = 3,
            CenterLeft = 4,
            CenterTop = 5,
            CenterRight = 6,
            CenterBottom = 7,
            Center = 8,
            LeftWide = 9,
            TopWide = 10,
            RightWide = 11,
            BottomWide = 12,
            VcenterWide = 13,
            HcenterWide = 14,
            Wide = 15
        }


        /// <summary>
        /// Anchors the left edge of the node to the origin, the center or the end of its parent container. It changes how the left margin updates when the node moves or changes size. Use one of the [code]ANCHOR_*[/code] constants. Default value: [code]ANCHOR_BEGIN[/code].
        /// </summary>
        public float AnchorLeft
        {
            get
            {
                return GetAnchor((Margin)0);
            }
            set
            {
                _SetAnchor((Margin)0, value);
            }
        }

        /// <summary>
        /// Anchors the top edge of the node to the origin, the center or the end of its parent container. It changes how the top margin updates when the node moves or changes size. Use one of the [code]ANCHOR_*[/code] constants. Default value: [code]ANCHOR_BEGIN[/code].
        /// </summary>
        public float AnchorTop
        {
            get
            {
                return GetAnchor((Margin)1);
            }
            set
            {
                _SetAnchor((Margin)1, value);
            }
        }

        /// <summary>
        /// Anchors the right edge of the node to the origin, the center or the end of its parent container. It changes how the right margin updates when the node moves or changes size. Use one of the [code]ANCHOR_*[/code] constants. Default value: [code]ANCHOR_BEGIN[/code].
        /// </summary>
        public float AnchorRight
        {
            get
            {
                return GetAnchor((Margin)2);
            }
            set
            {
                _SetAnchor((Margin)2, value);
            }
        }

        /// <summary>
        /// Anchors the bottom edge of the node to the origin, the center, or the end of its parent container. It changes how the bottom margin updates when the node moves or changes size. Use one of the [code]ANCHOR_*[/code] constants. Default value: [code]ANCHOR_BEGIN[/code].
        /// </summary>
        public float AnchorBottom
        {
            get
            {
                return GetAnchor((Margin)3);
            }
            set
            {
                _SetAnchor((Margin)3, value);
            }
        }

        /// <summary>
        /// Distance between the node's left edge and its parent container, based on [member anchor_left].
        /// </summary>
        public float MarginLeft
        {
            get
            {
                return GetMargin((Margin)0);
            }
            set
            {
                SetMargin((Margin)0, value);
            }
        }

        /// <summary>
        /// Distance between the node's top edge and its parent container, based on [member anchor_top].
        /// </summary>
        public float MarginTop
        {
            get
            {
                return GetMargin((Margin)1);
            }
            set
            {
                SetMargin((Margin)1, value);
            }
        }

        /// <summary>
        /// Distance between the node's right edge and its parent container, based on [member anchor_right].
        /// </summary>
        public float MarginRight
        {
            get
            {
                return GetMargin((Margin)2);
            }
            set
            {
                SetMargin((Margin)2, value);
            }
        }

        /// <summary>
        /// Distance between the node's bottom edge and its parent container, based on [member anchor_bottom].
        /// Margins are often controlled by one or multiple parent [Container] nodes. Margins update automatically when you move or resize the node.
        /// </summary>
        public float MarginBottom
        {
            get
            {
                return GetMargin((Margin)3);
            }
            set
            {
                SetMargin((Margin)3, value);
            }
        }

        /// <summary>
        /// </summary>
        public Control.GrowDirection GrowHorizontal
        {
            get
            {
                return GetHGrowDirection();
            }
            set
            {
                SetHGrowDirection(value);
            }
        }

        /// <summary>
        /// </summary>
        public Control.GrowDirection GrowVertical
        {
            get
            {
                return GetVGrowDirection();
            }
            set
            {
                SetVGrowDirection(value);
            }
        }

        /// <summary>
        /// The node's position, relative to its parent. It corresponds to the rectangle's top-left corner. The property is not affected by [member rect_pivot_offset].
        /// </summary>
        public Vector2 RectPosition
        {
            get
            {
                return GetPosition();
            }
            set
            {
                SetPosition(value);
            }
        }

        /// <summary>
        /// The node's global position, relative to the world (usually to the top-left corner of the window).
        /// </summary>
        public Vector2 RectGlobalPosition
        {
            get
            {
                return GetGlobalPosition();
            }
            set
            {
                SetGlobalPosition(value);
            }
        }

        /// <summary>
        /// The size of the node's bounding rectangle, in pixels. [Container] nodes update this property automatically.
        /// </summary>
        public Vector2 RectSize
        {
            get
            {
                return GetSize();
            }
            set
            {
                SetSize(value);
            }
        }

        /// <summary>
        /// The minimum size of the node's bounding rectangle. If you set it to a value greater than (0, 0), the node's bounding rectangle will always have at least this size, even if its content is smaller. If it's set to (0, 0), the node sizes automatically to fit its content, be it a texture or child nodes.
        /// </summary>
        public Vector2 RectMinSize
        {
            get
            {
                return GetCustomMinimumSize();
            }
            set
            {
                SetCustomMinimumSize(value);
            }
        }

        /// <summary>
        /// The node's rotation around its pivot, in degrees. See [member rect_pivot_offset] to change the pivot's position.
        /// </summary>
        public float RectRotation
        {
            get
            {
                return GetRotationDegrees();
            }
            set
            {
                SetRotationDegrees(value);
            }
        }

        /// <summary>
        /// The node's scale, relative to its [member rect_size]. Change this property to scale the node around its [member rect_pivot_offset].
        /// </summary>
        public Vector2 RectScale
        {
            get
            {
                return GetScale();
            }
            set
            {
                SetScale(value);
            }
        }

        /// <summary>
        /// By default, the node's pivot is its top-left corner. When you change its [member rect_scale], it will scale around this pivot. Set this property to [member rect_size] / 2 to center the pivot in the node's rectangle.
        /// </summary>
        public Vector2 RectPivotOffset
        {
            get
            {
                return GetPivotOffset();
            }
            set
            {
                SetPivotOffset(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool RectClipContent
        {
            get
            {
                return IsClippingContents();
            }
            set
            {
                SetClipContents(value);
            }
        }

        /// <summary>
        /// Changes the tooltip text. The tooltip appears when the user's mouse cursor stays idle over this control for a few moments.
        /// </summary>
        public string HintTooltip
        {
            get
            {
                return _GetTooltip();
            }
            set
            {
                SetTooltip(value);
            }
        }

        /// <summary>
        /// Tells Godot which node it should give keyboard focus to if the user presses Shift+Tab, the left arrow on the keyboard or left on a gamepad. The node must be a [code]Control[/code]. If this property is not set, Godot will give focus to the closest [code]Control[/code] to the left of this one.
        /// </summary>
        public NodePath FocusNeighbourLeft
        {
            get
            {
                return GetFocusNeighbour((Margin)0);
            }
            set
            {
                SetFocusNeighbour((Margin)0, value);
            }
        }

        /// <summary>
        /// Tells Godot which node it should give keyboard focus to if the user presses Shift+Tab, the top arrow on the keyboard or top on a gamepad. The node must be a [code]Control[/code]. If this property is not set, Godot will give focus to the closest [code]Control[/code] to the bottom of this one.
        /// </summary>
        public NodePath FocusNeighbourTop
        {
            get
            {
                return GetFocusNeighbour((Margin)1);
            }
            set
            {
                SetFocusNeighbour((Margin)1, value);
            }
        }

        /// <summary>
        /// Tells Godot which node it should give keyboard focus to if the user presses Tab, the right arrow on the keyboard or right on a gamepad. The node must be a [code]Control[/code]. If this property is not set, Godot will give focus to the closest [code]Control[/code] to the bottom of this one.
        /// </summary>
        public NodePath FocusNeighbourRight
        {
            get
            {
                return GetFocusNeighbour((Margin)2);
            }
            set
            {
                SetFocusNeighbour((Margin)2, value);
            }
        }

        /// <summary>
        /// Tells Godot which node it should give keyboard focus to if the user presses Tab, the down arrow on the keyboard, or down on a gamepad. The node must be a [code]Control[/code]. If this property is not set, Godot will give focus to the closest [code]Control[/code] to the bottom of this one.
        /// If the user presses Tab, Godot will give focus to the closest node to the right first, then to the bottom. If the user presses Shift+Tab, Godot will look to the left of the node, then above it.
        /// </summary>
        public NodePath FocusNeighbourBottom
        {
            get
            {
                return GetFocusNeighbour((Margin)3);
            }
            set
            {
                SetFocusNeighbour((Margin)3, value);
            }
        }

        /// <summary>
        /// </summary>
        public NodePath FocusNext
        {
            get
            {
                return GetFocusNext();
            }
            set
            {
                SetFocusNext(value);
            }
        }

        /// <summary>
        /// </summary>
        public NodePath FocusPrevious
        {
            get
            {
                return GetFocusPrevious();
            }
            set
            {
                SetFocusPrevious(value);
            }
        }

        /// <summary>
        /// The focus access mode for the control (None, Click or All). Only one Control can be focused at the same time, and it will receive keyboard signals.
        /// </summary>
        public Control.FocusModeEnum FocusMode
        {
            get
            {
                return GetFocusMode();
            }
            set
            {
                SetFocusMode(value);
            }
        }

        /// <summary>
        /// Controls whether the control will be able to receive mouse button input events through [method _gui_input] and how these events should be handled. Use one of the [code]MOUSE_FILTER_*[/code] constants. See the constants to learn what each does.
        /// </summary>
        public Control.MouseFilterEnum MouseFilter
        {
            get
            {
                return GetMouseFilter();
            }
            set
            {
                SetMouseFilter(value);
            }
        }

        /// <summary>
        /// The default cursor shape for this control. Useful for Godot plugins and applications or games that use the system's mouse cursors.
        /// </summary>
        public Control.CursorShape MouseDefaultCursorShape
        {
            get
            {
                return GetDefaultCursorShape();
            }
            set
            {
                SetDefaultCursorShape(value);
            }
        }

        /// <summary>
        /// Tells the parent [Container] nodes how they should resize and place the node on the X axis. Use one of the [code]SIZE_*[/code] constants to change the flags. See the constants to learn what each does.
        /// </summary>
        public int SizeFlagsHorizontal
        {
            get
            {
                return GetHSizeFlags();
            }
            set
            {
                SetHSizeFlags(value);
            }
        }

        /// <summary>
        /// Tells the parent [Container] nodes how they should resize and place the node on the Y axis. Use one of the [code]SIZE_*[/code] constants to change the flags. See the constants to learn what each does.
        /// </summary>
        public int SizeFlagsVertical
        {
            get
            {
                return GetVSizeFlags();
            }
            set
            {
                SetVSizeFlags(value);
            }
        }

        /// <summary>
        /// If the node and at least one of its neighbours uses the [code]SIZE_EXPAND[/code] size flag, the parent [Container] will let it take more or less space depending on this property. If this node has a stretch ratio of 2 and its neighbour a ratio of 1, this node will take two thirds of the available space.
        /// </summary>
        public float SizeFlagsStretchRatio
        {
            get
            {
                return GetStretchRatio();
            }
            set
            {
                SetStretchRatio(value);
            }
        }

        /// <summary>
        /// Changing this property replaces the current [Theme] resource this node and all its [code]Control[/code] children use.
        /// </summary>
        public Theme Theme
        {
            get
            {
                return GetTheme();
            }
            set
            {
                SetTheme(value);
            }
        }


        private const string nativeName = "Control";

        public Control() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Control_Ctor(this);
        }

        internal Control(bool memoryOwn) : base(memoryOwn) {}

        /// <summary>
        /// Returns the minimum size this Control can shrink to. The node can never be smaller than this minimum size.
        /// </summary>
        [GodotMethod("_get_minimum_size")]
        public virtual Vector2 _GetMinimumSize()
        {
            return default(Vector2);
        }

        /// <summary>
        /// The node's parent forwards input events to this method. Use it to process and accept inputs on UI elements. See [method accept_event].
        /// Replaces Godot 2's [code]_input_event[/code].
        /// </summary>
        [GodotMethod("_gui_input")]
        public virtual void _GuiInput(InputEvent @event)
        {
            return;
        }

        /// <summary>
        /// </summary>
        [GodotMethod("can_drop_data")]
        public virtual bool CanDropData(Vector2 position, object data)
        {
            return default(bool);
        }

        /// <summary>
        /// </summary>
        [GodotMethod("drop_data")]
        public virtual void DropData(Vector2 position, object data)
        {
            return;
        }

        /// <summary>
        /// </summary>
        [GodotMethod("get_drag_data")]
        public virtual Object GetDragData(Vector2 position)
        {
            return default(Object);
        }

        /// <summary>
        /// </summary>
        [GodotMethod("has_point")]
        public virtual bool HasPoint(Vector2 point)
        {
            return default(bool);
        }

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "accept_event");

        /// <summary>
        /// Marks an input event as handled. Once you accept an input event, it stops propagating, even to nodes listening to [method Node._unhandled_input] or [method Node._unhandled_key_input].
        /// </summary>
        [GodotMethod("accept_event")]
        public void AcceptEvent()
        {
            NativeCalls.godot_icall_0_12(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_minimum_size");

        /// <summary>
        /// Return the minimum size this Control can shrink to. A control will never be displayed or resized smaller than its minimum size.
        /// </summary>
        [GodotMethod("get_minimum_size")]
        public Vector2 GetMinimumSize()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_1, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_combined_minimum_size");

        /// <summary>
        /// </summary>
        [GodotMethod("get_combined_minimum_size")]
        public Vector2 GetCombinedMinimumSize()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_2, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_anchors_preset");

        /// <summary>
        /// </summary>
        [GodotMethod("set_anchors_preset")]
        public void SetAnchorsPreset(Control.LayoutPreset preset, bool keepMargin = false)
        {
            NativeCalls.godot_icall_2_254(method_bind_3, Object.GetPtr(this), preset, keepMargin);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_margins_preset");

        /// <summary>
        /// </summary>
        [GodotMethod("set_margins_preset")]
        public void SetMarginsPreset(Control.LayoutPreset preset, Control.LayoutPresetMode resizeMode = (Control.LayoutPresetMode)0, int margin = 0)
        {
            NativeCalls.godot_icall_3_255(method_bind_4, Object.GetPtr(this), preset, resizeMode, margin);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_anchors_and_margins_preset");

        /// <summary>
        /// </summary>
        [GodotMethod("set_anchors_and_margins_preset")]
        public void SetAnchorsAndMarginsPreset(Control.LayoutPreset preset, Control.LayoutPresetMode resizeMode = (Control.LayoutPresetMode)0, int margin = 0)
        {
            NativeCalls.godot_icall_3_255(method_bind_5, Object.GetPtr(this), preset, resizeMode, margin);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_anchor");

        /// <summary>
        /// </summary>
        [GodotMethod("set_anchor")]
        public void SetAnchor(Margin margin, float anchor, bool keepMargin = false, bool pushOppositeAnchor = true)
        {
            NativeCalls.godot_icall_4_256(method_bind_6, Object.GetPtr(this), margin, anchor, keepMargin, pushOppositeAnchor);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_anchor");

        internal void _SetAnchor(Margin margin, float anchor)
        {
            NativeCalls.godot_icall_2_198(method_bind_7, Object.GetPtr(this), margin, anchor);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_anchor");

        [GodotMethod("get_anchor")]
        public float GetAnchor(Margin margin)
        {
            return NativeCalls.godot_icall_1_199(method_bind_8, Object.GetPtr(this), margin);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_margin");

        [GodotMethod("set_margin")]
        public void SetMargin(Margin margin, float offset)
        {
            NativeCalls.godot_icall_2_198(method_bind_9, Object.GetPtr(this), margin, offset);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_anchor_and_margin");

        /// <summary>
        /// </summary>
        [GodotMethod("set_anchor_and_margin")]
        public void SetAnchorAndMargin(Margin margin, float anchor, float offset, bool pushOppositeAnchor = false)
        {
            NativeCalls.godot_icall_4_257(method_bind_10, Object.GetPtr(this), margin, anchor, offset, pushOppositeAnchor);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_begin");

        /// <summary>
        /// Sets MARGIN_LEFT and MARGIN_TOP at the same time. This is a helper (see [method set_margin]).
        /// </summary>
        [GodotMethod("set_begin")]
        public void SetBegin(Vector2 position)
        {
            NativeCalls.godot_icall_1_45(method_bind_11, Object.GetPtr(this), ref position);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_end");

        /// <summary>
        /// Sets MARGIN_RIGHT and MARGIN_BOTTOM at the same time. This is a helper (see [method set_margin]).
        /// </summary>
        [GodotMethod("set_end")]
        public void SetEnd(Vector2 position)
        {
            NativeCalls.godot_icall_1_45(method_bind_12, Object.GetPtr(this), ref position);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_position");

        [GodotMethod("set_position")]
        public void SetPosition(Vector2 position)
        {
            NativeCalls.godot_icall_1_45(method_bind_13, Object.GetPtr(this), ref position);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_size");

        [GodotMethod("set_size")]
        public void SetSize(Vector2 size)
        {
            NativeCalls.godot_icall_1_45(method_bind_14, Object.GetPtr(this), ref size);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_custom_minimum_size");

        [GodotMethod("set_custom_minimum_size")]
        public void SetCustomMinimumSize(Vector2 size)
        {
            NativeCalls.godot_icall_1_45(method_bind_15, Object.GetPtr(this), ref size);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_global_position");

        [GodotMethod("set_global_position")]
        public void SetGlobalPosition(Vector2 position)
        {
            NativeCalls.godot_icall_1_45(method_bind_16, Object.GetPtr(this), ref position);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rotation");

        /// <summary>
        /// Set the rotation (in radians).
        /// </summary>
        [GodotMethod("set_rotation")]
        public void SetRotation(float radians)
        {
            NativeCalls.godot_icall_1_10(method_bind_17, Object.GetPtr(this), radians);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rotation_degrees");

        [GodotMethod("set_rotation_degrees")]
        public void SetRotationDegrees(float degrees)
        {
            NativeCalls.godot_icall_1_10(method_bind_18, Object.GetPtr(this), degrees);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_scale");

        [GodotMethod("set_scale")]
        public void SetScale(Vector2 scale)
        {
            NativeCalls.godot_icall_1_45(method_bind_19, Object.GetPtr(this), ref scale);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_pivot_offset");

        [GodotMethod("set_pivot_offset")]
        public void SetPivotOffset(Vector2 pivotOffset)
        {
            NativeCalls.godot_icall_1_45(method_bind_20, Object.GetPtr(this), ref pivotOffset);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_margin");

        [GodotMethod("get_margin")]
        public float GetMargin(Margin margin)
        {
            return NativeCalls.godot_icall_1_199(method_bind_21, Object.GetPtr(this), margin);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_begin");

        /// <summary>
        /// </summary>
        [GodotMethod("get_begin")]
        public Vector2 GetBegin()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_22, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_end");

        /// <summary>
        /// Returns MARGIN_LEFT and MARGIN_TOP at the same time. This is a helper (see [method set_margin]).
        /// </summary>
        [GodotMethod("get_end")]
        public Vector2 GetEnd()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_23, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_position");

        [GodotMethod("get_position")]
        public Vector2 GetPosition()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_24, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_size");

        [GodotMethod("get_size")]
        public Vector2 GetSize()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_25, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rotation");

        /// <summary>
        /// Return the rotation (in radians)
        /// </summary>
        [GodotMethod("get_rotation")]
        public float GetRotation()
        {
            return NativeCalls.godot_icall_0_9(method_bind_26, Object.GetPtr(this));
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rotation_degrees");

        [GodotMethod("get_rotation_degrees")]
        public float GetRotationDegrees()
        {
            return NativeCalls.godot_icall_0_9(method_bind_27, Object.GetPtr(this));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_scale");

        [GodotMethod("get_scale")]
        public Vector2 GetScale()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_28, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_pivot_offset");

        [GodotMethod("get_pivot_offset")]
        public Vector2 GetPivotOffset()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_29, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_custom_minimum_size");

        [GodotMethod("get_custom_minimum_size")]
        public Vector2 GetCustomMinimumSize()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_30, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_parent_area_size");

        /// <summary>
        /// </summary>
        [GodotMethod("get_parent_area_size")]
        public Vector2 GetParentAreaSize()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_31, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_global_position");

        [GodotMethod("get_global_position")]
        public Vector2 GetGlobalPosition()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_32, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rect");

        /// <summary>
        /// Return position and size of the Control, relative to the top-left corner of the parent Control. This is a helper (see [method get_position], [method get_size]).
        /// </summary>
        [GodotMethod("get_rect")]
        public Rect2 GetRect()
        {
            object ret = NativeCalls.godot_icall_0_121(method_bind_33, Object.GetPtr(this));
            return (Rect2)ret;
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_global_rect");

        /// <summary>
        /// Return position and size of the Control, relative to the top-left corner of the [i]window[/i] Control. This is a helper (see [method get_global_position], [method get_size]).
        /// </summary>
        [GodotMethod("get_global_rect")]
        public Rect2 GetGlobalRect()
        {
            object ret = NativeCalls.godot_icall_0_121(method_bind_34, Object.GetPtr(this));
            return (Rect2)ret;
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "show_modal");

        /// <summary>
        /// Display a Control as modal. Control must be a subwindow. Modal controls capture the input signals until closed or the area outside them is accessed. When a modal control loses focus, or the ESC key is pressed, they automatically hide. Modal controls are used extensively for popup dialogs and menus.
        /// </summary>
        [GodotMethod("show_modal")]
        public void ShowModal(bool exclusive = false)
        {
            NativeCalls.godot_icall_1_11(method_bind_35, Object.GetPtr(this), exclusive);
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_focus_mode");

        [GodotMethod("set_focus_mode")]
        public void SetFocusMode(Control.FocusModeEnum mode)
        {
            NativeCalls.godot_icall_1_161(method_bind_36, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_focus_mode");

        [GodotMethod("get_focus_mode")]
        public Control.FocusModeEnum GetFocusMode()
        {
            return NativeCalls.godot_icall_0_162(method_bind_37, Object.GetPtr(this));
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_focus");

        /// <summary>
        /// Return whether the Control is the current focused control (see [method set_focus_mode]).
        /// </summary>
        [GodotMethod("has_focus")]
        public bool HasFocus()
        {
            return NativeCalls.godot_icall_0_3(method_bind_38, Object.GetPtr(this));
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "grab_focus");

        /// <summary>
        /// Steal the focus from another control and become the focused control (see [method set_focus_mode]).
        /// </summary>
        [GodotMethod("grab_focus")]
        public void GrabFocus()
        {
            NativeCalls.godot_icall_0_12(method_bind_39, Object.GetPtr(this));
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "release_focus");

        /// <summary>
        /// Give up the focus, no other control will be able to receive keyboard input.
        /// </summary>
        [GodotMethod("release_focus")]
        public void ReleaseFocus()
        {
            NativeCalls.godot_icall_0_12(method_bind_40, Object.GetPtr(this));
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_focus_owner");

        /// <summary>
        /// Return which control is owning the keyboard focus, or null if no one.
        /// </summary>
        [GodotMethod("get_focus_owner")]
        public Control GetFocusOwner()
        {
            return NativeCalls.godot_icall_0_258(method_bind_41, Object.GetPtr(this));
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_h_size_flags");

        [GodotMethod("set_h_size_flags")]
        public void SetHSizeFlags(int flags)
        {
            NativeCalls.godot_icall_1_0(method_bind_42, Object.GetPtr(this), flags);
        }

        private IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_h_size_flags");

        [GodotMethod("get_h_size_flags")]
        public int GetHSizeFlags()
        {
            return NativeCalls.godot_icall_0_1(method_bind_43, Object.GetPtr(this));
        }

        private IntPtr method_bind_44 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_stretch_ratio");

        [GodotMethod("set_stretch_ratio")]
        public void SetStretchRatio(float ratio)
        {
            NativeCalls.godot_icall_1_10(method_bind_44, Object.GetPtr(this), ratio);
        }

        private IntPtr method_bind_45 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_stretch_ratio");

        [GodotMethod("get_stretch_ratio")]
        public float GetStretchRatio()
        {
            return NativeCalls.godot_icall_0_9(method_bind_45, Object.GetPtr(this));
        }

        private IntPtr method_bind_46 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_v_size_flags");

        [GodotMethod("set_v_size_flags")]
        public void SetVSizeFlags(int flags)
        {
            NativeCalls.godot_icall_1_0(method_bind_46, Object.GetPtr(this), flags);
        }

        private IntPtr method_bind_47 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_v_size_flags");

        [GodotMethod("get_v_size_flags")]
        public int GetVSizeFlags()
        {
            return NativeCalls.godot_icall_0_1(method_bind_47, Object.GetPtr(this));
        }

        private IntPtr method_bind_48 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_theme");

        [GodotMethod("set_theme")]
        public void SetTheme(Theme theme)
        {
            NativeCalls.godot_icall_1_24(method_bind_48, Object.GetPtr(this), Object.GetPtr(theme));
        }

        private IntPtr method_bind_49 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_theme");

        [GodotMethod("get_theme")]
        public Theme GetTheme()
        {
            return NativeCalls.godot_icall_0_259(method_bind_49, Object.GetPtr(this));
        }

        private IntPtr method_bind_50 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_icon_override");

        /// <summary>
        /// Overrides the [code]name[/code] icon in the [theme] resource the node uses. If [code]icon[/code] is empty, Godot clears the override.
        /// </summary>
        [GodotMethod("add_icon_override")]
        public void AddIconOverride(string name, Texture texture)
        {
            NativeCalls.godot_icall_2_89(method_bind_50, Object.GetPtr(this), name, Object.GetPtr(texture));
        }

        private IntPtr method_bind_51 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_shader_override");

        /// <summary>
        /// Overrides the [code]name[/code] shader in the [theme] resource the node uses. If [code]shader[/code] is empty, Godot clears the override.
        /// </summary>
        [GodotMethod("add_shader_override")]
        public void AddShaderOverride(string name, Shader shader)
        {
            NativeCalls.godot_icall_2_89(method_bind_51, Object.GetPtr(this), name, Object.GetPtr(shader));
        }

        private IntPtr method_bind_52 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_stylebox_override");

        /// <summary>
        /// Overrides the [code]name[/code] [Stylebox] in the [theme] resource the node uses. If [code]stylebox[/code] is empty, Godot clears the override.
        /// </summary>
        [GodotMethod("add_stylebox_override")]
        public void AddStyleboxOverride(string name, StyleBox stylebox)
        {
            NativeCalls.godot_icall_2_89(method_bind_52, Object.GetPtr(this), name, Object.GetPtr(stylebox));
        }

        private IntPtr method_bind_53 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_font_override");

        /// <summary>
        /// Overrides the [code]name[/code] font in the [theme] resource the node uses. If [code]font[/code] is empty, Godot clears the override.
        /// </summary>
        [GodotMethod("add_font_override")]
        public void AddFontOverride(string name, Font font)
        {
            NativeCalls.godot_icall_2_89(method_bind_53, Object.GetPtr(this), name, Object.GetPtr(font));
        }

        private IntPtr method_bind_54 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_color_override");

        /// <summary>
        /// Overrides the color in the [theme] resource the node uses.
        /// </summary>
        [GodotMethod("add_color_override")]
        public void AddColorOverride(string name, Color color)
        {
            NativeCalls.godot_icall_2_260(method_bind_54, Object.GetPtr(this), name, ref color);
        }

        private IntPtr method_bind_55 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_constant_override");

        /// <summary>
        /// Overrides an integer constant in the [Theme] resource the node uses. If the [code]constant[/code] is invalid, Godot clears the override. See [member Theme.INVALID_CONSTANT] for more information.
        /// </summary>
        [GodotMethod("add_constant_override")]
        public void AddConstantOverride(string name, int constant)
        {
            NativeCalls.godot_icall_2_95(method_bind_55, Object.GetPtr(this), name, constant);
        }

        private IntPtr method_bind_56 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_icon");

        /// <summary>
        /// </summary>
        [GodotMethod("get_icon")]
        public Texture GetIcon(string name, string type = "")
        {
            return NativeCalls.godot_icall_2_261(method_bind_56, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_57 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_stylebox");

        /// <summary>
        /// </summary>
        [GodotMethod("get_stylebox")]
        public StyleBox GetStylebox(string name, string type = "")
        {
            return NativeCalls.godot_icall_2_262(method_bind_57, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_58 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_font");

        /// <summary>
        /// </summary>
        [GodotMethod("get_font")]
        public Font GetFont(string name, string type = "")
        {
            return NativeCalls.godot_icall_2_263(method_bind_58, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_59 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_color");

        /// <summary>
        /// </summary>
        [GodotMethod("get_color")]
        public Color GetColor(string name, string type = "")
        {
            object ret = NativeCalls.godot_icall_2_264(method_bind_59, Object.GetPtr(this), name, type);
            return (Color)ret;
        }

        private IntPtr method_bind_60 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_constant");

        /// <summary>
        /// </summary>
        [GodotMethod("get_constant")]
        public int GetConstant(string name, string type = "")
        {
            return NativeCalls.godot_icall_2_265(method_bind_60, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_61 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_icon_override");

        /// <summary>
        /// </summary>
        [GodotMethod("has_icon_override")]
        public bool HasIconOverride(string name)
        {
            return NativeCalls.godot_icall_1_70(method_bind_61, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_62 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_shader_override");

        /// <summary>
        /// </summary>
        [GodotMethod("has_shader_override")]
        public bool HasShaderOverride(string name)
        {
            return NativeCalls.godot_icall_1_70(method_bind_62, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_63 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_stylebox_override");

        /// <summary>
        /// </summary>
        [GodotMethod("has_stylebox_override")]
        public bool HasStyleboxOverride(string name)
        {
            return NativeCalls.godot_icall_1_70(method_bind_63, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_64 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_font_override");

        /// <summary>
        /// </summary>
        [GodotMethod("has_font_override")]
        public bool HasFontOverride(string name)
        {
            return NativeCalls.godot_icall_1_70(method_bind_64, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_65 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_color_override");

        /// <summary>
        /// </summary>
        [GodotMethod("has_color_override")]
        public bool HasColorOverride(string name)
        {
            return NativeCalls.godot_icall_1_70(method_bind_65, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_66 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_constant_override");

        /// <summary>
        /// </summary>
        [GodotMethod("has_constant_override")]
        public bool HasConstantOverride(string name)
        {
            return NativeCalls.godot_icall_1_70(method_bind_66, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_67 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_icon");

        /// <summary>
        /// </summary>
        [GodotMethod("has_icon")]
        public bool HasIcon(string name, string type = "")
        {
            return NativeCalls.godot_icall_2_251(method_bind_67, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_68 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_stylebox");

        /// <summary>
        /// </summary>
        [GodotMethod("has_stylebox")]
        public bool HasStylebox(string name, string type = "")
        {
            return NativeCalls.godot_icall_2_251(method_bind_68, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_69 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_font");

        /// <summary>
        /// </summary>
        [GodotMethod("has_font")]
        public bool HasFont(string name, string type = "")
        {
            return NativeCalls.godot_icall_2_251(method_bind_69, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_70 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_color");

        /// <summary>
        /// </summary>
        [GodotMethod("has_color")]
        public bool HasColor(string name, string type = "")
        {
            return NativeCalls.godot_icall_2_251(method_bind_70, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_71 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_constant");

        /// <summary>
        /// </summary>
        [GodotMethod("has_constant")]
        public bool HasConstant(string name, string type = "")
        {
            return NativeCalls.godot_icall_2_251(method_bind_71, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_72 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_parent_control");

        /// <summary>
        /// </summary>
        [GodotMethod("get_parent_control")]
        public Control GetParentControl()
        {
            return NativeCalls.godot_icall_0_258(method_bind_72, Object.GetPtr(this));
        }

        private IntPtr method_bind_73 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_h_grow_direction");

        [GodotMethod("set_h_grow_direction")]
        public void SetHGrowDirection(Control.GrowDirection direction)
        {
            NativeCalls.godot_icall_1_266(method_bind_73, Object.GetPtr(this), direction);
        }

        private IntPtr method_bind_74 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_h_grow_direction");

        [GodotMethod("get_h_grow_direction")]
        public Control.GrowDirection GetHGrowDirection()
        {
            return NativeCalls.godot_icall_0_267(method_bind_74, Object.GetPtr(this));
        }

        private IntPtr method_bind_75 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_v_grow_direction");

        [GodotMethod("set_v_grow_direction")]
        public void SetVGrowDirection(Control.GrowDirection direction)
        {
            NativeCalls.godot_icall_1_266(method_bind_75, Object.GetPtr(this), direction);
        }

        private IntPtr method_bind_76 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_v_grow_direction");

        [GodotMethod("get_v_grow_direction")]
        public Control.GrowDirection GetVGrowDirection()
        {
            return NativeCalls.godot_icall_0_267(method_bind_76, Object.GetPtr(this));
        }

        private IntPtr method_bind_77 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tooltip");

        [GodotMethod("set_tooltip")]
        public void SetTooltip(string tooltip)
        {
            NativeCalls.godot_icall_1_43(method_bind_77, Object.GetPtr(this), tooltip);
        }

        private IntPtr method_bind_78 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tooltip");

        /// <summary>
        /// Return the tooltip, which will appear when the cursor is resting over this control.
        /// &lt;param name="atPosition"&gt;If the param is null, then the default value is new Vector2(0, 0)&lt;/param&gt;
        /// </summary>
        [GodotMethod("get_tooltip")]
        public string GetTooltip(Nullable<Vector2> atPosition = null)
        {
            Vector2 atPosition_in = atPosition.HasValue ? atPosition.Value : new Vector2(0, 0);
            return NativeCalls.godot_icall_1_268(method_bind_78, Object.GetPtr(this), ref atPosition_in);
        }

        private IntPtr method_bind_79 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_tooltip");

        internal string _GetTooltip()
        {
            return NativeCalls.godot_icall_0_2(method_bind_79, Object.GetPtr(this));
        }

        private IntPtr method_bind_80 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_default_cursor_shape");

        [GodotMethod("set_default_cursor_shape")]
        public void SetDefaultCursorShape(Control.CursorShape shape)
        {
            NativeCalls.godot_icall_1_269(method_bind_80, Object.GetPtr(this), shape);
        }

        private IntPtr method_bind_81 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_default_cursor_shape");

        [GodotMethod("get_default_cursor_shape")]
        public Control.CursorShape GetDefaultCursorShape()
        {
            return NativeCalls.godot_icall_0_270(method_bind_81, Object.GetPtr(this));
        }

        private IntPtr method_bind_82 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cursor_shape");

        /// <summary>
        /// Returns the mouse cursor shape the control displays on mouse hover, one of the [code]CURSOR_*[/code] constants.
        /// &lt;param name="position"&gt;If the param is null, then the default value is new Vector2(0, 0)&lt;/param&gt;
        /// </summary>
        [GodotMethod("get_cursor_shape")]
        public Control.CursorShape GetCursorShape(Nullable<Vector2> position = null)
        {
            Vector2 position_in = position.HasValue ? position.Value : new Vector2(0, 0);
            return NativeCalls.godot_icall_1_271(method_bind_82, Object.GetPtr(this), ref position_in);
        }

        private IntPtr method_bind_83 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_focus_neighbour");

        [GodotMethod("set_focus_neighbour")]
        public void SetFocusNeighbour(Margin margin, NodePath neighbour)
        {
            NativeCalls.godot_icall_2_272(method_bind_83, Object.GetPtr(this), margin, NodePath.GetPtr(neighbour));
        }

        private IntPtr method_bind_84 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_focus_neighbour");

        [GodotMethod("get_focus_neighbour")]
        public NodePath GetFocusNeighbour(Margin margin)
        {
            IntPtr ret = NativeCalls.godot_icall_1_273(method_bind_84, Object.GetPtr(this), margin);
            return new NodePath(ret);
        }

        private IntPtr method_bind_85 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_focus_next");

        [GodotMethod("set_focus_next")]
        public void SetFocusNext(NodePath next)
        {
            NativeCalls.godot_icall_1_78(method_bind_85, Object.GetPtr(this), NodePath.GetPtr(next));
        }

        private IntPtr method_bind_86 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_focus_next");

        [GodotMethod("get_focus_next")]
        public NodePath GetFocusNext()
        {
            IntPtr ret = NativeCalls.godot_icall_0_79(method_bind_86, Object.GetPtr(this));
            return new NodePath(ret);
        }

        private IntPtr method_bind_87 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_focus_previous");

        [GodotMethod("set_focus_previous")]
        public void SetFocusPrevious(NodePath previous)
        {
            NativeCalls.godot_icall_1_78(method_bind_87, Object.GetPtr(this), NodePath.GetPtr(previous));
        }

        private IntPtr method_bind_88 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_focus_previous");

        [GodotMethod("get_focus_previous")]
        public NodePath GetFocusPrevious()
        {
            IntPtr ret = NativeCalls.godot_icall_0_79(method_bind_88, Object.GetPtr(this));
            return new NodePath(ret);
        }

        private IntPtr method_bind_89 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "force_drag");

        /// <summary>
        /// </summary>
        [GodotMethod("force_drag")]
        public void ForceDrag(object data, Control preview)
        {
            NativeCalls.godot_icall_2_274(method_bind_89, Object.GetPtr(this), data, Object.GetPtr(preview));
        }

        private IntPtr method_bind_90 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_mouse_filter");

        [GodotMethod("set_mouse_filter")]
        public void SetMouseFilter(Control.MouseFilterEnum filter)
        {
            NativeCalls.godot_icall_1_275(method_bind_90, Object.GetPtr(this), filter);
        }

        private IntPtr method_bind_91 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mouse_filter");

        [GodotMethod("get_mouse_filter")]
        public Control.MouseFilterEnum GetMouseFilter()
        {
            return NativeCalls.godot_icall_0_276(method_bind_91, Object.GetPtr(this));
        }

        private IntPtr method_bind_92 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_clip_contents");

        [GodotMethod("set_clip_contents")]
        public void SetClipContents(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_92, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_93 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_clipping_contents");

        [GodotMethod("is_clipping_contents")]
        public bool IsClippingContents()
        {
            return NativeCalls.godot_icall_0_3(method_bind_93, Object.GetPtr(this));
        }

        private IntPtr method_bind_94 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "grab_click_focus");

        /// <summary>
        /// </summary>
        [GodotMethod("grab_click_focus")]
        public void GrabClickFocus()
        {
            NativeCalls.godot_icall_0_12(method_bind_94, Object.GetPtr(this));
        }

        private IntPtr method_bind_95 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_drag_forwarding");

        /// <summary>
        /// </summary>
        [GodotMethod("set_drag_forwarding")]
        public void SetDragForwarding(Control target)
        {
            NativeCalls.godot_icall_1_42(method_bind_95, Object.GetPtr(this), Object.GetPtr(target));
        }

        private IntPtr method_bind_96 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_drag_preview");

        /// <summary>
        /// </summary>
        [GodotMethod("set_drag_preview")]
        public void SetDragPreview(Control control)
        {
            NativeCalls.godot_icall_1_42(method_bind_96, Object.GetPtr(this), Object.GetPtr(control));
        }

        private IntPtr method_bind_97 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "warp_mouse");

        /// <summary>
        /// </summary>
        [GodotMethod("warp_mouse")]
        public void WarpMouse(Vector2 toPosition)
        {
            NativeCalls.godot_icall_1_45(method_bind_97, Object.GetPtr(this), ref toPosition);
        }

        private IntPtr method_bind_98 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "minimum_size_changed");

        /// <summary>
        /// </summary>
        [GodotMethod("minimum_size_changed")]
        public void MinimumSizeChanged()
        {
            NativeCalls.godot_icall_0_12(method_bind_98, Object.GetPtr(this));
        }
    }
}
