using System;

namespace Godot
{
    /// <summary>
    /// A 6-sided 3D texture typically used for faking reflections. It can be used to make an object look as if it's reflecting its surroundings. This usually delivers much better performance than other reflection methods.
    /// </summary>
    public class CubeMap : Resource
    {
        public enum FlagsEnum
        {
            Mipmaps = 1,
            Repeat = 2,
            Filter = 4,
            sDefault = 7
        }

        public enum Side
        {
            Left = 0,
            Right = 1,
            Bottom = 2,
            Top = 3,
            Front = 4,
            Back = 5
        }

        public enum Storage
        {
            Raw = 0,
            CompressLossy = 1,
            CompressLossless = 2
        }


        /// <summary>
        /// The render flags for the [code]CubeMap[/code]. See the [code]FLAG_*[/code] constants for details.
        /// </summary>
        public int Flags
        {
            get
            {
                return GetFlags();
            }
            set
            {
                SetFlags(value);
            }
        }

        /// <summary>
        /// The [code]CubeMap[/code]'s storage mode. See [code]STORAGE_*[/code] constants.
        /// </summary>
        public CubeMap.Storage StorageMode
        {
            get
            {
                return GetStorage();
            }
            set
            {
                SetStorage(value);
            }
        }

        /// <summary>
        /// The lossy storage quality of the [code]CubeMap[/code] if the storage mode is set to STORAGE_COMPRESS_LOSSY.
        /// </summary>
        public float LossyStorageQuality
        {
            get
            {
                return GetLossyStorageQuality();
            }
            set
            {
                SetLossyStorageQuality(value);
            }
        }


        private const string nativeName = "CubeMap";

        public CubeMap() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_CubeMap_Ctor(this);
        }

        internal CubeMap(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_width");

        /// <summary>
        /// Returns the [code]CubeMap[/code]'s width.
        /// </summary>
        [GodotMethod("get_width")]
        public int GetWidth()
        {
            return NativeCalls.godot_icall_0_1(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_height");

        /// <summary>
        /// Returns the [code]CubeMap[/code]'s height.
        /// </summary>
        [GodotMethod("get_height")]
        public int GetHeight()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_flags");

        [GodotMethod("set_flags")]
        public void SetFlags(int flags)
        {
            NativeCalls.godot_icall_1_0(method_bind_2, Object.GetPtr(this), flags);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_flags");

        [GodotMethod("get_flags")]
        public int GetFlags()
        {
            return NativeCalls.godot_icall_0_1(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_side");

        /// <summary>
        /// Sets an [Image] for a side of the [code]CubeMap[/code] using one of the [code]SIDE_*[/code] constants or an integer 0-5.
        /// </summary>
        [GodotMethod("set_side")]
        public void SetSide(CubeMap.Side side, Image image)
        {
            NativeCalls.godot_icall_2_277(method_bind_4, Object.GetPtr(this), side, Object.GetPtr(image));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_side");

        /// <summary>
        /// Returns an [Image] for a side of the [code]CubeMap[/code] using one of the [code]SIDE_*[/code] constants or an integer 0-5.
        /// </summary>
        [GodotMethod("get_side")]
        public Image GetSide(CubeMap.Side side)
        {
            return NativeCalls.godot_icall_1_278(method_bind_5, Object.GetPtr(this), side);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_storage");

        [GodotMethod("set_storage")]
        public void SetStorage(CubeMap.Storage mode)
        {
            NativeCalls.godot_icall_1_279(method_bind_6, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_storage");

        [GodotMethod("get_storage")]
        public CubeMap.Storage GetStorage()
        {
            return NativeCalls.godot_icall_0_280(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_lossy_storage_quality");

        [GodotMethod("set_lossy_storage_quality")]
        public void SetLossyStorageQuality(float quality)
        {
            NativeCalls.godot_icall_1_10(method_bind_8, Object.GetPtr(this), quality);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_lossy_storage_quality");

        [GodotMethod("get_lossy_storage_quality")]
        public float GetLossyStorageQuality()
        {
            return NativeCalls.godot_icall_0_9(method_bind_9, Object.GetPtr(this));
        }
    }
}
