using System;

namespace Godot
{
    /// <summary>
    /// A curve that can be saved and re-used for other objects. By default it ranges between [code]0[/code] and [code]1[/code] on the y-axis and positions points relative to the [code]0.5[/code] y-position.
    /// </summary>
    public class Curve : Resource
    {
        public enum TangentMode
        {
            Free = 0,
            Linear = 1,
            ModeCount = 2
        }


        /// <summary>
        /// The minimum value the curve can reach. Default value: [code]0[/code].
        /// </summary>
        public float MinValue
        {
            get
            {
                return GetMinValue();
            }
            set
            {
                SetMinValue(value);
            }
        }

        /// <summary>
        /// The maximum value the curve can reach. Default value: [code]1[/code].
        /// </summary>
        public float MaxValue
        {
            get
            {
                return GetMaxValue();
            }
            set
            {
                SetMaxValue(value);
            }
        }

        /// <summary>
        /// The number of points to include in the baked (i.e. cached) curve data.
        /// </summary>
        public int BakeResolution
        {
            get
            {
                return GetBakeResolution();
            }
            set
            {
                SetBakeResolution(value);
            }
        }

        public object[] _Data
        {
            get
            {
                return _GetData();
            }
            set
            {
                _SetData(value);
            }
        }


        private const string nativeName = "Curve";

        public Curve() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Curve_Ctor(this);
        }

        internal Curve(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_point");

        /// <summary>
        /// Adds a point to the curve. For each side, if the [code]*_mode[/code] is [code]TANGENT_LINEAR[/code], the [code]*_tangent[/code] angle (in degrees) uses the slope of the curve halfway to the adjacent point. Allows custom assignments to the [code]*_tangent[/code] angle if [code]*_mode[/code] is set to [code]TANGENT_FREE[/code].
        /// </summary>
        [GodotMethod("add_point")]
        public int AddPoint(Vector2 position, float leftTangent = (float)0, float rightTangent = (float)0, Curve.TangentMode leftMode = (Curve.TangentMode)0, Curve.TangentMode rightMode = (Curve.TangentMode)0)
        {
            return NativeCalls.godot_icall_5_281(method_bind_0, Object.GetPtr(this), ref position, leftTangent, rightTangent, leftMode, rightMode);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_point");

        /// <summary>
        /// Removes the point at [code]index[/code] from the curve.
        /// </summary>
        [GodotMethod("remove_point")]
        public void RemovePoint(int index)
        {
            NativeCalls.godot_icall_1_0(method_bind_1, Object.GetPtr(this), index);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_points");

        /// <summary>
        /// Removes all points from the curve.
        /// </summary>
        [GodotMethod("clear_points")]
        public void ClearPoints()
        {
            NativeCalls.godot_icall_0_12(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_point_position");

        /// <summary>
        /// Returns the curve coordinates for the point at [code]index[/code].
        /// </summary>
        [GodotMethod("get_point_position")]
        public Vector2 GetPointPosition(int index)
        {
            object ret = NativeCalls.godot_icall_1_282(method_bind_3, Object.GetPtr(this), index);
            return (Vector2)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_point_value");

        /// <summary>
        /// Assigns the vertical position [code]y[/code] to the point at [code]index[/code].
        /// </summary>
        [GodotMethod("set_point_value")]
        public void SetPointValue(int index, float y)
        {
            NativeCalls.godot_icall_2_28(method_bind_4, Object.GetPtr(this), index, y);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_point_offset");

        /// <summary>
        /// Sets the offset from [code]0.5[/code]
        /// </summary>
        [GodotMethod("set_point_offset")]
        public int SetPointOffset(int index, float offset)
        {
            return NativeCalls.godot_icall_2_283(method_bind_5, Object.GetPtr(this), index, offset);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "interpolate");

        /// <summary>
        /// Returns the y value for the point that would exist at x-position [code]offset[/code] along the curve.
        /// </summary>
        [GodotMethod("interpolate")]
        public float Interpolate(float offset)
        {
            return NativeCalls.godot_icall_1_284(method_bind_6, Object.GetPtr(this), offset);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "interpolate_baked");

        /// <summary>
        /// Returns the y value for the point that would exist at x-position [code]offset[/code] along the curve using the baked cache. Bakes the curve's points if not already baked.
        /// </summary>
        [GodotMethod("interpolate_baked")]
        public float InterpolateBaked(float offset)
        {
            return NativeCalls.godot_icall_1_284(method_bind_7, Object.GetPtr(this), offset);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_point_left_tangent");

        /// <summary>
        /// Returns the left tangent angle (in degrees) for the point at [code]index[/code].
        /// </summary>
        [GodotMethod("get_point_left_tangent")]
        public float GetPointLeftTangent(int index)
        {
            return NativeCalls.godot_icall_1_7(method_bind_8, Object.GetPtr(this), index);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_point_right_tangent");

        /// <summary>
        /// Returns the right tangent angle (in degrees) for the point at [code]index[/code].
        /// </summary>
        [GodotMethod("get_point_right_tangent")]
        public float GetPointRightTangent(int index)
        {
            return NativeCalls.godot_icall_1_7(method_bind_9, Object.GetPtr(this), index);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_point_left_mode");

        /// <summary>
        /// Returns the left [code]TangentMode[/code] for the point at [code]index[/code].
        /// </summary>
        [GodotMethod("get_point_left_mode")]
        public Curve.TangentMode GetPointLeftMode(int index)
        {
            return NativeCalls.godot_icall_1_285(method_bind_10, Object.GetPtr(this), index);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_point_right_mode");

        /// <summary>
        /// Returns the right [code]TangentMode[/code] for the point at [code]index[/code].
        /// </summary>
        [GodotMethod("get_point_right_mode")]
        public Curve.TangentMode GetPointRightMode(int index)
        {
            return NativeCalls.godot_icall_1_285(method_bind_11, Object.GetPtr(this), index);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_point_left_tangent");

        /// <summary>
        /// Sets the left tangent angle for the point at [code]index[/code] to [code]tangent[/code].
        /// </summary>
        [GodotMethod("set_point_left_tangent")]
        public void SetPointLeftTangent(int index, float tangent)
        {
            NativeCalls.godot_icall_2_28(method_bind_12, Object.GetPtr(this), index, tangent);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_point_right_tangent");

        /// <summary>
        /// Sets the right tangent angle for the point at [code]index[/code] to [code]tangent[/code].
        /// </summary>
        [GodotMethod("set_point_right_tangent")]
        public void SetPointRightTangent(int index, float tangent)
        {
            NativeCalls.godot_icall_2_28(method_bind_13, Object.GetPtr(this), index, tangent);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_point_left_mode");

        /// <summary>
        /// Sets the left [code]TangentMode[/code] for the point at [code]index[/code] to [code]mode[/code].
        /// </summary>
        [GodotMethod("set_point_left_mode")]
        public void SetPointLeftMode(int index, Curve.TangentMode mode)
        {
            NativeCalls.godot_icall_2_286(method_bind_14, Object.GetPtr(this), index, mode);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_point_right_mode");

        /// <summary>
        /// Sets the right [code]TangentMode[/code] for the point at [code]index[/code] to [code]mode[/code].
        /// </summary>
        [GodotMethod("set_point_right_mode")]
        public void SetPointRightMode(int index, Curve.TangentMode mode)
        {
            NativeCalls.godot_icall_2_286(method_bind_15, Object.GetPtr(this), index, mode);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_min_value");

        [GodotMethod("get_min_value")]
        public float GetMinValue()
        {
            return NativeCalls.godot_icall_0_9(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_min_value");

        [GodotMethod("set_min_value")]
        public void SetMinValue(float min)
        {
            NativeCalls.godot_icall_1_10(method_bind_17, Object.GetPtr(this), min);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_max_value");

        [GodotMethod("get_max_value")]
        public float GetMaxValue()
        {
            return NativeCalls.godot_icall_0_9(method_bind_18, Object.GetPtr(this));
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_max_value");

        [GodotMethod("set_max_value")]
        public void SetMaxValue(float max)
        {
            NativeCalls.godot_icall_1_10(method_bind_19, Object.GetPtr(this), max);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clean_dupes");

        /// <summary>
        /// Removes points that are closer than [code]CMP_EPSILON[/code] (0.00001) units to their neighbor on the curve.
        /// </summary>
        [GodotMethod("clean_dupes")]
        public void CleanDupes()
        {
            NativeCalls.godot_icall_0_12(method_bind_20, Object.GetPtr(this));
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "bake");

        /// <summary>
        /// Recomputes the baked cache of points for the curve.
        /// </summary>
        [GodotMethod("bake")]
        public void Bake()
        {
            NativeCalls.godot_icall_0_12(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bake_resolution");

        [GodotMethod("get_bake_resolution")]
        public int GetBakeResolution()
        {
            return NativeCalls.godot_icall_0_1(method_bind_22, Object.GetPtr(this));
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bake_resolution");

        [GodotMethod("set_bake_resolution")]
        public void SetBakeResolution(int resolution)
        {
            NativeCalls.godot_icall_1_0(method_bind_23, Object.GetPtr(this), resolution);
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_data");

        internal object[] _GetData()
        {
            return NativeCalls.godot_icall_0_21(method_bind_24, Object.GetPtr(this));
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_data");

        internal void _SetData(object[] data)
        {
            NativeCalls.godot_icall_1_154(method_bind_25, Object.GetPtr(this), data);
        }
    }
}
