using System;
using System.Collections.Generic;

namespace Godot
{
    /// <summary>
    /// This class describes a Bezier curve in 2D space. It is mainly used to give a shape to a [Path2D], but can be manually sampled for other purposes.
    /// It keeps a cache of precalculated points along the curve, to speed further calculations up.
    /// </summary>
    public class Curve2D : Resource
    {
        /// <summary>
        /// The distance in pixels between two adjacent cached points. Changing it forces the cache to be recomputed the next time the [method get_baked_points] or [method get_baked_length] function is called. The smaller the distance, the more points in the cache and the more memory it will consume, so use with care.
        /// </summary>
        public float BakeInterval
        {
            get
            {
                return GetBakeInterval();
            }
            set
            {
                SetBakeInterval(value);
            }
        }

        public Dictionary<object, object> _Data
        {
            get
            {
                return _GetData();
            }
            set
            {
                _SetData(value);
            }
        }


        private const string nativeName = "Curve2D";

        public Curve2D() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Curve2D_Ctor(this);
        }

        internal Curve2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_point_count");

        /// <summary>
        /// Returns the number of points describing the curve.
        /// </summary>
        [GodotMethod("get_point_count")]
        public int GetPointCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_point");

        /// <summary>
        /// Adds a point to a curve, at "position", with control points "in" and "out".
        /// If "at_position" is given, the point is inserted before the point number "at_position", moving that point (and every point after) after the inserted point. If "at_position" is not given, or is an illegal value (at_position &lt;0 or at_position &gt;= [method get_point_count]), the point will be appended at the end of the point list.
        /// &lt;param name="@in"&gt;If the param is null, then the default value is new Vector2(0, 0)&lt;/param&gt;
        /// &lt;param name="@out"&gt;If the param is null, then the default value is new Vector2(0, 0)&lt;/param&gt;
        /// </summary>
        [GodotMethod("add_point")]
        public void AddPoint(Vector2 position, Nullable<Vector2> @in = null, Nullable<Vector2> @out = null, int atPosition = -1)
        {
            Vector2 @in_in = @in.HasValue ? @in.Value : new Vector2(0, 0);
            Vector2 @out_in = @out.HasValue ? @out.Value : new Vector2(0, 0);
            NativeCalls.godot_icall_4_287(method_bind_1, Object.GetPtr(this), ref position, ref @in_in, ref @out_in, atPosition);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_point_position");

        /// <summary>
        /// Sets the position for the vertex "idx". If the index is out of bounds, the function sends an error to the console.
        /// </summary>
        [GodotMethod("set_point_position")]
        public void SetPointPosition(int idx, Vector2 position)
        {
            NativeCalls.godot_icall_2_288(method_bind_2, Object.GetPtr(this), idx, ref position);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_point_position");

        /// <summary>
        /// Returns the position of the vertex "idx". If the index is out of bounds, the function sends an error to the console, and returns (0, 0).
        /// </summary>
        [GodotMethod("get_point_position")]
        public Vector2 GetPointPosition(int idx)
        {
            object ret = NativeCalls.godot_icall_1_282(method_bind_3, Object.GetPtr(this), idx);
            return (Vector2)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_point_in");

        /// <summary>
        /// Sets the position of the control point leading to the vertex "idx". If the index is out of bounds, the function sends an error to the console.
        /// </summary>
        [GodotMethod("set_point_in")]
        public void SetPointIn(int idx, Vector2 position)
        {
            NativeCalls.godot_icall_2_288(method_bind_4, Object.GetPtr(this), idx, ref position);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_point_in");

        /// <summary>
        /// Returns the position of the control point leading to the vertex "idx". If the index is out of bounds, the function sends an error to the console, and returns (0, 0).
        /// </summary>
        [GodotMethod("get_point_in")]
        public Vector2 GetPointIn(int idx)
        {
            object ret = NativeCalls.godot_icall_1_282(method_bind_5, Object.GetPtr(this), idx);
            return (Vector2)ret;
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_point_out");

        /// <summary>
        /// Sets the position of the control point leading out of the vertex "idx". If the index is out of bounds, the function sends an error to the console.
        /// </summary>
        [GodotMethod("set_point_out")]
        public void SetPointOut(int idx, Vector2 position)
        {
            NativeCalls.godot_icall_2_288(method_bind_6, Object.GetPtr(this), idx, ref position);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_point_out");

        /// <summary>
        /// Returns the position of the control point leading out of the vertex "idx". If the index is out of bounds, the function sends an error to the console, and returns (0, 0).
        /// </summary>
        [GodotMethod("get_point_out")]
        public Vector2 GetPointOut(int idx)
        {
            object ret = NativeCalls.godot_icall_1_282(method_bind_7, Object.GetPtr(this), idx);
            return (Vector2)ret;
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_point");

        /// <summary>
        /// Deletes the point "idx" from the curve. Sends an error to the console if "idx" is out of bounds.
        /// </summary>
        [GodotMethod("remove_point")]
        public void RemovePoint(int idx)
        {
            NativeCalls.godot_icall_1_0(method_bind_8, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_points");

        /// <summary>
        /// Removes all points from the curve.
        /// </summary>
        [GodotMethod("clear_points")]
        public void ClearPoints()
        {
            NativeCalls.godot_icall_0_12(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "interpolate");

        /// <summary>
        /// Returns the position between the vertex "idx" and the vertex "idx"+1, where "t" controls if the point is the first vertex (t = 0.0), the last vertex (t = 1.0), or in between. Values of "t" outside the range (0.0 &gt;= t  &lt;=1) give strange, but predictable results.
        /// If "idx" is out of bounds it is truncated to the first or last vertex, and "t" is ignored. If the curve has no points, the function sends an error to the console, and returns (0, 0).
        /// </summary>
        [GodotMethod("interpolate")]
        public Vector2 Interpolate(int idx, float t)
        {
            object ret = NativeCalls.godot_icall_2_289(method_bind_10, Object.GetPtr(this), idx, t);
            return (Vector2)ret;
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "interpolatef");

        /// <summary>
        /// Returns the position at the vertex "fofs". It calls [method interpolate] using the integer part of fofs as "idx", and its fractional part as "t".
        /// </summary>
        [GodotMethod("interpolatef")]
        public Vector2 Interpolatef(float fofs)
        {
            object ret = NativeCalls.godot_icall_1_290(method_bind_11, Object.GetPtr(this), fofs);
            return (Vector2)ret;
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bake_interval");

        [GodotMethod("set_bake_interval")]
        public void SetBakeInterval(float distance)
        {
            NativeCalls.godot_icall_1_10(method_bind_12, Object.GetPtr(this), distance);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bake_interval");

        [GodotMethod("get_bake_interval")]
        public float GetBakeInterval()
        {
            return NativeCalls.godot_icall_0_9(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_baked_length");

        /// <summary>
        /// Returns the total length of the curve, based on the cached points. Given enough density (see [method set_bake_interval]), it should be approximate enough.
        /// </summary>
        [GodotMethod("get_baked_length")]
        public float GetBakedLength()
        {
            return NativeCalls.godot_icall_0_9(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "interpolate_baked");

        /// <summary>
        /// Returns a point within the curve at position "offset", where "offset" is measured as a pixel distance along the curve.
        /// To do that, it finds the two cached points where the "offset" lies between, then interpolates the values. This interpolation is cubic if "cubic" is set to true, or linear if set to false.
        /// Cubic interpolation tends to follow the curves better, but linear is faster (and often, precise enough).
        /// </summary>
        [GodotMethod("interpolate_baked")]
        public Vector2 InterpolateBaked(float offset, bool cubic = false)
        {
            object ret = NativeCalls.godot_icall_2_291(method_bind_15, Object.GetPtr(this), offset, cubic);
            return (Vector2)ret;
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_baked_points");

        /// <summary>
        /// Returns the cache of points as a [PoolVector2Array].
        /// </summary>
        [GodotMethod("get_baked_points")]
        public Vector2[] GetBakedPoints()
        {
            return NativeCalls.godot_icall_0_238(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tessellate");

        /// <summary>
        /// Returns a list of points along the curve, with a curvature controlled point density. That is, the curvier parts will have more points than the straighter parts.
        /// This approximation makes straight segments between each point, then subdivides those segments until the resulting shape is similar enough.
        /// "max_stages" controls how many subdivisions a curve segment may face before it is considered approximate enough. Each subdivision splits the segment in half, so the default 5 stages may mean up to 32 subdivisions per curve segment. Increase with care!
        /// "tolerance_degrees" controls how many degrees the midpoint of a segment may deviate from the real curve, before the segment has to be subdivided.
        /// </summary>
        [GodotMethod("tessellate")]
        public Vector2[] Tessellate(int maxStages = 5, float toleranceDegrees = (float)4)
        {
            return NativeCalls.godot_icall_2_292(method_bind_17, Object.GetPtr(this), maxStages, toleranceDegrees);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_data");

        internal Dictionary<object, object> _GetData()
        {
            return NativeCalls.godot_icall_0_169(method_bind_18, Object.GetPtr(this));
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_data");

        internal void _SetData(Dictionary<object, object> arg0)
        {
            NativeCalls.godot_icall_1_168(method_bind_19, Object.GetPtr(this), arg0);
        }
    }
}
