using System;

namespace Godot
{
    /// <summary>
    /// Damped spring constraint for 2D physics. This resembles a spring joint that always wants to go back to a given length.
    /// </summary>
    public class DampedSpringJoint2D : Joint2D
    {
        /// <summary>
        /// The spring joint's maximum length. The two attached bodies cannot stretch it past this value. Default value: [code]50[/code]
        /// </summary>
        public float Length
        {
            get
            {
                return GetLength();
            }
            set
            {
                SetLength(value);
            }
        }

        /// <summary>
        /// When the bodies attached to the spring joint move they stretch or squash it. The joint always tries to resize towards this length. Default value: [code]0[/code]
        /// </summary>
        public float RestLength
        {
            get
            {
                return GetRestLength();
            }
            set
            {
                SetRestLength(value);
            }
        }

        /// <summary>
        /// The higher the value, the less the bodies attached to the joint will deform it. The joint applies an opposing force to the bodies, the product of the stiffness multiplied by the size difference from its resting length. Default value: [code]20[/code]
        /// </summary>
        public float Stiffness
        {
            get
            {
                return GetStiffness();
            }
            set
            {
                SetStiffness(value);
            }
        }

        /// <summary>
        /// The spring joint's damping ratio. A value between [code]0[/code] and [code]1[/code]. When the two bodies move into different directions the system tries to align them to the spring axis again. A high [code]damping[/code] value forces the attached bodies to align faster. Default value: [code]1[/code]
        /// </summary>
        public float Damping
        {
            get
            {
                return GetDamping();
            }
            set
            {
                SetDamping(value);
            }
        }


        private const string nativeName = "DampedSpringJoint2D";

        public DampedSpringJoint2D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_DampedSpringJoint2D_Ctor(this);
        }

        internal DampedSpringJoint2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_length");

        [GodotMethod("set_length")]
        public void SetLength(float length)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), length);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_length");

        [GodotMethod("get_length")]
        public float GetLength()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rest_length");

        [GodotMethod("set_rest_length")]
        public void SetRestLength(float restLength)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), restLength);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rest_length");

        [GodotMethod("get_rest_length")]
        public float GetRestLength()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_stiffness");

        [GodotMethod("set_stiffness")]
        public void SetStiffness(float stiffness)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), stiffness);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_stiffness");

        [GodotMethod("get_stiffness")]
        public float GetStiffness()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_damping");

        [GodotMethod("set_damping")]
        public void SetDamping(float damping)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), damping);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_damping");

        [GodotMethod("get_damping")]
        public float GetDamping()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }
    }
}
