using System;

namespace Godot
{
    /// <summary>
    /// DynamicFont renders vector font files (such as TTF or OTF) dynamically at runtime instead of using a prerendered texture atlas like [BitmapFont]. This trades the faster loading time of [BitmapFont]s for the ability to change font parameters like size and spacing during runtime. [DynamicFontData] is used for referencing the font file paths.
    /// </summary>
    public class DynamicFont : Font
    {
        public enum SpacingType
        {
            Top = 0,
            Bottom = 1,
            Char = 2,
            Space = 3
        }


        /// <summary>
        /// The font size.
        /// </summary>
        public int Size
        {
            get
            {
                return GetSize();
            }
            set
            {
                SetSize(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] mipmapping is used.
        /// </summary>
        public bool UseMipmaps
        {
            get
            {
                return GetUseMipmaps();
            }
            set
            {
                SetUseMipmaps(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] filtering is used.
        /// </summary>
        public bool UseFilter
        {
            get
            {
                return GetUseFilter();
            }
            set
            {
                SetUseFilter(value);
            }
        }

        /// <summary>
        /// Extra spacing at the top in pixels.
        /// </summary>
        public int ExtraSpacingTop
        {
            get
            {
                return GetSpacing(0);
            }
            set
            {
                SetSpacing(0, value);
            }
        }

        /// <summary>
        /// Extra spacing at the bottom in pixels.
        /// </summary>
        public int ExtraSpacingBottom
        {
            get
            {
                return GetSpacing(1);
            }
            set
            {
                SetSpacing(1, value);
            }
        }

        /// <summary>
        /// Extra character spacing in pixels.
        /// </summary>
        public int ExtraSpacingChar
        {
            get
            {
                return GetSpacing(2);
            }
            set
            {
                SetSpacing(2, value);
            }
        }

        /// <summary>
        /// Extra space spacing in pixels.
        /// </summary>
        public int ExtraSpacingSpace
        {
            get
            {
                return GetSpacing(3);
            }
            set
            {
                SetSpacing(3, value);
            }
        }

        /// <summary>
        /// The font data.
        /// </summary>
        public DynamicFontData FontData
        {
            get
            {
                return GetFontData();
            }
            set
            {
                SetFontData(value);
            }
        }


        private const string nativeName = "DynamicFont";

        public DynamicFont() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_DynamicFont_Ctor(this);
        }

        internal DynamicFont(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_font_data");

        [GodotMethod("set_font_data")]
        public void SetFontData(DynamicFontData data)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(data));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_font_data");

        [GodotMethod("get_font_data")]
        public DynamicFontData GetFontData()
        {
            return NativeCalls.godot_icall_0_304(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_size");

        [GodotMethod("set_size")]
        public void SetSize(int data)
        {
            NativeCalls.godot_icall_1_0(method_bind_2, Object.GetPtr(this), data);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_size");

        [GodotMethod("get_size")]
        public int GetSize()
        {
            return NativeCalls.godot_icall_0_1(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_use_mipmaps");

        [GodotMethod("set_use_mipmaps")]
        public void SetUseMipmaps(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_4, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_use_mipmaps");

        [GodotMethod("get_use_mipmaps")]
        public bool GetUseMipmaps()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_use_filter");

        [GodotMethod("set_use_filter")]
        public void SetUseFilter(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_6, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_use_filter");

        [GodotMethod("get_use_filter")]
        public bool GetUseFilter()
        {
            return NativeCalls.godot_icall_0_3(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_spacing");

        [GodotMethod("set_spacing")]
        public void SetSpacing(int type, int value)
        {
            NativeCalls.godot_icall_2_32(method_bind_8, Object.GetPtr(this), type, value);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_spacing");

        [GodotMethod("get_spacing")]
        public int GetSpacing(int type)
        {
            return NativeCalls.godot_icall_1_6(method_bind_9, Object.GetPtr(this), type);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_fallback");

        /// <summary>
        /// Adds a fallback font.
        /// </summary>
        [GodotMethod("add_fallback")]
        public void AddFallback(DynamicFontData data)
        {
            NativeCalls.godot_icall_1_24(method_bind_10, Object.GetPtr(this), Object.GetPtr(data));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fallback");

        /// <summary>
        /// Sets the fallback font at index [code]idx[/code].
        /// </summary>
        [GodotMethod("set_fallback")]
        public void SetFallback(int idx, DynamicFontData data)
        {
            NativeCalls.godot_icall_2_67(method_bind_11, Object.GetPtr(this), idx, Object.GetPtr(data));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fallback");

        /// <summary>
        /// Returns the fallback font at index [code]idx[/code].
        /// </summary>
        [GodotMethod("get_fallback")]
        public DynamicFontData GetFallback(int idx)
        {
            return NativeCalls.godot_icall_1_305(method_bind_12, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_fallback");

        /// <summary>
        /// Removes the fallback font at index [code]idx[/code].
        /// </summary>
        [GodotMethod("remove_fallback")]
        public void RemoveFallback(int idx)
        {
            NativeCalls.godot_icall_1_0(method_bind_13, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fallback_count");

        /// <summary>
        /// Returns the number of fallback fonts.
        /// </summary>
        [GodotMethod("get_fallback_count")]
        public int GetFallbackCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_14, Object.GetPtr(this));
        }
    }
}
