using System;

namespace Godot
{
    /// <summary>
    /// Resource for environment nodes (like [WorldEnvironment]) that define multiple environment operations (such as background [Sky] or [Color], ambient light, fog, depth-of-field...).	These parameters affect the final render of the scene. The order of these operations is:
    /// - DOF Blur
    /// - Motion Blur
    /// - Bloom
    /// - Tonemap (auto exposure)
    /// - Adjustments
    /// </summary>
    public class Environment : Resource
    {
        public enum SSAOBlur
        {
            Disabled = 0,
            Blur1x1 = 1,
            Blur2x2 = 2,
            Blur3x3 = 3
        }

        public enum ToneMapper
        {
            Linear = 0,
            Reinhardt = 1,
            Filmic = 2,
            Aces = 3
        }

        public enum GlowBlendModeEnum
        {
            Additive = 0,
            Screen = 1,
            Softlight = 2,
            Replace = 3
        }

        public enum BGMode
        {
            Keep = 5,
            ClearColor = 0,
            Color = 1,
            Sky = 2,
            ColorSky = 3,
            Canvas = 4,
            Max = 6
        }

        public enum SSAOQuality
        {
            Low = 0,
            Medium = 1,
            High = 2
        }

        public enum DOFBlurQuality
        {
            Low = 0,
            Medium = 1,
            High = 2
        }


        /// <summary>
        /// Defines the mode of background.
        /// </summary>
        public Environment.BGMode BackgroundMode
        {
            get
            {
                return GetBackground();
            }
            set
            {
                SetBackground(value);
            }
        }

        /// <summary>
        /// [Sky] resource defined as background.
        /// </summary>
        public Sky BackgroundSky
        {
            get
            {
                return GetSky();
            }
            set
            {
                SetSky(value);
            }
        }

        /// <summary>
        /// [Sky] resource's custom field of view.
        /// </summary>
        public float BackgroundSkyCustomFov
        {
            get
            {
                return GetSkyCustomFov();
            }
            set
            {
                SetSkyCustomFov(value);
            }
        }

        /// <summary>
        /// Color displayed for clear areas of the scene (if using Custom color or Color+Sky background modes).
        /// </summary>
        public Color BackgroundColor
        {
            get
            {
                return GetBgColor();
            }
            set
            {
                SetBgColor(value);
            }
        }

        /// <summary>
        /// Power of light emitted by the background.
        /// </summary>
        public float BackgroundEnergy
        {
            get
            {
                return GetBgEnergy();
            }
            set
            {
                SetBgEnergy(value);
            }
        }

        /// <summary>
        /// Maximum layer id (if using Layer background mode).
        /// </summary>
        public int BackgroundCanvasMaxLayer
        {
            get
            {
                return GetCanvasMaxLayer();
            }
            set
            {
                SetCanvasMaxLayer(value);
            }
        }

        /// <summary>
        /// [Color] of the ambient light.
        /// </summary>
        public Color AmbientLightColor
        {
            get
            {
                return GetAmbientLightColor();
            }
            set
            {
                SetAmbientLightColor(value);
            }
        }

        /// <summary>
        /// Energy of the ambient light. The higher the value, the stronger the light.
        /// </summary>
        public float AmbientLightEnergy
        {
            get
            {
                return GetAmbientLightEnergy();
            }
            set
            {
                SetAmbientLightEnergy(value);
            }
        }

        /// <summary>
        /// Defines the amount of light that the sky brings on the scene. A value of 0 means that the sky's light emission has no effect on the scene illumination, thus all ambient illumination is provided by the ambient light. On the contrary, a value of 1 means that all the light that affects the scene is provided by the sky, thus the ambient light parameter has no effect on the scene.
        /// </summary>
        public float AmbientLightSkyContribution
        {
            get
            {
                return GetAmbientLightSkyContribution();
            }
            set
            {
                SetAmbientLightSkyContribution(value);
            }
        }

        /// <summary>
        /// Enables the fog. Needs fog_height_enabled and/or for_depth_enabled to actually display fog.
        /// </summary>
        public bool FogEnabled
        {
            get
            {
                return IsFogEnabled();
            }
            set
            {
                SetFogEnabled(value);
            }
        }

        /// <summary>
        /// Fog's [Color].
        /// </summary>
        public Color FogColor
        {
            get
            {
                return GetFogColor();
            }
            set
            {
                SetFogColor(value);
            }
        }

        /// <summary>
        /// Sun [Color].
        /// </summary>
        public Color FogSunColor
        {
            get
            {
                return GetFogSunColor();
            }
            set
            {
                SetFogSunColor(value);
            }
        }

        /// <summary>
        /// Amount of sun that affects the fog rendering.
        /// </summary>
        public float FogSunAmount
        {
            get
            {
                return GetFogSunAmount();
            }
            set
            {
                SetFogSunAmount(value);
            }
        }

        /// <summary>
        /// Enables the fog depth.
        /// </summary>
        public bool FogDepthEnabled
        {
            get
            {
                return IsFogDepthEnabled();
            }
            set
            {
                SetFogDepthEnabled(value);
            }
        }

        /// <summary>
        /// Fog's depth starting distance from the camera.
        /// </summary>
        public float FogDepthBegin
        {
            get
            {
                return GetFogDepthBegin();
            }
            set
            {
                SetFogDepthBegin(value);
            }
        }

        /// <summary>
        /// Value defining the fog depth intensity.
        /// </summary>
        public float FogDepthCurve
        {
            get
            {
                return GetFogDepthCurve();
            }
            set
            {
                SetFogDepthCurve(value);
            }
        }

        /// <summary>
        /// Enables fog's light transmission. If enabled, lets reflections light to be transmitted by the fog.
        /// </summary>
        public bool FogTransmitEnabled
        {
            get
            {
                return IsFogTransmitEnabled();
            }
            set
            {
                SetFogTransmitEnabled(value);
            }
        }

        /// <summary>
        /// Amount of light that the fog transmits.
        /// </summary>
        public float FogTransmitCurve
        {
            get
            {
                return GetFogTransmitCurve();
            }
            set
            {
                SetFogTransmitCurve(value);
            }
        }

        /// <summary>
        /// Enables the fog height.
        /// </summary>
        public bool FogHeightEnabled
        {
            get
            {
                return IsFogHeightEnabled();
            }
            set
            {
                SetFogHeightEnabled(value);
            }
        }

        /// <summary>
        /// Minimum height of fog.
        /// </summary>
        public float FogHeightMin
        {
            get
            {
                return GetFogHeightMin();
            }
            set
            {
                SetFogHeightMin(value);
            }
        }

        /// <summary>
        /// Maximum height of fog.
        /// </summary>
        public float FogHeightMax
        {
            get
            {
                return GetFogHeightMax();
            }
            set
            {
                SetFogHeightMax(value);
            }
        }

        /// <summary>
        /// Value defining the fog height intensity.
        /// </summary>
        public float FogHeightCurve
        {
            get
            {
                return GetFogHeightCurve();
            }
            set
            {
                SetFogHeightCurve(value);
            }
        }

        /// <summary>
        /// Tonemapping mode.
        /// </summary>
        public Environment.ToneMapper TonemapMode
        {
            get
            {
                return GetTonemapper();
            }
            set
            {
                SetTonemapper(value);
            }
        }

        /// <summary>
        /// Default exposure for tonemap.
        /// </summary>
        public float TonemapExposure
        {
            get
            {
                return GetTonemapExposure();
            }
            set
            {
                SetTonemapExposure(value);
            }
        }

        /// <summary>
        /// White reference value for tonemap.
        /// </summary>
        public float TonemapWhite
        {
            get
            {
                return GetTonemapWhite();
            }
            set
            {
                SetTonemapWhite(value);
            }
        }

        /// <summary>
        /// Enables the tonemapping auto exposure mode of the scene renderer. If activated, the renderer will automatically determine the exposure setting to adapt to the illumination of the scene and the observed light.
        /// </summary>
        public bool AutoExposureEnabled
        {
            get
            {
                return GetTonemapAutoExposure();
            }
            set
            {
                SetTonemapAutoExposure(value);
            }
        }

        /// <summary>
        /// Scale of the auto exposure effect. Affects the intensity of auto exposure.
        /// </summary>
        public float AutoExposureScale
        {
            get
            {
                return GetTonemapAutoExposureGrey();
            }
            set
            {
                SetTonemapAutoExposureGrey(value);
            }
        }

        /// <summary>
        /// Minimum luminance value for the auto exposure.
        /// </summary>
        public float AutoExposureMinLuma
        {
            get
            {
                return GetTonemapAutoExposureMin();
            }
            set
            {
                SetTonemapAutoExposureMin(value);
            }
        }

        /// <summary>
        /// Maximum luminance value for the auto exposure.
        /// </summary>
        public float AutoExposureMaxLuma
        {
            get
            {
                return GetTonemapAutoExposureMax();
            }
            set
            {
                SetTonemapAutoExposureMax(value);
            }
        }

        /// <summary>
        /// Speed of the auto exposure effect. Affects the time needed for the camera to perform auto exposure.
        /// </summary>
        public float AutoExposureSpeed
        {
            get
            {
                return GetTonemapAutoExposureSpeed();
            }
            set
            {
                SetTonemapAutoExposureSpeed(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool SsReflectionsEnabled
        {
            get
            {
                return IsSsrEnabled();
            }
            set
            {
                SetSsrEnabled(value);
            }
        }

        /// <summary>
        /// </summary>
        public int SsReflectionsMaxSteps
        {
            get
            {
                return GetSsrMaxSteps();
            }
            set
            {
                SetSsrMaxSteps(value);
            }
        }

        /// <summary>
        /// </summary>
        public float SsReflectionsFadeIn
        {
            get
            {
                return GetSsrFadeIn();
            }
            set
            {
                SetSsrFadeIn(value);
            }
        }

        /// <summary>
        /// </summary>
        public float SsReflectionsFadeOut
        {
            get
            {
                return GetSsrFadeOut();
            }
            set
            {
                SetSsrFadeOut(value);
            }
        }

        /// <summary>
        /// </summary>
        public float SsReflectionsDepthTolerance
        {
            get
            {
                return GetSsrDepthTolerance();
            }
            set
            {
                SetSsrDepthTolerance(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool SsReflectionsRoughness
        {
            get
            {
                return IsSsrRough();
            }
            set
            {
                SetSsrRough(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool SsaoEnabled
        {
            get
            {
                return IsSsaoEnabled();
            }
            set
            {
                SetSsaoEnabled(value);
            }
        }

        /// <summary>
        /// </summary>
        public float SsaoRadius
        {
            get
            {
                return GetSsaoRadius();
            }
            set
            {
                SetSsaoRadius(value);
            }
        }

        /// <summary>
        /// </summary>
        public float SsaoIntensity
        {
            get
            {
                return GetSsaoIntensity();
            }
            set
            {
                SetSsaoIntensity(value);
            }
        }

        /// <summary>
        /// </summary>
        public float SsaoRadius2
        {
            get
            {
                return GetSsaoRadius2();
            }
            set
            {
                SetSsaoRadius2(value);
            }
        }

        /// <summary>
        /// </summary>
        public float SsaoIntensity2
        {
            get
            {
                return GetSsaoIntensity2();
            }
            set
            {
                SetSsaoIntensity2(value);
            }
        }

        /// <summary>
        /// </summary>
        public float SsaoBias
        {
            get
            {
                return GetSsaoBias();
            }
            set
            {
                SetSsaoBias(value);
            }
        }

        /// <summary>
        /// </summary>
        public float SsaoLightAffect
        {
            get
            {
                return GetSsaoDirectLightAffect();
            }
            set
            {
                SetSsaoDirectLightAffect(value);
            }
        }

        /// <summary>
        /// </summary>
        public Color SsaoColor
        {
            get
            {
                return GetSsaoColor();
            }
            set
            {
                SetSsaoColor(value);
            }
        }

        /// <summary>
        /// </summary>
        public Environment.SSAOQuality SsaoQuality
        {
            get
            {
                return GetSsaoQuality();
            }
            set
            {
                SetSsaoQuality(value);
            }
        }

        /// <summary>
        /// </summary>
        public Environment.SSAOBlur SsaoBlur
        {
            get
            {
                return GetSsaoBlur();
            }
            set
            {
                SetSsaoBlur(value);
            }
        }

        /// <summary>
        /// </summary>
        public float SsaoEdgeSharpness
        {
            get
            {
                return GetSsaoEdgeSharpness();
            }
            set
            {
                SetSsaoEdgeSharpness(value);
            }
        }

        /// <summary>
        /// Enables the far blur effect.
        /// </summary>
        public bool DofBlurFarEnabled
        {
            get
            {
                return IsDofBlurFarEnabled();
            }
            set
            {
                SetDofBlurFarEnabled(value);
            }
        }

        /// <summary>
        /// Distance from the camera where the far blur effect affects the rendering.
        /// </summary>
        public float DofBlurFarDistance
        {
            get
            {
                return GetDofBlurFarDistance();
            }
            set
            {
                SetDofBlurFarDistance(value);
            }
        }

        /// <summary>
        /// Transition between no-blur area and far blur.
        /// </summary>
        public float DofBlurFarTransition
        {
            get
            {
                return GetDofBlurFarTransition();
            }
            set
            {
                SetDofBlurFarTransition(value);
            }
        }

        /// <summary>
        /// Amount of far blur.
        /// </summary>
        public float DofBlurFarAmount
        {
            get
            {
                return GetDofBlurFarAmount();
            }
            set
            {
                SetDofBlurFarAmount(value);
            }
        }

        /// <summary>
        /// Quality of the far blur quality.
        /// </summary>
        public Environment.DOFBlurQuality DofBlurFarQuality
        {
            get
            {
                return GetDofBlurFarQuality();
            }
            set
            {
                SetDofBlurFarQuality(value);
            }
        }

        /// <summary>
        /// Enables the near blur effect.
        /// </summary>
        public bool DofBlurNearEnabled
        {
            get
            {
                return IsDofBlurNearEnabled();
            }
            set
            {
                SetDofBlurNearEnabled(value);
            }
        }

        /// <summary>
        /// Distance from the camera where the near blur effect affects the rendering.
        /// </summary>
        public float DofBlurNearDistance
        {
            get
            {
                return GetDofBlurNearDistance();
            }
            set
            {
                SetDofBlurNearDistance(value);
            }
        }

        /// <summary>
        /// Transition between near blur and no-blur area.
        /// </summary>
        public float DofBlurNearTransition
        {
            get
            {
                return GetDofBlurNearTransition();
            }
            set
            {
                SetDofBlurNearTransition(value);
            }
        }

        /// <summary>
        /// Amount of near blur.
        /// </summary>
        public float DofBlurNearAmount
        {
            get
            {
                return GetDofBlurNearAmount();
            }
            set
            {
                SetDofBlurNearAmount(value);
            }
        }

        /// <summary>
        /// Quality of the near blur quality.
        /// </summary>
        public Environment.DOFBlurQuality DofBlurNearQuality
        {
            get
            {
                return GetDofBlurNearQuality();
            }
            set
            {
                SetDofBlurNearQuality(value);
            }
        }

        /// <summary>
        /// Enables glow rendering.
        /// </summary>
        public bool GlowEnabled
        {
            get
            {
                return IsGlowEnabled();
            }
            set
            {
                SetGlowEnabled(value);
            }
        }

        /// <summary>
        /// First level of glow (most local).
        /// </summary>
        public bool GlowLevels__1
        {
            get
            {
                return IsGlowLevelEnabled(0);
            }
            set
            {
                SetGlowLevel(0, value);
            }
        }

        /// <summary>
        /// Second level of glow.
        /// </summary>
        public bool GlowLevels__2
        {
            get
            {
                return IsGlowLevelEnabled(1);
            }
            set
            {
                SetGlowLevel(1, value);
            }
        }

        /// <summary>
        /// Third level of glow.
        /// </summary>
        public bool GlowLevels__3
        {
            get
            {
                return IsGlowLevelEnabled(2);
            }
            set
            {
                SetGlowLevel(2, value);
            }
        }

        /// <summary>
        /// Fourth level of glow.
        /// </summary>
        public bool GlowLevels__4
        {
            get
            {
                return IsGlowLevelEnabled(3);
            }
            set
            {
                SetGlowLevel(3, value);
            }
        }

        /// <summary>
        /// Fifth level of glow.
        /// </summary>
        public bool GlowLevels__5
        {
            get
            {
                return IsGlowLevelEnabled(4);
            }
            set
            {
                SetGlowLevel(4, value);
            }
        }

        /// <summary>
        /// Sixth level of glow.
        /// </summary>
        public bool GlowLevels__6
        {
            get
            {
                return IsGlowLevelEnabled(5);
            }
            set
            {
                SetGlowLevel(5, value);
            }
        }

        /// <summary>
        /// Seventh level of glow (most global).
        /// </summary>
        public bool GlowLevels__7
        {
            get
            {
                return IsGlowLevelEnabled(6);
            }
            set
            {
                SetGlowLevel(6, value);
            }
        }

        /// <summary>
        /// Glow intensity.
        /// </summary>
        public float GlowIntensity
        {
            get
            {
                return GetGlowIntensity();
            }
            set
            {
                SetGlowIntensity(value);
            }
        }

        /// <summary>
        /// Glow strength.
        /// </summary>
        public float GlowStrength
        {
            get
            {
                return GetGlowStrength();
            }
            set
            {
                SetGlowStrength(value);
            }
        }

        /// <summary>
        /// Bloom value (global glow).
        /// </summary>
        public float GlowBloom
        {
            get
            {
                return GetGlowBloom();
            }
            set
            {
                SetGlowBloom(value);
            }
        }

        /// <summary>
        /// Glow blending mode.
        /// </summary>
        public Environment.GlowBlendModeEnum GlowBlendMode
        {
            get
            {
                return GetGlowBlendMode();
            }
            set
            {
                SetGlowBlendMode(value);
            }
        }

        /// <summary>
        /// Bleed threshold of the HDR glow.
        /// </summary>
        public float GlowHdrThreshold
        {
            get
            {
                return GetGlowHdrBleedThreshold();
            }
            set
            {
                SetGlowHdrBleedThreshold(value);
            }
        }

        /// <summary>
        /// Bleed scale of the HDR glow.
        /// </summary>
        public float GlowHdrScale
        {
            get
            {
                return GetGlowHdrBleedScale();
            }
            set
            {
                SetGlowHdrBleedScale(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool GlowBicubicUpscale
        {
            get
            {
                return IsGlowBicubicUpscaleEnabled();
            }
            set
            {
                SetGlowBicubicUpscale(value);
            }
        }

        /// <summary>
        /// Enables the adjustment_* options provided by this resource. If false, adjustments modifications will have no effect on the rendered scene.
        /// </summary>
        public bool AdjustmentEnabled
        {
            get
            {
                return IsAdjustmentEnabled();
            }
            set
            {
                SetAdjustmentEnable(value);
            }
        }

        /// <summary>
        /// Global brightness value of the rendered scene (default value is 1).
        /// </summary>
        public float AdjustmentBrightness
        {
            get
            {
                return GetAdjustmentBrightness();
            }
            set
            {
                SetAdjustmentBrightness(value);
            }
        }

        /// <summary>
        /// Global contrast value of the rendered scene (default value is 1).
        /// </summary>
        public float AdjustmentContrast
        {
            get
            {
                return GetAdjustmentContrast();
            }
            set
            {
                SetAdjustmentContrast(value);
            }
        }

        /// <summary>
        /// Global color saturation value of the rendered scene (default value is 1).
        /// </summary>
        public float AdjustmentSaturation
        {
            get
            {
                return GetAdjustmentSaturation();
            }
            set
            {
                SetAdjustmentSaturation(value);
            }
        }

        /// <summary>
        /// Applies the provided [Texture] resource to affect the global color aspect of the rendered scene.
        /// </summary>
        public Texture AdjustmentColorCorrection
        {
            get
            {
                return GetAdjustmentColorCorrection();
            }
            set
            {
                SetAdjustmentColorCorrection(value);
            }
        }


        private const string nativeName = "Environment";

        public Environment() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Environment_Ctor(this);
        }

        internal Environment(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_background");

        [GodotMethod("set_background")]
        public void SetBackground(Environment.BGMode mode)
        {
            NativeCalls.godot_icall_1_346(method_bind_0, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_sky");

        [GodotMethod("set_sky")]
        public void SetSky(Sky sky)
        {
            NativeCalls.godot_icall_1_24(method_bind_1, Object.GetPtr(this), Object.GetPtr(sky));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_sky_custom_fov");

        [GodotMethod("set_sky_custom_fov")]
        public void SetSkyCustomFov(float scale)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), scale);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bg_color");

        [GodotMethod("set_bg_color")]
        public void SetBgColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_3, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bg_energy");

        [GodotMethod("set_bg_energy")]
        public void SetBgEnergy(float energy)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), energy);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_canvas_max_layer");

        [GodotMethod("set_canvas_max_layer")]
        public void SetCanvasMaxLayer(int layer)
        {
            NativeCalls.godot_icall_1_0(method_bind_5, Object.GetPtr(this), layer);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ambient_light_color");

        [GodotMethod("set_ambient_light_color")]
        public void SetAmbientLightColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_6, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ambient_light_energy");

        [GodotMethod("set_ambient_light_energy")]
        public void SetAmbientLightEnergy(float energy)
        {
            NativeCalls.godot_icall_1_10(method_bind_7, Object.GetPtr(this), energy);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ambient_light_sky_contribution");

        [GodotMethod("set_ambient_light_sky_contribution")]
        public void SetAmbientLightSkyContribution(float energy)
        {
            NativeCalls.godot_icall_1_10(method_bind_8, Object.GetPtr(this), energy);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_background");

        [GodotMethod("get_background")]
        public Environment.BGMode GetBackground()
        {
            return NativeCalls.godot_icall_0_347(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_sky");

        [GodotMethod("get_sky")]
        public Sky GetSky()
        {
            return NativeCalls.godot_icall_0_348(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_sky_custom_fov");

        [GodotMethod("get_sky_custom_fov")]
        public float GetSkyCustomFov()
        {
            return NativeCalls.godot_icall_0_9(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bg_color");

        [GodotMethod("get_bg_color")]
        public Color GetBgColor()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_12, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bg_energy");

        [GodotMethod("get_bg_energy")]
        public float GetBgEnergy()
        {
            return NativeCalls.godot_icall_0_9(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_canvas_max_layer");

        [GodotMethod("get_canvas_max_layer")]
        public int GetCanvasMaxLayer()
        {
            return NativeCalls.godot_icall_0_1(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ambient_light_color");

        [GodotMethod("get_ambient_light_color")]
        public Color GetAmbientLightColor()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_15, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ambient_light_energy");

        [GodotMethod("get_ambient_light_energy")]
        public float GetAmbientLightEnergy()
        {
            return NativeCalls.godot_icall_0_9(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ambient_light_sky_contribution");

        [GodotMethod("get_ambient_light_sky_contribution")]
        public float GetAmbientLightSkyContribution()
        {
            return NativeCalls.godot_icall_0_9(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fog_enabled");

        [GodotMethod("set_fog_enabled")]
        public void SetFogEnabled(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_18, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_fog_enabled");

        [GodotMethod("is_fog_enabled")]
        public bool IsFogEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fog_color");

        [GodotMethod("set_fog_color")]
        public void SetFogColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_20, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fog_color");

        [GodotMethod("get_fog_color")]
        public Color GetFogColor()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_21, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fog_sun_color");

        [GodotMethod("set_fog_sun_color")]
        public void SetFogSunColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_22, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fog_sun_color");

        [GodotMethod("get_fog_sun_color")]
        public Color GetFogSunColor()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_23, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fog_sun_amount");

        [GodotMethod("set_fog_sun_amount")]
        public void SetFogSunAmount(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_24, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fog_sun_amount");

        [GodotMethod("get_fog_sun_amount")]
        public float GetFogSunAmount()
        {
            return NativeCalls.godot_icall_0_9(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fog_depth_enabled");

        [GodotMethod("set_fog_depth_enabled")]
        public void SetFogDepthEnabled(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_26, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_fog_depth_enabled");

        [GodotMethod("is_fog_depth_enabled")]
        public bool IsFogDepthEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_27, Object.GetPtr(this));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fog_depth_begin");

        [GodotMethod("set_fog_depth_begin")]
        public void SetFogDepthBegin(float distance)
        {
            NativeCalls.godot_icall_1_10(method_bind_28, Object.GetPtr(this), distance);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fog_depth_begin");

        [GodotMethod("get_fog_depth_begin")]
        public float GetFogDepthBegin()
        {
            return NativeCalls.godot_icall_0_9(method_bind_29, Object.GetPtr(this));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fog_depth_curve");

        [GodotMethod("set_fog_depth_curve")]
        public void SetFogDepthCurve(float curve)
        {
            NativeCalls.godot_icall_1_10(method_bind_30, Object.GetPtr(this), curve);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fog_depth_curve");

        [GodotMethod("get_fog_depth_curve")]
        public float GetFogDepthCurve()
        {
            return NativeCalls.godot_icall_0_9(method_bind_31, Object.GetPtr(this));
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fog_transmit_enabled");

        [GodotMethod("set_fog_transmit_enabled")]
        public void SetFogTransmitEnabled(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_32, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_fog_transmit_enabled");

        [GodotMethod("is_fog_transmit_enabled")]
        public bool IsFogTransmitEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_33, Object.GetPtr(this));
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fog_transmit_curve");

        [GodotMethod("set_fog_transmit_curve")]
        public void SetFogTransmitCurve(float curve)
        {
            NativeCalls.godot_icall_1_10(method_bind_34, Object.GetPtr(this), curve);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fog_transmit_curve");

        [GodotMethod("get_fog_transmit_curve")]
        public float GetFogTransmitCurve()
        {
            return NativeCalls.godot_icall_0_9(method_bind_35, Object.GetPtr(this));
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fog_height_enabled");

        [GodotMethod("set_fog_height_enabled")]
        public void SetFogHeightEnabled(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_36, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_fog_height_enabled");

        [GodotMethod("is_fog_height_enabled")]
        public bool IsFogHeightEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_37, Object.GetPtr(this));
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fog_height_min");

        [GodotMethod("set_fog_height_min")]
        public void SetFogHeightMin(float height)
        {
            NativeCalls.godot_icall_1_10(method_bind_38, Object.GetPtr(this), height);
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fog_height_min");

        [GodotMethod("get_fog_height_min")]
        public float GetFogHeightMin()
        {
            return NativeCalls.godot_icall_0_9(method_bind_39, Object.GetPtr(this));
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fog_height_max");

        [GodotMethod("set_fog_height_max")]
        public void SetFogHeightMax(float height)
        {
            NativeCalls.godot_icall_1_10(method_bind_40, Object.GetPtr(this), height);
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fog_height_max");

        [GodotMethod("get_fog_height_max")]
        public float GetFogHeightMax()
        {
            return NativeCalls.godot_icall_0_9(method_bind_41, Object.GetPtr(this));
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fog_height_curve");

        [GodotMethod("set_fog_height_curve")]
        public void SetFogHeightCurve(float curve)
        {
            NativeCalls.godot_icall_1_10(method_bind_42, Object.GetPtr(this), curve);
        }

        private IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fog_height_curve");

        [GodotMethod("get_fog_height_curve")]
        public float GetFogHeightCurve()
        {
            return NativeCalls.godot_icall_0_9(method_bind_43, Object.GetPtr(this));
        }

        private IntPtr method_bind_44 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tonemapper");

        [GodotMethod("set_tonemapper")]
        public void SetTonemapper(Environment.ToneMapper mode)
        {
            NativeCalls.godot_icall_1_349(method_bind_44, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_45 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tonemapper");

        [GodotMethod("get_tonemapper")]
        public Environment.ToneMapper GetTonemapper()
        {
            return NativeCalls.godot_icall_0_350(method_bind_45, Object.GetPtr(this));
        }

        private IntPtr method_bind_46 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tonemap_exposure");

        [GodotMethod("set_tonemap_exposure")]
        public void SetTonemapExposure(float exposure)
        {
            NativeCalls.godot_icall_1_10(method_bind_46, Object.GetPtr(this), exposure);
        }

        private IntPtr method_bind_47 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tonemap_exposure");

        [GodotMethod("get_tonemap_exposure")]
        public float GetTonemapExposure()
        {
            return NativeCalls.godot_icall_0_9(method_bind_47, Object.GetPtr(this));
        }

        private IntPtr method_bind_48 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tonemap_white");

        [GodotMethod("set_tonemap_white")]
        public void SetTonemapWhite(float white)
        {
            NativeCalls.godot_icall_1_10(method_bind_48, Object.GetPtr(this), white);
        }

        private IntPtr method_bind_49 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tonemap_white");

        [GodotMethod("get_tonemap_white")]
        public float GetTonemapWhite()
        {
            return NativeCalls.godot_icall_0_9(method_bind_49, Object.GetPtr(this));
        }

        private IntPtr method_bind_50 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tonemap_auto_exposure");

        [GodotMethod("set_tonemap_auto_exposure")]
        public void SetTonemapAutoExposure(bool autoExposure)
        {
            NativeCalls.godot_icall_1_11(method_bind_50, Object.GetPtr(this), autoExposure);
        }

        private IntPtr method_bind_51 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tonemap_auto_exposure");

        [GodotMethod("get_tonemap_auto_exposure")]
        public bool GetTonemapAutoExposure()
        {
            return NativeCalls.godot_icall_0_3(method_bind_51, Object.GetPtr(this));
        }

        private IntPtr method_bind_52 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tonemap_auto_exposure_max");

        [GodotMethod("set_tonemap_auto_exposure_max")]
        public void SetTonemapAutoExposureMax(float exposureMax)
        {
            NativeCalls.godot_icall_1_10(method_bind_52, Object.GetPtr(this), exposureMax);
        }

        private IntPtr method_bind_53 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tonemap_auto_exposure_max");

        [GodotMethod("get_tonemap_auto_exposure_max")]
        public float GetTonemapAutoExposureMax()
        {
            return NativeCalls.godot_icall_0_9(method_bind_53, Object.GetPtr(this));
        }

        private IntPtr method_bind_54 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tonemap_auto_exposure_min");

        [GodotMethod("set_tonemap_auto_exposure_min")]
        public void SetTonemapAutoExposureMin(float exposureMin)
        {
            NativeCalls.godot_icall_1_10(method_bind_54, Object.GetPtr(this), exposureMin);
        }

        private IntPtr method_bind_55 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tonemap_auto_exposure_min");

        [GodotMethod("get_tonemap_auto_exposure_min")]
        public float GetTonemapAutoExposureMin()
        {
            return NativeCalls.godot_icall_0_9(method_bind_55, Object.GetPtr(this));
        }

        private IntPtr method_bind_56 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tonemap_auto_exposure_speed");

        [GodotMethod("set_tonemap_auto_exposure_speed")]
        public void SetTonemapAutoExposureSpeed(float exposureSpeed)
        {
            NativeCalls.godot_icall_1_10(method_bind_56, Object.GetPtr(this), exposureSpeed);
        }

        private IntPtr method_bind_57 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tonemap_auto_exposure_speed");

        [GodotMethod("get_tonemap_auto_exposure_speed")]
        public float GetTonemapAutoExposureSpeed()
        {
            return NativeCalls.godot_icall_0_9(method_bind_57, Object.GetPtr(this));
        }

        private IntPtr method_bind_58 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tonemap_auto_exposure_grey");

        [GodotMethod("set_tonemap_auto_exposure_grey")]
        public void SetTonemapAutoExposureGrey(float exposureGrey)
        {
            NativeCalls.godot_icall_1_10(method_bind_58, Object.GetPtr(this), exposureGrey);
        }

        private IntPtr method_bind_59 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tonemap_auto_exposure_grey");

        [GodotMethod("get_tonemap_auto_exposure_grey")]
        public float GetTonemapAutoExposureGrey()
        {
            return NativeCalls.godot_icall_0_9(method_bind_59, Object.GetPtr(this));
        }

        private IntPtr method_bind_60 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ssr_enabled");

        [GodotMethod("set_ssr_enabled")]
        public void SetSsrEnabled(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_60, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_61 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_ssr_enabled");

        [GodotMethod("is_ssr_enabled")]
        public bool IsSsrEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_61, Object.GetPtr(this));
        }

        private IntPtr method_bind_62 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ssr_max_steps");

        [GodotMethod("set_ssr_max_steps")]
        public void SetSsrMaxSteps(int maxSteps)
        {
            NativeCalls.godot_icall_1_0(method_bind_62, Object.GetPtr(this), maxSteps);
        }

        private IntPtr method_bind_63 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ssr_max_steps");

        [GodotMethod("get_ssr_max_steps")]
        public int GetSsrMaxSteps()
        {
            return NativeCalls.godot_icall_0_1(method_bind_63, Object.GetPtr(this));
        }

        private IntPtr method_bind_64 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ssr_fade_in");

        [GodotMethod("set_ssr_fade_in")]
        public void SetSsrFadeIn(float fadeIn)
        {
            NativeCalls.godot_icall_1_10(method_bind_64, Object.GetPtr(this), fadeIn);
        }

        private IntPtr method_bind_65 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ssr_fade_in");

        [GodotMethod("get_ssr_fade_in")]
        public float GetSsrFadeIn()
        {
            return NativeCalls.godot_icall_0_9(method_bind_65, Object.GetPtr(this));
        }

        private IntPtr method_bind_66 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ssr_fade_out");

        [GodotMethod("set_ssr_fade_out")]
        public void SetSsrFadeOut(float fadeOut)
        {
            NativeCalls.godot_icall_1_10(method_bind_66, Object.GetPtr(this), fadeOut);
        }

        private IntPtr method_bind_67 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ssr_fade_out");

        [GodotMethod("get_ssr_fade_out")]
        public float GetSsrFadeOut()
        {
            return NativeCalls.godot_icall_0_9(method_bind_67, Object.GetPtr(this));
        }

        private IntPtr method_bind_68 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ssr_depth_tolerance");

        [GodotMethod("set_ssr_depth_tolerance")]
        public void SetSsrDepthTolerance(float depthTolerance)
        {
            NativeCalls.godot_icall_1_10(method_bind_68, Object.GetPtr(this), depthTolerance);
        }

        private IntPtr method_bind_69 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ssr_depth_tolerance");

        [GodotMethod("get_ssr_depth_tolerance")]
        public float GetSsrDepthTolerance()
        {
            return NativeCalls.godot_icall_0_9(method_bind_69, Object.GetPtr(this));
        }

        private IntPtr method_bind_70 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ssr_rough");

        [GodotMethod("set_ssr_rough")]
        public void SetSsrRough(bool rough)
        {
            NativeCalls.godot_icall_1_11(method_bind_70, Object.GetPtr(this), rough);
        }

        private IntPtr method_bind_71 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_ssr_rough");

        [GodotMethod("is_ssr_rough")]
        public bool IsSsrRough()
        {
            return NativeCalls.godot_icall_0_3(method_bind_71, Object.GetPtr(this));
        }

        private IntPtr method_bind_72 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ssao_enabled");

        [GodotMethod("set_ssao_enabled")]
        public void SetSsaoEnabled(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_72, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_73 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_ssao_enabled");

        [GodotMethod("is_ssao_enabled")]
        public bool IsSsaoEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_73, Object.GetPtr(this));
        }

        private IntPtr method_bind_74 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ssao_radius");

        [GodotMethod("set_ssao_radius")]
        public void SetSsaoRadius(float radius)
        {
            NativeCalls.godot_icall_1_10(method_bind_74, Object.GetPtr(this), radius);
        }

        private IntPtr method_bind_75 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ssao_radius");

        [GodotMethod("get_ssao_radius")]
        public float GetSsaoRadius()
        {
            return NativeCalls.godot_icall_0_9(method_bind_75, Object.GetPtr(this));
        }

        private IntPtr method_bind_76 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ssao_intensity");

        [GodotMethod("set_ssao_intensity")]
        public void SetSsaoIntensity(float intensity)
        {
            NativeCalls.godot_icall_1_10(method_bind_76, Object.GetPtr(this), intensity);
        }

        private IntPtr method_bind_77 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ssao_intensity");

        [GodotMethod("get_ssao_intensity")]
        public float GetSsaoIntensity()
        {
            return NativeCalls.godot_icall_0_9(method_bind_77, Object.GetPtr(this));
        }

        private IntPtr method_bind_78 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ssao_radius2");

        [GodotMethod("set_ssao_radius2")]
        public void SetSsaoRadius2(float radius)
        {
            NativeCalls.godot_icall_1_10(method_bind_78, Object.GetPtr(this), radius);
        }

        private IntPtr method_bind_79 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ssao_radius2");

        [GodotMethod("get_ssao_radius2")]
        public float GetSsaoRadius2()
        {
            return NativeCalls.godot_icall_0_9(method_bind_79, Object.GetPtr(this));
        }

        private IntPtr method_bind_80 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ssao_intensity2");

        [GodotMethod("set_ssao_intensity2")]
        public void SetSsaoIntensity2(float intensity)
        {
            NativeCalls.godot_icall_1_10(method_bind_80, Object.GetPtr(this), intensity);
        }

        private IntPtr method_bind_81 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ssao_intensity2");

        [GodotMethod("get_ssao_intensity2")]
        public float GetSsaoIntensity2()
        {
            return NativeCalls.godot_icall_0_9(method_bind_81, Object.GetPtr(this));
        }

        private IntPtr method_bind_82 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ssao_bias");

        [GodotMethod("set_ssao_bias")]
        public void SetSsaoBias(float bias)
        {
            NativeCalls.godot_icall_1_10(method_bind_82, Object.GetPtr(this), bias);
        }

        private IntPtr method_bind_83 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ssao_bias");

        [GodotMethod("get_ssao_bias")]
        public float GetSsaoBias()
        {
            return NativeCalls.godot_icall_0_9(method_bind_83, Object.GetPtr(this));
        }

        private IntPtr method_bind_84 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ssao_direct_light_affect");

        [GodotMethod("set_ssao_direct_light_affect")]
        public void SetSsaoDirectLightAffect(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_84, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_85 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ssao_direct_light_affect");

        [GodotMethod("get_ssao_direct_light_affect")]
        public float GetSsaoDirectLightAffect()
        {
            return NativeCalls.godot_icall_0_9(method_bind_85, Object.GetPtr(this));
        }

        private IntPtr method_bind_86 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ssao_color");

        [GodotMethod("set_ssao_color")]
        public void SetSsaoColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_86, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_87 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ssao_color");

        [GodotMethod("get_ssao_color")]
        public Color GetSsaoColor()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_87, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_88 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ssao_blur");

        [GodotMethod("set_ssao_blur")]
        public void SetSsaoBlur(Environment.SSAOBlur mode)
        {
            NativeCalls.godot_icall_1_351(method_bind_88, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_89 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ssao_blur");

        [GodotMethod("get_ssao_blur")]
        public Environment.SSAOBlur GetSsaoBlur()
        {
            return NativeCalls.godot_icall_0_352(method_bind_89, Object.GetPtr(this));
        }

        private IntPtr method_bind_90 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ssao_quality");

        [GodotMethod("set_ssao_quality")]
        public void SetSsaoQuality(Environment.SSAOQuality quality)
        {
            NativeCalls.godot_icall_1_353(method_bind_90, Object.GetPtr(this), quality);
        }

        private IntPtr method_bind_91 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ssao_quality");

        [GodotMethod("get_ssao_quality")]
        public Environment.SSAOQuality GetSsaoQuality()
        {
            return NativeCalls.godot_icall_0_354(method_bind_91, Object.GetPtr(this));
        }

        private IntPtr method_bind_92 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ssao_edge_sharpness");

        [GodotMethod("set_ssao_edge_sharpness")]
        public void SetSsaoEdgeSharpness(float edgeSharpness)
        {
            NativeCalls.godot_icall_1_10(method_bind_92, Object.GetPtr(this), edgeSharpness);
        }

        private IntPtr method_bind_93 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ssao_edge_sharpness");

        [GodotMethod("get_ssao_edge_sharpness")]
        public float GetSsaoEdgeSharpness()
        {
            return NativeCalls.godot_icall_0_9(method_bind_93, Object.GetPtr(this));
        }

        private IntPtr method_bind_94 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_dof_blur_far_enabled");

        [GodotMethod("set_dof_blur_far_enabled")]
        public void SetDofBlurFarEnabled(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_94, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_95 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_dof_blur_far_enabled");

        [GodotMethod("is_dof_blur_far_enabled")]
        public bool IsDofBlurFarEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_95, Object.GetPtr(this));
        }

        private IntPtr method_bind_96 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_dof_blur_far_distance");

        [GodotMethod("set_dof_blur_far_distance")]
        public void SetDofBlurFarDistance(float intensity)
        {
            NativeCalls.godot_icall_1_10(method_bind_96, Object.GetPtr(this), intensity);
        }

        private IntPtr method_bind_97 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_dof_blur_far_distance");

        [GodotMethod("get_dof_blur_far_distance")]
        public float GetDofBlurFarDistance()
        {
            return NativeCalls.godot_icall_0_9(method_bind_97, Object.GetPtr(this));
        }

        private IntPtr method_bind_98 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_dof_blur_far_transition");

        [GodotMethod("set_dof_blur_far_transition")]
        public void SetDofBlurFarTransition(float intensity)
        {
            NativeCalls.godot_icall_1_10(method_bind_98, Object.GetPtr(this), intensity);
        }

        private IntPtr method_bind_99 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_dof_blur_far_transition");

        [GodotMethod("get_dof_blur_far_transition")]
        public float GetDofBlurFarTransition()
        {
            return NativeCalls.godot_icall_0_9(method_bind_99, Object.GetPtr(this));
        }

        private IntPtr method_bind_100 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_dof_blur_far_amount");

        [GodotMethod("set_dof_blur_far_amount")]
        public void SetDofBlurFarAmount(float intensity)
        {
            NativeCalls.godot_icall_1_10(method_bind_100, Object.GetPtr(this), intensity);
        }

        private IntPtr method_bind_101 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_dof_blur_far_amount");

        [GodotMethod("get_dof_blur_far_amount")]
        public float GetDofBlurFarAmount()
        {
            return NativeCalls.godot_icall_0_9(method_bind_101, Object.GetPtr(this));
        }

        private IntPtr method_bind_102 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_dof_blur_far_quality");

        [GodotMethod("set_dof_blur_far_quality")]
        public void SetDofBlurFarQuality(Environment.DOFBlurQuality intensity)
        {
            NativeCalls.godot_icall_1_355(method_bind_102, Object.GetPtr(this), intensity);
        }

        private IntPtr method_bind_103 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_dof_blur_far_quality");

        [GodotMethod("get_dof_blur_far_quality")]
        public Environment.DOFBlurQuality GetDofBlurFarQuality()
        {
            return NativeCalls.godot_icall_0_356(method_bind_103, Object.GetPtr(this));
        }

        private IntPtr method_bind_104 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_dof_blur_near_enabled");

        [GodotMethod("set_dof_blur_near_enabled")]
        public void SetDofBlurNearEnabled(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_104, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_105 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_dof_blur_near_enabled");

        [GodotMethod("is_dof_blur_near_enabled")]
        public bool IsDofBlurNearEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_105, Object.GetPtr(this));
        }

        private IntPtr method_bind_106 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_dof_blur_near_distance");

        [GodotMethod("set_dof_blur_near_distance")]
        public void SetDofBlurNearDistance(float intensity)
        {
            NativeCalls.godot_icall_1_10(method_bind_106, Object.GetPtr(this), intensity);
        }

        private IntPtr method_bind_107 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_dof_blur_near_distance");

        [GodotMethod("get_dof_blur_near_distance")]
        public float GetDofBlurNearDistance()
        {
            return NativeCalls.godot_icall_0_9(method_bind_107, Object.GetPtr(this));
        }

        private IntPtr method_bind_108 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_dof_blur_near_transition");

        [GodotMethod("set_dof_blur_near_transition")]
        public void SetDofBlurNearTransition(float intensity)
        {
            NativeCalls.godot_icall_1_10(method_bind_108, Object.GetPtr(this), intensity);
        }

        private IntPtr method_bind_109 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_dof_blur_near_transition");

        [GodotMethod("get_dof_blur_near_transition")]
        public float GetDofBlurNearTransition()
        {
            return NativeCalls.godot_icall_0_9(method_bind_109, Object.GetPtr(this));
        }

        private IntPtr method_bind_110 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_dof_blur_near_amount");

        [GodotMethod("set_dof_blur_near_amount")]
        public void SetDofBlurNearAmount(float intensity)
        {
            NativeCalls.godot_icall_1_10(method_bind_110, Object.GetPtr(this), intensity);
        }

        private IntPtr method_bind_111 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_dof_blur_near_amount");

        [GodotMethod("get_dof_blur_near_amount")]
        public float GetDofBlurNearAmount()
        {
            return NativeCalls.godot_icall_0_9(method_bind_111, Object.GetPtr(this));
        }

        private IntPtr method_bind_112 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_dof_blur_near_quality");

        [GodotMethod("set_dof_blur_near_quality")]
        public void SetDofBlurNearQuality(Environment.DOFBlurQuality level)
        {
            NativeCalls.godot_icall_1_355(method_bind_112, Object.GetPtr(this), level);
        }

        private IntPtr method_bind_113 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_dof_blur_near_quality");

        [GodotMethod("get_dof_blur_near_quality")]
        public Environment.DOFBlurQuality GetDofBlurNearQuality()
        {
            return NativeCalls.godot_icall_0_356(method_bind_113, Object.GetPtr(this));
        }

        private IntPtr method_bind_114 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_glow_enabled");

        [GodotMethod("set_glow_enabled")]
        public void SetGlowEnabled(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_114, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_115 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_glow_enabled");

        [GodotMethod("is_glow_enabled")]
        public bool IsGlowEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_115, Object.GetPtr(this));
        }

        private IntPtr method_bind_116 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_glow_level");

        [GodotMethod("set_glow_level")]
        public void SetGlowLevel(int idx, bool enabled)
        {
            NativeCalls.godot_icall_2_51(method_bind_116, Object.GetPtr(this), idx, enabled);
        }

        private IntPtr method_bind_117 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_glow_level_enabled");

        [GodotMethod("is_glow_level_enabled")]
        public bool IsGlowLevelEnabled(int idx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_117, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_118 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_glow_intensity");

        [GodotMethod("set_glow_intensity")]
        public void SetGlowIntensity(float intensity)
        {
            NativeCalls.godot_icall_1_10(method_bind_118, Object.GetPtr(this), intensity);
        }

        private IntPtr method_bind_119 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_glow_intensity");

        [GodotMethod("get_glow_intensity")]
        public float GetGlowIntensity()
        {
            return NativeCalls.godot_icall_0_9(method_bind_119, Object.GetPtr(this));
        }

        private IntPtr method_bind_120 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_glow_strength");

        [GodotMethod("set_glow_strength")]
        public void SetGlowStrength(float strength)
        {
            NativeCalls.godot_icall_1_10(method_bind_120, Object.GetPtr(this), strength);
        }

        private IntPtr method_bind_121 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_glow_strength");

        [GodotMethod("get_glow_strength")]
        public float GetGlowStrength()
        {
            return NativeCalls.godot_icall_0_9(method_bind_121, Object.GetPtr(this));
        }

        private IntPtr method_bind_122 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_glow_bloom");

        [GodotMethod("set_glow_bloom")]
        public void SetGlowBloom(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_122, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_123 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_glow_bloom");

        [GodotMethod("get_glow_bloom")]
        public float GetGlowBloom()
        {
            return NativeCalls.godot_icall_0_9(method_bind_123, Object.GetPtr(this));
        }

        private IntPtr method_bind_124 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_glow_blend_mode");

        [GodotMethod("set_glow_blend_mode")]
        public void SetGlowBlendMode(Environment.GlowBlendModeEnum mode)
        {
            NativeCalls.godot_icall_1_357(method_bind_124, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_125 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_glow_blend_mode");

        [GodotMethod("get_glow_blend_mode")]
        public Environment.GlowBlendModeEnum GetGlowBlendMode()
        {
            return NativeCalls.godot_icall_0_358(method_bind_125, Object.GetPtr(this));
        }

        private IntPtr method_bind_126 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_glow_hdr_bleed_threshold");

        [GodotMethod("set_glow_hdr_bleed_threshold")]
        public void SetGlowHdrBleedThreshold(float threshold)
        {
            NativeCalls.godot_icall_1_10(method_bind_126, Object.GetPtr(this), threshold);
        }

        private IntPtr method_bind_127 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_glow_hdr_bleed_threshold");

        [GodotMethod("get_glow_hdr_bleed_threshold")]
        public float GetGlowHdrBleedThreshold()
        {
            return NativeCalls.godot_icall_0_9(method_bind_127, Object.GetPtr(this));
        }

        private IntPtr method_bind_128 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_glow_hdr_bleed_scale");

        [GodotMethod("set_glow_hdr_bleed_scale")]
        public void SetGlowHdrBleedScale(float scale)
        {
            NativeCalls.godot_icall_1_10(method_bind_128, Object.GetPtr(this), scale);
        }

        private IntPtr method_bind_129 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_glow_hdr_bleed_scale");

        [GodotMethod("get_glow_hdr_bleed_scale")]
        public float GetGlowHdrBleedScale()
        {
            return NativeCalls.godot_icall_0_9(method_bind_129, Object.GetPtr(this));
        }

        private IntPtr method_bind_130 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_glow_bicubic_upscale");

        [GodotMethod("set_glow_bicubic_upscale")]
        public void SetGlowBicubicUpscale(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_130, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_131 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_glow_bicubic_upscale_enabled");

        [GodotMethod("is_glow_bicubic_upscale_enabled")]
        public bool IsGlowBicubicUpscaleEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_131, Object.GetPtr(this));
        }

        private IntPtr method_bind_132 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_adjustment_enable");

        [GodotMethod("set_adjustment_enable")]
        public void SetAdjustmentEnable(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_132, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_133 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_adjustment_enabled");

        [GodotMethod("is_adjustment_enabled")]
        public bool IsAdjustmentEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_133, Object.GetPtr(this));
        }

        private IntPtr method_bind_134 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_adjustment_brightness");

        [GodotMethod("set_adjustment_brightness")]
        public void SetAdjustmentBrightness(float brightness)
        {
            NativeCalls.godot_icall_1_10(method_bind_134, Object.GetPtr(this), brightness);
        }

        private IntPtr method_bind_135 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_adjustment_brightness");

        [GodotMethod("get_adjustment_brightness")]
        public float GetAdjustmentBrightness()
        {
            return NativeCalls.godot_icall_0_9(method_bind_135, Object.GetPtr(this));
        }

        private IntPtr method_bind_136 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_adjustment_contrast");

        [GodotMethod("set_adjustment_contrast")]
        public void SetAdjustmentContrast(float contrast)
        {
            NativeCalls.godot_icall_1_10(method_bind_136, Object.GetPtr(this), contrast);
        }

        private IntPtr method_bind_137 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_adjustment_contrast");

        [GodotMethod("get_adjustment_contrast")]
        public float GetAdjustmentContrast()
        {
            return NativeCalls.godot_icall_0_9(method_bind_137, Object.GetPtr(this));
        }

        private IntPtr method_bind_138 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_adjustment_saturation");

        [GodotMethod("set_adjustment_saturation")]
        public void SetAdjustmentSaturation(float saturation)
        {
            NativeCalls.godot_icall_1_10(method_bind_138, Object.GetPtr(this), saturation);
        }

        private IntPtr method_bind_139 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_adjustment_saturation");

        [GodotMethod("get_adjustment_saturation")]
        public float GetAdjustmentSaturation()
        {
            return NativeCalls.godot_icall_0_9(method_bind_139, Object.GetPtr(this));
        }

        private IntPtr method_bind_140 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_adjustment_color_correction");

        [GodotMethod("set_adjustment_color_correction")]
        public void SetAdjustmentColorCorrection(Texture colorCorrection)
        {
            NativeCalls.godot_icall_1_24(method_bind_140, Object.GetPtr(this), Object.GetPtr(colorCorrection));
        }

        private IntPtr method_bind_141 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_adjustment_color_correction");

        [GodotMethod("get_adjustment_color_correction")]
        public Texture GetAdjustmentColorCorrection()
        {
            return NativeCalls.godot_icall_0_119(method_bind_141, Object.GetPtr(this));
        }
    }
}
