using System;

namespace Godot
{
    /// <summary>
    /// File type. This is used to permanently store data into the user device's file system and to read from it. This can be used to store game save data or player configuration files, for example.
    /// Here's a sample on how to write and read from a file:
    /// [codeblock]
    /// func save(content):
    /// var file = File.new()
    /// file.open("user://save_game.dat", file.WRITE)
    /// file.store_string(content)
    /// file.close()
    /// func load():
    /// var file = File.new()
    /// file.open("user://save_game.dat", file.READ)
    /// var content = file.get_as_text()
    /// file.close()
    /// return content
    /// [/codeblock]
    /// </summary>
    public class File : Reference
    {
        public enum CompressionMode
        {
            Fastlz = 0,
            Deflate = 1,
            Zstd = 2,
            Gzip = 3
        }

        public enum ModeFlags
        {
            Read = 1,
            Write = 2,
            ReadWrite = 3,
            WriteRead = 7
        }


        /// <summary>
        /// If [code]true[/code] the file's endianness is swapped. Use this if you're dealing with files written in big endian machines.
        /// Note that this is about the file format, not CPU type. This is always reseted to [code]false[/code] whenever you open the file.
        /// </summary>
        public bool EndianSwap
        {
            get
            {
                return GetEndianSwap();
            }
            set
            {
                SetEndianSwap(value);
            }
        }


        private const string nativeName = "_File";

        public File() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_File_Ctor(this);
        }

        internal File(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "open_encrypted");

        /// <summary>
        /// Opens an encrypted file in write or read mode. You need to pass a binary key to encrypt/decrypt it.
        /// </summary>
        [GodotMethod("open_encrypted")]
        public Error OpenEncrypted(string path, int modeFlags, byte[] key)
        {
            return NativeCalls.godot_icall_3_1038(method_bind_0, Object.GetPtr(this), path, modeFlags, key);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "open_encrypted_with_pass");

        /// <summary>
        /// Opens an encrypted file in write or read mode. You need to pass a password to encrypt/decrypt it.
        /// </summary>
        [GodotMethod("open_encrypted_with_pass")]
        public Error OpenEncryptedWithPass(string path, int modeFlags, string pass)
        {
            return NativeCalls.godot_icall_3_1039(method_bind_1, Object.GetPtr(this), path, modeFlags, pass);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "open_compressed");

        /// <summary>
        /// Opens a compressed file for reading or writing. Use COMPRESSION_* constants to set [code]compression_mode[/code].
        /// </summary>
        [GodotMethod("open_compressed")]
        public Error OpenCompressed(string path, int modeFlags, int compressionMode = 0)
        {
            return NativeCalls.godot_icall_3_1040(method_bind_2, Object.GetPtr(this), path, modeFlags, compressionMode);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "open");

        /// <summary>
        /// Opens the file for writing or reading, depending on the flags.
        /// </summary>
        [GodotMethod("open")]
        public Error Open(string path, int flags)
        {
            return NativeCalls.godot_icall_2_575(method_bind_3, Object.GetPtr(this), path, flags);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "close");

        /// <summary>
        /// Closes the currently opened file.
        /// </summary>
        [GodotMethod("close")]
        public void Close()
        {
            NativeCalls.godot_icall_0_12(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_open");

        /// <summary>
        /// Returns [code]true[/code] if the file is currently opened.
        /// </summary>
        [GodotMethod("is_open")]
        public bool IsOpen()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "seek");

        /// <summary>
        /// Change the file reading/writing cursor to the specified position (in bytes from the beginning of the file).
        /// </summary>
        [GodotMethod("seek")]
        public void Seek(int position)
        {
            NativeCalls.godot_icall_1_0(method_bind_6, Object.GetPtr(this), position);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "seek_end");

        /// <summary>
        /// Changes the file reading/writing cursor to the specified position (in bytes from the end of the file). Note that this is an offset, so you should use negative numbers or the cursor will be at the end of the file.
        /// </summary>
        [GodotMethod("seek_end")]
        public void SeekEnd(int position = 0)
        {
            NativeCalls.godot_icall_1_0(method_bind_7, Object.GetPtr(this), position);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_position");

        /// <summary>
        /// Returns the file cursor's position.
        /// </summary>
        [GodotMethod("get_position")]
        public int GetPosition()
        {
            return NativeCalls.godot_icall_0_1(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_len");

        /// <summary>
        /// Returns the size of the file in bytes.
        /// </summary>
        [GodotMethod("get_len")]
        public int GetLen()
        {
            return NativeCalls.godot_icall_0_1(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "eof_reached");

        /// <summary>
        /// Returns [code]true[/code] if the file cursor has reached the end of the file.
        /// </summary>
        [GodotMethod("eof_reached")]
        public bool EofReached()
        {
            return NativeCalls.godot_icall_0_3(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_8");

        /// <summary>
        /// Returns the next 8 bits from the file as an integer.
        /// </summary>
        [GodotMethod("get_8")]
        public int Get8()
        {
            return NativeCalls.godot_icall_0_1(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_16");

        /// <summary>
        /// Returns the next 16 bits from the file as an integer.
        /// </summary>
        [GodotMethod("get_16")]
        public int Get16()
        {
            return NativeCalls.godot_icall_0_1(method_bind_12, Object.GetPtr(this));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_32");

        /// <summary>
        /// Returns the next 32 bits from the file as an integer.
        /// </summary>
        [GodotMethod("get_32")]
        public int Get32()
        {
            return NativeCalls.godot_icall_0_1(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_64");

        /// <summary>
        /// Returns the next 64 bits from the file as an integer.
        /// </summary>
        [GodotMethod("get_64")]
        public int Get64()
        {
            return NativeCalls.godot_icall_0_1(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_float");

        /// <summary>
        /// Returns the next 32 bits from the file as a floating point number.
        /// </summary>
        [GodotMethod("get_float")]
        public float GetFloat()
        {
            return NativeCalls.godot_icall_0_9(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_double");

        /// <summary>
        /// Returns the next 64 bits from the file as a floating point number.
        /// </summary>
        [GodotMethod("get_double")]
        public float GetDouble()
        {
            return NativeCalls.godot_icall_0_9(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_real");

        /// <summary>
        /// Returns the next bits from the file as a floating point number.
        /// </summary>
        [GodotMethod("get_real")]
        public float GetReal()
        {
            return NativeCalls.godot_icall_0_9(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_buffer");

        /// <summary>
        /// Returns next [code]len[/code] bytes of the file as a [PoolByteArray].
        /// </summary>
        [GodotMethod("get_buffer")]
        public byte[] GetBuffer(int len)
        {
            return NativeCalls.godot_icall_1_1041(method_bind_18, Object.GetPtr(this), len);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_line");

        /// <summary>
        /// Returns the next line of the file as a [String].
        /// </summary>
        [GodotMethod("get_line")]
        public string GetLine()
        {
            return NativeCalls.godot_icall_0_2(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_as_text");

        /// <summary>
        /// Returns the whole file as a [String].
        /// </summary>
        [GodotMethod("get_as_text")]
        public string GetAsText()
        {
            return NativeCalls.godot_icall_0_2(method_bind_20, Object.GetPtr(this));
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_md5");

        /// <summary>
        /// Returns an MD5 String representing the file at the given path or an empty [String] on failure.
        /// </summary>
        [GodotMethod("get_md5")]
        public string GetMd5(string path)
        {
            return NativeCalls.godot_icall_1_73(method_bind_21, Object.GetPtr(this), path);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_sha256");

        /// <summary>
        /// Returns a SHA-256 [String] representing the file at the given path or an empty [String] on failure.
        /// </summary>
        [GodotMethod("get_sha256")]
        public string GetSha256(string path)
        {
            return NativeCalls.godot_icall_1_73(method_bind_22, Object.GetPtr(this), path);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_endian_swap");

        [GodotMethod("get_endian_swap")]
        public bool GetEndianSwap()
        {
            return NativeCalls.godot_icall_0_3(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_endian_swap");

        [GodotMethod("set_endian_swap")]
        public void SetEndianSwap(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_24, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_error");

        /// <summary>
        /// Returns the last error that happened when trying to perform operations. Compare with the [code]ERR_FILE_*[/code] constants from [@GlobalScope].
        /// </summary>
        [GodotMethod("get_error")]
        public Error GetError()
        {
            return NativeCalls.godot_icall_0_325(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_var");

        /// <summary>
        /// Returns the next [Variant] value from the file.
        /// </summary>
        [GodotMethod("get_var")]
        public object GetVar()
        {
            return NativeCalls.godot_icall_0_461(method_bind_26, Object.GetPtr(this));
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_csv_line");

        /// <summary>
        /// Returns the next value of the file in CSV (Comma Separated Values) format. You can pass a different delimiter to use other than the default "," (comma).
        /// </summary>
        [GodotMethod("get_csv_line")]
        public string[] GetCsvLine(string delim = ",")
        {
            return NativeCalls.godot_icall_1_252(method_bind_27, Object.GetPtr(this), delim);
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "store_8");

        /// <summary>
        /// Stores an integer as 8 bits in the file.
        /// </summary>
        [GodotMethod("store_8")]
        public void Store8(int value)
        {
            NativeCalls.godot_icall_1_0(method_bind_28, Object.GetPtr(this), value);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "store_16");

        /// <summary>
        /// Stores an integer as 16 bits in the file.
        /// </summary>
        [GodotMethod("store_16")]
        public void Store16(int value)
        {
            NativeCalls.godot_icall_1_0(method_bind_29, Object.GetPtr(this), value);
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "store_32");

        /// <summary>
        /// Stores an integer as 32 bits in the file.
        /// </summary>
        [GodotMethod("store_32")]
        public void Store32(int value)
        {
            NativeCalls.godot_icall_1_0(method_bind_30, Object.GetPtr(this), value);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "store_64");

        /// <summary>
        /// Stores an integer as 64 bits in the file.
        /// </summary>
        [GodotMethod("store_64")]
        public void Store64(int value)
        {
            NativeCalls.godot_icall_1_0(method_bind_31, Object.GetPtr(this), value);
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "store_float");

        /// <summary>
        /// Stores a floating point number as 32 bits in the file.
        /// </summary>
        [GodotMethod("store_float")]
        public void StoreFloat(float value)
        {
            NativeCalls.godot_icall_1_10(method_bind_32, Object.GetPtr(this), value);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "store_double");

        /// <summary>
        /// Stores a floating point number as 64 bits in the file.
        /// </summary>
        [GodotMethod("store_double")]
        public void StoreDouble(float value)
        {
            NativeCalls.godot_icall_1_10(method_bind_33, Object.GetPtr(this), value);
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "store_real");

        /// <summary>
        /// Stores a floating point number in the file.
        /// </summary>
        [GodotMethod("store_real")]
        public void StoreReal(float value)
        {
            NativeCalls.godot_icall_1_10(method_bind_34, Object.GetPtr(this), value);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "store_buffer");

        /// <summary>
        /// Stores the given array of bytes in the file.
        /// </summary>
        [GodotMethod("store_buffer")]
        public void StoreBuffer(byte[] buffer)
        {
            NativeCalls.godot_icall_1_131(method_bind_35, Object.GetPtr(this), buffer);
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "store_line");

        /// <summary>
        /// Stores the given [String] as a line in the file.
        /// </summary>
        [GodotMethod("store_line")]
        public void StoreLine(string line)
        {
            NativeCalls.godot_icall_1_43(method_bind_36, Object.GetPtr(this), line);
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "store_string");

        /// <summary>
        /// Stores the given [String] in the file.
        /// </summary>
        [GodotMethod("store_string")]
        public void StoreString(string @string)
        {
            NativeCalls.godot_icall_1_43(method_bind_37, Object.GetPtr(this), @string);
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "store_var");

        /// <summary>
        /// Stores any Variant value in the file.
        /// </summary>
        [GodotMethod("store_var")]
        public void StoreVar(object value)
        {
            NativeCalls.godot_icall_1_463(method_bind_38, Object.GetPtr(this), value);
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "store_pascal_string");

        /// <summary>
        /// Stores the given [String] as a line in the file in Pascal format (i.e. also store the length of the string).
        /// </summary>
        [GodotMethod("store_pascal_string")]
        public void StorePascalString(string @string)
        {
            NativeCalls.godot_icall_1_43(method_bind_39, Object.GetPtr(this), @string);
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_pascal_string");

        /// <summary>
        /// Returns a [String] saved in Pascal format from the file.
        /// </summary>
        [GodotMethod("get_pascal_string")]
        public string GetPascalString()
        {
            return NativeCalls.godot_icall_0_2(method_bind_40, Object.GetPtr(this));
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "file_exists");

        /// <summary>
        /// Returns [code]true[/code] if the file exists in the given path.
        /// </summary>
        [GodotMethod("file_exists")]
        public bool FileExists(string path)
        {
            return NativeCalls.godot_icall_1_70(method_bind_41, Object.GetPtr(this), path);
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_modified_time");

        /// <summary>
        /// Returns the last time the [code]file[/code] was modified in unix timestamp format or returns a [String] "ERROR IN [code]file[/code]". This unix timestamp can be converted to datetime by using [method OS.get_datetime_from_unix_time].
        /// </summary>
        [GodotMethod("get_modified_time")]
        public int GetModifiedTime(string file)
        {
            return NativeCalls.godot_icall_1_87(method_bind_42, Object.GetPtr(this), file);
        }
    }
}
