using System;

namespace Godot
{
    /// <summary>
    /// FileDialog is a preset dialog used to choose files and directories in the filesystem. It supports filter masks.
    /// </summary>
    public class FileDialog : ConfirmationDialog
    {
        public enum ModeEnum
        {
            OpenFile = 0,
            OpenFiles = 1,
            OpenDir = 2,
            OpenAny = 3,
            SaveFile = 4
        }

        public enum AccessEnum
        {
            Resources = 0,
            Userdata = 1,
            Filesystem = 2
        }


        /// <summary>
        /// If [code]true[/code], changing the [code]mode[/code] property will set the window title accordingly (e. g. setting mode to [code]MODE_OPEN_FILE[/code] will change the window title to "Open a File").
        /// </summary>
        public bool ModeOverridesTitle
        {
            get
            {
                return IsModeOverridingTitle();
            }
            set
            {
                SetModeOverridesTitle(value);
            }
        }

        /// <summary>
        /// </summary>
        public FileDialog.ModeEnum Mode
        {
            get
            {
                return GetMode();
            }
            set
            {
                SetMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public FileDialog.AccessEnum Access
        {
            get
            {
                return GetAccess();
            }
            set
            {
                SetAccess(value);
            }
        }

        /// <summary>
        /// </summary>
        public string[] Filters
        {
            get
            {
                return GetFilters();
            }
            set
            {
                SetFilters(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool ShowHiddenFiles
        {
            get
            {
                return IsShowingHiddenFiles();
            }
            set
            {
                SetShowHiddenFiles(value);
            }
        }

        /// <summary>
        /// The current working directory of the file dialog.
        /// </summary>
        public string CurrentDir
        {
            get
            {
                return GetCurrentDir();
            }
            set
            {
                SetCurrentDir(value);
            }
        }

        /// <summary>
        /// The currently selected file of the file dialog.
        /// </summary>
        public string CurrentFile
        {
            get
            {
                return GetCurrentFile();
            }
            set
            {
                SetCurrentFile(value);
            }
        }

        /// <summary>
        /// The currently selected file path of the file dialog.
        /// </summary>
        public string CurrentPath
        {
            get
            {
                return GetCurrentPath();
            }
            set
            {
                SetCurrentPath(value);
            }
        }


        private const string nativeName = "FileDialog";

        public FileDialog() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_FileDialog_Ctor(this);
        }

        internal FileDialog(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_filters");

        /// <summary>
        /// Clear all the added filters in the dialog.
        /// </summary>
        [GodotMethod("clear_filters")]
        public void ClearFilters()
        {
            NativeCalls.godot_icall_0_12(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_filter");

        /// <summary>
        /// Add a custom filter. Filter format is: "mask ; description", example (C++): dialog-&gt;add_filter("*.png ; PNG Images");
        /// </summary>
        [GodotMethod("add_filter")]
        public void AddFilter(string filter)
        {
            NativeCalls.godot_icall_1_43(method_bind_1, Object.GetPtr(this), filter);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_filters");

        [GodotMethod("set_filters")]
        public void SetFilters(string[] filters)
        {
            NativeCalls.godot_icall_1_340(method_bind_2, Object.GetPtr(this), filters);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_filters");

        [GodotMethod("get_filters")]
        public string[] GetFilters()
        {
            return NativeCalls.godot_icall_0_72(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_current_dir");

        [GodotMethod("get_current_dir")]
        public string GetCurrentDir()
        {
            return NativeCalls.godot_icall_0_2(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_current_file");

        [GodotMethod("get_current_file")]
        public string GetCurrentFile()
        {
            return NativeCalls.godot_icall_0_2(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_current_path");

        [GodotMethod("get_current_path")]
        public string GetCurrentPath()
        {
            return NativeCalls.godot_icall_0_2(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_current_dir");

        [GodotMethod("set_current_dir")]
        public void SetCurrentDir(string dir)
        {
            NativeCalls.godot_icall_1_43(method_bind_7, Object.GetPtr(this), dir);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_current_file");

        [GodotMethod("set_current_file")]
        public void SetCurrentFile(string file)
        {
            NativeCalls.godot_icall_1_43(method_bind_8, Object.GetPtr(this), file);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_current_path");

        [GodotMethod("set_current_path")]
        public void SetCurrentPath(string path)
        {
            NativeCalls.godot_icall_1_43(method_bind_9, Object.GetPtr(this), path);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_mode_overrides_title");

        [GodotMethod("set_mode_overrides_title")]
        public void SetModeOverridesTitle(bool @override)
        {
            NativeCalls.godot_icall_1_11(method_bind_10, Object.GetPtr(this), @override);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_mode_overriding_title");

        [GodotMethod("is_mode_overriding_title")]
        public bool IsModeOverridingTitle()
        {
            return NativeCalls.godot_icall_0_3(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_mode");

        [GodotMethod("set_mode")]
        public void SetMode(FileDialog.ModeEnum mode)
        {
            NativeCalls.godot_icall_1_359(method_bind_12, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mode");

        [GodotMethod("get_mode")]
        public FileDialog.ModeEnum GetMode()
        {
            return NativeCalls.godot_icall_0_360(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_vbox");

        /// <summary>
        /// Return the vertical box container of the dialog, custom controls can be added to it.
        /// </summary>
        [GodotMethod("get_vbox")]
        public VBoxContainer GetVbox()
        {
            return NativeCalls.godot_icall_0_310(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_access");

        [GodotMethod("set_access")]
        public void SetAccess(FileDialog.AccessEnum access)
        {
            NativeCalls.godot_icall_1_361(method_bind_15, Object.GetPtr(this), access);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_access");

        [GodotMethod("get_access")]
        public FileDialog.AccessEnum GetAccess()
        {
            return NativeCalls.godot_icall_0_362(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_show_hidden_files");

        [GodotMethod("set_show_hidden_files")]
        public void SetShowHiddenFiles(bool show)
        {
            NativeCalls.godot_icall_1_11(method_bind_17, Object.GetPtr(this), show);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_showing_hidden_files");

        [GodotMethod("is_showing_hidden_files")]
        public bool IsShowingHiddenFiles()
        {
            return NativeCalls.godot_icall_0_3(method_bind_18, Object.GetPtr(this));
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "deselect_items");

        /// <summary>
        /// </summary>
        [GodotMethod("deselect_items")]
        public void DeselectItems()
        {
            NativeCalls.godot_icall_0_12(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "invalidate");

        /// <summary>
        /// Invalidate and update the current dialog content list.
        /// </summary>
        [GodotMethod("invalidate")]
        public void Invalidate()
        {
            NativeCalls.godot_icall_0_12(method_bind_20, Object.GetPtr(this));
        }
    }
}
