using System;

namespace Godot
{
    /// <summary>
    /// In GDScript, functions are not [i]first-class objects[/i]. This means it is impossible to store them directly as variables, return them from another function, or pass them as arguments.
    /// However, by creating a [code]FuncRef[/code] using the [method @GDScript.funcref] function, a reference to a function in a given object can be created, passed around and called.
    /// </summary>
    public class FuncRef : Reference
    {
        private const string nativeName = "FuncRef";

        public FuncRef() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_FuncRef_Ctor(this);
        }

        internal FuncRef(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "call_func");

        /// <summary>
        /// Calls the referenced function previously set by [method set_function] or [method @GDScript.funcref].
        /// </summary>
        [GodotMethod("call_func")]
        public object CallFunc(params object[] @args)
        {
            return NativeCalls.godot_icall_1_365(method_bind_0, Object.GetPtr(this), @args);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_instance");

        /// <summary>
        /// The object containing the referenced function. This object must be of a type actually inheriting from [Object], not a built-in type such as [int], [Vector2] or [Dictionary].
        /// </summary>
        [GodotMethod("set_instance")]
        public void SetInstance(Object instance)
        {
            NativeCalls.godot_icall_1_42(method_bind_1, Object.GetPtr(this), Object.GetPtr(instance));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_function");

        /// <summary>
        /// The name of the referenced function to call on the object, without parentheses or any parameters.
        /// </summary>
        [GodotMethod("set_function")]
        public void SetFunction(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_2, Object.GetPtr(this), name);
        }
    }
}
