using System;

namespace Godot
{
    /// <summary>
    /// Base node for geometry based visual instances. Shares some common functionality like visibility and custom materials.
    /// </summary>
    public class GeometryInstance : VisualInstance
    {
        public enum Flags
        {
            UseBakedLight = 0,
            Max = 1
        }

        public enum ShadowCastingSetting
        {
            Off = 0,
            On = 1,
            DoubleSided = 2,
            ShadowsOnly = 3
        }


        /// <summary>
        /// The material override for the whole geometry.
        /// If there is a material in material_override, it will be used instead of any material set in any material slot of the mesh.
        /// </summary>
        public Material MaterialOverride
        {
            get
            {
                return GetMaterialOverride();
            }
            set
            {
                SetMaterialOverride(value);
            }
        }

        /// <summary>
        /// The selected shadow casting flag. See SHADOW_CASTING_SETTING_* constants for values.
        /// </summary>
        public GeometryInstance.ShadowCastingSetting CastShadow
        {
            get
            {
                return GetCastShadowsSetting();
            }
            set
            {
                SetCastShadowsSetting(value);
            }
        }

        /// <summary>
        /// The extra distance added to the GeometryInstance's bounding box ([AABB]) to increase its cull box.
        /// </summary>
        public float ExtraCullMargin
        {
            get
            {
                return GetExtraCullMargin();
            }
            set
            {
                SetExtraCullMargin(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] this GeometryInstance will be used when baking lights using a [GIProbe] and/or any other form of baked lighting.
        /// </summary>
        public bool UseInBakedLight
        {
            get
            {
                return GetFlag((GeometryInstance.Flags)0);
            }
            set
            {
                SetFlag((GeometryInstance.Flags)0, value);
            }
        }

        /// <summary>
        /// The GeometryInstance's min LOD distance.
        /// </summary>
        public float LodMinDistance
        {
            get
            {
                return GetLodMinDistance();
            }
            set
            {
                SetLodMinDistance(value);
            }
        }

        /// <summary>
        /// The GeometryInstance's min LOD margin.
        /// </summary>
        public float LodMinHysteresis
        {
            get
            {
                return GetLodMinHysteresis();
            }
            set
            {
                SetLodMinHysteresis(value);
            }
        }

        /// <summary>
        /// The GeometryInstance's max LOD distance.
        /// </summary>
        public float LodMaxDistance
        {
            get
            {
                return GetLodMaxDistance();
            }
            set
            {
                SetLodMaxDistance(value);
            }
        }

        /// <summary>
        /// The GeometryInstance's max LOD margin.
        /// </summary>
        public float LodMaxHysteresis
        {
            get
            {
                return GetLodMaxHysteresis();
            }
            set
            {
                SetLodMaxHysteresis(value);
            }
        }


        private const string nativeName = "GeometryInstance";

        internal GeometryInstance() {}

        internal GeometryInstance(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_material_override");

        [GodotMethod("set_material_override")]
        public void SetMaterialOverride(Material material)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(material));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_material_override");

        [GodotMethod("get_material_override")]
        public Material GetMaterialOverride()
        {
            return NativeCalls.godot_icall_0_222(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_flag");

        [GodotMethod("set_flag")]
        public void SetFlag(GeometryInstance.Flags flag, bool value)
        {
            NativeCalls.godot_icall_2_379(method_bind_2, Object.GetPtr(this), flag, value);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_flag");

        [GodotMethod("get_flag")]
        public bool GetFlag(GeometryInstance.Flags flag)
        {
            return NativeCalls.godot_icall_1_380(method_bind_3, Object.GetPtr(this), flag);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_cast_shadows_setting");

        [GodotMethod("set_cast_shadows_setting")]
        public void SetCastShadowsSetting(GeometryInstance.ShadowCastingSetting shadowCastingSetting)
        {
            NativeCalls.godot_icall_1_381(method_bind_4, Object.GetPtr(this), shadowCastingSetting);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cast_shadows_setting");

        [GodotMethod("get_cast_shadows_setting")]
        public GeometryInstance.ShadowCastingSetting GetCastShadowsSetting()
        {
            return NativeCalls.godot_icall_0_382(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_lod_max_hysteresis");

        [GodotMethod("set_lod_max_hysteresis")]
        public void SetLodMaxHysteresis(float mode)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_lod_max_hysteresis");

        [GodotMethod("get_lod_max_hysteresis")]
        public float GetLodMaxHysteresis()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_lod_max_distance");

        [GodotMethod("set_lod_max_distance")]
        public void SetLodMaxDistance(float mode)
        {
            NativeCalls.godot_icall_1_10(method_bind_8, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_lod_max_distance");

        [GodotMethod("get_lod_max_distance")]
        public float GetLodMaxDistance()
        {
            return NativeCalls.godot_icall_0_9(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_lod_min_hysteresis");

        [GodotMethod("set_lod_min_hysteresis")]
        public void SetLodMinHysteresis(float mode)
        {
            NativeCalls.godot_icall_1_10(method_bind_10, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_lod_min_hysteresis");

        [GodotMethod("get_lod_min_hysteresis")]
        public float GetLodMinHysteresis()
        {
            return NativeCalls.godot_icall_0_9(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_lod_min_distance");

        [GodotMethod("set_lod_min_distance")]
        public void SetLodMinDistance(float mode)
        {
            NativeCalls.godot_icall_1_10(method_bind_12, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_lod_min_distance");

        [GodotMethod("get_lod_min_distance")]
        public float GetLodMinDistance()
        {
            return NativeCalls.godot_icall_0_9(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_extra_cull_margin");

        [GodotMethod("set_extra_cull_margin")]
        public void SetExtraCullMargin(float margin)
        {
            NativeCalls.godot_icall_1_10(method_bind_14, Object.GetPtr(this), margin);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_extra_cull_margin");

        [GodotMethod("get_extra_cull_margin")]
        public float GetExtraCullMargin()
        {
            return NativeCalls.godot_icall_0_9(method_bind_15, Object.GetPtr(this));
        }
    }
}
