using System;

namespace Godot
{
    /// <summary>
    /// GraphEdit manages the showing of GraphNodes it contains, as well as connections and disconnections between them. Signals are sent for each of these two events. Disconnection between GraphNodes slots is disabled by default.
    /// It is greatly advised to enable low processor usage mode (see [method OS.set_low_processor_usage_mode]) when using GraphEdits.
    /// </summary>
    public class GraphEdit : Control
    {
        /// <summary>
        /// If [code]true[/code], enables disconnection of existing connections in the GraphEdit by dragging the right end.
        /// </summary>
        public bool RightDisconnects
        {
            get
            {
                return IsRightDisconnectsEnabled();
            }
            set
            {
                SetRightDisconnects(value);
            }
        }

        /// <summary>
        /// The scroll offset.
        /// </summary>
        public Vector2 ScrollOffset
        {
            get
            {
                return GetScrollOfs();
            }
            set
            {
                SetScrollOfs(value);
            }
        }

        /// <summary>
        /// The snapping distance in pixels.
        /// </summary>
        public int SnapDistance
        {
            get
            {
                return GetSnap();
            }
            set
            {
                SetSnap(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], enables snapping.
        /// </summary>
        public bool UseSnap
        {
            get
            {
                return IsUsingSnap();
            }
            set
            {
                SetUseSnap(value);
            }
        }

        /// <summary>
        /// The current zoom value.
        /// </summary>
        public float Zoom
        {
            get
            {
                return GetZoom();
            }
            set
            {
                SetZoom(value);
            }
        }


        private const string nativeName = "GraphEdit";

        public GraphEdit() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_GraphEdit_Ctor(this);
        }

        internal GraphEdit(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "connect_node");

        /// <summary>
        /// Create a connection between 'from_port' slot of 'from' GraphNode and 'to_port' slot of 'to' GraphNode. If the connection already exists, no connection is created.
        /// </summary>
        [GodotMethod("connect_node")]
        public Error ConnectNode(string from, int fromPort, string to, int toPort)
        {
            return NativeCalls.godot_icall_4_391(method_bind_0, Object.GetPtr(this), from, fromPort, to, toPort);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_node_connected");

        /// <summary>
        /// Return true if the 'from_port' slot of 'from' GraphNode is connected to the 'to_port' slot of 'to' GraphNode.
        /// </summary>
        [GodotMethod("is_node_connected")]
        public bool IsNodeConnected(string from, int fromPort, string to, int toPort)
        {
            return NativeCalls.godot_icall_4_392(method_bind_1, Object.GetPtr(this), from, fromPort, to, toPort);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "disconnect_node");

        /// <summary>
        /// Remove the connection between 'from_port' slot of 'from' GraphNode and 'to_port' slot of 'to' GraphNode, if connection exists.
        /// </summary>
        [GodotMethod("disconnect_node")]
        public void DisconnectNode(string from, int fromPort, string to, int toPort)
        {
            NativeCalls.godot_icall_4_393(method_bind_2, Object.GetPtr(this), from, fromPort, to, toPort);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connection_list");

        /// <summary>
        /// Return an Array containing the list of connections. A connection consists in a structure of the form {from_slot: 0, from: "GraphNode name 0", to_slot: 1, to: "GraphNode name 1" }
        /// </summary>
        [GodotMethod("get_connection_list")]
        public object[] GetConnectionList()
        {
            return NativeCalls.godot_icall_0_21(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_scroll_ofs");

        [GodotMethod("get_scroll_ofs")]
        public Vector2 GetScrollOfs()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_4, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_scroll_ofs");

        [GodotMethod("set_scroll_ofs")]
        public void SetScrollOfs(Vector2 ofs)
        {
            NativeCalls.godot_icall_1_45(method_bind_5, Object.GetPtr(this), ref ofs);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_zoom");

        [GodotMethod("set_zoom")]
        public void SetZoom(float pZoom)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), pZoom);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_zoom");

        [GodotMethod("get_zoom")]
        public float GetZoom()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_snap");

        [GodotMethod("set_snap")]
        public void SetSnap(int pixels)
        {
            NativeCalls.godot_icall_1_0(method_bind_8, Object.GetPtr(this), pixels);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_snap");

        [GodotMethod("get_snap")]
        public int GetSnap()
        {
            return NativeCalls.godot_icall_0_1(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_use_snap");

        [GodotMethod("set_use_snap")]
        public void SetUseSnap(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_10, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_using_snap");

        [GodotMethod("is_using_snap")]
        public bool IsUsingSnap()
        {
            return NativeCalls.godot_icall_0_3(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_right_disconnects");

        [GodotMethod("set_right_disconnects")]
        public void SetRightDisconnects(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_12, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_right_disconnects_enabled");

        [GodotMethod("is_right_disconnects_enabled")]
        public bool IsRightDisconnectsEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_selected");

        /// <summary>
        /// </summary>
        [GodotMethod("set_selected")]
        public void SetSelected(Node node)
        {
            NativeCalls.godot_icall_1_42(method_bind_14, Object.GetPtr(this), Object.GetPtr(node));
        }
    }
}
