using System;

namespace Godot
{
    /// <summary>
    /// GridMap lets you place meshes on a grid interactively. It works both from the editor and can help you create in-game level editors.
    /// GridMaps use a [MeshLibrary] which contain a list of tiles: meshes with materials plus optional collisions and extra elements.
    /// A GridMap contains a collection of cells. Each grid cell refers to a [MeshLibrary] item. All cells in the map have the same dimensions.
    /// A GridMap is split into a sparse collection of octants for efficient rendering and physics processing. Every octant has the same dimensions and can contain several cells.
    /// </summary>
    public class GridMap : Spatial
    {
        public const int InvalidCellItem = -1;

        /// <summary>
        /// The assigned [MeshLibrary].
        /// </summary>
        public MeshLibrary Theme
        {
            get
            {
                return GetTheme();
            }
            set
            {
                SetTheme(value);
            }
        }

        /// <summary>
        /// The dimensions of the grid's cells.
        /// </summary>
        public Vector3 CellSize
        {
            get
            {
                return GetCellSize();
            }
            set
            {
                SetCellSize(value);
            }
        }

        /// <summary>
        /// The size of each octant measured in number of cells. This applies to all three axis.
        /// </summary>
        public int CellOctantSize
        {
            get
            {
                return GetOctantSize();
            }
            set
            {
                SetOctantSize(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] grid items are centered on the X axis.
        /// </summary>
        public bool CellCenterX
        {
            get
            {
                return GetCenterX();
            }
            set
            {
                SetCenterX(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] grid items are centered on the Y axis.
        /// </summary>
        public bool CellCenterY
        {
            get
            {
                return GetCenterY();
            }
            set
            {
                SetCenterY(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] grid items are centered on the Z axis.
        /// </summary>
        public bool CellCenterZ
        {
            get
            {
                return GetCenterZ();
            }
            set
            {
                SetCenterZ(value);
            }
        }

        /// <summary>
        /// </summary>
        public float CellScale
        {
            get
            {
                return GetCellScale();
            }
            set
            {
                SetCellScale(value);
            }
        }

        /// <summary>
        /// </summary>
        public int CollisionLayer
        {
            get
            {
                return GetCollisionLayer();
            }
            set
            {
                SetCollisionLayer(value);
            }
        }

        /// <summary>
        /// </summary>
        public int CollisionMask
        {
            get
            {
                return GetCollisionMask();
            }
            set
            {
                SetCollisionMask(value);
            }
        }


        private const string nativeName = "GridMap";

        public GridMap() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_GridMap_Ctor(this);
        }

        internal GridMap(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_layer");

        [GodotMethod("set_collision_layer")]
        public void SetCollisionLayer(int layer)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, Object.GetPtr(this), layer);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_layer");

        [GodotMethod("get_collision_layer")]
        public int GetCollisionLayer()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_mask");

        [GodotMethod("set_collision_mask")]
        public void SetCollisionMask(int mask)
        {
            NativeCalls.godot_icall_1_0(method_bind_2, Object.GetPtr(this), mask);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_mask");

        [GodotMethod("get_collision_mask")]
        public int GetCollisionMask()
        {
            return NativeCalls.godot_icall_0_1(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_mask_bit");

        /// <summary>
        /// </summary>
        [GodotMethod("set_collision_mask_bit")]
        public void SetCollisionMaskBit(int bit, bool value)
        {
            NativeCalls.godot_icall_2_51(method_bind_4, Object.GetPtr(this), bit, value);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_mask_bit");

        /// <summary>
        /// </summary>
        [GodotMethod("get_collision_mask_bit")]
        public bool GetCollisionMaskBit(int bit)
        {
            return NativeCalls.godot_icall_1_29(method_bind_5, Object.GetPtr(this), bit);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_layer_bit");

        /// <summary>
        /// </summary>
        [GodotMethod("set_collision_layer_bit")]
        public void SetCollisionLayerBit(int bit, bool value)
        {
            NativeCalls.godot_icall_2_51(method_bind_6, Object.GetPtr(this), bit, value);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_layer_bit");

        /// <summary>
        /// </summary>
        [GodotMethod("get_collision_layer_bit")]
        public bool GetCollisionLayerBit(int bit)
        {
            return NativeCalls.godot_icall_1_29(method_bind_7, Object.GetPtr(this), bit);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_theme");

        [GodotMethod("set_theme")]
        public void SetTheme(MeshLibrary theme)
        {
            NativeCalls.godot_icall_1_24(method_bind_8, Object.GetPtr(this), Object.GetPtr(theme));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_theme");

        [GodotMethod("get_theme")]
        public MeshLibrary GetTheme()
        {
            return NativeCalls.godot_icall_0_397(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_cell_size");

        [GodotMethod("set_cell_size")]
        public void SetCellSize(Vector3 size)
        {
            NativeCalls.godot_icall_1_104(method_bind_10, Object.GetPtr(this), ref size);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cell_size");

        [GodotMethod("get_cell_size")]
        public Vector3 GetCellSize()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_11, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_cell_scale");

        [GodotMethod("set_cell_scale")]
        public void SetCellScale(float scale)
        {
            NativeCalls.godot_icall_1_10(method_bind_12, Object.GetPtr(this), scale);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cell_scale");

        [GodotMethod("get_cell_scale")]
        public float GetCellScale()
        {
            return NativeCalls.godot_icall_0_9(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_octant_size");

        [GodotMethod("set_octant_size")]
        public void SetOctantSize(int size)
        {
            NativeCalls.godot_icall_1_0(method_bind_14, Object.GetPtr(this), size);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_octant_size");

        [GodotMethod("get_octant_size")]
        public int GetOctantSize()
        {
            return NativeCalls.godot_icall_0_1(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_cell_item");

        /// <summary>
        /// Set the mesh index for the cell referenced by its grid-based X, Y and Z coordinates.
        /// A negative item index will clear the cell.
        /// Optionally, the item's orientation can be passed.
        /// </summary>
        [GodotMethod("set_cell_item")]
        public void SetCellItem(int x, int y, int z, int item, int orientation = 0)
        {
            NativeCalls.godot_icall_5_398(method_bind_16, Object.GetPtr(this), x, y, z, item, orientation);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cell_item");

        /// <summary>
        /// The [MeshLibrary] item index located at the grid-based X, Y and Z coordinates. If the cell is empty, [INVALID_CELL_ITEM] will be returned.
        /// </summary>
        [GodotMethod("get_cell_item")]
        public int GetCellItem(int x, int y, int z)
        {
            return NativeCalls.godot_icall_3_399(method_bind_17, Object.GetPtr(this), x, y, z);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cell_item_orientation");

        /// <summary>
        /// The orientation of the cell at the grid-based X, Y and Z coordinates. -1 is retuned if the cell is empty.
        /// </summary>
        [GodotMethod("get_cell_item_orientation")]
        public int GetCellItemOrientation(int x, int y, int z)
        {
            return NativeCalls.godot_icall_3_399(method_bind_18, Object.GetPtr(this), x, y, z);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "world_to_map");

        /// <summary>
        /// </summary>
        [GodotMethod("world_to_map")]
        public Vector3 WorldToMap(Vector3 pos)
        {
            object ret = NativeCalls.godot_icall_1_35(method_bind_19, Object.GetPtr(this), ref pos);
            return (Vector3)ret;
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "map_to_world");

        /// <summary>
        /// </summary>
        [GodotMethod("map_to_world")]
        public Vector3 MapToWorld(int x, int y, int z)
        {
            object ret = NativeCalls.godot_icall_3_400(method_bind_20, Object.GetPtr(this), x, y, z);
            return (Vector3)ret;
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "resource_changed");

        /// <summary>
        /// </summary>
        [GodotMethod("resource_changed")]
        public void ResourceChanged(Resource resource)
        {
            NativeCalls.godot_icall_1_24(method_bind_21, Object.GetPtr(this), Object.GetPtr(resource));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_center_x");

        [GodotMethod("set_center_x")]
        public void SetCenterX(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_22, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_center_x");

        [GodotMethod("get_center_x")]
        public bool GetCenterX()
        {
            return NativeCalls.godot_icall_0_3(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_center_y");

        [GodotMethod("set_center_y")]
        public void SetCenterY(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_24, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_center_y");

        [GodotMethod("get_center_y")]
        public bool GetCenterY()
        {
            return NativeCalls.godot_icall_0_3(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_center_z");

        [GodotMethod("set_center_z")]
        public void SetCenterZ(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_26, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_center_z");

        [GodotMethod("get_center_z")]
        public bool GetCenterZ()
        {
            return NativeCalls.godot_icall_0_3(method_bind_27, Object.GetPtr(this));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_clip");

        /// <summary>
        /// </summary>
        [GodotMethod("set_clip")]
        public void SetClip(bool enabled, bool clipabove = true, int floor = 0, Vector3.Axis axis = (Vector3.Axis)0)
        {
            NativeCalls.godot_icall_4_401(method_bind_28, Object.GetPtr(this), enabled, clipabove, floor, axis);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear");

        /// <summary>
        /// Clear all cells.
        /// </summary>
        [GodotMethod("clear")]
        public void Clear()
        {
            NativeCalls.godot_icall_0_12(method_bind_29, Object.GetPtr(this));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_used_cells");

        /// <summary>
        /// Array of [Vector3] with the non empty cell coordinates in the grid map.
        /// </summary>
        [GodotMethod("get_used_cells")]
        public object[] GetUsedCells()
        {
            return NativeCalls.godot_icall_0_21(method_bind_30, Object.GetPtr(this));
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_meshes");

        /// <summary>
        /// Array of [Transform] and [Mesh] references corresponding to the non empty cells in the grid. The transforms are specified in world space.
        /// </summary>
        [GodotMethod("get_meshes")]
        public object[] GetMeshes()
        {
            return NativeCalls.godot_icall_0_21(method_bind_31, Object.GetPtr(this));
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bake_meshes");

        /// <summary>
        /// </summary>
        [GodotMethod("get_bake_meshes")]
        public object[] GetBakeMeshes()
        {
            return NativeCalls.godot_icall_0_21(method_bind_32, Object.GetPtr(this));
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bake_mesh_instance");

        /// <summary>
        /// </summary>
        [GodotMethod("get_bake_mesh_instance")]
        public RID GetBakeMeshInstance(int idx)
        {
            IntPtr ret = NativeCalls.godot_icall_1_402(method_bind_33, Object.GetPtr(this), idx);
            return new RID(ret);
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_baked_meshes");

        /// <summary>
        /// </summary>
        [GodotMethod("clear_baked_meshes")]
        public void ClearBakedMeshes()
        {
            NativeCalls.godot_icall_0_12(method_bind_34, Object.GetPtr(this));
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "make_baked_meshes");

        /// <summary>
        /// </summary>
        [GodotMethod("make_baked_meshes")]
        public void MakeBakedMeshes(bool genLightmapUv = false, float lightmapUvTexelSize = 0.1f)
        {
            NativeCalls.godot_icall_2_403(method_bind_35, Object.GetPtr(this), genLightmapUv, lightmapUvTexelSize);
        }
    }
}
