using System;
using System.Collections.Generic;

namespace Godot
{
    /// <summary>
    /// Hyper-text transfer protocol client (sometimes called "User Agent"). Used to make HTTP requests to download web content, upload files and other data or to communicate with various services, among other use cases.
    /// Note that this client only needs to connect to a host once (see [method connect_to_host]) to send multiple requests. Because of this, methods that take URLs usually take just the part after the host instead of the full URL, as the client is already connected to a host. See [method request] for a full example and to get started.
    /// A [code]HTTPClient[/code] should be reused between multiple requests or to connect to different hosts instead of creating one client per request. Supports SSL and SSL server certificate verification. HTTP status codes in the 2xx range indicate success, 3xx redirection (i.e. "try again, but over here"), 4xx something was wrong with the request, and 5xx something went wrong on the server's side.
    /// For more information on HTTP, see https://developer.mozilla.org/en-US/docs/Web/HTTP (or read RFC 2616 to get it straight from the source: https://tools.ietf.org/html/rfc2616).
    /// </summary>
    public class HTTPClient : Reference
    {
        public enum Status
        {
            Disconnected = 0,
            Resolving = 1,
            CantResolve = 2,
            Connecting = 3,
            CantConnect = 4,
            Connected = 5,
            Requesting = 6,
            Body = 7,
            ConnectionError = 8,
            SslHandshakeError = 9
        }

        public enum Method
        {
            Get = 0,
            Head = 1,
            Post = 2,
            Put = 3,
            Delete = 4,
            Options = 5,
            Trace = 6,
            Connect = 7,
            Patch = 8,
            Max = 9
        }

        public enum ResponseCode
        {
            Continue = 100,
            SwitchingProtocols = 101,
            Processing = 102,
            Ok = 200,
            Created = 201,
            Accepted = 202,
            NonAuthoritativeInformation = 203,
            NoContent = 204,
            ResetContent = 205,
            PartialContent = 206,
            MultiStatus = 207,
            AlreadyReported = 208,
            ImUsed = 226,
            MultipleChoices = 300,
            MovedPermanently = 301,
            Found = 302,
            SeeOther = 303,
            NotModified = 304,
            UseProxy = 305,
            SwitchProxy = 306,
            TemporaryRedirect = 307,
            PermanentRedirect = 308,
            BadRequest = 400,
            Unauthorized = 401,
            PaymentRequired = 402,
            Forbidden = 403,
            NotFound = 404,
            MethodNotAllowed = 405,
            NotAcceptable = 406,
            ProxyAuthenticationRequired = 407,
            RequestTimeout = 408,
            Conflict = 409,
            Gone = 410,
            LengthRequired = 411,
            PreconditionFailed = 412,
            RequestEntityTooLarge = 413,
            RequestUriTooLong = 414,
            UnsupportedMediaType = 415,
            RequestedRangeNotSatisfiable = 416,
            ExpectationFailed = 417,
            ImATeapot = 418,
            MisdirectedRequest = 421,
            UnprocessableEntity = 422,
            Locked = 423,
            FailedDependency = 424,
            UpgradeRequired = 426,
            PreconditionRequired = 428,
            TooManyRequests = 429,
            RequestHeaderFieldsTooLarge = 431,
            UnavailableForLegalReasons = 451,
            InternalServerError = 500,
            NotImplemented = 501,
            BadGateway = 502,
            ServiceUnavailable = 503,
            GatewayTimeout = 504,
            HttpVersionNotSupported = 505,
            VariantAlsoNegotiates = 506,
            InsufficientStorage = 507,
            LoopDetected = 508,
            NotExtended = 510,
            NetworkAuthRequired = 511
        }


        /// <summary>
        /// If [code]true[/code], execution will block until all data is read from the response.
        /// </summary>
        public bool BlockingModeEnabled
        {
            get
            {
                return IsBlockingModeEnabled();
            }
            set
            {
                SetBlockingMode(value);
            }
        }

        /// <summary>
        /// The connection to use for this client.
        /// </summary>
        public StreamPeer Connection
        {
            get
            {
                return GetConnection();
            }
            set
            {
                SetConnection(value);
            }
        }


        private const string nativeName = "HTTPClient";

        public HTTPClient() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_HTTPClient_Ctor(this);
        }

        internal HTTPClient(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "connect_to_host");

        /// <summary>
        /// Connect to a host. This needs to be done before any requests are sent.
        /// The host should not have http:// prepended but will strip the protocol identifier if provided.
        /// If no [code]port[/code] is specified (or [code]-1[/code] is used), it is automatically set to 80 for HTTP and 443 for HTTPS (if [code]use_ssl[/code] is enabled).
        /// [code]verify_host[/code] will check the SSL identity of the host if set to [code]true[/code].
        /// </summary>
        [GodotMethod("connect_to_host")]
        public Error ConnectToHost(string host, int port = -1, bool useSsl = false, bool verifyHost = true)
        {
            return NativeCalls.godot_icall_4_404(method_bind_0, Object.GetPtr(this), host, port, useSsl, verifyHost);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_connection");

        [GodotMethod("set_connection")]
        public void SetConnection(StreamPeer connection)
        {
            NativeCalls.godot_icall_1_24(method_bind_1, Object.GetPtr(this), Object.GetPtr(connection));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connection");

        [GodotMethod("get_connection")]
        public StreamPeer GetConnection()
        {
            return NativeCalls.godot_icall_0_405(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "request_raw");

        /// <summary>
        /// Sends a raw request to the connected host. The URL parameter is just the part after the host, so for [code]http://somehost.com/index.php[/code], it is [code]index.php[/code].
        /// Headers are HTTP request headers. For available HTTP methods, see [code]METHOD_*[/code].
        /// Sends the body data raw, as a byte array and does not encode it in any way.
        /// </summary>
        [GodotMethod("request_raw")]
        public Error RequestRaw(HTTPClient.Method method, string url, string[] headers, byte[] body)
        {
            return NativeCalls.godot_icall_4_406(method_bind_3, Object.GetPtr(this), method, url, headers, body);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "request");

        /// <summary>
        /// Sends a request to the connected host. The URL parameter is just the part after the host, so for [code]http://somehost.com/index.php[/code], it is [code]index.php[/code].
        /// Headers are HTTP request headers. For available HTTP methods, see [code]METHOD_*[/code].
        /// To create a POST request with query strings to push to the server, do:
        /// [codeblock]
        /// var fields = {"username" : "user", "password" : "pass"}
        /// var queryString = httpClient.query_string_from_dict(fields)
        /// var headers = ["Content-Type: application/x-www-form-urlencoded", "Content-Length: " + str(queryString.length())]
        /// var result = httpClient.request(httpClient.METHOD_POST, "index.php", headers, queryString)
        /// [/codeblock]
        /// </summary>
        [GodotMethod("request")]
        public Error Request(HTTPClient.Method method, string url, string[] headers, string body = "")
        {
            return NativeCalls.godot_icall_4_407(method_bind_4, Object.GetPtr(this), method, url, headers, body);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "close");

        /// <summary>
        /// Closes the current connection, allowing reuse of this [code]HTTPClient[/code].
        /// </summary>
        [GodotMethod("close")]
        public void Close()
        {
            NativeCalls.godot_icall_0_12(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_response");

        /// <summary>
        /// If [code]true[/code] this [code]HTTPClient[/code] has a response available.
        /// </summary>
        [GodotMethod("has_response")]
        public bool HasResponse()
        {
            return NativeCalls.godot_icall_0_3(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_response_chunked");

        /// <summary>
        /// If [code]true[/code] this [code]HTTPClient[/code] has a response that is chunked.
        /// </summary>
        [GodotMethod("is_response_chunked")]
        public bool IsResponseChunked()
        {
            return NativeCalls.godot_icall_0_3(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_response_code");

        /// <summary>
        /// Returns the response's HTTP status code.
        /// </summary>
        [GodotMethod("get_response_code")]
        public int GetResponseCode()
        {
            return NativeCalls.godot_icall_0_1(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_response_headers");

        /// <summary>
        /// Returns the response headers.
        /// </summary>
        [GodotMethod("get_response_headers")]
        public string[] GetResponseHeaders()
        {
            return NativeCalls.godot_icall_0_72(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_response_headers_as_dictionary");

        /// <summary>
        /// Returns all response headers as dictionary where the case-sensitivity of the keys and values is kept like the server delivers it. A value is a simple String, this string can have more than one value where "; " is used as separator.
        /// Structure: ("key":"value1; value2")
        /// Example: (content-length:12), (Content-Type:application/json; charset=UTF-8)
        /// </summary>
        [GodotMethod("get_response_headers_as_dictionary")]
        public Dictionary<object, object> GetResponseHeadersAsDictionary()
        {
            return NativeCalls.godot_icall_0_169(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_response_body_length");

        /// <summary>
        /// Returns the response's body length.
        /// </summary>
        [GodotMethod("get_response_body_length")]
        public int GetResponseBodyLength()
        {
            return NativeCalls.godot_icall_0_1(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "read_response_body_chunk");

        /// <summary>
        /// Reads one chunk from the response.
        /// </summary>
        [GodotMethod("read_response_body_chunk")]
        public byte[] ReadResponseBodyChunk()
        {
            return NativeCalls.godot_icall_0_132(method_bind_12, Object.GetPtr(this));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_read_chunk_size");

        /// <summary>
        /// Sets the size of the buffer used and maximum bytes to read per iteration. see [method read_response_body_chunk]
        /// </summary>
        [GodotMethod("set_read_chunk_size")]
        public void SetReadChunkSize(int bytes)
        {
            NativeCalls.godot_icall_1_0(method_bind_13, Object.GetPtr(this), bytes);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_blocking_mode");

        [GodotMethod("set_blocking_mode")]
        public void SetBlockingMode(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_14, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_blocking_mode_enabled");

        [GodotMethod("is_blocking_mode_enabled")]
        public bool IsBlockingModeEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_status");

        /// <summary>
        /// Returns a STATUS_* enum constant. Need to call [method poll] in order to get status updates.
        /// </summary>
        [GodotMethod("get_status")]
        public HTTPClient.Status GetStatus()
        {
            return NativeCalls.godot_icall_0_408(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "poll");

        /// <summary>
        /// This needs to be called in order to have any request processed. Check results with [method get_status]
        /// </summary>
        [GodotMethod("poll")]
        public Error Poll()
        {
            return NativeCalls.godot_icall_0_325(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "query_string_from_dict");

        /// <summary>
        /// Generates a GET/POST application/x-www-form-urlencoded style query string from a provided dictionary, e.g.:
        /// [codeblock]
        /// var fields = {"username": "user", "password": "pass"}
        /// String queryString = httpClient.query_string_from_dict(fields)
        /// returns:= "username=user&amp;password=pass"
        /// [/codeblock]
        /// </summary>
        [GodotMethod("query_string_from_dict")]
        public string QueryStringFromDict(Dictionary<object, object> fields)
        {
            return NativeCalls.godot_icall_1_409(method_bind_18, Object.GetPtr(this), fields);
        }
    }
}
