using System;

namespace Godot
{
    /// <summary>
    /// A node with the ability to send HTTP requests. Uses [HTTPClient] internally.
    /// Can be used to make HTTP requests, i.e. download or upload files or web content via HTTP.
    /// </summary>
    public class HTTPRequest : Node
    {
        public enum Result
        {
            Success = 0,
            ChunkedBodySizeMismatch = 1,
            CantConnect = 2,
            CantResolve = 3,
            ConnectionError = 4,
            SslHandshakeError = 5,
            NoResponse = 6,
            BodySizeLimitExceeded = 7,
            RequestFailed = 8,
            DownloadFileCantOpen = 9,
            DownloadFileWriteError = 10,
            RedirectLimitReached = 11
        }


        /// <summary>
        /// The file to download into. Will output any received file into it.
        /// </summary>
        public string DownloadFile
        {
            get
            {
                return GetDownloadFile();
            }
            set
            {
                SetDownloadFile(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] multithreading is used to improve performance.
        /// </summary>
        public bool UseThreads
        {
            get
            {
                return IsUsingThreads();
            }
            set
            {
                SetUseThreads(value);
            }
        }

        /// <summary>
        /// Maximum allowed size for response bodies.
        /// </summary>
        public int BodySizeLimit
        {
            get
            {
                return GetBodySizeLimit();
            }
            set
            {
                SetBodySizeLimit(value);
            }
        }

        /// <summary>
        /// Maximum number of allowed redirects.
        /// </summary>
        public int MaxRedirects
        {
            get
            {
                return GetMaxRedirects();
            }
            set
            {
                SetMaxRedirects(value);
            }
        }


        private const string nativeName = "HTTPRequest";

        public HTTPRequest() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_HTTPRequest_Ctor(this);
        }

        internal HTTPRequest(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "request");

        /// <summary>
        /// &lt;param name="customHeaders"&gt;If the param is null, then the default value is new string[] {}&lt;/param&gt;
        /// </summary>
        [GodotMethod("request")]
        public Error Request(string url, string[] customHeaders = null, bool sslValidateDomain = true, HTTPClient.Method method = (HTTPClient.Method)0, string requestData = "")
        {
            string[] customHeaders_in = customHeaders != null ? customHeaders : new string[] {};
            return NativeCalls.godot_icall_5_410(method_bind_0, Object.GetPtr(this), url, customHeaders_in, sslValidateDomain, method, requestData);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "cancel_request");

        /// <summary>
        /// Cancels the current request.
        /// </summary>
        [GodotMethod("cancel_request")]
        public void CancelRequest()
        {
            NativeCalls.godot_icall_0_12(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_http_client_status");

        /// <summary>
        /// Returns the current status of the underlying [HTTPClient]. See [code]STATUS_*[/code] enum on [HTTPClient].
        /// </summary>
        [GodotMethod("get_http_client_status")]
        public HTTPClient.Status GetHttpClientStatus()
        {
            return NativeCalls.godot_icall_0_408(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_use_threads");

        [GodotMethod("set_use_threads")]
        public void SetUseThreads(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_3, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_using_threads");

        [GodotMethod("is_using_threads")]
        public bool IsUsingThreads()
        {
            return NativeCalls.godot_icall_0_3(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_body_size_limit");

        [GodotMethod("set_body_size_limit")]
        public void SetBodySizeLimit(int bytes)
        {
            NativeCalls.godot_icall_1_0(method_bind_5, Object.GetPtr(this), bytes);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_body_size_limit");

        [GodotMethod("get_body_size_limit")]
        public int GetBodySizeLimit()
        {
            return NativeCalls.godot_icall_0_1(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_max_redirects");

        [GodotMethod("set_max_redirects")]
        public void SetMaxRedirects(int amount)
        {
            NativeCalls.godot_icall_1_0(method_bind_7, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_max_redirects");

        [GodotMethod("get_max_redirects")]
        public int GetMaxRedirects()
        {
            return NativeCalls.godot_icall_0_1(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_download_file");

        [GodotMethod("set_download_file")]
        public void SetDownloadFile(string path)
        {
            NativeCalls.godot_icall_1_43(method_bind_9, Object.GetPtr(this), path);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_download_file");

        [GodotMethod("get_download_file")]
        public string GetDownloadFile()
        {
            return NativeCalls.godot_icall_0_2(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_downloaded_bytes");

        /// <summary>
        /// Returns the amount of bytes this HTTPRequest downloaded.
        /// </summary>
        [GodotMethod("get_downloaded_bytes")]
        public int GetDownloadedBytes()
        {
            return NativeCalls.godot_icall_0_1(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_body_size");

        /// <summary>
        /// Returns the response body length.
        /// </summary>
        [GodotMethod("get_body_size")]
        public int GetBodySize()
        {
            return NativeCalls.godot_icall_0_1(method_bind_12, Object.GetPtr(this));
        }
    }
}
