using System;
using System.Collections.Generic;

namespace Godot
{
    /// <summary>
    /// Native image datatype. Contains image data, which can be converted to a [Texture], and several functions to interact with it. The maximum width and height for an [code]Image[/code] is 16384 pixels.
    /// </summary>
    public class Image : Resource
    {
        public enum AlphaMode
        {
            None = 0,
            Bit = 1,
            Blend = 2
        }

        public enum CompressSource
        {
            Generic = 0,
            Srgb = 1,
            Normal = 2
        }

        public enum Interpolation
        {
            Nearest = 0,
            Bilinear = 1,
            Cubic = 2
        }

        public enum CompressMode
        {
            S3tc = 0,
            Pvrtc2 = 1,
            Pvrtc4 = 2,
            Etc = 3,
            Etc2 = 4
        }

        public enum Format
        {
            L8 = 0,
            La8 = 1,
            R8 = 2,
            Rg8 = 3,
            Rgb8 = 4,
            Rgba8 = 5,
            Rgba4444 = 6,
            Rgba5551 = 7,
            Rf = 8,
            Rgf = 9,
            Rgbf = 10,
            Rgbaf = 11,
            Rh = 12,
            Rgh = 13,
            Rgbh = 14,
            Rgbah = 15,
            Rgbe9995 = 16,
            Dxt1 = 17,
            Dxt3 = 18,
            Dxt5 = 19,
            RgtcR = 20,
            RgtcRg = 21,
            BptcRgba = 22,
            BptcRgbf = 23,
            BptcRgbfu = 24,
            Pvrtc2 = 25,
            Pvrtc2a = 26,
            Pvrtc4 = 27,
            Pvrtc4a = 28,
            Etc = 29,
            Etc2R11 = 30,
            Etc2R11s = 31,
            Etc2Rg11 = 32,
            Etc2Rg11s = 33,
            Etc2Rgb8 = 34,
            Etc2Rgba8 = 35,
            Etc2Rgb8a1 = 36,
            Max = 37
        }


        /// <summary>
        /// Holds all of the image's color data in a given format. See [code]FORMAT_*[/code] constants.
        /// </summary>
        public Dictionary<object, object> Data
        {
            get
            {
                return _GetData();
            }
            set
            {
                _SetData(value);
            }
        }


        private const string nativeName = "Image";

        public Image() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Image_Ctor(this);
        }

        internal Image(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_width");

        /// <summary>
        /// Returns the image's width.
        /// </summary>
        [GodotMethod("get_width")]
        public int GetWidth()
        {
            return NativeCalls.godot_icall_0_1(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_height");

        /// <summary>
        /// Returns the image's height.
        /// </summary>
        [GodotMethod("get_height")]
        public int GetHeight()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_size");

        /// <summary>
        /// Returns the image's size (width and height).
        /// </summary>
        [GodotMethod("get_size")]
        public Vector2 GetSize()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_2, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_mipmaps");

        /// <summary>
        /// Returns [code]true[/code] if the image has generated mipmaps.
        /// </summary>
        [GodotMethod("has_mipmaps")]
        public bool HasMipmaps()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_format");

        /// <summary>
        /// Returns the image's raw data.
        /// </summary>
        [GodotMethod("get_format")]
        public Image.Format GetFormat()
        {
            return NativeCalls.godot_icall_0_418(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_data");

        /// <summary>
        /// Returns the image's raw data.
        /// </summary>
        [GodotMethod("get_data")]
        public byte[] GetData()
        {
            return NativeCalls.godot_icall_0_132(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "convert");

        /// <summary>
        /// Converts the image's format. See [code]FORMAT_*[/code] constants.
        /// </summary>
        [GodotMethod("convert")]
        public void Convert(Image.Format format)
        {
            NativeCalls.godot_icall_1_419(method_bind_6, Object.GetPtr(this), format);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mipmap_offset");

        /// <summary>
        /// Returns the offset where the image's mipmap with index [code]mipmap[/code] is stored in the [code]data[/code] dictionary.
        /// </summary>
        [GodotMethod("get_mipmap_offset")]
        public int GetMipmapOffset(int mipmap)
        {
            return NativeCalls.godot_icall_1_6(method_bind_7, Object.GetPtr(this), mipmap);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "resize_to_po2");

        /// <summary>
        /// Resizes the image to the nearest power of 2 for the width and height. If [code]square[/code] is [code]true[/code] then set width and height to be the same.
        /// </summary>
        [GodotMethod("resize_to_po2")]
        public void ResizeToPo2(bool square = false)
        {
            NativeCalls.godot_icall_1_11(method_bind_8, Object.GetPtr(this), square);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "resize");

        /// <summary>
        /// Resizes the image to the given [code]width[/code] and [code]height[/code]. New pixels are calculated using [code]interpolation[/code]. See [code]interpolation[/code] constants.
        /// </summary>
        [GodotMethod("resize")]
        public void Resize(int width, int height, Image.Interpolation interpolation = (Image.Interpolation)1)
        {
            NativeCalls.godot_icall_3_420(method_bind_9, Object.GetPtr(this), width, height, interpolation);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shrink_x2");

        /// <summary>
        /// Shrinks the image by a factor of 2.
        /// </summary>
        [GodotMethod("shrink_x2")]
        public void ShrinkX2()
        {
            NativeCalls.godot_icall_0_12(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "expand_x2_hq2x");

        /// <summary>
        /// Stretches the image and enlarges it by a factor of 2. No interpolation is done.
        /// </summary>
        [GodotMethod("expand_x2_hq2x")]
        public void ExpandX2Hq2x()
        {
            NativeCalls.godot_icall_0_12(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "crop");

        /// <summary>
        /// Crops the image to the given [code]width[/code] and [code]height[/code]. If the specified size is larger than the current size, the extra area is filled with black pixels.
        /// </summary>
        [GodotMethod("crop")]
        public void Crop(int width, int height)
        {
            NativeCalls.godot_icall_2_32(method_bind_12, Object.GetPtr(this), width, height);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "flip_x");

        /// <summary>
        /// Flips the image horizontally.
        /// </summary>
        [GodotMethod("flip_x")]
        public void FlipX()
        {
            NativeCalls.godot_icall_0_12(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "flip_y");

        /// <summary>
        /// Flips the image vertically.
        /// </summary>
        [GodotMethod("flip_y")]
        public void FlipY()
        {
            NativeCalls.godot_icall_0_12(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "generate_mipmaps");

        /// <summary>
        /// Generates mipmaps for the image. Mipmaps are pre-calculated and lower resolution copies of the image. Mipmaps are automatically used if the image needs to be scaled down when rendered. This improves image quality and the performance of the rendering. Returns an error if the image is compressed, in a custom format or if the image's width/height is 0.
        /// </summary>
        [GodotMethod("generate_mipmaps")]
        public Error GenerateMipmaps()
        {
            return NativeCalls.godot_icall_0_325(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_mipmaps");

        /// <summary>
        /// Removes the image's mipmaps.
        /// </summary>
        [GodotMethod("clear_mipmaps")]
        public void ClearMipmaps()
        {
            NativeCalls.godot_icall_0_12(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create");

        /// <summary>
        /// Creates an empty image of given size and format. See [code]FORMAT_*[/code] constants. If [code]use_mipmaps[/code] is true then generate mipmaps for this image. See the [code]generate_mipmaps[/code] method.
        /// </summary>
        [GodotMethod("create")]
        public void Create(int width, int height, bool useMipmaps, Image.Format format)
        {
            NativeCalls.godot_icall_4_421(method_bind_17, Object.GetPtr(this), width, height, useMipmaps, format);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create_from_data");

        /// <summary>
        /// Creates a new image of given size and format. See [code]FORMAT_*[/code] constants. Fills the image with the given raw data. If [code]use_mipmaps[/code] is true then generate mipmaps for this image. See the [code]generate_mipmaps[/code] method.
        /// </summary>
        [GodotMethod("create_from_data")]
        public void CreateFromData(int width, int height, bool useMipmaps, Image.Format format, byte[] data)
        {
            NativeCalls.godot_icall_5_422(method_bind_18, Object.GetPtr(this), width, height, useMipmaps, format, data);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_empty");

        /// <summary>
        /// Returns [code]true[/code] if the image has no data.
        /// </summary>
        [GodotMethod("is_empty")]
        public bool IsEmpty()
        {
            return NativeCalls.godot_icall_0_3(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "load");

        /// <summary>
        /// Loads an image from file [code]path[/code].
        /// </summary>
        [GodotMethod("load")]
        public Error Load(string path)
        {
            return NativeCalls.godot_icall_1_170(method_bind_20, Object.GetPtr(this), path);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "save_png");

        /// <summary>
        /// Saves the image as a PNG file to [code]path[/code].
        /// </summary>
        [GodotMethod("save_png")]
        public Error SavePng(string path)
        {
            return NativeCalls.godot_icall_1_170(method_bind_21, Object.GetPtr(this), path);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "detect_alpha");

        /// <summary>
        /// Returns ALPHA_BLEND if the image has data for alpha values. Returns ALPHA_BIT if all the alpha values are below a certain threshold or the maximum value. Returns ALPHA_NONE if no data for alpha values is found.
        /// </summary>
        [GodotMethod("detect_alpha")]
        public Image.AlphaMode DetectAlpha()
        {
            return NativeCalls.godot_icall_0_423(method_bind_22, Object.GetPtr(this));
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_invisible");

        /// <summary>
        /// Returns [code]true[/code] if all the image's pixels have an alpha value of 0. Returns [code]false[/code] if any pixel has an alpha value higher than 0.
        /// </summary>
        [GodotMethod("is_invisible")]
        public bool IsInvisible()
        {
            return NativeCalls.godot_icall_0_3(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "compress");

        /// <summary>
        /// Compresses the image to use less memory. Can not directly access pixel data while the image is compressed. Returns error if the chosen compression mode is not available. See [code]COMPRESS_*[/code] constants.
        /// </summary>
        [GodotMethod("compress")]
        public Error Compress(Image.CompressMode mode, Image.CompressSource source, float lossyQuality)
        {
            return NativeCalls.godot_icall_3_424(method_bind_24, Object.GetPtr(this), mode, source, lossyQuality);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "decompress");

        /// <summary>
        /// Decompresses the image if it is compressed. Returns an error if decompress function is not available.
        /// </summary>
        [GodotMethod("decompress")]
        public Error Decompress()
        {
            return NativeCalls.godot_icall_0_325(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_compressed");

        /// <summary>
        /// Returns [code]true[/code] if the image is compressed.
        /// </summary>
        [GodotMethod("is_compressed")]
        public bool IsCompressed()
        {
            return NativeCalls.godot_icall_0_3(method_bind_26, Object.GetPtr(this));
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "fix_alpha_edges");

        /// <summary>
        /// Blends low-alpha pixels with nearby pixels.
        /// </summary>
        [GodotMethod("fix_alpha_edges")]
        public void FixAlphaEdges()
        {
            NativeCalls.godot_icall_0_12(method_bind_27, Object.GetPtr(this));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "premultiply_alpha");

        /// <summary>
        /// Multiplies color values with alpha values. Resulting color values for a pixel are [code](color * alpha)/256[/code].
        /// </summary>
        [GodotMethod("premultiply_alpha")]
        public void PremultiplyAlpha()
        {
            NativeCalls.godot_icall_0_12(method_bind_28, Object.GetPtr(this));
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "srgb_to_linear");

        /// <summary>
        /// Converts the raw data from the sRGB colorspace to a linear scale.
        /// </summary>
        [GodotMethod("srgb_to_linear")]
        public void SrgbToLinear()
        {
            NativeCalls.godot_icall_0_12(method_bind_29, Object.GetPtr(this));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "normalmap_to_xy");

        /// <summary>
        /// Converts the image's data to represent coordinates on a 3D plane. This is used when the image represents a normalmap. A normalmap can add lots of detail to a 3D surface without increasing the polygon count.
        /// </summary>
        [GodotMethod("normalmap_to_xy")]
        public void NormalmapToXy()
        {
            NativeCalls.godot_icall_0_12(method_bind_30, Object.GetPtr(this));
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "blit_rect");

        /// <summary>
        /// Copies [code]src_rect[/code] from [code]src[/code] image to this image at coordinates [code]dst[/code].
        /// </summary>
        [GodotMethod("blit_rect")]
        public void BlitRect(Image src, Rect2 srcRect, Vector2 dst)
        {
            NativeCalls.godot_icall_3_425(method_bind_31, Object.GetPtr(this), Object.GetPtr(src), ref srcRect, ref dst);
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "blit_rect_mask");

        /// <summary>
        /// Blits [code]src_rect[/code] area from [code]src[/code] image to this image at the coordinates given by [code]dst[/code]. [code]src[/code] pixel is copied onto [code]dst[/code] if the corresponding [code]mask[/code] pixel's alpha value is not 0. [code]src[/code] image and [code]mask[/code] image [b]must[/b] have the same size (width and height) but they can have different formats.
        /// </summary>
        [GodotMethod("blit_rect_mask")]
        public void BlitRectMask(Image src, Image mask, Rect2 srcRect, Vector2 dst)
        {
            NativeCalls.godot_icall_4_426(method_bind_32, Object.GetPtr(this), Object.GetPtr(src), Object.GetPtr(mask), ref srcRect, ref dst);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "blend_rect");

        /// <summary>
        /// Alpha-blends [code]src_rect[/code] from [code]src[/code] image to this image at coordinates [code]dest[/code].
        /// </summary>
        [GodotMethod("blend_rect")]
        public void BlendRect(Image src, Rect2 srcRect, Vector2 dst)
        {
            NativeCalls.godot_icall_3_425(method_bind_33, Object.GetPtr(this), Object.GetPtr(src), ref srcRect, ref dst);
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "blend_rect_mask");

        /// <summary>
        /// Alpha-blends [code]src_rect[/code] from [code]src[/code] image to this image using [code]mask[/code] image at coordinates [code]dst[/code]. Alpha channels are required for both [code]src[/code] and [code]mask[/code]. [code]dst[/code] pixels and [code]src[/code] pixels will blend if the corresponding mask pixel's alpha value is not 0. [code]src[/code] image and [code]mask[/code] image [b]must[/b] have the same size (width and height) but they can have different formats.
        /// </summary>
        [GodotMethod("blend_rect_mask")]
        public void BlendRectMask(Image src, Image mask, Rect2 srcRect, Vector2 dst)
        {
            NativeCalls.godot_icall_4_426(method_bind_34, Object.GetPtr(this), Object.GetPtr(src), Object.GetPtr(mask), ref srcRect, ref dst);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "fill");

        /// <summary>
        /// Fills the image with a given [Color].
        /// </summary>
        [GodotMethod("fill")]
        public void Fill(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_35, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_used_rect");

        /// <summary>
        /// Returns a [Rect2] enclosing the visible portion of the image.
        /// </summary>
        [GodotMethod("get_used_rect")]
        public Rect2 GetUsedRect()
        {
            object ret = NativeCalls.godot_icall_0_121(method_bind_36, Object.GetPtr(this));
            return (Rect2)ret;
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rect");

        /// <summary>
        /// Returns a new image that is a copy of the image's area specified with [code]rect[/code].
        /// </summary>
        [GodotMethod("get_rect")]
        public Image GetRect(Rect2 rect)
        {
            return NativeCalls.godot_icall_1_427(method_bind_37, Object.GetPtr(this), ref rect);
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "copy_from");

        /// <summary>
        /// Copies [code]src[/code] image to this image.
        /// </summary>
        [GodotMethod("copy_from")]
        public void CopyFrom(Image src)
        {
            NativeCalls.godot_icall_1_24(method_bind_38, Object.GetPtr(this), Object.GetPtr(src));
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_data");

        internal void _SetData(Dictionary<object, object> data)
        {
            NativeCalls.godot_icall_1_168(method_bind_39, Object.GetPtr(this), data);
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_data");

        internal Dictionary<object, object> _GetData()
        {
            return NativeCalls.godot_icall_0_169(method_bind_40, Object.GetPtr(this));
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "lock");

        /// <summary>
        /// Locks the data for writing access.
        /// </summary>
        [GodotMethod("lock")]
        public void Lock()
        {
            NativeCalls.godot_icall_0_12(method_bind_41, Object.GetPtr(this));
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "unlock");

        /// <summary>
        /// Unlocks the data and prevents changes.
        /// </summary>
        [GodotMethod("unlock")]
        public void Unlock()
        {
            NativeCalls.godot_icall_0_12(method_bind_42, Object.GetPtr(this));
        }

        private IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_pixel");

        /// <summary>
        /// Sets the [Color] of the pixel at [code](x, y)[/code] if the image is locked. Example:
        /// [codeblock]
        /// var img = Image.new()
        /// img.create(img_width, img_height, false, Image.FORMAT_RGBA8)
        /// img.lock()
        /// img.set_pixel(x, y, color) # Works
        /// img.unlock()
        /// img.set_pixel(x, y, color) # Does not have an effect
        /// [/codeblock]
        /// </summary>
        [GodotMethod("set_pixel")]
        public void SetPixel(int x, int y, Color color)
        {
            NativeCalls.godot_icall_3_428(method_bind_43, Object.GetPtr(this), x, y, ref color);
        }

        private IntPtr method_bind_44 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_pixel");

        /// <summary>
        /// Returns the color of the pixel at [code](x, y)[/code] if the image is locked. If the image is unlocked it always returns a [Color] with the value [code](0, 0, 0, 1.0)[/code].
        /// </summary>
        [GodotMethod("get_pixel")]
        public Color GetPixel(int x, int y)
        {
            object ret = NativeCalls.godot_icall_2_429(method_bind_44, Object.GetPtr(this), x, y);
            return (Color)ret;
        }

        private IntPtr method_bind_45 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "load_png_from_buffer");

        /// <summary>
        /// </summary>
        [GodotMethod("load_png_from_buffer")]
        public Error LoadPngFromBuffer(byte[] buffer)
        {
            return NativeCalls.godot_icall_1_430(method_bind_45, Object.GetPtr(this), buffer);
        }

        private IntPtr method_bind_46 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "load_jpg_from_buffer");

        /// <summary>
        /// </summary>
        [GodotMethod("load_jpg_from_buffer")]
        public Error LoadJpgFromBuffer(byte[] buffer)
        {
            return NativeCalls.godot_icall_1_430(method_bind_46, Object.GetPtr(this), buffer);
        }
    }
}
