using System;

namespace Godot
{
    /// <summary>
    /// A [Texture] based on an [Image]. Can be created from an [Image] with [method create_from_image].
    /// </summary>
    public class ImageTexture : Texture
    {
        public enum StorageEnum
        {
            Raw = 0,
            CompressLossy = 1,
            CompressLossless = 2
        }


        /// <summary>
        /// The storage type (raw, lossy, or compressed).
        /// </summary>
        public ImageTexture.StorageEnum Storage
        {
            get
            {
                return GetStorage();
            }
            set
            {
                SetStorage(value);
            }
        }

        /// <summary>
        /// The storage quality for [code]ImageTexture[/code].STORAGE_COMPRESS_LOSSY.
        /// </summary>
        public float LossyQuality
        {
            get
            {
                return GetLossyStorageQuality();
            }
            set
            {
                SetLossyStorageQuality(value);
            }
        }


        private const string nativeName = "ImageTexture";

        public ImageTexture() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ImageTexture_Ctor(this);
        }

        internal ImageTexture(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create");

        /// <summary>
        /// Create a new [code]ImageTexture[/code] with "width" and "height".
        /// "format" one of [Image].FORMAT_*.
        /// "flags" one or more of [Texture].FLAG_*.
        /// </summary>
        [GodotMethod("create")]
        public void Create(int width, int height, Image.Format format, int flags = 7)
        {
            NativeCalls.godot_icall_4_431(method_bind_0, Object.GetPtr(this), width, height, format, flags);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create_from_image");

        /// <summary>
        /// Create a new [code]ImageTexture[/code] from an [Image] with "flags" from [Texture].FLAG_*.
        /// </summary>
        [GodotMethod("create_from_image")]
        public void CreateFromImage(Image image, int flags = 7)
        {
            NativeCalls.godot_icall_2_432(method_bind_1, Object.GetPtr(this), Object.GetPtr(image), flags);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_format");

        /// <summary>
        /// Return the format of the [code]ImageTexture[/code], one of [Image].FORMAT_*.
        /// </summary>
        [GodotMethod("get_format")]
        public Image.Format GetFormat()
        {
            return NativeCalls.godot_icall_0_418(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "load");

        /// <summary>
        /// Load an [code]ImageTexture[/code].
        /// </summary>
        [GodotMethod("load")]
        public void Load(string path)
        {
            NativeCalls.godot_icall_1_43(method_bind_3, Object.GetPtr(this), path);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_data");

        /// <summary>
        /// Set the [Image] of this [code]ImageTexture[/code].
        /// </summary>
        [GodotMethod("set_data")]
        public void SetData(Image image)
        {
            NativeCalls.godot_icall_1_24(method_bind_4, Object.GetPtr(this), Object.GetPtr(image));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_storage");

        [GodotMethod("set_storage")]
        public void SetStorage(ImageTexture.StorageEnum mode)
        {
            NativeCalls.godot_icall_1_433(method_bind_5, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_storage");

        [GodotMethod("get_storage")]
        public ImageTexture.StorageEnum GetStorage()
        {
            return NativeCalls.godot_icall_0_434(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_lossy_storage_quality");

        [GodotMethod("set_lossy_storage_quality")]
        public void SetLossyStorageQuality(float quality)
        {
            NativeCalls.godot_icall_1_10(method_bind_7, Object.GetPtr(this), quality);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_lossy_storage_quality");

        [GodotMethod("get_lossy_storage_quality")]
        public float GetLossyStorageQuality()
        {
            return NativeCalls.godot_icall_0_9(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_size_override");

        /// <summary>
        /// Resizes the [code]ImageTexture[/code] to the specified dimensions.
        /// </summary>
        [GodotMethod("set_size_override")]
        public void SetSizeOverride(Vector2 size)
        {
            NativeCalls.godot_icall_1_45(method_bind_9, Object.GetPtr(this), ref size);
        }
    }
}
