using System;

namespace Godot
{
    /// <summary>
    /// A Singleton that deals with inputs. This includes key presses, mouse buttons and movement, joypads, and input actions. Actions and their events can be set in the Project Settings / Input Map tab. Or be set with [InputMap].
    /// </summary>
    public static class Input
    {
        public enum MouseMode
        {
            Visible = 0,
            Hidden = 1,
            Captured = 2,
            Confined = 3
        }

        public enum CursorShape
        {
            Arrow = 0,
            Ibeam = 1,
            PointingHand = 2,
            Cross = 3,
            Wait = 4,
            Busy = 5,
            Drag = 6,
            CanDrop = 7,
            Forbidden = 8,
            Vsize = 9,
            Hsize = 10,
            Bdiagsize = 11,
            Fdiagsize = 12,
            Move = 13,
            Vsplit = 14,
            Hsplit = 15,
            Help = 16
        }


        private const string nativeName = "Input";
        internal static IntPtr ptr = NativeCalls.godot_icall_Input_get_singleton();

        private static IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_key_pressed");

        /// <summary>
        /// Returns [code]true[/code] if you are pressing the key. You can pass [code]KEY_*[/code], which are pre-defined constants listed in [@GlobalScope].
        /// </summary>
        [GodotMethod("is_key_pressed")]
        public static bool IsKeyPressed(int scancode)
        {
            return NativeCalls.godot_icall_1_29(method_bind_0, ptr, scancode);
        }

        private static IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_mouse_button_pressed");

        /// <summary>
        /// Returns [code]true[/code] if you are pressing the mouse button. You can pass [code]BUTTON_*[/code], which are pre-defined constants listed in [@GlobalScope].
        /// </summary>
        [GodotMethod("is_mouse_button_pressed")]
        public static bool IsMouseButtonPressed(int button)
        {
            return NativeCalls.godot_icall_1_29(method_bind_1, ptr, button);
        }

        private static IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_joy_button_pressed");

        /// <summary>
        /// Returns [code]true[/code] if you are pressing the joypad button. (see [code]JOY_*[/code] constants in [@GlobalScope])
        /// </summary>
        [GodotMethod("is_joy_button_pressed")]
        public static bool IsJoyButtonPressed(int device, int button)
        {
            return NativeCalls.godot_icall_2_33(method_bind_2, ptr, device, button);
        }

        private static IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_action_pressed");

        /// <summary>
        /// Returns [code]true[/code] if you are pressing the action event.
        /// </summary>
        [GodotMethod("is_action_pressed")]
        public static bool IsActionPressed(string action)
        {
            return NativeCalls.godot_icall_1_70(method_bind_3, ptr, action);
        }

        private static IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_action_just_pressed");

        /// <summary>
        /// Returns [code]true[/code] when you start pressing the action event.
        /// </summary>
        [GodotMethod("is_action_just_pressed")]
        public static bool IsActionJustPressed(string action)
        {
            return NativeCalls.godot_icall_1_70(method_bind_4, ptr, action);
        }

        private static IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_action_just_released");

        /// <summary>
        /// Returns [code]true[/code] when you stop pressing the action event.
        /// </summary>
        [GodotMethod("is_action_just_released")]
        public static bool IsActionJustReleased(string action)
        {
            return NativeCalls.godot_icall_1_70(method_bind_5, ptr, action);
        }

        private static IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_joy_mapping");

        /// <summary>
        /// Add a new mapping entry (in SDL2 format) to the mapping database. Optionally update already connected devices.
        /// </summary>
        [GodotMethod("add_joy_mapping")]
        public static void AddJoyMapping(string mapping, bool updateExisting = false)
        {
            NativeCalls.godot_icall_2_92(method_bind_6, ptr, mapping, updateExisting);
        }

        private static IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_joy_mapping");

        /// <summary>
        /// Removes all mappings from the internal db that match the given uid.
        /// </summary>
        [GodotMethod("remove_joy_mapping")]
        public static void RemoveJoyMapping(string guid)
        {
            NativeCalls.godot_icall_1_43(method_bind_7, ptr, guid);
        }

        private static IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "joy_connection_changed");

        /// <summary>
        /// </summary>
        [GodotMethod("joy_connection_changed")]
        public static void JoyConnectionChanged(int device, bool connected, string name, string guid)
        {
            NativeCalls.godot_icall_4_438(method_bind_8, ptr, device, connected, name, guid);
        }

        private static IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_joy_known");

        /// <summary>
        /// Returns [code]true[/code] if the system knows the specified device. This means that it sets all button and axis indices exactly as defined in the [code]JOY_*[/code] constants (see [@GlobalScope]). Unknown joypads are not expected to match these constants, but you can still retrieve events from them.
        /// </summary>
        [GodotMethod("is_joy_known")]
        public static bool IsJoyKnown(int device)
        {
            return NativeCalls.godot_icall_1_29(method_bind_9, ptr, device);
        }

        private static IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_joy_axis");

        /// <summary>
        /// Returns the current value of the joypad axis at given index (see [code]JOY_*[/code] constants in [@GlobalScope])
        /// </summary>
        [GodotMethod("get_joy_axis")]
        public static float GetJoyAxis(int device, int axis)
        {
            return NativeCalls.godot_icall_2_56(method_bind_10, ptr, device, axis);
        }

        private static IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_joy_name");

        /// <summary>
        /// Returns the name of the joypad at the specified device index
        /// </summary>
        [GodotMethod("get_joy_name")]
        public static string GetJoyName(int device)
        {
            return NativeCalls.godot_icall_1_108(method_bind_11, ptr, device);
        }

        private static IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_joy_guid");

        /// <summary>
        /// Returns a SDL2 compatible device guid on platforms that use gamepad remapping. Returns "Default Gamepad" otherwise.
        /// </summary>
        [GodotMethod("get_joy_guid")]
        public static string GetJoyGuid(int device)
        {
            return NativeCalls.godot_icall_1_108(method_bind_12, ptr, device);
        }

        private static IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_connected_joypads");

        /// <summary>
        /// Returns an [Array] containing the device IDs of all currently connected joypads.
        /// </summary>
        [GodotMethod("get_connected_joypads")]
        public static object[] GetConnectedJoypads()
        {
            return NativeCalls.godot_icall_0_21(method_bind_13, ptr);
        }

        private static IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_joy_vibration_strength");

        /// <summary>
        /// Returns the strength of the joypad vibration: x is the strength of the weak motor, and y is the strength of the strong motor.
        /// </summary>
        [GodotMethod("get_joy_vibration_strength")]
        public static Vector2 GetJoyVibrationStrength(int device)
        {
            object ret = NativeCalls.godot_icall_1_282(method_bind_14, ptr, device);
            return (Vector2)ret;
        }

        private static IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_joy_vibration_duration");

        /// <summary>
        /// Returns the duration of the current vibration effect in seconds.
        /// </summary>
        [GodotMethod("get_joy_vibration_duration")]
        public static float GetJoyVibrationDuration(int device)
        {
            return NativeCalls.godot_icall_1_7(method_bind_15, ptr, device);
        }

        private static IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_joy_button_string");

        /// <summary>
        /// </summary>
        [GodotMethod("get_joy_button_string")]
        public static string GetJoyButtonString(int buttonIndex)
        {
            return NativeCalls.godot_icall_1_108(method_bind_16, ptr, buttonIndex);
        }

        private static IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_joy_button_index_from_string");

        /// <summary>
        /// </summary>
        [GodotMethod("get_joy_button_index_from_string")]
        public static int GetJoyButtonIndexFromString(string button)
        {
            return NativeCalls.godot_icall_1_87(method_bind_17, ptr, button);
        }

        private static IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_joy_axis_string");

        /// <summary>
        /// </summary>
        [GodotMethod("get_joy_axis_string")]
        public static string GetJoyAxisString(int axisIndex)
        {
            return NativeCalls.godot_icall_1_108(method_bind_18, ptr, axisIndex);
        }

        private static IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_joy_axis_index_from_string");

        /// <summary>
        /// </summary>
        [GodotMethod("get_joy_axis_index_from_string")]
        public static int GetJoyAxisIndexFromString(string axis)
        {
            return NativeCalls.godot_icall_1_87(method_bind_19, ptr, axis);
        }

        private static IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "start_joy_vibration");

        /// <summary>
        /// Starts to vibrate the joypad. Joypads usually come with two rumble motors, a strong and a weak one. weak_magnitude is the strength of the weak motor (between 0 and 1) and strong_magnitude is the strength of the strong motor (between 0 and 1). duration is the duration of the effect in seconds (a duration of 0 will try to play the vibration indefinitely).
        /// Note that not every hardware is compatible with long effect durations, it is recommended to restart an effect if in need to play it for more than a few seconds.
        /// </summary>
        [GodotMethod("start_joy_vibration")]
        public static void StartJoyVibration(int device, float weakMagnitude, float strongMagnitude, float duration = (float)0)
        {
            NativeCalls.godot_icall_4_439(method_bind_20, ptr, device, weakMagnitude, strongMagnitude, duration);
        }

        private static IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "stop_joy_vibration");

        /// <summary>
        /// Stops the vibration of the joypad.
        /// </summary>
        [GodotMethod("stop_joy_vibration")]
        public static void StopJoyVibration(int device)
        {
            NativeCalls.godot_icall_1_0(method_bind_21, ptr, device);
        }

        private static IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_gravity");

        /// <summary>
        /// </summary>
        [GodotMethod("get_gravity")]
        public static Vector3 GetGravity()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_22, ptr);
            return (Vector3)ret;
        }

        private static IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_accelerometer");

        /// <summary>
        /// If the device has an accelerometer, this will return the movement.
        /// </summary>
        [GodotMethod("get_accelerometer")]
        public static Vector3 GetAccelerometer()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_23, ptr);
            return (Vector3)ret;
        }

        private static IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_magnetometer");

        /// <summary>
        /// If the device has a magnetometer, this will return the magnetic field strength in micro-Tesla for all axes.
        /// </summary>
        [GodotMethod("get_magnetometer")]
        public static Vector3 GetMagnetometer()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_24, ptr);
            return (Vector3)ret;
        }

        private static IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_gyroscope");

        /// <summary>
        /// If the device has a gyroscope, this will return the rate of rotation in rad/s around a device's x, y, and z axis.
        /// </summary>
        [GodotMethod("get_gyroscope")]
        public static Vector3 GetGyroscope()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_25, ptr);
            return (Vector3)ret;
        }

        private static IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_last_mouse_speed");

        /// <summary>
        /// Returns the mouse speed for the last time the cursor was moved, and this until the next frame where the mouse moves. This means that even if the mouse is not moving, this function will still return the value of the last motion.
        /// </summary>
        [GodotMethod("get_last_mouse_speed")]
        public static Vector2 GetLastMouseSpeed()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_26, ptr);
            return (Vector2)ret;
        }

        private static IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mouse_button_mask");

        /// <summary>
        /// Returns mouse buttons as a bitmask. If multiple mouse buttons are pressed at the same time the bits are added together.
        /// </summary>
        [GodotMethod("get_mouse_button_mask")]
        public static int GetMouseButtonMask()
        {
            return NativeCalls.godot_icall_0_1(method_bind_27, ptr);
        }

        private static IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_mouse_mode");

        /// <summary>
        /// Set the mouse mode. See the constants for more information.
        /// </summary>
        [GodotMethod("set_mouse_mode")]
        public static void SetMouseMode(Input.MouseMode mode)
        {
            NativeCalls.godot_icall_1_440(method_bind_28, ptr, mode);
        }

        private static IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mouse_mode");

        /// <summary>
        /// Return the mouse mode. See the constants for more information.
        /// </summary>
        [GodotMethod("get_mouse_mode")]
        public static Input.MouseMode GetMouseMode()
        {
            return NativeCalls.godot_icall_0_441(method_bind_29, ptr);
        }

        private static IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "warp_mouse_position");

        /// <summary>
        /// Sets the mouse position to the specified vector.
        /// </summary>
        [GodotMethod("warp_mouse_position")]
        public static void WarpMousePosition(Vector2 to)
        {
            NativeCalls.godot_icall_1_45(method_bind_30, ptr, ref to);
        }

        private static IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "action_press");

        /// <summary>
        /// This will simulate pressing the specificed action.
        /// </summary>
        [GodotMethod("action_press")]
        public static void ActionPress(string action)
        {
            NativeCalls.godot_icall_1_43(method_bind_31, ptr, action);
        }

        private static IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "action_release");

        /// <summary>
        /// If the specified action is already pressed, this will release it.
        /// </summary>
        [GodotMethod("action_release")]
        public static void ActionRelease(string action)
        {
            NativeCalls.godot_icall_1_43(method_bind_32, ptr, action);
        }

        private static IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_custom_mouse_cursor");

        /// <summary>
        /// Set a custom mouse cursor image, which is only visible inside the game window. The hotspot can also be specified. See enum [code]CURSOR_*[/code] for the list of shapes.
        /// &lt;param name="hotspot"&gt;If the param is null, then the default value is new Vector2(0, 0)&lt;/param&gt;
        /// </summary>
        [GodotMethod("set_custom_mouse_cursor")]
        public static void SetCustomMouseCursor(Resource image, Input.CursorShape shape = (Input.CursorShape)0, Nullable<Vector2> hotspot = null)
        {
            Vector2 hotspot_in = hotspot.HasValue ? hotspot.Value : new Vector2(0, 0);
            NativeCalls.godot_icall_3_442(method_bind_33, ptr, Object.GetPtr(image), shape, ref hotspot_in);
        }

        private static IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "parse_input_event");

        /// <summary>
        /// </summary>
        [GodotMethod("parse_input_event")]
        public static void ParseInputEvent(InputEvent @event)
        {
            NativeCalls.godot_icall_1_24(method_bind_34, ptr, Object.GetPtr(@event));
        }
    }
}
