using System;

namespace Godot
{
    /// <summary>
    /// Base class of all sort of input event. See [method Node._input].
    /// </summary>
    public class InputEvent : Resource
    {
        /// <summary>
        /// The event's device ID.
        /// </summary>
        public int Device
        {
            get
            {
                return GetDevice();
            }
            set
            {
                SetDevice(value);
            }
        }


        private const string nativeName = "InputEvent";

        internal InputEvent() {}

        internal InputEvent(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_device");

        [GodotMethod("set_device")]
        public void SetDevice(int device)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, Object.GetPtr(this), device);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_device");

        [GodotMethod("get_device")]
        public int GetDevice()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_pressed");

        /// <summary>
        /// Returns [code]true[/code] if this input event is pressed. Not relevant for the event types [code]MOUSE_MOTION[/code], [code]SCREEN_DRAG[/code] or [code]NONE[/code].
        /// </summary>
        [GodotMethod("is_pressed")]
        public bool IsPressed()
        {
            return NativeCalls.godot_icall_0_3(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_action");

        /// <summary>
        /// Returns [code]true[/code] if this input event matches a pre-defined action of any type.
        /// </summary>
        [GodotMethod("is_action")]
        public bool IsAction(string action)
        {
            return NativeCalls.godot_icall_1_70(method_bind_3, Object.GetPtr(this), action);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_action_pressed");

        /// <summary>
        /// Returns [code]true[/code] if the given action is being pressed (and is not an echo event for KEY events). Not relevant for the event types [code]MOUSE_MOTION[/code], [code]SCREEN_DRAG[/code] or [code]NONE[/code].
        /// </summary>
        [GodotMethod("is_action_pressed")]
        public bool IsActionPressed(string action)
        {
            return NativeCalls.godot_icall_1_70(method_bind_4, Object.GetPtr(this), action);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_action_released");

        /// <summary>
        /// Returns [code]true[/code] if the given action is released (i.e. not pressed). Not relevant for the event types [code]MOUSE_MOTION[/code], [code]SCREEN_DRAG[/code] or [code]NONE[/code].
        /// </summary>
        [GodotMethod("is_action_released")]
        public bool IsActionReleased(string action)
        {
            return NativeCalls.godot_icall_1_70(method_bind_5, Object.GetPtr(this), action);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_echo");

        /// <summary>
        /// Returns [code]true[/code] if this input event is an echo event (only for events of type KEY).
        /// </summary>
        [GodotMethod("is_echo")]
        public bool IsEcho()
        {
            return NativeCalls.godot_icall_0_3(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "as_text");

        /// <summary>
        /// Returns a [String] representation of the event.
        /// </summary>
        [GodotMethod("as_text")]
        public string AsText()
        {
            return NativeCalls.godot_icall_0_2(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "action_match");

        /// <summary>
        /// Returns [code]true[/code] if this event matches [code]event[/code].
        /// </summary>
        [GodotMethod("action_match")]
        public bool ActionMatch(InputEvent @event)
        {
            return NativeCalls.godot_icall_1_443(method_bind_8, Object.GetPtr(this), Object.GetPtr(@event));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shortcut_match");

        /// <summary>
        /// </summary>
        [GodotMethod("shortcut_match")]
        public bool ShortcutMatch(InputEvent @event)
        {
            return NativeCalls.godot_icall_1_443(method_bind_9, Object.GetPtr(this), Object.GetPtr(@event));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_action_type");

        /// <summary>
        /// Returns [code]true[/code] if this input event's type is one of the [code]InputEvent[/code] constants.
        /// </summary>
        [GodotMethod("is_action_type")]
        public bool IsActionType()
        {
            return NativeCalls.godot_icall_0_3(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "xformed_by");

        /// <summary>
        /// &lt;param name="localOfs"&gt;If the param is null, then the default value is new Vector2(0, 0)&lt;/param&gt;
        /// </summary>
        [GodotMethod("xformed_by")]
        public InputEvent XformedBy(Transform2D xform, Nullable<Vector2> localOfs = null)
        {
            Vector2 localOfs_in = localOfs.HasValue ? localOfs.Value : new Vector2(0, 0);
            return NativeCalls.godot_icall_2_444(method_bind_11, Object.GetPtr(this), ref xform, ref localOfs_in);
        }
    }
}
