using System;

namespace Godot
{
    /// <summary>
    /// Stores key presses on the keyboard. Supports key presses, key releases and [member echo] events.
    /// </summary>
    public class InputEventKey : InputEventWithModifiers
    {
        /// <summary>
        /// If [code]true[/code] the key's state is pressed. If [code]false[/code] the key's state is released.
        /// </summary>
        public bool Pressed
        {
            get
            {
                return IsPressed();
            }
            set
            {
                SetPressed(value);
            }
        }

        /// <summary>
        /// Key scancode, one of the [code]KEY_*[/code] constants in [@global Scope].
        /// </summary>
        public int Scancode
        {
            get
            {
                return GetScancode();
            }
            set
            {
                SetScancode(value);
            }
        }

        /// <summary>
        /// Key unicode identifier when relevant.
        /// </summary>
        public int Unicode
        {
            get
            {
                return GetUnicode();
            }
            set
            {
                SetUnicode(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the key was already pressed before this event. It means the user is holding the key down.
        /// </summary>
        public bool Echo
        {
            get
            {
                return IsEcho();
            }
            set
            {
                SetEcho(value);
            }
        }


        private const string nativeName = "InputEventKey";

        public InputEventKey() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_InputEventKey_Ctor(this);
        }

        internal InputEventKey(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_pressed");

        [GodotMethod("set_pressed")]
        public void SetPressed(bool pressed)
        {
            NativeCalls.godot_icall_1_11(method_bind_0, Object.GetPtr(this), pressed);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_scancode");

        [GodotMethod("set_scancode")]
        public void SetScancode(int scancode)
        {
            NativeCalls.godot_icall_1_0(method_bind_1, Object.GetPtr(this), scancode);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_scancode");

        [GodotMethod("get_scancode")]
        public int GetScancode()
        {
            return NativeCalls.godot_icall_0_1(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_unicode");

        [GodotMethod("set_unicode")]
        public void SetUnicode(int unicode)
        {
            NativeCalls.godot_icall_1_0(method_bind_3, Object.GetPtr(this), unicode);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_unicode");

        [GodotMethod("get_unicode")]
        public int GetUnicode()
        {
            return NativeCalls.godot_icall_0_1(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_echo");

        [GodotMethod("set_echo")]
        public void SetEcho(bool echo)
        {
            NativeCalls.godot_icall_1_11(method_bind_5, Object.GetPtr(this), echo);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_scancode_with_modifiers");

        /// <summary>
        /// </summary>
        [GodotMethod("get_scancode_with_modifiers")]
        public int GetScancodeWithModifiers()
        {
            return NativeCalls.godot_icall_0_1(method_bind_6, Object.GetPtr(this));
        }
    }
}
