using System;

namespace Godot
{
    /// <summary>
    /// Returned by [method JSON.parse], [code]JSONParseResult[/code] contains decoded JSON or error information if JSON source not successfully parsed. You can check if JSON source was successfully parsed with [code]if json_result.error == 0[/code].
    /// </summary>
    public class JSONParseResult : Reference
    {
        /// <summary>
        /// The error type if JSON source was not successfully parsed. See [@GlobalScope] ERR_* constants.
        /// </summary>
        public Error Error
        {
            get
            {
                return GetError();
            }
            set
            {
                SetError(value);
            }
        }

        /// <summary>
        /// The error message if JSON source was not successfully parsed. See [@GlobalScope] ERR_* constants.
        /// </summary>
        public string ErrorString
        {
            get
            {
                return GetErrorString();
            }
            set
            {
                SetErrorString(value);
            }
        }

        /// <summary>
        /// The line number where the error occurred if JSON source was not successfully parsed.
        /// </summary>
        public int ErrorLine
        {
            get
            {
                return GetErrorLine();
            }
            set
            {
                SetErrorLine(value);
            }
        }

        /// <summary>
        /// A [Variant] containing the parsed JSON. Use typeof() to check if it is what you expect. For example, if JSON source starts with curly braces ([code]{}[/code]) a [Dictionary] will be returned, if JSON source starts with braces ([code][][/code]) an [Array] will be returned.
        /// [i]Be aware that the JSON specification does not define integer or float types, but only a number type. Therefore, parsing a JSON text will convert all numerical values to float types.[/i]
        /// [codeblock]
        /// p = JSON.parse('["hello", "world", "!"]')
        /// if typeof(p) == TYPE_ARRAY:
        /// print(p[0]) # prints 'hello'
        /// else:
        /// print("unexpected results")
        /// [/codeblock]
        /// </summary>
        public object Result
        {
            get
            {
                return GetResult();
            }
            set
            {
                SetResult(value);
            }
        }


        private const string nativeName = "JSONParseResult";

        public JSONParseResult() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_JSONParseResult_Ctor(this);
        }

        internal JSONParseResult(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_error");

        [GodotMethod("get_error")]
        public Error GetError()
        {
            return NativeCalls.godot_icall_0_325(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_error_string");

        [GodotMethod("get_error_string")]
        public string GetErrorString()
        {
            return NativeCalls.godot_icall_0_2(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_error_line");

        [GodotMethod("get_error_line")]
        public int GetErrorLine()
        {
            return NativeCalls.godot_icall_0_1(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_result");

        [GodotMethod("get_result")]
        public object GetResult()
        {
            return NativeCalls.godot_icall_0_461(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_error");

        [GodotMethod("set_error")]
        public void SetError(Error error)
        {
            NativeCalls.godot_icall_1_462(method_bind_4, Object.GetPtr(this), error);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_error_string");

        [GodotMethod("set_error_string")]
        public void SetErrorString(string errorString)
        {
            NativeCalls.godot_icall_1_43(method_bind_5, Object.GetPtr(this), errorString);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_error_line");

        [GodotMethod("set_error_line")]
        public void SetErrorLine(int errorLine)
        {
            NativeCalls.godot_icall_1_0(method_bind_6, Object.GetPtr(this), errorLine);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_result");

        [GodotMethod("set_result")]
        public void SetResult(object result)
        {
            NativeCalls.godot_icall_1_463(method_bind_7, Object.GetPtr(this), result);
        }
    }
}
