using System;

namespace Godot
{
    /// <summary>
    /// Kinematic bodies are special types of bodies that are meant to be user-controlled. They are not affected by physics at all (to other types of bodies, such a character or a rigid body, these are the same as a static body). They have however, two main uses:
    /// Simulated Motion: When these bodies are moved manually, either from code or from an AnimationPlayer (with process mode set to fixed), the physics will automatically compute an estimate of their linear and angular velocity. This makes them very useful for moving platforms or other AnimationPlayer-controlled objects (like a door, a bridge that opens, etc).
    /// Kinematic Characters: KinematicBody also has an API for moving objects (the [method move_and_collide] and [method move_and_slide] methods) while performing collision tests. This makes them really useful to implement characters that collide against a world, but that don't require advanced physics.
    /// </summary>
    public class KinematicBody : PhysicsBody
    {
        /// <summary>
        /// </summary>
        public bool AxisLockLinearX
        {
            get
            {
                return GetAxisLock((PhysicsServer.BodyAxis)1);
            }
            set
            {
                SetAxisLock((PhysicsServer.BodyAxis)1, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool AxisLockLinearY
        {
            get
            {
                return GetAxisLock((PhysicsServer.BodyAxis)2);
            }
            set
            {
                SetAxisLock((PhysicsServer.BodyAxis)2, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool AxisLockLinearZ
        {
            get
            {
                return GetAxisLock((PhysicsServer.BodyAxis)4);
            }
            set
            {
                SetAxisLock((PhysicsServer.BodyAxis)4, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool AxisLockAngularX
        {
            get
            {
                return GetAxisLock((PhysicsServer.BodyAxis)8);
            }
            set
            {
                SetAxisLock((PhysicsServer.BodyAxis)8, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool AxisLockAngularY
        {
            get
            {
                return GetAxisLock((PhysicsServer.BodyAxis)16);
            }
            set
            {
                SetAxisLock((PhysicsServer.BodyAxis)16, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool AxisLockAngularZ
        {
            get
            {
                return GetAxisLock((PhysicsServer.BodyAxis)32);
            }
            set
            {
                SetAxisLock((PhysicsServer.BodyAxis)32, value);
            }
        }

        /// <summary>
        /// If the body is at least this close to another body, this body will consider them to be colliding.
        /// </summary>
        public float Collision__safeMargin
        {
            get
            {
                return GetSafeMargin();
            }
            set
            {
                SetSafeMargin(value);
            }
        }


        private const string nativeName = "KinematicBody";

        public KinematicBody() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_KinematicBody_Ctor(this);
        }

        internal KinematicBody(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "move_and_collide");

        /// <summary>
        /// Moves the body along the vector [code]rel_vec[/code]. The body will stop if it collides. Returns a [KinematicCollision], which contains information about the collision.
        /// </summary>
        [GodotMethod("move_and_collide")]
        public KinematicCollision MoveAndCollide(Vector3 relVec)
        {
            return NativeCalls.godot_icall_1_465(method_bind_0, Object.GetPtr(this), ref relVec);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "move_and_slide");

        /// <summary>
        /// Moves the body along a vector. If the body collides with another, it will slide along the other body rather than stop immediately. If the other body is a [code]KinematicBody[/code] or [RigidBody], it will also be affected by the motion of the other body. You can use this to make moving or rotating platforms, or to make nodes push other nodes.
        /// [code]linear_velocity[/code] is a value in pixels per second. Unlike in for example [method move_and_collide], you should [i]not[/i] multiply it with [code]delta[/code] — this is done by the method.
        /// [code]floor_normal[/code] is the up direction, used to determine what is a wall and what is a floor or a ceiling. If set to the default value of [code]Vector3(0, 0, 0)[/code], everything is considered a wall. This is useful for topdown games.
        /// If the body is standing on a slope and the horizontal speed (relative to the floor's speed) goes below [code]slope_stop_min_velocity[/code], the body will stop completely. This prevents the body from sliding down slopes when you include gravity in [code]linear_velocity[/code]. When set to lower values, the body will not be able to stand still on steep slopes.
        /// If the body collides, it will change direction a maximum of [code]max_bounces[/code] times before it stops.
        /// [code]floor_max_angle[/code] is the maximum angle (in radians) where a slope is still considered a floor (or a ceiling), rather than a wall. The default value equals 45 degrees.
        /// Returns the movement that remained when the body stopped. To get more detailed information about collisions that occured, use [method get_slide_collision].
        /// &lt;param name="floorNormal"&gt;If the param is null, then the default value is new Vector3(0, 0, 0)&lt;/param&gt;
        /// </summary>
        [GodotMethod("move_and_slide")]
        public Vector3 MoveAndSlide(Vector3 linearVelocity, Nullable<Vector3> floorNormal = null, float slopeStopMinVelocity = 0.05f, int maxSlides = 4, float floorMaxAngle = 0.785398f)
        {
            Vector3 floorNormal_in = floorNormal.HasValue ? floorNormal.Value : new Vector3(0, 0, 0);
            object ret = NativeCalls.godot_icall_5_466(method_bind_1, Object.GetPtr(this), ref linearVelocity, ref floorNormal_in, slopeStopMinVelocity, maxSlides, floorMaxAngle);
            return (Vector3)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "test_move");

        /// <summary>
        /// Checks for collisions without moving the body. Virtually sets the node's position, scale and rotation to that of the given [Transform], then tries to move the body along the vector [code]rel_vec[/code]. Returns [code]true[/code] if a collision would occur.
        /// </summary>
        [GodotMethod("test_move")]
        public bool TestMove(Transform from, Vector3 relVec)
        {
            return NativeCalls.godot_icall_2_467(method_bind_2, Object.GetPtr(this), ref from, ref relVec);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_on_floor");

        /// <summary>
        /// Returns [code]true[/code] if the body is on the floor. Only updates when calling [method move_and_slide].
        /// </summary>
        [GodotMethod("is_on_floor")]
        public bool IsOnFloor()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_on_ceiling");

        /// <summary>
        /// Returns [code]true[/code] if the body is on the ceiling. Only updates when calling [method move_and_slide].
        /// </summary>
        [GodotMethod("is_on_ceiling")]
        public bool IsOnCeiling()
        {
            return NativeCalls.godot_icall_0_3(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_on_wall");

        /// <summary>
        /// Returns [code]true[/code] if the body is on a wall. Only updates when calling [method move_and_slide].
        /// </summary>
        [GodotMethod("is_on_wall")]
        public bool IsOnWall()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_floor_velocity");

        /// <summary>
        /// Returns the velocity of the floor. Only updates when calling [method move_and_slide].
        /// </summary>
        [GodotMethod("get_floor_velocity")]
        public Vector3 GetFloorVelocity()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_6, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_axis_lock");

        [GodotMethod("set_axis_lock")]
        public void SetAxisLock(PhysicsServer.BodyAxis axis, bool @lock)
        {
            NativeCalls.godot_icall_2_468(method_bind_7, Object.GetPtr(this), axis, @lock);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_axis_lock");

        [GodotMethod("get_axis_lock")]
        public bool GetAxisLock(PhysicsServer.BodyAxis axis)
        {
            return NativeCalls.godot_icall_1_469(method_bind_8, Object.GetPtr(this), axis);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_safe_margin");

        [GodotMethod("set_safe_margin")]
        public void SetSafeMargin(float pixels)
        {
            NativeCalls.godot_icall_1_10(method_bind_9, Object.GetPtr(this), pixels);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_safe_margin");

        [GodotMethod("get_safe_margin")]
        public float GetSafeMargin()
        {
            return NativeCalls.godot_icall_0_9(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_slide_count");

        /// <summary>
        /// Returns the number of times the body collided and changed direction during the last call to [method move_and_slide].
        /// </summary>
        [GodotMethod("get_slide_count")]
        public int GetSlideCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_slide_collision");

        /// <summary>
        /// Returns a [KinematicCollision], which contains information about a collision that occured during the last [method move_and_slide] call. Since the body can collide several times in a single call to [method move_and_slide], you must specify the index of the collision in the range 0 to ([method get_slide_count] - 1).
        /// </summary>
        [GodotMethod("get_slide_collision")]
        public KinematicCollision GetSlideCollision(int slideIdx)
        {
            return NativeCalls.godot_icall_1_470(method_bind_12, Object.GetPtr(this), slideIdx);
        }
    }
}
