using System;

namespace Godot
{
    /// <summary>
    /// Contains collision data for KinematicBody2D collisions. When a [KinematicBody2D] is moved using [method KinematicBody2D.move_and_collide], it stops if it detects a collision with another body. If a collision is detected, a KinematicCollision2D object is returned.
    /// This object contains information about the collision, including the colliding object, the remaining motion, and the collision position. This information can be used to calculate a collision response.
    /// </summary>
    public class KinematicCollision2D : Reference
    {
        /// <summary>
        /// The point of collision.
        /// </summary>
        public Vector2 Position
        {
            get
            {
                return GetPosition();
            }
        }

        /// <summary>
        /// The colliding body's shape's normal at the point of collision.
        /// </summary>
        public Vector2 Normal
        {
            get
            {
                return GetNormal();
            }
        }

        /// <summary>
        /// The distance the moving object traveled before collision.
        /// </summary>
        public Vector2 Travel
        {
            get
            {
                return GetTravel();
            }
        }

        /// <summary>
        /// The moving object's remaining movement vector.
        /// </summary>
        public Vector2 Remainder
        {
            get
            {
                return GetRemainder();
            }
        }

        /// <summary>
        /// The moving object's colliding shape.
        /// </summary>
        public Object LocalShape
        {
            get
            {
                return GetLocalShape();
            }
        }

        /// <summary>
        /// The colliding body.
        /// </summary>
        public Object Collider
        {
            get
            {
                return GetCollider();
            }
        }

        /// <summary>
        /// The colliding body's unique [RID].
        /// </summary>
        public int ColliderId
        {
            get
            {
                return GetColliderId();
            }
        }

        /// <summary>
        /// The colliding body's shape.
        /// </summary>
        public Object ColliderShape
        {
            get
            {
                return GetColliderShape();
            }
        }

        /// <summary>
        /// The colliding shape's index. See [CollisionObject2D].
        /// </summary>
        public int ColliderShapeIndex
        {
            get
            {
                return GetColliderShapeIndex();
            }
        }

        /// <summary>
        /// The colliding object's velocity.
        /// </summary>
        public Vector2 ColliderVelocity
        {
            get
            {
                return GetColliderVelocity();
            }
        }

        /// <summary>
        /// The colliding body's metadata. See [Object].
        /// </summary>
        public object ColliderMetadata
        {
            get
            {
                return GetColliderMetadata();
            }
        }


        private const string nativeName = "KinematicCollision2D";

        public KinematicCollision2D() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_KinematicCollision2D_Ctor(this);
        }

        internal KinematicCollision2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_position");

        [GodotMethod("get_position")]
        public Vector2 GetPosition()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_0, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_normal");

        [GodotMethod("get_normal")]
        public Vector2 GetNormal()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_1, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_travel");

        [GodotMethod("get_travel")]
        public Vector2 GetTravel()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_2, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_remainder");

        [GodotMethod("get_remainder")]
        public Vector2 GetRemainder()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_3, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_local_shape");

        [GodotMethod("get_local_shape")]
        public Object GetLocalShape()
        {
            return NativeCalls.godot_icall_0_475(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collider");

        [GodotMethod("get_collider")]
        public Object GetCollider()
        {
            return NativeCalls.godot_icall_0_475(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collider_id");

        [GodotMethod("get_collider_id")]
        public int GetColliderId()
        {
            return NativeCalls.godot_icall_0_1(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collider_shape");

        [GodotMethod("get_collider_shape")]
        public Object GetColliderShape()
        {
            return NativeCalls.godot_icall_0_475(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collider_shape_index");

        [GodotMethod("get_collider_shape_index")]
        public int GetColliderShapeIndex()
        {
            return NativeCalls.godot_icall_0_1(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collider_velocity");

        [GodotMethod("get_collider_velocity")]
        public Vector2 GetColliderVelocity()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_9, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collider_metadata");

        [GodotMethod("get_collider_metadata")]
        public object GetColliderMetadata()
        {
            return NativeCalls.godot_icall_0_461(method_bind_10, Object.GetPtr(this));
        }
    }
}
