using System;

namespace Godot
{
    /// <summary>
    /// Label displays plain text on the screen. It gives you control over the horizontal and vertical alignment, and can wrap the text inside the node's bounding rectangle. It doesn't support bold, italics or other formatting. For that, use [RichTextLabel] instead.
    /// Note that contrarily to most other [Control]s, Label's [member Control.mouse_filter] defaults to MOUSE_FILTER_IGNORE (i.e. it doesn't react to mouse input events).
    /// </summary>
    public class Label : Control
    {
        public enum AlignEnum
        {
            Left = 0,
            Center = 1,
            Right = 2,
            Fill = 3
        }

        public enum VAlign
        {
            Top = 0,
            Center = 1,
            Bottom = 2,
            Fill = 3
        }


        /// <summary>
        /// The text to display on screen.
        /// </summary>
        public string Text
        {
            get
            {
                return GetText();
            }
            set
            {
                SetText(value);
            }
        }

        /// <summary>
        /// Controls the text's horizontal align. Supports left, center, right, and fill, or justify. Set it to one of the [code]ALIGN_*[/code] constants.
        /// </summary>
        public Label.AlignEnum Align
        {
            get
            {
                return GetAlign();
            }
            set
            {
                SetAlign(value);
            }
        }

        /// <summary>
        /// Controls the text's vertical align. Supports top, center, bottom, and fill. Set it to one of the [code]VALIGN_*[/code] constants.
        /// </summary>
        public Label.VAlign Valign
        {
            get
            {
                return GetValign();
            }
            set
            {
                SetValign(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], wraps the text inside the node's bounding rectangle. If you resize the node, it will change its height automatically to show all the text. Default: false.
        /// </summary>
        public bool Autowrap
        {
            get
            {
                return HasAutowrap();
            }
            set
            {
                SetAutowrap(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], the Label only shows the text that fits inside its bounding rectangle. It also lets you scale the node down freely.
        /// </summary>
        public bool ClipText
        {
            get
            {
                return IsClippingText();
            }
            set
            {
                SetClipText(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], all the text displays as UPPERCASE.
        /// </summary>
        public bool Uppercase
        {
            get
            {
                return IsUppercase();
            }
            set
            {
                SetUppercase(value);
            }
        }

        /// <summary>
        /// Restricts the number of characters to display. Set to -1 to disable.
        /// </summary>
        public int VisibleCharacters
        {
            get
            {
                return GetVisibleCharacters();
            }
            set
            {
                SetVisibleCharacters(value);
            }
        }

        /// <summary>
        /// Limits the count of visible characters. If you set [code]percent_visible[/code] to 50, only up to half of the text's characters will display on screen. Useful to animate the text in a dialog box.
        /// </summary>
        public float PercentVisible
        {
            get
            {
                return GetPercentVisible();
            }
            set
            {
                SetPercentVisible(value);
            }
        }

        /// <summary>
        /// The node ignores the first [code]lines_skipped[/code] lines before it starts to display text.
        /// </summary>
        public int LinesSkipped
        {
            get
            {
                return GetLinesSkipped();
            }
            set
            {
                SetLinesSkipped(value);
            }
        }

        /// <summary>
        /// Limits the lines of text the node shows on screen.
        /// </summary>
        public int MaxLinesVisible
        {
            get
            {
                return GetMaxLinesVisible();
            }
            set
            {
                SetMaxLinesVisible(value);
            }
        }


        private const string nativeName = "Label";

        public Label() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Label_Ctor(this);
        }

        internal Label(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_align");

        [GodotMethod("set_align")]
        public void SetAlign(Label.AlignEnum align)
        {
            NativeCalls.godot_icall_1_476(method_bind_0, Object.GetPtr(this), align);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_align");

        [GodotMethod("get_align")]
        public Label.AlignEnum GetAlign()
        {
            return NativeCalls.godot_icall_0_477(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_valign");

        [GodotMethod("set_valign")]
        public void SetValign(Label.VAlign valign)
        {
            NativeCalls.godot_icall_1_478(method_bind_2, Object.GetPtr(this), valign);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_valign");

        [GodotMethod("get_valign")]
        public Label.VAlign GetValign()
        {
            return NativeCalls.godot_icall_0_479(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_text");

        [GodotMethod("set_text")]
        public void SetText(string text)
        {
            NativeCalls.godot_icall_1_43(method_bind_4, Object.GetPtr(this), text);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_text");

        [GodotMethod("get_text")]
        public string GetText()
        {
            return NativeCalls.godot_icall_0_2(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_autowrap");

        [GodotMethod("set_autowrap")]
        public void SetAutowrap(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_6, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_autowrap");

        [GodotMethod("has_autowrap")]
        public bool HasAutowrap()
        {
            return NativeCalls.godot_icall_0_3(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_clip_text");

        [GodotMethod("set_clip_text")]
        public void SetClipText(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_8, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_clipping_text");

        [GodotMethod("is_clipping_text")]
        public bool IsClippingText()
        {
            return NativeCalls.godot_icall_0_3(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_uppercase");

        [GodotMethod("set_uppercase")]
        public void SetUppercase(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_10, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_uppercase");

        [GodotMethod("is_uppercase")]
        public bool IsUppercase()
        {
            return NativeCalls.godot_icall_0_3(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_line_height");

        /// <summary>
        /// Returns the font size in pixels.
        /// </summary>
        [GodotMethod("get_line_height")]
        public int GetLineHeight()
        {
            return NativeCalls.godot_icall_0_1(method_bind_12, Object.GetPtr(this));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_line_count");

        /// <summary>
        /// Returns the amount of lines of text the Label has.
        /// </summary>
        [GodotMethod("get_line_count")]
        public int GetLineCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_visible_line_count");

        /// <summary>
        /// Returns the number of lines shown. Useful if the [code]Label[/code] 's height cannot currently display all lines.
        /// </summary>
        [GodotMethod("get_visible_line_count")]
        public int GetVisibleLineCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_total_character_count");

        /// <summary>
        /// Returns the total length of the text.
        /// </summary>
        [GodotMethod("get_total_character_count")]
        public int GetTotalCharacterCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_visible_characters");

        [GodotMethod("set_visible_characters")]
        public void SetVisibleCharacters(int amount)
        {
            NativeCalls.godot_icall_1_0(method_bind_16, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_visible_characters");

        [GodotMethod("get_visible_characters")]
        public int GetVisibleCharacters()
        {
            return NativeCalls.godot_icall_0_1(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_percent_visible");

        [GodotMethod("set_percent_visible")]
        public void SetPercentVisible(float percentVisible)
        {
            NativeCalls.godot_icall_1_10(method_bind_18, Object.GetPtr(this), percentVisible);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_percent_visible");

        [GodotMethod("get_percent_visible")]
        public float GetPercentVisible()
        {
            return NativeCalls.godot_icall_0_9(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_lines_skipped");

        [GodotMethod("set_lines_skipped")]
        public void SetLinesSkipped(int linesSkipped)
        {
            NativeCalls.godot_icall_1_0(method_bind_20, Object.GetPtr(this), linesSkipped);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_lines_skipped");

        [GodotMethod("get_lines_skipped")]
        public int GetLinesSkipped()
        {
            return NativeCalls.godot_icall_0_1(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_max_lines_visible");

        [GodotMethod("set_max_lines_visible")]
        public void SetMaxLinesVisible(int linesVisible)
        {
            NativeCalls.godot_icall_1_0(method_bind_22, Object.GetPtr(this), linesVisible);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_max_lines_visible");

        [GodotMethod("get_max_lines_visible")]
        public int GetMaxLinesVisible()
        {
            return NativeCalls.godot_icall_0_1(method_bind_23, Object.GetPtr(this));
        }
    }
}
