using System;

namespace Godot
{
    /// <summary>
    /// Casts light in a 2D environment. Light is defined by a (usually grayscale) texture, a color, an energy value, a mode (see constants), and various other parameters (range and shadows-related). Note that Light2D can be used as a mask.
    /// </summary>
    public class Light2D : Node2D
    {
        public enum ShadowFilterEnum
        {
            None = 0,
            Pcf3 = 1,
            Pcf5 = 2,
            Pcf7 = 3,
            Pcf9 = 4,
            Pcf13 = 5
        }

        public enum ModeEnum
        {
            Add = 0,
            Sub = 1,
            Mix = 2,
            Mask = 3
        }


        /// <summary>
        /// If [code]true[/code] Light2D will emit light. Default value: [code]true[/code].
        /// </summary>
        public bool Enabled
        {
            get
            {
                return IsEnabled();
            }
            set
            {
                SetEnabled(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] Light2D will only appear when editing the scene. Default value: [code]false[/code].
        /// </summary>
        public bool EditorOnly
        {
            get
            {
                return IsEditorOnly();
            }
            set
            {
                SetEditorOnly(value);
            }
        }

        /// <summary>
        /// [Texture] used for the Light2D's appearance.
        /// </summary>
        public Texture Texture
        {
            get
            {
                return GetTexture();
            }
            set
            {
                SetTexture(value);
            }
        }

        /// <summary>
        /// The offset of the Light2D's [code]texture[/code].
        /// </summary>
        public Vector2 Offset
        {
            get
            {
                return GetTextureOffset();
            }
            set
            {
                SetTextureOffset(value);
            }
        }

        /// <summary>
        /// The [code]texture[/code]'s scale factor.
        /// </summary>
        public float TextureScale
        {
            get
            {
                return GetTextureScale();
            }
            set
            {
                SetTextureScale(value);
            }
        }

        /// <summary>
        /// The Light2D's [Color].
        /// </summary>
        public Color Color
        {
            get
            {
                return GetColor();
            }
            set
            {
                SetColor(value);
            }
        }

        /// <summary>
        /// The Light2D's energy value. The larger the value, the stronger the light.
        /// </summary>
        public float Energy
        {
            get
            {
                return GetEnergy();
            }
            set
            {
                SetEnergy(value);
            }
        }

        /// <summary>
        /// The Light2D's mode. See MODE_* constants for values.
        /// </summary>
        public Light2D.ModeEnum Mode
        {
            get
            {
                return GetMode();
            }
            set
            {
                SetMode(value);
            }
        }

        /// <summary>
        /// The height of the Light2D. Used with 2D normal mapping.
        /// </summary>
        public float RangeHeight
        {
            get
            {
                return GetHeight();
            }
            set
            {
                SetHeight(value);
            }
        }

        /// <summary>
        /// Minimum [code]z[/code] value of objects that are affected by the Light2D. Default value: [code]-1024[/code].
        /// </summary>
        public int RangeZMin
        {
            get
            {
                return GetZRangeMin();
            }
            set
            {
                SetZRangeMin(value);
            }
        }

        /// <summary>
        /// Maximum [code]Z[/code] value of objects that are affected by the Light2D. Default value: [code]1024[/code].
        /// </summary>
        public int RangeZMax
        {
            get
            {
                return GetZRangeMax();
            }
            set
            {
                SetZRangeMax(value);
            }
        }

        /// <summary>
        /// Minimum layer value of objects that are affected by the Light2D. Default value: [code]0[/code].
        /// </summary>
        public int RangeLayerMin
        {
            get
            {
                return GetLayerRangeMin();
            }
            set
            {
                SetLayerRangeMin(value);
            }
        }

        /// <summary>
        /// Maximum layer value of objects that are affected by the Light2D. Default value: [code]0[/code].
        /// </summary>
        public int RangeLayerMax
        {
            get
            {
                return GetLayerRangeMax();
            }
            set
            {
                SetLayerRangeMax(value);
            }
        }

        /// <summary>
        /// The layer mask. Only objects with a matching mask will be affected by the Light2D.
        /// </summary>
        public int RangeItemCullMask
        {
            get
            {
                return GetItemCullMask();
            }
            set
            {
                SetItemCullMask(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the Light2D will cast shadows. Default value: [code]false[/code].
        /// </summary>
        public bool ShadowEnabled
        {
            get
            {
                return IsShadowEnabled();
            }
            set
            {
                SetShadowEnabled(value);
            }
        }

        /// <summary>
        /// [Color] of shadows cast by the Light2D.
        /// </summary>
        public Color ShadowColor
        {
            get
            {
                return GetShadowColor();
            }
            set
            {
                SetShadowColor(value);
            }
        }

        /// <summary>
        /// Shadow buffer size. Default value: [code]2048[/code].
        /// </summary>
        public int ShadowBufferSize
        {
            get
            {
                return GetShadowBufferSize();
            }
            set
            {
                SetShadowBufferSize(value);
            }
        }

        /// <summary>
        /// Smooth shadow gradient length.
        /// </summary>
        public float ShadowGradientLength
        {
            get
            {
                return GetShadowGradientLength();
            }
            set
            {
                SetShadowGradientLength(value);
            }
        }

        /// <summary>
        /// Shadow filter type. Use SHADOW_FILTER_* constants to set [code]shadow_filter[/code]. Default value: [code]None[/code].
        /// </summary>
        public Light2D.ShadowFilterEnum ShadowFilter
        {
            get
            {
                return GetShadowFilter();
            }
            set
            {
                SetShadowFilter(value);
            }
        }

        /// <summary>
        /// Smoothing value for shadows.
        /// </summary>
        public float ShadowFilterSmooth
        {
            get
            {
                return GetShadowSmooth();
            }
            set
            {
                SetShadowSmooth(value);
            }
        }

        /// <summary>
        /// The shadow mask. Used with [LightOccluder2D] to cast shadows. Only occluders with a matching shadow mask will cast shadows.
        /// </summary>
        public int ShadowItemCullMask
        {
            get
            {
                return GetItemShadowCullMask();
            }
            set
            {
                SetItemShadowCullMask(value);
            }
        }


        private const string nativeName = "Light2D";

        public Light2D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Light2D_Ctor(this);
        }

        internal Light2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_enabled");

        [GodotMethod("set_enabled")]
        public void SetEnabled(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_0, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_enabled");

        [GodotMethod("is_enabled")]
        public bool IsEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_editor_only");

        [GodotMethod("set_editor_only")]
        public void SetEditorOnly(bool editorOnly)
        {
            NativeCalls.godot_icall_1_11(method_bind_2, Object.GetPtr(this), editorOnly);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_editor_only");

        [GodotMethod("is_editor_only")]
        public bool IsEditorOnly()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_texture");

        [GodotMethod("set_texture")]
        public void SetTexture(Texture texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_4, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture");

        [GodotMethod("get_texture")]
        public Texture GetTexture()
        {
            return NativeCalls.godot_icall_0_119(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_texture_offset");

        [GodotMethod("set_texture_offset")]
        public void SetTextureOffset(Vector2 textureOffset)
        {
            NativeCalls.godot_icall_1_45(method_bind_6, Object.GetPtr(this), ref textureOffset);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture_offset");

        [GodotMethod("get_texture_offset")]
        public Vector2 GetTextureOffset()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_7, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_color");

        [GodotMethod("set_color")]
        public void SetColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_8, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_color");

        [GodotMethod("get_color")]
        public Color GetColor()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_9, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_height");

        [GodotMethod("set_height")]
        public void SetHeight(float height)
        {
            NativeCalls.godot_icall_1_10(method_bind_10, Object.GetPtr(this), height);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_height");

        [GodotMethod("get_height")]
        public float GetHeight()
        {
            return NativeCalls.godot_icall_0_9(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_energy");

        [GodotMethod("set_energy")]
        public void SetEnergy(float energy)
        {
            NativeCalls.godot_icall_1_10(method_bind_12, Object.GetPtr(this), energy);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_energy");

        [GodotMethod("get_energy")]
        public float GetEnergy()
        {
            return NativeCalls.godot_icall_0_9(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_texture_scale");

        [GodotMethod("set_texture_scale")]
        public void SetTextureScale(float textureScale)
        {
            NativeCalls.godot_icall_1_10(method_bind_14, Object.GetPtr(this), textureScale);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture_scale");

        [GodotMethod("get_texture_scale")]
        public float GetTextureScale()
        {
            return NativeCalls.godot_icall_0_9(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_z_range_min");

        [GodotMethod("set_z_range_min")]
        public void SetZRangeMin(int z)
        {
            NativeCalls.godot_icall_1_0(method_bind_16, Object.GetPtr(this), z);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_z_range_min");

        [GodotMethod("get_z_range_min")]
        public int GetZRangeMin()
        {
            return NativeCalls.godot_icall_0_1(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_z_range_max");

        [GodotMethod("set_z_range_max")]
        public void SetZRangeMax(int z)
        {
            NativeCalls.godot_icall_1_0(method_bind_18, Object.GetPtr(this), z);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_z_range_max");

        [GodotMethod("get_z_range_max")]
        public int GetZRangeMax()
        {
            return NativeCalls.godot_icall_0_1(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_layer_range_min");

        [GodotMethod("set_layer_range_min")]
        public void SetLayerRangeMin(int layer)
        {
            NativeCalls.godot_icall_1_0(method_bind_20, Object.GetPtr(this), layer);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_layer_range_min");

        [GodotMethod("get_layer_range_min")]
        public int GetLayerRangeMin()
        {
            return NativeCalls.godot_icall_0_1(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_layer_range_max");

        [GodotMethod("set_layer_range_max")]
        public void SetLayerRangeMax(int layer)
        {
            NativeCalls.godot_icall_1_0(method_bind_22, Object.GetPtr(this), layer);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_layer_range_max");

        [GodotMethod("get_layer_range_max")]
        public int GetLayerRangeMax()
        {
            return NativeCalls.godot_icall_0_1(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_cull_mask");

        [GodotMethod("set_item_cull_mask")]
        public void SetItemCullMask(int itemCullMask)
        {
            NativeCalls.godot_icall_1_0(method_bind_24, Object.GetPtr(this), itemCullMask);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_cull_mask");

        [GodotMethod("get_item_cull_mask")]
        public int GetItemCullMask()
        {
            return NativeCalls.godot_icall_0_1(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_item_shadow_cull_mask");

        [GodotMethod("set_item_shadow_cull_mask")]
        public void SetItemShadowCullMask(int itemShadowCullMask)
        {
            NativeCalls.godot_icall_1_0(method_bind_26, Object.GetPtr(this), itemShadowCullMask);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_shadow_cull_mask");

        [GodotMethod("get_item_shadow_cull_mask")]
        public int GetItemShadowCullMask()
        {
            return NativeCalls.godot_icall_0_1(method_bind_27, Object.GetPtr(this));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_mode");

        [GodotMethod("set_mode")]
        public void SetMode(Light2D.ModeEnum mode)
        {
            NativeCalls.godot_icall_1_485(method_bind_28, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mode");

        [GodotMethod("get_mode")]
        public Light2D.ModeEnum GetMode()
        {
            return NativeCalls.godot_icall_0_486(method_bind_29, Object.GetPtr(this));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shadow_enabled");

        [GodotMethod("set_shadow_enabled")]
        public void SetShadowEnabled(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_30, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_shadow_enabled");

        [GodotMethod("is_shadow_enabled")]
        public bool IsShadowEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_31, Object.GetPtr(this));
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shadow_buffer_size");

        [GodotMethod("set_shadow_buffer_size")]
        public void SetShadowBufferSize(int size)
        {
            NativeCalls.godot_icall_1_0(method_bind_32, Object.GetPtr(this), size);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shadow_buffer_size");

        [GodotMethod("get_shadow_buffer_size")]
        public int GetShadowBufferSize()
        {
            return NativeCalls.godot_icall_0_1(method_bind_33, Object.GetPtr(this));
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shadow_smooth");

        [GodotMethod("set_shadow_smooth")]
        public void SetShadowSmooth(float smooth)
        {
            NativeCalls.godot_icall_1_10(method_bind_34, Object.GetPtr(this), smooth);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shadow_smooth");

        [GodotMethod("get_shadow_smooth")]
        public float GetShadowSmooth()
        {
            return NativeCalls.godot_icall_0_9(method_bind_35, Object.GetPtr(this));
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shadow_gradient_length");

        [GodotMethod("set_shadow_gradient_length")]
        public void SetShadowGradientLength(float multiplier)
        {
            NativeCalls.godot_icall_1_10(method_bind_36, Object.GetPtr(this), multiplier);
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shadow_gradient_length");

        [GodotMethod("get_shadow_gradient_length")]
        public float GetShadowGradientLength()
        {
            return NativeCalls.godot_icall_0_9(method_bind_37, Object.GetPtr(this));
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shadow_filter");

        [GodotMethod("set_shadow_filter")]
        public void SetShadowFilter(Light2D.ShadowFilterEnum filter)
        {
            NativeCalls.godot_icall_1_487(method_bind_38, Object.GetPtr(this), filter);
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shadow_filter");

        [GodotMethod("get_shadow_filter")]
        public Light2D.ShadowFilterEnum GetShadowFilter()
        {
            return NativeCalls.godot_icall_0_488(method_bind_39, Object.GetPtr(this));
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shadow_color");

        [GodotMethod("set_shadow_color")]
        public void SetShadowColor(Color shadowColor)
        {
            NativeCalls.godot_icall_1_202(method_bind_40, Object.GetPtr(this), ref shadowColor);
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shadow_color");

        [GodotMethod("get_shadow_color")]
        public Color GetShadowColor()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_41, Object.GetPtr(this));
            return (Color)ret;
        }
    }
}
