using System;

namespace Godot
{
    /// <summary>
    /// A line through several points in 2D space.
    /// </summary>
    public class Line2D : Node2D
    {
        public enum LineTextureMode
        {
            None = 0,
            Tile = 1
        }

        public enum LineCapMode
        {
            None = 0,
            Box = 1,
            Round = 2
        }

        public enum LineJointMode
        {
            Sharp = 0,
            Bevel = 1,
            Round = 2
        }


        /// <summary>
        /// The points that form the lines. The line is drawn between every point set in this array.
        /// </summary>
        public Vector2[] Points
        {
            get
            {
                return GetPoints();
            }
            set
            {
                SetPoints(value);
            }
        }

        /// <summary>
        /// The line's width.
        /// </summary>
        public float Width
        {
            get
            {
                return GetWidth();
            }
            set
            {
                SetWidth(value);
            }
        }

        /// <summary>
        /// The line's color. Will not be used if a gradient is set.
        /// </summary>
        public Color DefaultColor
        {
            get
            {
                return GetDefaultColor();
            }
            set
            {
                SetDefaultColor(value);
            }
        }

        /// <summary>
        /// The gradient is drawn through the whole line from start to finish. The default color will not be used if a gradient is set.
        /// </summary>
        public Gradient Gradient
        {
            get
            {
                return GetGradient();
            }
            set
            {
                SetGradient(value);
            }
        }

        /// <summary>
        /// The texture used for the line's texture. Uses [code]texture_mode[/code] for drawing style.
        /// </summary>
        public Texture Texture
        {
            get
            {
                return GetTexture();
            }
            set
            {
                SetTexture(value);
            }
        }

        /// <summary>
        /// The style to render the [code]texture[/code] on the line. Use [code]LINE_TEXTURE_*[/code] constants. Default value: [code]LINE_TEXTURE_NONE[/code].
        /// </summary>
        public Line2D.LineTextureMode TextureMode
        {
            get
            {
                return GetTextureMode();
            }
            set
            {
                SetTextureMode(value);
            }
        }

        /// <summary>
        /// The style for the points inbetween the start and the end.
        /// </summary>
        public Line2D.LineJointMode JointMode
        {
            get
            {
                return GetJointMode();
            }
            set
            {
                SetJointMode(value);
            }
        }

        /// <summary>
        /// Controls the style of the line's first point. Use [code]LINE_CAP_*[/code] constants. Default value: [code]LINE_CAP_NONE[/code].
        /// </summary>
        public Line2D.LineCapMode BeginCapMode
        {
            get
            {
                return GetBeginCapMode();
            }
            set
            {
                SetBeginCapMode(value);
            }
        }

        /// <summary>
        /// Controls the style of the line's last point. Use [code]LINE_CAP_*[/code] constants. Default value: [code]LINE_CAP_NONE[/code].
        /// </summary>
        public Line2D.LineCapMode EndCapMode
        {
            get
            {
                return GetEndCapMode();
            }
            set
            {
                SetEndCapMode(value);
            }
        }

        /// <summary>
        /// The direction difference in radians between vector points. This value is only used if [code]joint mode[/code] is set to [code]LINE_JOINT_SHARP[/code].
        /// </summary>
        public float SharpLimit
        {
            get
            {
                return GetSharpLimit();
            }
            set
            {
                SetSharpLimit(value);
            }
        }

        /// <summary>
        /// The smoothness of the rounded joints and caps. This is only used if a cap or joint is set as round.
        /// </summary>
        public int RoundPrecision
        {
            get
            {
                return GetRoundPrecision();
            }
            set
            {
                SetRoundPrecision(value);
            }
        }


        private const string nativeName = "Line2D";

        public Line2D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Line2D_Ctor(this);
        }

        internal Line2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_points");

        [GodotMethod("set_points")]
        public void SetPoints(Vector2[] points)
        {
            NativeCalls.godot_icall_1_237(method_bind_0, Object.GetPtr(this), points);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_points");

        [GodotMethod("get_points")]
        public Vector2[] GetPoints()
        {
            return NativeCalls.godot_icall_0_238(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_point_position");

        /// <summary>
        /// Overwites the position in point [code]i[/code] with the supplied [code]position[/code].
        /// </summary>
        [GodotMethod("set_point_position")]
        public void SetPointPosition(int i, Vector2 position)
        {
            NativeCalls.godot_icall_2_288(method_bind_2, Object.GetPtr(this), i, ref position);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_point_position");

        /// <summary>
        /// Returns point [code]i[/code]'s position.
        /// </summary>
        [GodotMethod("get_point_position")]
        public Vector2 GetPointPosition(int i)
        {
            object ret = NativeCalls.godot_icall_1_282(method_bind_3, Object.GetPtr(this), i);
            return (Vector2)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_point_count");

        /// <summary>
        /// Returns the Line2D's amount of points.
        /// </summary>
        [GodotMethod("get_point_count")]
        public int GetPointCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_point");

        /// <summary>
        /// Add a point at the [code]position[/code]. Appends the point at the end of the line.
        /// </summary>
        [GodotMethod("add_point")]
        public void AddPoint(Vector2 position)
        {
            NativeCalls.godot_icall_1_45(method_bind_5, Object.GetPtr(this), ref position);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_point");

        /// <summary>
        /// Remove the point at index [code]i[/code] from the line.
        /// </summary>
        [GodotMethod("remove_point")]
        public void RemovePoint(int i)
        {
            NativeCalls.godot_icall_1_0(method_bind_6, Object.GetPtr(this), i);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_width");

        [GodotMethod("set_width")]
        public void SetWidth(float width)
        {
            NativeCalls.godot_icall_1_10(method_bind_7, Object.GetPtr(this), width);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_width");

        [GodotMethod("get_width")]
        public float GetWidth()
        {
            return NativeCalls.godot_icall_0_9(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_default_color");

        [GodotMethod("set_default_color")]
        public void SetDefaultColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_9, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_default_color");

        [GodotMethod("get_default_color")]
        public Color GetDefaultColor()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_10, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_gradient");

        [GodotMethod("set_gradient")]
        public void SetGradient(Gradient color)
        {
            NativeCalls.godot_icall_1_24(method_bind_11, Object.GetPtr(this), Object.GetPtr(color));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_gradient");

        [GodotMethod("get_gradient")]
        public Gradient GetGradient()
        {
            return NativeCalls.godot_icall_0_390(method_bind_12, Object.GetPtr(this));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_texture");

        [GodotMethod("set_texture")]
        public void SetTexture(Texture texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_13, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture");

        [GodotMethod("get_texture")]
        public Texture GetTexture()
        {
            return NativeCalls.godot_icall_0_119(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_texture_mode");

        [GodotMethod("set_texture_mode")]
        public void SetTextureMode(Line2D.LineTextureMode mode)
        {
            NativeCalls.godot_icall_1_490(method_bind_15, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture_mode");

        [GodotMethod("get_texture_mode")]
        public Line2D.LineTextureMode GetTextureMode()
        {
            return NativeCalls.godot_icall_0_491(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_joint_mode");

        [GodotMethod("set_joint_mode")]
        public void SetJointMode(Line2D.LineJointMode mode)
        {
            NativeCalls.godot_icall_1_492(method_bind_17, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_joint_mode");

        [GodotMethod("get_joint_mode")]
        public Line2D.LineJointMode GetJointMode()
        {
            return NativeCalls.godot_icall_0_493(method_bind_18, Object.GetPtr(this));
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_begin_cap_mode");

        [GodotMethod("set_begin_cap_mode")]
        public void SetBeginCapMode(Line2D.LineCapMode mode)
        {
            NativeCalls.godot_icall_1_494(method_bind_19, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_begin_cap_mode");

        [GodotMethod("get_begin_cap_mode")]
        public Line2D.LineCapMode GetBeginCapMode()
        {
            return NativeCalls.godot_icall_0_495(method_bind_20, Object.GetPtr(this));
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_end_cap_mode");

        [GodotMethod("set_end_cap_mode")]
        public void SetEndCapMode(Line2D.LineCapMode mode)
        {
            NativeCalls.godot_icall_1_494(method_bind_21, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_end_cap_mode");

        [GodotMethod("get_end_cap_mode")]
        public Line2D.LineCapMode GetEndCapMode()
        {
            return NativeCalls.godot_icall_0_495(method_bind_22, Object.GetPtr(this));
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_sharp_limit");

        [GodotMethod("set_sharp_limit")]
        public void SetSharpLimit(float limit)
        {
            NativeCalls.godot_icall_1_10(method_bind_23, Object.GetPtr(this), limit);
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_sharp_limit");

        [GodotMethod("get_sharp_limit")]
        public float GetSharpLimit()
        {
            return NativeCalls.godot_icall_0_9(method_bind_24, Object.GetPtr(this));
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_round_precision");

        [GodotMethod("set_round_precision")]
        public void SetRoundPrecision(int precision)
        {
            NativeCalls.godot_icall_1_0(method_bind_25, Object.GetPtr(this), precision);
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_round_precision");

        [GodotMethod("get_round_precision")]
        public int GetRoundPrecision()
        {
            return NativeCalls.godot_icall_0_1(method_bind_26, Object.GetPtr(this));
        }
    }
}
