using System;

namespace Godot
{
    /// <summary>
    /// LineEdit provides a single line string editor, used for text fields.
    /// </summary>
    public class LineEdit : Control
    {
        public enum AlignEnum
        {
            Left = 0,
            Center = 1,
            Right = 2,
            Fill = 3
        }

        public enum MenuItems
        {
            Cut = 0,
            Copy = 1,
            Paste = 2,
            Clear = 3,
            SelectAll = 4,
            Undo = 5,
            Redo = 6,
            Max = 7
        }


        /// <summary>
        /// String value of the [LineEdit].
        /// </summary>
        public string Text
        {
            get
            {
                return GetText();
            }
            set
            {
                SetText(value);
            }
        }

        /// <summary>
        /// Text alignment as defined in the ALIGN_* enum.
        /// </summary>
        public LineEdit.AlignEnum Align
        {
            get
            {
                return GetAlign();
            }
            set
            {
                SetAlign(value);
            }
        }

        /// <summary>
        /// Maximum amount of characters that can be entered inside the [LineEdit]. If [code]0[/code], there is no limit.
        /// </summary>
        public int MaxLength
        {
            get
            {
                return GetMaxLength();
            }
            set
            {
                SetMaxLength(value);
            }
        }

        /// <summary>
        /// If [code]false[/code] existing text cannot be modified and new text cannot be added.
        /// </summary>
        public bool Editable
        {
            get
            {
                return IsEditable();
            }
            set
            {
                SetEditable(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] every character is shown as "*".
        /// </summary>
        public bool Secret
        {
            get
            {
                return IsSecret();
            }
            set
            {
                SetSecret(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the [LineEdit] width will increase to stay longer than the [member text]. It will [b]not[/b] compress if the [member text] is shortened.
        /// </summary>
        public bool ExpandToTextLength
        {
            get
            {
                return GetExpandToTextLength();
            }
            set
            {
                SetExpandToTextLength(value);
            }
        }

        /// <summary>
        /// Defines how the [LineEdit] can grab focus (Keyboard and mouse, only keyboard, or none). See [code]enum FocusMode[/code] in [Control] for details.
        /// </summary>
        public Control.FocusModeEnum FocusMode
        {
            get
            {
                return GetFocusMode();
            }
            set
            {
                SetFocusMode(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the context menu will appear when right clicked.
        /// </summary>
        public bool ContextMenuEnabled
        {
            get
            {
                return IsContextMenuEnabled();
            }
            set
            {
                SetContextMenuEnabled(value);
            }
        }

        /// <summary>
        /// Text shown when the [LineEdit] is empty. It is [b]not[/b] the [LineEdit]'s default value (see [member text]).
        /// </summary>
        public string PlaceholderText
        {
            get
            {
                return GetPlaceholder();
            }
            set
            {
                SetPlaceholder(value);
            }
        }

        /// <summary>
        /// Opacity of the [member placeholder_text]. From [code]0[/code] to [code]1[/code].
        /// </summary>
        public float PlaceholderAlpha
        {
            get
            {
                return GetPlaceholderAlpha();
            }
            set
            {
                SetPlaceholderAlpha(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the caret (visual cursor) blinks.
        /// </summary>
        public bool CaretBlink
        {
            get
            {
                return CursorGetBlinkEnabled();
            }
            set
            {
                CursorSetBlinkEnabled(value);
            }
        }

        /// <summary>
        /// Duration (in seconds) of a caret's blinking cycle.
        /// </summary>
        public float CaretBlinkSpeed
        {
            get
            {
                return CursorGetBlinkSpeed();
            }
            set
            {
                CursorSetBlinkSpeed(value);
            }
        }

        /// <summary>
        /// The cursor's position inside the [code]LineEdit[/code]. When set, the text may scroll to accomodate it.
        /// </summary>
        public int CaretPosition
        {
            get
            {
                return GetCursorPosition();
            }
            set
            {
                SetCursorPosition(value);
            }
        }


        private const string nativeName = "LineEdit";

        public LineEdit() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_LineEdit_Ctor(this);
        }

        internal LineEdit(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_align");

        [GodotMethod("set_align")]
        public void SetAlign(LineEdit.AlignEnum align)
        {
            NativeCalls.godot_icall_1_496(method_bind_0, Object.GetPtr(this), align);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_align");

        [GodotMethod("get_align")]
        public LineEdit.AlignEnum GetAlign()
        {
            return NativeCalls.godot_icall_0_497(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear");

        /// <summary>
        /// Erases the [LineEdit] text.
        /// </summary>
        [GodotMethod("clear")]
        public void Clear()
        {
            NativeCalls.godot_icall_0_12(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "select");

        /// <summary>
        /// Selects characters inside [LineEdit] between [code]from[/code] and [code]to[/code]. By default [code]from[/code] is at the beginning and [code]to[/code] at the end.
        /// [codeblock]
        /// text = "Welcome"
        /// select()     # Welcome
        /// select(4)    # ome
        /// select(2, 5) # lco
        /// [/codeblock]
        /// </summary>
        [GodotMethod("select")]
        public void Select(int from = 0, int to = -1)
        {
            NativeCalls.godot_icall_2_32(method_bind_3, Object.GetPtr(this), from, to);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "select_all");

        /// <summary>
        /// Selects the whole [String].
        /// </summary>
        [GodotMethod("select_all")]
        public void SelectAll()
        {
            NativeCalls.godot_icall_0_12(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "deselect");

        /// <summary>
        /// Clears the current selection.
        /// </summary>
        [GodotMethod("deselect")]
        public void Deselect()
        {
            NativeCalls.godot_icall_0_12(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_text");

        [GodotMethod("set_text")]
        public void SetText(string text)
        {
            NativeCalls.godot_icall_1_43(method_bind_6, Object.GetPtr(this), text);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_text");

        [GodotMethod("get_text")]
        public string GetText()
        {
            return NativeCalls.godot_icall_0_2(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_placeholder");

        [GodotMethod("set_placeholder")]
        public void SetPlaceholder(string text)
        {
            NativeCalls.godot_icall_1_43(method_bind_8, Object.GetPtr(this), text);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_placeholder");

        [GodotMethod("get_placeholder")]
        public string GetPlaceholder()
        {
            return NativeCalls.godot_icall_0_2(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_placeholder_alpha");

        [GodotMethod("set_placeholder_alpha")]
        public void SetPlaceholderAlpha(float alpha)
        {
            NativeCalls.godot_icall_1_10(method_bind_10, Object.GetPtr(this), alpha);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_placeholder_alpha");

        [GodotMethod("get_placeholder_alpha")]
        public float GetPlaceholderAlpha()
        {
            return NativeCalls.godot_icall_0_9(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_cursor_position");

        [GodotMethod("set_cursor_position")]
        public void SetCursorPosition(int position)
        {
            NativeCalls.godot_icall_1_0(method_bind_12, Object.GetPtr(this), position);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cursor_position");

        [GodotMethod("get_cursor_position")]
        public int GetCursorPosition()
        {
            return NativeCalls.godot_icall_0_1(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_expand_to_text_length");

        [GodotMethod("set_expand_to_text_length")]
        public void SetExpandToTextLength(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_14, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_expand_to_text_length");

        [GodotMethod("get_expand_to_text_length")]
        public bool GetExpandToTextLength()
        {
            return NativeCalls.godot_icall_0_3(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "cursor_set_blink_enabled");

        [GodotMethod("cursor_set_blink_enabled")]
        public void CursorSetBlinkEnabled(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_16, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "cursor_get_blink_enabled");

        [GodotMethod("cursor_get_blink_enabled")]
        public bool CursorGetBlinkEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "cursor_set_blink_speed");

        [GodotMethod("cursor_set_blink_speed")]
        public void CursorSetBlinkSpeed(float blinkSpeed)
        {
            NativeCalls.godot_icall_1_10(method_bind_18, Object.GetPtr(this), blinkSpeed);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "cursor_get_blink_speed");

        [GodotMethod("cursor_get_blink_speed")]
        public float CursorGetBlinkSpeed()
        {
            return NativeCalls.godot_icall_0_9(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_max_length");

        [GodotMethod("set_max_length")]
        public void SetMaxLength(int chars)
        {
            NativeCalls.godot_icall_1_0(method_bind_20, Object.GetPtr(this), chars);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_max_length");

        [GodotMethod("get_max_length")]
        public int GetMaxLength()
        {
            return NativeCalls.godot_icall_0_1(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "append_at_cursor");

        /// <summary>
        /// Adds [code]text[/code] after the cursor. If the resulting value is longer than [member max_length], nothing happens.
        /// </summary>
        [GodotMethod("append_at_cursor")]
        public void AppendAtCursor(string text)
        {
            NativeCalls.godot_icall_1_43(method_bind_22, Object.GetPtr(this), text);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_editable");

        [GodotMethod("set_editable")]
        public void SetEditable(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_23, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_editable");

        [GodotMethod("is_editable")]
        public bool IsEditable()
        {
            return NativeCalls.godot_icall_0_3(method_bind_24, Object.GetPtr(this));
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_secret");

        [GodotMethod("set_secret")]
        public void SetSecret(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_25, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_secret");

        [GodotMethod("is_secret")]
        public bool IsSecret()
        {
            return NativeCalls.godot_icall_0_3(method_bind_26, Object.GetPtr(this));
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "menu_option");

        /// <summary>
        /// Executes a given action as defined in the MENU_* enum.
        /// </summary>
        [GodotMethod("menu_option")]
        public void MenuOption(int option)
        {
            NativeCalls.godot_icall_1_0(method_bind_27, Object.GetPtr(this), option);
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_menu");

        /// <summary>
        /// Returns the [PopupMenu] of this [code]LineEdit[/code]. By default, this menu is displayed when right-clicking on the [LineEdit].
        /// </summary>
        [GodotMethod("get_menu")]
        public PopupMenu GetMenu()
        {
            return NativeCalls.godot_icall_0_498(method_bind_28, Object.GetPtr(this));
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_context_menu_enabled");

        [GodotMethod("set_context_menu_enabled")]
        public void SetContextMenuEnabled(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_29, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_context_menu_enabled");

        [GodotMethod("is_context_menu_enabled")]
        public bool IsContextMenuEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_30, Object.GetPtr(this));
        }
    }
}
