using System;

namespace Godot
{
    /// <summary>
    /// Mesh is a type of [Resource] that contains vertex-array based geometry, divided in [i]surfaces[/i]. Each surface contains a completely separate array and a material used to draw it. Design wise, a mesh with multiple surfaces is preferred to a single surface, because objects created in 3D editing software commonly contain multiple materials.
    /// </summary>
    public class Mesh : Resource
    {
        public enum BlendShapeMode
        {
            Normalized = 0,
            Relative = 1
        }

        public enum PrimitiveType
        {
            Points = 0,
            Lines = 1,
            LineStrip = 2,
            LineLoop = 3,
            Triangles = 4,
            TriangleStrip = 5,
            TriangleFan = 6
        }

        public enum ArrayFormat
        {
            FormatVertex = 1,
            FormatNormal = 2,
            FormatTangent = 4,
            FormatColor = 8,
            FormatTexUv = 16,
            FormatTexUv2 = 32,
            FormatBones = 64,
            FormatWeights = 128,
            FormatIndex = 256,
            CompressBase = 9,
            CompressVertex = 512,
            CompressNormal = 1024,
            CompressTangent = 2048,
            CompressColor = 4096,
            CompressTexUv = 8192,
            CompressTexUv2 = 16384,
            CompressBones = 32768,
            CompressWeights = 65536,
            CompressIndex = 131072,
            FlagUse2dVertices = 262144,
            FlagUse16BitBones = 524288,
            CompressDefault = 97792
        }

        public enum ArrayType
        {
            Vertex = 0,
            Normal = 1,
            Tangent = 2,
            Color = 3,
            TexUv = 4,
            TexUv2 = 5,
            Bones = 6,
            Weights = 7,
            Index = 8,
            Max = 9
        }


        /// <summary>
        /// </summary>
        public Vector2 LightmapSizeHint
        {
            get
            {
                return GetLightmapSizeHint();
            }
            set
            {
                SetLightmapSizeHint(value);
            }
        }


        private const string nativeName = "Mesh";

        internal Mesh() {}

        internal Mesh(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_lightmap_size_hint");

        [GodotMethod("set_lightmap_size_hint")]
        public void SetLightmapSizeHint(Vector2 size)
        {
            NativeCalls.godot_icall_1_45(method_bind_0, Object.GetPtr(this), ref size);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_lightmap_size_hint");

        [GodotMethod("get_lightmap_size_hint")]
        public Vector2 GetLightmapSizeHint()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_1, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create_trimesh_shape");

        /// <summary>
        /// Calculate a [ConcavePolygonShape] from the mesh.
        /// </summary>
        [GodotMethod("create_trimesh_shape")]
        public Shape CreateTrimeshShape()
        {
            return NativeCalls.godot_icall_0_241(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create_convex_shape");

        /// <summary>
        /// Calculate a [ConvexPolygonShape] from the mesh.
        /// </summary>
        [GodotMethod("create_convex_shape")]
        public Shape CreateConvexShape()
        {
            return NativeCalls.godot_icall_0_241(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create_outline");

        /// <summary>
        /// Calculate an outline mesh at a defined offset (margin) from the original mesh. Note: Typically returns the vertices in reverse order (e.g. clockwise to anti-clockwise).
        /// </summary>
        [GodotMethod("create_outline")]
        public Mesh CreateOutline(float margin)
        {
            return NativeCalls.godot_icall_1_502(method_bind_4, Object.GetPtr(this), margin);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_faces");

        /// <summary>
        /// Returns all the vertices that make up the faces of the mesh. Each three vertices represent one triangle.
        /// </summary>
        [GodotMethod("get_faces")]
        public Vector3[] GetFaces()
        {
            return NativeCalls.godot_icall_0_246(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "generate_triangle_mesh");

        /// <summary>
        /// Generate a [TriangleMesh] from the mesh.
        /// </summary>
        [GodotMethod("generate_triangle_mesh")]
        public TriangleMesh GenerateTriangleMesh()
        {
            return NativeCalls.godot_icall_0_503(method_bind_6, Object.GetPtr(this));
        }
    }
}
