using System;

namespace Godot
{
    /// <summary>
    /// MeshInstance is a node that takes a [Mesh] resource and adds it to the current scenario by creating an instance of it. This is the class most often used to get 3D geometry rendered and can be used to instance a single [Mesh] in many places. This allows to reuse geometry and save on resources. When a [Mesh] has to be instanced more than thousands of times at close proximity, consider using a [MultiMesh] in a [MultiMeshInstance] instead.
    /// </summary>
    public class MeshInstance : GeometryInstance
    {
        /// <summary>
        /// The [Mesh] resource for the instance.
        /// </summary>
        public Mesh Mesh
        {
            get
            {
                return GetMesh();
            }
            set
            {
                SetMesh(value);
            }
        }

        /// <summary>
        /// [NodePath] to the [Skeleton] associated with the instance.
        /// </summary>
        public NodePath Skeleton
        {
            get
            {
                return GetSkeletonPath();
            }
            set
            {
                SetSkeletonPath(value);
            }
        }


        private const string nativeName = "MeshInstance";

        public MeshInstance() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_MeshInstance_Ctor(this);
        }

        internal MeshInstance(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_mesh");

        [GodotMethod("set_mesh")]
        public void SetMesh(Mesh mesh)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(mesh));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mesh");

        [GodotMethod("get_mesh")]
        public Mesh GetMesh()
        {
            return NativeCalls.godot_icall_0_511(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_skeleton_path");

        [GodotMethod("set_skeleton_path")]
        public void SetSkeletonPath(NodePath skeletonPath)
        {
            NativeCalls.godot_icall_1_78(method_bind_2, Object.GetPtr(this), NodePath.GetPtr(skeletonPath));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_skeleton_path");

        [GodotMethod("get_skeleton_path")]
        public NodePath GetSkeletonPath()
        {
            IntPtr ret = NativeCalls.godot_icall_0_79(method_bind_3, Object.GetPtr(this));
            return new NodePath(ret);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_surface_material");

        /// <summary>
        /// Sets the [Material] for a surface of the [Mesh] resource.
        /// </summary>
        [GodotMethod("set_surface_material")]
        public void SetSurfaceMaterial(int surface, Material material)
        {
            NativeCalls.godot_icall_2_67(method_bind_4, Object.GetPtr(this), surface, Object.GetPtr(material));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_surface_material");

        /// <summary>
        /// Returns the [Material] for a surface of the [Mesh] resource.
        /// </summary>
        [GodotMethod("get_surface_material")]
        public Material GetSurfaceMaterial(int surface)
        {
            return NativeCalls.godot_icall_1_114(method_bind_5, Object.GetPtr(this), surface);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create_trimesh_collision");

        /// <summary>
        /// This helper creates a [StaticBody] child node with a [ConcavePolygonShape] collision shape calculated from the mesh geometry. It's mainly used for testing.
        /// </summary>
        [GodotMethod("create_trimesh_collision")]
        public void CreateTrimeshCollision()
        {
            NativeCalls.godot_icall_0_12(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create_convex_collision");

        /// <summary>
        /// This helper creates a [StaticBody] child node with a [ConvexPolygonShape] collision shape calculated from the mesh geometry. It's mainly used for testing.
        /// </summary>
        [GodotMethod("create_convex_collision")]
        public void CreateConvexCollision()
        {
            NativeCalls.godot_icall_0_12(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create_debug_tangents");

        /// <summary>
        /// This helper creates a [MeshInstance] child node with gizmos at every vertex calculated from the mesh geometry. It's mainly used for testing.
        /// </summary>
        [GodotMethod("create_debug_tangents")]
        public void CreateDebugTangents()
        {
            NativeCalls.godot_icall_0_12(method_bind_8, Object.GetPtr(this));
        }
    }
}
