using System;

namespace Godot
{
    /// <summary>
    /// This is a generic mobile VR implementation where you need to provide details about the phone and HMD used. It does not rely on any existing framework. This is the most basic interface we have. For the best effect you do need a mobile phone with a gyroscope and accelerometer.
    /// Note that even though there is no positional tracking the camera will assume the headset is at a height of 1.85 meters.
    /// </summary>
    public class MobileVRInterface : ARVRInterface
    {
        /// <summary>
        /// The interocular distance, also known as the interpupillary distance. The distance between the pupils of the left and right eye.
        /// </summary>
        public float Iod
        {
            get
            {
                return GetIod();
            }
            set
            {
                SetIod(value);
            }
        }

        /// <summary>
        /// The width of the display in centimeters.
        /// </summary>
        public float DisplayWidth
        {
            get
            {
                return GetDisplayWidth();
            }
            set
            {
                SetDisplayWidth(value);
            }
        }

        /// <summary>
        /// The distance between the display and the lenses inside of the device in centimeters.
        /// </summary>
        public float DisplayToLens
        {
            get
            {
                return GetDisplayToLens();
            }
            set
            {
                SetDisplayToLens(value);
            }
        }

        /// <summary>
        /// The oversample setting. Because of the lens distortion we have to render our buffers at a higher resolution then the screen can natively handle. A value between 1.5 and 2.0 often provides good results but at the cost of performance.
        /// </summary>
        public float Oversample
        {
            get
            {
                return GetOversample();
            }
            set
            {
                SetOversample(value);
            }
        }

        /// <summary>
        /// The k1 lens factor is one of the two constants that define the strength of the lens used and directly influences the lens distortion effect.
        /// </summary>
        public float K1
        {
            get
            {
                return GetK1();
            }
            set
            {
                SetK1(value);
            }
        }

        /// <summary>
        /// The k2 lens factor, see k1.
        /// </summary>
        public float K2
        {
            get
            {
                return GetK2();
            }
            set
            {
                SetK2(value);
            }
        }


        private const string nativeName = "MobileVRInterface";

        public MobileVRInterface() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_MobileVRInterface_Ctor(this);
        }

        internal MobileVRInterface(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_iod");

        [GodotMethod("set_iod")]
        public void SetIod(float iod)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), iod);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_iod");

        [GodotMethod("get_iod")]
        public float GetIod()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_display_width");

        [GodotMethod("set_display_width")]
        public void SetDisplayWidth(float displayWidth)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), displayWidth);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_display_width");

        [GodotMethod("get_display_width")]
        public float GetDisplayWidth()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_display_to_lens");

        [GodotMethod("set_display_to_lens")]
        public void SetDisplayToLens(float displayToLens)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), displayToLens);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_display_to_lens");

        [GodotMethod("get_display_to_lens")]
        public float GetDisplayToLens()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_oversample");

        [GodotMethod("set_oversample")]
        public void SetOversample(float oversample)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), oversample);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_oversample");

        [GodotMethod("get_oversample")]
        public float GetOversample()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_k1");

        [GodotMethod("set_k1")]
        public void SetK1(float k)
        {
            NativeCalls.godot_icall_1_10(method_bind_8, Object.GetPtr(this), k);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_k1");

        [GodotMethod("get_k1")]
        public float GetK1()
        {
            return NativeCalls.godot_icall_0_9(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_k2");

        [GodotMethod("set_k2")]
        public void SetK2(float k)
        {
            NativeCalls.godot_icall_1_10(method_bind_10, Object.GetPtr(this), k);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_k2");

        [GodotMethod("get_k2")]
        public float GetK2()
        {
            return NativeCalls.godot_icall_0_9(method_bind_11, Object.GetPtr(this));
        }
    }
}
