using System;

namespace Godot
{
    /// <summary>
    /// MultiMesh provides low level mesh instancing. If the amount of [Mesh] instances needed goes from hundreds to thousands (and most need to be visible at close proximity) creating such a large amount of [MeshInstance] nodes may affect performance by using too much CPU or video memory.
    /// For this case a MultiMesh becomes very useful, as it can draw thousands of instances with little API overhead.
    /// As a drawback, if the instances are too far away of each other, performance may be reduced as every single instance will always rendered (they are spatially indexed as one, for the whole object).
    /// Since instances may have any behavior, the AABB used for visibility must be provided by the user.
    /// </summary>
    public class MultiMesh : Resource
    {
        public enum TransformFormatEnum
        {
            Transform2d = 0,
            Transform3d = 1
        }

        public enum ColorFormatEnum
        {
            None = 0,
            Color8bit = 1,
            Float = 2
        }


        /// <summary>
        /// </summary>
        public MultiMesh.ColorFormatEnum ColorFormat
        {
            get
            {
                return GetColorFormat();
            }
            set
            {
                SetColorFormat(value);
            }
        }

        /// <summary>
        /// </summary>
        public MultiMesh.TransformFormatEnum TransformFormat
        {
            get
            {
                return GetTransformFormat();
            }
            set
            {
                SetTransformFormat(value);
            }
        }

        /// <summary>
        /// </summary>
        public int InstanceCount
        {
            get
            {
                return GetInstanceCount();
            }
            set
            {
                SetInstanceCount(value);
            }
        }

        /// <summary>
        /// </summary>
        public Mesh Mesh
        {
            get
            {
                return GetMesh();
            }
            set
            {
                SetMesh(value);
            }
        }

        public Vector3[] TransformArray
        {
            get
            {
                return _GetTransformArray();
            }
            set
            {
                _SetTransformArray(value);
            }
        }

        public Color[] ColorArray
        {
            get
            {
                return _GetColorArray();
            }
            set
            {
                _SetColorArray(value);
            }
        }


        private const string nativeName = "MultiMesh";

        public MultiMesh() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_MultiMesh_Ctor(this);
        }

        internal MultiMesh(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_mesh");

        [GodotMethod("set_mesh")]
        public void SetMesh(Mesh mesh)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(mesh));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mesh");

        [GodotMethod("get_mesh")]
        public Mesh GetMesh()
        {
            return NativeCalls.godot_icall_0_511(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_color_format");

        [GodotMethod("set_color_format")]
        public void SetColorFormat(MultiMesh.ColorFormatEnum format)
        {
            NativeCalls.godot_icall_1_515(method_bind_2, Object.GetPtr(this), format);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_color_format");

        [GodotMethod("get_color_format")]
        public MultiMesh.ColorFormatEnum GetColorFormat()
        {
            return NativeCalls.godot_icall_0_516(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_transform_format");

        [GodotMethod("set_transform_format")]
        public void SetTransformFormat(MultiMesh.TransformFormatEnum format)
        {
            NativeCalls.godot_icall_1_517(method_bind_4, Object.GetPtr(this), format);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_transform_format");

        [GodotMethod("get_transform_format")]
        public MultiMesh.TransformFormatEnum GetTransformFormat()
        {
            return NativeCalls.godot_icall_0_518(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_instance_count");

        [GodotMethod("set_instance_count")]
        public void SetInstanceCount(int count)
        {
            NativeCalls.godot_icall_1_0(method_bind_6, Object.GetPtr(this), count);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_instance_count");

        [GodotMethod("get_instance_count")]
        public int GetInstanceCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_instance_transform");

        /// <summary>
        /// Set the transform for a specific instance.
        /// </summary>
        [GodotMethod("set_instance_transform")]
        public void SetInstanceTransform(int instance, Transform transform)
        {
            NativeCalls.godot_icall_2_230(method_bind_8, Object.GetPtr(this), instance, ref transform);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_instance_transform");

        /// <summary>
        /// Return the transform of a specific instance.
        /// </summary>
        [GodotMethod("get_instance_transform")]
        public Transform GetInstanceTransform(int instance)
        {
            object ret = NativeCalls.godot_icall_1_231(method_bind_9, Object.GetPtr(this), instance);
            return (Transform)ret;
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_instance_color");

        /// <summary>
        /// Set the color of a specific instance.
        /// </summary>
        [GodotMethod("set_instance_color")]
        public void SetInstanceColor(int instance, Color color)
        {
            NativeCalls.godot_icall_2_384(method_bind_10, Object.GetPtr(this), instance, ref color);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_instance_color");

        /// <summary>
        /// Get the color of a specific instance.
        /// </summary>
        [GodotMethod("get_instance_color")]
        public Color GetInstanceColor(int instance)
        {
            object ret = NativeCalls.godot_icall_1_385(method_bind_11, Object.GetPtr(this), instance);
            return (Color)ret;
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_aabb");

        /// <summary>
        /// Return the visibility AABB.
        /// </summary>
        [GodotMethod("get_aabb")]
        public AABB GetAabb()
        {
            object ret = NativeCalls.godot_icall_0_118(method_bind_12, Object.GetPtr(this));
            return (AABB)ret;
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_transform_array");

        internal void _SetTransformArray(Vector3[] arg0)
        {
            NativeCalls.godot_icall_1_245(method_bind_13, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_transform_array");

        internal Vector3[] _GetTransformArray()
        {
            return NativeCalls.godot_icall_0_246(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_color_array");

        internal void _SetColorArray(Color[] arg0)
        {
            NativeCalls.godot_icall_1_388(method_bind_15, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_color_array");

        internal Color[] _GetColorArray()
        {
            return NativeCalls.godot_icall_0_389(method_bind_16, Object.GetPtr(this));
        }
    }
}
