using System;

namespace Godot
{
    /// <summary>
    /// The Navigation node is used for basic or advanced navigation. By default it will automatically collect all child [code]NavigationMesh[/code] resources, but they can also be added on the fly through scripting. It can be used for generating a simple path between two points or it can be used to ensure that a navigation agent is angled perfectly to the terrain it is navigating.
    /// </summary>
    public class Navigation : Spatial
    {
        /// <summary>
        /// Defines which direction is up. The default defines 0,1,0 as up which is the world up direction. To make this a ceiling use 0,-1,0 to define down as up.
        /// </summary>
        public Vector3 UpVector
        {
            get
            {
                return GetUpVector();
            }
            set
            {
                SetUpVector(value);
            }
        }


        private const string nativeName = "Navigation";

        public Navigation() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Navigation_Ctor(this);
        }

        internal Navigation(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "navmesh_add");

        /// <summary>
        /// Adds a [code]NavigationMesh[/code] to the list of NavigationMesh's in this node. Returns an id. Its position, rotation and scale are associated with the [code]Transform[/code] passed. The [code]Node[/code] (or [code]Object[/code]) that owns this node is an optional parameter.
        /// </summary>
        [GodotMethod("navmesh_add")]
        public int NavmeshAdd(NavigationMesh mesh, Transform xform, Object owner = null)
        {
            return NativeCalls.godot_icall_3_520(method_bind_0, Object.GetPtr(this), Object.GetPtr(mesh), ref xform, Object.GetPtr(owner));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "navmesh_set_transform");

        /// <summary>
        /// Associates a [code]NavigationMesh[/code]'s id with a [code]Transform[/code]. Its position, rotation and scale are based on the [code]Transform[/code] passed.
        /// </summary>
        [GodotMethod("navmesh_set_transform")]
        public void NavmeshSetTransform(int id, Transform xform)
        {
            NativeCalls.godot_icall_2_230(method_bind_1, Object.GetPtr(this), id, ref xform);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "navmesh_remove");

        /// <summary>
        /// Removes a [code]NavigationMesh[/code] from the list of NavigationMesh's in this node.
        /// </summary>
        [GodotMethod("navmesh_remove")]
        public void NavmeshRemove(int id)
        {
            NativeCalls.godot_icall_1_0(method_bind_2, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_simple_path");

        /// <summary>
        /// Returns a path of points as a [code]PoolVector3Array[/code]. If [code]optimize[/code] is false the [code]NavigationMesh[/code] agent properties will be taken into account, otherwise it will return the nearest path and ignore agent radius, height, etc.
        /// </summary>
        [GodotMethod("get_simple_path")]
        public Vector3[] GetSimplePath(Vector3 start, Vector3 end, bool optimize = true)
        {
            return NativeCalls.godot_icall_3_521(method_bind_3, Object.GetPtr(this), ref start, ref end, optimize);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_closest_point_to_segment");

        /// <summary>
        /// Returns the nearest point to the line segment passed. The third optional parameter takes collisions into account.
        /// </summary>
        [GodotMethod("get_closest_point_to_segment")]
        public Vector3 GetClosestPointToSegment(Vector3 start, Vector3 end, bool useCollision = false)
        {
            object ret = NativeCalls.godot_icall_3_522(method_bind_4, Object.GetPtr(this), ref start, ref end, useCollision);
            return (Vector3)ret;
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_closest_point");

        /// <summary>
        /// Returns the closest navigation point to the point passed.
        /// </summary>
        [GodotMethod("get_closest_point")]
        public Vector3 GetClosestPoint(Vector3 toPoint)
        {
            object ret = NativeCalls.godot_icall_1_35(method_bind_5, Object.GetPtr(this), ref toPoint);
            return (Vector3)ret;
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_closest_point_normal");

        /// <summary>
        /// Returns the surface normal of the navigation mesh at the point passed. For instance, if the point passed was at a 45 degree slope it would return something like (0.5,0.5,0). This is useful for rotating a navigation agent in accordance with the [code]NavigationMesh[/code].
        /// </summary>
        [GodotMethod("get_closest_point_normal")]
        public Vector3 GetClosestPointNormal(Vector3 toPoint)
        {
            object ret = NativeCalls.godot_icall_1_35(method_bind_6, Object.GetPtr(this), ref toPoint);
            return (Vector3)ret;
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_closest_point_owner");

        /// <summary>
        /// Returns the nearest [code]NavigationMeshInstance[/code] to the point passed.
        /// </summary>
        [GodotMethod("get_closest_point_owner")]
        public Object GetClosestPointOwner(Vector3 toPoint)
        {
            return NativeCalls.godot_icall_1_523(method_bind_7, Object.GetPtr(this), ref toPoint);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_up_vector");

        [GodotMethod("set_up_vector")]
        public void SetUpVector(Vector3 up)
        {
            NativeCalls.godot_icall_1_104(method_bind_8, Object.GetPtr(this), ref up);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_up_vector");

        [GodotMethod("get_up_vector")]
        public Vector3 GetUpVector()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_9, Object.GetPtr(this));
            return (Vector3)ret;
        }
    }
}
