using System;

namespace Godot
{
    /// <summary>
    /// Better known as 9-slice panels, NinePatchRect produces clean panels of any size, based on a small texture. To do so, it splits the texture in a 3 by 3 grid. When you scale the node, it tiles the texture's sides horizontally or vertically, the center on both axes but it doesn't scale or tile the corners.
    /// </summary>
    public class NinePatchRect : Control
    {
        public enum AxisStretchMode
        {
            Stretch = 0,
            Tile = 1,
            TileFit = 2
        }


        /// <summary>
        /// The node's texture resource.
        /// </summary>
        public Texture Texture
        {
            get
            {
                return GetTexture();
            }
            set
            {
                SetTexture(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], draw the panel's center. Else, only draw the 9-slice's borders. Default value: [code]true[/code]
        /// </summary>
        public bool DrawCenter
        {
            get
            {
                return IsDrawCenterEnabled();
            }
            set
            {
                SetDrawCenter(value);
            }
        }

        /// <summary>
        /// Rectangular region of the texture to sample from. If you're working with an atlas, use this property to define the area the 9-slice should use. All other properties are relative to this one.
        /// </summary>
        public Rect2 RegionRect
        {
            get
            {
                return GetRegionRect();
            }
            set
            {
                SetRegionRect(value);
            }
        }

        /// <summary>
        /// The height of the 9-slice's left column.
        /// </summary>
        public int PatchMarginLeft
        {
            get
            {
                return GetPatchMargin((Margin)0);
            }
            set
            {
                SetPatchMargin((Margin)0, value);
            }
        }

        /// <summary>
        /// The height of the 9-slice's top row.
        /// </summary>
        public int PatchMarginTop
        {
            get
            {
                return GetPatchMargin((Margin)1);
            }
            set
            {
                SetPatchMargin((Margin)1, value);
            }
        }

        /// <summary>
        /// The height of the 9-slice's right column.
        /// </summary>
        public int PatchMarginRight
        {
            get
            {
                return GetPatchMargin((Margin)2);
            }
            set
            {
                SetPatchMargin((Margin)2, value);
            }
        }

        /// <summary>
        /// The height of the 9-slice's bottom row. A margin of 16 means the 9-slice's bottom corners and side will have a height of 16 pixels. You can set all 4 margin values individually to create panels with non-uniform borders.
        /// </summary>
        public int PatchMarginBottom
        {
            get
            {
                return GetPatchMargin((Margin)3);
            }
            set
            {
                SetPatchMargin((Margin)3, value);
            }
        }

        /// <summary>
        /// Doesn't do anything at the time of writing.
        /// </summary>
        public NinePatchRect.AxisStretchMode AxisStretchHorizontal
        {
            get
            {
                return GetHAxisStretchMode();
            }
            set
            {
                SetHAxisStretchMode(value);
            }
        }

        /// <summary>
        /// Doesn't do anything at the time of writing.
        /// </summary>
        public NinePatchRect.AxisStretchMode AxisStretchVertical
        {
            get
            {
                return GetVAxisStretchMode();
            }
            set
            {
                SetVAxisStretchMode(value);
            }
        }


        private const string nativeName = "NinePatchRect";

        public NinePatchRect() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_NinePatchRect_Ctor(this);
        }

        internal NinePatchRect(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_texture");

        [GodotMethod("set_texture")]
        public void SetTexture(Texture texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture");

        [GodotMethod("get_texture")]
        public Texture GetTexture()
        {
            return NativeCalls.godot_icall_0_119(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_patch_margin");

        [GodotMethod("set_patch_margin")]
        public void SetPatchMargin(Margin margin, int value)
        {
            NativeCalls.godot_icall_2_196(method_bind_2, Object.GetPtr(this), margin, value);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_patch_margin");

        [GodotMethod("get_patch_margin")]
        public int GetPatchMargin(Margin margin)
        {
            return NativeCalls.godot_icall_1_197(method_bind_3, Object.GetPtr(this), margin);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_region_rect");

        [GodotMethod("set_region_rect")]
        public void SetRegionRect(Rect2 rect)
        {
            NativeCalls.godot_icall_1_120(method_bind_4, Object.GetPtr(this), ref rect);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_region_rect");

        [GodotMethod("get_region_rect")]
        public Rect2 GetRegionRect()
        {
            object ret = NativeCalls.godot_icall_0_121(method_bind_5, Object.GetPtr(this));
            return (Rect2)ret;
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_draw_center");

        [GodotMethod("set_draw_center")]
        public void SetDrawCenter(bool drawCenter)
        {
            NativeCalls.godot_icall_1_11(method_bind_6, Object.GetPtr(this), drawCenter);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_draw_center_enabled");

        [GodotMethod("is_draw_center_enabled")]
        public bool IsDrawCenterEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_h_axis_stretch_mode");

        [GodotMethod("set_h_axis_stretch_mode")]
        public void SetHAxisStretchMode(NinePatchRect.AxisStretchMode mode)
        {
            NativeCalls.godot_icall_1_539(method_bind_8, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_h_axis_stretch_mode");

        [GodotMethod("get_h_axis_stretch_mode")]
        public NinePatchRect.AxisStretchMode GetHAxisStretchMode()
        {
            return NativeCalls.godot_icall_0_540(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_v_axis_stretch_mode");

        [GodotMethod("set_v_axis_stretch_mode")]
        public void SetVAxisStretchMode(NinePatchRect.AxisStretchMode mode)
        {
            NativeCalls.godot_icall_1_539(method_bind_10, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_v_axis_stretch_mode");

        [GodotMethod("get_v_axis_stretch_mode")]
        public NinePatchRect.AxisStretchMode GetVAxisStretchMode()
        {
            return NativeCalls.godot_icall_0_540(method_bind_11, Object.GetPtr(this));
        }
    }
}
