using System;

namespace Godot
{
    /// <summary>
    /// Nodes are Godot's building blocks. They can be assigned as the child of another node, resulting in a tree arrangement. A given node can contain any number of nodes as children with the requirement that all siblings (direct children of a node) should have unique names.
    /// A tree of nodes is called a [i]scene[/i]. Scenes can be saved to the disk and then instanced into other scenes. This allows for very high flexibility in the architecture and data model of Godot projects. Nodes can also optionally be added to groups. This makes it possible to access a number of nodes from code (an "enemies" group, for example) to perform grouped actions.
    /// [b]Scene tree:[/b] The [SceneTree] contains the active tree of nodes. When a node is added to the scene tree, it receives the NOTIFICATION_ENTER_TREE notification and its [method _enter_tree] callback is triggered. Child nodes are always added [i]after[/i] their parent node, i.e. the [method _enter_tree] callback of a parent node will be triggered before its child's.
    /// Once all nodes have been added in the scene tree, they receive the NOTIFICATION_READY notification and their respective [method _ready] callbacks are triggered. For groups of nodes, the [method _ready] callback is called in reverse order, starting with the children and moving up to the parent nodes.
    /// This means that when adding a node to the scene tree, the following order will be used for the callbacks: [method _enter_tree] of the parent, [method _enter_tree] of the children, [method _ready] of the children and finally [method _ready] of the parent (recursively for the entire scene tree).
    /// [b]Processing:[/b] Nodes can override the "process" state, so that they receive a callback on each frame requesting them to process (do something). Normal processing (callback [method _process], toggled with [method set_process]) happens as fast as possible and is dependent on the frame rate, so the processing time [i]delta[/i] is passed as an argument. Physics processing (callback [method _physics_process], toggled with [method set_physics_process]) happens a fixed number of times per second (60 by default) and is useful for code related to the physics engine.
    /// Nodes can also process input events. When present, the [method _input] function will be called for each input that the program receives. In many cases, this can be overkill (unless used for simple projects), and the [method _unhandled_input] function might be preferred; it is called when the input event was not handled by anyone else (typically, GUI [Control] nodes), ensuring that the node only receives the events that were meant for it.
    /// To keep track of the scene hierarchy (especially when instancing scenes into other scenes), an "owner" can be set for the node with [method set_owner]. This keeps track of who instanced what. This is mostly useful when writing editors and tools, though.
    /// Finally, when a node is freed with [method free] or [method queue_free], it will also free all its children.
    /// [b]Networking with nodes:[/b] After connecting to a server (or making one, see [NetworkedMultiplayerENet]) it is possible to use the built-in RPC (remote procedure call) system to communicate over the network. By calling [method rpc] with a method name, it will be called locally and in all connected peers (peers = clients and the server that accepts connections), with behaviour varying depending on the network mode ([method set_network_mode]) of the receiving peer. To identify which node receives the RPC call Godot will use its [NodePath] (make sure node names are the same on all peers).
    /// </summary>
    public class Node : Object
    {
        public const int NotificationEnterTree = 10;
        public const int NotificationExitTree = 11;
        public const int NotificationMovedInParent = 12;
        public const int NotificationReady = 13;
        public const int NotificationPaused = 14;
        public const int NotificationUnpaused = 15;
        public const int NotificationPhysicsProcess = 16;
        public const int NotificationProcess = 17;
        public const int NotificationParented = 18;
        public const int NotificationUnparented = 19;
        public const int NotificationInstanced = 20;
        public const int NotificationDragBegin = 21;
        public const int NotificationDragEnd = 22;
        public const int NotificationPathChanged = 23;
        public const int NotificationTranslationChanged = 24;
        public const int NotificationInternalProcess = 25;
        public const int NotificationInternalPhysicsProcess = 26;

        public enum PauseModeEnum
        {
            Inherit = 0,
            Stop = 1,
            Process = 2
        }

        public enum DuplicateFlags
        {
            Signals = 1,
            Groups = 2,
            Scripts = 4,
            UseInstancing = 8
        }

        public enum RPCMode
        {
            Disabled = 0,
            Remote = 1,
            Sync = 2,
            Master = 3,
            Slave = 4
        }


        public NodePath _ImportPath
        {
            get
            {
                return _GetImportPath();
            }
            set
            {
                _SetImportPath(value);
            }
        }

        /// <summary>
        /// </summary>
        public Node.PauseModeEnum PauseMode
        {
            get
            {
                return GetPauseMode();
            }
            set
            {
                SetPauseMode(value);
            }
        }

        public bool Editor__displayFolded
        {
            get
            {
                return IsDisplayedFolded();
            }
            set
            {
                SetDisplayFolded(value);
            }
        }

        /// <summary>
        /// The name of the node. This name is unique among the siblings (other child nodes from the same parent).
        /// When set to an existing name, the node will be automatically renamed
        /// </summary>
        public string Name
        {
            get
            {
                return GetName();
            }
            set
            {
                SetName(value);
            }
        }

        /// <summary>
        /// When a scene is instanced from a file, its topmost node contains the filename from which it was loaded.
        /// </summary>
        public string Filename
        {
            get
            {
                return GetFilename();
            }
            set
            {
                SetFilename(value);
            }
        }

        /// <summary>
        /// The node owner. A node can have any other node as owner (as long as it is a valid parent, grandparent, etc. ascending in the tree). When saving a node (using SceneSaver) all the nodes it owns will be saved with it. This allows for the creation of complex [SceneTree]s, with instancing and subinstancing.
        /// </summary>
        public Node Owner
        {
            get
            {
                return GetOwner();
            }
            set
            {
                SetOwner(value);
            }
        }


        private const string nativeName = "Node";

        public Node() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Node_Ctor(this);
        }

        internal Node(bool memoryOwn) : base(memoryOwn) {}

        /// <summary>
        /// Called when the node enters the [SceneTree] (e.g. upon instancing, scene changing, or after calling [method add_child] in a script). If the node has children, its [method _enter_tree] callback will be called first, and then that of the children.
        /// Corresponds to the NOTIFICATION_ENTER_TREE notification in [method Object._notification].
        /// </summary>
        [GodotMethod("_enter_tree")]
        public virtual void _EnterTree()
        {
            return;
        }

        /// <summary>
        /// Called when the node is about to leave the [SceneTree] (e.g. upon freeing, scene changing, or after calling [method remove_child] in a script). If the node has children, its [method _exit_tree] callback will be called last, after all its children have left the tree.
        /// Corresponds to the NOTIFICATION_EXIT_TREE notification in [method Object._notification] and signal [signal tree_exiting]. To get notified when the node has already left the active tree, connect to the [signal tree_exited]
        /// </summary>
        [GodotMethod("_exit_tree")]
        public virtual void _ExitTree()
        {
            return;
        }

        /// <summary>
        /// Called when there is an input event. The input event propagates through the node tree until a node consumes it.
        /// It is only called if input processing is enabled, which is done automatically if this method is overriden, and can be toggled with [method set_process_input].
        /// </summary>
        [GodotMethod("_input")]
        public virtual void _Input(InputEvent @event)
        {
            return;
        }

        /// <summary>
        /// Called during the physics processing step of the main loop. Physics processing means that the frame rate is synced to the physics, i.e. the [code]delta[/code] variable should be constant.
        /// It is only called if physics processing is enabled, which is done automatically if this method is overriden, and can be toggled with [method set_physics_process].
        /// Corresponds to the NOTIFICATION_PHYSICS_PROCESS notification in [method Object._notification].
        /// </summary>
        [GodotMethod("_physics_process")]
        public virtual void _PhysicsProcess(float delta)
        {
            return;
        }

        /// <summary>
        /// Called during the processing step of the main loop. Processing happens at every frame and as fast as possible, so the [code]delta[/code] time since the previous frame is not constant.
        /// It is only called if processing is enabled, which is done automatically if this method is overriden, and can be toggled with [method set_process].
        /// Corresponds to the NOTIFICATION_PROCESS notification in [method Object._notification].
        /// </summary>
        [GodotMethod("_process")]
        public virtual void _Process(float delta)
        {
            return;
        }

        /// <summary>
        /// Called when the node is "ready", i.e. when both the node and its children have entered the scene tree. If the node has children, their [method _ready] callbacks get triggered first, and the parent node will receive the ready notification afterwards.
        /// Corresponds to the NOTIFICATION_READY notification in [method Object._notification].
        /// </summary>
        [GodotMethod("_ready")]
        public virtual void _Ready()
        {
            return;
        }

        /// <summary>
        /// Propagated to all nodes when the previous InputEvent is not consumed by any nodes.
        /// It is only called if unhandled input processing is enabled, which is done automatically if this method is overriden, and can be toggled with [method set_process_unhandled_input].
        /// </summary>
        [GodotMethod("_unhandled_input")]
        public virtual void _UnhandledInput(InputEvent @event)
        {
            return;
        }

        /// <summary>
        /// </summary>
        [GodotMethod("_unhandled_key_input")]
        public virtual void _UnhandledKeyInput(InputEventKey @event)
        {
            return;
        }

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_child_below_node");

        /// <summary>
        /// Adds a child node. The child is placed below the given node in the list of children.
        /// Setting "legible_unique_name" [code]true[/code] creates child nodes with human-readable names, based on the name of the node being instanced instead of its type.
        /// </summary>
        [GodotMethod("add_child_below_node")]
        public void AddChildBelowNode(Node node, Node childNode, bool legibleUniqueName = false)
        {
            NativeCalls.godot_icall_3_541(method_bind_0, Object.GetPtr(this), Object.GetPtr(node), Object.GetPtr(childNode), legibleUniqueName);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_name");

        [GodotMethod("set_name")]
        public void SetName(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_1, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_name");

        [GodotMethod("get_name")]
        public string GetName()
        {
            return NativeCalls.godot_icall_0_2(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_child");

        /// <summary>
        /// Adds a child node. Nodes can have any number of children, but every child must have a unique name. Child nodes are automatically deleted when the parent node is deleted, so an entire scene can be removed by deleting its topmost node.
        /// Setting "legible_unique_name" [code]true[/code] creates child nodes with human-readable names, based on the name of the node being instanced instead of its type.
        /// </summary>
        [GodotMethod("add_child")]
        public void AddChild(Node node, bool legibleUniqueName = false)
        {
            NativeCalls.godot_icall_2_374(method_bind_3, Object.GetPtr(this), Object.GetPtr(node), legibleUniqueName);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_child");

        /// <summary>
        /// Removes a child node. The node is NOT deleted and must be deleted manually.
        /// </summary>
        [GodotMethod("remove_child")]
        public void RemoveChild(Node node)
        {
            NativeCalls.godot_icall_1_42(method_bind_4, Object.GetPtr(this), Object.GetPtr(node));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_child_count");

        /// <summary>
        /// Returns the number of child nodes.
        /// </summary>
        [GodotMethod("get_child_count")]
        public int GetChildCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_children");

        /// <summary>
        /// Returns an array of references to node's children.
        /// </summary>
        [GodotMethod("get_children")]
        public object[] GetChildren()
        {
            return NativeCalls.godot_icall_0_21(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_child");

        /// <summary>
        /// Returns a child node by its index (see [method get_child_count]). This method is often used for iterating all children of a node.
        /// </summary>
        [GodotMethod("get_child")]
        public Node GetChild(int idx)
        {
            return NativeCalls.godot_icall_1_542(method_bind_7, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_node");

        /// <summary>
        /// Returns [code]true[/code] if the node that the [NodePath] points to exists.
        /// </summary>
        [GodotMethod("has_node")]
        public bool HasNode(NodePath path)
        {
            return NativeCalls.godot_icall_1_543(method_bind_8, Object.GetPtr(this), NodePath.GetPtr(path));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node");

        /// <summary>
        /// Fetches a node. The [NodePath] can be either a relative path (from the current node) or an absolute path (in the scene tree) to a node. If the path does not exist, a [code]null instance[/code] is returned and attempts to access it will result in an "Attempt to call &lt;method&gt; on a null instance." error.
        /// Note: fetching absolute paths only works when the node is inside the scene tree (see [method is_inside_tree]).
        /// [i]Example:[/i] Assume your current node is Character and the following tree:
        /// [codeblock]
        /// /root
        /// /root/Character
        /// /root/Character/Sword
        /// /root/Character/Backpack/Dagger
        /// /root/MyGame
        /// /root/Swamp/Alligator
        /// /root/Swamp/Mosquito
        /// /root/Swamp/Goblin
        /// [/codeblock]
        /// Possible paths are:
        /// [codeblock]
        /// get_node("Sword")
        /// get_node("Backpack/Dagger")
        /// get_node("../Swamp/Alligator")
        /// get_node("/root/MyGame")
        /// [/codeblock]
        /// </summary>
        [GodotMethod("get_node")]
        public Node GetNode(NodePath path)
        {
            return NativeCalls.godot_icall_1_544(method_bind_9, Object.GetPtr(this), NodePath.GetPtr(path));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_parent");

        /// <summary>
        /// Returns the parent node of the current node, or an empty [code]Node[/code] if the node lacks a parent.
        /// </summary>
        [GodotMethod("get_parent")]
        public Node GetParent()
        {
            return NativeCalls.godot_icall_0_200(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "find_node");

        /// <summary>
        /// Finds a descendant of this node whose name matches [code]mask[/code] as in [method String.match] (i.e. case sensitive, but '*' matches zero or more characters and '?' matches any single character except '.'). Note that it does not match against the full path, just against individual node names.
        /// </summary>
        [GodotMethod("find_node")]
        public Node FindNode(string mask, bool recursive = true, bool owned = true)
        {
            return NativeCalls.godot_icall_3_545(method_bind_11, Object.GetPtr(this), mask, recursive, owned);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_node_and_resource");

        /// <summary>
        /// </summary>
        [GodotMethod("has_node_and_resource")]
        public bool HasNodeAndResource(NodePath path)
        {
            return NativeCalls.godot_icall_1_543(method_bind_12, Object.GetPtr(this), NodePath.GetPtr(path));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_and_resource");

        /// <summary>
        /// </summary>
        [GodotMethod("get_node_and_resource")]
        public object[] GetNodeAndResource(NodePath path)
        {
            return NativeCalls.godot_icall_1_546(method_bind_13, Object.GetPtr(this), NodePath.GetPtr(path));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_inside_tree");

        /// <summary>
        /// Returns [code]true[/code] if this node is currently inside a [SceneTree].
        /// </summary>
        [GodotMethod("is_inside_tree")]
        public bool IsInsideTree()
        {
            return NativeCalls.godot_icall_0_3(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_a_parent_of");

        /// <summary>
        /// Returns [code]true[/code] if the given node is a direct or indirect child of the current node.
        /// </summary>
        [GodotMethod("is_a_parent_of")]
        public bool IsAParentOf(Node node)
        {
            return NativeCalls.godot_icall_1_105(method_bind_15, Object.GetPtr(this), Object.GetPtr(node));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_greater_than");

        /// <summary>
        /// Returns [code]true[/code] if the given node occurs later in the scene hierarchy than the current node.
        /// </summary>
        [GodotMethod("is_greater_than")]
        public bool IsGreaterThan(Node node)
        {
            return NativeCalls.godot_icall_1_105(method_bind_16, Object.GetPtr(this), Object.GetPtr(node));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_path");

        /// <summary>
        /// Returns the absolute path of the current node. This only works if the current node is inside the scene tree (see [method is_inside_tree]).
        /// </summary>
        [GodotMethod("get_path")]
        public NodePath GetPath()
        {
            IntPtr ret = NativeCalls.godot_icall_0_79(method_bind_17, Object.GetPtr(this));
            return new NodePath(ret);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_path_to");

        /// <summary>
        /// Returns the relative path from the current node to the specified node in "node" argument. Both nodes must be in the same scene, or the function will fail.
        /// </summary>
        [GodotMethod("get_path_to")]
        public NodePath GetPathTo(Node node)
        {
            IntPtr ret = NativeCalls.godot_icall_1_547(method_bind_18, Object.GetPtr(this), Object.GetPtr(node));
            return new NodePath(ret);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_to_group");

        /// <summary>
        /// Adds the node to a group. Groups are helpers to name and organize a subset of nodes, for example "enemies" or "collectables". A node can be in any number of groups. Nodes can be assigned a group at any time, but will not be added until they are inside the scene tree (see [method is_inside_tree]).
        /// </summary>
        [GodotMethod("add_to_group")]
        public void AddToGroup(string group, bool persistent = false)
        {
            NativeCalls.godot_icall_2_92(method_bind_19, Object.GetPtr(this), group, persistent);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_from_group");

        /// <summary>
        /// Removes a node from a group.
        /// </summary>
        [GodotMethod("remove_from_group")]
        public void RemoveFromGroup(string group)
        {
            NativeCalls.godot_icall_1_43(method_bind_20, Object.GetPtr(this), group);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_in_group");

        /// <summary>
        /// Returns [code]true[/code] if this node is in the specified group.
        /// </summary>
        [GodotMethod("is_in_group")]
        public bool IsInGroup(string group)
        {
            return NativeCalls.godot_icall_1_70(method_bind_21, Object.GetPtr(this), group);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "move_child");

        /// <summary>
        /// Moves a child node to a different position (order) amongst the other children. Since calls, signals, etc are performed by tree order, changing the order of children nodes may be useful.
        /// </summary>
        [GodotMethod("move_child")]
        public void MoveChild(Node childNode, int toPosition)
        {
            NativeCalls.godot_icall_2_548(method_bind_22, Object.GetPtr(this), Object.GetPtr(childNode), toPosition);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_groups");

        /// <summary>
        /// Returns an array listing the groups that the node is a member of.
        /// </summary>
        [GodotMethod("get_groups")]
        public object[] GetGroups()
        {
            return NativeCalls.godot_icall_0_21(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "raise");

        /// <summary>
        /// Moves this node to the top of the array of nodes of the parent node. This is often useful in GUIs ([Control] nodes), because their order of drawing depends on their order in the tree.
        /// </summary>
        [GodotMethod("raise")]
        public void Raise()
        {
            NativeCalls.godot_icall_0_12(method_bind_24, Object.GetPtr(this));
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_owner");

        [GodotMethod("set_owner")]
        public void SetOwner(Node owner)
        {
            NativeCalls.godot_icall_1_42(method_bind_25, Object.GetPtr(this), Object.GetPtr(owner));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_owner");

        [GodotMethod("get_owner")]
        public Node GetOwner()
        {
            return NativeCalls.godot_icall_0_200(method_bind_26, Object.GetPtr(this));
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_and_skip");

        /// <summary>
        /// Removes a node and sets all its children as children of the parent node (if it exists). All event subscriptions that pass by the removed node will be unsubscribed.
        /// </summary>
        [GodotMethod("remove_and_skip")]
        public void RemoveAndSkip()
        {
            NativeCalls.godot_icall_0_12(method_bind_27, Object.GetPtr(this));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_index");

        /// <summary>
        /// Returns the node's index, i.e. its position among the siblings of its parent.
        /// </summary>
        [GodotMethod("get_index")]
        public int GetIndex()
        {
            return NativeCalls.godot_icall_0_1(method_bind_28, Object.GetPtr(this));
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "print_tree");

        /// <summary>
        /// Prints the scene to stdout. Used mainly for debugging purposes.
        /// </summary>
        [GodotMethod("print_tree")]
        public void PrintTree()
        {
            NativeCalls.godot_icall_0_12(method_bind_29, Object.GetPtr(this));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_filename");

        [GodotMethod("set_filename")]
        public void SetFilename(string filename)
        {
            NativeCalls.godot_icall_1_43(method_bind_30, Object.GetPtr(this), filename);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_filename");

        [GodotMethod("get_filename")]
        public string GetFilename()
        {
            return NativeCalls.godot_icall_0_2(method_bind_31, Object.GetPtr(this));
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "propagate_notification");

        /// <summary>
        /// Notifies the current node and all its children recursively by calling notification() on all of them.
        /// </summary>
        [GodotMethod("propagate_notification")]
        public void PropagateNotification(int what)
        {
            NativeCalls.godot_icall_1_0(method_bind_32, Object.GetPtr(this), what);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "propagate_call");

        /// <summary>
        /// Calls the given method (if present) with the arguments given in [code]args[/code] on this node and recursively on all its children. If the parent_first argument is [code]true[/code] then the method will be called on the current node first, then on all children. If it is [code]false[/code] then the children will be called first.
        /// &lt;param name="args"&gt;If the param is null, then the default value is new object[] {}&lt;/param&gt;
        /// </summary>
        [GodotMethod("propagate_call")]
        public void PropagateCall(string method, object[] args = null, bool parentFirst = false)
        {
            object[] args_in = args != null ? args : new object[] {};
            NativeCalls.godot_icall_3_549(method_bind_33, Object.GetPtr(this), method, args_in, parentFirst);
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_physics_process");

        /// <summary>
        /// Enables or disables physics (i.e. fixed framerate) processing. When a node is being processed, it will receive a NOTIFICATION_PHYSICS_PROCESS at a fixed (usually 60 fps, see [OS] to change) interval (and the [method _physics_process] callback will be called if exists). Enabled automatically if [method _physics_process] is overriden. Any calls to this before [method _ready] will be ignored.
        /// </summary>
        [GodotMethod("set_physics_process")]
        public void SetPhysicsProcess(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_34, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_physics_process_delta_time");

        /// <summary>
        /// Returns the time elapsed since the last physics-bound frame (see [method _physics_process]). This is always a constant value in physics processing unless the frames per second is changed in [OS].
        /// </summary>
        [GodotMethod("get_physics_process_delta_time")]
        public float GetPhysicsProcessDeltaTime()
        {
            return NativeCalls.godot_icall_0_9(method_bind_35, Object.GetPtr(this));
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_physics_processing");

        /// <summary>
        /// Returns [code]true[/code] if physics processing is enabled (see [method set_physics_process]).
        /// </summary>
        [GodotMethod("is_physics_processing")]
        public bool IsPhysicsProcessing()
        {
            return NativeCalls.godot_icall_0_3(method_bind_36, Object.GetPtr(this));
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_process_delta_time");

        /// <summary>
        /// Returns the time elapsed (in seconds) since the last process callback. This value may vary from frame to frame.
        /// </summary>
        [GodotMethod("get_process_delta_time")]
        public float GetProcessDeltaTime()
        {
            return NativeCalls.godot_icall_0_9(method_bind_37, Object.GetPtr(this));
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_process");

        /// <summary>
        /// Enables or disables processing. When a node is being processed, it will receive a NOTIFICATION_PROCESS on every drawn frame (and the [method _process] callback will be called if exists). Enabled automatically if [method _process] is overriden. Any calls to this before [method _ready] will be ignored.
        /// </summary>
        [GodotMethod("set_process")]
        public void SetProcess(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_38, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_processing");

        /// <summary>
        /// Returns [code]true[/code] if processing is enabled (see [method set_process]).
        /// </summary>
        [GodotMethod("is_processing")]
        public bool IsProcessing()
        {
            return NativeCalls.godot_icall_0_3(method_bind_39, Object.GetPtr(this));
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_process_input");

        /// <summary>
        /// Enables or disables input processing. This is not required for GUI controls! Enabled automatically if [method _input] is overriden. Any calls to this before [method _ready] will be ignored.
        /// </summary>
        [GodotMethod("set_process_input")]
        public void SetProcessInput(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_40, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_processing_input");

        /// <summary>
        /// Returns [code]true[/code] if the node is processing input (see [method set_process_input]).
        /// </summary>
        [GodotMethod("is_processing_input")]
        public bool IsProcessingInput()
        {
            return NativeCalls.godot_icall_0_3(method_bind_41, Object.GetPtr(this));
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_process_unhandled_input");

        /// <summary>
        /// Enables unhandled input processing. This is not required for GUI controls! It enables the node to receive all input that was not previously handled (usually by a [Control]). Enabled automatically if [method _unhandled_input] is overriden. Any calls to this before [method _ready] will be ignored.
        /// </summary>
        [GodotMethod("set_process_unhandled_input")]
        public void SetProcessUnhandledInput(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_42, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_processing_unhandled_input");

        /// <summary>
        /// Returns [code]true[/code] if the node is processing unhandled input (see [method set_process_unhandled_input]).
        /// </summary>
        [GodotMethod("is_processing_unhandled_input")]
        public bool IsProcessingUnhandledInput()
        {
            return NativeCalls.godot_icall_0_3(method_bind_43, Object.GetPtr(this));
        }

        private IntPtr method_bind_44 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_process_unhandled_key_input");

        /// <summary>
        /// Enables unhandled key input processing. Enabled automatically if [method _unhandled_key_input] is overriden. Any calls to this before [method _ready] will be ignored.
        /// </summary>
        [GodotMethod("set_process_unhandled_key_input")]
        public void SetProcessUnhandledKeyInput(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_44, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_45 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_processing_unhandled_key_input");

        /// <summary>
        /// Returns [code]true[/code] if the node is processing unhandled key input (see [method set_process_unhandled_key_input]).
        /// </summary>
        [GodotMethod("is_processing_unhandled_key_input")]
        public bool IsProcessingUnhandledKeyInput()
        {
            return NativeCalls.godot_icall_0_3(method_bind_45, Object.GetPtr(this));
        }

        private IntPtr method_bind_46 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_pause_mode");

        [GodotMethod("set_pause_mode")]
        public void SetPauseMode(Node.PauseModeEnum mode)
        {
            NativeCalls.godot_icall_1_550(method_bind_46, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_47 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_pause_mode");

        [GodotMethod("get_pause_mode")]
        public Node.PauseModeEnum GetPauseMode()
        {
            return NativeCalls.godot_icall_0_551(method_bind_47, Object.GetPtr(this));
        }

        private IntPtr method_bind_48 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "can_process");

        /// <summary>
        /// Returns [code]true[/code] if the node can process while the scene tree is paused (see [method set_pause_mode]). Always returns [code]true[/code] if the scene tree is not paused, and [code]false[/code] if the node is not in the tree. FIXME: Why FAIL_COND?
        /// </summary>
        [GodotMethod("can_process")]
        public bool CanProcess()
        {
            return NativeCalls.godot_icall_0_3(method_bind_48, Object.GetPtr(this));
        }

        private IntPtr method_bind_49 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "print_stray_nodes");

        /// <summary>
        /// </summary>
        [GodotMethod("print_stray_nodes")]
        public void PrintStrayNodes()
        {
            NativeCalls.godot_icall_0_12(method_bind_49, Object.GetPtr(this));
        }

        private IntPtr method_bind_50 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_position_in_parent");

        /// <summary>
        /// Returns the node's order in the scene tree branch. For example, if called on the first child node the position is [code]0[/code].
        /// </summary>
        [GodotMethod("get_position_in_parent")]
        public int GetPositionInParent()
        {
            return NativeCalls.godot_icall_0_1(method_bind_50, Object.GetPtr(this));
        }

        private IntPtr method_bind_51 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_display_folded");

        /// <summary>
        /// Sets the folded state of the node in the Scene dock.
        /// </summary>
        [GodotMethod("set_display_folded")]
        public void SetDisplayFolded(bool fold)
        {
            NativeCalls.godot_icall_1_11(method_bind_51, Object.GetPtr(this), fold);
        }

        private IntPtr method_bind_52 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_displayed_folded");

        /// <summary>
        /// Returns [code]true[/code] if the node is folded (collapsed) in the Scene dock.
        /// </summary>
        [GodotMethod("is_displayed_folded")]
        public bool IsDisplayedFolded()
        {
            return NativeCalls.godot_icall_0_3(method_bind_52, Object.GetPtr(this));
        }

        private IntPtr method_bind_53 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_process_internal");

        /// <summary>
        /// </summary>
        [GodotMethod("set_process_internal")]
        public void SetProcessInternal(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_53, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_54 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_processing_internal");

        /// <summary>
        /// </summary>
        [GodotMethod("is_processing_internal")]
        public bool IsProcessingInternal()
        {
            return NativeCalls.godot_icall_0_3(method_bind_54, Object.GetPtr(this));
        }

        private IntPtr method_bind_55 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_physics_process_internal");

        /// <summary>
        /// </summary>
        [GodotMethod("set_physics_process_internal")]
        public void SetPhysicsProcessInternal(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_55, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_56 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_physics_processing_internal");

        /// <summary>
        /// </summary>
        [GodotMethod("is_physics_processing_internal")]
        public bool IsPhysicsProcessingInternal()
        {
            return NativeCalls.godot_icall_0_3(method_bind_56, Object.GetPtr(this));
        }

        private IntPtr method_bind_57 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tree");

        /// <summary>
        /// Returns the [SceneTree] that contains this node.
        /// </summary>
        [GodotMethod("get_tree")]
        public SceneTree GetTree()
        {
            return NativeCalls.godot_icall_0_552(method_bind_57, Object.GetPtr(this));
        }

        private IntPtr method_bind_58 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "duplicate");

        /// <summary>
        /// Duplicates the node, returning a new node.
        /// You can fine-tune the behavior using the [code]flags[/code]. See DUPLICATE_* constants.
        /// </summary>
        [GodotMethod("duplicate")]
        public Node Duplicate(int flags = 15)
        {
            return NativeCalls.godot_icall_1_542(method_bind_58, Object.GetPtr(this), flags);
        }

        private IntPtr method_bind_59 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "replace_by");

        /// <summary>
        /// Replaces a node in a scene by the given one. Subscriptions that pass through this node will be lost.
        /// </summary>
        [GodotMethod("replace_by")]
        public void ReplaceBy(Node node, bool keepData = false)
        {
            NativeCalls.godot_icall_2_374(method_bind_59, Object.GetPtr(this), Object.GetPtr(node), keepData);
        }

        private IntPtr method_bind_60 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_scene_instance_load_placeholder");

        /// <summary>
        /// </summary>
        [GodotMethod("set_scene_instance_load_placeholder")]
        public void SetSceneInstanceLoadPlaceholder(bool loadPlaceholder)
        {
            NativeCalls.godot_icall_1_11(method_bind_60, Object.GetPtr(this), loadPlaceholder);
        }

        private IntPtr method_bind_61 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_scene_instance_load_placeholder");

        /// <summary>
        /// </summary>
        [GodotMethod("get_scene_instance_load_placeholder")]
        public bool GetSceneInstanceLoadPlaceholder()
        {
            return NativeCalls.godot_icall_0_3(method_bind_61, Object.GetPtr(this));
        }

        private IntPtr method_bind_62 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_viewport");

        /// <summary>
        /// Returns the node's [Viewport].
        /// </summary>
        [GodotMethod("get_viewport")]
        public Viewport GetViewport()
        {
            return NativeCalls.godot_icall_0_553(method_bind_62, Object.GetPtr(this));
        }

        private IntPtr method_bind_63 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "queue_free");

        /// <summary>
        /// Queues a node for deletion at the end of the current frame. When deleted, all of its child nodes will be deleted as well. This method ensures it's safe to delete the node, contrary to [method Object.free]. Use [method Object.is_queued_for_deletion] to check whether a node will be deleted at the end of the frame.
        /// </summary>
        [GodotMethod("queue_free")]
        public void QueueFree()
        {
            NativeCalls.godot_icall_0_12(method_bind_63, Object.GetPtr(this));
        }

        private IntPtr method_bind_64 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "request_ready");

        /// <summary>
        /// Requests that [code]_ready[/code] be called again.
        /// </summary>
        [GodotMethod("request_ready")]
        public void RequestReady()
        {
            NativeCalls.godot_icall_0_12(method_bind_64, Object.GetPtr(this));
        }

        private IntPtr method_bind_65 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_network_master");

        /// <summary>
        /// Sets the node network master to the peer with the given peer ID. The network master is the peer that has authority over it on the network. Inherited from the parent node by default, which ultimately defaults to peer ID 1 (the server).
        /// </summary>
        [GodotMethod("set_network_master")]
        public void SetNetworkMaster(int id, bool recursive = true)
        {
            NativeCalls.godot_icall_2_51(method_bind_65, Object.GetPtr(this), id, recursive);
        }

        private IntPtr method_bind_66 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_network_master");

        /// <summary>
        /// Returns the peer ID of the network master for this node.
        /// </summary>
        [GodotMethod("get_network_master")]
        public int GetNetworkMaster()
        {
            return NativeCalls.godot_icall_0_1(method_bind_66, Object.GetPtr(this));
        }

        private IntPtr method_bind_67 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_network_master");

        /// <summary>
        /// </summary>
        [GodotMethod("is_network_master")]
        public bool IsNetworkMaster()
        {
            return NativeCalls.godot_icall_0_3(method_bind_67, Object.GetPtr(this));
        }

        private IntPtr method_bind_68 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rpc_config");

        /// <summary>
        /// Changes the method's RPC mode (one of RPC_MODE_* constants).
        /// </summary>
        [GodotMethod("rpc_config")]
        public void RpcConfig(string method, Node.RPCMode mode)
        {
            NativeCalls.godot_icall_2_554(method_bind_68, Object.GetPtr(this), method, mode);
        }

        private IntPtr method_bind_69 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rset_config");

        /// <summary>
        /// Changes the property's RPC mode (one of RPC_MODE_* constants).
        /// </summary>
        [GodotMethod("rset_config")]
        public void RsetConfig(string property, Node.RPCMode mode)
        {
            NativeCalls.godot_icall_2_554(method_bind_69, Object.GetPtr(this), property, mode);
        }

        private IntPtr method_bind_70 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_import_path");

        internal void _SetImportPath(NodePath importPath)
        {
            NativeCalls.godot_icall_1_78(method_bind_70, Object.GetPtr(this), NodePath.GetPtr(importPath));
        }

        private IntPtr method_bind_71 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_import_path");

        internal NodePath _GetImportPath()
        {
            IntPtr ret = NativeCalls.godot_icall_0_79(method_bind_71, Object.GetPtr(this));
            return new NodePath(ret);
        }

        private IntPtr method_bind_72 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rpc");

        /// <summary>
        /// Sends a remote procedure call request to all peers on the network (and locally), optionally sending additional data as arguments. Call request will be received by nodes with the same [NodePath].
        /// </summary>
        [GodotMethod("rpc")]
        public object Rpc(string method, params object[] @args)
        {
            return NativeCalls.godot_icall_2_555(method_bind_72, Object.GetPtr(this), method, @args);
        }

        private IntPtr method_bind_73 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rpc_unreliable");

        /// <summary>
        /// Sends a [method rpc] using an unreliable protocol.
        /// </summary>
        [GodotMethod("rpc_unreliable")]
        public object RpcUnreliable(string method, params object[] @args)
        {
            return NativeCalls.godot_icall_2_555(method_bind_73, Object.GetPtr(this), method, @args);
        }

        private IntPtr method_bind_74 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rpc_id");

        /// <summary>
        /// Sends a [method rpc] to a specific peer identified by [i]peer_id[/i].
        /// </summary>
        [GodotMethod("rpc_id")]
        public object RpcId(int peerId, string method, params object[] @args)
        {
            return NativeCalls.godot_icall_3_556(method_bind_74, Object.GetPtr(this), peerId, method, @args);
        }

        private IntPtr method_bind_75 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rpc_unreliable_id");

        /// <summary>
        /// Sends a [method rpc] to a specific peer identified by [i]peer_id[/i] using an unreliable protocol.
        /// </summary>
        [GodotMethod("rpc_unreliable_id")]
        public object RpcUnreliableId(int peerId, string method, params object[] @args)
        {
            return NativeCalls.godot_icall_3_556(method_bind_75, Object.GetPtr(this), peerId, method, @args);
        }

        private IntPtr method_bind_76 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rset");

        /// <summary>
        /// Remotely changes property's value on other peers (and locally).
        /// </summary>
        [GodotMethod("rset")]
        public void Rset(string property, object value)
        {
            NativeCalls.godot_icall_2_337(method_bind_76, Object.GetPtr(this), property, value);
        }

        private IntPtr method_bind_77 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rset_id");

        /// <summary>
        /// Remotely changes property's value on a specific peer identified by [i]peer_id[/i].
        /// </summary>
        [GodotMethod("rset_id")]
        public void RsetId(int peerId, string property, object value)
        {
            NativeCalls.godot_icall_3_557(method_bind_77, Object.GetPtr(this), peerId, property, value);
        }

        private IntPtr method_bind_78 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rset_unreliable");

        /// <summary>
        /// Remotely changes property's value on other peers (and locally) using an unreliable protocol.
        /// </summary>
        [GodotMethod("rset_unreliable")]
        public void RsetUnreliable(string property, object value)
        {
            NativeCalls.godot_icall_2_337(method_bind_78, Object.GetPtr(this), property, value);
        }

        private IntPtr method_bind_79 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rset_unreliable_id");

        /// <summary>
        /// Remotely changes property's value on a specific peer identified by [i]peer_id[/i] using an unreliable protocol.
        /// </summary>
        [GodotMethod("rset_unreliable_id")]
        public void RsetUnreliableId(int peerId, string property, object value)
        {
            NativeCalls.godot_icall_3_557(method_bind_79, Object.GetPtr(this), peerId, property, value);
        }
    }
}
