using System;

namespace Godot
{
    /// <summary>
    /// A 2D game object, with a position, rotation and scale. All 2D physics nodes and sprites inherit from Node2D. Use Node2D as a parent node to move, scale and rotate children in a 2D project. Also gives control on the node's render order.
    /// </summary>
    public class Node2D : CanvasItem
    {
        /// <summary>
        /// Position, relative to the node's parent.
        /// </summary>
        public Vector2 Position
        {
            get
            {
                return GetPosition();
            }
            set
            {
                SetPosition(value);
            }
        }

        /// <summary>
        /// Rotation in radians, relative to the node's parent.
        /// </summary>
        public float Rotation
        {
            get
            {
                return GetRotation();
            }
            set
            {
                SetRotation(value);
            }
        }

        /// <summary>
        /// Rotation in degrees, relative to the node's parent.
        /// </summary>
        public float RotationDegrees
        {
            get
            {
                return GetRotationDegrees();
            }
            set
            {
                SetRotationDegrees(value);
            }
        }

        /// <summary>
        /// The node's scale. Unscaled value: [code](1, 1)[/code]
        /// </summary>
        public Vector2 Scale
        {
            get
            {
                return GetScale();
            }
            set
            {
                SetScale(value);
            }
        }

        /// <summary>
        /// Local [Transform2D].
        /// </summary>
        public Transform2D Transform
        {
            get
            {
                return GetTransform();
            }
            set
            {
                SetTransform(value);
            }
        }

        /// <summary>
        /// Global position.
        /// </summary>
        public Vector2 GlobalPosition
        {
            get
            {
                return GetGlobalPosition();
            }
            set
            {
                SetGlobalPosition(value);
            }
        }

        /// <summary>
        /// Global rotation in radians.
        /// </summary>
        public float GlobalRotation
        {
            get
            {
                return GetGlobalRotation();
            }
            set
            {
                SetGlobalRotation(value);
            }
        }

        /// <summary>
        /// Global rotation in degrees.
        /// </summary>
        public float GlobalRotationDegrees
        {
            get
            {
                return GetGlobalRotationDegrees();
            }
            set
            {
                SetGlobalRotationDegrees(value);
            }
        }

        /// <summary>
        /// Global scale.
        /// </summary>
        public Vector2 GlobalScale
        {
            get
            {
                return GetGlobalScale();
            }
            set
            {
                SetGlobalScale(value);
            }
        }

        /// <summary>
        /// Global [Transform2D].
        /// </summary>
        public Transform2D GlobalTransform
        {
            get
            {
                return GetGlobalTransform();
            }
            set
            {
                SetGlobalTransform(value);
            }
        }

        /// <summary>
        /// Z-index. Controls the order in which the nodes render. A node with a higher Z-index will display in front of others.
        /// </summary>
        public int ZIndex
        {
            get
            {
                return GetZIndex();
            }
            set
            {
                SetZIndex(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the node's Z-index is relative to its parent's Z-index. If this node's Z-index is 2 and its parent's effective Z-index is 3, then this node's effective Z-index will be 2 + 3 = 5.
        /// </summary>
        public bool ZAsRelative
        {
            get
            {
                return IsZRelative();
            }
            set
            {
                SetZAsRelative(value);
            }
        }


        private const string nativeName = "Node2D";

        public Node2D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Node2D_Ctor(this);
        }

        internal Node2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_position");

        [GodotMethod("set_position")]
        public void SetPosition(Vector2 position)
        {
            NativeCalls.godot_icall_1_45(method_bind_0, Object.GetPtr(this), ref position);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rotation");

        [GodotMethod("set_rotation")]
        public void SetRotation(float radians)
        {
            NativeCalls.godot_icall_1_10(method_bind_1, Object.GetPtr(this), radians);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rotation_degrees");

        [GodotMethod("set_rotation_degrees")]
        public void SetRotationDegrees(float degrees)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), degrees);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_scale");

        [GodotMethod("set_scale")]
        public void SetScale(Vector2 scale)
        {
            NativeCalls.godot_icall_1_45(method_bind_3, Object.GetPtr(this), ref scale);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_position");

        [GodotMethod("get_position")]
        public Vector2 GetPosition()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_4, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rotation");

        [GodotMethod("get_rotation")]
        public float GetRotation()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rotation_degrees");

        [GodotMethod("get_rotation_degrees")]
        public float GetRotationDegrees()
        {
            return NativeCalls.godot_icall_0_9(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_scale");

        [GodotMethod("get_scale")]
        public Vector2 GetScale()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_7, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rotate");

        /// <summary>
        /// Applies a rotation to the node, in radians, starting from its current rotation.
        /// </summary>
        [GodotMethod("rotate")]
        public void Rotate(float radians)
        {
            NativeCalls.godot_icall_1_10(method_bind_8, Object.GetPtr(this), radians);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "move_local_x");

        /// <summary>
        /// Applies a local translation on the node's X axis based on the [method Node._process]'s [code]delta[/code]. If [code]scaled[/code] is false, normalizes the movement.
        /// </summary>
        [GodotMethod("move_local_x")]
        public void MoveLocalX(float delta, bool scaled = false)
        {
            NativeCalls.godot_icall_2_83(method_bind_9, Object.GetPtr(this), delta, scaled);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "move_local_y");

        /// <summary>
        /// Applies a local translation on the node's Y axis based on the [method Node._process]'s [code]delta[/code]. If [code]scaled[/code] is false, normalizes the movement.
        /// </summary>
        [GodotMethod("move_local_y")]
        public void MoveLocalY(float delta, bool scaled = false)
        {
            NativeCalls.godot_icall_2_83(method_bind_10, Object.GetPtr(this), delta, scaled);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "translate");

        /// <summary>
        /// Translates the node by the given [code]offset[/code] in local coordinates.
        /// </summary>
        [GodotMethod("translate")]
        public void Translate(Vector2 offset)
        {
            NativeCalls.godot_icall_1_45(method_bind_11, Object.GetPtr(this), ref offset);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "global_translate");

        /// <summary>
        /// Adds the 'offset' vector to the node's global position.
        /// </summary>
        [GodotMethod("global_translate")]
        public void GlobalTranslate(Vector2 offset)
        {
            NativeCalls.godot_icall_1_45(method_bind_12, Object.GetPtr(this), ref offset);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "apply_scale");

        /// <summary>
        /// Multiplies the current scale by the 'ratio' vector.
        /// </summary>
        [GodotMethod("apply_scale")]
        public void ApplyScale(Vector2 ratio)
        {
            NativeCalls.godot_icall_1_45(method_bind_13, Object.GetPtr(this), ref ratio);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_global_position");

        [GodotMethod("set_global_position")]
        public void SetGlobalPosition(Vector2 position)
        {
            NativeCalls.godot_icall_1_45(method_bind_14, Object.GetPtr(this), ref position);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_global_position");

        [GodotMethod("get_global_position")]
        public Vector2 GetGlobalPosition()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_15, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_global_rotation");

        [GodotMethod("set_global_rotation")]
        public void SetGlobalRotation(float radians)
        {
            NativeCalls.godot_icall_1_10(method_bind_16, Object.GetPtr(this), radians);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_global_rotation");

        [GodotMethod("get_global_rotation")]
        public float GetGlobalRotation()
        {
            return NativeCalls.godot_icall_0_9(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_global_rotation_degrees");

        [GodotMethod("set_global_rotation_degrees")]
        public void SetGlobalRotationDegrees(float degrees)
        {
            NativeCalls.godot_icall_1_10(method_bind_18, Object.GetPtr(this), degrees);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_global_rotation_degrees");

        [GodotMethod("get_global_rotation_degrees")]
        public float GetGlobalRotationDegrees()
        {
            return NativeCalls.godot_icall_0_9(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_global_scale");

        [GodotMethod("set_global_scale")]
        public void SetGlobalScale(Vector2 scale)
        {
            NativeCalls.godot_icall_1_45(method_bind_20, Object.GetPtr(this), ref scale);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_global_scale");

        [GodotMethod("get_global_scale")]
        public Vector2 GetGlobalScale()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_21, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_transform");

        [GodotMethod("set_transform")]
        public void SetTransform(Transform2D xform)
        {
            NativeCalls.godot_icall_1_219(method_bind_22, Object.GetPtr(this), ref xform);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_global_transform");

        [GodotMethod("set_global_transform")]
        public void SetGlobalTransform(Transform2D xform)
        {
            NativeCalls.godot_icall_1_219(method_bind_23, Object.GetPtr(this), ref xform);
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "look_at");

        /// <summary>
        /// Rotates the node so it points towards the 'point'.
        /// </summary>
        [GodotMethod("look_at")]
        public void LookAt(Vector2 point)
        {
            NativeCalls.godot_icall_1_45(method_bind_24, Object.GetPtr(this), ref point);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_angle_to");

        /// <summary>
        /// Returns the angle between the node and the 'point' in radians.
        /// </summary>
        [GodotMethod("get_angle_to")]
        public float GetAngleTo(Vector2 point)
        {
            return NativeCalls.godot_icall_1_558(method_bind_25, Object.GetPtr(this), ref point);
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "to_local");

        /// <summary>
        /// Converts a global point's coordinates to local coordinates.
        /// </summary>
        [GodotMethod("to_local")]
        public Vector2 ToLocal(Vector2 globalPoint)
        {
            object ret = NativeCalls.godot_icall_1_223(method_bind_26, Object.GetPtr(this), ref globalPoint);
            return (Vector2)ret;
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "to_global");

        /// <summary>
        /// Converts a local point's coordinates to global coordinates.
        /// </summary>
        [GodotMethod("to_global")]
        public Vector2 ToGlobal(Vector2 localPoint)
        {
            object ret = NativeCalls.godot_icall_1_223(method_bind_27, Object.GetPtr(this), ref localPoint);
            return (Vector2)ret;
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_z_index");

        [GodotMethod("set_z_index")]
        public void SetZIndex(int zIndex)
        {
            NativeCalls.godot_icall_1_0(method_bind_28, Object.GetPtr(this), zIndex);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_z_index");

        [GodotMethod("get_z_index")]
        public int GetZIndex()
        {
            return NativeCalls.godot_icall_0_1(method_bind_29, Object.GetPtr(this));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_z_as_relative");

        [GodotMethod("set_z_as_relative")]
        public void SetZAsRelative(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_30, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_z_relative");

        [GodotMethod("is_z_relative")]
        public bool IsZRelative()
        {
            return NativeCalls.godot_icall_0_3(method_bind_31, Object.GetPtr(this));
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_relative_transform_to_parent");

        /// <summary>
        /// Returns the [Transform2D] relative to this node's parent.
        /// </summary>
        [GodotMethod("get_relative_transform_to_parent")]
        public Transform2D GetRelativeTransformToParent(Node parent)
        {
            object ret = NativeCalls.godot_icall_1_559(method_bind_32, Object.GetPtr(this), Object.GetPtr(parent));
            return (Transform2D)ret;
        }
    }
}
