using System;
using System.Collections.Generic;

namespace Godot
{
    /// <summary>
    /// Operating System functions. OS Wraps the most common functionality to communicate with the host Operating System, such as: mouse grabbing, mouse cursors, clipboard, video mode, date and time, timers, environment variables, execution of binaries, command line, etc.
    /// </summary>
    public static class OS
    {
        public enum SystemDir
        {
            Desktop = 0,
            Dcim = 1,
            Documents = 2,
            Downloads = 3,
            Movies = 4,
            Music = 5,
            Pictures = 6,
            Ringtones = 7
        }

        public enum ScreenOrientationEnum
        {
            Landscape = 0,
            Portrait = 1,
            ReverseLandscape = 2,
            ReversePortrait = 3,
            SensorLandscape = 4,
            SensorPortrait = 5,
            Sensor = 6
        }

        public enum PowerState
        {
            Unknown = 0,
            OnBattery = 1,
            NoBattery = 2,
            Charging = 3,
            Charged = 4
        }

        public enum Month
        {
            January = 1,
            February = 2,
            March = 3,
            April = 4,
            May = 5,
            June = 6,
            July = 7,
            August = 8,
            September = 9,
            October = 10,
            November = 11,
            December = 12
        }

        public enum Weekday
        {
            Sunday = 0,
            Monday = 1,
            Tuesday = 2,
            Wednesday = 3,
            Thursday = 4,
            Friday = 5,
            Saturday = 6
        }


        /// <summary>
        /// The clipboard from the host OS. Might be unavailable on some platforms.
        /// </summary>
        public static string Clipboard
        {
            get
            {
                return GetClipboard();
            }
            set
            {
                SetClipboard(value);
            }
        }

        /// <summary>
        /// The current screen index (starting from 0).
        /// </summary>
        public static int CurrentScreen
        {
            get
            {
                return GetCurrentScreen();
            }
            set
            {
                SetCurrentScreen(value);
            }
        }

        /// <summary>
        /// </summary>
        public static int ExitCode
        {
            get
            {
                return GetExitCode();
            }
            set
            {
                SetExitCode(value);
            }
        }

        /// <summary>
        /// </summary>
        public static bool VsyncEnabled
        {
            get
            {
                return IsVsyncEnabled();
            }
            set
            {
                SetUseVsync(value);
            }
        }

        /// <summary>
        /// </summary>
        public static bool LowProcessorUsageMode
        {
            get
            {
                return IsInLowProcessorUsageMode();
            }
            set
            {
                SetLowProcessorUsageMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public static bool KeepScreenOn
        {
            get
            {
                return IsKeepScreenOn();
            }
            set
            {
                SetKeepScreenOn(value);
            }
        }

        /// <summary>
        /// The current screen orientation.
        /// </summary>
        public static OS.ScreenOrientationEnum ScreenOrientation
        {
            get
            {
                return GetScreenOrientation();
            }
            set
            {
                SetScreenOrientation(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], removes the window frame.
        /// </summary>
        public static bool WindowBorderless
        {
            get
            {
                return GetBorderlessWindow();
            }
            set
            {
                SetBorderlessWindow(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], the window is fullscreen.
        /// </summary>
        public static bool WindowFullscreen
        {
            get
            {
                return IsWindowFullscreen();
            }
            set
            {
                SetWindowFullscreen(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], the window is maximized.
        /// </summary>
        public static bool WindowMaximized
        {
            get
            {
                return IsWindowMaximized();
            }
            set
            {
                SetWindowMaximized(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], the window is minimized.
        /// </summary>
        public static bool WindowMinimized
        {
            get
            {
                return IsWindowMinimized();
            }
            set
            {
                SetWindowMinimized(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], the window is resizable by the user.
        /// </summary>
        public static bool WindowResizable
        {
            get
            {
                return IsWindowResizable();
            }
            set
            {
                SetWindowResizable(value);
            }
        }

        /// <summary>
        /// The window position relative to the screen, the origin is the top left corner, +Y axis goes to the bottom and +X axis goes to the right.
        /// </summary>
        public static Vector2 WindowPosition
        {
            get
            {
                return GetWindowPosition();
            }
            set
            {
                SetWindowPosition(value);
            }
        }

        /// <summary>
        /// The size of the window (without counting window manager decorations).
        /// </summary>
        public static Vector2 WindowSize
        {
            get
            {
                return GetWindowSize();
            }
            set
            {
                SetWindowSize(value);
            }
        }


        private const string nativeName = "_OS";
        internal static IntPtr ptr = NativeCalls.godot_icall__OS_get_singleton();

        private static IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_clipboard");

        [GodotMethod("set_clipboard")]
        public static void SetClipboard(string clipboard)
        {
            NativeCalls.godot_icall_1_43(method_bind_0, ptr, clipboard);
        }

        private static IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_clipboard");

        [GodotMethod("get_clipboard")]
        public static string GetClipboard()
        {
            return NativeCalls.godot_icall_0_2(method_bind_1, ptr);
        }

        private static IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_screen_count");

        /// <summary>
        /// Returns the number of displays attached to the host machine.
        /// </summary>
        [GodotMethod("get_screen_count")]
        public static int GetScreenCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_2, ptr);
        }

        private static IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_current_screen");

        [GodotMethod("get_current_screen")]
        public static int GetCurrentScreen()
        {
            return NativeCalls.godot_icall_0_1(method_bind_3, ptr);
        }

        private static IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_current_screen");

        [GodotMethod("set_current_screen")]
        public static void SetCurrentScreen(int screen)
        {
            NativeCalls.godot_icall_1_0(method_bind_4, ptr, screen);
        }

        private static IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_screen_position");

        /// <summary>
        /// Returns the position of the specified screen by index. If no screen index is provided, the current screen will be used.
        /// </summary>
        [GodotMethod("get_screen_position")]
        public static Vector2 GetScreenPosition(int screen = -1)
        {
            object ret = NativeCalls.godot_icall_1_282(method_bind_5, ptr, screen);
            return (Vector2)ret;
        }

        private static IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_screen_size");

        /// <summary>
        /// Returns the dimensions in pixels of the specified screen.
        /// </summary>
        [GodotMethod("get_screen_size")]
        public static Vector2 GetScreenSize(int screen = -1)
        {
            object ret = NativeCalls.godot_icall_1_282(method_bind_6, ptr, screen);
            return (Vector2)ret;
        }

        private static IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_screen_dpi");

        /// <summary>
        /// Returns the dots per inch density of the specified screen.
        /// On Android Devices, the actual screen densities are grouped into six generalized densities:
        /// ldpi    - 120 dpi
        /// mdpi    - 160 dpi
        /// hdpi    - 240 dpi
        /// xhdpi   - 320 dpi
        /// xxhdpi  - 480 dpi
        /// xxxhdpi - 640 dpi
        /// </summary>
        [GodotMethod("get_screen_dpi")]
        public static int GetScreenDpi(int screen = -1)
        {
            return NativeCalls.godot_icall_1_6(method_bind_7, ptr, screen);
        }

        private static IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_window_position");

        [GodotMethod("get_window_position")]
        public static Vector2 GetWindowPosition()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_8, ptr);
            return (Vector2)ret;
        }

        private static IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_window_position");

        [GodotMethod("set_window_position")]
        public static void SetWindowPosition(Vector2 position)
        {
            NativeCalls.godot_icall_1_45(method_bind_9, ptr, ref position);
        }

        private static IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_window_size");

        [GodotMethod("get_window_size")]
        public static Vector2 GetWindowSize()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_10, ptr);
            return (Vector2)ret;
        }

        private static IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_window_size");

        [GodotMethod("set_window_size")]
        public static void SetWindowSize(Vector2 size)
        {
            NativeCalls.godot_icall_1_45(method_bind_11, ptr, ref size);
        }

        private static IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_window_fullscreen");

        [GodotMethod("set_window_fullscreen")]
        public static void SetWindowFullscreen(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_12, ptr, enabled);
        }

        private static IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_window_fullscreen");

        [GodotMethod("is_window_fullscreen")]
        public static bool IsWindowFullscreen()
        {
            return NativeCalls.godot_icall_0_3(method_bind_13, ptr);
        }

        private static IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_window_resizable");

        [GodotMethod("set_window_resizable")]
        public static void SetWindowResizable(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_14, ptr, enabled);
        }

        private static IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_window_resizable");

        [GodotMethod("is_window_resizable")]
        public static bool IsWindowResizable()
        {
            return NativeCalls.godot_icall_0_3(method_bind_15, ptr);
        }

        private static IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_window_minimized");

        [GodotMethod("set_window_minimized")]
        public static void SetWindowMinimized(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_16, ptr, enabled);
        }

        private static IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_window_minimized");

        [GodotMethod("is_window_minimized")]
        public static bool IsWindowMinimized()
        {
            return NativeCalls.godot_icall_0_3(method_bind_17, ptr);
        }

        private static IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_window_maximized");

        [GodotMethod("set_window_maximized")]
        public static void SetWindowMaximized(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_18, ptr, enabled);
        }

        private static IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_window_maximized");

        [GodotMethod("is_window_maximized")]
        public static bool IsWindowMaximized()
        {
            return NativeCalls.godot_icall_0_3(method_bind_19, ptr);
        }

        private static IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "request_attention");

        /// <summary>
        /// Request the user attention to the window. It'll flash the taskbar button on Windows or bounce the dock icon on OSX.
        /// </summary>
        [GodotMethod("request_attention")]
        public static void RequestAttention()
        {
            NativeCalls.godot_icall_0_12(method_bind_20, ptr);
        }

        private static IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_borderless_window");

        [GodotMethod("set_borderless_window")]
        public static void SetBorderlessWindow(bool borderless)
        {
            NativeCalls.godot_icall_1_11(method_bind_21, ptr, borderless);
        }

        private static IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_borderless_window");

        [GodotMethod("get_borderless_window")]
        public static bool GetBorderlessWindow()
        {
            return NativeCalls.godot_icall_0_3(method_bind_22, ptr);
        }

        private static IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ime_position");

        /// <summary>
        /// </summary>
        [GodotMethod("set_ime_position")]
        public static void SetImePosition(Vector2 position)
        {
            NativeCalls.godot_icall_1_45(method_bind_23, ptr, ref position);
        }

        private static IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_screen_orientation");

        [GodotMethod("set_screen_orientation")]
        public static void SetScreenOrientation(OS.ScreenOrientationEnum orientation)
        {
            NativeCalls.godot_icall_1_1064(method_bind_24, ptr, orientation);
        }

        private static IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_screen_orientation");

        [GodotMethod("get_screen_orientation")]
        public static OS.ScreenOrientationEnum GetScreenOrientation()
        {
            return NativeCalls.godot_icall_0_1065(method_bind_25, ptr);
        }

        private static IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_keep_screen_on");

        [GodotMethod("set_keep_screen_on")]
        public static void SetKeepScreenOn(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_26, ptr, enabled);
        }

        private static IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_keep_screen_on");

        [GodotMethod("is_keep_screen_on")]
        public static bool IsKeepScreenOn()
        {
            return NativeCalls.godot_icall_0_3(method_bind_27, ptr);
        }

        private static IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_touchscreen_ui_hint");

        /// <summary>
        /// Returns [code]true[/code] if the device has a touchscreen or emulates one.
        /// </summary>
        [GodotMethod("has_touchscreen_ui_hint")]
        public static bool HasTouchscreenUiHint()
        {
            return NativeCalls.godot_icall_0_3(method_bind_28, ptr);
        }

        private static IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_window_title");

        /// <summary>
        /// Sets the window title to the specified string.
        /// </summary>
        [GodotMethod("set_window_title")]
        public static void SetWindowTitle(string title)
        {
            NativeCalls.godot_icall_1_43(method_bind_29, ptr, title);
        }

        private static IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_low_processor_usage_mode");

        [GodotMethod("set_low_processor_usage_mode")]
        public static void SetLowProcessorUsageMode(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_30, ptr, enable);
        }

        private static IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_in_low_processor_usage_mode");

        [GodotMethod("is_in_low_processor_usage_mode")]
        public static bool IsInLowProcessorUsageMode()
        {
            return NativeCalls.godot_icall_0_3(method_bind_31, ptr);
        }

        private static IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_processor_count");

        /// <summary>
        /// Returns the number of cores available in the host machine.
        /// </summary>
        [GodotMethod("get_processor_count")]
        public static int GetProcessorCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_32, ptr);
        }

        private static IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_executable_path");

        /// <summary>
        /// Returns the path to the current engine executable.
        /// </summary>
        [GodotMethod("get_executable_path")]
        public static string GetExecutablePath()
        {
            return NativeCalls.godot_icall_0_2(method_bind_33, ptr);
        }

        private static IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "execute");

        /// <summary>
        /// Execute the binary file in given path, optionally blocking until it returns. A process ID is returned.
        /// &lt;param name="output"&gt;If the param is null, then the default value is new object[] {}&lt;/param&gt;
        /// </summary>
        [GodotMethod("execute")]
        public static int Execute(string path, string[] arguments, bool blocking, object[] output = null)
        {
            object[] output_in = output != null ? output : new object[] {};
            return NativeCalls.godot_icall_4_1066(method_bind_34, ptr, path, arguments, blocking, output_in);
        }

        private static IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "kill");

        /// <summary>
        /// Kill a process ID (this method can be used to kill processes that were not spawned by the game).
        /// </summary>
        [GodotMethod("kill")]
        public static Error Kill(int pid)
        {
            return NativeCalls.godot_icall_1_1028(method_bind_35, ptr, pid);
        }

        private static IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "shell_open");

        /// <summary>
        /// Requests the OS to open a resource with the most appropriate program. For example.
        /// [code]OS.shell_open("C:\\Users\name\Downloads")[/code] on Windows opens the file explorer at the downloads folders of the user.
        /// [code]OS.shell_open("http://godotengine.org")[/code] opens the default web browser on the official Godot website.
        /// </summary>
        [GodotMethod("shell_open")]
        public static Error ShellOpen(string uri)
        {
            return NativeCalls.godot_icall_1_170(method_bind_36, ptr, uri);
        }

        private static IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_process_id");

        /// <summary>
        /// Returns the game process ID
        /// </summary>
        [GodotMethod("get_process_id")]
        public static int GetProcessId()
        {
            return NativeCalls.godot_icall_0_1(method_bind_37, ptr);
        }

        private static IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_environment");

        /// <summary>
        /// Returns an environment variable.
        /// </summary>
        [GodotMethod("get_environment")]
        public static string GetEnvironment(string environment)
        {
            return NativeCalls.godot_icall_1_73(method_bind_38, ptr, environment);
        }

        private static IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_environment");

        /// <summary>
        /// Returns [code]true[/code] if an environment variable exists.
        /// </summary>
        [GodotMethod("has_environment")]
        public static bool HasEnvironment(string environment)
        {
            return NativeCalls.godot_icall_1_70(method_bind_39, ptr, environment);
        }

        private static IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_name");

        /// <summary>
        /// Returns the name of the host OS. Possible values are: "Android", "Haiku", "iOS", "HTML5", "OSX", "Server", "Windows", "UWP", "X11".
        /// </summary>
        [GodotMethod("get_name")]
        public static string GetName()
        {
            return NativeCalls.godot_icall_0_2(method_bind_40, ptr);
        }

        private static IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cmdline_args");

        /// <summary>
        /// Returns the command line arguments passed to the engine.
        /// </summary>
        [GodotMethod("get_cmdline_args")]
        public static string[] GetCmdlineArgs()
        {
            return NativeCalls.godot_icall_0_72(method_bind_41, ptr);
        }

        private static IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_datetime");

        /// <summary>
        /// Returns current datetime as a dictionary of keys: year, month, day, weekday, dst (daylight savings time), hour, minute, second.
        /// </summary>
        [GodotMethod("get_datetime")]
        public static Dictionary<object, object> GetDatetime(bool utc = false)
        {
            return NativeCalls.godot_icall_1_448(method_bind_42, ptr, utc);
        }

        private static IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_date");

        /// <summary>
        /// Returns current date as a dictionary of keys: year, month, day, weekday, dst (daylight savings time).
        /// </summary>
        [GodotMethod("get_date")]
        public static Dictionary<object, object> GetDate(bool utc = false)
        {
            return NativeCalls.godot_icall_1_448(method_bind_43, ptr, utc);
        }

        private static IntPtr method_bind_44 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_time");

        /// <summary>
        /// Returns current time as a dictionary of keys: hour, minute, second.
        /// </summary>
        [GodotMethod("get_time")]
        public static Dictionary<object, object> GetTime(bool utc = false)
        {
            return NativeCalls.godot_icall_1_448(method_bind_44, ptr, utc);
        }

        private static IntPtr method_bind_45 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_time_zone_info");

        /// <summary>
        /// Returns the current time zone as a dictionary with the keys: bias and name.
        /// </summary>
        [GodotMethod("get_time_zone_info")]
        public static Dictionary<object, object> GetTimeZoneInfo()
        {
            return NativeCalls.godot_icall_0_169(method_bind_45, ptr);
        }

        private static IntPtr method_bind_46 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_unix_time");

        /// <summary>
        /// Return	the current unix timestamp.
        /// </summary>
        [GodotMethod("get_unix_time")]
        public static int GetUnixTime()
        {
            return NativeCalls.godot_icall_0_1(method_bind_46, ptr);
        }

        private static IntPtr method_bind_47 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_datetime_from_unix_time");

        /// <summary>
        /// Get a dictionary of time values when given epoch time.
        /// Dictionary Time values will be a union of values from [method get_time] and [method get_date] dictionaries (with the exception of dst = day light standard time, as it cannot be determined from epoch).
        /// </summary>
        [GodotMethod("get_datetime_from_unix_time")]
        public static Dictionary<object, object> GetDatetimeFromUnixTime(int unixTimeVal)
        {
            return NativeCalls.godot_icall_1_888(method_bind_47, ptr, unixTimeVal);
        }

        private static IntPtr method_bind_48 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_unix_time_from_datetime");

        /// <summary>
        /// Get an epoch time value from a dictionary of time values.
        /// [code]datetime[/code] must be populated with the following keys: year, month, day, hour, minute, second.
        /// You can pass the output from [method get_datetime_from_unix_time] directly into this function. Daylight savings time (dst), if present, is ignored.
        /// </summary>
        [GodotMethod("get_unix_time_from_datetime")]
        public static int GetUnixTimeFromDatetime(Dictionary<object, object> datetime)
        {
            return NativeCalls.godot_icall_1_1067(method_bind_48, ptr, datetime);
        }

        private static IntPtr method_bind_49 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_system_time_secs");

        /// <summary>
        /// Returns the epoch time of the operating system in seconds.
        /// </summary>
        [GodotMethod("get_system_time_secs")]
        public static int GetSystemTimeSecs()
        {
            return NativeCalls.godot_icall_0_1(method_bind_49, ptr);
        }

        private static IntPtr method_bind_50 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_icon");

        /// <summary>
        /// Sets the game's icon.
        /// </summary>
        [GodotMethod("set_icon")]
        public static void SetIcon(Image icon)
        {
            NativeCalls.godot_icall_1_24(method_bind_50, ptr, Object.GetPtr(icon));
        }

        private static IntPtr method_bind_51 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_exit_code");

        [GodotMethod("get_exit_code")]
        public static int GetExitCode()
        {
            return NativeCalls.godot_icall_0_1(method_bind_51, ptr);
        }

        private static IntPtr method_bind_52 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_exit_code");

        [GodotMethod("set_exit_code")]
        public static void SetExitCode(int code)
        {
            NativeCalls.godot_icall_1_0(method_bind_52, ptr, code);
        }

        private static IntPtr method_bind_53 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "delay_usec");

        /// <summary>
        /// Delay executing of the current thread by given microseconds.
        /// </summary>
        [GodotMethod("delay_usec")]
        public static void DelayUsec(int usec)
        {
            NativeCalls.godot_icall_1_0(method_bind_53, ptr, usec);
        }

        private static IntPtr method_bind_54 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "delay_msec");

        /// <summary>
        /// Delay executing of the current thread by given milliseconds.
        /// </summary>
        [GodotMethod("delay_msec")]
        public static void DelayMsec(int msec)
        {
            NativeCalls.godot_icall_1_0(method_bind_54, ptr, msec);
        }

        private static IntPtr method_bind_55 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_ticks_msec");

        /// <summary>
        /// Returns the amount of time passed in milliseconds since the engine started.
        /// </summary>
        [GodotMethod("get_ticks_msec")]
        public static int GetTicksMsec()
        {
            return NativeCalls.godot_icall_0_1(method_bind_55, ptr);
        }

        private static IntPtr method_bind_56 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_splash_tick_msec");

        /// <summary>
        /// </summary>
        [GodotMethod("get_splash_tick_msec")]
        public static int GetSplashTickMsec()
        {
            return NativeCalls.godot_icall_0_1(method_bind_56, ptr);
        }

        private static IntPtr method_bind_57 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_locale");

        /// <summary>
        /// Returns the host OS locale.
        /// </summary>
        [GodotMethod("get_locale")]
        public static string GetLocale()
        {
            return NativeCalls.godot_icall_0_2(method_bind_57, ptr);
        }

        private static IntPtr method_bind_58 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_latin_keyboard_variant");

        /// <summary>
        /// Returns the current latin keyboard variant as a String.
        /// Possible return values are: "QWERTY", "AZERTY", "QZERTY", "DVORAK", "NEO", "COLEMAK" or "ERROR".
        /// </summary>
        [GodotMethod("get_latin_keyboard_variant")]
        public static string GetLatinKeyboardVariant()
        {
            return NativeCalls.godot_icall_0_2(method_bind_58, ptr);
        }

        private static IntPtr method_bind_59 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_model_name");

        /// <summary>
        /// Returns the model name of the current device.
        /// </summary>
        [GodotMethod("get_model_name")]
        public static string GetModelName()
        {
            return NativeCalls.godot_icall_0_2(method_bind_59, ptr);
        }

        private static IntPtr method_bind_60 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "can_draw");

        /// <summary>
        /// Returns [code]true[/code] if the host OS allows drawing.
        /// </summary>
        [GodotMethod("can_draw")]
        public static bool CanDraw()
        {
            return NativeCalls.godot_icall_0_3(method_bind_60, ptr);
        }

        private static IntPtr method_bind_61 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_userfs_persistent");

        /// <summary>
        /// If [code]true[/code], the [code]user://[/code] file system is persistent, so that its state is the same after a player quits and starts the game again. Relevant to the HTML5 platform, where this persistence may be unavailable.
        /// </summary>
        [GodotMethod("is_userfs_persistent")]
        public static bool IsUserfsPersistent()
        {
            return NativeCalls.godot_icall_0_3(method_bind_61, ptr);
        }

        private static IntPtr method_bind_62 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_stdout_verbose");

        /// <summary>
        /// Returns [code]true[/code] if the engine was executed with -v (verbose stdout).
        /// </summary>
        [GodotMethod("is_stdout_verbose")]
        public static bool IsStdoutVerbose()
        {
            return NativeCalls.godot_icall_0_3(method_bind_62, ptr);
        }

        private static IntPtr method_bind_63 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "can_use_threads");

        /// <summary>
        /// Returns [code]true[/code] if the current host platform is using multiple threads.
        /// </summary>
        [GodotMethod("can_use_threads")]
        public static bool CanUseThreads()
        {
            return NativeCalls.godot_icall_0_3(method_bind_63, ptr);
        }

        private static IntPtr method_bind_64 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_debug_build");

        /// <summary>
        /// Returns [code]true[/code] if the build is a debug build.
        /// Returns [code]true[/code] when running in the editor.
        /// Returns [code]false[/code] if the build is a release build.
        /// </summary>
        [GodotMethod("is_debug_build")]
        public static bool IsDebugBuild()
        {
            return NativeCalls.godot_icall_0_3(method_bind_64, ptr);
        }

        private static IntPtr method_bind_65 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "dump_memory_to_file");

        /// <summary>
        /// Dumps the memory allocation ringlist to a file (only works in debug).
        /// Entry format per line: "Address - Size - Description"
        /// </summary>
        [GodotMethod("dump_memory_to_file")]
        public static void DumpMemoryToFile(string file)
        {
            NativeCalls.godot_icall_1_43(method_bind_65, ptr, file);
        }

        private static IntPtr method_bind_66 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "dump_resources_to_file");

        /// <summary>
        /// Dumps all used resources to file (only works in debug).
        /// Entry format per line: "Resource Type : Resource Location"
        /// At the end of the file is a statistic of all used Resource Types.
        /// </summary>
        [GodotMethod("dump_resources_to_file")]
        public static void DumpResourcesToFile(string file)
        {
            NativeCalls.godot_icall_1_43(method_bind_66, ptr, file);
        }

        private static IntPtr method_bind_67 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_virtual_keyboard");

        /// <summary>
        /// Returns [code]true[/code] if the platform has a virtual keyboard, [code]false[/code] otherwise.
        /// </summary>
        [GodotMethod("has_virtual_keyboard")]
        public static bool HasVirtualKeyboard()
        {
            return NativeCalls.godot_icall_0_3(method_bind_67, ptr);
        }

        private static IntPtr method_bind_68 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "show_virtual_keyboard");

        /// <summary>
        /// Shows the virtual keyboard if the platform has one. The [i]existing_text[/i] parameter is useful for implementing your own LineEdit, as it tells the virtual keyboard what text has already been typed (the virtual keyboard uses it for auto-correct and predictions).
        /// </summary>
        [GodotMethod("show_virtual_keyboard")]
        public static void ShowVirtualKeyboard(string existingText = "")
        {
            NativeCalls.godot_icall_1_43(method_bind_68, ptr, existingText);
        }

        private static IntPtr method_bind_69 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "hide_virtual_keyboard");

        /// <summary>
        /// Hides the virtual keyboard if it is shown, does nothing otherwise.
        /// </summary>
        [GodotMethod("hide_virtual_keyboard")]
        public static void HideVirtualKeyboard()
        {
            NativeCalls.godot_icall_0_12(method_bind_69, ptr);
        }

        private static IntPtr method_bind_70 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_virtual_keyboard_height");

        /// <summary>
        /// </summary>
        [GodotMethod("get_virtual_keyboard_height")]
        public static int GetVirtualKeyboardHeight()
        {
            return NativeCalls.godot_icall_0_1(method_bind_70, ptr);
        }

        private static IntPtr method_bind_71 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "print_resources_in_use");

        /// <summary>
        /// Shows all resources currently used by the game.
        /// </summary>
        [GodotMethod("print_resources_in_use")]
        public static void PrintResourcesInUse(bool @short = false)
        {
            NativeCalls.godot_icall_1_11(method_bind_71, ptr, @short);
        }

        private static IntPtr method_bind_72 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "print_all_resources");

        /// <summary>
        /// Shows all resources in the game. Optionally the list can be written to a file.
        /// </summary>
        [GodotMethod("print_all_resources")]
        public static void PrintAllResources(string tofile = "")
        {
            NativeCalls.godot_icall_1_43(method_bind_72, ptr, tofile);
        }

        private static IntPtr method_bind_73 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_static_memory_usage");

        /// <summary>
        /// Returns the amount of static memory being used by the program in bytes.
        /// </summary>
        [GodotMethod("get_static_memory_usage")]
        public static int GetStaticMemoryUsage()
        {
            return NativeCalls.godot_icall_0_1(method_bind_73, ptr);
        }

        private static IntPtr method_bind_74 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_static_memory_peak_usage");

        /// <summary>
        /// Returns the max amount of static memory used (only works in debug).
        /// </summary>
        [GodotMethod("get_static_memory_peak_usage")]
        public static int GetStaticMemoryPeakUsage()
        {
            return NativeCalls.godot_icall_0_1(method_bind_74, ptr);
        }

        private static IntPtr method_bind_75 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_dynamic_memory_usage");

        /// <summary>
        /// Returns the total amount of dynamic memory used (only works in debug).
        /// </summary>
        [GodotMethod("get_dynamic_memory_usage")]
        public static int GetDynamicMemoryUsage()
        {
            return NativeCalls.godot_icall_0_1(method_bind_75, ptr);
        }

        private static IntPtr method_bind_76 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_user_data_dir");

        /// <summary>
        /// Returns the absolute directory path where user data is written ([code]user://[/code]).
        /// </summary>
        [GodotMethod("get_user_data_dir")]
        public static string GetUserDataDir()
        {
            return NativeCalls.godot_icall_0_2(method_bind_76, ptr);
        }

        private static IntPtr method_bind_77 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_system_dir");

        /// <summary>
        /// Returns the actual path to commonly used folders across different platforms. Available locations are specified in [OS.SystemDir].
        /// </summary>
        [GodotMethod("get_system_dir")]
        public static string GetSystemDir(OS.SystemDir dir)
        {
            return NativeCalls.godot_icall_1_1068(method_bind_77, ptr, dir);
        }

        private static IntPtr method_bind_78 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_unique_id");

        /// <summary>
        /// Returns a string that is unique to the device. Currently only works on Android and iOS. Returns empty string on other platforms.
        /// </summary>
        [GodotMethod("get_unique_id")]
        public static string GetUniqueId()
        {
            return NativeCalls.godot_icall_0_2(method_bind_78, ptr);
        }

        private static IntPtr method_bind_79 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_ok_left_and_cancel_right");

        /// <summary>
        /// Returns [code]true[/code] if the "Okay" button should appear on the left and "Cancel" on the right.
        /// </summary>
        [GodotMethod("is_ok_left_and_cancel_right")]
        public static bool IsOkLeftAndCancelRight()
        {
            return NativeCalls.godot_icall_0_3(method_bind_79, ptr);
        }

        private static IntPtr method_bind_80 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "print_all_textures_by_size");

        /// <summary>
        /// Shows the list of loaded textures sorted by size in memory.
        /// </summary>
        [GodotMethod("print_all_textures_by_size")]
        public static void PrintAllTexturesBySize()
        {
            NativeCalls.godot_icall_0_12(method_bind_80, ptr);
        }

        private static IntPtr method_bind_81 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "print_resources_by_type");

        /// <summary>
        /// Shows the number of resources loaded by the game of the given types.
        /// </summary>
        [GodotMethod("print_resources_by_type")]
        public static void PrintResourcesByType(string[] types)
        {
            NativeCalls.godot_icall_1_340(method_bind_81, ptr, types);
        }

        private static IntPtr method_bind_82 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "native_video_play");

        /// <summary>
        /// Plays native video from the specified path, at the given volume and with audio and subtitle tracks.
        /// </summary>
        [GodotMethod("native_video_play")]
        public static Error NativeVideoPlay(string path, float volume, string audioTrack, string subtitleTrack)
        {
            return NativeCalls.godot_icall_4_1069(method_bind_82, ptr, path, volume, audioTrack, subtitleTrack);
        }

        private static IntPtr method_bind_83 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "native_video_is_playing");

        /// <summary>
        /// Returns [code]true[/code] if native video is playing.
        /// </summary>
        [GodotMethod("native_video_is_playing")]
        public static bool NativeVideoIsPlaying()
        {
            return NativeCalls.godot_icall_0_3(method_bind_83, ptr);
        }

        private static IntPtr method_bind_84 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "native_video_stop");

        /// <summary>
        /// Stops native video playback.
        /// </summary>
        [GodotMethod("native_video_stop")]
        public static void NativeVideoStop()
        {
            NativeCalls.godot_icall_0_12(method_bind_84, ptr);
        }

        private static IntPtr method_bind_85 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "native_video_pause");

        /// <summary>
        /// Pauses native video playback.
        /// </summary>
        [GodotMethod("native_video_pause")]
        public static void NativeVideoPause()
        {
            NativeCalls.godot_icall_0_12(method_bind_85, ptr);
        }

        private static IntPtr method_bind_86 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "native_video_unpause");

        /// <summary>
        /// Resumes native video playback.
        /// </summary>
        [GodotMethod("native_video_unpause")]
        public static void NativeVideoUnpause()
        {
            NativeCalls.godot_icall_0_12(method_bind_86, ptr);
        }

        private static IntPtr method_bind_87 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_scancode_string");

        /// <summary>
        /// Returns the given scancode as a string (e.g. Return values: "Escape", "Shift+Escape").
        /// </summary>
        [GodotMethod("get_scancode_string")]
        public static string GetScancodeString(int code)
        {
            return NativeCalls.godot_icall_1_108(method_bind_87, ptr, code);
        }

        private static IntPtr method_bind_88 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_scancode_unicode");

        /// <summary>
        /// Returns [code]true[/code] if the input code has a unicode character.
        /// </summary>
        [GodotMethod("is_scancode_unicode")]
        public static bool IsScancodeUnicode(int code)
        {
            return NativeCalls.godot_icall_1_29(method_bind_88, ptr, code);
        }

        private static IntPtr method_bind_89 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "find_scancode_from_string");

        /// <summary>
        /// Returns the scancode of the given string (e.g. "Escape")
        /// </summary>
        [GodotMethod("find_scancode_from_string")]
        public static int FindScancodeFromString(string @string)
        {
            return NativeCalls.godot_icall_1_87(method_bind_89, ptr, @string);
        }

        private static IntPtr method_bind_90 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_use_file_access_save_and_swap");

        /// <summary>
        /// Enables backup saves if [code]enabled[/code] is [code]true[/code].
        /// </summary>
        [GodotMethod("set_use_file_access_save_and_swap")]
        public static void SetUseFileAccessSaveAndSwap(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_90, ptr, enabled);
        }

        private static IntPtr method_bind_91 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "alert");

        /// <summary>
        /// Displays a modal dialog box utilizing the host OS.
        /// </summary>
        [GodotMethod("alert")]
        public static void Alert(string text, string title = "Alert!")
        {
            NativeCalls.godot_icall_2_69(method_bind_91, ptr, text, title);
        }

        private static IntPtr method_bind_92 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_thread_name");

        /// <summary>
        /// Sets the name of the current thread.
        /// </summary>
        [GodotMethod("set_thread_name")]
        public static Error SetThreadName(string name)
        {
            return NativeCalls.godot_icall_1_170(method_bind_92, ptr, name);
        }

        private static IntPtr method_bind_93 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_use_vsync");

        [GodotMethod("set_use_vsync")]
        public static void SetUseVsync(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_93, ptr, enable);
        }

        private static IntPtr method_bind_94 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_vsync_enabled");

        [GodotMethod("is_vsync_enabled")]
        public static bool IsVsyncEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_94, ptr);
        }

        private static IntPtr method_bind_95 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_feature");

        /// <summary>
        /// </summary>
        [GodotMethod("has_feature")]
        public static bool HasFeature(string tagName)
        {
            return NativeCalls.godot_icall_1_70(method_bind_95, ptr, tagName);
        }

        private static IntPtr method_bind_96 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_power_state");

        /// <summary>
        /// Returns the current state of the device regarding battery and power. See [code]POWERSTATE_*[/code] constants.
        /// </summary>
        [GodotMethod("get_power_state")]
        public static OS.PowerState GetPowerState()
        {
            return NativeCalls.godot_icall_0_1070(method_bind_96, ptr);
        }

        private static IntPtr method_bind_97 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_power_seconds_left");

        /// <summary>
        /// Returns the time in seconds before the device runs out of battery.
        /// </summary>
        [GodotMethod("get_power_seconds_left")]
        public static int GetPowerSecondsLeft()
        {
            return NativeCalls.godot_icall_0_1(method_bind_97, ptr);
        }

        private static IntPtr method_bind_98 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_power_percent_left");

        /// <summary>
        /// Returns the amount of battery left in the device as a percentage.
        /// </summary>
        [GodotMethod("get_power_percent_left")]
        public static int GetPowerPercentLeft()
        {
            return NativeCalls.godot_icall_0_1(method_bind_98, ptr);
        }
    }
}
