using System;

namespace Godot
{
    /// <summary>
    /// Base class for all non built-in types. Everything not a built-in type starts the inheritance chain from this class.
    /// Objects do not manage memory, if inheriting from one the object will most likely have to be deleted manually (call the [method free] function from the script or delete from C++).
    /// Some derivatives add memory management, such as [Reference] (which keeps a reference count and deletes itself automatically when no longer referenced) and [Node], which deletes the children tree when deleted.
    /// Objects export properties, which are mainly useful for storage and editing, but not really so much in programming. Properties are exported in [method _get_property_list] and handled in [method _get] and [method _set]. However, scripting languages and C++ have simpler means to export them.
    /// Objects also receive notifications ([method _notification]). Notifications are a simple way to notify the object about simple events, so they can all be handled together.
    /// </summary>
    public class Object : IDisposable
    {
        public const int NotificationPostinitialize = 0;
        public const int NotificationPredelete = 1;

        public enum ConnectFlags
        {
            Deferred = 1,
            Persist = 2,
            Oneshot = 4
        }


        private const string nativeName = "Object";

        private bool disposed = false;
        internal IntPtr ptr;
        internal bool memoryOwn;

        public Object() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Object_Ctor(this);
        }

        internal Object(bool memoryOwn)
        {
            this.memoryOwn = memoryOwn;
        }

        public IntPtr NativeInstance
        {
            get { return ptr; }
        }

        internal static IntPtr GetPtr(Object instance)
        {
            return instance == null ? IntPtr.Zero : instance.ptr;
        }

        ~Object()
        {
            Dispose(false);
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        public virtual void Dispose(bool disposing)
        {
            if (disposed) return;
            if (ptr != IntPtr.Zero)
            {
                if (memoryOwn)
                {
                    memoryOwn = false;
                    NativeCalls.godot_icall_Object_Dtor(this, ptr);
                }
            }
            this.ptr = IntPtr.Zero;
            GC.SuppressFinalize(this);
            disposed = true;
        }

        public SignalAwaiter ToSignal(Object source, string signal)
        {
            return new SignalAwaiter(source, signal, this);
        }

        /// <summary>
        /// Returns the given property. Returns [code]null[/code] if the [code]property[/code] does not exist.
        /// </summary>
        [GodotMethod("_get")]
        public virtual void _Get(string property)
        {
            return;
        }

        /// <summary>
        /// Returns the object's property list as an [Array] of dictionaries. Dictionaries must contain: name:String, type:int (see TYPE_* enum in [@GlobalScope]) and optionally: hint:int (see PROPERTY_HINT_* in [@GlobalScope]), hint_string:String, usage:int (see PROPERTY_USAGE_* in [@GlobalScope]).
        /// </summary>
        [GodotMethod("_get_property_list")]
        public virtual object[] _GetPropertyList()
        {
            return default(object[]);
        }

        /// <summary>
        /// The virtual method called upon initialization.
        /// </summary>
        [GodotMethod("_init")]
        public virtual void _Init()
        {
            return;
        }

        /// <summary>
        /// Notify the object internally using an ID.
        /// </summary>
        [GodotMethod("_notification")]
        public virtual void _Notification(int what)
        {
            return;
        }

        /// <summary>
        /// Sets a property. Returns [code]true[/code] if the [code]property[/code] exists.
        /// </summary>
        [GodotMethod("_set")]
        public virtual bool _Set(string property, object value)
        {
            return default(bool);
        }

        /// <summary>
        /// Deletes the object from memory.
        /// </summary>
        [GodotMethod("free")]
        public void Free()
        {
            Call("free");
        }

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_class");

        /// <summary>
        /// Returns the object's class as a [String].
        /// </summary>
        [GodotMethod("get_class")]
        public string GetClass()
        {
            return NativeCalls.godot_icall_0_2(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_class");

        /// <summary>
        /// Returns [code]true[/code] if the object inherits from the given [code]type[/code].
        /// </summary>
        [GodotMethod("is_class")]
        public bool IsClass(string type)
        {
            return NativeCalls.godot_icall_1_70(method_bind_1, Object.GetPtr(this), type);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set");

        /// <summary>
        /// Set property into the object.
        /// </summary>
        [GodotMethod("set")]
        public void Set(string property, object value)
        {
            NativeCalls.godot_icall_2_337(method_bind_2, Object.GetPtr(this), property, value);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get");

        /// <summary>
        /// Returns a [Variant] for a [code]property[/code].
        /// </summary>
        [GodotMethod("get")]
        public object Get(string property)
        {
            return NativeCalls.godot_icall_1_338(method_bind_3, Object.GetPtr(this), property);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_indexed");

        /// <summary>
        /// </summary>
        [GodotMethod("set_indexed")]
        public void SetIndexed(NodePath property, object value)
        {
            NativeCalls.godot_icall_2_560(method_bind_4, Object.GetPtr(this), NodePath.GetPtr(property), value);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_indexed");

        /// <summary>
        /// </summary>
        [GodotMethod("get_indexed")]
        public object GetIndexed(NodePath property)
        {
            return NativeCalls.godot_icall_1_561(method_bind_5, Object.GetPtr(this), NodePath.GetPtr(property));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_property_list");

        /// <summary>
        /// Returns the list of properties as an [Array] of dictionaries. Dictionaries contain: name:String, type:int (see TYPE_* enum in [@GlobalScope]) and optionally: hint:int (see PROPERTY_HINT_* in [@GlobalScope]), hint_string:String, usage:int (see PROPERTY_USAGE_* in [@GlobalScope]).
        /// </summary>
        [GodotMethod("get_property_list")]
        public object[] GetPropertyList()
        {
            return NativeCalls.godot_icall_0_21(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_method_list");

        /// <summary>
        /// Returns the object's methods and their signatures as an [Array].
        /// </summary>
        [GodotMethod("get_method_list")]
        public object[] GetMethodList()
        {
            return NativeCalls.godot_icall_0_21(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "notification");

        /// <summary>
        /// Notify the object of something.
        /// </summary>
        [GodotMethod("notification")]
        public void Notification(int what, bool reversed = false)
        {
            NativeCalls.godot_icall_2_51(method_bind_8, Object.GetPtr(this), what, reversed);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_instance_id");

        /// <summary>
        /// Returns the object's unique instance ID.
        /// </summary>
        [GodotMethod("get_instance_id")]
        public int GetInstanceId()
        {
            return NativeCalls.godot_icall_0_1(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_script");

        /// <summary>
        /// Set a script into the object, scripts extend the object functionality.
        /// </summary>
        [GodotMethod("set_script")]
        public void SetScript(Reference script)
        {
            NativeCalls.godot_icall_1_24(method_bind_10, Object.GetPtr(this), Object.GetPtr(script));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_script");

        /// <summary>
        /// Returns the object's [Script] or [code]null[/code] if one doesn't exist.
        /// </summary>
        [GodotMethod("get_script")]
        public Reference GetScript()
        {
            return NativeCalls.godot_icall_0_562(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_meta");

        /// <summary>
        /// Set a metadata into the object. Metadata is serialized. Metadata can be [i]anything[/i].
        /// </summary>
        [GodotMethod("set_meta")]
        public void SetMeta(string name, object value)
        {
            NativeCalls.godot_icall_2_337(method_bind_12, Object.GetPtr(this), name, value);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_meta");

        /// <summary>
        /// Returns the object's metadata for the given [code]name[/code].
        /// </summary>
        [GodotMethod("get_meta")]
        public object GetMeta(string name)
        {
            return NativeCalls.godot_icall_1_338(method_bind_13, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_meta");

        /// <summary>
        /// Returns [code]true[/code] if a metadata is found with the given [code]name[/code].
        /// </summary>
        [GodotMethod("has_meta")]
        public bool HasMeta(string name)
        {
            return NativeCalls.godot_icall_1_70(method_bind_14, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_meta_list");

        /// <summary>
        /// Returns the object's metadata as a [PoolStringArray].
        /// </summary>
        [GodotMethod("get_meta_list")]
        public string[] GetMetaList()
        {
            return NativeCalls.godot_icall_0_72(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_user_signal");

        /// <summary>
        /// Adds a user-defined [code]signal[/code]. Arguments are optional, but can be added as an [Array] of dictionaries, each containing "name" and "type" (from [@GlobalScope] TYPE_*).
        /// &lt;param name="arguments"&gt;If the param is null, then the default value is new object[] {}&lt;/param&gt;
        /// </summary>
        [GodotMethod("add_user_signal")]
        public void AddUserSignal(string signal, object[] arguments = null)
        {
            object[] arguments_in = arguments != null ? arguments : new object[] {};
            NativeCalls.godot_icall_2_563(method_bind_16, Object.GetPtr(this), signal, arguments_in);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_user_signal");

        /// <summary>
        /// Returns [code]true[/code] if the given user-defined [code]signal[/code] exists.
        /// </summary>
        [GodotMethod("has_user_signal")]
        public bool HasUserSignal(string signal)
        {
            return NativeCalls.godot_icall_1_70(method_bind_17, Object.GetPtr(this), signal);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "emit_signal");

        /// <summary>
        /// Emits the given [code]signal[/code].
        /// </summary>
        [GodotMethod("emit_signal")]
        public object EmitSignal(string signal, params object[] @args)
        {
            return NativeCalls.godot_icall_2_555(method_bind_18, Object.GetPtr(this), signal, @args);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "call");

        /// <summary>
        /// Calls the [code]method[/code] on the object and returns a result. Pass parameters as a comma separated list.
        /// </summary>
        [GodotMethod("call")]
        public object Call(string method, params object[] @args)
        {
            return NativeCalls.godot_icall_2_555(method_bind_19, Object.GetPtr(this), method, @args);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "call_deferred");

        /// <summary>
        /// Calls the [code]method[/code] on the object during idle time and returns a result. Pass parameters as a comma separated list.
        /// </summary>
        [GodotMethod("call_deferred")]
        public object CallDeferred(string method, params object[] @args)
        {
            return NativeCalls.godot_icall_2_555(method_bind_20, Object.GetPtr(this), method, @args);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "callv");

        /// <summary>
        /// Calls the [code]method[/code] on the object and returns a result. Pass parameters as an [Array].
        /// </summary>
        [GodotMethod("callv")]
        public object Callv(string method, object[] argArray)
        {
            return NativeCalls.godot_icall_2_564(method_bind_21, Object.GetPtr(this), method, argArray);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_method");

        /// <summary>
        /// Returns [code]true[/code] if the object contains the given [code]method[/code].
        /// </summary>
        [GodotMethod("has_method")]
        public bool HasMethod(string method)
        {
            return NativeCalls.godot_icall_1_70(method_bind_22, Object.GetPtr(this), method);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_signal_list");

        /// <summary>
        /// Returns the list of signals as an [Array] of dictionaries.
        /// </summary>
        [GodotMethod("get_signal_list")]
        public object[] GetSignalList()
        {
            return NativeCalls.godot_icall_0_21(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_signal_connection_list");

        /// <summary>
        /// Returns an [Array] of connections for the given [code]signal[/code].
        /// </summary>
        [GodotMethod("get_signal_connection_list")]
        public object[] GetSignalConnectionList(string signal)
        {
            return NativeCalls.godot_icall_1_446(method_bind_24, Object.GetPtr(this), signal);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_incoming_connections");

        /// <summary>
        /// Returns an [Array] of dictionaries with information about signals that are connected to the object.
        /// Inside each [Dictionary] there are 3 fields:
        /// - "source" is a reference to signal emitter.
        /// - "signal_name" is name of connected signal.
        /// - "method_name" is a name of method to which signal is connected.
        /// </summary>
        [GodotMethod("get_incoming_connections")]
        public object[] GetIncomingConnections()
        {
            return NativeCalls.godot_icall_0_21(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "connect");

        /// <summary>
        /// Connects a [code]signal[/code] to a [code]method[/code] on a [code]target[/code] object. Pass optional [code]binds[/code] to the call. Use [code]flags[/code] to set deferred or one shot connections. See [code]CONNECT_*[/code] constants. A [code]signal[/code] can only be connected once to a [code]method[/code]. It will throw an error if already connected. To avoid this, first use [method is_connected] to check for existing connections.
        /// &lt;param name="binds"&gt;If the param is null, then the default value is new object[] {}&lt;/param&gt;
        /// </summary>
        [GodotMethod("connect")]
        public Error Connect(string signal, Object target, string method, object[] binds = null, int flags = 0)
        {
            object[] binds_in = binds != null ? binds : new object[] {};
            return NativeCalls.godot_icall_5_565(method_bind_26, Object.GetPtr(this), signal, Object.GetPtr(target), method, binds_in, flags);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "disconnect");

        /// <summary>
        /// Disconnects a [code]signal[/code] from a [code]method[/code] on the given [code]target[/code].
        /// </summary>
        [GodotMethod("disconnect")]
        public void Disconnect(string signal, Object target, string method)
        {
            NativeCalls.godot_icall_3_566(method_bind_27, Object.GetPtr(this), signal, Object.GetPtr(target), method);
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_connected");

        /// <summary>
        /// Returns [code]true[/code] if a connection exists for a given [code]signal[/code], [code]target[/code], and [code]method[/code].
        /// </summary>
        [GodotMethod("is_connected")]
        public bool IsConnected(string signal, Object target, string method)
        {
            return NativeCalls.godot_icall_3_567(method_bind_28, Object.GetPtr(this), signal, Object.GetPtr(target), method);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_block_signals");

        /// <summary>
        /// If set to true, signal emission is blocked.
        /// </summary>
        [GodotMethod("set_block_signals")]
        public void SetBlockSignals(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_29, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_blocking_signals");

        /// <summary>
        /// Returns [code]true[/code] if signal emission blocking is enabled.
        /// </summary>
        [GodotMethod("is_blocking_signals")]
        public bool IsBlockingSignals()
        {
            return NativeCalls.godot_icall_0_3(method_bind_30, Object.GetPtr(this));
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "property_list_changed_notify");

        /// <summary>
        /// </summary>
        [GodotMethod("property_list_changed_notify")]
        public void PropertyListChangedNotify()
        {
            NativeCalls.godot_icall_0_12(method_bind_31, Object.GetPtr(this));
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_message_translation");

        /// <summary>
        /// Define whether the object can translate strings (with calls to [method tr]). Default is true.
        /// </summary>
        [GodotMethod("set_message_translation")]
        public void SetMessageTranslation(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_32, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "can_translate_messages");

        /// <summary>
        /// Returns [code]true[/code] if the object can translate strings.
        /// </summary>
        [GodotMethod("can_translate_messages")]
        public bool CanTranslateMessages()
        {
            return NativeCalls.godot_icall_0_3(method_bind_33, Object.GetPtr(this));
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tr");

        /// <summary>
        /// Translate a message. Only works if message translation is enabled (which it is by default). See [method set_message_translation].
        /// </summary>
        [GodotMethod("tr")]
        public string Tr(string message)
        {
            return NativeCalls.godot_icall_1_73(method_bind_34, Object.GetPtr(this), message);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_queued_for_deletion");

        /// <summary>
        /// Returns [code]true[/code] if the [code]queue_free[/code] method was called for the object.
        /// </summary>
        [GodotMethod("is_queued_for_deletion")]
        public bool IsQueuedForDeletion()
        {
            return NativeCalls.godot_icall_0_3(method_bind_35, Object.GetPtr(this));
        }
    }
}
