using System;
using System.Collections.Generic;

namespace Godot
{
    /// <summary>
    /// A simplified interface to a scene file. Provides access to operations and checks that can be performed on the scene resource itself.
    /// TODO: explain ownership, and that node does not need to own itself
    /// </summary>
    public class PackedScene : Resource
    {
        public enum GenEditState
        {
            Disabled = 0,
            Instance = 1,
            Main = 2
        }


        /// <summary>
        /// A dictionary representation of the scene contents.
        /// Available keys include "rnames" and "variants" for resources, "node_count", "nodes", "node_paths" for nodes, "editable_instances" for base scene children overrides, "conn_count" and "conns" for signal connections, and "version" for the format style of the PackedScene.
        /// </summary>
        public Dictionary<object, object> _Bundled
        {
            get
            {
                return _GetBundledScene();
            }
            set
            {
                _SetBundledScene(value);
            }
        }


        private const string nativeName = "PackedScene";

        public PackedScene() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_PackedScene_Ctor(this);
        }

        internal PackedScene(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "pack");

        /// <summary>
        /// Pack will ignore any sub-nodes not owned by given node. See [method Node.set_owner].
        /// </summary>
        [GodotMethod("pack")]
        public Error Pack(Node path)
        {
            return NativeCalls.godot_icall_1_578(method_bind_0, Object.GetPtr(this), Object.GetPtr(path));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "instance");

        /// <summary>
        /// Instantiates the scene's node hierarchy. Triggers child scene instantiation(s). Triggers the [enum Object.NOTIFICATION_INSTANCED] notification on the root node.
        /// </summary>
        [GodotMethod("instance")]
        public Node Instance(PackedScene.GenEditState editState = (PackedScene.GenEditState)0)
        {
            return NativeCalls.godot_icall_1_579(method_bind_1, Object.GetPtr(this), editState);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "can_instance");

        /// <summary>
        /// Returns [code]true[/code] if the scene file has nodes.
        /// </summary>
        [GodotMethod("can_instance")]
        public bool CanInstance()
        {
            return NativeCalls.godot_icall_0_3(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_bundled_scene");

        internal void _SetBundledScene(Dictionary<object, object> arg0)
        {
            NativeCalls.godot_icall_1_168(method_bind_3, Object.GetPtr(this), arg0);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_bundled_scene");

        internal Dictionary<object, object> _GetBundledScene()
        {
            return NativeCalls.godot_icall_0_169(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_state");

        /// <summary>
        /// Returns the [code]SceneState[/code] representing the scene file contents.
        /// </summary>
        [GodotMethod("get_state")]
        public SceneState GetState()
        {
            return NativeCalls.godot_icall_0_580(method_bind_5, Object.GetPtr(this));
        }
    }
}
