using System;

namespace Godot
{
    /// <summary>
    /// A ParallaxBackground uses one or more [ParallaxLayer] child nodes to create a parallax effect. Each [ParallaxLayer] can move at a different speed using [member ParallaxLayer.motion_offset]. This creates an illusion of depth in a 2D game. If not used with a [Camera2D], you must manually calculate the [member scroll_offset].
    /// </summary>
    public class ParallaxBackground : CanvasLayer
    {
        /// <summary>
        /// The ParallaxBackground's scroll value. Calculated automatically when using a [Camera2D], but can be used to manually manage scrolling when no camera is present.
        /// </summary>
        public Vector2 ScrollOffset
        {
            get
            {
                return GetScrollOffset();
            }
            set
            {
                SetScrollOffset(value);
            }
        }

        /// <summary>
        /// Base position offset of all [ParallaxLayer] children.
        /// </summary>
        public Vector2 ScrollBaseOffset
        {
            get
            {
                return GetScrollBaseOffset();
            }
            set
            {
                SetScrollBaseOffset(value);
            }
        }

        /// <summary>
        /// Base motion scale of all [ParallaxLayer] children.
        /// </summary>
        public Vector2 ScrollBaseScale
        {
            get
            {
                return GetScrollBaseScale();
            }
            set
            {
                SetScrollBaseScale(value);
            }
        }

        /// <summary>
        /// Top left limits for scrolling to begin. If the camera is outside of this limit the background will stop scrolling. Must be lower than [member scroll_limit_end] to work.
        /// </summary>
        public Vector2 ScrollLimitBegin
        {
            get
            {
                return GetLimitBegin();
            }
            set
            {
                SetLimitBegin(value);
            }
        }

        /// <summary>
        /// Right bottom limits for scrolling to end. If the camera is outside of this limit the background will stop scrolling. Must be higher than [member scroll_limit_begin] to work.
        /// </summary>
        public Vector2 ScrollLimitEnd
        {
            get
            {
                return GetLimitEnd();
            }
            set
            {
                SetLimitEnd(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] elements in [ParallaxLayer] child aren't affected by the zoom level of the camera.
        /// </summary>
        public bool ScrollIgnoreCameraZoom
        {
            get
            {
                return IsIgnoreCameraZoom();
            }
            set
            {
                SetIgnoreCameraZoom(value);
            }
        }


        private const string nativeName = "ParallaxBackground";

        public ParallaxBackground() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ParallaxBackground_Ctor(this);
        }

        internal ParallaxBackground(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_scroll_offset");

        [GodotMethod("set_scroll_offset")]
        public void SetScrollOffset(Vector2 ofs)
        {
            NativeCalls.godot_icall_1_45(method_bind_0, Object.GetPtr(this), ref ofs);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_scroll_offset");

        [GodotMethod("get_scroll_offset")]
        public Vector2 GetScrollOffset()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_1, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_scroll_base_offset");

        [GodotMethod("set_scroll_base_offset")]
        public void SetScrollBaseOffset(Vector2 ofs)
        {
            NativeCalls.godot_icall_1_45(method_bind_2, Object.GetPtr(this), ref ofs);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_scroll_base_offset");

        [GodotMethod("get_scroll_base_offset")]
        public Vector2 GetScrollBaseOffset()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_3, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_scroll_base_scale");

        [GodotMethod("set_scroll_base_scale")]
        public void SetScrollBaseScale(Vector2 scale)
        {
            NativeCalls.godot_icall_1_45(method_bind_4, Object.GetPtr(this), ref scale);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_scroll_base_scale");

        [GodotMethod("get_scroll_base_scale")]
        public Vector2 GetScrollBaseScale()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_5, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_limit_begin");

        [GodotMethod("set_limit_begin")]
        public void SetLimitBegin(Vector2 ofs)
        {
            NativeCalls.godot_icall_1_45(method_bind_6, Object.GetPtr(this), ref ofs);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_limit_begin");

        [GodotMethod("get_limit_begin")]
        public Vector2 GetLimitBegin()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_7, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_limit_end");

        [GodotMethod("set_limit_end")]
        public void SetLimitEnd(Vector2 ofs)
        {
            NativeCalls.godot_icall_1_45(method_bind_8, Object.GetPtr(this), ref ofs);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_limit_end");

        [GodotMethod("get_limit_end")]
        public Vector2 GetLimitEnd()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_9, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ignore_camera_zoom");

        [GodotMethod("set_ignore_camera_zoom")]
        public void SetIgnoreCameraZoom(bool ignore)
        {
            NativeCalls.godot_icall_1_11(method_bind_10, Object.GetPtr(this), ignore);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_ignore_camera_zoom");

        [GodotMethod("is_ignore_camera_zoom")]
        public bool IsIgnoreCameraZoom()
        {
            return NativeCalls.godot_icall_0_3(method_bind_11, Object.GetPtr(this));
        }
    }
}
