using System;

namespace Godot
{
    /// <summary>
    /// 3D particle node used to create a variety of particle systems and effects. [code]Particles[/code] features an emitter that generates some number of particles at a given rate.
    /// Use the [code]process_material[/code] property to add a [ParticlesMaterial] to configure particle appearance and behavior. Alternatively, you can add a [ShaderMaterial] which will be applied to all particles.
    /// </summary>
    public class Particles : GeometryInstance
    {
        public const int MaxDrawPasses = 4;

        public enum DrawOrderEnum
        {
            Index = 0,
            Lifetime = 1,
            ViewDepth = 2
        }


        /// <summary>
        /// If [code]true[/code] particles are being emitted. Default value: [code]true[/code].
        /// </summary>
        public bool Emitting
        {
            get
            {
                return IsEmitting();
            }
            set
            {
                SetEmitting(value);
            }
        }

        /// <summary>
        /// Number of particles to emit.
        /// </summary>
        public int Amount
        {
            get
            {
                return GetAmount();
            }
            set
            {
                SetAmount(value);
            }
        }

        /// <summary>
        /// Amount of time each particle will exist. Default value: [code]1[/code].
        /// </summary>
        public float Lifetime
        {
            get
            {
                return GetLifetime();
            }
            set
            {
                SetLifetime(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] only [code]amount[/code] particles will be emitted. Default value: [code]false[/code].
        /// </summary>
        public bool OneShot
        {
            get
            {
                return GetOneShot();
            }
            set
            {
                SetOneShot(value);
            }
        }

        /// <summary>
        /// </summary>
        public float Preprocess
        {
            get
            {
                return GetPreProcessTime();
            }
            set
            {
                SetPreProcessTime(value);
            }
        }

        /// <summary>
        /// Speed scaling ratio. Default value: [code]1[/code].
        /// </summary>
        public float SpeedScale
        {
            get
            {
                return GetSpeedScale();
            }
            set
            {
                SetSpeedScale(value);
            }
        }

        /// <summary>
        /// Time ratio between each emission. If [code]0[/code] particles are emitted continuously. If [code]1[/code] all particles are emitted simultaneously. Default value: [code]0[/code].
        /// </summary>
        public float Explosiveness
        {
            get
            {
                return GetExplosivenessRatio();
            }
            set
            {
                SetExplosivenessRatio(value);
            }
        }

        /// <summary>
        /// Emission randomness ratio. Default value: [code]0[/code].
        /// </summary>
        public float Randomness
        {
            get
            {
                return GetRandomnessRatio();
            }
            set
            {
                SetRandomnessRatio(value);
            }
        }

        /// <summary>
        /// </summary>
        public int FixedFps
        {
            get
            {
                return GetFixedFps();
            }
            set
            {
                SetFixedFps(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool FractDelta
        {
            get
            {
                return GetFractionalDelta();
            }
            set
            {
                SetFractionalDelta(value);
            }
        }

        /// <summary>
        /// </summary>
        public AABB VisibilityAabb
        {
            get
            {
                return GetVisibilityAabb();
            }
            set
            {
                SetVisibilityAabb(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] particles use the parent node's coordinate space. If [code]false[/code] they use global coordinates. Default value: [code]true[/code].
        /// </summary>
        public bool LocalCoords
        {
            get
            {
                return GetUseLocalCoordinates();
            }
            set
            {
                SetUseLocalCoordinates(value);
            }
        }

        /// <summary>
        /// Particle draw order. Uses [code]DRAW_ORDER_*[/code] values. Default value: [code]DRAW_ORDER_INDEX[/code].
        /// </summary>
        public Particles.DrawOrderEnum DrawOrder
        {
            get
            {
                return GetDrawOrder();
            }
            set
            {
                SetDrawOrder(value);
            }
        }

        /// <summary>
        /// [Material] for processing particles. Can be a [ParticlesMaterial] or a [ShaderMaterial].
        /// </summary>
        public Material ProcessMaterial
        {
            get
            {
                return GetProcessMaterial();
            }
            set
            {
                SetProcessMaterial(value);
            }
        }

        /// <summary>
        /// </summary>
        public int DrawPasses
        {
            get
            {
                return GetDrawPasses();
            }
            set
            {
                SetDrawPasses(value);
            }
        }

        /// <summary>
        /// </summary>
        public Mesh DrawPass1
        {
            get
            {
                return GetDrawPassMesh(0);
            }
            set
            {
                SetDrawPassMesh(0, value);
            }
        }

        /// <summary>
        /// </summary>
        public Mesh DrawPass2
        {
            get
            {
                return GetDrawPassMesh(1);
            }
            set
            {
                SetDrawPassMesh(1, value);
            }
        }

        /// <summary>
        /// </summary>
        public Mesh DrawPass3
        {
            get
            {
                return GetDrawPassMesh(2);
            }
            set
            {
                SetDrawPassMesh(2, value);
            }
        }

        /// <summary>
        /// </summary>
        public Mesh DrawPass4
        {
            get
            {
                return GetDrawPassMesh(3);
            }
            set
            {
                SetDrawPassMesh(3, value);
            }
        }


        private const string nativeName = "Particles";

        public Particles() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Particles_Ctor(this);
        }

        internal Particles(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_emitting");

        [GodotMethod("set_emitting")]
        public void SetEmitting(bool emitting)
        {
            NativeCalls.godot_icall_1_11(method_bind_0, Object.GetPtr(this), emitting);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_amount");

        [GodotMethod("set_amount")]
        public void SetAmount(int amount)
        {
            NativeCalls.godot_icall_1_0(method_bind_1, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_lifetime");

        [GodotMethod("set_lifetime")]
        public void SetLifetime(float secs)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), secs);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_one_shot");

        [GodotMethod("set_one_shot")]
        public void SetOneShot(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_3, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_pre_process_time");

        [GodotMethod("set_pre_process_time")]
        public void SetPreProcessTime(float secs)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), secs);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_explosiveness_ratio");

        [GodotMethod("set_explosiveness_ratio")]
        public void SetExplosivenessRatio(float ratio)
        {
            NativeCalls.godot_icall_1_10(method_bind_5, Object.GetPtr(this), ratio);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_randomness_ratio");

        [GodotMethod("set_randomness_ratio")]
        public void SetRandomnessRatio(float ratio)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), ratio);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_visibility_aabb");

        [GodotMethod("set_visibility_aabb")]
        public void SetVisibilityAabb(AABB aabb)
        {
            NativeCalls.godot_icall_1_117(method_bind_7, Object.GetPtr(this), ref aabb);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_use_local_coordinates");

        [GodotMethod("set_use_local_coordinates")]
        public void SetUseLocalCoordinates(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_8, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fixed_fps");

        [GodotMethod("set_fixed_fps")]
        public void SetFixedFps(int fps)
        {
            NativeCalls.godot_icall_1_0(method_bind_9, Object.GetPtr(this), fps);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fractional_delta");

        [GodotMethod("set_fractional_delta")]
        public void SetFractionalDelta(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_10, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_process_material");

        [GodotMethod("set_process_material")]
        public void SetProcessMaterial(Material material)
        {
            NativeCalls.godot_icall_1_24(method_bind_11, Object.GetPtr(this), Object.GetPtr(material));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_speed_scale");

        [GodotMethod("set_speed_scale")]
        public void SetSpeedScale(float scale)
        {
            NativeCalls.godot_icall_1_10(method_bind_12, Object.GetPtr(this), scale);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_emitting");

        [GodotMethod("is_emitting")]
        public bool IsEmitting()
        {
            return NativeCalls.godot_icall_0_3(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_amount");

        [GodotMethod("get_amount")]
        public int GetAmount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_lifetime");

        [GodotMethod("get_lifetime")]
        public float GetLifetime()
        {
            return NativeCalls.godot_icall_0_9(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_one_shot");

        [GodotMethod("get_one_shot")]
        public bool GetOneShot()
        {
            return NativeCalls.godot_icall_0_3(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_pre_process_time");

        [GodotMethod("get_pre_process_time")]
        public float GetPreProcessTime()
        {
            return NativeCalls.godot_icall_0_9(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_explosiveness_ratio");

        [GodotMethod("get_explosiveness_ratio")]
        public float GetExplosivenessRatio()
        {
            return NativeCalls.godot_icall_0_9(method_bind_18, Object.GetPtr(this));
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_randomness_ratio");

        [GodotMethod("get_randomness_ratio")]
        public float GetRandomnessRatio()
        {
            return NativeCalls.godot_icall_0_9(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_visibility_aabb");

        [GodotMethod("get_visibility_aabb")]
        public AABB GetVisibilityAabb()
        {
            object ret = NativeCalls.godot_icall_0_118(method_bind_20, Object.GetPtr(this));
            return (AABB)ret;
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_use_local_coordinates");

        [GodotMethod("get_use_local_coordinates")]
        public bool GetUseLocalCoordinates()
        {
            return NativeCalls.godot_icall_0_3(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fixed_fps");

        [GodotMethod("get_fixed_fps")]
        public int GetFixedFps()
        {
            return NativeCalls.godot_icall_0_1(method_bind_22, Object.GetPtr(this));
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fractional_delta");

        [GodotMethod("get_fractional_delta")]
        public bool GetFractionalDelta()
        {
            return NativeCalls.godot_icall_0_3(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_process_material");

        [GodotMethod("get_process_material")]
        public Material GetProcessMaterial()
        {
            return NativeCalls.godot_icall_0_222(method_bind_24, Object.GetPtr(this));
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_speed_scale");

        [GodotMethod("get_speed_scale")]
        public float GetSpeedScale()
        {
            return NativeCalls.godot_icall_0_9(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_draw_order");

        [GodotMethod("set_draw_order")]
        public void SetDrawOrder(Particles.DrawOrderEnum order)
        {
            NativeCalls.godot_icall_1_582(method_bind_26, Object.GetPtr(this), order);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_draw_order");

        [GodotMethod("get_draw_order")]
        public Particles.DrawOrderEnum GetDrawOrder()
        {
            return NativeCalls.godot_icall_0_583(method_bind_27, Object.GetPtr(this));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_draw_passes");

        [GodotMethod("set_draw_passes")]
        public void SetDrawPasses(int passes)
        {
            NativeCalls.godot_icall_1_0(method_bind_28, Object.GetPtr(this), passes);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_draw_pass_mesh");

        [GodotMethod("set_draw_pass_mesh")]
        public void SetDrawPassMesh(int pass, Mesh mesh)
        {
            NativeCalls.godot_icall_2_67(method_bind_29, Object.GetPtr(this), pass, Object.GetPtr(mesh));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_draw_passes");

        [GodotMethod("get_draw_passes")]
        public int GetDrawPasses()
        {
            return NativeCalls.godot_icall_0_1(method_bind_30, Object.GetPtr(this));
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_draw_pass_mesh");

        [GodotMethod("get_draw_pass_mesh")]
        public Mesh GetDrawPassMesh(int pass)
        {
            return NativeCalls.godot_icall_1_513(method_bind_31, Object.GetPtr(this), pass);
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "restart");

        /// <summary>
        /// </summary>
        [GodotMethod("restart")]
        public void Restart()
        {
            NativeCalls.godot_icall_0_12(method_bind_32, Object.GetPtr(this));
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "capture_aabb");

        /// <summary>
        /// </summary>
        [GodotMethod("capture_aabb")]
        public AABB CaptureAabb()
        {
            object ret = NativeCalls.godot_icall_0_118(method_bind_33, Object.GetPtr(this));
            return (AABB)ret;
        }
    }
}
