using System;

namespace Godot
{
    /// <summary>
    /// ParticlesMaterial defines particle properties and behavior. It is used in the [code]process_material[/code] of [Particles] and [Particles2D] emitter nodes.
    /// Some of this material's properties are applied to each particle when emitted, while others can have a [CurveTexture] applied to vary values over the lifetime of the particle.
    /// </summary>
    public class ParticlesMaterial : Material
    {
        public enum Flags
        {
            AlignYToVelocity = 0,
            RotateY = 1,
            Max = 4
        }

        public enum EmissionShapeEnum
        {
            Point = 0,
            Sphere = 1,
            Box = 2,
            Points = 3,
            DirectedPoints = 4
        }

        public enum Parameter
        {
            InitialLinearVelocity = 0,
            AngularVelocity = 1,
            OrbitVelocity = 2,
            LinearAccel = 3,
            RadialAccel = 4,
            TangentialAccel = 5,
            Damping = 6,
            Angle = 7,
            Scale = 8,
            HueVariation = 9,
            AnimSpeed = 10,
            AnimOffset = 11,
            Max = 12
        }


        /// <summary>
        /// Emitter will emit [code]amount[/code] divided by [code]trail_divisor[/code] particles. The remaining particles will be used as trail(s).
        /// </summary>
        public int TrailDivisor
        {
            get
            {
                return GetTrailDivisor();
            }
            set
            {
                SetTrailDivisor(value);
            }
        }

        /// <summary>
        /// Trail particles' size will vary along this [CurveTexture].
        /// </summary>
        public CurveTexture TrailSizeModifier
        {
            get
            {
                return GetTrailSizeModifier();
            }
            set
            {
                SetTrailSizeModifier(value);
            }
        }

        /// <summary>
        /// Trail particles' color will vary along this [GradientTexture].
        /// </summary>
        public GradientTexture TrailColorModifier
        {
            get
            {
                return GetTrailColorModifier();
            }
            set
            {
                SetTrailColorModifier(value);
            }
        }

        /// <summary>
        /// Particles will be emitted inside this region. Use [code]EMISSION_SHAPE_*[/code] constants for values. Default value: [code]EMISSION_SHAPE_POINT[/code].
        /// </summary>
        public ParticlesMaterial.EmissionShapeEnum EmissionShape
        {
            get
            {
                return GetEmissionShape();
            }
            set
            {
                SetEmissionShape(value);
            }
        }

        /// <summary>
        /// The sphere's radius if [code]emission_shape[/code] is set to [code]EMISSION_SHAPE_SPHERE[/code].
        /// </summary>
        public float EmissionSphereRadius
        {
            get
            {
                return GetEmissionSphereRadius();
            }
            set
            {
                SetEmissionSphereRadius(value);
            }
        }

        /// <summary>
        /// The box's extents if [code]emission_shape[/code] is set to [code]EMISSION_SHAPE_BOX[/code].
        /// </summary>
        public Vector3 EmissionBoxExtents
        {
            get
            {
                return GetEmissionBoxExtents();
            }
            set
            {
                SetEmissionBoxExtents(value);
            }
        }

        /// <summary>
        /// </summary>
        public Texture EmissionPointTexture
        {
            get
            {
                return GetEmissionPointTexture();
            }
            set
            {
                SetEmissionPointTexture(value);
            }
        }

        /// <summary>
        /// </summary>
        public Texture EmissionNormalTexture
        {
            get
            {
                return GetEmissionNormalTexture();
            }
            set
            {
                SetEmissionNormalTexture(value);
            }
        }

        /// <summary>
        /// </summary>
        public Texture EmissionColorTexture
        {
            get
            {
                return GetEmissionColorTexture();
            }
            set
            {
                SetEmissionColorTexture(value);
            }
        }

        /// <summary>
        /// The number of emission points if [code]emission_shape[/code] is set to [code]EMISSION_SHAPE_POINTS[/code] or [code]EMISSION_SHAPE_DIRECTED_POINTS[/code].
        /// </summary>
        public int EmissionPointCount
        {
            get
            {
                return GetEmissionPointCount();
            }
            set
            {
                SetEmissionPointCount(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool FlagAlignY
        {
            get
            {
                return GetFlag((ParticlesMaterial.Flags)0);
            }
            set
            {
                SetFlag((ParticlesMaterial.Flags)0, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool FlagRotateY
        {
            get
            {
                return GetFlag((ParticlesMaterial.Flags)1);
            }
            set
            {
                SetFlag((ParticlesMaterial.Flags)1, value);
            }
        }

        /// <summary>
        /// If [code]true[/code] particles will not move on the z axis. Default value: [code]true[/code] for [Particles2D], [code]false[/code] for [Particles].
        /// </summary>
        public bool FlagDisableZ
        {
            get
            {
                return GetFlag((ParticlesMaterial.Flags)2);
            }
            set
            {
                SetFlag((ParticlesMaterial.Flags)2, value);
            }
        }

        /// <summary>
        /// Each particle's initial direction range from [code]+spread[/code] to [code]-spread[/code] degrees. Default value: [code]45[/code].
        /// </summary>
        public float Spread
        {
            get
            {
                return GetSpread();
            }
            set
            {
                SetSpread(value);
            }
        }

        /// <summary>
        /// </summary>
        public float Flatness
        {
            get
            {
                return GetFlatness();
            }
            set
            {
                SetFlatness(value);
            }
        }

        /// <summary>
        /// Gravity applied to every particle. Default value: [code](0, 98, 0)[/code].
        /// </summary>
        public Vector3 Gravity
        {
            get
            {
                return GetGravity();
            }
            set
            {
                SetGravity(value);
            }
        }

        /// <summary>
        /// Initial velocity for each particle.
        /// </summary>
        public float InitialVelocity
        {
            get
            {
                return GetParam((ParticlesMaterial.Parameter)0);
            }
            set
            {
                SetParam((ParticlesMaterial.Parameter)0, value);
            }
        }

        /// <summary>
        /// Initial velocity randomness ratio. Default value: [code]0[/code].
        /// </summary>
        public float InitialVelocityRandom
        {
            get
            {
                return GetParamRandomness((ParticlesMaterial.Parameter)0);
            }
            set
            {
                SetParamRandomness((ParticlesMaterial.Parameter)0, value);
            }
        }

        /// <summary>
        /// Initial angular velocity applied to each particle.
        /// </summary>
        public float AngularVelocity
        {
            get
            {
                return GetParam((ParticlesMaterial.Parameter)1);
            }
            set
            {
                SetParam((ParticlesMaterial.Parameter)1, value);
            }
        }

        /// <summary>
        /// Angular velocity randomness ratio. Default value: [code]0[/code].
        /// </summary>
        public float AngularVelocityRandom
        {
            get
            {
                return GetParamRandomness((ParticlesMaterial.Parameter)1);
            }
            set
            {
                SetParamRandomness((ParticlesMaterial.Parameter)1, value);
            }
        }

        /// <summary>
        /// Each particle's angular velocity will vary along this [CurveTexture].
        /// </summary>
        public Texture AngularVelocityCurve
        {
            get
            {
                return GetParamTexture((ParticlesMaterial.Parameter)1);
            }
            set
            {
                SetParamTexture((ParticlesMaterial.Parameter)1, value);
            }
        }

        /// <summary>
        /// Orbital velocity applied to each particle.
        /// </summary>
        public float OrbitVelocity
        {
            get
            {
                return GetParam((ParticlesMaterial.Parameter)2);
            }
            set
            {
                SetParam((ParticlesMaterial.Parameter)2, value);
            }
        }

        /// <summary>
        /// Orbital velocity randomness ratio. Default value: [code]0[/code].
        /// </summary>
        public float OrbitVelocityRandom
        {
            get
            {
                return GetParamRandomness((ParticlesMaterial.Parameter)2);
            }
            set
            {
                SetParamRandomness((ParticlesMaterial.Parameter)2, value);
            }
        }

        /// <summary>
        /// Each particle's orbital velocity will vary along this [CurveTexture].
        /// </summary>
        public Texture OrbitVelocityCurve
        {
            get
            {
                return GetParamTexture((ParticlesMaterial.Parameter)2);
            }
            set
            {
                SetParamTexture((ParticlesMaterial.Parameter)2, value);
            }
        }

        /// <summary>
        /// Linear acceleration applied to each particle.
        /// </summary>
        public float LinearAccel
        {
            get
            {
                return GetParam((ParticlesMaterial.Parameter)3);
            }
            set
            {
                SetParam((ParticlesMaterial.Parameter)3, value);
            }
        }

        /// <summary>
        /// Linear acceleration randomness ratio. Default value: [code]0[/code].
        /// </summary>
        public float LinearAccelRandom
        {
            get
            {
                return GetParamRandomness((ParticlesMaterial.Parameter)3);
            }
            set
            {
                SetParamRandomness((ParticlesMaterial.Parameter)3, value);
            }
        }

        /// <summary>
        /// Each particle's linear acceleration will vary along this [CurveTexture].
        /// </summary>
        public Texture LinearAccelCurve
        {
            get
            {
                return GetParamTexture((ParticlesMaterial.Parameter)3);
            }
            set
            {
                SetParamTexture((ParticlesMaterial.Parameter)3, value);
            }
        }

        /// <summary>
        /// Linear acceleration applied to each particle.
        /// </summary>
        public float RadialAccel
        {
            get
            {
                return GetParam((ParticlesMaterial.Parameter)4);
            }
            set
            {
                SetParam((ParticlesMaterial.Parameter)4, value);
            }
        }

        /// <summary>
        /// Radial acceleration randomness ratio. Default value: [code]0[/code].
        /// </summary>
        public float RadialAccelRandom
        {
            get
            {
                return GetParamRandomness((ParticlesMaterial.Parameter)4);
            }
            set
            {
                SetParamRandomness((ParticlesMaterial.Parameter)4, value);
            }
        }

        /// <summary>
        /// Each particle's radial acceleration will vary along this [CurveTexture].
        /// </summary>
        public Texture RadialAccelCurve
        {
            get
            {
                return GetParamTexture((ParticlesMaterial.Parameter)4);
            }
            set
            {
                SetParamTexture((ParticlesMaterial.Parameter)4, value);
            }
        }

        /// <summary>
        /// Tangential acceleration applied to each particle. Tangential acceleration is perpendicular to the particle's velocity.
        /// </summary>
        public float TangentialAccel
        {
            get
            {
                return GetParam((ParticlesMaterial.Parameter)5);
            }
            set
            {
                SetParam((ParticlesMaterial.Parameter)5, value);
            }
        }

        /// <summary>
        /// Tangential acceleration randomness ratio. Default value: [code]0[/code].
        /// </summary>
        public float TangentialAccelRandom
        {
            get
            {
                return GetParamRandomness((ParticlesMaterial.Parameter)5);
            }
            set
            {
                SetParamRandomness((ParticlesMaterial.Parameter)5, value);
            }
        }

        /// <summary>
        /// Each particle's tangential acceleration will vary along this [CurveTexture].
        /// </summary>
        public Texture TangentialAccelCurve
        {
            get
            {
                return GetParamTexture((ParticlesMaterial.Parameter)5);
            }
            set
            {
                SetParamTexture((ParticlesMaterial.Parameter)5, value);
            }
        }

        /// <summary>
        /// The rate at which particles lose velocity.
        /// </summary>
        public float Damping
        {
            get
            {
                return GetParam((ParticlesMaterial.Parameter)6);
            }
            set
            {
                SetParam((ParticlesMaterial.Parameter)6, value);
            }
        }

        /// <summary>
        /// Damping randomness ratio. Default value: [code]0[/code].
        /// </summary>
        public float DampingRandom
        {
            get
            {
                return GetParamRandomness((ParticlesMaterial.Parameter)6);
            }
            set
            {
                SetParamRandomness((ParticlesMaterial.Parameter)6, value);
            }
        }

        /// <summary>
        /// Damping will vary along this [CurveTexture].
        /// </summary>
        public Texture DampingCurve
        {
            get
            {
                return GetParamTexture((ParticlesMaterial.Parameter)6);
            }
            set
            {
                SetParamTexture((ParticlesMaterial.Parameter)6, value);
            }
        }

        /// <summary>
        /// Initial rotation applied to each particle.
        /// </summary>
        public float Angle
        {
            get
            {
                return GetParam((ParticlesMaterial.Parameter)7);
            }
            set
            {
                SetParam((ParticlesMaterial.Parameter)7, value);
            }
        }

        /// <summary>
        /// Rotation randomness ratio. Default value: [code]0[/code].
        /// </summary>
        public float AngleRandom
        {
            get
            {
                return GetParamRandomness((ParticlesMaterial.Parameter)7);
            }
            set
            {
                SetParamRandomness((ParticlesMaterial.Parameter)7, value);
            }
        }

        /// <summary>
        /// Each particle's rotation will be animated along this [CurveTexture].
        /// </summary>
        public Texture AngleCurve
        {
            get
            {
                return GetParamTexture((ParticlesMaterial.Parameter)7);
            }
            set
            {
                SetParamTexture((ParticlesMaterial.Parameter)7, value);
            }
        }

        /// <summary>
        /// Initial scale applied to each particle.
        /// </summary>
        public float Scale
        {
            get
            {
                return GetParam((ParticlesMaterial.Parameter)8);
            }
            set
            {
                SetParam((ParticlesMaterial.Parameter)8, value);
            }
        }

        /// <summary>
        /// Scale randomness ratio. Default value: [code]0[/code].
        /// </summary>
        public float ScaleRandom
        {
            get
            {
                return GetParamRandomness((ParticlesMaterial.Parameter)8);
            }
            set
            {
                SetParamRandomness((ParticlesMaterial.Parameter)8, value);
            }
        }

        /// <summary>
        /// Each particle's scale will vary along this [CurveTexture].
        /// </summary>
        public Texture ScaleCurve
        {
            get
            {
                return GetParamTexture((ParticlesMaterial.Parameter)8);
            }
            set
            {
                SetParamTexture((ParticlesMaterial.Parameter)8, value);
            }
        }

        /// <summary>
        /// Each particle's initial color. If the [Particle2D]'s [code]texture[/code] is defined, it will be multiplied by this color.
        /// </summary>
        public Color Color
        {
            get
            {
                return GetColor();
            }
            set
            {
                SetColor(value);
            }
        }

        /// <summary>
        /// Each particle's color will vary along this [GradientTexture].
        /// </summary>
        public Texture ColorRamp
        {
            get
            {
                return GetColorRamp();
            }
            set
            {
                SetColorRamp(value);
            }
        }

        /// <summary>
        /// Initial hue variation applied to each particle.
        /// </summary>
        public float HueVariation
        {
            get
            {
                return GetParam((ParticlesMaterial.Parameter)9);
            }
            set
            {
                SetParam((ParticlesMaterial.Parameter)9, value);
            }
        }

        /// <summary>
        /// Hue variation randomness ratio. Default value: [code]0[/code].
        /// </summary>
        public float HueVariationRandom
        {
            get
            {
                return GetParamRandomness((ParticlesMaterial.Parameter)9);
            }
            set
            {
                SetParamRandomness((ParticlesMaterial.Parameter)9, value);
            }
        }

        /// <summary>
        /// Each particle's hue will vary along this [CurveTexture].
        /// </summary>
        public Texture HueVariationCurve
        {
            get
            {
                return GetParamTexture((ParticlesMaterial.Parameter)9);
            }
            set
            {
                SetParamTexture((ParticlesMaterial.Parameter)9, value);
            }
        }

        /// <summary>
        /// Particle animation speed.
        /// </summary>
        public float AnimSpeed
        {
            get
            {
                return GetParam((ParticlesMaterial.Parameter)10);
            }
            set
            {
                SetParam((ParticlesMaterial.Parameter)10, value);
            }
        }

        /// <summary>
        /// Animation speed randomness ratio. Default value: [code]0[/code].
        /// </summary>
        public float AnimSpeedRandom
        {
            get
            {
                return GetParamRandomness((ParticlesMaterial.Parameter)10);
            }
            set
            {
                SetParamRandomness((ParticlesMaterial.Parameter)10, value);
            }
        }

        /// <summary>
        /// Each particle's animation speed will vary along this [CurveTexture].
        /// </summary>
        public Texture AnimSpeedCurve
        {
            get
            {
                return GetParamTexture((ParticlesMaterial.Parameter)10);
            }
            set
            {
                SetParamTexture((ParticlesMaterial.Parameter)10, value);
            }
        }

        /// <summary>
        /// Particle animation offset.
        /// </summary>
        public float AnimOffset
        {
            get
            {
                return GetParam((ParticlesMaterial.Parameter)11);
            }
            set
            {
                SetParam((ParticlesMaterial.Parameter)11, value);
            }
        }

        /// <summary>
        /// Animation offset randomness ratio. Default value: [code]0[/code].
        /// </summary>
        public float AnimOffsetRandom
        {
            get
            {
                return GetParamRandomness((ParticlesMaterial.Parameter)11);
            }
            set
            {
                SetParamRandomness((ParticlesMaterial.Parameter)11, value);
            }
        }

        /// <summary>
        /// Each particle's animation offset will vary along this [CurveTexture].
        /// </summary>
        public Texture AnimOffsetCurve
        {
            get
            {
                return GetParamTexture((ParticlesMaterial.Parameter)11);
            }
            set
            {
                SetParamTexture((ParticlesMaterial.Parameter)11, value);
            }
        }

        /// <summary>
        /// If [code]true[/code] animation will loop. Default value: [code]false[/code].
        /// </summary>
        public bool AnimLoop
        {
            get
            {
                return GetFlag((ParticlesMaterial.Flags)3);
            }
            set
            {
                SetFlag((ParticlesMaterial.Flags)3, value);
            }
        }


        private const string nativeName = "ParticlesMaterial";

        public ParticlesMaterial() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ParticlesMaterial_Ctor(this);
        }

        internal ParticlesMaterial(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_spread");

        [GodotMethod("set_spread")]
        public void SetSpread(float degrees)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), degrees);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_spread");

        [GodotMethod("get_spread")]
        public float GetSpread()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_flatness");

        [GodotMethod("set_flatness")]
        public void SetFlatness(float amount)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_flatness");

        [GodotMethod("get_flatness")]
        public float GetFlatness()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_param");

        [GodotMethod("set_param")]
        public void SetParam(ParticlesMaterial.Parameter param, float value)
        {
            NativeCalls.godot_icall_2_586(method_bind_4, Object.GetPtr(this), param, value);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_param");

        [GodotMethod("get_param")]
        public float GetParam(ParticlesMaterial.Parameter param)
        {
            return NativeCalls.godot_icall_1_587(method_bind_5, Object.GetPtr(this), param);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_param_randomness");

        [GodotMethod("set_param_randomness")]
        public void SetParamRandomness(ParticlesMaterial.Parameter param, float randomness)
        {
            NativeCalls.godot_icall_2_586(method_bind_6, Object.GetPtr(this), param, randomness);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_param_randomness");

        [GodotMethod("get_param_randomness")]
        public float GetParamRandomness(ParticlesMaterial.Parameter param)
        {
            return NativeCalls.godot_icall_1_587(method_bind_7, Object.GetPtr(this), param);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_param_texture");

        [GodotMethod("set_param_texture")]
        public void SetParamTexture(ParticlesMaterial.Parameter param, Texture texture)
        {
            NativeCalls.godot_icall_2_588(method_bind_8, Object.GetPtr(this), param, Object.GetPtr(texture));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_param_texture");

        [GodotMethod("get_param_texture")]
        public Texture GetParamTexture(ParticlesMaterial.Parameter param)
        {
            return NativeCalls.godot_icall_1_589(method_bind_9, Object.GetPtr(this), param);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_color");

        [GodotMethod("set_color")]
        public void SetColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_10, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_color");

        [GodotMethod("get_color")]
        public Color GetColor()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_11, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_color_ramp");

        [GodotMethod("set_color_ramp")]
        public void SetColorRamp(Texture ramp)
        {
            NativeCalls.godot_icall_1_24(method_bind_12, Object.GetPtr(this), Object.GetPtr(ramp));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_color_ramp");

        [GodotMethod("get_color_ramp")]
        public Texture GetColorRamp()
        {
            return NativeCalls.godot_icall_0_119(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_flag");

        [GodotMethod("set_flag")]
        public void SetFlag(ParticlesMaterial.Flags flag, bool enable)
        {
            NativeCalls.godot_icall_2_590(method_bind_14, Object.GetPtr(this), flag, enable);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_flag");

        [GodotMethod("get_flag")]
        public bool GetFlag(ParticlesMaterial.Flags flag)
        {
            return NativeCalls.godot_icall_1_591(method_bind_15, Object.GetPtr(this), flag);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_emission_shape");

        [GodotMethod("set_emission_shape")]
        public void SetEmissionShape(ParticlesMaterial.EmissionShapeEnum shape)
        {
            NativeCalls.godot_icall_1_592(method_bind_16, Object.GetPtr(this), shape);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_emission_shape");

        [GodotMethod("get_emission_shape")]
        public ParticlesMaterial.EmissionShapeEnum GetEmissionShape()
        {
            return NativeCalls.godot_icall_0_593(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_emission_sphere_radius");

        [GodotMethod("set_emission_sphere_radius")]
        public void SetEmissionSphereRadius(float radius)
        {
            NativeCalls.godot_icall_1_10(method_bind_18, Object.GetPtr(this), radius);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_emission_sphere_radius");

        [GodotMethod("get_emission_sphere_radius")]
        public float GetEmissionSphereRadius()
        {
            return NativeCalls.godot_icall_0_9(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_emission_box_extents");

        [GodotMethod("set_emission_box_extents")]
        public void SetEmissionBoxExtents(Vector3 extents)
        {
            NativeCalls.godot_icall_1_104(method_bind_20, Object.GetPtr(this), ref extents);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_emission_box_extents");

        [GodotMethod("get_emission_box_extents")]
        public Vector3 GetEmissionBoxExtents()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_21, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_emission_point_texture");

        [GodotMethod("set_emission_point_texture")]
        public void SetEmissionPointTexture(Texture texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_22, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_emission_point_texture");

        [GodotMethod("get_emission_point_texture")]
        public Texture GetEmissionPointTexture()
        {
            return NativeCalls.godot_icall_0_119(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_emission_normal_texture");

        [GodotMethod("set_emission_normal_texture")]
        public void SetEmissionNormalTexture(Texture texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_24, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_emission_normal_texture");

        [GodotMethod("get_emission_normal_texture")]
        public Texture GetEmissionNormalTexture()
        {
            return NativeCalls.godot_icall_0_119(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_emission_color_texture");

        [GodotMethod("set_emission_color_texture")]
        public void SetEmissionColorTexture(Texture texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_26, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_emission_color_texture");

        [GodotMethod("get_emission_color_texture")]
        public Texture GetEmissionColorTexture()
        {
            return NativeCalls.godot_icall_0_119(method_bind_27, Object.GetPtr(this));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_emission_point_count");

        [GodotMethod("set_emission_point_count")]
        public void SetEmissionPointCount(int pointCount)
        {
            NativeCalls.godot_icall_1_0(method_bind_28, Object.GetPtr(this), pointCount);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_emission_point_count");

        [GodotMethod("get_emission_point_count")]
        public int GetEmissionPointCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_29, Object.GetPtr(this));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_trail_divisor");

        [GodotMethod("set_trail_divisor")]
        public void SetTrailDivisor(int divisor)
        {
            NativeCalls.godot_icall_1_0(method_bind_30, Object.GetPtr(this), divisor);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_trail_divisor");

        [GodotMethod("get_trail_divisor")]
        public int GetTrailDivisor()
        {
            return NativeCalls.godot_icall_0_1(method_bind_31, Object.GetPtr(this));
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_trail_size_modifier");

        [GodotMethod("set_trail_size_modifier")]
        public void SetTrailSizeModifier(CurveTexture texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_32, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_trail_size_modifier");

        [GodotMethod("get_trail_size_modifier")]
        public CurveTexture GetTrailSizeModifier()
        {
            return NativeCalls.godot_icall_0_594(method_bind_33, Object.GetPtr(this));
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_trail_color_modifier");

        [GodotMethod("set_trail_color_modifier")]
        public void SetTrailColorModifier(GradientTexture texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_34, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_trail_color_modifier");

        [GodotMethod("get_trail_color_modifier")]
        public GradientTexture GetTrailColorModifier()
        {
            return NativeCalls.godot_icall_0_595(method_bind_35, Object.GetPtr(this));
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_gravity");

        [GodotMethod("get_gravity")]
        public Vector3 GetGravity()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_36, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_gravity");

        [GodotMethod("set_gravity")]
        public void SetGravity(Vector3 accelVec)
        {
            NativeCalls.godot_icall_1_104(method_bind_37, Object.GetPtr(this), ref accelVec);
        }
    }
}
