using System;

namespace Godot
{
    /// <summary>
    /// This node takes its parent [Path], and returns the coordinates of a point within it, given a distance from the first vertex.
    /// It is useful for making other nodes follow a path, without coding the movement pattern. For that, the nodes must be descendants of this node. Then, when setting an offset in this node, the descendant nodes will move accordingly.
    /// </summary>
    public class PathFollow : Spatial
    {
        public enum RotationModeEnum
        {
            None = 0,
            Y = 1,
            Xy = 2,
            Xyz = 3
        }


        /// <summary>
        /// The distance from the first vertex, measured in 3D units along the path. This sets this node's position to a point within the path.
        /// </summary>
        public float Offset
        {
            get
            {
                return GetOffset();
            }
            set
            {
                SetOffset(value);
            }
        }

        /// <summary>
        /// The distance from the first vertex, considering 0.0 as the first vertex and 1.0 as the last. This is just another way of expressing the offset within the path, as the offset supplied is multiplied internally by the path's length.
        /// </summary>
        public float UnitOffset
        {
            get
            {
                return GetUnitOffset();
            }
            set
            {
                SetUnitOffset(value);
            }
        }

        /// <summary>
        /// The node's offset along the curve.
        /// </summary>
        public float HOffset
        {
            get
            {
                return GetHOffset();
            }
            set
            {
                SetHOffset(value);
            }
        }

        /// <summary>
        /// The node's offset perpendicular to the curve.
        /// </summary>
        public float VOffset
        {
            get
            {
                return GetVOffset();
            }
            set
            {
                SetVOffset(value);
            }
        }

        /// <summary>
        /// Allows or forbids rotation on one or more axes, depending on the constants being used.
        /// </summary>
        public PathFollow.RotationModeEnum RotationMode
        {
            get
            {
                return GetRotationMode();
            }
            set
            {
                SetRotationMode(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the position between two cached points is interpolated cubically, and linearly otherwise.
        /// The points along the [Curve3D] of the [Path] are precomputed before use, for faster calculations. The point at the requested offset is then calculated interpolating between two adjacent cached points. This may present a problem if the curve makes sharp turns, as the cached points may not follow the curve closely enough.
        /// There are two answers to this problem: Either increase the number of cached points and increase memory consumption, or make a cubic interpolation between two points at the cost of (slightly) slower calculations.
        /// </summary>
        public bool CubicInterp
        {
            get
            {
                return GetCubicInterpolation();
            }
            set
            {
                SetCubicInterpolation(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], any offset outside the path's length will wrap around, instead of stopping at the ends. Use it for cyclic paths.
        /// </summary>
        public bool Loop
        {
            get
            {
                return HasLoop();
            }
            set
            {
                SetLoop(value);
            }
        }


        private const string nativeName = "PathFollow";

        public PathFollow() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_PathFollow_Ctor(this);
        }

        internal PathFollow(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_offset");

        [GodotMethod("set_offset")]
        public void SetOffset(float offset)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), offset);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_offset");

        [GodotMethod("get_offset")]
        public float GetOffset()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_h_offset");

        [GodotMethod("set_h_offset")]
        public void SetHOffset(float hOffset)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), hOffset);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_h_offset");

        [GodotMethod("get_h_offset")]
        public float GetHOffset()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_v_offset");

        [GodotMethod("set_v_offset")]
        public void SetVOffset(float vOffset)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), vOffset);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_v_offset");

        [GodotMethod("get_v_offset")]
        public float GetVOffset()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_unit_offset");

        [GodotMethod("set_unit_offset")]
        public void SetUnitOffset(float unitOffset)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), unitOffset);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_unit_offset");

        [GodotMethod("get_unit_offset")]
        public float GetUnitOffset()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rotation_mode");

        [GodotMethod("set_rotation_mode")]
        public void SetRotationMode(PathFollow.RotationModeEnum rotationMode)
        {
            NativeCalls.godot_icall_1_598(method_bind_8, Object.GetPtr(this), rotationMode);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rotation_mode");

        [GodotMethod("get_rotation_mode")]
        public PathFollow.RotationModeEnum GetRotationMode()
        {
            return NativeCalls.godot_icall_0_599(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_cubic_interpolation");

        [GodotMethod("set_cubic_interpolation")]
        public void SetCubicInterpolation(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_10, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cubic_interpolation");

        [GodotMethod("get_cubic_interpolation")]
        public bool GetCubicInterpolation()
        {
            return NativeCalls.godot_icall_0_3(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_loop");

        [GodotMethod("set_loop")]
        public void SetLoop(bool loop)
        {
            NativeCalls.godot_icall_1_11(method_bind_12, Object.GetPtr(this), loop);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_loop");

        [GodotMethod("has_loop")]
        public bool HasLoop()
        {
            return NativeCalls.godot_icall_0_3(method_bind_13, Object.GetPtr(this));
        }
    }
}
