using System;

namespace Godot
{
    /// <summary>
    /// This node takes its parent [Path2D], and returns the coordinates of a point within it, given a distance from the first vertex.
    /// It is useful for making other nodes follow a path, without coding the movement pattern. For that, the nodes must be descendants of this node. Then, when setting an offset in this node, the descendant nodes will move accordingly.
    /// </summary>
    public class PathFollow2D : Node2D
    {
        /// <summary>
        /// The distance along the path in pixels.
        /// </summary>
        public float Offset
        {
            get
            {
                return GetOffset();
            }
            set
            {
                SetOffset(value);
            }
        }

        /// <summary>
        /// The distance along the path as a number in the range 0.0 (for the first vertex) to 1.0 (for the last). This is just another way of expressing the offset within the path, as the offset supplied is multiplied internally by the path's length.
        /// </summary>
        public float UnitOffset
        {
            get
            {
                return GetUnitOffset();
            }
            set
            {
                SetUnitOffset(value);
            }
        }

        /// <summary>
        /// The node's offset along the curve.
        /// </summary>
        public float HOffset
        {
            get
            {
                return GetHOffset();
            }
            set
            {
                SetHOffset(value);
            }
        }

        /// <summary>
        /// The node's offset perpendicular to the curve.
        /// </summary>
        public float VOffset
        {
            get
            {
                return GetVOffset();
            }
            set
            {
                SetVOffset(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], this node rotates to follow the path, making its descendants rotate.
        /// </summary>
        public bool Rotate
        {
            get
            {
                return IsRotating();
            }
            set
            {
                SetRotate(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the position between two cached points is interpolated cubically, and linearly otherwise.
        /// The points along the [Curve2D] of the [Path2D] are precomputed before use, for faster calculations. The point at the requested offset is then calculated interpolating between two adjacent cached points. This may present a problem if the curve makes sharp turns, as the cached points may not follow the curve closely enough.
        /// There are two answers to this problem: Either increase the number of cached points and increase memory consumption, or make a cubic interpolation between two points at the cost of (slightly) slower calculations.
        /// </summary>
        public bool CubicInterp
        {
            get
            {
                return GetCubicInterpolation();
            }
            set
            {
                SetCubicInterpolation(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], any offset outside the path's length will wrap around, instead of stopping at the ends. Use it for cyclic paths.
        /// </summary>
        public bool Loop
        {
            get
            {
                return HasLoop();
            }
            set
            {
                SetLoop(value);
            }
        }

        /// <summary>
        /// </summary>
        public float Lookahead
        {
            get
            {
                return GetLookahead();
            }
            set
            {
                SetLookahead(value);
            }
        }


        private const string nativeName = "PathFollow2D";

        public PathFollow2D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_PathFollow2D_Ctor(this);
        }

        internal PathFollow2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_offset");

        [GodotMethod("set_offset")]
        public void SetOffset(float offset)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), offset);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_offset");

        [GodotMethod("get_offset")]
        public float GetOffset()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_h_offset");

        [GodotMethod("set_h_offset")]
        public void SetHOffset(float hOffset)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), hOffset);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_h_offset");

        [GodotMethod("get_h_offset")]
        public float GetHOffset()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_v_offset");

        [GodotMethod("set_v_offset")]
        public void SetVOffset(float vOffset)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), vOffset);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_v_offset");

        [GodotMethod("get_v_offset")]
        public float GetVOffset()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_unit_offset");

        [GodotMethod("set_unit_offset")]
        public void SetUnitOffset(float unitOffset)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), unitOffset);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_unit_offset");

        [GodotMethod("get_unit_offset")]
        public float GetUnitOffset()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rotate");

        [GodotMethod("set_rotate")]
        public void SetRotate(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_8, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_rotating");

        [GodotMethod("is_rotating")]
        public bool IsRotating()
        {
            return NativeCalls.godot_icall_0_3(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_cubic_interpolation");

        [GodotMethod("set_cubic_interpolation")]
        public void SetCubicInterpolation(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_10, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cubic_interpolation");

        [GodotMethod("get_cubic_interpolation")]
        public bool GetCubicInterpolation()
        {
            return NativeCalls.godot_icall_0_3(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_loop");

        [GodotMethod("set_loop")]
        public void SetLoop(bool loop)
        {
            NativeCalls.godot_icall_1_11(method_bind_12, Object.GetPtr(this), loop);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_loop");

        [GodotMethod("has_loop")]
        public bool HasLoop()
        {
            return NativeCalls.godot_icall_0_3(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_lookahead");

        [GodotMethod("set_lookahead")]
        public void SetLookahead(float lookahead)
        {
            NativeCalls.godot_icall_1_10(method_bind_14, Object.GetPtr(this), lookahead);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_lookahead");

        [GodotMethod("get_lookahead")]
        public float GetLookahead()
        {
            return NativeCalls.godot_icall_0_9(method_bind_15, Object.GetPtr(this));
        }
    }
}
