using System;

namespace Godot
{
    /// <summary>
    /// Direct access object to a physics body in the [Physics2DServer]. This object is passed via the direct state callback of rigid/character bodies, and is intended for changing the direct state of that body.
    /// </summary>
    public class Physics2DDirectBodyState : Object
    {
        /// <summary>
        /// The timestep (delta) used for the simulation.
        /// </summary>
        public float Step
        {
            get
            {
                return GetStep();
            }
        }

        /// <summary>
        /// The inverse of the mass of the body.
        /// </summary>
        public float InverseMass
        {
            get
            {
                return GetInverseMass();
            }
        }

        /// <summary>
        /// The inverse of the inertia of the body.
        /// </summary>
        public float InverseInertia
        {
            get
            {
                return GetInverseInertia();
            }
        }

        /// <summary>
        /// The rate at which the body stops rotating, if there are not any other forces moving it.
        /// </summary>
        public float TotalAngularDamp
        {
            get
            {
                return GetTotalAngularDamp();
            }
        }

        /// <summary>
        /// The rate at which the body stops moving, if there are not any other forces moving it.
        /// </summary>
        public float TotalLinearDamp
        {
            get
            {
                return GetTotalLinearDamp();
            }
        }

        /// <summary>
        /// The total gravity vector being currently applied to this body.
        /// </summary>
        public Vector2 TotalGravity
        {
            get
            {
                return GetTotalGravity();
            }
        }

        /// <summary>
        /// The angular velocity of the body.
        /// </summary>
        public float AngularVelocity
        {
            get
            {
                return GetAngularVelocity();
            }
            set
            {
                SetAngularVelocity(value);
            }
        }

        /// <summary>
        /// The linear velocity of the body.
        /// </summary>
        public Vector2 LinearVelocity
        {
            get
            {
                return GetLinearVelocity();
            }
            set
            {
                SetLinearVelocity(value);
            }
        }

        /// <summary>
        /// [code]true[/code] if this body is currently sleeping (not active).
        /// </summary>
        public bool Sleeping
        {
            get
            {
                return IsSleeping();
            }
            set
            {
                SetSleepState(value);
            }
        }

        /// <summary>
        /// The transformation matrix of the body.
        /// </summary>
        public Transform2D Transform
        {
            get
            {
                return GetTransform();
            }
            set
            {
                SetTransform(value);
            }
        }


        private const string nativeName = "Physics2DDirectBodyState";

        internal Physics2DDirectBodyState() {}

        internal Physics2DDirectBodyState(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_total_gravity");

        [GodotMethod("get_total_gravity")]
        public Vector2 GetTotalGravity()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_0, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_total_linear_damp");

        [GodotMethod("get_total_linear_damp")]
        public float GetTotalLinearDamp()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_total_angular_damp");

        [GodotMethod("get_total_angular_damp")]
        public float GetTotalAngularDamp()
        {
            return NativeCalls.godot_icall_0_9(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_inverse_mass");

        [GodotMethod("get_inverse_mass")]
        public float GetInverseMass()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_inverse_inertia");

        [GodotMethod("get_inverse_inertia")]
        public float GetInverseInertia()
        {
            return NativeCalls.godot_icall_0_9(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_linear_velocity");

        [GodotMethod("set_linear_velocity")]
        public void SetLinearVelocity(Vector2 velocity)
        {
            NativeCalls.godot_icall_1_45(method_bind_5, Object.GetPtr(this), ref velocity);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_linear_velocity");

        [GodotMethod("get_linear_velocity")]
        public Vector2 GetLinearVelocity()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_6, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_angular_velocity");

        [GodotMethod("set_angular_velocity")]
        public void SetAngularVelocity(float velocity)
        {
            NativeCalls.godot_icall_1_10(method_bind_7, Object.GetPtr(this), velocity);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_angular_velocity");

        [GodotMethod("get_angular_velocity")]
        public float GetAngularVelocity()
        {
            return NativeCalls.godot_icall_0_9(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_transform");

        [GodotMethod("set_transform")]
        public void SetTransform(Transform2D transform)
        {
            NativeCalls.godot_icall_1_219(method_bind_9, Object.GetPtr(this), ref transform);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_transform");

        [GodotMethod("get_transform")]
        public Transform2D GetTransform()
        {
            object ret = NativeCalls.godot_icall_0_220(method_bind_10, Object.GetPtr(this));
            return (Transform2D)ret;
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_sleep_state");

        [GodotMethod("set_sleep_state")]
        public void SetSleepState(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_11, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_sleeping");

        [GodotMethod("is_sleeping")]
        public bool IsSleeping()
        {
            return NativeCalls.godot_icall_0_3(method_bind_12, Object.GetPtr(this));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_count");

        /// <summary>
        /// Return the amount of contacts this body has with other bodies. Note that by default this returns 0 unless bodies are configured to log contacts.
        /// </summary>
        [GodotMethod("get_contact_count")]
        public int GetContactCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_local_position");

        /// <summary>
        /// Return the local position (of this body) of the contact point.
        /// </summary>
        [GodotMethod("get_contact_local_position")]
        public Vector2 GetContactLocalPosition(int contactIdx)
        {
            object ret = NativeCalls.godot_icall_1_282(method_bind_14, Object.GetPtr(this), contactIdx);
            return (Vector2)ret;
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_local_normal");

        /// <summary>
        /// Return the local normal (of this body) of the contact point.
        /// </summary>
        [GodotMethod("get_contact_local_normal")]
        public Vector2 GetContactLocalNormal(int contactIdx)
        {
            object ret = NativeCalls.godot_icall_1_282(method_bind_15, Object.GetPtr(this), contactIdx);
            return (Vector2)ret;
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_local_shape");

        /// <summary>
        /// Return the local shape index of the collision.
        /// </summary>
        [GodotMethod("get_contact_local_shape")]
        public int GetContactLocalShape(int contactIdx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_16, Object.GetPtr(this), contactIdx);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_collider");

        /// <summary>
        /// Return the [RID] of the collider.
        /// </summary>
        [GodotMethod("get_contact_collider")]
        public RID GetContactCollider(int contactIdx)
        {
            IntPtr ret = NativeCalls.godot_icall_1_402(method_bind_17, Object.GetPtr(this), contactIdx);
            return new RID(ret);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_collider_position");

        /// <summary>
        /// Return the contact position in the collider.
        /// </summary>
        [GodotMethod("get_contact_collider_position")]
        public Vector2 GetContactColliderPosition(int contactIdx)
        {
            object ret = NativeCalls.godot_icall_1_282(method_bind_18, Object.GetPtr(this), contactIdx);
            return (Vector2)ret;
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_collider_id");

        /// <summary>
        /// Return the object id of the collider.
        /// </summary>
        [GodotMethod("get_contact_collider_id")]
        public int GetContactColliderId(int contactIdx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_19, Object.GetPtr(this), contactIdx);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_collider_object");

        /// <summary>
        /// Return the collider object, this depends on how it was created (will return a scene node if such was used to create it).
        /// </summary>
        [GodotMethod("get_contact_collider_object")]
        public Object GetContactColliderObject(int contactIdx)
        {
            return NativeCalls.godot_icall_1_232(method_bind_20, Object.GetPtr(this), contactIdx);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_collider_shape");

        /// <summary>
        /// Return the collider shape index.
        /// </summary>
        [GodotMethod("get_contact_collider_shape")]
        public int GetContactColliderShape(int contactIdx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_21, Object.GetPtr(this), contactIdx);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_collider_shape_metadata");

        /// <summary>
        /// Return the metadata of the collided shape. This metadata is different from [method Object.get_meta], and is set with [method Physics2DServer.shape_set_data].
        /// </summary>
        [GodotMethod("get_contact_collider_shape_metadata")]
        public object GetContactColliderShapeMetadata(int contactIdx)
        {
            return NativeCalls.godot_icall_1_454(method_bind_22, Object.GetPtr(this), contactIdx);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_contact_collider_velocity_at_position");

        /// <summary>
        /// Return the linear velocity vector at contact point of the collider.
        /// </summary>
        [GodotMethod("get_contact_collider_velocity_at_position")]
        public Vector2 GetContactColliderVelocityAtPosition(int contactIdx)
        {
            object ret = NativeCalls.godot_icall_1_282(method_bind_23, Object.GetPtr(this), contactIdx);
            return (Vector2)ret;
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_step");

        [GodotMethod("get_step")]
        public float GetStep()
        {
            return NativeCalls.godot_icall_0_9(method_bind_24, Object.GetPtr(this));
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "integrate_forces");

        /// <summary>
        /// Call the built-in force integration code.
        /// </summary>
        [GodotMethod("integrate_forces")]
        public void IntegrateForces()
        {
            NativeCalls.godot_icall_0_12(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_space_state");

        /// <summary>
        /// Return the current state of space, useful for queries.
        /// </summary>
        [GodotMethod("get_space_state")]
        public Physics2DDirectSpaceState GetSpaceState()
        {
            return NativeCalls.godot_icall_0_601(method_bind_26, Object.GetPtr(this));
        }
    }
}
