using System;
using System.Collections.Generic;

namespace Godot
{
    /// <summary>
    /// Direct access object to a space in the [Physics2DServer]. It's used mainly to do queries against objects and areas residing in a given space.
    /// </summary>
    public class Physics2DDirectSpaceState : Object
    {
        private const string nativeName = "Physics2DDirectSpaceState";

        internal Physics2DDirectSpaceState() {}

        internal Physics2DDirectSpaceState(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "intersect_point");

        /// <summary>
        /// Checks whether a point is inside any shape. The shapes the point is inside of are returned in an array containing dictionaries with the following fields:
        /// [code]collider[/code]: The colliding object.
        /// [code]collider_id[/code]: The colliding object's ID.
        /// [code]metadata[/code]: The intersecting shape's metadata. This metadata is different from [method Object.get_meta], and is set with [method Physics2DServer.shape_set_data].
        /// [code]rid[/code]: The intersecting object's [RID].
        /// [code]shape[/code]: The shape index of the colliding shape.
        /// Additionally, the method can take an array of objects or [RID]s that are to be excluded from collisions, or a bitmask representing the physics layers to check in.
        /// &lt;param name="exclude"&gt;If the param is null, then the default value is new object[] {}&lt;/param&gt;
        /// </summary>
        [GodotMethod("intersect_point")]
        public object[] IntersectPoint(Vector2 point, int maxResults = 32, object[] exclude = null, int collisionLayer = 2147483647)
        {
            object[] exclude_in = exclude != null ? exclude : new object[] {};
            return NativeCalls.godot_icall_4_602(method_bind_0, Object.GetPtr(this), ref point, maxResults, exclude_in, collisionLayer);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "intersect_ray");

        /// <summary>
        /// Intersects a ray in a given space. The returned object is a dictionary with the following fields:
        /// [code]collider[/code]: The colliding object.
        /// [code]collider_id[/code]: The colliding object's ID.
        /// [code]metadata[/code]: The intersecting shape's metadata. This metadata is different from [method Object.get_meta], and is set with [method Physics2DServer.shape_set_data].
        /// [code]normal[/code]: The object's surface normal at the intersection point.
        /// [code]position[/code]: The intersection point.
        /// [code]rid[/code]: The intersecting object's [RID].
        /// [code]shape[/code]: The shape index of the colliding shape.
        /// If the ray did not intersect anything, then an empty dictionary ([code]dir.empty()==true[/code]) is returned instead.
        /// Additionally, the method can take an array of objects or [RID]s that are to be excluded from collisions, or a bitmask representing the physics layers to check in.
        /// &lt;param name="exclude"&gt;If the param is null, then the default value is new object[] {}&lt;/param&gt;
        /// </summary>
        [GodotMethod("intersect_ray")]
        public Dictionary<object, object> IntersectRay(Vector2 from, Vector2 to, object[] exclude = null, int collisionLayer = 2147483647)
        {
            object[] exclude_in = exclude != null ? exclude : new object[] {};
            return NativeCalls.godot_icall_4_603(method_bind_1, Object.GetPtr(this), ref from, ref to, exclude_in, collisionLayer);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "intersect_shape");

        /// <summary>
        /// Checks the intersections of a shape, given through a [Physics2DShapeQueryParameters] object, against the space. Note that this method does not take into account the [code]motion[/code] property of the object. The intersected shapes are returned in an array containing dictionaries with the following fields:
        /// [code]collider[/code]: The colliding object.
        /// [code]collider_id[/code]: The colliding object's ID.
        /// [code]metadata[/code]: The intersecting shape's metadata. This metadata is different from [method Object.get_meta], and is set with [method Physics2DServer.shape_set_data].
        /// [code]rid[/code]: The intersecting object's [RID].
        /// [code]shape[/code]: The shape index of the colliding shape.
        /// The number of intersections can be limited with the second parameter, to reduce the processing time.
        /// </summary>
        [GodotMethod("intersect_shape")]
        public object[] IntersectShape(Physics2DShapeQueryParameters shape, int maxResults = 32)
        {
            return NativeCalls.godot_icall_2_604(method_bind_2, Object.GetPtr(this), Object.GetPtr(shape), maxResults);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "cast_motion");

        /// <summary>
        /// Checks how far the shape can travel toward a point. Note that both the shape and the motion are supplied through a [Physics2DShapeQueryParameters] object. The method will return an array with two floats between 0 and 1, both representing a fraction of [code]motion[/code]. The first is how far the shape can move without triggering a collision, and the second is the point at which a collision will occur. If no collision is detected, the returned array will be [1, 1].
        /// If the shape can not move, the array will be empty ([code]dir.empty()==true[/code]).
        /// </summary>
        [GodotMethod("cast_motion")]
        public object[] CastMotion(Physics2DShapeQueryParameters shape)
        {
            return NativeCalls.godot_icall_1_605(method_bind_3, Object.GetPtr(this), Object.GetPtr(shape));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "collide_shape");

        /// <summary>
        /// Checks the intersections of a shape, given through a [Physics2DShapeQueryParameters] object, against the space. The resulting array contains a list of points where the shape intersects another. Like with [method intersect_shape], the number of returned results can be limited to save processing time.
        /// </summary>
        [GodotMethod("collide_shape")]
        public object[] CollideShape(Physics2DShapeQueryParameters shape, int maxResults = 32)
        {
            return NativeCalls.godot_icall_2_604(method_bind_4, Object.GetPtr(this), Object.GetPtr(shape), maxResults);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rest_info");

        /// <summary>
        /// Checks the intersections of a shape, given through a [Physics2DShapeQueryParameters] object, against the space. If it collides with more than one shape, the nearest one is selected. Note that this method does not take into account the [code]motion[/code] property of the object. The returned object is a dictionary containing the following fields:
        /// [code]collider_id[/code]: The colliding object's ID.
        /// [code]linear_velocity[/code]: The colliding object's velocity [Vector2]. If the object is an [Area2D], the result is [code](0, 0)[/code].
        /// [code]metadata[/code]: The intersecting shape's metadata. This metadata is different from [method Object.get_meta], and is set with [method Physics2DServer.shape_set_data].
        /// [code]normal[/code]: The object's surface normal at the intersection point.
        /// [code]point[/code]: The intersection point.
        /// [code]rid[/code]: The intersecting object's [RID].
        /// [code]shape[/code]: The shape index of the colliding shape.
        /// If the shape did not intersect anything, then an empty dictionary ([code]dir.empty()==true[/code]) is returned instead.
        /// </summary>
        [GodotMethod("get_rest_info")]
        public Dictionary<object, object> GetRestInfo(Physics2DShapeQueryParameters shape)
        {
            return NativeCalls.godot_icall_1_606(method_bind_5, Object.GetPtr(this), Object.GetPtr(shape));
        }
    }
}
