using System;

namespace Godot
{
    /// <summary>
    /// PhysicsBody2D is an abstract base class for implementing a physics body. All *Body2D types inherit from it.
    /// </summary>
    public class PhysicsBody2D : CollisionObject2D
    {
        /// <summary>
        /// Both collision_layer and collision_mask. Returns collision_layer when accessed. Updates collision_layers and collision_mask when modified.
        /// </summary>
        public int Layers
        {
            get
            {
                return _GetLayers();
            }
            set
            {
                _SetLayers(value);
            }
        }

        /// <summary>
        /// The physics layers this area is in.
        /// Collidable objects can exist in any of 32 different layers. These layers work like a tagging system, and are not visual. A collidable can use these layers to select with which objects it can collide, using the collision_mask property.
        /// A contact is detected if object A is in any of the layers that object B scans, or object B is in any layer scanned by object A.
        /// </summary>
        public int CollisionLayer
        {
            get
            {
                return GetCollisionLayer();
            }
            set
            {
                SetCollisionLayer(value);
            }
        }

        /// <summary>
        /// The physics layers this area can scan for collisions.
        /// </summary>
        public int CollisionMask
        {
            get
            {
                return GetCollisionMask();
            }
            set
            {
                SetCollisionMask(value);
            }
        }


        private const string nativeName = "PhysicsBody2D";

        internal PhysicsBody2D() {}

        internal PhysicsBody2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_layer");

        [GodotMethod("set_collision_layer")]
        public void SetCollisionLayer(int layer)
        {
            NativeCalls.godot_icall_1_0(method_bind_0, Object.GetPtr(this), layer);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_layer");

        [GodotMethod("get_collision_layer")]
        public int GetCollisionLayer()
        {
            return NativeCalls.godot_icall_0_1(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_mask");

        [GodotMethod("set_collision_mask")]
        public void SetCollisionMask(int mask)
        {
            NativeCalls.godot_icall_1_0(method_bind_2, Object.GetPtr(this), mask);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_mask");

        [GodotMethod("get_collision_mask")]
        public int GetCollisionMask()
        {
            return NativeCalls.godot_icall_0_1(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_mask_bit");

        /// <summary>
        /// Set/clear individual bits on the collision mask. This makes selecting the areas scanned easier.
        /// </summary>
        [GodotMethod("set_collision_mask_bit")]
        public void SetCollisionMaskBit(int bit, bool value)
        {
            NativeCalls.godot_icall_2_51(method_bind_4, Object.GetPtr(this), bit, value);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_mask_bit");

        /// <summary>
        /// Return an individual bit on the collision mask.
        /// </summary>
        [GodotMethod("get_collision_mask_bit")]
        public bool GetCollisionMaskBit(int bit)
        {
            return NativeCalls.godot_icall_1_29(method_bind_5, Object.GetPtr(this), bit);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_layer_bit");

        /// <summary>
        /// Set/clear individual bits on the layer mask. This makes getting a body in/out of only one layer easier.
        /// </summary>
        [GodotMethod("set_collision_layer_bit")]
        public void SetCollisionLayerBit(int bit, bool value)
        {
            NativeCalls.godot_icall_2_51(method_bind_6, Object.GetPtr(this), bit, value);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_layer_bit");

        /// <summary>
        /// Return an individual bit on the collision mask.
        /// </summary>
        [GodotMethod("get_collision_layer_bit")]
        public bool GetCollisionLayerBit(int bit)
        {
            return NativeCalls.godot_icall_1_29(method_bind_7, Object.GetPtr(this), bit);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_layers");

        internal void _SetLayers(int mask)
        {
            NativeCalls.godot_icall_1_0(method_bind_8, Object.GetPtr(this), mask);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_layers");

        internal int _GetLayers()
        {
            return NativeCalls.godot_icall_0_1(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_collision_exception_with");

        /// <summary>
        /// Adds a body to the list of bodies that this body can't collide with.
        /// </summary>
        [GodotMethod("add_collision_exception_with")]
        public void AddCollisionExceptionWith(Node body)
        {
            NativeCalls.godot_icall_1_42(method_bind_10, Object.GetPtr(this), Object.GetPtr(body));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_collision_exception_with");

        /// <summary>
        /// Removes a body from the list of bodies that this body can't collide with.
        /// </summary>
        [GodotMethod("remove_collision_exception_with")]
        public void RemoveCollisionExceptionWith(Node body)
        {
            NativeCalls.godot_icall_1_42(method_bind_11, Object.GetPtr(this), Object.GetPtr(body));
        }
    }
}
