using System;

namespace Godot
{
    /// <summary>
    /// A Polygon2D is defined by a set of points. Each point is connected to the next, with the final point being connected to the first, resulting in a closed polygon. Polygon2Ds can be filled with color (solid or gradient) or filled with a given texture.
    /// </summary>
    public class Polygon2D : Node2D
    {
        /// <summary>
        /// The polygon's list of vertices. The final point will be connected to the first.
        /// </summary>
        public Vector2[] Polygon
        {
            get
            {
                return GetPolygon();
            }
            set
            {
                SetPolygon(value);
            }
        }

        /// <summary>
        /// Texture coordinates for each vertex of the polygon. There should be one [code]uv[/code] per polygon vertex. If there are fewer, undefined vertices will use [code](0, 0)[/code].
        /// </summary>
        public Vector2[] Uv
        {
            get
            {
                return GetUv();
            }
            set
            {
                SetUv(value);
            }
        }

        /// <summary>
        /// The polygon's fill color. If [code]texture[/code] is defined, it will be multiplied by this color. It will also be the default color for vertices not set in [code]vertex_colors[/code].
        /// </summary>
        public Color Color
        {
            get
            {
                return GetColor();
            }
            set
            {
                SetColor(value);
            }
        }

        /// <summary>
        /// Color for each vertex. Colors are interpolated between vertices, resulting in smooth gradients. There should be one per polygon vertex. If there are fewer, undefined vertices will use [code]color[/code].
        /// </summary>
        public Color[] VertexColors
        {
            get
            {
                return GetVertexColors();
            }
            set
            {
                SetVertexColors(value);
            }
        }

        /// <summary>
        /// The offset applied to each vertex.
        /// </summary>
        public Vector2 Offset
        {
            get
            {
                return GetOffset();
            }
            set
            {
                SetOffset(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] polygon edges will be anti-aliased. Default value: [code]false[/code].
        /// </summary>
        public bool Antialiased
        {
            get
            {
                return GetAntialiased();
            }
            set
            {
                SetAntialiased(value);
            }
        }

        /// <summary>
        /// The polygon's fill texture. Use [code]uv[/code] to set texture coordinates.
        /// </summary>
        public Texture Texture
        {
            get
            {
                return GetTexture();
            }
            set
            {
                SetTexture(value);
            }
        }

        /// <summary>
        /// Amount to offset the polygon's [code]texture[/code]. If [code](0, 0)[/code] the texture's origin (its top-left corner) will be placed at the polygon's [code]position[/code].
        /// </summary>
        public Vector2 TextureOffset
        {
            get
            {
                return GetTextureOffset();
            }
            set
            {
                SetTextureOffset(value);
            }
        }

        /// <summary>
        /// Amount to multiply the [code]uv[/code] coordinates when using a [code]texture[/code]. Larger values make the texture smaller, and vice versa.
        /// </summary>
        public Vector2 TextureScale
        {
            get
            {
                return GetTextureScale();
            }
            set
            {
                SetTextureScale(value);
            }
        }

        /// <summary>
        /// The texture's rotation in degrees.
        /// </summary>
        public float TextureRotationDegrees
        {
            get
            {
                return GetTextureRotationDegrees();
            }
            set
            {
                SetTextureRotationDegrees(value);
            }
        }

        /// <summary>
        /// The texture's rotation in radians.
        /// </summary>
        public float TextureRotation
        {
            get
            {
                return GetTextureRotation();
            }
            set
            {
                SetTextureRotation(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] polygon will be inverted, containing the area outside the defined points and extending to the [code]invert_border[/code]. Default value: [code]false[/code].
        /// </summary>
        public bool InvertEnable
        {
            get
            {
                return GetInvert();
            }
            set
            {
                SetInvert(value);
            }
        }

        /// <summary>
        /// Added padding applied to the bounding box when using [code]invert[/code]. Setting this value too small may result in a "Bad Polygon" error. Default value: [code]100[/code].
        /// </summary>
        public float InvertBorder
        {
            get
            {
                return GetInvertBorder();
            }
            set
            {
                SetInvertBorder(value);
            }
        }


        private const string nativeName = "Polygon2D";

        public Polygon2D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Polygon2D_Ctor(this);
        }

        internal Polygon2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_polygon");

        [GodotMethod("set_polygon")]
        public void SetPolygon(Vector2[] polygon)
        {
            NativeCalls.godot_icall_1_237(method_bind_0, Object.GetPtr(this), polygon);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_polygon");

        [GodotMethod("get_polygon")]
        public Vector2[] GetPolygon()
        {
            return NativeCalls.godot_icall_0_238(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_uv");

        [GodotMethod("set_uv")]
        public void SetUv(Vector2[] uv)
        {
            NativeCalls.godot_icall_1_237(method_bind_2, Object.GetPtr(this), uv);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_uv");

        [GodotMethod("get_uv")]
        public Vector2[] GetUv()
        {
            return NativeCalls.godot_icall_0_238(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_color");

        [GodotMethod("set_color")]
        public void SetColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_4, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_color");

        [GodotMethod("get_color")]
        public Color GetColor()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_5, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_vertex_colors");

        [GodotMethod("set_vertex_colors")]
        public void SetVertexColors(Color[] vertexColors)
        {
            NativeCalls.godot_icall_1_388(method_bind_6, Object.GetPtr(this), vertexColors);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_vertex_colors");

        [GodotMethod("get_vertex_colors")]
        public Color[] GetVertexColors()
        {
            return NativeCalls.godot_icall_0_389(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_texture");

        [GodotMethod("set_texture")]
        public void SetTexture(Texture texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_8, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture");

        [GodotMethod("get_texture")]
        public Texture GetTexture()
        {
            return NativeCalls.godot_icall_0_119(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_texture_offset");

        [GodotMethod("set_texture_offset")]
        public void SetTextureOffset(Vector2 textureOffset)
        {
            NativeCalls.godot_icall_1_45(method_bind_10, Object.GetPtr(this), ref textureOffset);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture_offset");

        [GodotMethod("get_texture_offset")]
        public Vector2 GetTextureOffset()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_11, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_texture_rotation");

        [GodotMethod("set_texture_rotation")]
        public void SetTextureRotation(float textureRotation)
        {
            NativeCalls.godot_icall_1_10(method_bind_12, Object.GetPtr(this), textureRotation);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture_rotation");

        [GodotMethod("get_texture_rotation")]
        public float GetTextureRotation()
        {
            return NativeCalls.godot_icall_0_9(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_texture_rotation_degrees");

        [GodotMethod("set_texture_rotation_degrees")]
        public void SetTextureRotationDegrees(float textureRotation)
        {
            NativeCalls.godot_icall_1_10(method_bind_14, Object.GetPtr(this), textureRotation);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture_rotation_degrees");

        [GodotMethod("get_texture_rotation_degrees")]
        public float GetTextureRotationDegrees()
        {
            return NativeCalls.godot_icall_0_9(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_texture_scale");

        [GodotMethod("set_texture_scale")]
        public void SetTextureScale(Vector2 textureScale)
        {
            NativeCalls.godot_icall_1_45(method_bind_16, Object.GetPtr(this), ref textureScale);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture_scale");

        [GodotMethod("get_texture_scale")]
        public Vector2 GetTextureScale()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_17, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_invert");

        [GodotMethod("set_invert")]
        public void SetInvert(bool invert)
        {
            NativeCalls.godot_icall_1_11(method_bind_18, Object.GetPtr(this), invert);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_invert");

        [GodotMethod("get_invert")]
        public bool GetInvert()
        {
            return NativeCalls.godot_icall_0_3(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_antialiased");

        [GodotMethod("set_antialiased")]
        public void SetAntialiased(bool antialiased)
        {
            NativeCalls.godot_icall_1_11(method_bind_20, Object.GetPtr(this), antialiased);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_antialiased");

        [GodotMethod("get_antialiased")]
        public bool GetAntialiased()
        {
            return NativeCalls.godot_icall_0_3(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_invert_border");

        [GodotMethod("set_invert_border")]
        public void SetInvertBorder(float invertBorder)
        {
            NativeCalls.godot_icall_1_10(method_bind_22, Object.GetPtr(this), invertBorder);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_invert_border");

        [GodotMethod("get_invert_border")]
        public float GetInvertBorder()
        {
            return NativeCalls.godot_icall_0_9(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_offset");

        [GodotMethod("set_offset")]
        public void SetOffset(Vector2 offset)
        {
            NativeCalls.godot_icall_1_45(method_bind_24, Object.GetPtr(this), ref offset);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_offset");

        [GodotMethod("get_offset")]
        public Vector2 GetOffset()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_25, Object.GetPtr(this));
            return (Vector2)ret;
        }
    }
}
