using System;

namespace Godot
{
    /// <summary>
    /// Range is a base class for [Control] nodes that change a floating point [i]value[/i] between a [i]minimum[/i] and a [i]maximum[/i], using [i]step[/i] and [i]page[/i], for example a [ScrollBar].
    /// </summary>
    public class Range : Control
    {
        /// <summary>
        /// Minimum value. Range is clamped if [code]value[/code] is less than [code]min_value[/code]. Default value: 0.
        /// </summary>
        public float MinValue
        {
            get
            {
                return GetMin();
            }
            set
            {
                SetMin(value);
            }
        }

        /// <summary>
        /// Maximum value. Range is clamped if [code]value[/code] is greater than [code]max_value[/code]. Default value: 100.
        /// </summary>
        public float MaxValue
        {
            get
            {
                return GetMax();
            }
            set
            {
                SetMax(value);
            }
        }

        /// <summary>
        /// If greater than 0, [code]value[/code] will always be rounded to a multiple of [code]step[/code]. If [code]rounded[/code] is also [code]true[/code], [code]value[/code] will first be rounded to a multiple of [code]step[/code] then rounded to the nearest integer.
        /// </summary>
        public float Step
        {
            get
            {
                return GetStep();
            }
            set
            {
                SetStep(value);
            }
        }

        /// <summary>
        /// Page size. Used mainly for [ScrollBar]. ScrollBar's length is its size multiplied by [code]page[/code] over the difference between [code]min_value[/code] and [code]max_value[/code].
        /// </summary>
        public float Page
        {
            get
            {
                return GetPage();
            }
            set
            {
                SetPage(value);
            }
        }

        /// <summary>
        /// Range's current value.
        /// </summary>
        public float Value
        {
            get
            {
                return GetValue();
            }
            set
            {
                SetValue(value);
            }
        }

        /// <summary>
        /// The value mapped between 0 and 1.
        /// </summary>
        public float Ratio
        {
            get
            {
                return GetAsRatio();
            }
            set
            {
                SetAsRatio(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] and [code]min_value[/code] is greater than 0, [code]value[/code] will be represented exponentially rather than linearly.
        /// </summary>
        public bool ExpEdit
        {
            get
            {
                return IsRatioExp();
            }
            set
            {
                SetExpRatio(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], [code]value[/code] will always be rounded to the nearest integer.
        /// </summary>
        public bool Rounded
        {
            get
            {
                return IsUsingRoundedValues();
            }
            set
            {
                SetUseRoundedValues(value);
            }
        }


        private const string nativeName = "Range";

        internal Range() {}

        internal Range(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_value");

        [GodotMethod("get_value")]
        public float GetValue()
        {
            return NativeCalls.godot_icall_0_9(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_min");

        [GodotMethod("get_min")]
        public float GetMin()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_max");

        [GodotMethod("get_max")]
        public float GetMax()
        {
            return NativeCalls.godot_icall_0_9(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_step");

        [GodotMethod("get_step")]
        public float GetStep()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_page");

        [GodotMethod("get_page")]
        public float GetPage()
        {
            return NativeCalls.godot_icall_0_9(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_as_ratio");

        [GodotMethod("get_as_ratio")]
        public float GetAsRatio()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_value");

        [GodotMethod("set_value")]
        public void SetValue(float value)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), value);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_min");

        [GodotMethod("set_min")]
        public void SetMin(float minimum)
        {
            NativeCalls.godot_icall_1_10(method_bind_7, Object.GetPtr(this), minimum);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_max");

        [GodotMethod("set_max")]
        public void SetMax(float maximum)
        {
            NativeCalls.godot_icall_1_10(method_bind_8, Object.GetPtr(this), maximum);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_step");

        [GodotMethod("set_step")]
        public void SetStep(float step)
        {
            NativeCalls.godot_icall_1_10(method_bind_9, Object.GetPtr(this), step);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_page");

        [GodotMethod("set_page")]
        public void SetPage(float pagesize)
        {
            NativeCalls.godot_icall_1_10(method_bind_10, Object.GetPtr(this), pagesize);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_as_ratio");

        [GodotMethod("set_as_ratio")]
        public void SetAsRatio(float value)
        {
            NativeCalls.godot_icall_1_10(method_bind_11, Object.GetPtr(this), value);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_use_rounded_values");

        [GodotMethod("set_use_rounded_values")]
        public void SetUseRoundedValues(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_12, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_using_rounded_values");

        [GodotMethod("is_using_rounded_values")]
        public bool IsUsingRoundedValues()
        {
            return NativeCalls.godot_icall_0_3(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_exp_ratio");

        [GodotMethod("set_exp_ratio")]
        public void SetExpRatio(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_14, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_ratio_exp");

        [GodotMethod("is_ratio_exp")]
        public bool IsRatioExp()
        {
            return NativeCalls.godot_icall_0_3(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "share");

        /// <summary>
        /// Binds two Ranges together along with any Ranges previously grouped with either of them. When any of Range's member variables change, it will share the new value with all other Ranges in its group.
        /// </summary>
        [GodotMethod("share")]
        public void Share(Node with)
        {
            NativeCalls.godot_icall_1_42(method_bind_16, Object.GetPtr(this), Object.GetPtr(with));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "unshare");

        /// <summary>
        /// Stop Range from sharing its member variables with any other Range.
        /// </summary>
        [GodotMethod("unshare")]
        public void Unshare()
        {
            NativeCalls.godot_icall_0_12(method_bind_17, Object.GetPtr(this));
        }
    }
}
