using System;

namespace Godot
{
    /// <summary>
    /// A RayCast represents a line from its origin to its destination position, [code]cast_to[/code]. It is used to query the 2D space in order to find the closest object along the path of the ray.
    /// RayCast2D can ignore some objects by adding them to the exception list via [code]add_exception[/code], by setting proper filtering with collision layers, or by filtering object types with type masks.
    /// Only enabled raycasts will be able to query the space and report collisions.
    /// RayCast2D calculates intersection every physics frame (see [Node]), and the result is cached so it can be used later until the next frame. If multiple queries are required between physics frames (or during the same frame) use [method force_raycast_update] after adjusting the raycast.
    /// </summary>
    public class RayCast2D : Node2D
    {
        /// <summary>
        /// If [code]true[/code], collisions will be reported. Default value: [code]false[/code].
        /// </summary>
        public bool Enabled
        {
            get
            {
                return IsEnabled();
            }
            set
            {
                SetEnabled(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], the parent node will be excluded from collision detection. Default value: [code]true[/code].
        /// </summary>
        public bool ExcludeParent
        {
            get
            {
                return GetExcludeParentBody();
            }
            set
            {
                SetExcludeParentBody(value);
            }
        }

        /// <summary>
        /// The ray's destination point, relative to the RayCast's [code]position[/code].
        /// </summary>
        public Vector2 CastTo
        {
            get
            {
                return GetCastTo();
            }
            set
            {
                SetCastTo(value);
            }
        }

        /// <summary>
        /// The ray's collision mask. Only objects in at least one collision layer enabled in the mask will be detected.
        /// </summary>
        public int CollisionMask
        {
            get
            {
                return GetCollisionMask();
            }
            set
            {
                SetCollisionMask(value);
            }
        }


        private const string nativeName = "RayCast2D";

        public RayCast2D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_RayCast2D_Ctor(this);
        }

        internal RayCast2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_enabled");

        [GodotMethod("set_enabled")]
        public void SetEnabled(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_0, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_enabled");

        [GodotMethod("is_enabled")]
        public bool IsEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_cast_to");

        [GodotMethod("set_cast_to")]
        public void SetCastTo(Vector2 localPoint)
        {
            NativeCalls.godot_icall_1_45(method_bind_2, Object.GetPtr(this), ref localPoint);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cast_to");

        [GodotMethod("get_cast_to")]
        public Vector2 GetCastTo()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_3, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_colliding");

        /// <summary>
        /// Return whether the closest object the ray is pointing to is colliding with the vector (considering the vector length).
        /// </summary>
        [GodotMethod("is_colliding")]
        public bool IsColliding()
        {
            return NativeCalls.godot_icall_0_3(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "force_raycast_update");

        /// <summary>
        /// Updates the collision information for the ray.  Use this method to update the collision information immediately instead of waiting for the next [code]_physics_process[/code] call, for example if the ray or its parent has changed state. Note: [code]enabled == true[/code] is not required for this to work.
        /// </summary>
        [GodotMethod("force_raycast_update")]
        public void ForceRaycastUpdate()
        {
            NativeCalls.godot_icall_0_12(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collider");

        /// <summary>
        /// Returns the closest object the ray is pointing to. Note that this does not consider the length of the ray, so you must also use [method is_colliding] to check if the object returned is actually colliding with the ray.
        /// Example:
        /// [codeblock]
        /// if RayCast2D.is_colliding():
        /// var collider = RayCast2D.get_collider()
        /// [/codeblock]
        /// </summary>
        [GodotMethod("get_collider")]
        public Object GetCollider()
        {
            return NativeCalls.godot_icall_0_475(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collider_shape");

        /// <summary>
        /// Returns the collision shape of the closest object the ray is pointing to.  Note that this does not consider the length of the ray, so you must also use [method is_colliding] to check if the object returned is actually colliding with the ray.
        /// Example:
        /// [codeblock]
        /// if RayCast2D.is_colliding():
        /// var shape = RayCast2D.get_collider_shape()
        /// [/codeblock]
        /// </summary>
        [GodotMethod("get_collider_shape")]
        public int GetColliderShape()
        {
            return NativeCalls.godot_icall_0_1(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_point");

        /// <summary>
        /// Returns the collision point at which the ray intersects the closest object. Note: this point is in the [b]global[/b] coordinate system.
        /// </summary>
        [GodotMethod("get_collision_point")]
        public Vector2 GetCollisionPoint()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_8, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_normal");

        /// <summary>
        /// Returns the normal of the intersecting object's shape at the collision point.
        /// </summary>
        [GodotMethod("get_collision_normal")]
        public Vector2 GetCollisionNormal()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_9, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_exception_rid");

        /// <summary>
        /// Adds a collision exception so the ray does not report collisions with the specified [RID].
        /// </summary>
        [GodotMethod("add_exception_rid")]
        public void AddExceptionRid(RID rid)
        {
            NativeCalls.godot_icall_1_627(method_bind_10, Object.GetPtr(this), RID.GetPtr(rid));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_exception");

        /// <summary>
        /// Adds a collision exception so the ray does not report collisions with the specified node.
        /// </summary>
        [GodotMethod("add_exception")]
        public void AddException(Object node)
        {
            NativeCalls.godot_icall_1_42(method_bind_11, Object.GetPtr(this), Object.GetPtr(node));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_exception_rid");

        /// <summary>
        /// Removes a collision exception so the ray does report collisions with the specified [RID].
        /// </summary>
        [GodotMethod("remove_exception_rid")]
        public void RemoveExceptionRid(RID rid)
        {
            NativeCalls.godot_icall_1_627(method_bind_12, Object.GetPtr(this), RID.GetPtr(rid));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_exception");

        /// <summary>
        /// Removes a collision exception so the ray does report collisions with the specified node.
        /// </summary>
        [GodotMethod("remove_exception")]
        public void RemoveException(Object node)
        {
            NativeCalls.godot_icall_1_42(method_bind_13, Object.GetPtr(this), Object.GetPtr(node));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_exceptions");

        /// <summary>
        /// Removes all collision exceptions for this ray.
        /// </summary>
        [GodotMethod("clear_exceptions")]
        public void ClearExceptions()
        {
            NativeCalls.godot_icall_0_12(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_mask");

        [GodotMethod("set_collision_mask")]
        public void SetCollisionMask(int mask)
        {
            NativeCalls.godot_icall_1_0(method_bind_15, Object.GetPtr(this), mask);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_mask");

        [GodotMethod("get_collision_mask")]
        public int GetCollisionMask()
        {
            return NativeCalls.godot_icall_0_1(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_mask_bit");

        /// <summary>
        /// </summary>
        [GodotMethod("set_collision_mask_bit")]
        public void SetCollisionMaskBit(int bit, bool value)
        {
            NativeCalls.godot_icall_2_51(method_bind_17, Object.GetPtr(this), bit, value);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_mask_bit");

        /// <summary>
        /// </summary>
        [GodotMethod("get_collision_mask_bit")]
        public bool GetCollisionMaskBit(int bit)
        {
            return NativeCalls.godot_icall_1_29(method_bind_18, Object.GetPtr(this), bit);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_exclude_parent_body");

        [GodotMethod("set_exclude_parent_body")]
        public void SetExcludeParentBody(bool mask)
        {
            NativeCalls.godot_icall_1_11(method_bind_19, Object.GetPtr(this), mask);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_exclude_parent_body");

        [GodotMethod("get_exclude_parent_body")]
        public bool GetExcludeParentBody()
        {
            return NativeCalls.godot_icall_0_3(method_bind_20, Object.GetPtr(this));
        }
    }
}
