using System;

namespace Godot
{
    /// <summary>
    /// Regular Expression (or regex) is a compact programming language that can be used to recognise strings that follow a specific pattern, such as URLs, email addresses, complete sentences, etc. For instance, a regex of [code]ab[0-9][/code] would find any string that is [code]ab[/code] followed by any number from [code]0[/code] to [code]9[/code]. For a more in-depth look, you can easily find various tutorials and detailed explainations on the Internet.
    /// To begin, the RegEx object needs to be compiled with the search pattern using [method compile] before it can be used.
    /// [codeblock]
    /// var regex = RegEx.new()
    /// regex.compile("\\w-(\\d+)")
    /// [/codeblock]
    /// The search pattern must be escaped first for gdscript before it is escaped for the expression. For example, [code]compile("\\d+")[/code] would be read by RegEx as [code]\d+[/code]. Similarly, [code]compile("\"(?:\\\\.|[^\"])*\"")[/code] would be read as [code]"(?:\\.|[^"])*"[/code]
    /// Using [method search] you can find the pattern within the given text. If a pattern is found, [RegExMatch] is returned and you can retrieve details of the results using fuctions such as [method RegExMatch.get_string] and [method RegExMatch.get_start].
    /// [codeblock]
    /// var regex = RegEx.new()
    /// regex.compile("\\w-(\\d+)")
    /// var result = regex.search("abc n-0123")
    /// if result:
    /// print(result.get_string()) # Would print n-0123
    /// [/codeblock]
    /// The results of capturing groups [code]()[/code] can be retrieved by passing the group number to the various functions in [RegExMatch]. Group 0 is the default and would always refer to the entire pattern. In the above example, calling [code]result.get_string(1)[/code] would give you [code]0123[/code].
    /// This version of RegEx also supports named capturing groups, and the names can be used to retrieve the results. If two or more groups have the same name, the name would only refer to the first one with a match.
    /// [codeblock]
    /// var regex = RegEx.new()
    /// regex.compile("d(?&lt;digit&gt;[0-9]+)|x(?&lt;digit&gt;[0-9a-f]+)")
    /// var result = regex.search("the number is x2f")
    /// if result:
    /// print(result.get_string("digit")) # Would print 2f
    /// [/codeblock]
    /// If you need to process multiple results, [method search_all] generates a list of all non-overlapping results. This can be combined with a for-loop for convenience.
    /// [codeblock]
    /// for result in regex.search_all("d01, d03, d0c, x3f and x42"):
    /// print(result.get_string("digit"))
    /// # Would print 01 03 3f 42
    /// # Note that d0c would not match
    /// [/codeblock]
    /// </summary>
    public class RegEx : Reference
    {
        private const string nativeName = "RegEx";

        public RegEx() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_RegEx_Ctor(this);
        }

        internal RegEx(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear");

        /// <summary>
        /// This method resets the state of the object, as it was freshly created. Namely, it unassigns the regular expression of this object.
        /// </summary>
        [GodotMethod("clear")]
        public void Clear()
        {
            NativeCalls.godot_icall_0_12(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "compile");

        /// <summary>
        /// Compiles and assign the search pattern to use. Returns OK if the compilation is successful. If an error is encountered the details are printed to STDOUT and FAILED is returned.
        /// </summary>
        [GodotMethod("compile")]
        public Error Compile(string pattern)
        {
            return NativeCalls.godot_icall_1_170(method_bind_1, Object.GetPtr(this), pattern);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "search");

        /// <summary>
        /// Searches the text for the compiled pattern. Returns a [RegExMatch] container of the first matching result if found, otherwise null. The region to search within can be specified without modifying where the start and end anchor would be.
        /// </summary>
        [GodotMethod("search")]
        public RegExMatch Search(string subject, int offset = 0, int end = -1)
        {
            return NativeCalls.godot_icall_3_725(method_bind_2, Object.GetPtr(this), subject, offset, end);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "search_all");

        /// <summary>
        /// Searches the text for the compiled pattern. Returns an array of [RegExMatch] containers for each non-overlapping result. If no results were found an empty array is returned instead. The region to search within can be specified without modifying where the start and end anchor would be.
        /// </summary>
        [GodotMethod("search_all")]
        public object[] SearchAll(string subject, int offset = 0, int end = -1)
        {
            return NativeCalls.godot_icall_3_726(method_bind_3, Object.GetPtr(this), subject, offset, end);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "sub");

        /// <summary>
        /// Searches the text for the compiled pattern and replaces it with the specified string. Escapes and backreferences such as [code]\1[/code] and [code]\g&lt;name&gt;[/code] expanded and resolved. By default only the first instance is replaced but it can be changed for all instances (global replacement). The region to search within can be specified without modifying where the start and end anchor would be.
        /// </summary>
        [GodotMethod("sub")]
        public string Sub(string subject, string replacement, bool all = false, int offset = 0, int end = -1)
        {
            return NativeCalls.godot_icall_5_727(method_bind_4, Object.GetPtr(this), subject, replacement, all, offset, end);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_valid");

        /// <summary>
        /// Returns whether this object has a valid search pattern assigned.
        /// </summary>
        [GodotMethod("is_valid")]
        public bool IsValid()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_pattern");

        /// <summary>
        /// Returns the original search pattern that was compiled.
        /// </summary>
        [GodotMethod("get_pattern")]
        public string GetPattern()
        {
            return NativeCalls.godot_icall_0_2(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_group_count");

        /// <summary>
        /// Returns the number of capturing groups in compiled pattern.
        /// </summary>
        [GodotMethod("get_group_count")]
        public int GetGroupCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_names");

        /// <summary>
        /// Returns an array of names of named capturing groups in the compiled pattern. They are ordered by appearance.
        /// </summary>
        [GodotMethod("get_names")]
        public object[] GetNames()
        {
            return NativeCalls.godot_icall_0_21(method_bind_8, Object.GetPtr(this));
        }
    }
}
