using System;

namespace Godot
{
    /// <summary>
    /// RemoteTransform leads the [Transform] of another [Spatial] derived Node (called the remote node) in the scene.
    /// It can be set to track another Node's position, rotation and/or scale. It can update using either global or local coordinates.
    /// </summary>
    public class RemoteTransform : Spatial
    {
        /// <summary>
        /// The [NodePath] to the remote node, relative to the RemoteTransform's position in the scene.
        /// </summary>
        public NodePath RemotePath
        {
            get
            {
                return GetRemoteNode();
            }
            set
            {
                SetRemoteNode(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] global coordinates are used. If [code]false[/code] local coordinates are used. Default value: [code]true[/code].
        /// </summary>
        public bool UseGlobalCoordinates
        {
            get
            {
                return GetUseGlobalCoordinates();
            }
            set
            {
                SetUseGlobalCoordinates(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the remote node's position is tracked. Default value: [code]true[/code].
        /// </summary>
        public bool UpdatePosition
        {
            get
            {
                return GetUpdatePosition();
            }
            set
            {
                SetUpdatePosition(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the remote node's rotation is tracked. Default value: [code]true[/code].
        /// </summary>
        public bool UpdateRotation
        {
            get
            {
                return GetUpdateRotation();
            }
            set
            {
                SetUpdateRotation(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the remote node's scale is tracked. Default value: [code]true[/code].
        /// </summary>
        public bool UpdateScale
        {
            get
            {
                return GetUpdateScale();
            }
            set
            {
                SetUpdateScale(value);
            }
        }


        private const string nativeName = "RemoteTransform";

        public RemoteTransform() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_RemoteTransform_Ctor(this);
        }

        internal RemoteTransform(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_remote_node");

        [GodotMethod("set_remote_node")]
        public void SetRemoteNode(NodePath path)
        {
            NativeCalls.godot_icall_1_78(method_bind_0, Object.GetPtr(this), NodePath.GetPtr(path));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_remote_node");

        [GodotMethod("get_remote_node")]
        public NodePath GetRemoteNode()
        {
            IntPtr ret = NativeCalls.godot_icall_0_79(method_bind_1, Object.GetPtr(this));
            return new NodePath(ret);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_use_global_coordinates");

        [GodotMethod("set_use_global_coordinates")]
        public void SetUseGlobalCoordinates(bool useGlobalCoordinates)
        {
            NativeCalls.godot_icall_1_11(method_bind_2, Object.GetPtr(this), useGlobalCoordinates);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_use_global_coordinates");

        [GodotMethod("get_use_global_coordinates")]
        public bool GetUseGlobalCoordinates()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_update_position");

        [GodotMethod("set_update_position")]
        public void SetUpdatePosition(bool updateRemotePosition)
        {
            NativeCalls.godot_icall_1_11(method_bind_4, Object.GetPtr(this), updateRemotePosition);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_update_position");

        [GodotMethod("get_update_position")]
        public bool GetUpdatePosition()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_update_rotation");

        [GodotMethod("set_update_rotation")]
        public void SetUpdateRotation(bool updateRemoteRotation)
        {
            NativeCalls.godot_icall_1_11(method_bind_6, Object.GetPtr(this), updateRemoteRotation);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_update_rotation");

        [GodotMethod("get_update_rotation")]
        public bool GetUpdateRotation()
        {
            return NativeCalls.godot_icall_0_3(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_update_scale");

        [GodotMethod("set_update_scale")]
        public void SetUpdateScale(bool updateRemoteScale)
        {
            NativeCalls.godot_icall_1_11(method_bind_8, Object.GetPtr(this), updateRemoteScale);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_update_scale");

        [GodotMethod("get_update_scale")]
        public bool GetUpdateScale()
        {
            return NativeCalls.godot_icall_0_3(method_bind_9, Object.GetPtr(this));
        }
    }
}
