using System;

namespace Godot
{
    /// <summary>
    /// Resource is the base class for all resource types. Resources are primarily data containers. They are reference counted and freed when no longer in use. They are also loaded only once from disk, and further attempts to load the resource will return the same reference (all this in contrast to a [Node], which is not reference counted and can be instanced from disk as many times as desired). Resources can be saved externally on disk or bundled into another object, such as a [Node] or another resource.
    /// </summary>
    public class Resource : Reference
    {
        /// <summary>
        /// </summary>
        public bool ResourceLocalToScene
        {
            get
            {
                return IsLocalToScene();
            }
            set
            {
                SetLocalToScene(value);
            }
        }

        /// <summary>
        /// </summary>
        public string ResourcePath
        {
            get
            {
                return GetPath();
            }
            set
            {
                SetPath(value);
            }
        }

        /// <summary>
        /// </summary>
        public string ResourceName
        {
            get
            {
                return GetName();
            }
            set
            {
                SetName(value);
            }
        }


        private const string nativeName = "Resource";

        public Resource() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Resource_Ctor(this);
        }

        internal Resource(bool memoryOwn) : base(memoryOwn) {}

        /// <summary>
        /// </summary>
        [GodotMethod("_setup_local_to_scene")]
        public virtual void _SetupLocalToScene()
        {
            return;
        }

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_path");

        [GodotMethod("set_path")]
        public void SetPath(string path)
        {
            NativeCalls.godot_icall_1_43(method_bind_0, Object.GetPtr(this), path);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "take_over_path");

        /// <summary>
        /// Set the path of the resource. Differs from set_path(), if another [code]Resource[/code] exists with "path" it over-takes it, instead of failing.
        /// </summary>
        [GodotMethod("take_over_path")]
        public void TakeOverPath(string path)
        {
            NativeCalls.godot_icall_1_43(method_bind_1, Object.GetPtr(this), path);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_path");

        [GodotMethod("get_path")]
        public string GetPath()
        {
            return NativeCalls.godot_icall_0_2(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_name");

        [GodotMethod("set_name")]
        public void SetName(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_3, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_name");

        [GodotMethod("get_name")]
        public string GetName()
        {
            return NativeCalls.godot_icall_0_2(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rid");

        /// <summary>
        /// Return the RID of the resource (or an empty RID). Many resources (such as [Texture], [Mesh], etc) are high level abstractions of resources stored in a server, so this function will return the original RID.
        /// </summary>
        [GodotMethod("get_rid")]
        public RID GetRid()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_5, Object.GetPtr(this));
            return new RID(ret);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_local_to_scene");

        [GodotMethod("set_local_to_scene")]
        public void SetLocalToScene(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_6, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_local_to_scene");

        [GodotMethod("is_local_to_scene")]
        public bool IsLocalToScene()
        {
            return NativeCalls.godot_icall_0_3(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_local_scene");

        /// <summary>
        /// </summary>
        [GodotMethod("get_local_scene")]
        public Node GetLocalScene()
        {
            return NativeCalls.godot_icall_0_200(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "setup_local_to_scene");

        /// <summary>
        /// </summary>
        [GodotMethod("setup_local_to_scene")]
        public void SetupLocalToScene()
        {
            NativeCalls.godot_icall_0_12(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "duplicate");

        /// <summary>
        /// </summary>
        [GodotMethod("duplicate")]
        public Resource Duplicate(bool subresources = false)
        {
            return NativeCalls.godot_icall_1_730(method_bind_10, Object.GetPtr(this), subresources);
        }
    }
}
