using System;

namespace Godot
{
    /// <summary>
    /// Rich text can contain custom text, fonts, images and some basic formatting. The label manages these as an internal tag stack. It also adapts itself to given width/heights.
    /// Note that assignments to [member bbcode_text] clear the tag stack and reconstruct it from the property's contents. Any edits made to [member bbcode_text] will erase previous edits made from other manual sources such as [method append_bbcode] and the [code]push_*[/code] / [method pop] methods.
    /// </summary>
    public class RichTextLabel : Control
    {
        public enum Align
        {
            Left = 0,
            Center = 1,
            Right = 2,
            Fill = 3
        }

        public enum ListType
        {
            Numbers = 0,
            Letters = 1,
            Dots = 2
        }

        public enum ItemType
        {
            Frame = 0,
            Text = 1,
            Image = 2,
            Newline = 3,
            Font = 4,
            Color = 5,
            Underline = 6,
            Align = 7,
            Indent = 8,
            List = 9,
            Table = 10,
            Meta = 11
        }


        /// <summary>
        /// If [code]true[/code] the label uses BBCode formatting. Default value: [code]false[/code].
        /// </summary>
        public bool BbcodeEnabled
        {
            get
            {
                return IsUsingBbcode();
            }
            set
            {
                SetUseBbcode(value);
            }
        }

        /// <summary>
        /// The label's text in BBCode format. Is not representative of manual modifications to the internal tag stack. Erases changes made by other methods when edited.
        /// </summary>
        public string BbcodeText
        {
            get
            {
                return GetBbcode();
            }
            set
            {
                SetBbcode(value);
            }
        }

        /// <summary>
        /// The restricted number of characters to display in the label.
        /// </summary>
        public int VisibleCharacters
        {
            get
            {
                return GetVisibleCharacters();
            }
            set
            {
                SetVisibleCharacters(value);
            }
        }

        /// <summary>
        /// The text's visibility, as a [float] between 0.0 and 1.0.
        /// </summary>
        public float PercentVisible
        {
            get
            {
                return GetPercentVisible();
            }
            set
            {
                SetPercentVisible(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], the label underlines meta tags such as [url]{text}[/url]. Default value: [code]true[/code].
        /// </summary>
        public bool MetaUnderlined
        {
            get
            {
                return IsMetaUnderlined();
            }
            set
            {
                SetMetaUnderline(value);
            }
        }

        /// <summary>
        /// The number of spaces associated with a single tab length. Does not affect "\t" in text tags, only indent tags.
        /// </summary>
        public int TabSize
        {
            get
            {
                return GetTabSize();
            }
            set
            {
                SetTabSize(value);
            }
        }

        /// <summary>
        /// The raw text of the label.
        /// When set, clears the tag stack and adds a raw text tag to the top of it. Does not parse bbcodes. Does not modify [member bbcode_text].
        /// </summary>
        public string Text
        {
            get
            {
                return GetText();
            }
            set
            {
                SetText(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], the scrollbar is visible. Does not block scrolling completely. See [method scroll_to_line]. Default value: [code]true[/code].
        /// </summary>
        public bool ScrollActive
        {
            get
            {
                return IsScrollActive();
            }
            set
            {
                SetScrollActive(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], the window scrolls down to display new content automatically. Default value: [code]false[/code].
        /// </summary>
        public bool ScrollFollowing
        {
            get
            {
                return IsScrollFollowing();
            }
            set
            {
                SetScrollFollow(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], the label allows text selection.
        /// </summary>
        public bool SelectionEnabled
        {
            get
            {
                return IsSelectionEnabled();
            }
            set
            {
                SetSelectionEnabled(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the label uses the custom font color. Default value: [code]false[/code].
        /// </summary>
        public bool OverrideSelectedFontColor
        {
            get
            {
                return IsOverridingSelectedFontColor();
            }
            set
            {
                SetOverrideSelectedFontColor(value);
            }
        }


        private const string nativeName = "RichTextLabel";

        public RichTextLabel() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_RichTextLabel_Ctor(this);
        }

        internal RichTextLabel(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_text");

        [GodotMethod("get_text")]
        public string GetText()
        {
            return NativeCalls.godot_icall_0_2(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_text");

        /// <summary>
        /// Adds raw non-bbcode-parsed text to the tag stack.
        /// </summary>
        [GodotMethod("add_text")]
        public void AddText(string text)
        {
            NativeCalls.godot_icall_1_43(method_bind_1, Object.GetPtr(this), text);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_text");

        [GodotMethod("set_text")]
        public void SetText(string text)
        {
            NativeCalls.godot_icall_1_43(method_bind_2, Object.GetPtr(this), text);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_image");

        /// <summary>
        /// Adds an image's opening and closing tags to the tag stack.
        /// </summary>
        [GodotMethod("add_image")]
        public void AddImage(Texture image)
        {
            NativeCalls.godot_icall_1_24(method_bind_3, Object.GetPtr(this), Object.GetPtr(image));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "newline");

        /// <summary>
        /// Adds a newline tag to the tag stack.
        /// </summary>
        [GodotMethod("newline")]
        public void Newline()
        {
            NativeCalls.godot_icall_0_12(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_line");

        /// <summary>
        /// Removes a line of content from the label. Returns [code]true[/code] if the line exists.
        /// </summary>
        [GodotMethod("remove_line")]
        public bool RemoveLine(int line)
        {
            return NativeCalls.godot_icall_1_29(method_bind_5, Object.GetPtr(this), line);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "push_font");

        /// <summary>
        /// Adds a [code][font][/code] tag to the tag stack. Overrides default fonts for its duration.
        /// </summary>
        [GodotMethod("push_font")]
        public void PushFont(Font font)
        {
            NativeCalls.godot_icall_1_24(method_bind_6, Object.GetPtr(this), Object.GetPtr(font));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "push_color");

        /// <summary>
        /// Adds a [code][color][/code] tag to the tag stack.
        /// </summary>
        [GodotMethod("push_color")]
        public void PushColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_7, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "push_align");

        /// <summary>
        /// Adds a [code][right][/code] tag to the tag stack.
        /// </summary>
        [GodotMethod("push_align")]
        public void PushAlign(RichTextLabel.Align align)
        {
            NativeCalls.godot_icall_1_733(method_bind_8, Object.GetPtr(this), align);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "push_indent");

        /// <summary>
        /// Adds an [code][indent][/code] tag to the tag stack. Multiplies "level" by current tab_size to determine new margin length.
        /// </summary>
        [GodotMethod("push_indent")]
        public void PushIndent(int level)
        {
            NativeCalls.godot_icall_1_0(method_bind_9, Object.GetPtr(this), level);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "push_list");

        /// <summary>
        /// Adds a list tag to the tag stack. Similar to the bbcodes [code][ol][/code] or [code][ul][/code], but supports more list types. Not fully implemented!
        /// </summary>
        [GodotMethod("push_list")]
        public void PushList(RichTextLabel.ListType type)
        {
            NativeCalls.godot_icall_1_734(method_bind_10, Object.GetPtr(this), type);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "push_meta");

        /// <summary>
        /// Adds a meta tag to the tag stack. Similar to the bbcode [code][url=something]{text}[/url][/code], but supports non-[String] metadata types.
        /// </summary>
        [GodotMethod("push_meta")]
        public void PushMeta(object data)
        {
            NativeCalls.godot_icall_1_463(method_bind_11, Object.GetPtr(this), data);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "push_underline");

        /// <summary>
        /// Adds a [code][u][/code] tag to the tag stack.
        /// </summary>
        [GodotMethod("push_underline")]
        public void PushUnderline()
        {
            NativeCalls.godot_icall_0_12(method_bind_12, Object.GetPtr(this));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "push_table");

        /// <summary>
        /// Adds a [code][table=columns][/code] tag to the tag stack.
        /// </summary>
        [GodotMethod("push_table")]
        public void PushTable(int columns)
        {
            NativeCalls.godot_icall_1_0(method_bind_13, Object.GetPtr(this), columns);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_table_column_expand");

        /// <summary>
        /// Edits the selected columns expansion options. If [code]expand[/code] is [code]true[/code], the column expands in proportion to its expansion ratio versus the other columns' ratios.
        /// For example, 2 columns with ratios 3 and 4 plus 70 pixels in available width would expand 30 and 40 pixels, respectively.
        /// Columns with a [code]false[/code] expand will not contribute to the total ratio.
        /// </summary>
        [GodotMethod("set_table_column_expand")]
        public void SetTableColumnExpand(int column, bool expand, int ratio)
        {
            NativeCalls.godot_icall_3_735(method_bind_14, Object.GetPtr(this), column, expand, ratio);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "push_cell");

        /// <summary>
        /// Adds a [code][cell][/code] tag to the tag stack. Must be inside a [table] tag. See [method push_table] for details.
        /// </summary>
        [GodotMethod("push_cell")]
        public void PushCell()
        {
            NativeCalls.godot_icall_0_12(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "pop");

        /// <summary>
        /// Terminates the current tag. Use after [code]push_*[/code] methods to close bbcodes manually. Does not need to follow [code]add_*[/code] methods.
        /// </summary>
        [GodotMethod("pop")]
        public void Pop()
        {
            NativeCalls.godot_icall_0_12(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear");

        /// <summary>
        /// Clears the tag stack and sets [member bbcode_text] to an empty string.
        /// </summary>
        [GodotMethod("clear")]
        public void Clear()
        {
            NativeCalls.godot_icall_0_12(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_meta_underline");

        [GodotMethod("set_meta_underline")]
        public void SetMetaUnderline(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_18, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_meta_underlined");

        [GodotMethod("is_meta_underlined")]
        public bool IsMetaUnderlined()
        {
            return NativeCalls.godot_icall_0_3(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_override_selected_font_color");

        [GodotMethod("set_override_selected_font_color")]
        public void SetOverrideSelectedFontColor(bool @override)
        {
            NativeCalls.godot_icall_1_11(method_bind_20, Object.GetPtr(this), @override);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_overriding_selected_font_color");

        [GodotMethod("is_overriding_selected_font_color")]
        public bool IsOverridingSelectedFontColor()
        {
            return NativeCalls.godot_icall_0_3(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_scroll_active");

        [GodotMethod("set_scroll_active")]
        public void SetScrollActive(bool active)
        {
            NativeCalls.godot_icall_1_11(method_bind_22, Object.GetPtr(this), active);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_scroll_active");

        [GodotMethod("is_scroll_active")]
        public bool IsScrollActive()
        {
            return NativeCalls.godot_icall_0_3(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_scroll_follow");

        [GodotMethod("set_scroll_follow")]
        public void SetScrollFollow(bool follow)
        {
            NativeCalls.godot_icall_1_11(method_bind_24, Object.GetPtr(this), follow);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_scroll_following");

        [GodotMethod("is_scroll_following")]
        public bool IsScrollFollowing()
        {
            return NativeCalls.godot_icall_0_3(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_v_scroll");

        /// <summary>
        /// Returns the vertical scrollbar.
        /// </summary>
        [GodotMethod("get_v_scroll")]
        public VScrollBar GetVScroll()
        {
            return NativeCalls.godot_icall_0_460(method_bind_26, Object.GetPtr(this));
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "scroll_to_line");

        /// <summary>
        /// Scrolls the window's top line to match [code]line[/code].
        /// </summary>
        [GodotMethod("scroll_to_line")]
        public void ScrollToLine(int line)
        {
            NativeCalls.godot_icall_1_0(method_bind_27, Object.GetPtr(this), line);
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tab_size");

        [GodotMethod("set_tab_size")]
        public void SetTabSize(int spaces)
        {
            NativeCalls.godot_icall_1_0(method_bind_28, Object.GetPtr(this), spaces);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tab_size");

        [GodotMethod("get_tab_size")]
        public int GetTabSize()
        {
            return NativeCalls.godot_icall_0_1(method_bind_29, Object.GetPtr(this));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_selection_enabled");

        [GodotMethod("set_selection_enabled")]
        public void SetSelectionEnabled(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_30, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_selection_enabled");

        [GodotMethod("is_selection_enabled")]
        public bool IsSelectionEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_31, Object.GetPtr(this));
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "parse_bbcode");

        /// <summary>
        /// The assignment version of [method append_bbcode]. Clears the tag stack and inserts the new content. Returns [code]OK[/code] if parses [code]bbcode[/code] successfully.
        /// </summary>
        [GodotMethod("parse_bbcode")]
        public Error ParseBbcode(string bbcode)
        {
            return NativeCalls.godot_icall_1_170(method_bind_32, Object.GetPtr(this), bbcode);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "append_bbcode");

        /// <summary>
        /// Parses [code]bbcode[/code] and adds tags to the tag stack as needed. Returns the result of the parsing, [code]OK[/code] if successful.
        /// </summary>
        [GodotMethod("append_bbcode")]
        public Error AppendBbcode(string bbcode)
        {
            return NativeCalls.godot_icall_1_170(method_bind_33, Object.GetPtr(this), bbcode);
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bbcode");

        [GodotMethod("set_bbcode")]
        public void SetBbcode(string text)
        {
            NativeCalls.godot_icall_1_43(method_bind_34, Object.GetPtr(this), text);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bbcode");

        [GodotMethod("get_bbcode")]
        public string GetBbcode()
        {
            return NativeCalls.godot_icall_0_2(method_bind_35, Object.GetPtr(this));
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_visible_characters");

        [GodotMethod("set_visible_characters")]
        public void SetVisibleCharacters(int amount)
        {
            NativeCalls.godot_icall_1_0(method_bind_36, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_visible_characters");

        [GodotMethod("get_visible_characters")]
        public int GetVisibleCharacters()
        {
            return NativeCalls.godot_icall_0_1(method_bind_37, Object.GetPtr(this));
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_percent_visible");

        [GodotMethod("set_percent_visible")]
        public void SetPercentVisible(float percentVisible)
        {
            NativeCalls.godot_icall_1_10(method_bind_38, Object.GetPtr(this), percentVisible);
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_percent_visible");

        [GodotMethod("get_percent_visible")]
        public float GetPercentVisible()
        {
            return NativeCalls.godot_icall_0_9(method_bind_39, Object.GetPtr(this));
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_total_character_count");

        /// <summary>
        /// Returns the total number of characters from text tags. Does not include bbcodes.
        /// </summary>
        [GodotMethod("get_total_character_count")]
        public int GetTotalCharacterCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_40, Object.GetPtr(this));
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_use_bbcode");

        [GodotMethod("set_use_bbcode")]
        public void SetUseBbcode(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_41, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_using_bbcode");

        [GodotMethod("is_using_bbcode")]
        public bool IsUsingBbcode()
        {
            return NativeCalls.godot_icall_0_3(method_bind_42, Object.GetPtr(this));
        }

        private IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_line_count");

        /// <summary>
        /// Returns the total number of newlines in the tag stack's text tags. Considers wrapped text as one line.
        /// </summary>
        [GodotMethod("get_line_count")]
        public int GetLineCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_43, Object.GetPtr(this));
        }

        private IntPtr method_bind_44 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_visible_line_count");

        /// <summary>
        /// Returns the number of visible lines.
        /// </summary>
        [GodotMethod("get_visible_line_count")]
        public int GetVisibleLineCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_44, Object.GetPtr(this));
        }
    }
}
