using System;

namespace Godot
{
    /// <summary>
    /// This is the node that implements full 3D physics. This means that you do not control a RigidBody directly. Instead you can apply forces to it (gravity, impulses, etc.), and the physics simulation will calculate the resulting movement, collision, bouncing, rotating, etc.
    /// This node can use custom force integration, for writing complex physics motion behavior per node.
    /// This node can shift state between regular Rigid body, Kinematic, Character or Static.
    /// Character mode forbids this node from being rotated.
    /// As a warning, don't change RigidBody's position every frame or very often. Sporadic changes work fine, but physics runs at a different granularity (fixed hz) than usual rendering (process callback) and maybe even in a separate thread, so changing this from a process loop will yield strange behavior.
    /// </summary>
    public class RigidBody : PhysicsBody
    {
        public enum ModeEnum
        {
            Rigid = 0,
            Static = 1,
            Character = 2,
            Kinematic = 3
        }


        /// <summary>
        /// The body mode from the MODE_* enum. Modes include: MODE_STATIC, MODE_KINEMATIC, MODE_RIGID, and MODE_CHARACTER.
        /// </summary>
        public RigidBody.ModeEnum Mode
        {
            get
            {
                return GetMode();
            }
            set
            {
                SetMode(value);
            }
        }

        /// <summary>
        /// RigidBody's mass.
        /// </summary>
        public float Mass
        {
            get
            {
                return GetMass();
            }
            set
            {
                SetMass(value);
            }
        }

        /// <summary>
        /// RigidBody's weight based on its mass and the global 3D gravity. Global values are set in "Project &gt; Project Settings &gt; Physics &gt; 3d".
        /// </summary>
        public float Weight
        {
            get
            {
                return GetWeight();
            }
            set
            {
                SetWeight(value);
            }
        }

        /// <summary>
        /// The body friction, from 0 (frictionless) to 1 (max friction).
        /// </summary>
        public float Friction
        {
            get
            {
                return GetFriction();
            }
            set
            {
                SetFriction(value);
            }
        }

        /// <summary>
        /// RigidBody's bounciness.
        /// </summary>
        public float Bounce
        {
            get
            {
                return GetBounce();
            }
            set
            {
                SetBounce(value);
            }
        }

        /// <summary>
        /// This is multiplied by the global 3D gravity setting found in "Project &gt; Project Settings &gt; Physics &gt; 3d" to produce RigidBody's gravity. E.g. a value of 1 will be normal gravity, 2 will apply double gravity, and 0.5 will apply half gravity to this object.
        /// </summary>
        public float GravityScale
        {
            get
            {
                return GetGravityScale();
            }
            set
            {
                SetGravityScale(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] internal force integration will be disabled (like gravity or air friction) for this body. Other than collision response, the body will only move as determined by the [method _integrate_forces] function, if defined.
        /// </summary>
        public bool CustomIntegrator
        {
            get
            {
                return IsUsingCustomIntegrator();
            }
            set
            {
                SetUseCustomIntegrator(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] continuous collision detection is used.
        /// Continuous collision detection tries to predict where a moving body will collide, instead of moving it and correcting its movement if it collided. Continuous collision detection is more precise, and misses less impacts by small, fast-moving objects. Not using continuous collision detection is faster to compute, but can miss small, fast-moving objects.
        /// </summary>
        public bool ContinuousCd
        {
            get
            {
                return IsUsingContinuousCollisionDetection();
            }
            set
            {
                SetUseContinuousCollisionDetection(value);
            }
        }

        /// <summary>
        /// The maximum contacts to report. Bodies can keep a log of the contacts with other bodies, this is enabled by setting the maximum amount of contacts reported to a number greater than 0.
        /// </summary>
        public int ContactsReported
        {
            get
            {
                return GetMaxContactsReported();
            }
            set
            {
                SetMaxContactsReported(value);
            }
        }

        /// <summary>
        /// If true, the RigidBody will emit signals when it collides with another RigidBody.
        /// </summary>
        public bool ContactMonitor
        {
            get
            {
                return IsContactMonitorEnabled();
            }
            set
            {
                SetContactMonitor(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] RigidBody is sleeping and will not calculate forces until woken up by a collision or the [code]apply_impulse[/code] method.
        /// </summary>
        public bool Sleeping
        {
            get
            {
                return IsSleeping();
            }
            set
            {
                SetSleeping(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the RigidBody will not calculate forces and will act as a static body while there is no movement. It will wake up when forces are applied through other collisions or when the [code]apply_impulse[/code] method is used.
        /// </summary>
        public bool CanSleep
        {
            get
            {
                return IsAbleToSleep();
            }
            set
            {
                SetCanSleep(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool AxisLockLinearX
        {
            get
            {
                return GetAxisLock((PhysicsServer.BodyAxis)1);
            }
            set
            {
                SetAxisLock((PhysicsServer.BodyAxis)1, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool AxisLockLinearY
        {
            get
            {
                return GetAxisLock((PhysicsServer.BodyAxis)2);
            }
            set
            {
                SetAxisLock((PhysicsServer.BodyAxis)2, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool AxisLockLinearZ
        {
            get
            {
                return GetAxisLock((PhysicsServer.BodyAxis)4);
            }
            set
            {
                SetAxisLock((PhysicsServer.BodyAxis)4, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool AxisLockAngularX
        {
            get
            {
                return GetAxisLock((PhysicsServer.BodyAxis)8);
            }
            set
            {
                SetAxisLock((PhysicsServer.BodyAxis)8, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool AxisLockAngularY
        {
            get
            {
                return GetAxisLock((PhysicsServer.BodyAxis)16);
            }
            set
            {
                SetAxisLock((PhysicsServer.BodyAxis)16, value);
            }
        }

        /// <summary>
        /// </summary>
        public bool AxisLockAngularZ
        {
            get
            {
                return GetAxisLock((PhysicsServer.BodyAxis)32);
            }
            set
            {
                SetAxisLock((PhysicsServer.BodyAxis)32, value);
            }
        }

        /// <summary>
        /// RigidBody's linear velocity. Can be used sporadically, but [b]DON'T SET THIS IN EVERY FRAME[/b], because physics may run in another thread and runs at a different granularity. Use [method _integrate_forces] as your process loop for precise control of the body state.
        /// </summary>
        public Vector3 LinearVelocity
        {
            get
            {
                return GetLinearVelocity();
            }
            set
            {
                SetLinearVelocity(value);
            }
        }

        /// <summary>
        /// RigidBody's linear damp. Default value: -1, cannot be less than -1. If this value is different from -1, any linear damp derived from the world or areas will be overridden.
        /// </summary>
        public float LinearDamp
        {
            get
            {
                return GetLinearDamp();
            }
            set
            {
                SetLinearDamp(value);
            }
        }

        /// <summary>
        /// RigidBody's rotational velocity.
        /// </summary>
        public Vector3 AngularVelocity
        {
            get
            {
                return GetAngularVelocity();
            }
            set
            {
                SetAngularVelocity(value);
            }
        }

        /// <summary>
        /// Damps RigidBody's rotational forces.
        /// </summary>
        public float AngularDamp
        {
            get
            {
                return GetAngularDamp();
            }
            set
            {
                SetAngularDamp(value);
            }
        }


        private const string nativeName = "RigidBody";

        public RigidBody() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_RigidBody_Ctor(this);
        }

        internal RigidBody(bool memoryOwn) : base(memoryOwn) {}

        /// <summary>
        /// Called during physics processing, allowing you to read and safely modify the simulation state for the object. By default it works in addition to the usual physics behavior, but [method set_use_custom_integrator] allows you to disable the default behavior and do fully custom force integration for a body.
        /// </summary>
        [GodotMethod("_integrate_forces")]
        public virtual void _IntegrateForces(PhysicsDirectBodyState state)
        {
            return;
        }

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_mode");

        [GodotMethod("set_mode")]
        public void SetMode(RigidBody.ModeEnum mode)
        {
            NativeCalls.godot_icall_1_736(method_bind_0, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mode");

        [GodotMethod("get_mode")]
        public RigidBody.ModeEnum GetMode()
        {
            return NativeCalls.godot_icall_0_737(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_mass");

        [GodotMethod("set_mass")]
        public void SetMass(float mass)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), mass);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mass");

        [GodotMethod("get_mass")]
        public float GetMass()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_weight");

        [GodotMethod("set_weight")]
        public void SetWeight(float weight)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), weight);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_weight");

        [GodotMethod("get_weight")]
        public float GetWeight()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_friction");

        [GodotMethod("set_friction")]
        public void SetFriction(float friction)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), friction);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_friction");

        [GodotMethod("get_friction")]
        public float GetFriction()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bounce");

        [GodotMethod("set_bounce")]
        public void SetBounce(float bounce)
        {
            NativeCalls.godot_icall_1_10(method_bind_8, Object.GetPtr(this), bounce);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bounce");

        [GodotMethod("get_bounce")]
        public float GetBounce()
        {
            return NativeCalls.godot_icall_0_9(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_linear_velocity");

        [GodotMethod("set_linear_velocity")]
        public void SetLinearVelocity(Vector3 linearVelocity)
        {
            NativeCalls.godot_icall_1_104(method_bind_10, Object.GetPtr(this), ref linearVelocity);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_linear_velocity");

        [GodotMethod("get_linear_velocity")]
        public Vector3 GetLinearVelocity()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_11, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_angular_velocity");

        [GodotMethod("set_angular_velocity")]
        public void SetAngularVelocity(Vector3 angularVelocity)
        {
            NativeCalls.godot_icall_1_104(method_bind_12, Object.GetPtr(this), ref angularVelocity);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_angular_velocity");

        [GodotMethod("get_angular_velocity")]
        public Vector3 GetAngularVelocity()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_13, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_gravity_scale");

        [GodotMethod("set_gravity_scale")]
        public void SetGravityScale(float gravityScale)
        {
            NativeCalls.godot_icall_1_10(method_bind_14, Object.GetPtr(this), gravityScale);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_gravity_scale");

        [GodotMethod("get_gravity_scale")]
        public float GetGravityScale()
        {
            return NativeCalls.godot_icall_0_9(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_linear_damp");

        [GodotMethod("set_linear_damp")]
        public void SetLinearDamp(float linearDamp)
        {
            NativeCalls.godot_icall_1_10(method_bind_16, Object.GetPtr(this), linearDamp);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_linear_damp");

        [GodotMethod("get_linear_damp")]
        public float GetLinearDamp()
        {
            return NativeCalls.godot_icall_0_9(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_angular_damp");

        [GodotMethod("set_angular_damp")]
        public void SetAngularDamp(float angularDamp)
        {
            NativeCalls.godot_icall_1_10(method_bind_18, Object.GetPtr(this), angularDamp);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_angular_damp");

        [GodotMethod("get_angular_damp")]
        public float GetAngularDamp()
        {
            return NativeCalls.godot_icall_0_9(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_max_contacts_reported");

        [GodotMethod("set_max_contacts_reported")]
        public void SetMaxContactsReported(int amount)
        {
            NativeCalls.godot_icall_1_0(method_bind_20, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_max_contacts_reported");

        [GodotMethod("get_max_contacts_reported")]
        public int GetMaxContactsReported()
        {
            return NativeCalls.godot_icall_0_1(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_use_custom_integrator");

        [GodotMethod("set_use_custom_integrator")]
        public void SetUseCustomIntegrator(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_22, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_using_custom_integrator");

        [GodotMethod("is_using_custom_integrator")]
        public bool IsUsingCustomIntegrator()
        {
            return NativeCalls.godot_icall_0_3(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_contact_monitor");

        [GodotMethod("set_contact_monitor")]
        public void SetContactMonitor(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_24, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_contact_monitor_enabled");

        [GodotMethod("is_contact_monitor_enabled")]
        public bool IsContactMonitorEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_use_continuous_collision_detection");

        [GodotMethod("set_use_continuous_collision_detection")]
        public void SetUseContinuousCollisionDetection(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_26, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_using_continuous_collision_detection");

        [GodotMethod("is_using_continuous_collision_detection")]
        public bool IsUsingContinuousCollisionDetection()
        {
            return NativeCalls.godot_icall_0_3(method_bind_27, Object.GetPtr(this));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_axis_velocity");

        /// <summary>
        /// Set an axis velocity. The velocity in the given vector axis will be set as the given vector length. This is useful for jumping behavior.
        /// </summary>
        [GodotMethod("set_axis_velocity")]
        public void SetAxisVelocity(Vector3 axisVelocity)
        {
            NativeCalls.godot_icall_1_104(method_bind_28, Object.GetPtr(this), ref axisVelocity);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "apply_impulse");

        /// <summary>
        /// Apply a positioned impulse (which will be affected by the body mass and shape). This is the equivalent of hitting a billiard ball with a cue: a force that is applied once, and only once. Both the impulse and the offset from the body origin are in global coordinates.
        /// </summary>
        [GodotMethod("apply_impulse")]
        public void ApplyImpulse(Vector3 position, Vector3 impulse)
        {
            NativeCalls.godot_icall_2_657(method_bind_29, Object.GetPtr(this), ref position, ref impulse);
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_sleeping");

        [GodotMethod("set_sleeping")]
        public void SetSleeping(bool sleeping)
        {
            NativeCalls.godot_icall_1_11(method_bind_30, Object.GetPtr(this), sleeping);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_sleeping");

        [GodotMethod("is_sleeping")]
        public bool IsSleeping()
        {
            return NativeCalls.godot_icall_0_3(method_bind_31, Object.GetPtr(this));
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_can_sleep");

        [GodotMethod("set_can_sleep")]
        public void SetCanSleep(bool ableToSleep)
        {
            NativeCalls.godot_icall_1_11(method_bind_32, Object.GetPtr(this), ableToSleep);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_able_to_sleep");

        [GodotMethod("is_able_to_sleep")]
        public bool IsAbleToSleep()
        {
            return NativeCalls.godot_icall_0_3(method_bind_33, Object.GetPtr(this));
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_axis_lock");

        [GodotMethod("set_axis_lock")]
        public void SetAxisLock(PhysicsServer.BodyAxis axis, bool @lock)
        {
            NativeCalls.godot_icall_2_468(method_bind_34, Object.GetPtr(this), axis, @lock);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_axis_lock");

        [GodotMethod("get_axis_lock")]
        public bool GetAxisLock(PhysicsServer.BodyAxis axis)
        {
            return NativeCalls.godot_icall_1_469(method_bind_35, Object.GetPtr(this), axis);
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_colliding_bodies");

        /// <summary>
        /// Return a list of the bodies colliding with this one. By default, number of max contacts reported is at 0 , see [method set_max_contacts_reported] to increase it.  Note that the result of this test is not immediate after moving objects. For performance, list of collisions is updated once per frame and before the physics step. Consider using signals instead.
        /// </summary>
        [GodotMethod("get_colliding_bodies")]
        public object[] GetCollidingBodies()
        {
            return NativeCalls.godot_icall_0_21(method_bind_36, Object.GetPtr(this));
        }
    }
}
