using System;

namespace Godot
{
    /// <summary>
    /// This node implements simulated 2D physics. You do not control a RigidBody2D directly. Instead you apply forces to it (gravity, impulses, etc.) and the physics simulation calculates the resulting movement based on its mass, friction, and other physical properties.
    /// A RigidBody2D has 4 behavior modes (see [member mode]):
    /// - [b]Rigid[/b]: The body behaves as a physical object. It collides with other bodies and responds to forces applied to it. This is the default mode.
    /// - [b]Static[/b]: The body behaves like a [StaticBody2D] and does not move.
    /// - [b]Character[/b]: Similar to [code]Rigid[/code] mode, but the body can not rotate.
    /// - [b]Kinematic[/b]: The body behaves like a [KinematicBody2D], and must be moved by code.
    /// [b]Note:[/b] You should not change a RigidBody2D's [code]position[/code] or [code]linear_velocity[/code] every frame or even very often. If you need to directly affect the body's state, use [method _integrate_forces], which allows you to directly access the physics state.
    /// If you need to override the default physics behavior, you can write a custom force integration. See [member custom_integrator].
    /// </summary>
    public class RigidBody2D : PhysicsBody2D
    {
        public enum ModeEnum
        {
            Rigid = 0,
            Static = 1,
            Character = 2,
            Kinematic = 3
        }

        public enum CCDMode
        {
            Disabled = 0,
            CastRay = 1,
            CastShape = 2
        }


        /// <summary>
        /// The body's mode. See [code]MODE_*[/code] constants. Default value: [code]MODE_RIGID[/code].
        /// </summary>
        public RigidBody2D.ModeEnum Mode
        {
            get
            {
                return GetMode();
            }
            set
            {
                SetMode(value);
            }
        }

        /// <summary>
        /// The body's mass. Default value: [code]1[/code].
        /// </summary>
        public float Mass
        {
            get
            {
                return GetMass();
            }
            set
            {
                SetMass(value);
            }
        }

        /// <summary>
        /// The body's moment of inertia. This is like mass, but for rotation: it determines how much torque it takes to rotate the body. The moment of inertia is usually computed automatically from the mass and the shapes, but this function allows you to set a custom value. Set 0 (or negative) inertia to return to automatically computing it.
        /// </summary>
        public float Inertia
        {
            get
            {
                return GetInertia();
            }
            set
            {
                SetInertia(value);
            }
        }

        /// <summary>
        /// The body's weight based on its mass and the "Default Gravity" value in "Project &gt; Project Settings &gt; Physics &gt; 2d".
        /// </summary>
        public float Weight
        {
            get
            {
                return GetWeight();
            }
            set
            {
                SetWeight(value);
            }
        }

        /// <summary>
        /// The body's friction. Values range from [code]0[/code] (frictionless) to [code]1[/code] (maximum friction). Default value: [code]1[/code].
        /// </summary>
        public float Friction
        {
            get
            {
                return GetFriction();
            }
            set
            {
                SetFriction(value);
            }
        }

        /// <summary>
        /// The body's bounciness. Default value: [code]0[/code].
        /// </summary>
        public float Bounce
        {
            get
            {
                return GetBounce();
            }
            set
            {
                SetBounce(value);
            }
        }

        /// <summary>
        /// Multiplies the gravity applied to the body. The body's gravity is calculated from the "Default Gravity" value in "Project &gt; Project Settings &gt; Physics &gt; 2d" and/or any additional gravity vector applied by [Area2D]s. Default value: [code]1[/code].
        /// </summary>
        public float GravityScale
        {
            get
            {
                return GetGravityScale();
            }
            set
            {
                SetGravityScale(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] internal force integration is disabled for this body. Aside from collision response, the body will only move as determined by the [method _integrate_forces] function.
        /// </summary>
        public bool CustomIntegrator
        {
            get
            {
                return IsUsingCustomIntegrator();
            }
            set
            {
                SetUseCustomIntegrator(value);
            }
        }

        /// <summary>
        /// Continuous collision detection mode. Default value: [code]CCD_MODE_DISABLED[/code].
        /// Continuous collision detection tries to predict where a moving body will collide instead of moving it and correcting its movement after collision. Continuous collision detection is slower, but more precise and misses fewer collisions with small, fast-moving objects. Raycasting and shapecasting methods are available. See [code]CCD_MODE_[/code] constants for details.
        /// </summary>
        public RigidBody2D.CCDMode ContinuousCd
        {
            get
            {
                return GetContinuousCollisionDetectionMode();
            }
            set
            {
                SetContinuousCollisionDetectionMode(value);
            }
        }

        /// <summary>
        /// The maximum number of contacts to report. Default value: [code]0[/code].
        /// </summary>
        public int ContactsReported
        {
            get
            {
                return GetMaxContactsReported();
            }
            set
            {
                SetMaxContactsReported(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the body will emit signals when it collides with another RigidBody2D. See also [member contacts_reported]. Default value: [code]false[/code].
        /// </summary>
        public bool ContactMonitor
        {
            get
            {
                return IsContactMonitorEnabled();
            }
            set
            {
                SetContactMonitor(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the body is sleeping and will not calculate forces until woken up by a collision or by using [method apply_impulse] or [method add_force].
        /// </summary>
        public bool Sleeping
        {
            get
            {
                return IsSleeping();
            }
            set
            {
                SetSleeping(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the body will not calculate forces and will act as a static body if there is no movement. The body will wake up when other forces are applied via collisions or by using [method apply_impulse] or [method add_force]. Default value: [code]true[/code].
        /// </summary>
        public bool CanSleep
        {
            get
            {
                return IsAbleToSleep();
            }
            set
            {
                SetCanSleep(value);
            }
        }

        /// <summary>
        /// The body's linear velocity.
        /// </summary>
        public Vector2 LinearVelocity
        {
            get
            {
                return GetLinearVelocity();
            }
            set
            {
                SetLinearVelocity(value);
            }
        }

        /// <summary>
        /// Damps the body's [member linear_velocity]. If [code]-1[/code] the body will use the "Default Linear Damp" in "Project &gt; Project Settings &gt; Physics &gt; 2d". Default value: [code]-1[/code].
        /// </summary>
        public float LinearDamp
        {
            get
            {
                return GetLinearDamp();
            }
            set
            {
                SetLinearDamp(value);
            }
        }

        /// <summary>
        /// The body's rotational velocity.
        /// </summary>
        public float AngularVelocity
        {
            get
            {
                return GetAngularVelocity();
            }
            set
            {
                SetAngularVelocity(value);
            }
        }

        /// <summary>
        /// Damps the body's [member angular_velocity]. If [code]-1[/code] the body will use the "Default Angular Damp" in "Project &gt; Project Settings &gt; Physics &gt; 2d". Default value: [code]-1[/code].
        /// </summary>
        public float AngularDamp
        {
            get
            {
                return GetAngularDamp();
            }
            set
            {
                SetAngularDamp(value);
            }
        }

        /// <summary>
        /// The body's total applied force.
        /// </summary>
        public Vector2 AppliedForce
        {
            get
            {
                return GetAppliedForce();
            }
            set
            {
                SetAppliedForce(value);
            }
        }

        /// <summary>
        /// The body's total applied torque.
        /// </summary>
        public float AppliedTorque
        {
            get
            {
                return GetAppliedTorque();
            }
            set
            {
                SetAppliedTorque(value);
            }
        }


        private const string nativeName = "RigidBody2D";

        public RigidBody2D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_RigidBody2D_Ctor(this);
        }

        internal RigidBody2D(bool memoryOwn) : base(memoryOwn) {}

        /// <summary>
        /// Allows you to read and safely modify the simulation state for the object. Use this instead of [Node._physics_process] if you need to directly change the body's [code]position[/code] or other physics properties. By default it works in addition to the usual physics behavior, but [member custom_integrator] allows you to disable the default behavior and write custom force integration for a body.
        /// </summary>
        [GodotMethod("_integrate_forces")]
        public virtual void _IntegrateForces(Physics2DDirectBodyState state)
        {
            return;
        }

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_mode");

        [GodotMethod("set_mode")]
        public void SetMode(RigidBody2D.ModeEnum mode)
        {
            NativeCalls.godot_icall_1_738(method_bind_0, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mode");

        [GodotMethod("get_mode")]
        public RigidBody2D.ModeEnum GetMode()
        {
            return NativeCalls.godot_icall_0_739(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_mass");

        [GodotMethod("set_mass")]
        public void SetMass(float mass)
        {
            NativeCalls.godot_icall_1_10(method_bind_2, Object.GetPtr(this), mass);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mass");

        [GodotMethod("get_mass")]
        public float GetMass()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_inertia");

        [GodotMethod("get_inertia")]
        public float GetInertia()
        {
            return NativeCalls.godot_icall_0_9(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_inertia");

        [GodotMethod("set_inertia")]
        public void SetInertia(float inertia)
        {
            NativeCalls.godot_icall_1_10(method_bind_5, Object.GetPtr(this), inertia);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_weight");

        [GodotMethod("set_weight")]
        public void SetWeight(float weight)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), weight);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_weight");

        [GodotMethod("get_weight")]
        public float GetWeight()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_friction");

        [GodotMethod("set_friction")]
        public void SetFriction(float friction)
        {
            NativeCalls.godot_icall_1_10(method_bind_8, Object.GetPtr(this), friction);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_friction");

        [GodotMethod("get_friction")]
        public float GetFriction()
        {
            return NativeCalls.godot_icall_0_9(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bounce");

        [GodotMethod("set_bounce")]
        public void SetBounce(float bounce)
        {
            NativeCalls.godot_icall_1_10(method_bind_10, Object.GetPtr(this), bounce);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bounce");

        [GodotMethod("get_bounce")]
        public float GetBounce()
        {
            return NativeCalls.godot_icall_0_9(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_gravity_scale");

        [GodotMethod("set_gravity_scale")]
        public void SetGravityScale(float gravityScale)
        {
            NativeCalls.godot_icall_1_10(method_bind_12, Object.GetPtr(this), gravityScale);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_gravity_scale");

        [GodotMethod("get_gravity_scale")]
        public float GetGravityScale()
        {
            return NativeCalls.godot_icall_0_9(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_linear_damp");

        [GodotMethod("set_linear_damp")]
        public void SetLinearDamp(float linearDamp)
        {
            NativeCalls.godot_icall_1_10(method_bind_14, Object.GetPtr(this), linearDamp);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_linear_damp");

        [GodotMethod("get_linear_damp")]
        public float GetLinearDamp()
        {
            return NativeCalls.godot_icall_0_9(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_angular_damp");

        [GodotMethod("set_angular_damp")]
        public void SetAngularDamp(float angularDamp)
        {
            NativeCalls.godot_icall_1_10(method_bind_16, Object.GetPtr(this), angularDamp);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_angular_damp");

        [GodotMethod("get_angular_damp")]
        public float GetAngularDamp()
        {
            return NativeCalls.godot_icall_0_9(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_linear_velocity");

        [GodotMethod("set_linear_velocity")]
        public void SetLinearVelocity(Vector2 linearVelocity)
        {
            NativeCalls.godot_icall_1_45(method_bind_18, Object.GetPtr(this), ref linearVelocity);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_linear_velocity");

        [GodotMethod("get_linear_velocity")]
        public Vector2 GetLinearVelocity()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_19, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_angular_velocity");

        [GodotMethod("set_angular_velocity")]
        public void SetAngularVelocity(float angularVelocity)
        {
            NativeCalls.godot_icall_1_10(method_bind_20, Object.GetPtr(this), angularVelocity);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_angular_velocity");

        [GodotMethod("get_angular_velocity")]
        public float GetAngularVelocity()
        {
            return NativeCalls.godot_icall_0_9(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_max_contacts_reported");

        [GodotMethod("set_max_contacts_reported")]
        public void SetMaxContactsReported(int amount)
        {
            NativeCalls.godot_icall_1_0(method_bind_22, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_max_contacts_reported");

        [GodotMethod("get_max_contacts_reported")]
        public int GetMaxContactsReported()
        {
            return NativeCalls.godot_icall_0_1(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_use_custom_integrator");

        [GodotMethod("set_use_custom_integrator")]
        public void SetUseCustomIntegrator(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_24, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_using_custom_integrator");

        [GodotMethod("is_using_custom_integrator")]
        public bool IsUsingCustomIntegrator()
        {
            return NativeCalls.godot_icall_0_3(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_contact_monitor");

        [GodotMethod("set_contact_monitor")]
        public void SetContactMonitor(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_26, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_contact_monitor_enabled");

        [GodotMethod("is_contact_monitor_enabled")]
        public bool IsContactMonitorEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_27, Object.GetPtr(this));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_continuous_collision_detection_mode");

        [GodotMethod("set_continuous_collision_detection_mode")]
        public void SetContinuousCollisionDetectionMode(RigidBody2D.CCDMode mode)
        {
            NativeCalls.godot_icall_1_740(method_bind_28, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_continuous_collision_detection_mode");

        [GodotMethod("get_continuous_collision_detection_mode")]
        public RigidBody2D.CCDMode GetContinuousCollisionDetectionMode()
        {
            return NativeCalls.godot_icall_0_741(method_bind_29, Object.GetPtr(this));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_axis_velocity");

        /// <summary>
        /// Sets the body's velocity on the given axis. The velocity in the given vector axis will be set as the given vector length. This is useful for jumping behavior.
        /// </summary>
        [GodotMethod("set_axis_velocity")]
        public void SetAxisVelocity(Vector2 axisVelocity)
        {
            NativeCalls.godot_icall_1_45(method_bind_30, Object.GetPtr(this), ref axisVelocity);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "apply_impulse");

        /// <summary>
        /// Applies a positioned impulse to the body (which will be affected by the body mass and shape). This is the equivalent of hitting a billiard ball with a cue: a force that is applied instantaneously. Both the impulse and the offset from the body origin are in global coordinates.
        /// </summary>
        [GodotMethod("apply_impulse")]
        public void ApplyImpulse(Vector2 offset, Vector2 impulse)
        {
            NativeCalls.godot_icall_2_742(method_bind_31, Object.GetPtr(this), ref offset, ref impulse);
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_applied_force");

        [GodotMethod("set_applied_force")]
        public void SetAppliedForce(Vector2 force)
        {
            NativeCalls.godot_icall_1_45(method_bind_32, Object.GetPtr(this), ref force);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_applied_force");

        [GodotMethod("get_applied_force")]
        public Vector2 GetAppliedForce()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_33, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_applied_torque");

        [GodotMethod("set_applied_torque")]
        public void SetAppliedTorque(float torque)
        {
            NativeCalls.godot_icall_1_10(method_bind_34, Object.GetPtr(this), torque);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_applied_torque");

        [GodotMethod("get_applied_torque")]
        public float GetAppliedTorque()
        {
            return NativeCalls.godot_icall_0_9(method_bind_35, Object.GetPtr(this));
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_force");

        /// <summary>
        /// Adds a positioned force to the body. Both the force and the offset from the body origin are in global coordinates.
        /// </summary>
        [GodotMethod("add_force")]
        public void AddForce(Vector2 offset, Vector2 force)
        {
            NativeCalls.godot_icall_2_742(method_bind_36, Object.GetPtr(this), ref offset, ref force);
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_sleeping");

        [GodotMethod("set_sleeping")]
        public void SetSleeping(bool sleeping)
        {
            NativeCalls.godot_icall_1_11(method_bind_37, Object.GetPtr(this), sleeping);
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_sleeping");

        [GodotMethod("is_sleeping")]
        public bool IsSleeping()
        {
            return NativeCalls.godot_icall_0_3(method_bind_38, Object.GetPtr(this));
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_can_sleep");

        [GodotMethod("set_can_sleep")]
        public void SetCanSleep(bool ableToSleep)
        {
            NativeCalls.godot_icall_1_11(method_bind_39, Object.GetPtr(this), ableToSleep);
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_able_to_sleep");

        [GodotMethod("is_able_to_sleep")]
        public bool IsAbleToSleep()
        {
            return NativeCalls.godot_icall_0_3(method_bind_40, Object.GetPtr(this));
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "test_motion");

        /// <summary>
        /// Returns [code]true[/code] if a collision would result from moving in the given vector. [code]margin[/code] increases the size of the shapes involved in the collision detection, and [code]result[/code] is an object of type [Physics2DTestMotionResult], which contains additional information about the collision (should there be one).
        /// </summary>
        [GodotMethod("test_motion")]
        public bool TestMotion(Vector2 motion, float margin = 0.08f, Physics2DTestMotionResult result = null)
        {
            return NativeCalls.godot_icall_3_743(method_bind_41, Object.GetPtr(this), ref motion, margin, Object.GetPtr(result));
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_colliding_bodies");

        /// <summary>
        /// Returns a list of the bodies colliding with this one. Use [member contacts_reported] to set the maximum number reported. You must also set [member contact_monitor] to [code]true[/code]. Note that the result of this test is not immediate after moving objects. For performance, list of collisions is updated once per frame and before the physics step. Consider using signals instead.
        /// </summary>
        [GodotMethod("get_colliding_bodies")]
        public object[] GetCollidingBodies()
        {
            return NativeCalls.godot_icall_0_21(method_bind_42, Object.GetPtr(this));
        }
    }
}
