using System;

namespace Godot
{
    /// <summary>
    /// A class stored as a resource. The script exends the functionality of all objects that instance it.
    /// The 'new' method of a script subclass creates a new instance. [method Object.set_script] extends an existing object, if that object's class matches one of the script's base classes.
    /// </summary>
    public class Script : Resource
    {
        /// <summary>
        /// The script source code, or an empty string if source code is not available. When set, does not reload the class implementation automatically.
        /// </summary>
        public string SourceCode
        {
            get
            {
                return GetSourceCode();
            }
            set
            {
                SetSourceCode(value);
            }
        }


        private const string nativeName = "Script";

        internal Script() {}

        internal Script(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "can_instance");

        /// <summary>
        /// Returns true if the script can be instanced.
        /// </summary>
        [GodotMethod("can_instance")]
        public bool CanInstance()
        {
            return NativeCalls.godot_icall_0_3(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "instance_has");

        /// <summary>
        /// Returns true if 'base_object' is an instance of this script.
        /// </summary>
        [GodotMethod("instance_has")]
        public bool InstanceHas(Object baseObject)
        {
            return NativeCalls.godot_icall_1_105(method_bind_1, Object.GetPtr(this), Object.GetPtr(baseObject));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_source_code");

        /// <summary>
        /// Returns true if the script contains non-empty source code.
        /// </summary>
        [GodotMethod("has_source_code")]
        public bool HasSourceCode()
        {
            return NativeCalls.godot_icall_0_3(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_source_code");

        [GodotMethod("get_source_code")]
        public string GetSourceCode()
        {
            return NativeCalls.godot_icall_0_2(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_source_code");

        [GodotMethod("set_source_code")]
        public void SetSourceCode(string source)
        {
            NativeCalls.godot_icall_1_43(method_bind_4, Object.GetPtr(this), source);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "reload");

        /// <summary>
        /// Reloads the script's class implementation. Returns an error code.
        /// </summary>
        [GodotMethod("reload")]
        public Error Reload(bool keepState = false)
        {
            return NativeCalls.godot_icall_1_576(method_bind_5, Object.GetPtr(this), keepState);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_base_script");

        /// <summary>
        /// </summary>
        [GodotMethod("get_base_script")]
        public Script GetBaseScript()
        {
            return NativeCalls.godot_icall_0_754(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_instance_base_type");

        /// <summary>
        /// </summary>
        [GodotMethod("get_instance_base_type")]
        public string GetInstanceBaseType()
        {
            return NativeCalls.godot_icall_0_2(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_script_signal");

        /// <summary>
        /// Returns true if the script, or a base class, defines a signal with the given name.
        /// </summary>
        [GodotMethod("has_script_signal")]
        public bool HasScriptSignal(string signalName)
        {
            return NativeCalls.godot_icall_1_70(method_bind_8, Object.GetPtr(this), signalName);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_tool");

        /// <summary>
        /// Returns true if the script is a tool script. A tool script can run in the editor.
        /// </summary>
        [GodotMethod("is_tool")]
        public bool IsTool()
        {
            return NativeCalls.godot_icall_0_3(method_bind_9, Object.GetPtr(this));
        }
    }
}
