using System;

namespace Godot
{
    /// <summary>
    /// A ScrollContainer node with a [Control] child and scrollbar child ([HScrollbar], [VScrollBar], or both) will only draw the Control within the ScrollContainer area.  Scrollbars will automatically be drawn at the right (for vertical) or bottom (for horizontal) and will enable dragging to move the viewable Control (and its children) within the ScrollContainer.  Scrollbars will also automatically resize the grabber based on the minimum_size of the Control relative to the ScrollContainer.  Works great with a [Panel] control.  You can set EXPAND on children size flags, so they will upscale to ScrollContainer size if ScrollContainer size is bigger (scroll is invisible for chosen dimension).
    /// </summary>
    public class ScrollContainer : Container
    {
        /// <summary>
        /// If [code]true[/code], enables horizontal scrolling.
        /// </summary>
        public bool ScrollHorizontalEnabled
        {
            get
            {
                return IsHScrollEnabled();
            }
            set
            {
                SetEnableHScroll(value);
            }
        }

        /// <summary>
        /// The current horizontal scroll value.
        /// </summary>
        public int ScrollHorizontal
        {
            get
            {
                return GetHScroll();
            }
            set
            {
                SetHScroll(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], enables vertical scrolling.
        /// </summary>
        public bool ScrollVerticalEnabled
        {
            get
            {
                return IsVScrollEnabled();
            }
            set
            {
                SetEnableVScroll(value);
            }
        }

        /// <summary>
        /// The current horizontal scroll value.
        /// </summary>
        public int ScrollVertical
        {
            get
            {
                return GetVScroll();
            }
            set
            {
                SetVScroll(value);
            }
        }


        private const string nativeName = "ScrollContainer";

        public ScrollContainer() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_ScrollContainer_Ctor(this);
        }

        internal ScrollContainer(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_enable_h_scroll");

        [GodotMethod("set_enable_h_scroll")]
        public void SetEnableHScroll(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_0, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_h_scroll_enabled");

        [GodotMethod("is_h_scroll_enabled")]
        public bool IsHScrollEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_enable_v_scroll");

        [GodotMethod("set_enable_v_scroll")]
        public void SetEnableVScroll(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_2, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_v_scroll_enabled");

        [GodotMethod("is_v_scroll_enabled")]
        public bool IsVScrollEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_h_scroll");

        [GodotMethod("set_h_scroll")]
        public void SetHScroll(int val)
        {
            NativeCalls.godot_icall_1_0(method_bind_4, Object.GetPtr(this), val);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_h_scroll");

        [GodotMethod("get_h_scroll")]
        public int GetHScroll()
        {
            return NativeCalls.godot_icall_0_1(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_v_scroll");

        [GodotMethod("set_v_scroll")]
        public void SetVScroll(int val)
        {
            NativeCalls.godot_icall_1_0(method_bind_6, Object.GetPtr(this), val);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_v_scroll");

        [GodotMethod("get_v_scroll")]
        public int GetVScroll()
        {
            return NativeCalls.godot_icall_0_1(method_bind_7, Object.GetPtr(this));
        }
    }
}
