using System;

namespace Godot
{
    /// <summary>
    /// Skeleton provides a hierarchical interface for managing bones, including pose, rest and animation (see [Animation]). Skeleton will support rag doll dynamics in the future.
    /// </summary>
    public class Skeleton : Spatial
    {
        public const int NotificationUpdateSkeleton = 50;

        private const string nativeName = "Skeleton";

        public Skeleton() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Skeleton_Ctor(this);
        }

        internal Skeleton(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_bone");

        /// <summary>
        /// Add a bone, with name "name". [method get_bone_count] will become the bone index.
        /// </summary>
        [GodotMethod("add_bone")]
        public void AddBone(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_0, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "find_bone");

        /// <summary>
        /// Return the bone index that matches "name" as its name.
        /// </summary>
        [GodotMethod("find_bone")]
        public int FindBone(string name)
        {
            return NativeCalls.godot_icall_1_87(method_bind_1, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bone_name");

        /// <summary>
        /// Return the name of the bone at index "index"
        /// </summary>
        [GodotMethod("get_bone_name")]
        public string GetBoneName(int boneIdx)
        {
            return NativeCalls.godot_icall_1_108(method_bind_2, Object.GetPtr(this), boneIdx);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bone_parent");

        /// <summary>
        /// Return the bone index which is the parent of the bone at "bone_idx". If -1, then bone has no parent. Note that the parent bone returned will always be less than "bone_idx".
        /// </summary>
        [GodotMethod("get_bone_parent")]
        public int GetBoneParent(int boneIdx)
        {
            return NativeCalls.godot_icall_1_6(method_bind_3, Object.GetPtr(this), boneIdx);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bone_parent");

        /// <summary>
        /// Set the bone index "parent_idx" as the parent of the bone at "bone_idx". If -1, then bone has no parent. Note: "parent_idx" must be less than "bone_idx".
        /// </summary>
        [GodotMethod("set_bone_parent")]
        public void SetBoneParent(int boneIdx, int parentIdx)
        {
            NativeCalls.godot_icall_2_32(method_bind_4, Object.GetPtr(this), boneIdx, parentIdx);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bone_count");

        /// <summary>
        /// Return the amount of bones in the skeleton.
        /// </summary>
        [GodotMethod("get_bone_count")]
        public int GetBoneCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "unparent_bone_and_rest");

        /// <summary>
        /// </summary>
        [GodotMethod("unparent_bone_and_rest")]
        public void UnparentBoneAndRest(int boneIdx)
        {
            NativeCalls.godot_icall_1_0(method_bind_6, Object.GetPtr(this), boneIdx);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bone_rest");

        /// <summary>
        /// Return the rest transform for a bone "bone_idx".
        /// </summary>
        [GodotMethod("get_bone_rest")]
        public Transform GetBoneRest(int boneIdx)
        {
            object ret = NativeCalls.godot_icall_1_231(method_bind_7, Object.GetPtr(this), boneIdx);
            return (Transform)ret;
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bone_rest");

        /// <summary>
        /// Set the rest transform for bone "bone_idx"
        /// </summary>
        [GodotMethod("set_bone_rest")]
        public void SetBoneRest(int boneIdx, Transform rest)
        {
            NativeCalls.godot_icall_2_230(method_bind_8, Object.GetPtr(this), boneIdx, ref rest);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bone_disable_rest");

        /// <summary>
        /// </summary>
        [GodotMethod("set_bone_disable_rest")]
        public void SetBoneDisableRest(int boneIdx, bool disable)
        {
            NativeCalls.godot_icall_2_51(method_bind_9, Object.GetPtr(this), boneIdx, disable);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_bone_rest_disabled");

        /// <summary>
        /// </summary>
        [GodotMethod("is_bone_rest_disabled")]
        public bool IsBoneRestDisabled(int boneIdx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_10, Object.GetPtr(this), boneIdx);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "bind_child_node_to_bone");

        /// <summary>
        /// Deprecated soon.
        /// </summary>
        [GodotMethod("bind_child_node_to_bone")]
        public void BindChildNodeToBone(int boneIdx, Node node)
        {
            NativeCalls.godot_icall_2_766(method_bind_11, Object.GetPtr(this), boneIdx, Object.GetPtr(node));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "unbind_child_node_from_bone");

        /// <summary>
        /// Deprecated soon.
        /// </summary>
        [GodotMethod("unbind_child_node_from_bone")]
        public void UnbindChildNodeFromBone(int boneIdx, Node node)
        {
            NativeCalls.godot_icall_2_766(method_bind_12, Object.GetPtr(this), boneIdx, Object.GetPtr(node));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bound_child_nodes_to_bone");

        /// <summary>
        /// Deprecated soon.
        /// </summary>
        [GodotMethod("get_bound_child_nodes_to_bone")]
        public object[] GetBoundChildNodesToBone(int boneIdx)
        {
            return NativeCalls.godot_icall_1_116(method_bind_13, Object.GetPtr(this), boneIdx);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_bones");

        /// <summary>
        /// Clear all the bones in this skeleton.
        /// </summary>
        [GodotMethod("clear_bones")]
        public void ClearBones()
        {
            NativeCalls.godot_icall_0_12(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bone_pose");

        /// <summary>
        /// Return the pose transform for bone "bone_idx".
        /// </summary>
        [GodotMethod("get_bone_pose")]
        public Transform GetBonePose(int boneIdx)
        {
            object ret = NativeCalls.godot_icall_1_231(method_bind_15, Object.GetPtr(this), boneIdx);
            return (Transform)ret;
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bone_pose");

        /// <summary>
        /// Return the pose transform for bone "bone_idx".
        /// </summary>
        [GodotMethod("set_bone_pose")]
        public void SetBonePose(int boneIdx, Transform pose)
        {
            NativeCalls.godot_icall_2_230(method_bind_16, Object.GetPtr(this), boneIdx, ref pose);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bone_global_pose");

        /// <summary>
        /// </summary>
        [GodotMethod("set_bone_global_pose")]
        public void SetBoneGlobalPose(int boneIdx, Transform pose)
        {
            NativeCalls.godot_icall_2_230(method_bind_17, Object.GetPtr(this), boneIdx, ref pose);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bone_global_pose");

        /// <summary>
        /// </summary>
        [GodotMethod("get_bone_global_pose")]
        public Transform GetBoneGlobalPose(int boneIdx)
        {
            object ret = NativeCalls.godot_icall_1_231(method_bind_18, Object.GetPtr(this), boneIdx);
            return (Transform)ret;
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bone_custom_pose");

        /// <summary>
        /// </summary>
        [GodotMethod("get_bone_custom_pose")]
        public Transform GetBoneCustomPose(int boneIdx)
        {
            object ret = NativeCalls.godot_icall_1_231(method_bind_19, Object.GetPtr(this), boneIdx);
            return (Transform)ret;
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bone_custom_pose");

        /// <summary>
        /// </summary>
        [GodotMethod("set_bone_custom_pose")]
        public void SetBoneCustomPose(int boneIdx, Transform customPose)
        {
            NativeCalls.godot_icall_2_230(method_bind_20, Object.GetPtr(this), boneIdx, ref customPose);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bone_transform");

        /// <summary>
        /// </summary>
        [GodotMethod("get_bone_transform")]
        public Transform GetBoneTransform(int boneIdx)
        {
            object ret = NativeCalls.godot_icall_1_231(method_bind_21, Object.GetPtr(this), boneIdx);
            return (Transform)ret;
        }
    }
}
