using System;

namespace Godot
{
    /// <summary>
    /// Most basic 3D game object, with a 3D [Transform] and visibility settings. All other 3D game objects inherit from Spatial. Use Spatial as a parent node to move, scale, rotate and show/hide children in a 3D project.
    /// Affine operations (rotate, scale, translate) happen in parent's local coordinate system, unless the Spatial object is set as top level. Affine operations in this coordinate system correspond to direct affine operations on the Spatial's transform. The word local below refers to this coordinate system. The coordinate system that is attached to the Spatial object itself is referred to as object-local coordinate system.
    /// </summary>
    public class Spatial : Node
    {
        public const int NotificationTransformChanged = 29;
        public const int NotificationEnterWorld = 41;
        public const int NotificationExitWorld = 42;
        public const int NotificationVisibilityChanged = 43;

        /// <summary>
        /// Local space [Transform] of this node, with respect to the parent node.
        /// </summary>
        public Transform Transform
        {
            get
            {
                return GetTransform();
            }
            set
            {
                SetTransform(value);
            }
        }

        /// <summary>
        /// World space (global) [Transform] of this node.
        /// </summary>
        public Transform GlobalTransform
        {
            get
            {
                return GetGlobalTransform();
            }
            set
            {
                SetGlobalTransform(value);
            }
        }

        /// <summary>
        /// Local translation of this node.
        /// </summary>
        public Vector3 Translation
        {
            get
            {
                return GetTranslation();
            }
            set
            {
                SetTranslation(value);
            }
        }

        /// <summary>
        /// Rotation part of the local transformation, specified in terms of YXZ-Euler angles in the format (X-angle, Y-angle, Z-angle), in degrees.
        /// </summary>
        public Vector3 RotationDegrees
        {
            get
            {
                return GetRotationDegrees();
            }
            set
            {
                SetRotationDegrees(value);
            }
        }

        /// <summary>
        /// Rotation part of the local transformation, specified in terms of YXZ-Euler angles in the format (X-angle, Y-angle, Z-angle), in radians.
        /// Note that in the mathematical sense, rotation is a matrix and not a vector. The three Euler angles, which are the three indepdent parameters of the Euler-angle parametrization of the rotation matrix, are stored in a [Vector3] data structure not because the rotation is a vector, but only because [Vector3] exists as a convenient data-structure to store 3 floating point numbers. Therefore, applying affine operations on the rotation "vector" is not meaningful.
        /// </summary>
        public Vector3 Rotation
        {
            get
            {
                return GetRotation();
            }
            set
            {
                SetRotation(value);
            }
        }

        /// <summary>
        /// Scale part of the local transformation.
        /// </summary>
        public Vector3 Scale
        {
            get
            {
                return GetScale();
            }
            set
            {
                SetScale(value);
            }
        }

        /// <summary>
        /// Visibility of this node. Toggles if this node is rendered.
        /// </summary>
        public bool Visible
        {
            get
            {
                return IsVisible();
            }
            set
            {
                SetVisible(value);
            }
        }

        /// <summary>
        /// The SpatialGizmo for this node. Used for example in [EditorSpatialGizmo] as custom visualization and editing handles in Editor.
        /// </summary>
        public SpatialGizmo Gizmo
        {
            get
            {
                return GetGizmo();
            }
            set
            {
                SetGizmo(value);
            }
        }


        private const string nativeName = "Spatial";

        public Spatial() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Spatial_Ctor(this);
        }

        internal Spatial(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_transform");

        [GodotMethod("set_transform")]
        public void SetTransform(Transform local)
        {
            NativeCalls.godot_icall_1_155(method_bind_0, Object.GetPtr(this), ref local);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_transform");

        [GodotMethod("get_transform")]
        public Transform GetTransform()
        {
            object ret = NativeCalls.godot_icall_0_18(method_bind_1, Object.GetPtr(this));
            return (Transform)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_translation");

        [GodotMethod("set_translation")]
        public void SetTranslation(Vector3 translation)
        {
            NativeCalls.godot_icall_1_104(method_bind_2, Object.GetPtr(this), ref translation);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_translation");

        [GodotMethod("get_translation")]
        public Vector3 GetTranslation()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_3, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rotation");

        [GodotMethod("set_rotation")]
        public void SetRotation(Vector3 euler)
        {
            NativeCalls.godot_icall_1_104(method_bind_4, Object.GetPtr(this), ref euler);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rotation");

        [GodotMethod("get_rotation")]
        public Vector3 GetRotation()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_5, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_rotation_degrees");

        [GodotMethod("set_rotation_degrees")]
        public void SetRotationDegrees(Vector3 eulerDegrees)
        {
            NativeCalls.godot_icall_1_104(method_bind_6, Object.GetPtr(this), ref eulerDegrees);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_rotation_degrees");

        [GodotMethod("get_rotation_degrees")]
        public Vector3 GetRotationDegrees()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_7, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_scale");

        [GodotMethod("set_scale")]
        public void SetScale(Vector3 scale)
        {
            NativeCalls.godot_icall_1_104(method_bind_8, Object.GetPtr(this), ref scale);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_scale");

        [GodotMethod("get_scale")]
        public Vector3 GetScale()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_9, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_global_transform");

        [GodotMethod("set_global_transform")]
        public void SetGlobalTransform(Transform global)
        {
            NativeCalls.godot_icall_1_155(method_bind_10, Object.GetPtr(this), ref global);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_global_transform");

        [GodotMethod("get_global_transform")]
        public Transform GetGlobalTransform()
        {
            object ret = NativeCalls.godot_icall_0_18(method_bind_11, Object.GetPtr(this));
            return (Transform)ret;
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_parent_spatial");

        /// <summary>
        /// Returns the parent [code]Spatial[/code], or an empty [Object] if no parent exists or parent is not of type [code]Spatial[/code].
        /// </summary>
        [GodotMethod("get_parent_spatial")]
        public Spatial GetParentSpatial()
        {
            return NativeCalls.godot_icall_0_771(method_bind_12, Object.GetPtr(this));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_ignore_transform_notification");

        /// <summary>
        /// Set whether the node ignores notification that its transformation (global or local) changed.
        /// </summary>
        [GodotMethod("set_ignore_transform_notification")]
        public void SetIgnoreTransformNotification(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_13, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_as_toplevel");

        /// <summary>
        /// Makes the node ignore its parents transformations. Node transformations are only in global space.
        /// </summary>
        [GodotMethod("set_as_toplevel")]
        public void SetAsToplevel(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_14, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_set_as_toplevel");

        /// <summary>
        /// Returns whether this node is set as Toplevel, that is whether it ignores its parent nodes transformations.
        /// </summary>
        [GodotMethod("is_set_as_toplevel")]
        public bool IsSetAsToplevel()
        {
            return NativeCalls.godot_icall_0_3(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_world");

        /// <summary>
        /// Returns the current [World] resource this Spatial node is registered to.
        /// </summary>
        [GodotMethod("get_world")]
        public World GetWorld()
        {
            return NativeCalls.godot_icall_0_772(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "update_gizmo");

        /// <summary>
        /// Updates the [SpatialGizmo] of this node.
        /// </summary>
        [GodotMethod("update_gizmo")]
        public void UpdateGizmo()
        {
            NativeCalls.godot_icall_0_12(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_gizmo");

        [GodotMethod("set_gizmo")]
        public void SetGizmo(SpatialGizmo gizmo)
        {
            NativeCalls.godot_icall_1_24(method_bind_18, Object.GetPtr(this), Object.GetPtr(gizmo));
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_gizmo");

        [GodotMethod("get_gizmo")]
        public SpatialGizmo GetGizmo()
        {
            return NativeCalls.godot_icall_0_773(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_visible");

        [GodotMethod("set_visible")]
        public void SetVisible(bool visible)
        {
            NativeCalls.godot_icall_1_11(method_bind_20, Object.GetPtr(this), visible);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_visible");

        [GodotMethod("is_visible")]
        public bool IsVisible()
        {
            return NativeCalls.godot_icall_0_3(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_visible_in_tree");

        /// <summary>
        /// Returns whether the node is visible, taking into consideration that its parents visibility.
        /// </summary>
        [GodotMethod("is_visible_in_tree")]
        public bool IsVisibleInTree()
        {
            return NativeCalls.godot_icall_0_3(method_bind_22, Object.GetPtr(this));
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "show");

        /// <summary>
        /// Enables rendering of this node. Change Spatial Visible property to "True".
        /// </summary>
        [GodotMethod("show")]
        public void Show()
        {
            NativeCalls.godot_icall_0_12(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "hide");

        /// <summary>
        /// Disables rendering of this node. Change Spatial Visible property to false.
        /// </summary>
        [GodotMethod("hide")]
        public void Hide()
        {
            NativeCalls.godot_icall_0_12(method_bind_24, Object.GetPtr(this));
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_notify_local_transform");

        /// <summary>
        /// Set whether the node notifies about its local transformation changes. Spatial will not propagate this by default.
        /// </summary>
        [GodotMethod("set_notify_local_transform")]
        public void SetNotifyLocalTransform(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_25, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_local_transform_notification_enabled");

        /// <summary>
        /// Returns whether node notifies about its local transformation changes. Spatial will not propagate this by default.
        /// </summary>
        [GodotMethod("is_local_transform_notification_enabled")]
        public bool IsLocalTransformNotificationEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_26, Object.GetPtr(this));
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_notify_transform");

        /// <summary>
        /// Set whether the node notifies about its global and local transformation changes. Spatial will not propagate this by default.
        /// </summary>
        [GodotMethod("set_notify_transform")]
        public void SetNotifyTransform(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_27, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_transform_notification_enabled");

        /// <summary>
        /// Returns whether the node notifies about its global and local transformation changes. Spatial will not propagate this by default.
        /// </summary>
        [GodotMethod("is_transform_notification_enabled")]
        public bool IsTransformNotificationEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_28, Object.GetPtr(this));
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rotate");

        /// <summary>
        /// Rotates the local transformation around axis, a unit [Vector3], by specified angle in radians.
        /// </summary>
        [GodotMethod("rotate")]
        public void Rotate(Vector3 axis, float angle)
        {
            NativeCalls.godot_icall_2_774(method_bind_29, Object.GetPtr(this), ref axis, angle);
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "global_rotate");

        /// <summary>
        /// Rotates the global (world) transformation around axis, a unit [Vector3], by specified angle in radians. The rotation axis is in global coordinate system.
        /// </summary>
        [GodotMethod("global_rotate")]
        public void GlobalRotate(Vector3 axis, float angle)
        {
            NativeCalls.godot_icall_2_774(method_bind_30, Object.GetPtr(this), ref axis, angle);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "global_scale");

        /// <summary>
        /// </summary>
        [GodotMethod("global_scale")]
        public void GlobalScale(Vector3 scale)
        {
            NativeCalls.godot_icall_1_104(method_bind_31, Object.GetPtr(this), ref scale);
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "global_translate");

        /// <summary>
        /// Moves the global (world) transformation by [Vector3] offset. The offset is in global coordinate system.
        /// </summary>
        [GodotMethod("global_translate")]
        public void GlobalTranslate(Vector3 offset)
        {
            NativeCalls.godot_icall_1_104(method_bind_32, Object.GetPtr(this), ref offset);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rotate_object_local");

        /// <summary>
        /// Rotates the local transformation around axis, a unit [Vector3], by specified angle in radians. The rotation axis is in object-local coordinate system.
        /// </summary>
        [GodotMethod("rotate_object_local")]
        public void RotateObjectLocal(Vector3 axis, float angle)
        {
            NativeCalls.godot_icall_2_774(method_bind_33, Object.GetPtr(this), ref axis, angle);
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "scale_object_local");

        /// <summary>
        /// Scales the local transformation by given 3D scale factors in object-local coordinate system.
        /// </summary>
        [GodotMethod("scale_object_local")]
        public void ScaleObjectLocal(Vector3 scale)
        {
            NativeCalls.godot_icall_1_104(method_bind_34, Object.GetPtr(this), ref scale);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "translate_object_local");

        /// <summary>
        /// </summary>
        [GodotMethod("translate_object_local")]
        public void TranslateObjectLocal(Vector3 offset)
        {
            NativeCalls.godot_icall_1_104(method_bind_35, Object.GetPtr(this), ref offset);
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rotate_x");

        /// <summary>
        /// Rotates the local transformation around the X axis by angle in radians
        /// </summary>
        [GodotMethod("rotate_x")]
        public void RotateX(float angle)
        {
            NativeCalls.godot_icall_1_10(method_bind_36, Object.GetPtr(this), angle);
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rotate_y");

        /// <summary>
        /// Rotates the local transformation around the Y axis by angle in radians.
        /// </summary>
        [GodotMethod("rotate_y")]
        public void RotateY(float angle)
        {
            NativeCalls.godot_icall_1_10(method_bind_37, Object.GetPtr(this), angle);
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rotate_z");

        /// <summary>
        /// Rotates the local transformation around the Z axis by angle in radians.
        /// </summary>
        [GodotMethod("rotate_z")]
        public void RotateZ(float angle)
        {
            NativeCalls.godot_icall_1_10(method_bind_38, Object.GetPtr(this), angle);
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "translate");

        /// <summary>
        /// Changes the node's position by given offset [Vector3].
        /// </summary>
        [GodotMethod("translate")]
        public void Translate(Vector3 offset)
        {
            NativeCalls.godot_icall_1_104(method_bind_39, Object.GetPtr(this), ref offset);
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "orthonormalize");

        /// <summary>
        /// Resets this node's transformations (like scale, skew and taper) preserving its rotation and translation by performing Gram-Schmidt orthonormalization on this node's [Transform3D].
        /// </summary>
        [GodotMethod("orthonormalize")]
        public void Orthonormalize()
        {
            NativeCalls.godot_icall_0_12(method_bind_40, Object.GetPtr(this));
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_identity");

        /// <summary>
        /// Reset all transformations for this node. Set its [Transform3D] to identity matrix.
        /// </summary>
        [GodotMethod("set_identity")]
        public void SetIdentity()
        {
            NativeCalls.godot_icall_0_12(method_bind_41, Object.GetPtr(this));
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "look_at");

        /// <summary>
        /// Rotates itself to point into direction of target position. Operations take place in global space.
        /// </summary>
        [GodotMethod("look_at")]
        public void LookAt(Vector3 target, Vector3 up)
        {
            NativeCalls.godot_icall_2_657(method_bind_42, Object.GetPtr(this), ref target, ref up);
        }

        private IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "look_at_from_position");

        /// <summary>
        /// Moves the node to specified position and then rotates itself to point into direction of target position. Operations take place in global space.
        /// </summary>
        [GodotMethod("look_at_from_position")]
        public void LookAtFromPosition(Vector3 position, Vector3 target, Vector3 up)
        {
            NativeCalls.godot_icall_3_775(method_bind_43, Object.GetPtr(this), ref position, ref target, ref up);
        }

        private IntPtr method_bind_44 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "to_local");

        /// <summary>
        /// Transforms [Vector3] "global_point" from world space to this node's local space.
        /// </summary>
        [GodotMethod("to_local")]
        public Vector3 ToLocal(Vector3 globalPoint)
        {
            object ret = NativeCalls.godot_icall_1_35(method_bind_44, Object.GetPtr(this), ref globalPoint);
            return (Vector3)ret;
        }

        private IntPtr method_bind_45 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "to_global");

        /// <summary>
        /// Transforms [Vector3] "local_point" from this node's local space to world space.
        /// </summary>
        [GodotMethod("to_global")]
        public Vector3 ToGlobal(Vector3 localPoint)
        {
            object ret = NativeCalls.godot_icall_1_35(method_bind_45, Object.GetPtr(this), ref localPoint);
            return (Vector3)ret;
        }
    }
}
