using System;

namespace Godot
{
    /// <summary>
    /// A node that displays 2D texture information in a 3D environment.
    /// </summary>
    public class SpriteBase3D : GeometryInstance
    {
        public enum DrawFlags
        {
            Transparent = 0,
            Shaded = 1,
            DoubleSided = 2,
            Max = 3
        }

        public enum AlphaCutMode
        {
            Disabled = 0,
            Discard = 1,
            OpaquePrepass = 2
        }


        /// <summary>
        /// If [code]true[/code] texture will be centered. Default value: [code]true[/code].
        /// </summary>
        public bool Centered
        {
            get
            {
                return IsCentered();
            }
            set
            {
                SetCentered(value);
            }
        }

        /// <summary>
        /// The texture's drawing offset.
        /// </summary>
        public Vector2 Offset
        {
            get
            {
                return GetOffset();
            }
            set
            {
                SetOffset(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] texture is flipped horizontally. Default value: [code]false[/code].
        /// </summary>
        public bool FlipH
        {
            get
            {
                return IsFlippedH();
            }
            set
            {
                SetFlipH(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] texture is flipped vertically. Default value: [code]false[/code].
        /// </summary>
        public bool FlipV
        {
            get
            {
                return IsFlippedV();
            }
            set
            {
                SetFlipV(value);
            }
        }

        /// <summary>
        /// A color value that gets multiplied on, could be used for mood-coloring or to simulate the color of light.
        /// </summary>
        public Color Modulate
        {
            get
            {
                return GetModulate();
            }
            set
            {
                SetModulate(value);
            }
        }

        /// <summary>
        /// The objects visibility on a scale from [code]0[/code] fully invisible to [code]1[/code] fully visible.
        /// </summary>
        public float Opacity
        {
            get
            {
                return GetOpacity();
            }
            set
            {
                SetOpacity(value);
            }
        }

        /// <summary>
        /// The size of one pixel's width on the Sprite to scale it in 3D.
        /// </summary>
        public float PixelSize
        {
            get
            {
                return GetPixelSize();
            }
            set
            {
                SetPixelSize(value);
            }
        }

        /// <summary>
        /// The direction in which the front of the texture faces.
        /// </summary>
        public Vector3.Axis Axis
        {
            get
            {
                return GetAxis();
            }
            set
            {
                SetAxis(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the texture's transparency and the opacity are used to make those parts of the Sprite invisible. Default value: [code]true[/code].
        /// </summary>
        public bool Transparent
        {
            get
            {
                return GetDrawFlag((SpriteBase3D.DrawFlags)0);
            }
            set
            {
                SetDrawFlag((SpriteBase3D.DrawFlags)0, value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the [Light] in the [Environment] has effects on the Sprite. Default value: [code]false[/code].
        /// </summary>
        public bool Shaded
        {
            get
            {
                return GetDrawFlag((SpriteBase3D.DrawFlags)1);
            }
            set
            {
                SetDrawFlag((SpriteBase3D.DrawFlags)1, value);
            }
        }

        /// <summary>
        /// If [code]true[/code] texture can be seen from the back as well, if [code]false[/code], it is invisible when looking at it from behind. Default value: [code]true[/code].
        /// </summary>
        public bool DoubleSided
        {
            get
            {
                return GetDrawFlag((SpriteBase3D.DrawFlags)2);
            }
            set
            {
                SetDrawFlag((SpriteBase3D.DrawFlags)2, value);
            }
        }

        /// <summary>
        /// </summary>
        public SpriteBase3D.AlphaCutMode AlphaCut
        {
            get
            {
                return GetAlphaCutMode();
            }
            set
            {
                SetAlphaCutMode(value);
            }
        }


        private const string nativeName = "SpriteBase3D";

        internal SpriteBase3D() {}

        internal SpriteBase3D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_centered");

        [GodotMethod("set_centered")]
        public void SetCentered(bool centered)
        {
            NativeCalls.godot_icall_1_11(method_bind_0, Object.GetPtr(this), centered);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_centered");

        [GodotMethod("is_centered")]
        public bool IsCentered()
        {
            return NativeCalls.godot_icall_0_3(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_offset");

        [GodotMethod("set_offset")]
        public void SetOffset(Vector2 offset)
        {
            NativeCalls.godot_icall_1_45(method_bind_2, Object.GetPtr(this), ref offset);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_offset");

        [GodotMethod("get_offset")]
        public Vector2 GetOffset()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_3, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_flip_h");

        [GodotMethod("set_flip_h")]
        public void SetFlipH(bool flipH)
        {
            NativeCalls.godot_icall_1_11(method_bind_4, Object.GetPtr(this), flipH);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_flipped_h");

        [GodotMethod("is_flipped_h")]
        public bool IsFlippedH()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_flip_v");

        [GodotMethod("set_flip_v")]
        public void SetFlipV(bool flipV)
        {
            NativeCalls.godot_icall_1_11(method_bind_6, Object.GetPtr(this), flipV);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_flipped_v");

        [GodotMethod("is_flipped_v")]
        public bool IsFlippedV()
        {
            return NativeCalls.godot_icall_0_3(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_modulate");

        [GodotMethod("set_modulate")]
        public void SetModulate(Color modulate)
        {
            NativeCalls.godot_icall_1_202(method_bind_8, Object.GetPtr(this), ref modulate);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_modulate");

        [GodotMethod("get_modulate")]
        public Color GetModulate()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_9, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_opacity");

        [GodotMethod("set_opacity")]
        public void SetOpacity(float opacity)
        {
            NativeCalls.godot_icall_1_10(method_bind_10, Object.GetPtr(this), opacity);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_opacity");

        [GodotMethod("get_opacity")]
        public float GetOpacity()
        {
            return NativeCalls.godot_icall_0_9(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_pixel_size");

        [GodotMethod("set_pixel_size")]
        public void SetPixelSize(float pixelSize)
        {
            NativeCalls.godot_icall_1_10(method_bind_12, Object.GetPtr(this), pixelSize);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_pixel_size");

        [GodotMethod("get_pixel_size")]
        public float GetPixelSize()
        {
            return NativeCalls.godot_icall_0_9(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_axis");

        [GodotMethod("set_axis")]
        public void SetAxis(Vector3.Axis axis)
        {
            NativeCalls.godot_icall_1_803(method_bind_14, Object.GetPtr(this), axis);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_axis");

        [GodotMethod("get_axis")]
        public Vector3.Axis GetAxis()
        {
            return NativeCalls.godot_icall_0_804(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_draw_flag");

        [GodotMethod("set_draw_flag")]
        public void SetDrawFlag(SpriteBase3D.DrawFlags flag, bool enabled)
        {
            NativeCalls.godot_icall_2_805(method_bind_16, Object.GetPtr(this), flag, enabled);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_draw_flag");

        [GodotMethod("get_draw_flag")]
        public bool GetDrawFlag(SpriteBase3D.DrawFlags flag)
        {
            return NativeCalls.godot_icall_1_806(method_bind_17, Object.GetPtr(this), flag);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_alpha_cut_mode");

        [GodotMethod("set_alpha_cut_mode")]
        public void SetAlphaCutMode(SpriteBase3D.AlphaCutMode mode)
        {
            NativeCalls.godot_icall_1_807(method_bind_18, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_alpha_cut_mode");

        [GodotMethod("get_alpha_cut_mode")]
        public SpriteBase3D.AlphaCutMode GetAlphaCutMode()
        {
            return NativeCalls.godot_icall_0_808(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_rect");

        /// <summary>
        /// </summary>
        [GodotMethod("get_item_rect")]
        public Rect2 GetItemRect()
        {
            object ret = NativeCalls.godot_icall_0_121(method_bind_20, Object.GetPtr(this));
            return (Rect2)ret;
        }
    }
}
