using System;

namespace Godot
{
    /// <summary>
    /// Static body for 3D Physics. A static body is a simple body that is not intended to move. They don't consume any CPU resources in contrast to a [RigidBody3D] so they are great for scenario collision.
    /// A static body can also be animated by using simulated motion mode. This is useful for implementing functionalities such as moving platforms. When this mode is active the body can be animated and automatically computes linear and angular velocity to apply in that frame and to influence other bodies.
    /// Alternatively, a constant linear or angular velocity can be set for the static body, so even if it doesn't move, it affects other bodies as if it was moving (this is useful for simulating conveyor belts or conveyor wheels).
    /// </summary>
    public class StaticBody : PhysicsBody
    {
        /// <summary>
        /// The body friction, from 0 (frictionless) to 1 (full friction).
        /// </summary>
        public float Friction
        {
            get
            {
                return GetFriction();
            }
            set
            {
                SetFriction(value);
            }
        }

        /// <summary>
        /// The body bounciness.
        /// </summary>
        public float Bounce
        {
            get
            {
                return GetBounce();
            }
            set
            {
                SetBounce(value);
            }
        }

        /// <summary>
        /// The constant linear velocity for the body. This does not move the body, but affects other bodies that touch it, as if it was in a state of movement.
        /// </summary>
        public Vector3 ConstantLinearVelocity
        {
            get
            {
                return GetConstantLinearVelocity();
            }
            set
            {
                SetConstantLinearVelocity(value);
            }
        }

        /// <summary>
        /// The constant angular velocity for the body. This does not rotate the body, but affects other bodies that touch it, as if it was in a state of rotation.
        /// </summary>
        public Vector3 ConstantAngularVelocity
        {
            get
            {
                return GetConstantAngularVelocity();
            }
            set
            {
                SetConstantAngularVelocity(value);
            }
        }


        private const string nativeName = "StaticBody";

        public StaticBody() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_StaticBody_Ctor(this);
        }

        internal StaticBody(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_constant_linear_velocity");

        [GodotMethod("set_constant_linear_velocity")]
        public void SetConstantLinearVelocity(Vector3 vel)
        {
            NativeCalls.godot_icall_1_104(method_bind_0, Object.GetPtr(this), ref vel);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_constant_angular_velocity");

        [GodotMethod("set_constant_angular_velocity")]
        public void SetConstantAngularVelocity(Vector3 vel)
        {
            NativeCalls.godot_icall_1_104(method_bind_1, Object.GetPtr(this), ref vel);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_constant_linear_velocity");

        [GodotMethod("get_constant_linear_velocity")]
        public Vector3 GetConstantLinearVelocity()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_2, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_constant_angular_velocity");

        [GodotMethod("get_constant_angular_velocity")]
        public Vector3 GetConstantAngularVelocity()
        {
            object ret = NativeCalls.godot_icall_0_4(method_bind_3, Object.GetPtr(this));
            return (Vector3)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_friction");

        [GodotMethod("set_friction")]
        public void SetFriction(float friction)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), friction);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_friction");

        [GodotMethod("get_friction")]
        public float GetFriction()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bounce");

        [GodotMethod("set_bounce")]
        public void SetBounce(float bounce)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), bounce);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bounce");

        [GodotMethod("get_bounce")]
        public float GetBounce()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }
    }
}
