using System;

namespace Godot
{
    /// <summary>
    /// Static body for 2D Physics. A StaticBody2D is a body that is not intended to move. It is ideal for implementing objects in the environment, such as walls or platforms.
    /// Additionally, a constant linear or angular velocity can be set for the static body, which will affect colliding bodies as if it were moving (for example, a conveyor belt).
    /// </summary>
    public class StaticBody2D : PhysicsBody2D
    {
        /// <summary>
        /// Constant linear velocity for the body. This does not move the body, but affects colliding bodies, as if it were moving.
        /// </summary>
        public Vector2 ConstantLinearVelocity
        {
            get
            {
                return GetConstantLinearVelocity();
            }
            set
            {
                SetConstantLinearVelocity(value);
            }
        }

        /// <summary>
        /// Constant angular velocity for the body. This does not rotate the body, but affects colliding bodies, as if it were rotating.
        /// </summary>
        public float ConstantAngularVelocity
        {
            get
            {
                return GetConstantAngularVelocity();
            }
            set
            {
                SetConstantAngularVelocity(value);
            }
        }

        /// <summary>
        /// The body's friction. Values range from [code]0[/code] (no friction) to [code]1[/code] (full friction).
        /// </summary>
        public float Friction
        {
            get
            {
                return GetFriction();
            }
            set
            {
                SetFriction(value);
            }
        }

        /// <summary>
        /// The body's bounciness. Values range from [code]0[/code] (no bounce) to [code]1[/code] (full bounciness).
        /// </summary>
        public float Bounce
        {
            get
            {
                return GetBounce();
            }
            set
            {
                SetBounce(value);
            }
        }


        private const string nativeName = "StaticBody2D";

        public StaticBody2D() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_StaticBody2D_Ctor(this);
        }

        internal StaticBody2D(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_constant_linear_velocity");

        [GodotMethod("set_constant_linear_velocity")]
        public void SetConstantLinearVelocity(Vector2 vel)
        {
            NativeCalls.godot_icall_1_45(method_bind_0, Object.GetPtr(this), ref vel);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_constant_angular_velocity");

        [GodotMethod("set_constant_angular_velocity")]
        public void SetConstantAngularVelocity(float vel)
        {
            NativeCalls.godot_icall_1_10(method_bind_1, Object.GetPtr(this), vel);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_constant_linear_velocity");

        [GodotMethod("get_constant_linear_velocity")]
        public Vector2 GetConstantLinearVelocity()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_2, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_constant_angular_velocity");

        [GodotMethod("get_constant_angular_velocity")]
        public float GetConstantAngularVelocity()
        {
            return NativeCalls.godot_icall_0_9(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_friction");

        [GodotMethod("set_friction")]
        public void SetFriction(float friction)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), friction);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_friction");

        [GodotMethod("get_friction")]
        public float GetFriction()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bounce");

        [GodotMethod("set_bounce")]
        public void SetBounce(float bounce)
        {
            NativeCalls.godot_icall_1_10(method_bind_6, Object.GetPtr(this), bounce);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bounce");

        [GodotMethod("get_bounce")]
        public float GetBounce()
        {
            return NativeCalls.godot_icall_0_9(method_bind_7, Object.GetPtr(this));
        }
    }
}
