using System;

namespace Godot
{
    /// <summary>
    /// This stylebox can be used to achieve all kinds of looks without the need of a texture. Those properties are customizable:
    /// - Color
    /// - Border width (individual width for each border)
    /// - Rounded corners (individual radius for each corner)
    /// - Shadow
    /// About corner radius:
    /// Setting corner radius to high values is allowed. As soon as corners would overlap the stylebox will switch to a relative system. Example:
    /// [codeblock]
    /// height = 30
    /// corner_radius_top_left = 50
    /// corner_radius_bottom_left = 100
    /// [/codeblock]
    /// The relative system now would take the 1:2 ratio of the two left corners to calculate the actual corner width. Both corners added will [b]never[/b] be more than the height. Result:
    /// [codeblock]
    /// corner_radius_top_left: 10
    /// corner_radius_bottom_left: 20
    /// [/codeblock]
    /// </summary>
    public class StyleBoxFlat : StyleBox
    {
        /// <summary>
        /// The background color of the stylebox.
        /// </summary>
        public Color BgColor
        {
            get
            {
                return GetBgColor();
            }
            set
            {
                SetBgColor(value);
            }
        }

        /// <summary>
        /// Toggels drawing of the inner part of the stylebox.
        /// </summary>
        public bool DrawCenter
        {
            get
            {
                return IsDrawCenterEnabled();
            }
            set
            {
                SetDrawCenter(value);
            }
        }

        /// <summary>
        /// Border width for the left border.
        /// </summary>
        public int BorderWidthLeft
        {
            get
            {
                return GetBorderWidth((Margin)0);
            }
            set
            {
                SetBorderWidth((Margin)0, value);
            }
        }

        /// <summary>
        /// Border width for the top border.
        /// </summary>
        public int BorderWidthTop
        {
            get
            {
                return GetBorderWidth((Margin)1);
            }
            set
            {
                SetBorderWidth((Margin)1, value);
            }
        }

        /// <summary>
        /// Border width for the right border.
        /// </summary>
        public int BorderWidthRight
        {
            get
            {
                return GetBorderWidth((Margin)2);
            }
            set
            {
                SetBorderWidth((Margin)2, value);
            }
        }

        /// <summary>
        /// Border width for the bottom border.
        /// </summary>
        public int BorderWidthBottom
        {
            get
            {
                return GetBorderWidth((Margin)3);
            }
            set
            {
                SetBorderWidth((Margin)3, value);
            }
        }

        /// <summary>
        /// Sets the color of the border.
        /// </summary>
        public Color BorderColor
        {
            get
            {
                return GetBorderColor();
            }
            set
            {
                SetBorderColor(value);
            }
        }

        /// <summary>
        /// When set to true, the border will fade into the background color.
        /// </summary>
        public bool BorderBlend
        {
            get
            {
                return GetBorderBlend();
            }
            set
            {
                SetBorderBlend(value);
            }
        }

        /// <summary>
        /// The corner radius of the top left corner. When set to 0 the corner is not rounded.
        /// </summary>
        public int CornerRadiusTopLeft
        {
            get
            {
                return GetCornerRadius((Corner)0);
            }
            set
            {
                SetCornerRadius((Corner)0, value);
            }
        }

        /// <summary>
        /// The corner radius of the top right corner. When set to 0 the corner is not rounded.
        /// </summary>
        public int CornerRadiusTopRight
        {
            get
            {
                return GetCornerRadius((Corner)1);
            }
            set
            {
                SetCornerRadius((Corner)1, value);
            }
        }

        /// <summary>
        /// The corner radius of the bottom right corner. When set to 0 the corner is not rounded.
        /// </summary>
        public int CornerRadiusBottomRight
        {
            get
            {
                return GetCornerRadius((Corner)2);
            }
            set
            {
                SetCornerRadius((Corner)2, value);
            }
        }

        /// <summary>
        /// The corner radius of the bottom left corner. When set to 0 the corner is not rounded.
        /// </summary>
        public int CornerRadiusBottomLeft
        {
            get
            {
                return GetCornerRadius((Corner)3);
            }
            set
            {
                SetCornerRadius((Corner)3, value);
            }
        }

        /// <summary>
        /// This sets the amount of vertices used for each corner. Higher values result in rounder corners but take more processing power to compute. When choosing a value you should take the corner radius ([method set_corner_radius]) into account.
        /// For corner radius smaller than 10: 4-5 should be enough
        /// For corner radius smaller than 30: 8-12 should be enough ...
        /// </summary>
        public int CornerDetail
        {
            get
            {
                return GetCornerDetail();
            }
            set
            {
                SetCornerDetail(value);
            }
        }

        /// <summary>
        /// Expands the stylebox outside of the control rect on the left edge. Useful in combination with border_width_left. To draw a border outside the control rect.
        /// </summary>
        public float ExpandMarginLeft
        {
            get
            {
                return GetExpandMargin((Margin)0);
            }
            set
            {
                SetExpandMargin((Margin)0, value);
            }
        }

        /// <summary>
        /// Expands the stylebox outside of the control rect on the right edge. Useful in combination with border_width_right. To draw a border outside the control rect.
        /// </summary>
        public float ExpandMarginRight
        {
            get
            {
                return GetExpandMargin((Margin)2);
            }
            set
            {
                SetExpandMargin((Margin)2, value);
            }
        }

        /// <summary>
        /// Expands the stylebox outside of the control rect on the top edge. Useful in combination with border_width_top. To draw a border outside the control rect.
        /// </summary>
        public float ExpandMarginTop
        {
            get
            {
                return GetExpandMargin((Margin)1);
            }
            set
            {
                SetExpandMargin((Margin)1, value);
            }
        }

        /// <summary>
        /// Expands the stylebox outside of the control rect on the bottom edge. Useful in combination with border_width_bottom. To draw a border outside the control rect.
        /// </summary>
        public float ExpandMarginBottom
        {
            get
            {
                return GetExpandMargin((Margin)3);
            }
            set
            {
                SetExpandMargin((Margin)3, value);
            }
        }

        /// <summary>
        /// The color of the shadow. (This has no effect when shadow_size &lt; 1)
        /// </summary>
        public Color ShadowColor
        {
            get
            {
                return GetShadowColor();
            }
            set
            {
                SetShadowColor(value);
            }
        }

        /// <summary>
        /// The shadow size in pixels.
        /// </summary>
        public int ShadowSize
        {
            get
            {
                return GetShadowSize();
            }
            set
            {
                SetShadowSize(value);
            }
        }

        /// <summary>
        /// Anti Aliasing draws a small ring around edges. This ring fades to transparent. As a result edges look much smoother. This is only noticeable when using rounded corners.
        /// </summary>
        public bool AntiAliasing
        {
            get
            {
                return IsAntiAliased();
            }
            set
            {
                SetAntiAliased(value);
            }
        }

        /// <summary>
        /// This changes the size of the faded ring. Higher values can be used to achieve a "blurry" effect.
        /// </summary>
        public int AntiAliasingSize
        {
            get
            {
                return GetAaSize();
            }
            set
            {
                SetAaSize(value);
            }
        }


        private const string nativeName = "StyleBoxFlat";

        public StyleBoxFlat() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_StyleBoxFlat_Ctor(this);
        }

        internal StyleBoxFlat(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_bg_color");

        [GodotMethod("set_bg_color")]
        public void SetBgColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_0, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_bg_color");

        [GodotMethod("get_bg_color")]
        public Color GetBgColor()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_1, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_border_color");

        [GodotMethod("set_border_color")]
        public void SetBorderColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_2, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_border_color");

        [GodotMethod("get_border_color")]
        public Color GetBorderColor()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_3, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_border_width_all");

        /// <summary>
        /// </summary>
        [GodotMethod("set_border_width_all")]
        public void SetBorderWidthAll(int width)
        {
            NativeCalls.godot_icall_1_0(method_bind_4, Object.GetPtr(this), width);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_border_width_min");

        /// <summary>
        /// </summary>
        [GodotMethod("get_border_width_min")]
        public int GetBorderWidthMin()
        {
            return NativeCalls.godot_icall_0_1(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_border_width");

        [GodotMethod("set_border_width")]
        public void SetBorderWidth(Margin margin, int width)
        {
            NativeCalls.godot_icall_2_196(method_bind_6, Object.GetPtr(this), margin, width);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_border_width");

        [GodotMethod("get_border_width")]
        public int GetBorderWidth(Margin margin)
        {
            return NativeCalls.godot_icall_1_197(method_bind_7, Object.GetPtr(this), margin);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_border_blend");

        [GodotMethod("set_border_blend")]
        public void SetBorderBlend(bool blend)
        {
            NativeCalls.godot_icall_1_11(method_bind_8, Object.GetPtr(this), blend);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_border_blend");

        [GodotMethod("get_border_blend")]
        public bool GetBorderBlend()
        {
            return NativeCalls.godot_icall_0_3(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_corner_radius_individual");

        /// <summary>
        /// </summary>
        [GodotMethod("set_corner_radius_individual")]
        public void SetCornerRadiusIndividual(int radiusTopLeft, int radiusTopRight, int radiusBottomRight, int radiusBottomLeft)
        {
            NativeCalls.godot_icall_4_819(method_bind_10, Object.GetPtr(this), radiusTopLeft, radiusTopRight, radiusBottomRight, radiusBottomLeft);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_corner_radius_all");

        /// <summary>
        /// </summary>
        [GodotMethod("set_corner_radius_all")]
        public void SetCornerRadiusAll(int radius)
        {
            NativeCalls.godot_icall_1_0(method_bind_11, Object.GetPtr(this), radius);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_corner_radius");

        [GodotMethod("set_corner_radius")]
        public void SetCornerRadius(Corner corner, int radius)
        {
            NativeCalls.godot_icall_2_820(method_bind_12, Object.GetPtr(this), corner, radius);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_corner_radius");

        [GodotMethod("get_corner_radius")]
        public int GetCornerRadius(Corner corner)
        {
            return NativeCalls.godot_icall_1_821(method_bind_13, Object.GetPtr(this), corner);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_expand_margin");

        [GodotMethod("set_expand_margin")]
        public void SetExpandMargin(Margin margin, float size)
        {
            NativeCalls.godot_icall_2_198(method_bind_14, Object.GetPtr(this), margin, size);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_expand_margin_all");

        /// <summary>
        /// </summary>
        [GodotMethod("set_expand_margin_all")]
        public void SetExpandMarginAll(float size)
        {
            NativeCalls.godot_icall_1_10(method_bind_15, Object.GetPtr(this), size);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_expand_margin_individual");

        /// <summary>
        /// </summary>
        [GodotMethod("set_expand_margin_individual")]
        public void SetExpandMarginIndividual(float sizeLeft, float sizeTop, float sizeRight, float sizeBottom)
        {
            NativeCalls.godot_icall_4_822(method_bind_16, Object.GetPtr(this), sizeLeft, sizeTop, sizeRight, sizeBottom);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_expand_margin");

        [GodotMethod("get_expand_margin")]
        public float GetExpandMargin(Margin margin)
        {
            return NativeCalls.godot_icall_1_199(method_bind_17, Object.GetPtr(this), margin);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_draw_center");

        [GodotMethod("set_draw_center")]
        public void SetDrawCenter(bool drawCenter)
        {
            NativeCalls.godot_icall_1_11(method_bind_18, Object.GetPtr(this), drawCenter);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_draw_center_enabled");

        [GodotMethod("is_draw_center_enabled")]
        public bool IsDrawCenterEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shadow_color");

        [GodotMethod("set_shadow_color")]
        public void SetShadowColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_20, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shadow_color");

        [GodotMethod("get_shadow_color")]
        public Color GetShadowColor()
        {
            object ret = NativeCalls.godot_icall_0_203(method_bind_21, Object.GetPtr(this));
            return (Color)ret;
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shadow_size");

        [GodotMethod("set_shadow_size")]
        public void SetShadowSize(int size)
        {
            NativeCalls.godot_icall_1_0(method_bind_22, Object.GetPtr(this), size);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shadow_size");

        [GodotMethod("get_shadow_size")]
        public int GetShadowSize()
        {
            return NativeCalls.godot_icall_0_1(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_anti_aliased");

        [GodotMethod("set_anti_aliased")]
        public void SetAntiAliased(bool antiAliased)
        {
            NativeCalls.godot_icall_1_11(method_bind_24, Object.GetPtr(this), antiAliased);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_anti_aliased");

        [GodotMethod("is_anti_aliased")]
        public bool IsAntiAliased()
        {
            return NativeCalls.godot_icall_0_3(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_aa_size");

        [GodotMethod("set_aa_size")]
        public void SetAaSize(int size)
        {
            NativeCalls.godot_icall_1_0(method_bind_26, Object.GetPtr(this), size);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_aa_size");

        [GodotMethod("get_aa_size")]
        public int GetAaSize()
        {
            return NativeCalls.godot_icall_0_1(method_bind_27, Object.GetPtr(this));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_corner_detail");

        [GodotMethod("set_corner_detail")]
        public void SetCornerDetail(int detail)
        {
            NativeCalls.godot_icall_1_0(method_bind_28, Object.GetPtr(this), detail);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_corner_detail");

        [GodotMethod("get_corner_detail")]
        public int GetCornerDetail()
        {
            return NativeCalls.godot_icall_0_1(method_bind_29, Object.GetPtr(this));
        }
    }
}
