using System;

namespace Godot
{
    /// <summary>
    /// The [code]SurfaceTool[/code] is used to construct a [Mesh] by specifying vertex attributes individually. It can be used to construct a [Mesh] from script. All properties except index need to be added before a call to [method add_vertex]. For example adding vertex colors and UVs looks like
    /// [codeblock]
    /// var st = SurfaceTool.new()
    /// st.begin(Mesh.PRIMITIVE_TRIANGLES)
    /// st.add_color(Color(1, 0, 0))
    /// st.add_uv(Vector2(0, 0))
    /// st.add_vertex(Vector3(0, 0, 0))
    /// [/codeblock]
    /// The [code]SurfaceTool[/code] now contains one vertex of a triangle which has a UV coordinate and a specified [Color]. If another vertex were added without calls to [method add_uv] or [method add_color] then the last values would be used.
    /// It is very important that vertex attributes are passed [b]before[/b] the call to [method add_vertex], failure to do this will result in an error when committing the vertex information to a mesh.
    /// </summary>
    public class SurfaceTool : Reference
    {
        private const string nativeName = "SurfaceTool";

        public SurfaceTool() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_SurfaceTool_Ctor(this);
        }

        internal SurfaceTool(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "begin");

        /// <summary>
        /// Called before adding any Vertices. Takes the primitive type as an argument (e.g. Mesh.PRIMITIVE_TRIANGLES).
        /// </summary>
        [GodotMethod("begin")]
        public void Begin(Mesh.PrimitiveType primitive)
        {
            NativeCalls.godot_icall_1_825(method_bind_0, Object.GetPtr(this), primitive);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_vertex");

        /// <summary>
        /// Specify position of current Vertex. Should be called after specifying other vertex properties (e.g. Color, UV).
        /// </summary>
        [GodotMethod("add_vertex")]
        public void AddVertex(Vector3 vertex)
        {
            NativeCalls.godot_icall_1_104(method_bind_1, Object.GetPtr(this), ref vertex);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_color");

        /// <summary>
        /// Specify a [Color] for the next Vertex to use.
        /// </summary>
        [GodotMethod("add_color")]
        public void AddColor(Color color)
        {
            NativeCalls.godot_icall_1_202(method_bind_2, Object.GetPtr(this), ref color);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_normal");

        /// <summary>
        /// Specify a normal for the next Vertex to use.
        /// </summary>
        [GodotMethod("add_normal")]
        public void AddNormal(Vector3 normal)
        {
            NativeCalls.godot_icall_1_104(method_bind_3, Object.GetPtr(this), ref normal);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_tangent");

        /// <summary>
        /// Specify a Tangent for the next Vertex to use.
        /// </summary>
        [GodotMethod("add_tangent")]
        public void AddTangent(Plane tangent)
        {
            NativeCalls.godot_icall_1_436(method_bind_4, Object.GetPtr(this), ref tangent);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_uv");

        /// <summary>
        /// Specify UV Coordinate for next Vertex to use.
        /// </summary>
        [GodotMethod("add_uv")]
        public void AddUv(Vector2 uv)
        {
            NativeCalls.godot_icall_1_45(method_bind_5, Object.GetPtr(this), ref uv);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_uv2");

        /// <summary>
        /// Specify an optional second set of UV coordinates for next Vertex to use.
        /// </summary>
        [GodotMethod("add_uv2")]
        public void AddUv2(Vector2 uv2)
        {
            NativeCalls.godot_icall_1_45(method_bind_6, Object.GetPtr(this), ref uv2);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_bones");

        /// <summary>
        /// Add an array of bones for the next Vertex to use.
        /// </summary>
        [GodotMethod("add_bones")]
        public void AddBones(int[] bones)
        {
            NativeCalls.godot_icall_1_174(method_bind_7, Object.GetPtr(this), bones);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_weights");

        /// <summary>
        /// Specify weight value for next Vertex to use.
        /// </summary>
        [GodotMethod("add_weights")]
        public void AddWeights(float[] weights)
        {
            NativeCalls.godot_icall_1_387(method_bind_8, Object.GetPtr(this), weights);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_smooth_group");

        /// <summary>
        /// Specify whether current Vertex (if using only Vertex arrays) or current index (if also using index arrays) should utilize smooth normals for normal calculation.
        /// </summary>
        [GodotMethod("add_smooth_group")]
        public void AddSmoothGroup(bool smooth)
        {
            NativeCalls.godot_icall_1_11(method_bind_9, Object.GetPtr(this), smooth);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_triangle_fan");

        /// <summary>
        /// Insert a triangle fan made of array data into [Mesh] being constructed.
        /// &lt;param name="uvs"&gt;If the param is null, then the default value is new Vector2[] {}&lt;/param&gt;
        /// &lt;param name="colors"&gt;If the param is null, then the default value is new Color[] {}&lt;/param&gt;
        /// &lt;param name="uv2s"&gt;If the param is null, then the default value is new Vector2[] {}&lt;/param&gt;
        /// &lt;param name="normals"&gt;If the param is null, then the default value is new Vector3[] {}&lt;/param&gt;
        /// &lt;param name="tangents"&gt;If the param is null, then the default value is new object[] {}&lt;/param&gt;
        /// </summary>
        [GodotMethod("add_triangle_fan")]
        public void AddTriangleFan(Vector3[] vertexes, Vector2[] uvs = null, Color[] colors = null, Vector2[] uv2s = null, Vector3[] normals = null, object[] tangents = null)
        {
            Vector2[] uvs_in = uvs != null ? uvs : new Vector2[] {};
            Color[] colors_in = colors != null ? colors : new Color[] {};
            Vector2[] uv2s_in = uv2s != null ? uv2s : new Vector2[] {};
            Vector3[] normals_in = normals != null ? normals : new Vector3[] {};
            object[] tangents_in = tangents != null ? tangents : new object[] {};
            NativeCalls.godot_icall_6_826(method_bind_10, Object.GetPtr(this), vertexes, uvs_in, colors_in, uv2s_in, normals_in, tangents_in);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_index");

        /// <summary>
        /// Adds an index to index array if you are using indexed Vertices. Does not need to be called before adding Vertex.
        /// </summary>
        [GodotMethod("add_index")]
        public void AddIndex(int index)
        {
            NativeCalls.godot_icall_1_0(method_bind_11, Object.GetPtr(this), index);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "index");

        /// <summary>
        /// Shrinks Vertex array by creating an index array. Avoids reusing Vertices.
        /// </summary>
        [GodotMethod("index")]
        public void Index()
        {
            NativeCalls.godot_icall_0_12(method_bind_12, Object.GetPtr(this));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "deindex");

        /// <summary>
        /// Removes index array by expanding Vertex array.
        /// </summary>
        [GodotMethod("deindex")]
        public void Deindex()
        {
            NativeCalls.godot_icall_0_12(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "generate_normals");

        /// <summary>
        /// Generates normals from Vertices so you do not have to do it manually.
        /// </summary>
        [GodotMethod("generate_normals")]
        public void GenerateNormals()
        {
            NativeCalls.godot_icall_0_12(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "generate_tangents");

        /// <summary>
        /// </summary>
        [GodotMethod("generate_tangents")]
        public void GenerateTangents()
        {
            NativeCalls.godot_icall_0_12(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_to_format");

        /// <summary>
        /// </summary>
        [GodotMethod("add_to_format")]
        public void AddToFormat(int flags)
        {
            NativeCalls.godot_icall_1_0(method_bind_16, Object.GetPtr(this), flags);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_material");

        /// <summary>
        /// Sets [Material] to be used by the [Mesh] you are constructing.
        /// </summary>
        [GodotMethod("set_material")]
        public void SetMaterial(Material material)
        {
            NativeCalls.godot_icall_1_24(method_bind_17, Object.GetPtr(this), Object.GetPtr(material));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear");

        /// <summary>
        /// Clear all information passed into the surface tool so far.
        /// </summary>
        [GodotMethod("clear")]
        public void Clear()
        {
            NativeCalls.godot_icall_0_12(method_bind_18, Object.GetPtr(this));
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create_from");

        /// <summary>
        /// </summary>
        [GodotMethod("create_from")]
        public void CreateFrom(Mesh existing, int surface)
        {
            NativeCalls.godot_icall_2_432(method_bind_19, Object.GetPtr(this), Object.GetPtr(existing), surface);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "append_from");

        /// <summary>
        /// </summary>
        [GodotMethod("append_from")]
        public void AppendFrom(Mesh existing, int surface, Transform transform)
        {
            NativeCalls.godot_icall_3_827(method_bind_20, Object.GetPtr(this), Object.GetPtr(existing), surface, ref transform);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "commit");

        /// <summary>
        /// Returns a constructed [ArrayMesh] from current information passed in. If an existing [ArrayMesh] is passed in as an argument, will add an extra surface to the existing [ArrayMesh].
        /// </summary>
        [GodotMethod("commit")]
        public ArrayMesh Commit(ArrayMesh existing = null, int flags = 97792)
        {
            return NativeCalls.godot_icall_2_828(method_bind_21, Object.GetPtr(this), Object.GetPtr(existing), flags);
        }
    }
}
