using System;

namespace Godot
{
    /// <summary>
    /// Sets the active tab's [code]visible[/code] property to the value [code]true[/code]. Sets all other children's to [code]false[/code].
    /// Ignores non-[Control] children.
    /// Individual tabs are always visible unless you use [method set_tab_disabled] and [method set_tab_title] to hide it.
    /// To hide only a tab's content, nest the content inside a child [Control], so it receives the [code]TabContainer[/code]'s visibility setting instead.
    /// </summary>
    public class TabContainer : Control
    {
        public enum TabAlignEnum
        {
            Left = 0,
            Center = 1,
            Right = 2
        }


        /// <summary>
        /// The alignment of all tabs in the tab container. See the [code]ALIGN_*[/code] constants for details.
        /// </summary>
        public TabContainer.TabAlignEnum TabAlign
        {
            get
            {
                return GetTabAlign();
            }
            set
            {
                SetTabAlign(value);
            }
        }

        /// <summary>
        /// The current tab index. When set, this index's [Control] node's [code]visible[/code] property is set to [code]true[/code] and all others are set to [code]false[/code].
        /// </summary>
        public int CurrentTab
        {
            get
            {
                return GetCurrentTab();
            }
            set
            {
                SetCurrentTab(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] tabs are visible. If [code]false[/code] tabs' content and titles are hidden. Default value: [code]true[/code].
        /// </summary>
        public bool TabsVisible
        {
            get
            {
                return AreTabsVisible();
            }
            set
            {
                SetTabsVisible(value);
            }
        }


        private const string nativeName = "TabContainer";

        public TabContainer() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_TabContainer_Ctor(this);
        }

        internal TabContainer(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tab_count");

        /// <summary>
        /// Returns the number of tabs.
        /// </summary>
        [GodotMethod("get_tab_count")]
        public int GetTabCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_current_tab");

        [GodotMethod("set_current_tab")]
        public void SetCurrentTab(int tabIdx)
        {
            NativeCalls.godot_icall_1_0(method_bind_1, Object.GetPtr(this), tabIdx);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_current_tab");

        [GodotMethod("get_current_tab")]
        public int GetCurrentTab()
        {
            return NativeCalls.godot_icall_0_1(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_previous_tab");

        /// <summary>
        /// Returns the previously active tab index.
        /// </summary>
        [GodotMethod("get_previous_tab")]
        public int GetPreviousTab()
        {
            return NativeCalls.godot_icall_0_1(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_current_tab_control");

        /// <summary>
        /// Returns the child [Control] node located at the active tab index.
        /// </summary>
        [GodotMethod("get_current_tab_control")]
        public Control GetCurrentTabControl()
        {
            return NativeCalls.godot_icall_0_258(method_bind_4, Object.GetPtr(this));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tab_control");

        /// <summary>
        /// Returns the currently visible tab's [Control] node.
        /// </summary>
        [GodotMethod("get_tab_control")]
        public Control GetTabControl(int idx)
        {
            return NativeCalls.godot_icall_1_831(method_bind_5, Object.GetPtr(this), idx);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tab_align");

        [GodotMethod("set_tab_align")]
        public void SetTabAlign(TabContainer.TabAlignEnum align)
        {
            NativeCalls.godot_icall_1_832(method_bind_6, Object.GetPtr(this), align);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tab_align");

        [GodotMethod("get_tab_align")]
        public TabContainer.TabAlignEnum GetTabAlign()
        {
            return NativeCalls.godot_icall_0_833(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tabs_visible");

        [GodotMethod("set_tabs_visible")]
        public void SetTabsVisible(bool visible)
        {
            NativeCalls.godot_icall_1_11(method_bind_8, Object.GetPtr(this), visible);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "are_tabs_visible");

        [GodotMethod("are_tabs_visible")]
        public bool AreTabsVisible()
        {
            return NativeCalls.godot_icall_0_3(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tab_title");

        /// <summary>
        /// Sets a title for the tab at index [code]tab_idx[/code]. Tab titles default to the name of the indexed child node, but this can be overridden with [method set_tab_title].
        /// </summary>
        [GodotMethod("set_tab_title")]
        public void SetTabTitle(int tabIdx, string title)
        {
            NativeCalls.godot_icall_2_115(method_bind_10, Object.GetPtr(this), tabIdx, title);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tab_title");

        /// <summary>
        /// Returns the title of the tab at index [code]tab_idx[/code]. Tab titles default to the name of the indexed child node, but this can be overridden with [method set_tab_title].
        /// </summary>
        [GodotMethod("get_tab_title")]
        public string GetTabTitle(int tabIdx)
        {
            return NativeCalls.godot_icall_1_108(method_bind_11, Object.GetPtr(this), tabIdx);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tab_icon");

        /// <summary>
        /// Sets an icon for the tab at index [code]tab_idx[/code].
        /// </summary>
        [GodotMethod("set_tab_icon")]
        public void SetTabIcon(int tabIdx, Texture icon)
        {
            NativeCalls.godot_icall_2_67(method_bind_12, Object.GetPtr(this), tabIdx, Object.GetPtr(icon));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tab_icon");

        /// <summary>
        /// Returns the [Texture] for the tab at index [code]tab_idx[/code] or null if the tab has no [Texture].
        /// </summary>
        [GodotMethod("get_tab_icon")]
        public Texture GetTabIcon(int tabIdx)
        {
            return NativeCalls.godot_icall_1_157(method_bind_13, Object.GetPtr(this), tabIdx);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tab_disabled");

        /// <summary>
        /// If [code]disabled[/code] is false, hides the tab at index [code]tab_idx[/code]. Note that its title text will remain, unless also removed with [method set_tab_title].
        /// </summary>
        [GodotMethod("set_tab_disabled")]
        public void SetTabDisabled(int tabIdx, bool disabled)
        {
            NativeCalls.godot_icall_2_51(method_bind_14, Object.GetPtr(this), tabIdx, disabled);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tab_disabled");

        /// <summary>
        /// Returns [code]true[/code] if the tab at index [code]tab_idx[/code] is disabled.
        /// </summary>
        [GodotMethod("get_tab_disabled")]
        public bool GetTabDisabled(int tabIdx)
        {
            return NativeCalls.godot_icall_1_29(method_bind_15, Object.GetPtr(this), tabIdx);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_popup");

        /// <summary>
        /// If set on a [Popup] node instance, a popup menu icon appears in the top-right corner of the [code]TabContainer[/code]. Clicking it will expand the [Popup] node.
        /// </summary>
        [GodotMethod("set_popup")]
        public void SetPopup(Node popup)
        {
            NativeCalls.godot_icall_1_42(method_bind_16, Object.GetPtr(this), Object.GetPtr(popup));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_popup");

        /// <summary>
        /// Returns the [Popup] node instance if one has been set already with [method set_popup].
        /// </summary>
        [GodotMethod("get_popup")]
        public Popup GetPopup()
        {
            return NativeCalls.godot_icall_0_834(method_bind_17, Object.GetPtr(this));
        }
    }
}
