using System;

namespace Godot
{
    /// <summary>
    /// TextEdit is meant for editing large, multiline text. It also has facilities for editing code, such as syntax highlighting support and multiple levels of undo/redo.
    /// </summary>
    public class TextEdit : Control
    {
        public enum SearchFlags
        {
            MatchCase = 1,
            WholeWords = 2,
            Backwards = 4
        }

        public enum MenuItems
        {
            Cut = 0,
            Copy = 1,
            Paste = 2,
            Clear = 3,
            SelectAll = 4,
            Undo = 5,
            Max = 6
        }


        /// <summary>
        /// String value of the [TextEdit].
        /// </summary>
        public string Text
        {
            get
            {
                return GetText();
            }
            set
            {
                SetText(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] read-only mode is enabled. Existing text cannot be modified and new text cannot be added.
        /// </summary>
        public bool Readonly
        {
            get
            {
                return IsReadonly();
            }
            set
            {
                SetReadonly(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the line containing the cursor is highlighted.
        /// </summary>
        public bool HighlightCurrentLine
        {
            get
            {
                return IsHighlightCurrentLineEnabled();
            }
            set
            {
                SetHighlightCurrentLine(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool SyntaxHighlighting
        {
            get
            {
                return IsSyntaxColoringEnabled();
            }
            set
            {
                SetSyntaxColoring(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] line numbers are displayed to the left of the text.
        /// </summary>
        public bool ShowLineNumbers
        {
            get
            {
                return IsShowLineNumbersEnabled();
            }
            set
            {
                SetShowLineNumbers(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool HighlightAllOccurrences
        {
            get
            {
                return IsHighlightAllOccurrencesEnabled();
            }
            set
            {
                SetHighlightAllOccurrences(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool OverrideSelectedFontColor
        {
            get
            {
                return IsOverridingSelectedFontColor();
            }
            set
            {
                SetOverrideSelectedFontColor(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] a right click displays the context menu.
        /// </summary>
        public bool ContextMenuEnabled
        {
            get
            {
                return IsContextMenuEnabled();
            }
            set
            {
                SetContextMenuEnabled(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool SmoothScrolling
        {
            get
            {
                return IsSmoothScrollEnabled();
            }
            set
            {
                SetSmoothScrollEnable(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], enables text wrapping when it goes beyond he edge of what is visible.
        /// </summary>
        public float VScrollSpeed
        {
            get
            {
                return GetVScrollSpeed();
            }
            set
            {
                SetVScrollSpeed(value);
            }
        }

        /// <summary>
        /// </summary>
        public int HidingEnabled
        {
            get
            {
                return IsHidingEnabled();
            }
            set
            {
                SetHidingEnabled(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool WrapLines
        {
            get
            {
                return IsWrapping();
            }
            set
            {
                SetWrap(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the caret displays as a rectangle.
        /// If [code]false[/code] the caret displays as a bar.
        /// </summary>
        public bool CaretBlockMode
        {
            get
            {
                return CursorIsBlockMode();
            }
            set
            {
                CursorSetBlockMode(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the caret (visual cursor) blinks.
        /// </summary>
        public bool CaretBlink
        {
            get
            {
                return CursorGetBlinkEnabled();
            }
            set
            {
                CursorSetBlinkEnabled(value);
            }
        }

        /// <summary>
        /// Duration (in seconds) of a caret's blinking cycle.
        /// </summary>
        public float CaretBlinkSpeed
        {
            get
            {
                return CursorGetBlinkSpeed();
            }
            set
            {
                CursorSetBlinkSpeed(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] a right click moves the cursor at the mouse position before displaying the context menu.
        /// If [code]false[/code] the context menu disregards mouse location.
        /// </summary>
        public bool CaretMovingByRightClick
        {
            get
            {
                return IsRightClickMovingCaret();
            }
            set
            {
                SetRightClickMovesCaret(value);
            }
        }


        private const string nativeName = "TextEdit";

        public TextEdit() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_TextEdit_Ctor(this);
        }

        internal TextEdit(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_text");

        [GodotMethod("set_text")]
        public void SetText(string text)
        {
            NativeCalls.godot_icall_1_43(method_bind_0, Object.GetPtr(this), text);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "insert_text_at_cursor");

        /// <summary>
        /// Insert a given text at the cursor position.
        /// </summary>
        [GodotMethod("insert_text_at_cursor")]
        public void InsertTextAtCursor(string text)
        {
            NativeCalls.godot_icall_1_43(method_bind_1, Object.GetPtr(this), text);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_line_count");

        /// <summary>
        /// Return the amount of total lines in the text.
        /// </summary>
        [GodotMethod("get_line_count")]
        public int GetLineCount()
        {
            return NativeCalls.godot_icall_0_1(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_text");

        [GodotMethod("get_text")]
        public string GetText()
        {
            return NativeCalls.godot_icall_0_2(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_line");

        /// <summary>
        /// Return the text of a specific line.
        /// </summary>
        [GodotMethod("get_line")]
        public string GetLine(int line)
        {
            return NativeCalls.godot_icall_1_108(method_bind_4, Object.GetPtr(this), line);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "cursor_set_column");

        /// <summary>
        /// </summary>
        [GodotMethod("cursor_set_column")]
        public void CursorSetColumn(int column, bool adjustViewport = true)
        {
            NativeCalls.godot_icall_2_51(method_bind_5, Object.GetPtr(this), column, adjustViewport);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "cursor_set_line");

        /// <summary>
        /// </summary>
        [GodotMethod("cursor_set_line")]
        public void CursorSetLine(int line, bool adjustViewport = true, bool canBeHidden = true)
        {
            NativeCalls.godot_icall_3_839(method_bind_6, Object.GetPtr(this), line, adjustViewport, canBeHidden);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "cursor_get_column");

        /// <summary>
        /// Return the column the editing cursor is at.
        /// </summary>
        [GodotMethod("cursor_get_column")]
        public int CursorGetColumn()
        {
            return NativeCalls.godot_icall_0_1(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "cursor_get_line");

        /// <summary>
        /// Return the line the editing cursor is at.
        /// </summary>
        [GodotMethod("cursor_get_line")]
        public int CursorGetLine()
        {
            return NativeCalls.godot_icall_0_1(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "cursor_set_blink_enabled");

        [GodotMethod("cursor_set_blink_enabled")]
        public void CursorSetBlinkEnabled(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_9, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "cursor_get_blink_enabled");

        [GodotMethod("cursor_get_blink_enabled")]
        public bool CursorGetBlinkEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "cursor_set_blink_speed");

        [GodotMethod("cursor_set_blink_speed")]
        public void CursorSetBlinkSpeed(float blinkSpeed)
        {
            NativeCalls.godot_icall_1_10(method_bind_11, Object.GetPtr(this), blinkSpeed);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "cursor_get_blink_speed");

        [GodotMethod("cursor_get_blink_speed")]
        public float CursorGetBlinkSpeed()
        {
            return NativeCalls.godot_icall_0_9(method_bind_12, Object.GetPtr(this));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "cursor_set_block_mode");

        [GodotMethod("cursor_set_block_mode")]
        public void CursorSetBlockMode(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_13, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "cursor_is_block_mode");

        [GodotMethod("cursor_is_block_mode")]
        public bool CursorIsBlockMode()
        {
            return NativeCalls.godot_icall_0_3(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_right_click_moves_caret");

        [GodotMethod("set_right_click_moves_caret")]
        public void SetRightClickMovesCaret(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_15, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_right_click_moving_caret");

        [GodotMethod("is_right_click_moving_caret")]
        public bool IsRightClickMovingCaret()
        {
            return NativeCalls.godot_icall_0_3(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_readonly");

        [GodotMethod("set_readonly")]
        public void SetReadonly(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_17, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_readonly");

        [GodotMethod("is_readonly")]
        public bool IsReadonly()
        {
            return NativeCalls.godot_icall_0_3(method_bind_18, Object.GetPtr(this));
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_wrap");

        [GodotMethod("set_wrap")]
        public void SetWrap(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_19, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_wrapping");

        [GodotMethod("is_wrapping")]
        public bool IsWrapping()
        {
            return NativeCalls.godot_icall_0_3(method_bind_20, Object.GetPtr(this));
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_context_menu_enabled");

        [GodotMethod("set_context_menu_enabled")]
        public void SetContextMenuEnabled(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_21, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_context_menu_enabled");

        [GodotMethod("is_context_menu_enabled")]
        public bool IsContextMenuEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_22, Object.GetPtr(this));
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "cut");

        /// <summary>
        /// Cut the current selection.
        /// </summary>
        [GodotMethod("cut")]
        public void Cut()
        {
            NativeCalls.godot_icall_0_12(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "copy");

        /// <summary>
        /// Copy the current selection.
        /// </summary>
        [GodotMethod("copy")]
        public void Copy()
        {
            NativeCalls.godot_icall_0_12(method_bind_24, Object.GetPtr(this));
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "paste");

        /// <summary>
        /// Paste the current selection.
        /// </summary>
        [GodotMethod("paste")]
        public void Paste()
        {
            NativeCalls.godot_icall_0_12(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "select");

        /// <summary>
        /// Perform selection, from line/column to line/column.
        /// </summary>
        [GodotMethod("select")]
        public void Select(int fromLine, int fromColumn, int toLine, int toColumn)
        {
            NativeCalls.godot_icall_4_819(method_bind_26, Object.GetPtr(this), fromLine, fromColumn, toLine, toColumn);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "select_all");

        /// <summary>
        /// Select all the text.
        /// </summary>
        [GodotMethod("select_all")]
        public void SelectAll()
        {
            NativeCalls.godot_icall_0_12(method_bind_27, Object.GetPtr(this));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "deselect");

        /// <summary>
        /// Clears the current selection.
        /// </summary>
        [GodotMethod("deselect")]
        public void Deselect()
        {
            NativeCalls.godot_icall_0_12(method_bind_28, Object.GetPtr(this));
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_selection_active");

        /// <summary>
        /// Return true if the selection is active.
        /// </summary>
        [GodotMethod("is_selection_active")]
        public bool IsSelectionActive()
        {
            return NativeCalls.godot_icall_0_3(method_bind_29, Object.GetPtr(this));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_selection_from_line");

        /// <summary>
        /// Return the selection begin line.
        /// </summary>
        [GodotMethod("get_selection_from_line")]
        public int GetSelectionFromLine()
        {
            return NativeCalls.godot_icall_0_1(method_bind_30, Object.GetPtr(this));
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_selection_from_column");

        /// <summary>
        /// Return the selection begin column.
        /// </summary>
        [GodotMethod("get_selection_from_column")]
        public int GetSelectionFromColumn()
        {
            return NativeCalls.godot_icall_0_1(method_bind_31, Object.GetPtr(this));
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_selection_to_line");

        /// <summary>
        /// Return the selection end line.
        /// </summary>
        [GodotMethod("get_selection_to_line")]
        public int GetSelectionToLine()
        {
            return NativeCalls.godot_icall_0_1(method_bind_32, Object.GetPtr(this));
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_selection_to_column");

        /// <summary>
        /// Return the selection end column.
        /// </summary>
        [GodotMethod("get_selection_to_column")]
        public int GetSelectionToColumn()
        {
            return NativeCalls.godot_icall_0_1(method_bind_33, Object.GetPtr(this));
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_selection_text");

        /// <summary>
        /// Return the text inside the selection.
        /// </summary>
        [GodotMethod("get_selection_text")]
        public string GetSelectionText()
        {
            return NativeCalls.godot_icall_0_2(method_bind_34, Object.GetPtr(this));
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_word_under_cursor");

        /// <summary>
        /// </summary>
        [GodotMethod("get_word_under_cursor")]
        public string GetWordUnderCursor()
        {
            return NativeCalls.godot_icall_0_2(method_bind_35, Object.GetPtr(this));
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "search");

        /// <summary>
        /// Perform a search inside the text. Search flags can be specified in the SEARCH_* enum.
        /// </summary>
        [GodotMethod("search")]
        public int[] Search(string key, int flags, int fromLine, int fromColumn)
        {
            return NativeCalls.godot_icall_4_840(method_bind_36, Object.GetPtr(this), key, flags, fromLine, fromColumn);
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "undo");

        /// <summary>
        /// Perform undo operation.
        /// </summary>
        [GodotMethod("undo")]
        public void Undo()
        {
            NativeCalls.godot_icall_0_12(method_bind_37, Object.GetPtr(this));
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "redo");

        /// <summary>
        /// Perform redo operation.
        /// </summary>
        [GodotMethod("redo")]
        public void Redo()
        {
            NativeCalls.godot_icall_0_12(method_bind_38, Object.GetPtr(this));
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_undo_history");

        /// <summary>
        /// Clear the undo history.
        /// </summary>
        [GodotMethod("clear_undo_history")]
        public void ClearUndoHistory()
        {
            NativeCalls.godot_icall_0_12(method_bind_39, Object.GetPtr(this));
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_show_line_numbers");

        [GodotMethod("set_show_line_numbers")]
        public void SetShowLineNumbers(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_40, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_show_line_numbers_enabled");

        [GodotMethod("is_show_line_numbers_enabled")]
        public bool IsShowLineNumbersEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_41, Object.GetPtr(this));
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_hiding_enabled");

        [GodotMethod("set_hiding_enabled")]
        public void SetHidingEnabled(int enable)
        {
            NativeCalls.godot_icall_1_0(method_bind_42, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_hiding_enabled");

        [GodotMethod("is_hiding_enabled")]
        public int IsHidingEnabled()
        {
            return NativeCalls.godot_icall_0_1(method_bind_43, Object.GetPtr(this));
        }

        private IntPtr method_bind_44 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_line_as_hidden");

        /// <summary>
        /// </summary>
        [GodotMethod("set_line_as_hidden")]
        public void SetLineAsHidden(int line, bool enable)
        {
            NativeCalls.godot_icall_2_51(method_bind_44, Object.GetPtr(this), line, enable);
        }

        private IntPtr method_bind_45 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_line_hidden");

        /// <summary>
        /// </summary>
        [GodotMethod("is_line_hidden")]
        public bool IsLineHidden(int line)
        {
            return NativeCalls.godot_icall_1_29(method_bind_45, Object.GetPtr(this), line);
        }

        private IntPtr method_bind_46 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "fold_all_lines");

        /// <summary>
        /// </summary>
        [GodotMethod("fold_all_lines")]
        public void FoldAllLines()
        {
            NativeCalls.godot_icall_0_12(method_bind_46, Object.GetPtr(this));
        }

        private IntPtr method_bind_47 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "unhide_all_lines");

        /// <summary>
        /// </summary>
        [GodotMethod("unhide_all_lines")]
        public void UnhideAllLines()
        {
            NativeCalls.godot_icall_0_12(method_bind_47, Object.GetPtr(this));
        }

        private IntPtr method_bind_48 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "fold_line");

        /// <summary>
        /// </summary>
        [GodotMethod("fold_line")]
        public void FoldLine(int line)
        {
            NativeCalls.godot_icall_1_0(method_bind_48, Object.GetPtr(this), line);
        }

        private IntPtr method_bind_49 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "unfold_line");

        /// <summary>
        /// </summary>
        [GodotMethod("unfold_line")]
        public void UnfoldLine(int line)
        {
            NativeCalls.godot_icall_1_0(method_bind_49, Object.GetPtr(this), line);
        }

        private IntPtr method_bind_50 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "toggle_fold_line");

        /// <summary>
        /// Toggle the folding of the code block at the given line.
        /// </summary>
        [GodotMethod("toggle_fold_line")]
        public void ToggleFoldLine(int line)
        {
            NativeCalls.godot_icall_1_0(method_bind_50, Object.GetPtr(this), line);
        }

        private IntPtr method_bind_51 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "can_fold");

        /// <summary>
        /// </summary>
        [GodotMethod("can_fold")]
        public bool CanFold(int line)
        {
            return NativeCalls.godot_icall_1_29(method_bind_51, Object.GetPtr(this), line);
        }

        private IntPtr method_bind_52 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_folded");

        /// <summary>
        /// </summary>
        [GodotMethod("is_folded")]
        public bool IsFolded(int line)
        {
            return NativeCalls.godot_icall_1_29(method_bind_52, Object.GetPtr(this), line);
        }

        private IntPtr method_bind_53 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_highlight_all_occurrences");

        [GodotMethod("set_highlight_all_occurrences")]
        public void SetHighlightAllOccurrences(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_53, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_54 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_highlight_all_occurrences_enabled");

        [GodotMethod("is_highlight_all_occurrences_enabled")]
        public bool IsHighlightAllOccurrencesEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_54, Object.GetPtr(this));
        }

        private IntPtr method_bind_55 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_override_selected_font_color");

        [GodotMethod("set_override_selected_font_color")]
        public void SetOverrideSelectedFontColor(bool @override)
        {
            NativeCalls.godot_icall_1_11(method_bind_55, Object.GetPtr(this), @override);
        }

        private IntPtr method_bind_56 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_overriding_selected_font_color");

        [GodotMethod("is_overriding_selected_font_color")]
        public bool IsOverridingSelectedFontColor()
        {
            return NativeCalls.godot_icall_0_3(method_bind_56, Object.GetPtr(this));
        }

        private IntPtr method_bind_57 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_syntax_coloring");

        [GodotMethod("set_syntax_coloring")]
        public void SetSyntaxColoring(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_57, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_58 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_syntax_coloring_enabled");

        [GodotMethod("is_syntax_coloring_enabled")]
        public bool IsSyntaxColoringEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_58, Object.GetPtr(this));
        }

        private IntPtr method_bind_59 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_highlight_current_line");

        [GodotMethod("set_highlight_current_line")]
        public void SetHighlightCurrentLine(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_59, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_60 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_highlight_current_line_enabled");

        [GodotMethod("is_highlight_current_line_enabled")]
        public bool IsHighlightCurrentLineEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_60, Object.GetPtr(this));
        }

        private IntPtr method_bind_61 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_smooth_scroll_enable");

        [GodotMethod("set_smooth_scroll_enable")]
        public void SetSmoothScrollEnable(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_61, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_62 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_smooth_scroll_enabled");

        [GodotMethod("is_smooth_scroll_enabled")]
        public bool IsSmoothScrollEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_62, Object.GetPtr(this));
        }

        private IntPtr method_bind_63 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_v_scroll_speed");

        [GodotMethod("set_v_scroll_speed")]
        public void SetVScrollSpeed(float speed)
        {
            NativeCalls.godot_icall_1_10(method_bind_63, Object.GetPtr(this), speed);
        }

        private IntPtr method_bind_64 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_v_scroll_speed");

        [GodotMethod("get_v_scroll_speed")]
        public float GetVScrollSpeed()
        {
            return NativeCalls.godot_icall_0_9(method_bind_64, Object.GetPtr(this));
        }

        private IntPtr method_bind_65 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_keyword_color");

        /// <summary>
        /// Add a keyword and its color.
        /// </summary>
        [GodotMethod("add_keyword_color")]
        public void AddKeywordColor(string keyword, Color color)
        {
            NativeCalls.godot_icall_2_260(method_bind_65, Object.GetPtr(this), keyword, ref color);
        }

        private IntPtr method_bind_66 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_color_region");

        /// <summary>
        /// Add color region (given the delimiters) and its colors.
        /// </summary>
        [GodotMethod("add_color_region")]
        public void AddColorRegion(string beginKey, string endKey, Color color, bool lineOnly = false)
        {
            NativeCalls.godot_icall_4_841(method_bind_66, Object.GetPtr(this), beginKey, endKey, ref color, lineOnly);
        }

        private IntPtr method_bind_67 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_colors");

        /// <summary>
        /// Clear all the syntax coloring information.
        /// </summary>
        [GodotMethod("clear_colors")]
        public void ClearColors()
        {
            NativeCalls.godot_icall_0_12(method_bind_67, Object.GetPtr(this));
        }

        private IntPtr method_bind_68 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "menu_option");

        /// <summary>
        /// </summary>
        [GodotMethod("menu_option")]
        public void MenuOption(int option)
        {
            NativeCalls.godot_icall_1_0(method_bind_68, Object.GetPtr(this), option);
        }

        private IntPtr method_bind_69 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_menu");

        /// <summary>
        /// </summary>
        [GodotMethod("get_menu")]
        public PopupMenu GetMenu()
        {
            return NativeCalls.godot_icall_0_498(method_bind_69, Object.GetPtr(this));
        }
    }
}
