using System;

namespace Godot
{
    /// <summary>
    /// [code]TextureButton[/code] has the same functionality as [Button], except it uses sprites instead of Godot's [Theme] resource. It is faster to create, but it doesn't support localization like more complex Controls.
    /// The Normal state's texture is required. Others are optional.
    /// </summary>
    public class TextureButton : BaseButton
    {
        public enum StretchModeEnum
        {
            Scale = 0,
            Tile = 1,
            Keep = 2,
            KeepCentered = 3,
            KeepAspect = 4,
            KeepAspectCentered = 5,
            KeepAspectCovered = 6
        }


        /// <summary>
        /// Texture to display by default, when the node is [b]not[/b] in the disabled, focused, hover or pressed state.
        /// </summary>
        public Texture TextureNormal
        {
            get
            {
                return GetNormalTexture();
            }
            set
            {
                SetNormalTexture(value);
            }
        }

        /// <summary>
        /// Texture to display on mouse down over the node, if the node has keyboard focus and the player presses the enter key or if the player presses the [member BaseButton.shortcut] key.
        /// </summary>
        public Texture TexturePressed
        {
            get
            {
                return GetPressedTexture();
            }
            set
            {
                SetPressedTexture(value);
            }
        }

        /// <summary>
        /// Texture to display when the mouse hovers the node.
        /// </summary>
        public Texture TextureHover
        {
            get
            {
                return GetHoverTexture();
            }
            set
            {
                SetHoverTexture(value);
            }
        }

        /// <summary>
        /// Texture to display when the node is disabled. See [member BaseButton.disabled].
        /// </summary>
        public Texture TextureDisabled
        {
            get
            {
                return GetDisabledTexture();
            }
            set
            {
                SetDisabledTexture(value);
            }
        }

        /// <summary>
        /// Texture to display when the node has mouse or keyboard focus.
        /// </summary>
        public Texture TextureFocused
        {
            get
            {
                return GetFocusedTexture();
            }
            set
            {
                SetFocusedTexture(value);
            }
        }

        /// <summary>
        /// Pure black and white [Bitmap] image to use for click detection. On the mask, white pixels represent the button's clickable area. Use it to create buttons with curved shapes.
        /// </summary>
        public BitMap TextureClickMask
        {
            get
            {
                return GetClickMask();
            }
            set
            {
                SetClickMask(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the texture stretches to the edges of the node's bounding rectangle using the [member stretch_mode]. If [code]false[/code] the texture will not scale with the node. Default value: [code]false[/code].
        /// </summary>
        public bool Expand
        {
            get
            {
                return GetExpand();
            }
            set
            {
                SetExpand(value);
            }
        }

        /// <summary>
        /// Controls the texture's behavior when you resize the node's bounding rectangle, [b]only if[/b] [member expand] is [code]true[/code]. Set it to one of the [code]STRETCH_*[/code] constants. See the constants to learn more.
        /// </summary>
        public TextureButton.StretchModeEnum StretchMode
        {
            get
            {
                return GetStretchMode();
            }
            set
            {
                SetStretchMode(value);
            }
        }


        private const string nativeName = "TextureButton";

        public TextureButton() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_TextureButton_Ctor(this);
        }

        internal TextureButton(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_normal_texture");

        [GodotMethod("set_normal_texture")]
        public void SetNormalTexture(Texture texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_pressed_texture");

        [GodotMethod("set_pressed_texture")]
        public void SetPressedTexture(Texture texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_1, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_hover_texture");

        [GodotMethod("set_hover_texture")]
        public void SetHoverTexture(Texture texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_2, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_disabled_texture");

        [GodotMethod("set_disabled_texture")]
        public void SetDisabledTexture(Texture texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_3, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_focused_texture");

        [GodotMethod("set_focused_texture")]
        public void SetFocusedTexture(Texture texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_4, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_click_mask");

        [GodotMethod("set_click_mask")]
        public void SetClickMask(BitMap mask)
        {
            NativeCalls.godot_icall_1_24(method_bind_5, Object.GetPtr(this), Object.GetPtr(mask));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_expand");

        [GodotMethod("set_expand")]
        public void SetExpand(bool pExpand)
        {
            NativeCalls.godot_icall_1_11(method_bind_6, Object.GetPtr(this), pExpand);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_stretch_mode");

        [GodotMethod("set_stretch_mode")]
        public void SetStretchMode(TextureButton.StretchModeEnum pMode)
        {
            NativeCalls.godot_icall_1_846(method_bind_7, Object.GetPtr(this), pMode);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_normal_texture");

        [GodotMethod("get_normal_texture")]
        public Texture GetNormalTexture()
        {
            return NativeCalls.godot_icall_0_119(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_pressed_texture");

        [GodotMethod("get_pressed_texture")]
        public Texture GetPressedTexture()
        {
            return NativeCalls.godot_icall_0_119(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_hover_texture");

        [GodotMethod("get_hover_texture")]
        public Texture GetHoverTexture()
        {
            return NativeCalls.godot_icall_0_119(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_disabled_texture");

        [GodotMethod("get_disabled_texture")]
        public Texture GetDisabledTexture()
        {
            return NativeCalls.godot_icall_0_119(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_focused_texture");

        [GodotMethod("get_focused_texture")]
        public Texture GetFocusedTexture()
        {
            return NativeCalls.godot_icall_0_119(method_bind_12, Object.GetPtr(this));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_click_mask");

        [GodotMethod("get_click_mask")]
        public BitMap GetClickMask()
        {
            return NativeCalls.godot_icall_0_847(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_expand");

        [GodotMethod("get_expand")]
        public bool GetExpand()
        {
            return NativeCalls.godot_icall_0_3(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_stretch_mode");

        [GodotMethod("get_stretch_mode")]
        public TextureButton.StretchModeEnum GetStretchMode()
        {
            return NativeCalls.godot_icall_0_848(method_bind_15, Object.GetPtr(this));
        }
    }
}
