using System;

namespace Godot
{
    /// <summary>
    /// TextureProgress works like [ProgressBar] but it uses up to 3 textures instead of Godot's [Theme] resource. Works horizontally, vertically, and radially.
    /// </summary>
    public class TextureProgress : Range
    {
        public enum FillModeEnum
        {
            LeftToRight = 0,
            RightToLeft = 1,
            TopToBottom = 2,
            BottomToTop = 3,
            Clockwise = 4,
            CounterClockwise = 5
        }


        /// <summary>
        /// [Texture] that draws under the progress bar. The bar's background.
        /// </summary>
        public Texture TextureUnder
        {
            get
            {
                return GetUnderTexture();
            }
            set
            {
                SetUnderTexture(value);
            }
        }

        /// <summary>
        /// [Texture] that draws over the progress bar. Use it to add highlights or an upper-frame that hides part of [member texture_progress].
        /// </summary>
        public Texture TextureOver
        {
            get
            {
                return GetOverTexture();
            }
            set
            {
                SetOverTexture(value);
            }
        }

        /// <summary>
        /// [Texture] that clips based on the node's [code]value[/code] and [member fill_mode]. As [code]value[/code] increased, the texture fills up. It shows entirely when [code]value[/code] reaches [code]max_value[/code]. It doesn't show at all if [code]value[/code] is equal to [code]min_value[/code].
        /// The [code]value[/code] property comes from [Range]. See [member Range.value], [member Range.min_value], [member Range.max_value].
        /// </summary>
        public Texture TextureProgress_
        {
            get
            {
                return GetProgressTexture();
            }
            set
            {
                SetProgressTexture(value);
            }
        }

        /// <summary>
        /// The fill direction. Uses FILL_* constants.
        /// </summary>
        public int FillMode
        {
            get
            {
                return GetFillMode();
            }
            set
            {
                SetFillMode(value);
            }
        }

        /// <summary>
        /// Starting angle for the fill of [member texture_progress] if [member fill_mode] is [code]FILL_CLOCKWISE[/code] or [code]FILL_COUNTER_CLOCKWISE[/code]. When the node's [code]value[/code] is equal to its [code]min_value[/code], the texture doesn't show up at all. When the [code]value[/code] increases, the texture fills and tends towards [member radial_fill_degrees].
        /// </summary>
        public float RadialInitialAngle
        {
            get
            {
                return GetRadialInitialAngle();
            }
            set
            {
                SetRadialInitialAngle(value);
            }
        }

        /// <summary>
        /// Upper limit for the fill of [member texture_progress] if [member fill_mode] is [code]FILL_CLOCKWISE[/code] or [code]FILL_COUNTER_CLOCKWISE[/code]. When the node's [code]value[/code] is equal to its [code]max_value[/code], the texture fills up to this angle.
        /// See [member Range.value], [member Range.max_value].
        /// </summary>
        public float RadialFillDegrees
        {
            get
            {
                return GetFillDegrees();
            }
            set
            {
                SetFillDegrees(value);
            }
        }

        /// <summary>
        /// Offsets [member texture_progress] if [member fill_mode] is [code]FILL_CLOCKWISE[/code] or [code]FILL_COUNTER_CLOCKWISE[/code].
        /// </summary>
        public Vector2 RadialCenterOffset
        {
            get
            {
                return GetRadialCenterOffset();
            }
            set
            {
                SetRadialCenterOffset(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] Godot treats the bar's textures like [NinePatchRect]. Use [code]stretch_margin_*[/code], like [member stretch_margin_bottom], to set up the nine patch's 3x3 grid. Default value: [code]false[/code].
        /// </summary>
        public bool NinePatchStretch
        {
            get
            {
                return GetNinePatchStretch();
            }
            set
            {
                SetNinePatchStretch(value);
            }
        }

        /// <summary>
        /// The width of the 9-patch's left column.
        /// </summary>
        public int StretchMarginLeft
        {
            get
            {
                return GetStretchMargin((Margin)0);
            }
            set
            {
                SetStretchMargin((Margin)0, value);
            }
        }

        /// <summary>
        /// The height of the 9-patch's top row.
        /// </summary>
        public int StretchMarginTop
        {
            get
            {
                return GetStretchMargin((Margin)1);
            }
            set
            {
                SetStretchMargin((Margin)1, value);
            }
        }

        /// <summary>
        /// The width of the 9-patch's right column.
        /// </summary>
        public int StretchMarginRight
        {
            get
            {
                return GetStretchMargin((Margin)2);
            }
            set
            {
                SetStretchMargin((Margin)2, value);
            }
        }

        /// <summary>
        /// The height of the 9-patch's bottom row. A margin of 16 means the 9-slice's bottom corners and side will have a height of 16 pixels. You can set all 4 margin values individually to create panels with non-uniform borders.
        /// </summary>
        public int StretchMarginBottom
        {
            get
            {
                return GetStretchMargin((Margin)3);
            }
            set
            {
                SetStretchMargin((Margin)3, value);
            }
        }


        private const string nativeName = "TextureProgress";

        public TextureProgress() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_TextureProgress_Ctor(this);
        }

        internal TextureProgress(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_under_texture");

        [GodotMethod("set_under_texture")]
        public void SetUnderTexture(Texture tex)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(tex));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_under_texture");

        [GodotMethod("get_under_texture")]
        public Texture GetUnderTexture()
        {
            return NativeCalls.godot_icall_0_119(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_progress_texture");

        [GodotMethod("set_progress_texture")]
        public void SetProgressTexture(Texture tex)
        {
            NativeCalls.godot_icall_1_24(method_bind_2, Object.GetPtr(this), Object.GetPtr(tex));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_progress_texture");

        [GodotMethod("get_progress_texture")]
        public Texture GetProgressTexture()
        {
            return NativeCalls.godot_icall_0_119(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_over_texture");

        [GodotMethod("set_over_texture")]
        public void SetOverTexture(Texture tex)
        {
            NativeCalls.godot_icall_1_24(method_bind_4, Object.GetPtr(this), Object.GetPtr(tex));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_over_texture");

        [GodotMethod("get_over_texture")]
        public Texture GetOverTexture()
        {
            return NativeCalls.godot_icall_0_119(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fill_mode");

        [GodotMethod("set_fill_mode")]
        public void SetFillMode(int mode)
        {
            NativeCalls.godot_icall_1_0(method_bind_6, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fill_mode");

        [GodotMethod("get_fill_mode")]
        public int GetFillMode()
        {
            return NativeCalls.godot_icall_0_1(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_radial_initial_angle");

        [GodotMethod("set_radial_initial_angle")]
        public void SetRadialInitialAngle(float mode)
        {
            NativeCalls.godot_icall_1_10(method_bind_8, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_radial_initial_angle");

        [GodotMethod("get_radial_initial_angle")]
        public float GetRadialInitialAngle()
        {
            return NativeCalls.godot_icall_0_9(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_radial_center_offset");

        [GodotMethod("set_radial_center_offset")]
        public void SetRadialCenterOffset(Vector2 mode)
        {
            NativeCalls.godot_icall_1_45(method_bind_10, Object.GetPtr(this), ref mode);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_radial_center_offset");

        [GodotMethod("get_radial_center_offset")]
        public Vector2 GetRadialCenterOffset()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_11, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fill_degrees");

        [GodotMethod("set_fill_degrees")]
        public void SetFillDegrees(float mode)
        {
            NativeCalls.godot_icall_1_10(method_bind_12, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fill_degrees");

        [GodotMethod("get_fill_degrees")]
        public float GetFillDegrees()
        {
            return NativeCalls.godot_icall_0_9(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_stretch_margin");

        [GodotMethod("set_stretch_margin")]
        public void SetStretchMargin(Margin margin, int value)
        {
            NativeCalls.godot_icall_2_196(method_bind_14, Object.GetPtr(this), margin, value);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_stretch_margin");

        [GodotMethod("get_stretch_margin")]
        public int GetStretchMargin(Margin margin)
        {
            return NativeCalls.godot_icall_1_197(method_bind_15, Object.GetPtr(this), margin);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_nine_patch_stretch");

        [GodotMethod("set_nine_patch_stretch")]
        public void SetNinePatchStretch(bool stretch)
        {
            NativeCalls.godot_icall_1_11(method_bind_16, Object.GetPtr(this), stretch);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_nine_patch_stretch");

        [GodotMethod("get_nine_patch_stretch")]
        public bool GetNinePatchStretch()
        {
            return NativeCalls.godot_icall_0_3(method_bind_17, Object.GetPtr(this));
        }
    }
}
