using System;

namespace Godot
{
    /// <summary>
    /// Use TextureRect to draw icons and sprites in your User Interfaces. To create panels and menu boxes, take a look at [NinePatchFrame]. Its Stretch Mode property controls the texture's scale and placement. It can scale, tile and stay centered inside its bounding rectangle. TextureRect is one of the 5 most common nodes to create game UI.
    /// </summary>
    public class TextureRect : Control
    {
        public enum StretchModeEnum
        {
            ScaleOnExpand = 0,
            Scale = 1,
            Tile = 2,
            Keep = 3,
            KeepCentered = 4,
            KeepAspect = 5,
            KeepAspectCentered = 6,
            KeepAspectCovered = 7
        }


        /// <summary>
        /// The node's [Texture] resource.
        /// </summary>
        public Texture Texture
        {
            get
            {
                return GetTexture();
            }
            set
            {
                SetTexture(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], the texture scales to fit its bounding rectangle. Default value: [code]false[/code].
        /// </summary>
        public bool Expand
        {
            get
            {
                return HasExpand();
            }
            set
            {
                SetExpand(value);
            }
        }

        /// <summary>
        /// Controls the texture's behavior when you resize the node's bounding rectangle. Set it to one of the [code]STRETCH_*[/code] constants. See the constants to learn more.
        /// </summary>
        public TextureRect.StretchModeEnum StretchMode
        {
            get
            {
                return GetStretchMode();
            }
            set
            {
                SetStretchMode(value);
            }
        }


        private const string nativeName = "TextureRect";

        public TextureRect() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_TextureRect_Ctor(this);
        }

        internal TextureRect(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_texture");

        [GodotMethod("set_texture")]
        public void SetTexture(Texture texture)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(texture));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture");

        [GodotMethod("get_texture")]
        public Texture GetTexture()
        {
            return NativeCalls.godot_icall_0_119(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_expand");

        [GodotMethod("set_expand")]
        public void SetExpand(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_2, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_expand");

        [GodotMethod("has_expand")]
        public bool HasExpand()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_stretch_mode");

        [GodotMethod("set_stretch_mode")]
        public void SetStretchMode(TextureRect.StretchModeEnum stretchMode)
        {
            NativeCalls.godot_icall_1_849(method_bind_4, Object.GetPtr(this), stretchMode);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_stretch_mode");

        [GodotMethod("get_stretch_mode")]
        public TextureRect.StretchModeEnum GetStretchMode()
        {
            return NativeCalls.godot_icall_0_850(method_bind_5, Object.GetPtr(this));
        }
    }
}
