using System;

namespace Godot
{
    /// <summary>
    /// Theme for skinning controls. Controls can be skinned individually, but for complex applications it's more efficient to just create a global theme that defines everything. This theme can be applied to any [Control], and it and its children will automatically use it.
    /// Theme resources can be alternatively loaded by writing them in a .theme file, see docs for more info.
    /// </summary>
    public class Theme : Resource
    {
        /// <summary>
        /// The theme's default font.
        /// </summary>
        public Font DefaultFont
        {
            get
            {
                return GetDefaultFont();
            }
            set
            {
                SetDefaultFont(value);
            }
        }


        private const string nativeName = "Theme";

        public Theme() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Theme_Ctor(this);
        }

        internal Theme(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_icon");

        /// <summary>
        /// Sets Theme's icon [Texture] to [code]texture[/code] at [code]name[/code] in [code]type[/code].
        /// Does nothing if Theme does not have [code]type[/code].
        /// </summary>
        [GodotMethod("set_icon")]
        public void SetIcon(string name, string type, Texture texture)
        {
            NativeCalls.godot_icall_3_851(method_bind_0, Object.GetPtr(this), name, type, Object.GetPtr(texture));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_icon");

        /// <summary>
        /// Returns the icon [Texture] at [code]name[/code] if Theme has [code]type[/code].
        /// </summary>
        [GodotMethod("get_icon")]
        public Texture GetIcon(string name, string type)
        {
            return NativeCalls.godot_icall_2_261(method_bind_1, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_icon");

        /// <summary>
        /// Returns [code]true[/code] if icon [Texture] with [code]name[/code] is in [code]type[/code].
        /// Returns [code]false[/code] if Theme does not have [code]type[/code].
        /// </summary>
        [GodotMethod("has_icon")]
        public bool HasIcon(string name, string type)
        {
            return NativeCalls.godot_icall_2_251(method_bind_2, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_icon");

        /// <summary>
        /// Clears icon at [code]name[/code] if Theme has [code]type[/code].
        /// </summary>
        [GodotMethod("clear_icon")]
        public void ClearIcon(string name, string type)
        {
            NativeCalls.godot_icall_2_69(method_bind_3, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_icon_list");

        /// <summary>
        /// Returns all of the icons as a [PoolStringArray] filled with each [Texture]'s name, for use in [method get_icon], if Theme has [code]type[/code].
        /// </summary>
        [GodotMethod("get_icon_list")]
        public string[] GetIconList(string type)
        {
            return NativeCalls.godot_icall_1_252(method_bind_4, Object.GetPtr(this), type);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_stylebox");

        /// <summary>
        /// Sets Theme's [StyleBox] to [code]stylebox[/code] at [code]name[/code] in [code]type[/code].
        /// Does nothing if Theme does not have [code]type[/code].
        /// </summary>
        [GodotMethod("set_stylebox")]
        public void SetStylebox(string name, string type, StyleBox texture)
        {
            NativeCalls.godot_icall_3_851(method_bind_5, Object.GetPtr(this), name, type, Object.GetPtr(texture));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_stylebox");

        /// <summary>
        /// Returns the icon [StyleBox] at [code]name[/code] if Theme has [code]type[/code].
        /// </summary>
        [GodotMethod("get_stylebox")]
        public StyleBox GetStylebox(string name, string type)
        {
            return NativeCalls.godot_icall_2_262(method_bind_6, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_stylebox");

        /// <summary>
        /// Returns [code]true[/code] if [StyleBox] with [code]name[/code] is in [code]type[/code].
        /// Returns [code]false[/code] if Theme does not have [code]type[/code].
        /// </summary>
        [GodotMethod("has_stylebox")]
        public bool HasStylebox(string name, string type)
        {
            return NativeCalls.godot_icall_2_251(method_bind_7, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_stylebox");

        /// <summary>
        /// Clears [StyleBox] at [code]name[/code] if Theme has [code]type[/code].
        /// </summary>
        [GodotMethod("clear_stylebox")]
        public void ClearStylebox(string name, string type)
        {
            NativeCalls.godot_icall_2_69(method_bind_8, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_stylebox_list");

        /// <summary>
        /// Returns all of the [StyleBox]s as a [PoolStringArray] filled with each [StyleBox]'s name, for use in [method get_stylebox], if Theme has [code]type[/code].
        /// </summary>
        [GodotMethod("get_stylebox_list")]
        public string[] GetStyleboxList(string type)
        {
            return NativeCalls.godot_icall_1_252(method_bind_9, Object.GetPtr(this), type);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_stylebox_types");

        /// <summary>
        /// Returns all of the [StyleBox] types as a [PoolStringArray] filled with each [StyleBox]'s type, for use in [method get_stylebox] and/or [method get_stylebox_list], if Theme has [code]type[/code].
        /// </summary>
        [GodotMethod("get_stylebox_types")]
        public string[] GetStyleboxTypes()
        {
            return NativeCalls.godot_icall_0_72(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_font");

        /// <summary>
        /// Sets Theme's [Font] to [code]font[/code] at [code]name[/code] in [code]type[/code].
        /// Does nothing if Theme does not have [code]type[/code].
        /// </summary>
        [GodotMethod("set_font")]
        public void SetFont(string name, string type, Font font)
        {
            NativeCalls.godot_icall_3_851(method_bind_11, Object.GetPtr(this), name, type, Object.GetPtr(font));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_font");

        /// <summary>
        /// Returns the [Font] at [code]name[/code] if Theme has [code]type[/code].
        /// </summary>
        [GodotMethod("get_font")]
        public Font GetFont(string name, string type)
        {
            return NativeCalls.godot_icall_2_263(method_bind_12, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_font");

        /// <summary>
        /// Returns [code]true[/code] if [Font] with [code]name[/code] is in [code]type[/code].
        /// Returns [code]false[/code] if Theme does not have [code]type[/code].
        /// </summary>
        [GodotMethod("has_font")]
        public bool HasFont(string name, string type)
        {
            return NativeCalls.godot_icall_2_251(method_bind_13, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_font");

        /// <summary>
        /// Clears [Font] at [code]name[/code] if Theme has [code]type[/code].
        /// </summary>
        [GodotMethod("clear_font")]
        public void ClearFont(string name, string type)
        {
            NativeCalls.godot_icall_2_69(method_bind_14, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_font_list");

        /// <summary>
        /// Returns all of the [Font]s as a [PoolStringArray] filled with each [Font]'s name, for use in [method get_font], if Theme has [code]type[/code].
        /// </summary>
        [GodotMethod("get_font_list")]
        public string[] GetFontList(string type)
        {
            return NativeCalls.godot_icall_1_252(method_bind_15, Object.GetPtr(this), type);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_color");

        /// <summary>
        /// Sets Theme's [Color] to [code]color[/code] at [code]name[/code] in [code]type[/code].
        /// Does nothing if Theme does not have [code]type[/code].
        /// </summary>
        [GodotMethod("set_color")]
        public void SetColor(string name, string type, Color color)
        {
            NativeCalls.godot_icall_3_852(method_bind_16, Object.GetPtr(this), name, type, ref color);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_color");

        /// <summary>
        /// Returns the [Color] at [code]name[/code] if Theme has [code]type[/code].
        /// </summary>
        [GodotMethod("get_color")]
        public Color GetColor(string name, string type)
        {
            object ret = NativeCalls.godot_icall_2_264(method_bind_17, Object.GetPtr(this), name, type);
            return (Color)ret;
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_color");

        /// <summary>
        /// Returns [code]true[/code] if [Color] with [code]name[/code] is in [code]type[/code].
        /// Returns [code]false[/code] if Theme does not have [code]type[/code].
        /// </summary>
        [GodotMethod("has_color")]
        public bool HasColor(string name, string type)
        {
            return NativeCalls.godot_icall_2_251(method_bind_18, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_color");

        /// <summary>
        /// Clears theme [Color] at [code]name[/code] if Theme has [code]type[/code].
        /// </summary>
        [GodotMethod("clear_color")]
        public void ClearColor(string name, string type)
        {
            NativeCalls.godot_icall_2_69(method_bind_19, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_color_list");

        /// <summary>
        /// Returns all of the [Color]s as a [PoolStringArray] filled with each [Color]'s name, for use in [method get_color], if Theme has [code]type[/code].
        /// </summary>
        [GodotMethod("get_color_list")]
        public string[] GetColorList(string type)
        {
            return NativeCalls.godot_icall_1_252(method_bind_20, Object.GetPtr(this), type);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_constant");

        /// <summary>
        /// Sets Theme's constant to [code]constant[/code] at [code]name[/code] in [code]type[/code].
        /// Does nothing if Theme does not have [code]type[/code].
        /// </summary>
        [GodotMethod("set_constant")]
        public void SetConstant(string name, string type, int constant)
        {
            NativeCalls.godot_icall_3_714(method_bind_21, Object.GetPtr(this), name, type, constant);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_constant");

        /// <summary>
        /// Returns the constant at [code]name[/code] if Theme has [code]type[/code].
        /// </summary>
        [GodotMethod("get_constant")]
        public int GetConstant(string name, string type)
        {
            return NativeCalls.godot_icall_2_265(method_bind_22, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_constant");

        /// <summary>
        /// Returns [code]true[/code] if constant with [code]name[/code] is in [code]type[/code].
        /// Returns [code]false[/code] if Theme does not have [code]type[/code].
        /// </summary>
        [GodotMethod("has_constant")]
        public bool HasConstant(string name, string type)
        {
            return NativeCalls.godot_icall_2_251(method_bind_23, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_constant");

        /// <summary>
        /// Clears theme constant at [code]name[/code] if Theme has [code]type[/code].
        /// </summary>
        [GodotMethod("clear_constant")]
        public void ClearConstant(string name, string type)
        {
            NativeCalls.godot_icall_2_69(method_bind_24, Object.GetPtr(this), name, type);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_constant_list");

        /// <summary>
        /// Returns all of the constants as a [PoolStringArray] filled with each constant's name, for use in [method get_constant], if Theme has [code]type[/code].
        /// </summary>
        [GodotMethod("get_constant_list")]
        public string[] GetConstantList(string type)
        {
            return NativeCalls.godot_icall_1_252(method_bind_25, Object.GetPtr(this), type);
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_default_font");

        [GodotMethod("set_default_font")]
        public void SetDefaultFont(Font font)
        {
            NativeCalls.godot_icall_1_24(method_bind_26, Object.GetPtr(this), Object.GetPtr(font));
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_default_font");

        [GodotMethod("get_default_font")]
        public Font GetDefaultFont()
        {
            return NativeCalls.godot_icall_0_853(method_bind_27, Object.GetPtr(this));
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_type_list");

        /// <summary>
        /// Returns all of the types in [code]type[/code] as a [PoolStringArray] for use in any of the get_* functions, if Theme has [code]type[/code].
        /// </summary>
        [GodotMethod("get_type_list")]
        public string[] GetTypeList(string type)
        {
            return NativeCalls.godot_icall_1_252(method_bind_28, Object.GetPtr(this), type);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "copy_default_theme");

        /// <summary>
        /// Sets theme values to a copy of the default theme values.
        /// </summary>
        [GodotMethod("copy_default_theme")]
        public void CopyDefaultTheme()
        {
            NativeCalls.godot_icall_0_12(method_bind_29, Object.GetPtr(this));
        }
    }
}
