using System;

namespace Godot
{
    /// <summary>
    /// Node for 2D tile-based maps. Tilemaps use a [TileSet] which contain a list of tiles (textures plus optional collision, navigation, and/or occluder shapes) which are used to create grid-based maps.
    /// </summary>
    public class TileMap : Node2D
    {
        public const int InvalidCell = -1;

        public enum ModeEnum
        {
            Square = 0,
            Isometric = 1,
            Custom = 2
        }

        public enum TileOrigin
        {
            TopLeft = 0,
            Center = 1,
            BottomLeft = 2
        }

        public enum HalfOffset
        {
            X = 0,
            Y = 1,
            Disabled = 2
        }


        /// <summary>
        /// The TileMap orientation mode. Uses MODE_* constants. Default value: MODE_SQUARE.
        /// </summary>
        public TileMap.ModeEnum Mode
        {
            get
            {
                return GetMode();
            }
            set
            {
                SetMode(value);
            }
        }

        /// <summary>
        /// The assigned [TileSet].
        /// </summary>
        public TileSet TileSet
        {
            get
            {
                return GetTileset();
            }
            set
            {
                SetTileset(value);
            }
        }

        /// <summary>
        /// The TileMap's cell size.
        /// </summary>
        public Vector2 CellSize
        {
            get
            {
                return GetCellSize();
            }
            set
            {
                SetCellSize(value);
            }
        }

        /// <summary>
        /// The TileMap's quadrant size.  Optimizes drawing by batching, using chunks of this size. Default value: 16.
        /// </summary>
        public int CellQuadrantSize
        {
            get
            {
                return GetQuadrantSize();
            }
            set
            {
                SetQuadrantSize(value);
            }
        }

        /// <summary>
        /// The custom [Transform2D] to be applied to the TileMap's cells.
        /// </summary>
        public Transform2D CellCustomTransform
        {
            get
            {
                return GetCustomTransform();
            }
            set
            {
                SetCustomTransform(value);
            }
        }

        /// <summary>
        /// Amount to offset alternating tiles. Uses HALF_OFFSET_* constants. Default value: HALF_OFFSET_DISABLED.
        /// </summary>
        public TileMap.HalfOffset CellHalfOffset
        {
            get
            {
                return GetHalfOffset();
            }
            set
            {
                SetHalfOffset(value);
            }
        }

        /// <summary>
        /// Position for tile origin. Uses TILE_ORIGIN_* constants. Default value: TILE_ORIGIN_TOP_LEFT.
        /// </summary>
        public TileMap.TileOrigin CellTileOrigin
        {
            get
            {
                return GetTileOrigin();
            }
            set
            {
                SetTileOrigin(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the TileMap's children will be drawn in order of their Y coordinate. Default value: [code]false[/code].
        /// </summary>
        public bool CellYSort
        {
            get
            {
                return IsYSortModeEnabled();
            }
            set
            {
                SetYSortMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool CellClipUv
        {
            get
            {
                return GetClipUv();
            }
            set
            {
                SetClipUv(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] TileMap collisions will be handled as a kinematic body. If [code]false[/code] collisions will be handled as static body. Default value: [code]false[/code].
        /// </summary>
        public bool CollisionUseKinematic
        {
            get
            {
                return GetCollisionUseKinematic();
            }
            set
            {
                SetCollisionUseKinematic(value);
            }
        }

        /// <summary>
        /// Friction value for static body collisions (see [code]collision_use_kinematic[/code]). Default value: 1.
        /// </summary>
        public float CollisionFriction
        {
            get
            {
                return GetCollisionFriction();
            }
            set
            {
                SetCollisionFriction(value);
            }
        }

        /// <summary>
        /// Bounce value for static body collisions (see [code]collision_use_kinematic[/code]). Default value: 0.
        /// </summary>
        public float CollisionBounce
        {
            get
            {
                return GetCollisionBounce();
            }
            set
            {
                SetCollisionBounce(value);
            }
        }

        /// <summary>
        /// The collision layer(s) for all colliders in the TileMap.
        /// </summary>
        public int CollisionLayer
        {
            get
            {
                return GetCollisionLayer();
            }
            set
            {
                SetCollisionLayer(value);
            }
        }

        /// <summary>
        /// The collision mask(s) for all colliders in the TileMap.
        /// </summary>
        public int CollisionMask
        {
            get
            {
                return GetCollisionMask();
            }
            set
            {
                SetCollisionMask(value);
            }
        }

        /// <summary>
        /// The light mask assigned to all light occluders in the TileMap.  The TileSet's light occluders will cast shadows only from Light2D(s) that have the same light mask(s).
        /// </summary>
        public int OccluderLightMask
        {
            get
            {
                return GetOccluderLightMask();
            }
            set
            {
                SetOccluderLightMask(value);
            }
        }


        private const string nativeName = "TileMap";

        public TileMap() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_TileMap_Ctor(this);
        }

        internal TileMap(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tileset");

        [GodotMethod("set_tileset")]
        public void SetTileset(TileSet tileset)
        {
            NativeCalls.godot_icall_1_24(method_bind_0, Object.GetPtr(this), Object.GetPtr(tileset));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tileset");

        [GodotMethod("get_tileset")]
        public TileSet GetTileset()
        {
            return NativeCalls.godot_icall_0_854(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_mode");

        [GodotMethod("set_mode")]
        public void SetMode(TileMap.ModeEnum mode)
        {
            NativeCalls.godot_icall_1_855(method_bind_2, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mode");

        [GodotMethod("get_mode")]
        public TileMap.ModeEnum GetMode()
        {
            return NativeCalls.godot_icall_0_856(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_half_offset");

        [GodotMethod("set_half_offset")]
        public void SetHalfOffset(TileMap.HalfOffset halfOffset)
        {
            NativeCalls.godot_icall_1_857(method_bind_4, Object.GetPtr(this), halfOffset);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_half_offset");

        [GodotMethod("get_half_offset")]
        public TileMap.HalfOffset GetHalfOffset()
        {
            return NativeCalls.godot_icall_0_858(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_custom_transform");

        [GodotMethod("set_custom_transform")]
        public void SetCustomTransform(Transform2D customTransform)
        {
            NativeCalls.godot_icall_1_219(method_bind_6, Object.GetPtr(this), ref customTransform);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_custom_transform");

        [GodotMethod("get_custom_transform")]
        public Transform2D GetCustomTransform()
        {
            object ret = NativeCalls.godot_icall_0_220(method_bind_7, Object.GetPtr(this));
            return (Transform2D)ret;
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_cell_size");

        [GodotMethod("set_cell_size")]
        public void SetCellSize(Vector2 size)
        {
            NativeCalls.godot_icall_1_45(method_bind_8, Object.GetPtr(this), ref size);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cell_size");

        [GodotMethod("get_cell_size")]
        public Vector2 GetCellSize()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_9, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_quadrant_size");

        [GodotMethod("set_quadrant_size")]
        public void SetQuadrantSize(int size)
        {
            NativeCalls.godot_icall_1_0(method_bind_10, Object.GetPtr(this), size);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_quadrant_size");

        [GodotMethod("get_quadrant_size")]
        public int GetQuadrantSize()
        {
            return NativeCalls.godot_icall_0_1(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tile_origin");

        [GodotMethod("set_tile_origin")]
        public void SetTileOrigin(TileMap.TileOrigin origin)
        {
            NativeCalls.godot_icall_1_859(method_bind_12, Object.GetPtr(this), origin);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tile_origin");

        [GodotMethod("get_tile_origin")]
        public TileMap.TileOrigin GetTileOrigin()
        {
            return NativeCalls.godot_icall_0_860(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_clip_uv");

        [GodotMethod("set_clip_uv")]
        public void SetClipUv(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_14, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_clip_uv");

        [GodotMethod("get_clip_uv")]
        public bool GetClipUv()
        {
            return NativeCalls.godot_icall_0_3(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_y_sort_mode");

        [GodotMethod("set_y_sort_mode")]
        public void SetYSortMode(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_16, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_y_sort_mode_enabled");

        [GodotMethod("is_y_sort_mode_enabled")]
        public bool IsYSortModeEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_use_kinematic");

        [GodotMethod("set_collision_use_kinematic")]
        public void SetCollisionUseKinematic(bool useKinematic)
        {
            NativeCalls.godot_icall_1_11(method_bind_18, Object.GetPtr(this), useKinematic);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_use_kinematic");

        [GodotMethod("get_collision_use_kinematic")]
        public bool GetCollisionUseKinematic()
        {
            return NativeCalls.godot_icall_0_3(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_layer");

        [GodotMethod("set_collision_layer")]
        public void SetCollisionLayer(int layer)
        {
            NativeCalls.godot_icall_1_0(method_bind_20, Object.GetPtr(this), layer);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_layer");

        [GodotMethod("get_collision_layer")]
        public int GetCollisionLayer()
        {
            return NativeCalls.godot_icall_0_1(method_bind_21, Object.GetPtr(this));
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_mask");

        [GodotMethod("set_collision_mask")]
        public void SetCollisionMask(int mask)
        {
            NativeCalls.godot_icall_1_0(method_bind_22, Object.GetPtr(this), mask);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_mask");

        [GodotMethod("get_collision_mask")]
        public int GetCollisionMask()
        {
            return NativeCalls.godot_icall_0_1(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_layer_bit");

        /// <summary>
        /// </summary>
        [GodotMethod("set_collision_layer_bit")]
        public void SetCollisionLayerBit(int bit, bool value)
        {
            NativeCalls.godot_icall_2_51(method_bind_24, Object.GetPtr(this), bit, value);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_layer_bit");

        /// <summary>
        /// </summary>
        [GodotMethod("get_collision_layer_bit")]
        public bool GetCollisionLayerBit(int bit)
        {
            return NativeCalls.godot_icall_1_29(method_bind_25, Object.GetPtr(this), bit);
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_mask_bit");

        /// <summary>
        /// </summary>
        [GodotMethod("set_collision_mask_bit")]
        public void SetCollisionMaskBit(int bit, bool value)
        {
            NativeCalls.godot_icall_2_51(method_bind_26, Object.GetPtr(this), bit, value);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_mask_bit");

        /// <summary>
        /// </summary>
        [GodotMethod("get_collision_mask_bit")]
        public bool GetCollisionMaskBit(int bit)
        {
            return NativeCalls.godot_icall_1_29(method_bind_27, Object.GetPtr(this), bit);
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_friction");

        [GodotMethod("set_collision_friction")]
        public void SetCollisionFriction(float value)
        {
            NativeCalls.godot_icall_1_10(method_bind_28, Object.GetPtr(this), value);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_friction");

        [GodotMethod("get_collision_friction")]
        public float GetCollisionFriction()
        {
            return NativeCalls.godot_icall_0_9(method_bind_29, Object.GetPtr(this));
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_collision_bounce");

        [GodotMethod("set_collision_bounce")]
        public void SetCollisionBounce(float value)
        {
            NativeCalls.godot_icall_1_10(method_bind_30, Object.GetPtr(this), value);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_collision_bounce");

        [GodotMethod("get_collision_bounce")]
        public float GetCollisionBounce()
        {
            return NativeCalls.godot_icall_0_9(method_bind_31, Object.GetPtr(this));
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_occluder_light_mask");

        [GodotMethod("set_occluder_light_mask")]
        public void SetOccluderLightMask(int mask)
        {
            NativeCalls.godot_icall_1_0(method_bind_32, Object.GetPtr(this), mask);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_occluder_light_mask");

        [GodotMethod("get_occluder_light_mask")]
        public int GetOccluderLightMask()
        {
            return NativeCalls.godot_icall_0_1(method_bind_33, Object.GetPtr(this));
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_cell");

        /// <summary>
        /// Set the tile index for the cell referenced by its grid-based X and Y coordinates.
        /// A tile index of -1 clears the cell.
        /// Optionally, the tile can also be flipped over the X and Y coordinates, transposed, or be given autotile coordinates.
        /// &lt;param name="autotileCoord"&gt;If the param is null, then the default value is new Vector2(0, 0)&lt;/param&gt;
        /// </summary>
        [GodotMethod("set_cell")]
        public void SetCell(int x, int y, int tile, bool flipX = false, bool flipY = false, bool transpose = false, Nullable<Vector2> autotileCoord = null)
        {
            Vector2 autotileCoord_in = autotileCoord.HasValue ? autotileCoord.Value : new Vector2(0, 0);
            NativeCalls.godot_icall_7_861(method_bind_34, Object.GetPtr(this), x, y, tile, flipX, flipY, transpose, ref autotileCoord_in);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_cellv");

        /// <summary>
        /// Set the tile index for the cell referenced by a Vector2 of grid-based coordinates.
        /// A tile index of -1 clears the cell.
        /// Optionally, the tile can also be flipped over the X and Y axes or transposed.
        /// </summary>
        [GodotMethod("set_cellv")]
        public void SetCellv(Vector2 position, int tile, bool flipX = false, bool flipY = false, bool transpose = false)
        {
            NativeCalls.godot_icall_5_862(method_bind_35, Object.GetPtr(this), ref position, tile, flipX, flipY, transpose);
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cell");

        /// <summary>
        /// Return the tile index of the referenced cell.
        /// </summary>
        [GodotMethod("get_cell")]
        public int GetCell(int x, int y)
        {
            return NativeCalls.godot_icall_2_171(method_bind_36, Object.GetPtr(this), x, y);
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_cellv");

        /// <summary>
        /// Return the tile index of the cell referenced by a Vector2.
        /// </summary>
        [GodotMethod("get_cellv")]
        public int GetCellv(Vector2 position)
        {
            return NativeCalls.godot_icall_1_863(method_bind_37, Object.GetPtr(this), ref position);
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_cell_x_flipped");

        /// <summary>
        /// Return whether the referenced cell is flipped over the X axis.
        /// </summary>
        [GodotMethod("is_cell_x_flipped")]
        public bool IsCellXFlipped(int x, int y)
        {
            return NativeCalls.godot_icall_2_33(method_bind_38, Object.GetPtr(this), x, y);
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_cell_y_flipped");

        /// <summary>
        /// Return whether the referenced cell is flipped over the Y axis.
        /// </summary>
        [GodotMethod("is_cell_y_flipped")]
        public bool IsCellYFlipped(int x, int y)
        {
            return NativeCalls.godot_icall_2_33(method_bind_39, Object.GetPtr(this), x, y);
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_cell_transposed");

        /// <summary>
        /// Return whether the referenced cell is transposed, i.e. the X and Y axes are swapped (mirroring with regard to the (1,1) vector).
        /// </summary>
        [GodotMethod("is_cell_transposed")]
        public bool IsCellTransposed(int x, int y)
        {
            return NativeCalls.godot_icall_2_33(method_bind_40, Object.GetPtr(this), x, y);
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear");

        /// <summary>
        /// Clear all cells.
        /// </summary>
        [GodotMethod("clear")]
        public void Clear()
        {
            NativeCalls.godot_icall_0_12(method_bind_41, Object.GetPtr(this));
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_used_cells");

        /// <summary>
        /// Return an array of all cells containing a tile from the tileset (i.e. a tile index different from -1).
        /// </summary>
        [GodotMethod("get_used_cells")]
        public object[] GetUsedCells()
        {
            return NativeCalls.godot_icall_0_21(method_bind_42, Object.GetPtr(this));
        }

        private IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_used_cells_by_id");

        /// <summary>
        /// </summary>
        [GodotMethod("get_used_cells_by_id")]
        public object[] GetUsedCellsById(int id)
        {
            return NativeCalls.godot_icall_1_116(method_bind_43, Object.GetPtr(this), id);
        }

        private IntPtr method_bind_44 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_used_rect");

        /// <summary>
        /// </summary>
        [GodotMethod("get_used_rect")]
        public Rect2 GetUsedRect()
        {
            object ret = NativeCalls.godot_icall_0_121(method_bind_44, Object.GetPtr(this));
            return (Rect2)ret;
        }

        private IntPtr method_bind_45 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "map_to_world");

        /// <summary>
        /// Return the absolute world position corresponding to the tilemap (grid-based) coordinates given as an argument.
        /// Optionally, the tilemap's potential half offset can be ignored.
        /// </summary>
        [GodotMethod("map_to_world")]
        public Vector2 MapToWorld(Vector2 mapPosition, bool ignoreHalfOfs = false)
        {
            object ret = NativeCalls.godot_icall_2_864(method_bind_45, Object.GetPtr(this), ref mapPosition, ignoreHalfOfs);
            return (Vector2)ret;
        }

        private IntPtr method_bind_46 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "world_to_map");

        /// <summary>
        /// Return the tilemap (grid-based) coordinates corresponding to the absolute world position given as an argument.
        /// </summary>
        [GodotMethod("world_to_map")]
        public Vector2 WorldToMap(Vector2 worldPosition)
        {
            object ret = NativeCalls.godot_icall_1_223(method_bind_46, Object.GetPtr(this), ref worldPosition);
            return (Vector2)ret;
        }

        private IntPtr method_bind_47 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "update_bitmask_area");

        /// <summary>
        /// Applies autotiling rules to the cell (and its adjacent cells) referenced by its grid-based X and Y coordinates.
        /// </summary>
        [GodotMethod("update_bitmask_area")]
        public void UpdateBitmaskArea(Vector2 position)
        {
            NativeCalls.godot_icall_1_45(method_bind_47, Object.GetPtr(this), ref position);
        }

        private IntPtr method_bind_48 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "update_bitmask_region");

        /// <summary>
        /// Applies autotiling rules to the cells in the given region (specified by grid-based X and Y coordinates).
        /// Calling with invalid (or missing) parameters applies autotiling rules for the entire TileMap.
        /// &lt;param name="start"&gt;If the param is null, then the default value is new Vector2(0, 0)&lt;/param&gt;
        /// &lt;param name="end"&gt;If the param is null, then the default value is new Vector2(0, 0)&lt;/param&gt;
        /// </summary>
        [GodotMethod("update_bitmask_region")]
        public void UpdateBitmaskRegion(Nullable<Vector2> start = null, Nullable<Vector2> end = null)
        {
            Vector2 start_in = start.HasValue ? start.Value : new Vector2(0, 0);
            Vector2 end_in = end.HasValue ? end.Value : new Vector2(0, 0);
            NativeCalls.godot_icall_2_742(method_bind_48, Object.GetPtr(this), ref start_in, ref end_in);
        }
    }
}
