using System;

namespace Godot
{
    /// <summary>
    /// Counts down a specified interval and emits a signal on reaching 0. Can be set to repeat or "one shot" mode.
    /// </summary>
    public class Timer : Node
    {
        public enum TimerProcessMode
        {
            Physics = 0,
            Idle = 1
        }


        /// <summary>
        /// Processing mode. Uses TIMER_PROCESS_* constants as value.
        /// </summary>
        public Timer.TimerProcessMode ProcessMode
        {
            get
            {
                return GetTimerProcessMode();
            }
            set
            {
                SetTimerProcessMode(value);
            }
        }

        /// <summary>
        /// Wait time in seconds.
        /// </summary>
        public float WaitTime
        {
            get
            {
                return GetWaitTime();
            }
            set
            {
                SetWaitTime(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], Timer will stop when reaching 0. If [code]false[/code], it will restart. Default value: [code]false[/code].
        /// </summary>
        public bool OneShot
        {
            get
            {
                return IsOneShot();
            }
            set
            {
                SetOneShot(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], Timer will automatically start when entering the scene tree. Default value: [code]false[/code].
        /// </summary>
        public bool Autostart
        {
            get
            {
                return HasAutostart();
            }
            set
            {
                SetAutostart(value);
            }
        }

        /// <summary>
        /// If [code]true[/code], the timer is paused and will not process until it is unpaused again, even if [method start] is called.
        /// </summary>
        public bool Paused
        {
            get
            {
                return IsPaused();
            }
            set
            {
                SetPaused(value);
            }
        }

        /// <summary>
        /// The timer's remaining time in seconds. Returns 0 if the timer is inactive.
        /// </summary>
        public float TimeLeft
        {
            get
            {
                return GetTimeLeft();
            }
        }


        private const string nativeName = "Timer";

        public Timer() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Timer_Ctor(this);
        }

        internal Timer(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_wait_time");

        [GodotMethod("set_wait_time")]
        public void SetWaitTime(float timeSec)
        {
            NativeCalls.godot_icall_1_10(method_bind_0, Object.GetPtr(this), timeSec);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_wait_time");

        [GodotMethod("get_wait_time")]
        public float GetWaitTime()
        {
            return NativeCalls.godot_icall_0_9(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_one_shot");

        [GodotMethod("set_one_shot")]
        public void SetOneShot(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_2, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_one_shot");

        [GodotMethod("is_one_shot")]
        public bool IsOneShot()
        {
            return NativeCalls.godot_icall_0_3(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_autostart");

        [GodotMethod("set_autostart")]
        public void SetAutostart(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_4, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_autostart");

        [GodotMethod("has_autostart")]
        public bool HasAutostart()
        {
            return NativeCalls.godot_icall_0_3(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "start");

        /// <summary>
        /// Starts the timer. This also resets the remaining time to [code]wait_time[/code].
        /// Note: this method will not resume a paused timer. See [method set_paused].
        /// </summary>
        [GodotMethod("start")]
        public void Start()
        {
            NativeCalls.godot_icall_0_12(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "stop");

        /// <summary>
        /// Stop (cancel) the Timer.
        /// </summary>
        [GodotMethod("stop")]
        public void Stop()
        {
            NativeCalls.godot_icall_0_12(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_paused");

        [GodotMethod("set_paused")]
        public void SetPaused(bool paused)
        {
            NativeCalls.godot_icall_1_11(method_bind_8, Object.GetPtr(this), paused);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_paused");

        [GodotMethod("is_paused")]
        public bool IsPaused()
        {
            return NativeCalls.godot_icall_0_3(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_stopped");

        /// <summary>
        /// Returns [code]true[/code] if the timer is stopped.
        /// </summary>
        [GodotMethod("is_stopped")]
        public bool IsStopped()
        {
            return NativeCalls.godot_icall_0_3(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_time_left");

        [GodotMethod("get_time_left")]
        public float GetTimeLeft()
        {
            return NativeCalls.godot_icall_0_9(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_timer_process_mode");

        [GodotMethod("set_timer_process_mode")]
        public void SetTimerProcessMode(Timer.TimerProcessMode mode)
        {
            NativeCalls.godot_icall_1_874(method_bind_12, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_timer_process_mode");

        [GodotMethod("get_timer_process_mode")]
        public Timer.TimerProcessMode GetTimerProcessMode()
        {
            return NativeCalls.godot_icall_0_875(method_bind_13, Object.GetPtr(this));
        }
    }
}
