using System;

namespace Godot
{
    /// <summary>
    /// This shows a tree of items that can be selected, expanded and collapsed. The tree can have multiple columns with custom controls like text editing, buttons and popups. It can be useful for structured displays and interactions.
    /// Trees are built via code, using [TreeItem] objects to create the structure. They have a single root but multiple roots can be simulated if a dummy hidden root is added.
    /// [codeblock]
    /// func _ready():
    /// var tree = Tree.new()
    /// var root = tree.create_item()
    /// tree.set_hide_root(true)
    /// var child1 = tree.create_item(root)
    /// var child2 = tree.create_item(root)
    /// var subchild1 = tree.create_item(child1)
    /// subchild1.set_text(0, "Subchild1")
    /// [/codeblock]
    /// </summary>
    public class Tree : Control
    {
        public enum SelectModeEnum
        {
            Single = 0,
            Row = 1,
            Multi = 2
        }

        public enum DropModeFlagsEnum
        {
            Disabled = 0,
            OnItem = 1,
            Inbetween = 2
        }


        /// <summary>
        /// The amount of columns.
        /// </summary>
        public int Columns
        {
            get
            {
                return GetColumns();
            }
            set
            {
                SetColumns(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the currently selected cell may be selected again.
        /// </summary>
        public bool AllowReselect
        {
            get
            {
                return GetAllowReselect();
            }
            set
            {
                SetAllowReselect(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] a right mouse button click can select items.
        /// </summary>
        public bool AllowRmbSelect
        {
            get
            {
                return GetAllowRmbSelect();
            }
            set
            {
                SetAllowRmbSelect(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the folding arrow is hidden.
        /// </summary>
        public bool HideFolding
        {
            get
            {
                return IsFoldingHidden();
            }
            set
            {
                SetHideFolding(value);
            }
        }

        /// <summary>
        /// If [code]true[/code] the tree's root is hidden.
        /// </summary>
        public bool HideRoot
        {
            get
            {
                return IsRootHidden();
            }
            set
            {
                SetHideRoot(value);
            }
        }

        /// <summary>
        /// The drop mode as an OR combination of flags. See [code]DROP_MODE_*[/code] constants.
        /// </summary>
        public int DropModeFlags
        {
            get
            {
                return GetDropModeFlags();
            }
            set
            {
                SetDropModeFlags(value);
            }
        }

        /// <summary>
        /// Allow single or multiple selection. See the [code]SELECT_*[/code] constants.
        /// </summary>
        public Tree.SelectModeEnum SelectMode
        {
            get
            {
                return GetSelectMode();
            }
            set
            {
                SetSelectMode(value);
            }
        }


        private const string nativeName = "Tree";

        public Tree() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Tree_Ctor(this);
        }

        internal Tree(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear");

        /// <summary>
        /// Clears the tree. This removes all items.
        /// </summary>
        [GodotMethod("clear")]
        public void Clear()
        {
            NativeCalls.godot_icall_0_12(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create_item");

        /// <summary>
        /// Create an item in the tree and add it as the last child of [code]parent[/code]. If parent is not given, it will be added as the root's last child, or it'll the be the root itself if the tree is empty.
        /// </summary>
        [GodotMethod("create_item")]
        public Object CreateItem(Object parent = null, int idx = -1)
        {
            return NativeCalls.godot_icall_2_878(method_bind_1, Object.GetPtr(this), Object.GetPtr(parent), idx);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_root");

        /// <summary>
        /// Returns the tree's root item.
        /// </summary>
        [GodotMethod("get_root")]
        public TreeItem GetRoot()
        {
            return NativeCalls.godot_icall_0_879(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_column_min_width");

        /// <summary>
        /// Set the minimum width of a column.
        /// </summary>
        [GodotMethod("set_column_min_width")]
        public void SetColumnMinWidth(int column, int minWidth)
        {
            NativeCalls.godot_icall_2_32(method_bind_3, Object.GetPtr(this), column, minWidth);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_column_expand");

        /// <summary>
        /// If [code]true[/code] the column will have the "Expand" flag of [Control].
        /// </summary>
        [GodotMethod("set_column_expand")]
        public void SetColumnExpand(int column, bool expand)
        {
            NativeCalls.godot_icall_2_51(method_bind_4, Object.GetPtr(this), column, expand);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_column_width");

        /// <summary>
        /// Returns the column's width in pixels.
        /// </summary>
        [GodotMethod("get_column_width")]
        public int GetColumnWidth(int column)
        {
            return NativeCalls.godot_icall_1_6(method_bind_5, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_hide_root");

        [GodotMethod("set_hide_root")]
        public void SetHideRoot(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_6, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_root_hidden");

        [GodotMethod("is_root_hidden")]
        public bool IsRootHidden()
        {
            return NativeCalls.godot_icall_0_3(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_next_selected");

        /// <summary>
        /// Returns the next selected item after the given one.
        /// </summary>
        [GodotMethod("get_next_selected")]
        public TreeItem GetNextSelected(Object from)
        {
            return NativeCalls.godot_icall_1_880(method_bind_8, Object.GetPtr(this), Object.GetPtr(from));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_selected");

        /// <summary>
        /// Returns the currently selected item.
        /// </summary>
        [GodotMethod("get_selected")]
        public TreeItem GetSelected()
        {
            return NativeCalls.godot_icall_0_879(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_selected_column");

        /// <summary>
        /// Returns the current selection's column.
        /// </summary>
        [GodotMethod("get_selected_column")]
        public int GetSelectedColumn()
        {
            return NativeCalls.godot_icall_0_1(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_pressed_button");

        /// <summary>
        /// Returns the last pressed button's index.
        /// </summary>
        [GodotMethod("get_pressed_button")]
        public int GetPressedButton()
        {
            return NativeCalls.godot_icall_0_1(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_select_mode");

        [GodotMethod("set_select_mode")]
        public void SetSelectMode(Tree.SelectModeEnum mode)
        {
            NativeCalls.godot_icall_1_881(method_bind_12, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_select_mode");

        [GodotMethod("get_select_mode")]
        public Tree.SelectModeEnum GetSelectMode()
        {
            return NativeCalls.godot_icall_0_882(method_bind_13, Object.GetPtr(this));
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_columns");

        [GodotMethod("set_columns")]
        public void SetColumns(int amount)
        {
            NativeCalls.godot_icall_1_0(method_bind_14, Object.GetPtr(this), amount);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_columns");

        [GodotMethod("get_columns")]
        public int GetColumns()
        {
            return NativeCalls.godot_icall_0_1(method_bind_15, Object.GetPtr(this));
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_edited");

        /// <summary>
        /// Returns the currently edited item. This is only available for custom cell mode.
        /// </summary>
        [GodotMethod("get_edited")]
        public TreeItem GetEdited()
        {
            return NativeCalls.godot_icall_0_879(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_edited_column");

        /// <summary>
        /// Returns the column for the currently edited item. This is only available for custom cell mode.
        /// </summary>
        [GodotMethod("get_edited_column")]
        public int GetEditedColumn()
        {
            return NativeCalls.godot_icall_0_1(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_custom_popup_rect");

        /// <summary>
        /// Returns the rectangle for custom popups. Helper to create custom cell controls that display a popup. See [method TreeItem.set_cell_mode].
        /// </summary>
        [GodotMethod("get_custom_popup_rect")]
        public Rect2 GetCustomPopupRect()
        {
            object ret = NativeCalls.godot_icall_0_121(method_bind_18, Object.GetPtr(this));
            return (Rect2)ret;
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_area_rect");

        /// <summary>
        /// Returns the rectangle area for the specified item. If column is specified, only get the position and size of that column, otherwise get the rectangle containing all columns.
        /// </summary>
        [GodotMethod("get_item_area_rect")]
        public Rect2 GetItemAreaRect(Object item, int column = -1)
        {
            object ret = NativeCalls.godot_icall_2_883(method_bind_19, Object.GetPtr(this), Object.GetPtr(item), column);
            return (Rect2)ret;
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_item_at_position");

        /// <summary>
        /// Returns the tree item at the specified position (relative to the tree origin position).
        /// </summary>
        [GodotMethod("get_item_at_position")]
        public TreeItem GetItemAtPosition(Vector2 position)
        {
            return NativeCalls.godot_icall_1_884(method_bind_20, Object.GetPtr(this), ref position);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_column_at_position");

        /// <summary>
        /// Returns the column index under the given point.
        /// </summary>
        [GodotMethod("get_column_at_position")]
        public int GetColumnAtPosition(Vector2 position)
        {
            return NativeCalls.godot_icall_1_863(method_bind_21, Object.GetPtr(this), ref position);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_drop_section_at_position");

        /// <summary>
        /// </summary>
        [GodotMethod("get_drop_section_at_position")]
        public int GetDropSectionAtPosition(Vector2 position)
        {
            return NativeCalls.godot_icall_1_863(method_bind_22, Object.GetPtr(this), ref position);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "ensure_cursor_is_visible");

        /// <summary>
        /// Makes the currently selected item visible. This will scroll the tree to make sure the selected item is visible.
        /// </summary>
        [GodotMethod("ensure_cursor_is_visible")]
        public void EnsureCursorIsVisible()
        {
            NativeCalls.godot_icall_0_12(method_bind_23, Object.GetPtr(this));
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_column_titles_visible");

        /// <summary>
        /// If [code]true[/code] column titles are visible.
        /// </summary>
        [GodotMethod("set_column_titles_visible")]
        public void SetColumnTitlesVisible(bool visible)
        {
            NativeCalls.godot_icall_1_11(method_bind_24, Object.GetPtr(this), visible);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "are_column_titles_visible");

        /// <summary>
        /// Returns [code]true[/code] if the column titles are being shown.
        /// </summary>
        [GodotMethod("are_column_titles_visible")]
        public bool AreColumnTitlesVisible()
        {
            return NativeCalls.godot_icall_0_3(method_bind_25, Object.GetPtr(this));
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_column_title");

        /// <summary>
        /// Set the title of a column.
        /// </summary>
        [GodotMethod("set_column_title")]
        public void SetColumnTitle(int column, string title)
        {
            NativeCalls.godot_icall_2_115(method_bind_26, Object.GetPtr(this), column, title);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_column_title");

        /// <summary>
        /// Returns the column's title.
        /// </summary>
        [GodotMethod("get_column_title")]
        public string GetColumnTitle(int column)
        {
            return NativeCalls.godot_icall_1_108(method_bind_27, Object.GetPtr(this), column);
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_scroll");

        /// <summary>
        /// Returns the current scrolling position.
        /// </summary>
        [GodotMethod("get_scroll")]
        public Vector2 GetScroll()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_28, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_hide_folding");

        [GodotMethod("set_hide_folding")]
        public void SetHideFolding(bool hide)
        {
            NativeCalls.godot_icall_1_11(method_bind_29, Object.GetPtr(this), hide);
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_folding_hidden");

        [GodotMethod("is_folding_hidden")]
        public bool IsFoldingHidden()
        {
            return NativeCalls.godot_icall_0_3(method_bind_30, Object.GetPtr(this));
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_drop_mode_flags");

        [GodotMethod("set_drop_mode_flags")]
        public void SetDropModeFlags(int flags)
        {
            NativeCalls.godot_icall_1_0(method_bind_31, Object.GetPtr(this), flags);
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_drop_mode_flags");

        [GodotMethod("get_drop_mode_flags")]
        public int GetDropModeFlags()
        {
            return NativeCalls.godot_icall_0_1(method_bind_32, Object.GetPtr(this));
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_allow_rmb_select");

        [GodotMethod("set_allow_rmb_select")]
        public void SetAllowRmbSelect(bool allow)
        {
            NativeCalls.godot_icall_1_11(method_bind_33, Object.GetPtr(this), allow);
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_allow_rmb_select");

        [GodotMethod("get_allow_rmb_select")]
        public bool GetAllowRmbSelect()
        {
            return NativeCalls.godot_icall_0_3(method_bind_34, Object.GetPtr(this));
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_allow_reselect");

        [GodotMethod("set_allow_reselect")]
        public void SetAllowReselect(bool allow)
        {
            NativeCalls.godot_icall_1_11(method_bind_35, Object.GetPtr(this), allow);
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_allow_reselect");

        [GodotMethod("get_allow_reselect")]
        public bool GetAllowReselect()
        {
            return NativeCalls.godot_icall_0_3(method_bind_36, Object.GetPtr(this));
        }
    }
}
