using System;

namespace Godot
{
    /// <summary>
    /// Node useful for animations with unknown start and end points, procedural animations, making one node follow another, and other simple behavior.
    /// Because it is easy to get it wrong, here is a quick usage example:
    /// [codeblock]
    /// var tween = get_node("Tween")
    /// tween.interpolate_property(get_node("Node2D_to_move"), "transform/origin", Vector2(0,0), Vector2(100,100), 1, Tween.TRANS_LINEAR, Tween.EASE_IN_OUT)
    /// tween.start()
    /// [/codeblock]
    /// Some of the methods of this class require a property name. You can get the property name by hovering over the property in the inspector of the editor.
    /// Many of the methods accept [code]trans_type[/code] and [code]ease_type[/code]. The first accepts an TRANS_* constant, and refers to the way the timing of the animation is handled (you might want to see [code]http://easings.net/[/code] for some examples). The second accepts an EASE_* constant, and controls the where [code]trans_type[/code] is applied to the interpolation (in the beginning, the end, or both). If you don't know which transition and easing to pick, you can try different TRANS_* constants with EASE_IN_OUT, and use the one that looks best.
    /// </summary>
    public class Tween : Node
    {
        public enum TransitionType
        {
            Linear = 0,
            Sine = 1,
            Quint = 2,
            Quart = 3,
            Quad = 4,
            Expo = 5,
            Elastic = 6,
            Cubic = 7,
            Circ = 8,
            Bounce = 9,
            Back = 10
        }

        public enum TweenProcessMode
        {
            Physics = 0,
            Idle = 1
        }

        public enum EaseType
        {
            In = 0,
            Out = 1,
            InOut = 2,
            OutIn = 3
        }


        /// <summary>
        /// If [code]true[/code], the tween will repeat.
        /// </summary>
        public bool Repeat
        {
            get
            {
                return IsRepeat();
            }
            set
            {
                SetRepeat(value);
            }
        }

        /// <summary>
        /// </summary>
        public Tween.TweenProcessMode PlaybackProcessMode
        {
            get
            {
                return GetTweenProcessMode();
            }
            set
            {
                SetTweenProcessMode(value);
            }
        }

        /// <summary>
        /// The speed multiplier of the tween. Set it to 1 for normal speed, 2 for two times nromal speed, and 0.5 for half of the normal speed. Setting it to 0 would pause the animation, but you might consider using [method set_active] or [method stop_all] and [method resume_all] for this.
        /// </summary>
        public float PlaybackSpeed
        {
            get
            {
                return GetSpeedScale();
            }
            set
            {
                SetSpeedScale(value);
            }
        }


        private const string nativeName = "Tween";

        public Tween() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Tween_Ctor(this);
        }

        internal Tween(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_active");

        /// <summary>
        /// Returns true if any tweens are currently running, and false otherwise. Note that this method doesn't consider tweens that have ended.
        /// </summary>
        [GodotMethod("is_active")]
        public bool IsActive()
        {
            return NativeCalls.godot_icall_0_3(method_bind_0, Object.GetPtr(this));
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_active");

        /// <summary>
        /// Activate/deactivate the tween. You can use this for pausing animations, though [method stop_all] and [method resume_all] might be more fit for this.
        /// </summary>
        [GodotMethod("set_active")]
        public void SetActive(bool active)
        {
            NativeCalls.godot_icall_1_11(method_bind_1, Object.GetPtr(this), active);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_repeat");

        [GodotMethod("is_repeat")]
        public bool IsRepeat()
        {
            return NativeCalls.godot_icall_0_3(method_bind_2, Object.GetPtr(this));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_repeat");

        [GodotMethod("set_repeat")]
        public void SetRepeat(bool repeat)
        {
            NativeCalls.godot_icall_1_11(method_bind_3, Object.GetPtr(this), repeat);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_speed_scale");

        [GodotMethod("set_speed_scale")]
        public void SetSpeedScale(float speed)
        {
            NativeCalls.godot_icall_1_10(method_bind_4, Object.GetPtr(this), speed);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_speed_scale");

        [GodotMethod("get_speed_scale")]
        public float GetSpeedScale()
        {
            return NativeCalls.godot_icall_0_9(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_tween_process_mode");

        [GodotMethod("set_tween_process_mode")]
        public void SetTweenProcessMode(Tween.TweenProcessMode mode)
        {
            NativeCalls.godot_icall_1_895(method_bind_6, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_tween_process_mode");

        [GodotMethod("get_tween_process_mode")]
        public Tween.TweenProcessMode GetTweenProcessMode()
        {
            return NativeCalls.godot_icall_0_896(method_bind_7, Object.GetPtr(this));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "start");

        /// <summary>
        /// Start the tween node. You can define tweens both before and after this.
        /// </summary>
        [GodotMethod("start")]
        public bool Start()
        {
            return NativeCalls.godot_icall_0_3(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "reset");

        /// <summary>
        /// Resets a tween to the initial value (the one given, not the one before the tween), given its object and property/method pair. Passing empty String as key will reset all tweens for given object.
        /// </summary>
        [GodotMethod("reset")]
        public bool Reset(Object @object, string key = "")
        {
            return NativeCalls.godot_icall_2_897(method_bind_9, Object.GetPtr(this), Object.GetPtr(@object), key);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "reset_all");

        /// <summary>
        /// Resets all tweens to their initial values (the ones given, not those before the tween).
        /// </summary>
        [GodotMethod("reset_all")]
        public bool ResetAll()
        {
            return NativeCalls.godot_icall_0_3(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "stop");

        /// <summary>
        /// Stop animating a tween, given its object and property/method pair. Passing empty String as key will stop all tweens for given object.
        /// </summary>
        [GodotMethod("stop")]
        public bool Stop(Object @object, string key = "")
        {
            return NativeCalls.godot_icall_2_897(method_bind_11, Object.GetPtr(this), Object.GetPtr(@object), key);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "stop_all");

        /// <summary>
        /// Stop animating all tweens.
        /// </summary>
        [GodotMethod("stop_all")]
        public bool StopAll()
        {
            return NativeCalls.godot_icall_0_3(method_bind_12, Object.GetPtr(this));
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "resume");

        /// <summary>
        /// Continue animating a stopped tween, given its object and property/method pair. Passing empty String as key will resume all tweens for given object.
        /// </summary>
        [GodotMethod("resume")]
        public bool Resume(Object @object, string key = "")
        {
            return NativeCalls.godot_icall_2_897(method_bind_13, Object.GetPtr(this), Object.GetPtr(@object), key);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "resume_all");

        /// <summary>
        /// Continue animating all stopped tweens.
        /// </summary>
        [GodotMethod("resume_all")]
        public bool ResumeAll()
        {
            return NativeCalls.godot_icall_0_3(method_bind_14, Object.GetPtr(this));
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove");

        /// <summary>
        /// Stop animating and completely remove a tween, given its object and property/method pair. Passing empty String as key will remove all tweens for given object.
        /// </summary>
        [GodotMethod("remove")]
        public bool Remove(Object @object, string key = "")
        {
            return NativeCalls.godot_icall_2_897(method_bind_15, Object.GetPtr(this), Object.GetPtr(@object), key);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_all");

        /// <summary>
        /// Stop animating and completely remove all tweens.
        /// </summary>
        [GodotMethod("remove_all")]
        public bool RemoveAll()
        {
            return NativeCalls.godot_icall_0_3(method_bind_16, Object.GetPtr(this));
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "seek");

        /// <summary>
        /// Seek the animation to the given [code]time[/code] in seconds.
        /// </summary>
        [GodotMethod("seek")]
        public bool Seek(float time)
        {
            return NativeCalls.godot_icall_1_501(method_bind_17, Object.GetPtr(this), time);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "tell");

        /// <summary>
        /// Returns the current time of the tween.
        /// </summary>
        [GodotMethod("tell")]
        public float Tell()
        {
            return NativeCalls.godot_icall_0_9(method_bind_18, Object.GetPtr(this));
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_runtime");

        /// <summary>
        /// Returns the time needed for all tweens to end in seconds, measured from the start. Thus, if you have two tweens, one ending 10 seconds after the start and the other - 20 seconds, it would return 20 seconds, as by that time all tweens would have finished.
        /// </summary>
        [GodotMethod("get_runtime")]
        public float GetRuntime()
        {
            return NativeCalls.godot_icall_0_9(method_bind_19, Object.GetPtr(this));
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "interpolate_property");

        /// <summary>
        /// Animate [code]property[/code] of [code]object[/code] from [code]initial_val[/code] to [code]final_val[/code] for [code]duration[/code] seconds, [code]delay[/code] seconds later.
        /// [code]trans_type[/code] accepts TRANS_* constants, and is the way the animation is interpolated, while [code]ease_type[/code] accepts EASE_* constants, and controls the place of the interpolation (the beginning, the end, or both). You can read more about them in the class description.
        /// </summary>
        [GodotMethod("interpolate_property")]
        public bool InterpolateProperty(Object @object, NodePath property, object initialVal, object finalVal, float duration, Tween.TransitionType transType, Tween.EaseType easeType, float delay = (float)0)
        {
            return NativeCalls.godot_icall_8_898(method_bind_20, Object.GetPtr(this), Object.GetPtr(@object), NodePath.GetPtr(property), initialVal, finalVal, duration, transType, easeType, delay);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "interpolate_method");

        /// <summary>
        /// Animate [code]method[/code] of [code]object[/code] from [code]initial_val[/code] to [code]final_val[/code] for [code]duration[/code] seconds, [code]delay[/code] seconds later. Methods are animated by calling them with consecutive values.
        /// [code]trans_type[/code] accepts TRANS_* constants, and is the way the animation is interpolated, while [code]ease_type[/code] accepts EASE_* constants, and controls the place of the interpolation (the beginning, the end, or both). You can read more about them in the class description.
        /// </summary>
        [GodotMethod("interpolate_method")]
        public bool InterpolateMethod(Object @object, string method, object initialVal, object finalVal, float duration, Tween.TransitionType transType, Tween.EaseType easeType, float delay = (float)0)
        {
            return NativeCalls.godot_icall_8_899(method_bind_21, Object.GetPtr(this), Object.GetPtr(@object), method, initialVal, finalVal, duration, transType, easeType, delay);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "interpolate_callback");

        /// <summary>
        /// Call [code]callback[/code] of [code]object[/code] after [code]duration[/code]. [code]arg1[/code]-[code]arg5[/code] are arguments to be passed to the callback.
        /// </summary>
        [GodotMethod("interpolate_callback")]
        public bool InterpolateCallback(Object @object, float duration, string callback, object arg1 = null, object arg2 = null, object arg3 = null, object arg4 = null, object arg5 = null)
        {
            return NativeCalls.godot_icall_8_900(method_bind_22, Object.GetPtr(this), Object.GetPtr(@object), duration, callback, arg1, arg2, arg3, arg4, arg5);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "interpolate_deferred_callback");

        /// <summary>
        /// Call [code]callback[/code] of [code]object[/code] after [code]duration[/code] on the main thread (similar to [method Object.call_deferred]). [code]arg1[/code]-[code]arg5[/code] are arguments to be passed to the callback.
        /// </summary>
        [GodotMethod("interpolate_deferred_callback")]
        public bool InterpolateDeferredCallback(Object @object, float duration, string callback, object arg1 = null, object arg2 = null, object arg3 = null, object arg4 = null, object arg5 = null)
        {
            return NativeCalls.godot_icall_8_900(method_bind_23, Object.GetPtr(this), Object.GetPtr(@object), duration, callback, arg1, arg2, arg3, arg4, arg5);
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "follow_property");

        /// <summary>
        /// Follow [code]property[/code] of [code]object[/code] and apply it on [code]target_property[/code] of [code]target[/code], beginning from [code]initial_val[/code] for [code]duration[/code] seconds, [code]delay[/code] seconds later. Note that [code]target:target_property[/code] would equal [code]object:property[/code] at the end of the tween.
        /// [code]trans_type[/code] accepts TRANS_* constants, and is the way the animation is interpolated, while [code]ease_type[/code] accepts EASE_* constants, and controls the place of the interpolation (the beginning, the end, or both). You can read more about them in the class description.
        /// </summary>
        [GodotMethod("follow_property")]
        public bool FollowProperty(Object @object, NodePath property, object initialVal, Object target, NodePath targetProperty, float duration, Tween.TransitionType transType, Tween.EaseType easeType, float delay = (float)0)
        {
            return NativeCalls.godot_icall_9_901(method_bind_24, Object.GetPtr(this), Object.GetPtr(@object), NodePath.GetPtr(property), initialVal, Object.GetPtr(target), NodePath.GetPtr(targetProperty), duration, transType, easeType, delay);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "follow_method");

        /// <summary>
        /// Follow [code]method[/code] of [code]object[/code] and apply the returned value on [code]target_method[/code] of [code]target[/code], beginning from [code]initial_val[/code] for [code]duration[/code] seconds, [code]delay[/code] later. Methods are animated by calling them with consequitive values.
        /// [code]trans_type[/code] accepts TRANS_* constants, and is the way the animation is interpolated, while [code]ease_type[/code] accepts EASE_* constants, and controls the place of the interpolation (the beginning, the end, or both). You can read more about them in the class description.
        /// </summary>
        [GodotMethod("follow_method")]
        public bool FollowMethod(Object @object, string method, object initialVal, Object target, string targetMethod, float duration, Tween.TransitionType transType, Tween.EaseType easeType, float delay = (float)0)
        {
            return NativeCalls.godot_icall_9_902(method_bind_25, Object.GetPtr(this), Object.GetPtr(@object), method, initialVal, Object.GetPtr(target), targetMethod, duration, transType, easeType, delay);
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "targeting_property");

        /// <summary>
        /// Animate [code]property[/code] of [code]object[/code] from the current value of the [code]initial_val[/code] property of [code]initial[/code] to [code]final_val[/code] for [code]duration[/code] seconds, [code]delay[/code] seconds later.
        /// [code]trans_type[/code] accepts TRANS_* constants, and is the way the animation is interpolated, while [code]ease_type[/code] accepts EASE_* constants, and controls the place of the interpolation (the beginning, the end, or both). You can read more about them in the class description.
        /// </summary>
        [GodotMethod("targeting_property")]
        public bool TargetingProperty(Object @object, NodePath property, Object initial, NodePath initialVal, object finalVal, float duration, Tween.TransitionType transType, Tween.EaseType easeType, float delay = (float)0)
        {
            return NativeCalls.godot_icall_9_903(method_bind_26, Object.GetPtr(this), Object.GetPtr(@object), NodePath.GetPtr(property), Object.GetPtr(initial), NodePath.GetPtr(initialVal), finalVal, duration, transType, easeType, delay);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "targeting_method");

        /// <summary>
        /// Animate [code]method[/code] of [code]object[/code] from the value returned by [code]initial.initial_method[/code] to [code]final_val[/code] for [code]duration[/code] seconds, [code]delay[/code] seconds later. Methods are animated by calling them with consecutive values.
        /// [code]trans_type[/code] accepts TRANS_* constants, and is the way the animation is interpolated, while [code]ease_type[/code] accepts EASE_* constants, and controls the place of the interpolation (the beginning, the end, or both). You can read more about them in the class description.
        /// </summary>
        [GodotMethod("targeting_method")]
        public bool TargetingMethod(Object @object, string method, Object initial, string initialMethod, object finalVal, float duration, Tween.TransitionType transType, Tween.EaseType easeType, float delay = (float)0)
        {
            return NativeCalls.godot_icall_9_904(method_bind_27, Object.GetPtr(this), Object.GetPtr(@object), method, Object.GetPtr(initial), initialMethod, finalVal, duration, transType, easeType, delay);
        }
    }
}
