using System;

namespace Godot
{
    /// <summary>
    /// Helper to manage UndoRedo in the editor or custom tools. It works by storing calls to functions in both 'do' an 'undo' lists.
    /// Common behavior is to create an action, then add do/undo calls to functions or property changes, then committing the action.
    /// </summary>
    public class UndoRedo : Object
    {
        public enum MergeMode
        {
            Disable = 0,
            Ends = 1,
            All = 2
        }


        private const string nativeName = "UndoRedo";

        public UndoRedo() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_UndoRedo_Ctor(this);
        }

        internal UndoRedo(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "create_action");

        /// <summary>
        /// Create a new action. After this is called, do all your calls to [method add_do_method], [method add_undo_method], [method add_do_property] and [method add_undo_property].
        /// </summary>
        [GodotMethod("create_action")]
        public void CreateAction(string name, UndoRedo.MergeMode mergeMode = (UndoRedo.MergeMode)0)
        {
            NativeCalls.godot_icall_2_905(method_bind_0, Object.GetPtr(this), name, mergeMode);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "commit_action");

        /// <summary>
        /// Commit the action. All 'do' methods/properties are called/set when this function is called.
        /// </summary>
        [GodotMethod("commit_action")]
        public void CommitAction()
        {
            NativeCalls.godot_icall_0_12(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_do_method");

        /// <summary>
        /// </summary>
        [GodotMethod("add_do_method")]
        public object AddDoMethod(Object @object, string method, params object[] @args)
        {
            return NativeCalls.godot_icall_3_906(method_bind_2, Object.GetPtr(this), Object.GetPtr(@object), method, @args);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_undo_method");

        /// <summary>
        /// </summary>
        [GodotMethod("add_undo_method")]
        public object AddUndoMethod(Object @object, string method, params object[] @args)
        {
            return NativeCalls.godot_icall_3_906(method_bind_3, Object.GetPtr(this), Object.GetPtr(@object), method, @args);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_do_property");

        /// <summary>
        /// Set a property with a custom value.
        /// </summary>
        [GodotMethod("add_do_property")]
        public void AddDoProperty(Object @object, string property, object value)
        {
            NativeCalls.godot_icall_3_907(method_bind_4, Object.GetPtr(this), Object.GetPtr(@object), property, value);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_undo_property");

        /// <summary>
        /// Undo setting of a property with a custom value.
        /// </summary>
        [GodotMethod("add_undo_property")]
        public void AddUndoProperty(Object @object, string property, object value)
        {
            NativeCalls.godot_icall_3_907(method_bind_5, Object.GetPtr(this), Object.GetPtr(@object), property, value);
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_do_reference");

        /// <summary>
        /// Add a 'do' reference that will be erased if the 'do' history is lost. This is useful mostly for new nodes created for the 'do' call. Do not use for resources.
        /// </summary>
        [GodotMethod("add_do_reference")]
        public void AddDoReference(Object @object)
        {
            NativeCalls.godot_icall_1_42(method_bind_6, Object.GetPtr(this), Object.GetPtr(@object));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_undo_reference");

        /// <summary>
        /// Add an 'undo' reference that will be erased if the 'undo' history is lost. This is useful mostly for nodes removed with the 'do' call (not the 'undo' call!).
        /// </summary>
        [GodotMethod("add_undo_reference")]
        public void AddUndoReference(Object @object)
        {
            NativeCalls.godot_icall_1_42(method_bind_7, Object.GetPtr(this), Object.GetPtr(@object));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "clear_history");

        /// <summary>
        /// Clear the undo/redo history and associated references.
        /// </summary>
        [GodotMethod("clear_history")]
        public void ClearHistory()
        {
            NativeCalls.godot_icall_0_12(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_current_action_name");

        /// <summary>
        /// Get the name of the current action.
        /// </summary>
        [GodotMethod("get_current_action_name")]
        public string GetCurrentActionName()
        {
            return NativeCalls.godot_icall_0_2(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_version");

        /// <summary>
        /// Get the version, each time a new action is committed, the version number of the UndoRedo is increased automatically.
        /// This is useful mostly to check if something changed from a saved version.
        /// </summary>
        [GodotMethod("get_version")]
        public int GetVersion()
        {
            return NativeCalls.godot_icall_0_1(method_bind_10, Object.GetPtr(this));
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "redo");

        /// <summary>
        /// </summary>
        [GodotMethod("redo")]
        public void Redo()
        {
            NativeCalls.godot_icall_0_12(method_bind_11, Object.GetPtr(this));
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "undo");

        /// <summary>
        /// </summary>
        [GodotMethod("undo")]
        public void Undo()
        {
            NativeCalls.godot_icall_0_12(method_bind_12, Object.GetPtr(this));
        }
    }
}
