using System;

namespace Godot
{
    /// <summary>
    /// A Viewport creates a different view into the screen, or a sub-view inside another viewport. Children 2D Nodes will display on it, and children Camera 3D nodes will render on it too.
    /// Optionally, a viewport can have its own 2D or 3D world, so they don't share what they draw with other viewports.
    /// If a viewport is a child of a [Control], it will automatically take up its same rect and position, otherwise they must be set manually.
    /// Viewports can also choose to be audio listeners, so they generate positional audio depending on a 2D or 3D camera child of it.
    /// Also, viewports can be assigned to different screens in case the devices have multiple screens.
    /// Finally, viewports can also behave as render targets, in which case they will not be visible unless the associated texture is used to draw.
    /// </summary>
    public class Viewport : Node
    {
        public enum ClearMode
        {
            Always = 0,
            Never = 1,
            OnlyNextFrame = 2
        }

        public enum RenderInfo
        {
            ObjectsInFrame = 0,
            VerticesInFrame = 1,
            MaterialChangesInFrame = 2,
            ShaderChangesInFrame = 3,
            SurfaceChangesInFrame = 4,
            DrawCallsInFrame = 5,
            Max = 6
        }

        public enum UsageEnum
        {
            Usage2d = 0,
            Usage2dNoSampling = 1,
            Usage3d = 2,
            Usage3dNoEffects = 3
        }

        public enum DebugDrawEnum
        {
            Disabled = 0,
            Unshaded = 1,
            Overdraw = 2,
            Wireframe = 3
        }

        public enum ShadowAtlasQuadrantSubdiv
        {
            Disabled = 0,
            Subdiv1 = 1,
            Subdiv4 = 2,
            Subdiv16 = 3,
            Subdiv64 = 4,
            Subdiv256 = 5,
            Subdiv1024 = 6,
            Max = 7
        }

        public enum UpdateMode
        {
            Disabled = 0,
            Once = 1,
            WhenVisible = 2,
            Always = 3
        }

        public enum MSAA
        {
            Disabled = 0,
            Msaa2x = 1,
            Msaa4x = 2,
            Msaa8x = 3,
            Msaa16x = 4
        }


        /// <summary>
        /// </summary>
        public bool Arvr
        {
            get
            {
                return UseArvr();
            }
            set
            {
                SetUseArvr(value);
            }
        }

        /// <summary>
        /// </summary>
        public Vector2 Size
        {
            get
            {
                return GetSize();
            }
            set
            {
                SetSize(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool OwnWorld
        {
            get
            {
                return IsUsingOwnWorld();
            }
            set
            {
                SetUseOwnWorld(value);
            }
        }

        /// <summary>
        /// </summary>
        public World World
        {
            get
            {
                return GetWorld();
            }
            set
            {
                SetWorld(value);
            }
        }

        /// <summary>
        /// </summary>
        public World2D World2d
        {
            get
            {
                return GetWorld2d();
            }
            set
            {
                SetWorld2d(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool TransparentBg
        {
            get
            {
                return HasTransparentBackground();
            }
            set
            {
                SetTransparentBackground(value);
            }
        }

        /// <summary>
        /// </summary>
        public Viewport.MSAA Msaa
        {
            get
            {
                return GetMsaa();
            }
            set
            {
                SetMsaa(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool Hdr
        {
            get
            {
                return GetHdr();
            }
            set
            {
                SetHdr(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool Disable3d
        {
            get
            {
                return Is3dDisabled();
            }
            set
            {
                SetDisable3d(value);
            }
        }

        /// <summary>
        /// </summary>
        public Viewport.UsageEnum Usage
        {
            get
            {
                return GetUsage();
            }
            set
            {
                SetUsage(value);
            }
        }

        /// <summary>
        /// </summary>
        public Viewport.DebugDrawEnum DebugDraw
        {
            get
            {
                return GetDebugDraw();
            }
            set
            {
                SetDebugDraw(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool RenderTargetVFlip
        {
            get
            {
                return GetVflip();
            }
            set
            {
                SetVflip(value);
            }
        }

        /// <summary>
        /// </summary>
        public Viewport.ClearMode RenderTargetClearMode
        {
            get
            {
                return GetClearMode();
            }
            set
            {
                SetClearMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public Viewport.UpdateMode RenderTargetUpdateMode
        {
            get
            {
                return GetUpdateMode();
            }
            set
            {
                SetUpdateMode(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool AudioListenerEnable2d
        {
            get
            {
                return IsAudioListener2d();
            }
            set
            {
                SetAsAudioListener2d(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool AudioListenerEnable3d
        {
            get
            {
                return IsAudioListener();
            }
            set
            {
                SetAsAudioListener(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool PhysicsObjectPicking
        {
            get
            {
                return GetPhysicsObjectPicking();
            }
            set
            {
                SetPhysicsObjectPicking(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool GuiDisableInput
        {
            get
            {
                return IsInputDisabled();
            }
            set
            {
                SetDisableInput(value);
            }
        }

        /// <summary>
        /// </summary>
        public bool GuiSnapControlsToPixels
        {
            get
            {
                return IsSnapControlsToPixelsEnabled();
            }
            set
            {
                SetSnapControlsToPixels(value);
            }
        }

        /// <summary>
        /// </summary>
        public int ShadowAtlasSize
        {
            get
            {
                return GetShadowAtlasSize();
            }
            set
            {
                SetShadowAtlasSize(value);
            }
        }

        /// <summary>
        /// </summary>
        public Viewport.ShadowAtlasQuadrantSubdiv ShadowAtlasQuad0
        {
            get
            {
                return GetShadowAtlasQuadrantSubdiv(0);
            }
            set
            {
                SetShadowAtlasQuadrantSubdiv(0, value);
            }
        }

        /// <summary>
        /// </summary>
        public Viewport.ShadowAtlasQuadrantSubdiv ShadowAtlasQuad1
        {
            get
            {
                return GetShadowAtlasQuadrantSubdiv(1);
            }
            set
            {
                SetShadowAtlasQuadrantSubdiv(1, value);
            }
        }

        /// <summary>
        /// </summary>
        public Viewport.ShadowAtlasQuadrantSubdiv ShadowAtlasQuad2
        {
            get
            {
                return GetShadowAtlasQuadrantSubdiv(2);
            }
            set
            {
                SetShadowAtlasQuadrantSubdiv(2, value);
            }
        }

        /// <summary>
        /// </summary>
        public Viewport.ShadowAtlasQuadrantSubdiv ShadowAtlasQuad3
        {
            get
            {
                return GetShadowAtlasQuadrantSubdiv(3);
            }
            set
            {
                SetShadowAtlasQuadrantSubdiv(3, value);
            }
        }

        /// <summary>
        /// The canvas transform of the viewport, useful for changing the on-screen positions of all child [CanvasItem]s. This is relative to the global canvas transform of the viewport.
        /// </summary>
        public Transform2D CanvasTransform
        {
            get
            {
                return GetCanvasTransform();
            }
            set
            {
                SetCanvasTransform(value);
            }
        }

        /// <summary>
        /// The global canvas transform of the viewport. The canvas transform is relative to this.
        /// </summary>
        public Transform2D GlobalCanvasTransform
        {
            get
            {
                return GetGlobalCanvasTransform();
            }
            set
            {
                SetGlobalCanvasTransform(value);
            }
        }


        private const string nativeName = "Viewport";

        public Viewport() : this(false)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_Viewport_Ctor(this);
        }

        internal Viewport(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_use_arvr");

        [GodotMethod("set_use_arvr")]
        public void SetUseArvr(bool use)
        {
            NativeCalls.godot_icall_1_11(method_bind_0, Object.GetPtr(this), use);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "use_arvr");

        [GodotMethod("use_arvr")]
        public bool UseArvr()
        {
            return NativeCalls.godot_icall_0_3(method_bind_1, Object.GetPtr(this));
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_size");

        [GodotMethod("set_size")]
        public void SetSize(Vector2 size)
        {
            NativeCalls.godot_icall_1_45(method_bind_2, Object.GetPtr(this), ref size);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_size");

        [GodotMethod("get_size")]
        public Vector2 GetSize()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_3, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_world_2d");

        [GodotMethod("set_world_2d")]
        public void SetWorld2d(World2D world2d)
        {
            NativeCalls.godot_icall_1_24(method_bind_4, Object.GetPtr(this), Object.GetPtr(world2d));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_world_2d");

        [GodotMethod("get_world_2d")]
        public World2D GetWorld2d()
        {
            return NativeCalls.godot_icall_0_221(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "find_world_2d");

        /// <summary>
        /// Return the 2D world of the viewport.
        /// </summary>
        [GodotMethod("find_world_2d")]
        public World2D FindWorld2d()
        {
            return NativeCalls.godot_icall_0_221(method_bind_6, Object.GetPtr(this));
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_world");

        [GodotMethod("set_world")]
        public void SetWorld(World world)
        {
            NativeCalls.godot_icall_1_24(method_bind_7, Object.GetPtr(this), Object.GetPtr(world));
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_world");

        [GodotMethod("get_world")]
        public World GetWorld()
        {
            return NativeCalls.godot_icall_0_772(method_bind_8, Object.GetPtr(this));
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "find_world");

        /// <summary>
        /// Return the 3D world of the viewport, or if no such present, the one of the parent viewport.
        /// </summary>
        [GodotMethod("find_world")]
        public World FindWorld()
        {
            return NativeCalls.godot_icall_0_772(method_bind_9, Object.GetPtr(this));
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_canvas_transform");

        [GodotMethod("set_canvas_transform")]
        public void SetCanvasTransform(Transform2D xform)
        {
            NativeCalls.godot_icall_1_219(method_bind_10, Object.GetPtr(this), ref xform);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_canvas_transform");

        [GodotMethod("get_canvas_transform")]
        public Transform2D GetCanvasTransform()
        {
            object ret = NativeCalls.godot_icall_0_220(method_bind_11, Object.GetPtr(this));
            return (Transform2D)ret;
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_global_canvas_transform");

        [GodotMethod("set_global_canvas_transform")]
        public void SetGlobalCanvasTransform(Transform2D xform)
        {
            NativeCalls.godot_icall_1_219(method_bind_12, Object.GetPtr(this), ref xform);
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_global_canvas_transform");

        [GodotMethod("get_global_canvas_transform")]
        public Transform2D GetGlobalCanvasTransform()
        {
            object ret = NativeCalls.godot_icall_0_220(method_bind_13, Object.GetPtr(this));
            return (Transform2D)ret;
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_final_transform");

        /// <summary>
        /// Get the total transform of the viewport.
        /// </summary>
        [GodotMethod("get_final_transform")]
        public Transform2D GetFinalTransform()
        {
            object ret = NativeCalls.godot_icall_0_220(method_bind_14, Object.GetPtr(this));
            return (Transform2D)ret;
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_visible_rect");

        /// <summary>
        /// Return the final, visible rect in global screen coordinates.
        /// </summary>
        [GodotMethod("get_visible_rect")]
        public Rect2 GetVisibleRect()
        {
            object ret = NativeCalls.godot_icall_0_121(method_bind_15, Object.GetPtr(this));
            return (Rect2)ret;
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_transparent_background");

        [GodotMethod("set_transparent_background")]
        public void SetTransparentBackground(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_16, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_transparent_background");

        [GodotMethod("has_transparent_background")]
        public bool HasTransparentBackground()
        {
            return NativeCalls.godot_icall_0_3(method_bind_17, Object.GetPtr(this));
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_size_override");

        /// <summary>
        /// Set the size override of the viewport. If the enable parameter is true, it would use the override, otherwise it would use the default size. If the size parameter is equal to [code](-1, -1)[/code], it won't update the size.
        /// &lt;param name="size"&gt;If the param is null, then the default value is new Vector2(-1, -1)&lt;/param&gt;
        /// &lt;param name="margin"&gt;If the param is null, then the default value is new Vector2(0, 0)&lt;/param&gt;
        /// </summary>
        [GodotMethod("set_size_override")]
        public void SetSizeOverride(bool enable, Nullable<Vector2> size = null, Nullable<Vector2> margin = null)
        {
            Vector2 size_in = size.HasValue ? size.Value : new Vector2(-1, -1);
            Vector2 margin_in = margin.HasValue ? margin.Value : new Vector2(0, 0);
            NativeCalls.godot_icall_3_909(method_bind_18, Object.GetPtr(this), enable, ref size_in, ref margin_in);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_size_override");

        /// <summary>
        /// Get the size override set with [method set_size_override].
        /// </summary>
        [GodotMethod("get_size_override")]
        public Vector2 GetSizeOverride()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_19, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_size_override_enabled");

        /// <summary>
        /// Get the enabled status of the size override set with [method set_size_override].
        /// </summary>
        [GodotMethod("is_size_override_enabled")]
        public bool IsSizeOverrideEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_20, Object.GetPtr(this));
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_size_override_stretch");

        /// <summary>
        /// Set whether the size override affects stretch as well.
        /// </summary>
        [GodotMethod("set_size_override_stretch")]
        public void SetSizeOverrideStretch(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_21, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_size_override_stretch_enabled");

        /// <summary>
        /// Get the enabled status of the size strech override set with [method set_size_override_stretch].
        /// </summary>
        [GodotMethod("is_size_override_stretch_enabled")]
        public bool IsSizeOverrideStretchEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_22, Object.GetPtr(this));
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_vflip");

        [GodotMethod("set_vflip")]
        public void SetVflip(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_23, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_vflip");

        [GodotMethod("get_vflip")]
        public bool GetVflip()
        {
            return NativeCalls.godot_icall_0_3(method_bind_24, Object.GetPtr(this));
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_clear_mode");

        [GodotMethod("set_clear_mode")]
        public void SetClearMode(Viewport.ClearMode mode)
        {
            NativeCalls.godot_icall_1_910(method_bind_25, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_clear_mode");

        [GodotMethod("get_clear_mode")]
        public Viewport.ClearMode GetClearMode()
        {
            return NativeCalls.godot_icall_0_911(method_bind_26, Object.GetPtr(this));
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_update_mode");

        [GodotMethod("set_update_mode")]
        public void SetUpdateMode(Viewport.UpdateMode mode)
        {
            NativeCalls.godot_icall_1_912(method_bind_27, Object.GetPtr(this), mode);
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_update_mode");

        [GodotMethod("get_update_mode")]
        public Viewport.UpdateMode GetUpdateMode()
        {
            return NativeCalls.godot_icall_0_913(method_bind_28, Object.GetPtr(this));
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_msaa");

        [GodotMethod("set_msaa")]
        public void SetMsaa(Viewport.MSAA msaa)
        {
            NativeCalls.godot_icall_1_914(method_bind_29, Object.GetPtr(this), msaa);
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_msaa");

        [GodotMethod("get_msaa")]
        public Viewport.MSAA GetMsaa()
        {
            return NativeCalls.godot_icall_0_915(method_bind_30, Object.GetPtr(this));
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_hdr");

        [GodotMethod("set_hdr")]
        public void SetHdr(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_31, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_hdr");

        [GodotMethod("get_hdr")]
        public bool GetHdr()
        {
            return NativeCalls.godot_icall_0_3(method_bind_32, Object.GetPtr(this));
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_usage");

        [GodotMethod("set_usage")]
        public void SetUsage(Viewport.UsageEnum usage)
        {
            NativeCalls.godot_icall_1_916(method_bind_33, Object.GetPtr(this), usage);
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_usage");

        [GodotMethod("get_usage")]
        public Viewport.UsageEnum GetUsage()
        {
            return NativeCalls.godot_icall_0_917(method_bind_34, Object.GetPtr(this));
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_debug_draw");

        [GodotMethod("set_debug_draw")]
        public void SetDebugDraw(Viewport.DebugDrawEnum debugDraw)
        {
            NativeCalls.godot_icall_1_918(method_bind_35, Object.GetPtr(this), debugDraw);
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_debug_draw");

        [GodotMethod("get_debug_draw")]
        public Viewport.DebugDrawEnum GetDebugDraw()
        {
            return NativeCalls.godot_icall_0_919(method_bind_36, Object.GetPtr(this));
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_render_info");

        /// <summary>
        /// </summary>
        [GodotMethod("get_render_info")]
        public int GetRenderInfo(Viewport.RenderInfo info)
        {
            return NativeCalls.godot_icall_1_920(method_bind_37, Object.GetPtr(this), info);
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_texture");

        /// <summary>
        /// Get the viewport's texture, for use with various objects that you want to texture with the viewport.
        /// </summary>
        [GodotMethod("get_texture")]
        public ViewportTexture GetTexture()
        {
            return NativeCalls.godot_icall_0_921(method_bind_38, Object.GetPtr(this));
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_physics_object_picking");

        [GodotMethod("set_physics_object_picking")]
        public void SetPhysicsObjectPicking(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_39, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_physics_object_picking");

        [GodotMethod("get_physics_object_picking")]
        public bool GetPhysicsObjectPicking()
        {
            return NativeCalls.godot_icall_0_3(method_bind_40, Object.GetPtr(this));
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_viewport_rid");

        /// <summary>
        /// Get the viewport RID from the [VisualServer].
        /// </summary>
        [GodotMethod("get_viewport_rid")]
        public RID GetViewportRid()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_41, Object.GetPtr(this));
            return new RID(ret);
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "input");

        /// <summary>
        /// </summary>
        [GodotMethod("input")]
        public void Input(InputEvent localEvent)
        {
            NativeCalls.godot_icall_1_24(method_bind_42, Object.GetPtr(this), Object.GetPtr(localEvent));
        }

        private IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "unhandled_input");

        /// <summary>
        /// </summary>
        [GodotMethod("unhandled_input")]
        public void UnhandledInput(InputEvent localEvent)
        {
            NativeCalls.godot_icall_1_24(method_bind_43, Object.GetPtr(this), Object.GetPtr(localEvent));
        }

        private IntPtr method_bind_44 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "update_worlds");

        /// <summary>
        /// Force update of the 2D and 3D worlds.
        /// </summary>
        [GodotMethod("update_worlds")]
        public void UpdateWorlds()
        {
            NativeCalls.godot_icall_0_12(method_bind_44, Object.GetPtr(this));
        }

        private IntPtr method_bind_45 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_use_own_world");

        [GodotMethod("set_use_own_world")]
        public void SetUseOwnWorld(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_45, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_46 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_using_own_world");

        [GodotMethod("is_using_own_world")]
        public bool IsUsingOwnWorld()
        {
            return NativeCalls.godot_icall_0_3(method_bind_46, Object.GetPtr(this));
        }

        private IntPtr method_bind_47 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_camera");

        /// <summary>
        /// Return the active 3D camera.
        /// </summary>
        [GodotMethod("get_camera")]
        public Camera GetCamera()
        {
            return NativeCalls.godot_icall_0_922(method_bind_47, Object.GetPtr(this));
        }

        private IntPtr method_bind_48 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_as_audio_listener");

        [GodotMethod("set_as_audio_listener")]
        public void SetAsAudioListener(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_48, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_49 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_audio_listener");

        [GodotMethod("is_audio_listener")]
        public bool IsAudioListener()
        {
            return NativeCalls.godot_icall_0_3(method_bind_49, Object.GetPtr(this));
        }

        private IntPtr method_bind_50 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_as_audio_listener_2d");

        [GodotMethod("set_as_audio_listener_2d")]
        public void SetAsAudioListener2d(bool enable)
        {
            NativeCalls.godot_icall_1_11(method_bind_50, Object.GetPtr(this), enable);
        }

        private IntPtr method_bind_51 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_audio_listener_2d");

        [GodotMethod("is_audio_listener_2d")]
        public bool IsAudioListener2d()
        {
            return NativeCalls.godot_icall_0_3(method_bind_51, Object.GetPtr(this));
        }

        private IntPtr method_bind_52 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_attach_to_screen_rect");

        /// <summary>
        /// </summary>
        [GodotMethod("set_attach_to_screen_rect")]
        public void SetAttachToScreenRect(Rect2 rect)
        {
            NativeCalls.godot_icall_1_120(method_bind_52, Object.GetPtr(this), ref rect);
        }

        private IntPtr method_bind_53 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_mouse_position");

        /// <summary>
        /// Get the mouse position, relative to the viewport.
        /// </summary>
        [GodotMethod("get_mouse_position")]
        public Vector2 GetMousePosition()
        {
            object ret = NativeCalls.godot_icall_0_14(method_bind_53, Object.GetPtr(this));
            return (Vector2)ret;
        }

        private IntPtr method_bind_54 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "warp_mouse");

        /// <summary>
        /// Warp the mouse to a position, relative to the viewport.
        /// </summary>
        [GodotMethod("warp_mouse")]
        public void WarpMouse(Vector2 toPosition)
        {
            NativeCalls.godot_icall_1_45(method_bind_54, Object.GetPtr(this), ref toPosition);
        }

        private IntPtr method_bind_55 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "gui_has_modal_stack");

        /// <summary>
        /// Returns whether there are shown modals on-screen.
        /// </summary>
        [GodotMethod("gui_has_modal_stack")]
        public bool GuiHasModalStack()
        {
            return NativeCalls.godot_icall_0_3(method_bind_55, Object.GetPtr(this));
        }

        private IntPtr method_bind_56 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "gui_get_drag_data");

        /// <summary>
        /// Returns the drag data from the GUI, that was previously returned by [method Control.get_drag_data].
        /// </summary>
        [GodotMethod("gui_get_drag_data")]
        public object GuiGetDragData()
        {
            return NativeCalls.godot_icall_0_461(method_bind_56, Object.GetPtr(this));
        }

        private IntPtr method_bind_57 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_disable_input");

        [GodotMethod("set_disable_input")]
        public void SetDisableInput(bool disable)
        {
            NativeCalls.godot_icall_1_11(method_bind_57, Object.GetPtr(this), disable);
        }

        private IntPtr method_bind_58 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_input_disabled");

        [GodotMethod("is_input_disabled")]
        public bool IsInputDisabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_58, Object.GetPtr(this));
        }

        private IntPtr method_bind_59 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_disable_3d");

        [GodotMethod("set_disable_3d")]
        public void SetDisable3d(bool disable)
        {
            NativeCalls.godot_icall_1_11(method_bind_59, Object.GetPtr(this), disable);
        }

        private IntPtr method_bind_60 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_3d_disabled");

        [GodotMethod("is_3d_disabled")]
        public bool Is3dDisabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_60, Object.GetPtr(this));
        }

        private IntPtr method_bind_61 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shadow_atlas_size");

        [GodotMethod("set_shadow_atlas_size")]
        public void SetShadowAtlasSize(int size)
        {
            NativeCalls.godot_icall_1_0(method_bind_61, Object.GetPtr(this), size);
        }

        private IntPtr method_bind_62 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shadow_atlas_size");

        [GodotMethod("get_shadow_atlas_size")]
        public int GetShadowAtlasSize()
        {
            return NativeCalls.godot_icall_0_1(method_bind_62, Object.GetPtr(this));
        }

        private IntPtr method_bind_63 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_snap_controls_to_pixels");

        [GodotMethod("set_snap_controls_to_pixels")]
        public void SetSnapControlsToPixels(bool enabled)
        {
            NativeCalls.godot_icall_1_11(method_bind_63, Object.GetPtr(this), enabled);
        }

        private IntPtr method_bind_64 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "is_snap_controls_to_pixels_enabled");

        [GodotMethod("is_snap_controls_to_pixels_enabled")]
        public bool IsSnapControlsToPixelsEnabled()
        {
            return NativeCalls.godot_icall_0_3(method_bind_64, Object.GetPtr(this));
        }

        private IntPtr method_bind_65 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_shadow_atlas_quadrant_subdiv");

        [GodotMethod("set_shadow_atlas_quadrant_subdiv")]
        public void SetShadowAtlasQuadrantSubdiv(int quadrant, Viewport.ShadowAtlasQuadrantSubdiv subdiv)
        {
            NativeCalls.godot_icall_2_923(method_bind_65, Object.GetPtr(this), quadrant, subdiv);
        }

        private IntPtr method_bind_66 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_shadow_atlas_quadrant_subdiv");

        [GodotMethod("get_shadow_atlas_quadrant_subdiv")]
        public Viewport.ShadowAtlasQuadrantSubdiv GetShadowAtlasQuadrantSubdiv(int quadrant)
        {
            return NativeCalls.godot_icall_1_924(method_bind_66, Object.GetPtr(this), quadrant);
        }
    }
}
