using System;
using System.Collections.Generic;

namespace Godot
{
    /// <summary>
    /// A script implemented in the  Visual Script programming environment. The script extends the functionality of all objects that instance it.
    /// [method Object.set_script] extends an existing object, if that object's class matches one of the script's base classes.
    /// You are most likely to use this class via the Visual Script editor or when writing plugins for it.
    /// </summary>
    public class VisualScript : Script
    {
        public Dictionary<object, object> Data
        {
            get
            {
                return _GetData();
            }
            set
            {
                _SetData(value);
            }
        }

        private const string nativeName = "VisualScript";

        public VisualScript() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_VisualScript_Ctor(this);
        }

        internal VisualScript(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_function");

        /// <summary>
        /// Add a function with the specified name to the VisualScript.
        /// </summary>
        [GodotMethod("add_function")]
        public void AddFunction(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_0, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_function");

        /// <summary>
        /// Returns whether a function exists with the specified name.
        /// </summary>
        [GodotMethod("has_function")]
        public bool HasFunction(string name)
        {
            return NativeCalls.godot_icall_1_70(method_bind_1, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_function");

        /// <summary>
        /// Remove a specific function and its nodes from the script.
        /// </summary>
        [GodotMethod("remove_function")]
        public void RemoveFunction(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_2, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rename_function");

        /// <summary>
        /// Change the name of a function.
        /// </summary>
        [GodotMethod("rename_function")]
        public void RenameFunction(string name, string newName)
        {
            NativeCalls.godot_icall_2_69(method_bind_3, Object.GetPtr(this), name, newName);
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_function_scroll");

        /// <summary>
        /// Position the center of the screen for a function.
        /// </summary>
        [GodotMethod("set_function_scroll")]
        public void SetFunctionScroll(string name, Vector2 ofs)
        {
            NativeCalls.godot_icall_2_93(method_bind_4, Object.GetPtr(this), name, ref ofs);
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_function_scroll");

        /// <summary>
        /// Returns the position of the center of the screen for a given function.
        /// </summary>
        [GodotMethod("get_function_scroll")]
        public Vector2 GetFunctionScroll(string name)
        {
            object ret = NativeCalls.godot_icall_1_94(method_bind_5, Object.GetPtr(this), name);
            return (Vector2)ret;
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_node");

        /// <summary>
        /// Add a node to a function of the VisualScript.
        /// &lt;param name="position"&gt;If the param is null, then the default value is new Vector2(0, 0)&lt;/param&gt;
        /// </summary>
        [GodotMethod("add_node")]
        public void AddNode(string func, int id, VisualScriptNode node, Nullable<Vector2> position = null)
        {
            Vector2 position_in = position.HasValue ? position.Value : new Vector2(0, 0);
            NativeCalls.godot_icall_4_929(method_bind_6, Object.GetPtr(this), func, id, Object.GetPtr(node), ref position_in);
        }

        private IntPtr method_bind_7 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_node");

        /// <summary>
        /// Remove a specific node.
        /// </summary>
        [GodotMethod("remove_node")]
        public void RemoveNode(string func, int id)
        {
            NativeCalls.godot_icall_2_95(method_bind_7, Object.GetPtr(this), func, id);
        }

        private IntPtr method_bind_8 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_function_node_id");

        /// <summary>
        /// Returns the id of a function's entry point node.
        /// </summary>
        [GodotMethod("get_function_node_id")]
        public int GetFunctionNodeId(string name)
        {
            return NativeCalls.godot_icall_1_87(method_bind_8, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_9 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node");

        /// <summary>
        /// Returns a node given its id and its function.
        /// </summary>
        [GodotMethod("get_node")]
        public VisualScriptNode GetNode(string func, int id)
        {
            return NativeCalls.godot_icall_2_930(method_bind_9, Object.GetPtr(this), func, id);
        }

        private IntPtr method_bind_10 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_node");

        /// <summary>
        /// Returns whether a node exists with the given id.
        /// </summary>
        [GodotMethod("has_node")]
        public bool HasNode(string func, int id)
        {
            return NativeCalls.godot_icall_2_97(method_bind_10, Object.GetPtr(this), func, id);
        }

        private IntPtr method_bind_11 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_node_position");

        /// <summary>
        /// Position a node on the screen.
        /// </summary>
        [GodotMethod("set_node_position")]
        public void SetNodePosition(string func, int id, Vector2 position)
        {
            NativeCalls.godot_icall_3_931(method_bind_11, Object.GetPtr(this), func, id, ref position);
        }

        private IntPtr method_bind_12 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_node_position");

        /// <summary>
        /// Returns a node's position in pixels.
        /// </summary>
        [GodotMethod("get_node_position")]
        public Vector2 GetNodePosition(string func, int id)
        {
            object ret = NativeCalls.godot_icall_2_932(method_bind_12, Object.GetPtr(this), func, id);
            return (Vector2)ret;
        }

        private IntPtr method_bind_13 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "sequence_connect");

        /// <summary>
        /// Connect two sequence ports. The execution will flow from of [code]from_node[/code]'s [code]from_output[/code] into [code]to_node[/code].
        /// Unlike [method data_connect], there isn't a [code]to_port[/code], since the target node can have only one sequence port.
        /// </summary>
        [GodotMethod("sequence_connect")]
        public void SequenceConnect(string func, int fromNode, int fromOutput, int toNode)
        {
            NativeCalls.godot_icall_4_933(method_bind_13, Object.GetPtr(this), func, fromNode, fromOutput, toNode);
        }

        private IntPtr method_bind_14 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "sequence_disconnect");

        /// <summary>
        /// Disconnect two sequence ports previously connected with [method sequence_connect].
        /// </summary>
        [GodotMethod("sequence_disconnect")]
        public void SequenceDisconnect(string func, int fromNode, int fromOutput, int toNode)
        {
            NativeCalls.godot_icall_4_933(method_bind_14, Object.GetPtr(this), func, fromNode, fromOutput, toNode);
        }

        private IntPtr method_bind_15 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_sequence_connection");

        /// <summary>
        /// Returns whether the specified sequence ports are connected.
        /// </summary>
        [GodotMethod("has_sequence_connection")]
        public bool HasSequenceConnection(string func, int fromNode, int fromOutput, int toNode)
        {
            return NativeCalls.godot_icall_4_934(method_bind_15, Object.GetPtr(this), func, fromNode, fromOutput, toNode);
        }

        private IntPtr method_bind_16 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "data_connect");

        /// <summary>
        /// Connect two data ports. The value of [code]from_node[/code]'s [code]from_port[/code] would be fed into [code]to_node[/code]'s [code]to_port[/code].
        /// </summary>
        [GodotMethod("data_connect")]
        public void DataConnect(string func, int fromNode, int fromPort, int toNode, int toPort)
        {
            NativeCalls.godot_icall_5_935(method_bind_16, Object.GetPtr(this), func, fromNode, fromPort, toNode, toPort);
        }

        private IntPtr method_bind_17 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "data_disconnect");

        /// <summary>
        /// Disconnect two data ports previously connected with [method data_connect].
        /// </summary>
        [GodotMethod("data_disconnect")]
        public void DataDisconnect(string func, int fromNode, int fromPort, int toNode, int toPort)
        {
            NativeCalls.godot_icall_5_935(method_bind_17, Object.GetPtr(this), func, fromNode, fromPort, toNode, toPort);
        }

        private IntPtr method_bind_18 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_data_connection");

        /// <summary>
        /// Returns whether the specified data ports are connected.
        /// </summary>
        [GodotMethod("has_data_connection")]
        public bool HasDataConnection(string func, int fromNode, int fromPort, int toNode, int toPort)
        {
            return NativeCalls.godot_icall_5_936(method_bind_18, Object.GetPtr(this), func, fromNode, fromPort, toNode, toPort);
        }

        private IntPtr method_bind_19 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_variable");

        /// <summary>
        /// Add a variable to the VisualScript, optionally giving it a default value or marking it as exported.
        /// </summary>
        [GodotMethod("add_variable")]
        public void AddVariable(string name, object defaultValue = null, bool export = false)
        {
            NativeCalls.godot_icall_3_339(method_bind_19, Object.GetPtr(this), name, defaultValue, export);
        }

        private IntPtr method_bind_20 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_variable");

        /// <summary>
        /// Returns whether a variable exists with the specified name.
        /// </summary>
        [GodotMethod("has_variable")]
        public bool HasVariable(string name)
        {
            return NativeCalls.godot_icall_1_70(method_bind_20, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_21 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_variable");

        /// <summary>
        /// Remove a variable with the given name.
        /// </summary>
        [GodotMethod("remove_variable")]
        public void RemoveVariable(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_21, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_22 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_variable_default_value");

        /// <summary>
        /// Change the default (initial) value of a variable.
        /// </summary>
        [GodotMethod("set_variable_default_value")]
        public void SetVariableDefaultValue(string name, object value)
        {
            NativeCalls.godot_icall_2_337(method_bind_22, Object.GetPtr(this), name, value);
        }

        private IntPtr method_bind_23 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_variable_default_value");

        /// <summary>
        /// Returns the default (initial) value of a variable.
        /// </summary>
        [GodotMethod("get_variable_default_value")]
        public object GetVariableDefaultValue(string name)
        {
            return NativeCalls.godot_icall_1_338(method_bind_23, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_24 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_variable_info");

        /// <summary>
        /// Set a variable's info, using the same format as [method get_variable_info].
        /// </summary>
        [GodotMethod("set_variable_info")]
        public void SetVariableInfo(string name, Dictionary<object, object> value)
        {
            NativeCalls.godot_icall_2_937(method_bind_24, Object.GetPtr(this), name, value);
        }

        private IntPtr method_bind_25 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_variable_info");

        /// <summary>
        /// Returns the info for a given variable as a dictionary. The information includes its name, type, hint and usage.
        /// </summary>
        [GodotMethod("get_variable_info")]
        public Dictionary<object, object> GetVariableInfo(string name)
        {
            return NativeCalls.godot_icall_1_938(method_bind_25, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_26 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_variable_export");

        /// <summary>
        /// Change whether a variable is exported.
        /// </summary>
        [GodotMethod("set_variable_export")]
        public void SetVariableExport(string name, bool enable)
        {
            NativeCalls.godot_icall_2_92(method_bind_26, Object.GetPtr(this), name, enable);
        }

        private IntPtr method_bind_27 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_variable_export");

        /// <summary>
        /// Returns whether a variable is exported.
        /// </summary>
        [GodotMethod("get_variable_export")]
        public bool GetVariableExport(string name)
        {
            return NativeCalls.godot_icall_1_70(method_bind_27, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_28 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rename_variable");

        /// <summary>
        /// Change the name of a variable.
        /// </summary>
        [GodotMethod("rename_variable")]
        public void RenameVariable(string name, string newName)
        {
            NativeCalls.godot_icall_2_69(method_bind_28, Object.GetPtr(this), name, newName);
        }

        private IntPtr method_bind_29 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "add_custom_signal");

        /// <summary>
        /// Add a custom signal with the specified name to the VisualScript.
        /// </summary>
        [GodotMethod("add_custom_signal")]
        public void AddCustomSignal(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_29, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_30 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "has_custom_signal");

        /// <summary>
        /// Returns whether a signal exists with the specified name.
        /// </summary>
        [GodotMethod("has_custom_signal")]
        public bool HasCustomSignal(string name)
        {
            return NativeCalls.godot_icall_1_70(method_bind_30, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_31 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "custom_signal_add_argument");

        /// <summary>
        /// Add an argument to a custom signal added with [method add_custom_signal].
        /// </summary>
        [GodotMethod("custom_signal_add_argument")]
        public void CustomSignalAddArgument(string name, Variant.Type type, string argname, int index = -1)
        {
            NativeCalls.godot_icall_4_939(method_bind_31, Object.GetPtr(this), name, type, argname, index);
        }

        private IntPtr method_bind_32 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "custom_signal_set_argument_type");

        /// <summary>
        /// Change the type of a custom signal's argument.
        /// </summary>
        [GodotMethod("custom_signal_set_argument_type")]
        public void CustomSignalSetArgumentType(string name, int argidx, Variant.Type type)
        {
            NativeCalls.godot_icall_3_940(method_bind_32, Object.GetPtr(this), name, argidx, type);
        }

        private IntPtr method_bind_33 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "custom_signal_get_argument_type");

        /// <summary>
        /// Get the type of a custom signal's argument.
        /// </summary>
        [GodotMethod("custom_signal_get_argument_type")]
        public Variant.Type CustomSignalGetArgumentType(string name, int argidx)
        {
            return NativeCalls.godot_icall_2_941(method_bind_33, Object.GetPtr(this), name, argidx);
        }

        private IntPtr method_bind_34 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "custom_signal_set_argument_name");

        /// <summary>
        /// Rename a custom signal's argument.
        /// </summary>
        [GodotMethod("custom_signal_set_argument_name")]
        public void CustomSignalSetArgumentName(string name, int argidx, string argname)
        {
            NativeCalls.godot_icall_3_942(method_bind_34, Object.GetPtr(this), name, argidx, argname);
        }

        private IntPtr method_bind_35 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "custom_signal_get_argument_name");

        /// <summary>
        /// Get the name of a custom signal's argument.
        /// </summary>
        [GodotMethod("custom_signal_get_argument_name")]
        public string CustomSignalGetArgumentName(string name, int argidx)
        {
            return NativeCalls.godot_icall_2_88(method_bind_35, Object.GetPtr(this), name, argidx);
        }

        private IntPtr method_bind_36 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "custom_signal_remove_argument");

        /// <summary>
        /// Remove a specific custom signal's argument.
        /// </summary>
        [GodotMethod("custom_signal_remove_argument")]
        public void CustomSignalRemoveArgument(string name, int argidx)
        {
            NativeCalls.godot_icall_2_95(method_bind_36, Object.GetPtr(this), name, argidx);
        }

        private IntPtr method_bind_37 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "custom_signal_get_argument_count");

        /// <summary>
        /// Get the count of a custom signal's arguments.
        /// </summary>
        [GodotMethod("custom_signal_get_argument_count")]
        public int CustomSignalGetArgumentCount(string name)
        {
            return NativeCalls.godot_icall_1_87(method_bind_37, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_38 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "custom_signal_swap_argument");

        /// <summary>
        /// Swap two of the arguments of a custom signal.
        /// </summary>
        [GodotMethod("custom_signal_swap_argument")]
        public void CustomSignalSwapArgument(string name, int argidx, int withidx)
        {
            NativeCalls.godot_icall_3_713(method_bind_38, Object.GetPtr(this), name, argidx, withidx);
        }

        private IntPtr method_bind_39 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "remove_custom_signal");

        /// <summary>
        /// Remove a custom signal with the given name.
        /// </summary>
        [GodotMethod("remove_custom_signal")]
        public void RemoveCustomSignal(string name)
        {
            NativeCalls.godot_icall_1_43(method_bind_39, Object.GetPtr(this), name);
        }

        private IntPtr method_bind_40 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "rename_custom_signal");

        /// <summary>
        /// Change the name of a custom signal.
        /// </summary>
        [GodotMethod("rename_custom_signal")]
        public void RenameCustomSignal(string name, string newName)
        {
            NativeCalls.godot_icall_2_69(method_bind_40, Object.GetPtr(this), name, newName);
        }

        private IntPtr method_bind_41 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_instance_base_type");

        /// <summary>
        /// Set the base type of the script.
        /// </summary>
        [GodotMethod("set_instance_base_type")]
        public void SetInstanceBaseType(string type)
        {
            NativeCalls.godot_icall_1_43(method_bind_41, Object.GetPtr(this), type);
        }

        private IntPtr method_bind_42 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_set_data");

        internal void _SetData(Dictionary<object, object> data)
        {
            NativeCalls.godot_icall_1_168(method_bind_42, Object.GetPtr(this), data);
        }

        private IntPtr method_bind_43 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "_get_data");

        internal Dictionary<object, object> _GetData()
        {
            return NativeCalls.godot_icall_0_169(method_bind_43, Object.GetPtr(this));
        }
    }
}
