using System;

namespace Godot
{
    /// <summary>
    /// Class that has everything pertaining to a world. A physics space, a visual scenario and a sound space. Spatial nodes register their resources into the current world.
    /// </summary>
    public class World : Resource
    {
        /// <summary>
        /// The World's [Environment].
        /// </summary>
        public Environment Environment
        {
            get
            {
                return GetEnvironment();
            }
            set
            {
                SetEnvironment(value);
            }
        }

        /// <summary>
        /// The World's fallback_environment will be used if the World's [Environment] fails or is missing.
        /// </summary>
        public Environment FallbackEnvironment
        {
            get
            {
                return GetFallbackEnvironment();
            }
            set
            {
                SetFallbackEnvironment(value);
            }
        }

        /// <summary>
        /// The World's physics space.
        /// </summary>
        public RID Space
        {
            get
            {
                return GetSpace();
            }
        }

        /// <summary>
        /// The World's visual scenario.
        /// </summary>
        public RID Scenario
        {
            get
            {
                return GetScenario();
            }
        }

        /// <summary>
        /// The World's physics direct space state, used for making various queries. Might be used only during [code]_physics_process[/code].
        /// </summary>
        public PhysicsDirectSpaceState DirectSpaceState
        {
            get
            {
                return GetDirectSpaceState();
            }
        }


        private const string nativeName = "World";

        public World() : this(true)
        {
            if (ptr == IntPtr.Zero)
                ptr = NativeCalls.godot_icall_World_Ctor(this);
        }

        internal World(bool memoryOwn) : base(memoryOwn) {}

        private IntPtr method_bind_0 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_space");

        [GodotMethod("get_space")]
        public RID GetSpace()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_0, Object.GetPtr(this));
            return new RID(ret);
        }

        private IntPtr method_bind_1 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_scenario");

        [GodotMethod("get_scenario")]
        public RID GetScenario()
        {
            IntPtr ret = NativeCalls.godot_icall_0_201(method_bind_1, Object.GetPtr(this));
            return new RID(ret);
        }

        private IntPtr method_bind_2 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_environment");

        [GodotMethod("set_environment")]
        public void SetEnvironment(Environment env)
        {
            NativeCalls.godot_icall_1_24(method_bind_2, Object.GetPtr(this), Object.GetPtr(env));
        }

        private IntPtr method_bind_3 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_environment");

        [GodotMethod("get_environment")]
        public Environment GetEnvironment()
        {
            return NativeCalls.godot_icall_0_189(method_bind_3, Object.GetPtr(this));
        }

        private IntPtr method_bind_4 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "set_fallback_environment");

        [GodotMethod("set_fallback_environment")]
        public void SetFallbackEnvironment(Environment env)
        {
            NativeCalls.godot_icall_1_24(method_bind_4, Object.GetPtr(this), Object.GetPtr(env));
        }

        private IntPtr method_bind_5 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_fallback_environment");

        [GodotMethod("get_fallback_environment")]
        public Environment GetFallbackEnvironment()
        {
            return NativeCalls.godot_icall_0_189(method_bind_5, Object.GetPtr(this));
        }

        private IntPtr method_bind_6 = NativeCalls.godot_icall_ClassDB_get_method(nativeName, "get_direct_space_state");

        [GodotMethod("get_direct_space_state")]
        public PhysicsDirectSpaceState GetDirectSpaceState()
        {
            return NativeCalls.godot_icall_0_658(method_bind_6, Object.GetPtr(this));
        }
    }
}
